#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A specialized version of the Map class representing a map of Oceania.

This class inherits from the generic Map class and provides geographical data specific to Oceania.
It allows for rendering and customizing the appearance of Oceanian countries, including setting
specific fill and stroke colors for individual countries.

Use this class to create and manipulate a map focused on Oceania within a PDF document.
"""
import typing

from borb.pdf.color.color import Color
from borb.pdf.color.x11_color import X11Color
from borb.pdf.layout_element.layout_element import LayoutElement
from borb.pdf.layout_element.shape.map import Map


class MapOfOceania(Map):
    """
    A specialized version of the Map class representing a map of Oceania.

    This class inherits from the generic Map class and provides geographical data specific to Oceania.
    It allows for rendering and customizing the appearance of Oceanian countries, including setting
    specific fill and stroke colors for individual countries.

    Use this class to create and manipulate a map focused on Oceania within a PDF document.
    """

    __MAP_OF_OCEANIA_GEO_JSON_BYTES: bytes = (
        b'x\xda\xb5[[\x93\xdb\xb6\x92\xfe+*\xbf\xecK<E\xdc\xc9}s\\\xe5\xac\xd7g]\xd9\xf5&[\xeb=)\x17EQ\xb6\x12Y\x9a\x1aIvMR\xf9\xef\x0b\xf4\x05h\x90\x94l\x9f\xca\x99\x07\x8e>\xdc\xd1h\xa2\x1b\x1f\x9a\x7f<9?\xde\x8fO\xfeu\xf5\xe4\xc5\xd8\x9f/\x0f\xe3\xf3\xe3~?\x0e\xe7\xdd\xf1\xf0\xe4\xbb\xd5\x93-&\x9eb\x81\xff\xfbcZ4\x15x?\x1e?\x8e\xe7\x87\xc7\x98\\\xf2\xff\xe3\xb2?\xef~<\xee\x1f\xdfc+\xc3\xf1\xf8\xb0\xd9\x1d\xfa36\x14\xff\x94Uw\xcdw\xab\xa7\xfa\xce\xff\xf2\xdd*B}\x17"4w\x06\xa1\xbd\xf3\x00;\x84\xee\xceDh\xef,\xc3\x16 \xe5zh\xca\xdd9\x84\x01\xea\xfa;\xc5\xb0\x03\xe8\x196\x00\x03C\x1da\xe0\x96\xdb;\x15a{\xd70L\xa3\xea\xb8\xa9\x0e\x861\x83\xd8\xafk\xa0\xe5\x8e[\xee\xa8n\xc7\xb9i\xcc\xaa\xa1\x19F\x1c\x10{Y;\xe2\\\x9d\xca\xe7\x91u\xb2>M+\xe2<M%\x06\x13e\xe2a"\x95\x88Z.l\xa1\xf3\xc021\xd0V\xe0\xc2\x06\xe6\xd5\xdei\x86\x16Znx\xa5<@\xc3\x10\xfb\xa5\xc2\x8a\x84\xa0$t\xdcrY\xf4_RB\xd4\x03\xa7i\xa5q\xda\xce@\tK\x9dE\xa8\x00\x92\x885-|\xcb\xd0\x03\xd4\x0c-4\x95s\x1b\x80TWQn#\xe5\xaf\xb9\xdf\x06D\xa0Ka\x07\xb0e\xd8\xca\xba\x1a\x94\xc6\x94~y\nyV\nD\xe8\xb8>.>\xabd\x84\x1a\xa0\x91\x9a\x92aK\n\x9bu\x10\x9b\n\x0c-@+u\xd0\xf1J\xa2\xd6\xb8Z%]\x99\xb3\xaa\xa1\xae\x0bWu\x15\x15V\x0c\xbd\x14=\nH\x88\x1e\xd7I1\xc4\xf7\xd5\xd4\xb0\x93\x0b\xe3jY\xf3\x14\x84\xe8\xb24-\x8d\x8c\x9as42\xcf\xd0\x81\xbch(\x9e^p\xc7\xb9(\xcd\x96!n\x0e\x1dC-\x0b\xe3{\xc1\xea\x1a\xa1\x93"\xe0\xdc\x86!\x8f*\xfe\xfd\x19\xb7\xb9\xfb\x87\xe3\xfd\xf8p\xde\xc1.\xf7\xc7\x93C\xff\x11v\xc3\x1f\xfb\xfbK\xbfz=~^\xfdp\xd9\x1d\xc6>\xed\x88)\xef\xdd\xfexx\x7f\xad\xc0\xf6\xf8\xf0\xb1\xdf\xbf\x1b\x0f\xa9\xc0\xcb\xc3f\xbc\x1f\xe3\xe3p^\xbd9\xc7mtu\xdc\xaen\xd4\xda>\xc4Z\x87\xcb~\xcf=\r\xbb\xfe\xf3\xf6\xe6XN\xc7\x87\xf3\xcd\x02\xfd\xfe\\\x1a\xdd\xee\xeeO\xefT\x03\x15~LEv\xa7\xe3\xbb^\xbf\x1b?@\xd2\x0f9\xc9p\xd2\xeb\x1fJKitO\xfe~i\xbcn\xe1\x19\xcaoK)\xab\xf4\xcf\xf4\x90\x84O_~S6\xd6\xb3\x16~\x0f\xf0\xdc\x8aB\xf0[w\xb9\xdb\xf5\x81\xba\xed\xfaT\xa0[\x8f\xe5\xf7\xa0\xe0\xf7\x16\xd2\x07\xcc]AR\x0b\x002\xda.=\xbbPR87\xf7\xb1)\x0b\xfe\xf4\xf5xy_\x8b\x10\x17\xf3\xaa\x84AmRvl\xb4\xd9\xf6\xb1\xcce\xfc\xd4OKm{\x96^\x18\x8b\x14\xf07IO\x83x`6~\x18\x84\xf4Pn\x04\xda\xd2\xe4\x03u|\xbc\xf4\xa7\xdd\xf8\xf4\xf5\xf1\xf2i\x8c^\xc0\xd3\xd4u\x1a\xcc\xd8\x8de\x94{\xea\xdf\xf4Mz\xae\x93\xe8\xcc\x80\xbf\xb7\xf0{\x83\xe9\xa93\xd3\x01\xe8\xab$#\xca:\xf8\xdd\xc1\xb3*\x9a;\xfc0R\x87n\xb4\xe5\xb9q\xf0l\xe0iW\xf0O\xc3\xb3\x83B\xcd<\x1b\xeb\x05x\x1a(\xd4az\xe9j\xc7*\xa2A-\xccX~[P\x91\xc6\x83Z\x80\xb2vv\x03\xbfA\x15,\x08\xbdS\xa0\x1df+\n5\xa5\xf9\x0b\xc8\x19$\xaa\x92\x12\xa4_\x9b\xfe\xd7\xa7\x93%\xdem\x84\x9e\\\xde\xef\x0e\xbb\x92%_\xd2\xcbq\xae\x1e\xbf\x92z\x98f\xa3\xe0\x19\xd2\xb3\xd7\xe99\xac\xd3st\xe9\xb9\x1d },\xe9\xbd\xce\x8d\xfcv\xc4F61/>m\x883\x19\\z\xd3\xd6:\xc4g?\x9a&\xfd\x1e\x8a\x0e\x1d\xf6\xacCcT\xfd\xddx\xf9<\x9d\xd5\xfd^\xbc\x1b\xc7\xcfq\xe4\x9f\xeb\xfc\xb3\xcc\xa7\x99\xa1\xf6\xe52\x0f\x17Z \xab\x92\x88m\x1aF|\xc2ok0%JU7\n\xd6\xdc\xaa\r\xa4\x8d\xf0\xd4\xa5\x86\xddb\xb6\x11\x19-<\xb1\x82\xc3Bew\xfc$\xc4\xfe8\x13\xfa\xf9\xa1d\xff\xefx\xd8\xad~\x88\xd99\xf7\xf2\xdbW\rzy\xb4\xb3q:\x7fc\x9c\x97\x87/n\x0er\x0f\xa0\xcd\xa1\x15\xfb\x05g\xe4&?\xednnZ\xbf\x7f\xc0\x1e\xddf\x1b\x15#\xbewq\xa8v\xec\xe2+\xe3\xdd:N\xd4\xa9\x11\x9e\xad\xc3tQ\xf1\xbcX3\x8e5lT\xaa3\xa6q;\xe5\x03\xa6?\xf93\xda\xd8\xbf\xf6\x0c\x12\xc0\xa0\xdb\x86\x9c\x03r\xb5"\xee\xa4\xaf\x95\\\\\xc6\rb+}p\xcb\x8e \x9d<b~\xf6\xba)\xdf3\x0e\x13\xdc v|\xae\xa1\xfeBu\xec\xd1\xec\n\xa2\x03\x12\xbdh]\xe1\xa6\x94\xb7\xd5|<b\xc5\xe3\xa7\xf1(nO\xcc\x9f}-\xed\xc1\xb92\x9a\xfa\xd0\x8e1\xf6\xa1-\xfa\xbe\xec\x0bG\xdc \xc69h\x83\x079>\xbait\xd9%\x86\xfa\xec\xe1kt\xf7\x0c{\xed\xbaAg\x9a\xcb\xab\x0e\xc6\x9c\xcb+tx\r\x1f\t"n*\xdc\x82\xcf\x16q`\x0c\xf9\xec\xc4\xa9\x80\xf5\xd9\x8bSxP*\x18\x1dD\xc3g\xcd\x88\xa9}]a^C\x85\xdeg\x9e\x7f\xc44\x7fSa\xd6\t\x85g\xd7,_\xceW\xa5=\x90OSp\x83\xb8\x91X\xf3AO\xe1IY\xb7$\xbf\x82s\xbeF\xac\x18C\xfd\xc0\xe33x\xd2a\x17X\xe1\xe1O\xf3Y%\xe2v\x92\x9ftJ\xb3\xf7\xad\xf0\xec\xa8\xd9g\xe6\xfe\xf84\xc3\x98\x8fB\n\xf5Cgy\x1b\x1co^/j?\xcb\x9f\xc6g&\xf5M\xe9\xbfE\xac\xaa|]\xe6\x9f\xf47\xaaY[\x8dGO\xe4\x97\xf3q=#\xce\xeb\xd9!V\xac/\xd0>\xbfs\n\t\x04\x9d\xd7+\xe0\xf8\x9b\xa2\x7f\x94o\x19\xb7\x88\x8d\xd4_\xdd\xd4\xfa-q+\xb1\xc6\x83\xaab\x9aA\xe3\xb1V\xf1\x01<b\xe0\x15Z\xce\xc7\xf7M\xf1i^\xe3Y,\x0e\xb3\x9d`\xc3\xef/\xf0\x10\x9e\xc6\xabq=d~\x87X1vX\xde3n\x11+\xde\x1f\x0cb\xc3\x18\xdac\xfd\x89\xb8C\xacx\xbf\x81\xf1\xb2>h|?\x14\xbf\xdf\x05\xd3|p\xbfR|\xd6,X1\xf6\x88I~\xc8\x93\xa8\xbc\xdf\x04\x1co\xae\x8f{\xbe\xe2\xc3\xaa\xc6\xf5\x91\xd8\xe3x\x1b\xc6T\xde2\xee\xb0}\x9c_|q\rb\xcbX#V\x8c\xa1=\xd6\xc7\xb8\x11h\xc4\x9aq@\x8c\xe33\x9a\xcb\xab\n\xf3\xfea\x90\xaa\x888\x97\xb7\x88\xa9<\xad\xaf\xe2\xf1\x18\xae\x8f\xf20\xb4>ls\x0c\xda\x98\x98\x8f\xf37h\x93\xca\xf8\xf0\xfd(\xe5=\xe7g\xec\xaa\xfa\xc8\xbf)\xb6\xa1\xc6\xf3\xfc\xba\xaa>\xef\x9f\x06m\xa4\xc4\xb4~\x81\xfbo\xa5>\x18\xc7\xeb\x91\xf3\x9d\\/n\x9f\xf5\xcf\x90>\xb82>\x8f\xfaJ\xed\xb5\xac\xbf-c_\xe1\x0e\xeb\xf3\xfb`:~\x9f\xc8G\xa0\xf5\x0el\xa3\x1b~\x7f,SbJ\xb6g\x15\xf7\x97\xf3\xe9}\xc9\xf9A\xce\xc7*\xd6o?\xc1\x86\xb1\x93\xf2\xb2\xaa\x96\xb7%}\xd1\xb3\xfc\xdc\x7f\'\xd7\x8f\xf8\xbe\xac_\x05gz\xd0U<\xa6f}\xd4\x8c;\xa9o\xd6p\xfdL>j\xa9\xef\x96\xf6\x17]\xca{\xf9>\x15\xdcJ.3\xef\x17\xc4$g\xfd\xb0\x96\xf3\xbd\xf4\x99\x8a<\x1d\xebG\xe6\x9a\x9d\xdc\xbf\xac\xab\xd7\xdf\x92\xfe\xfb\xe2\xf3i\xb9\xbfZ\xd2\xd7\x96\xeb{\xde\xaf\xb3\x0f\xe6\x10;\xf61\xb5\xdc\xefm\xb6\x17VR\xd2\xd9\xde\x10\x01\xa8UM&j]xf\x858s\x8f\x0e\xb1\xafxP^o&B\x05mJ\xf6YI\xa6O\x0bJ\x96\xec\xbf\x91\x8cm\xf6\x1f\x9ca\x7f\xc0W\xf9\xa1\x94\xf7\xd2?qd\xef\xbb\xc2\x00\x1b\xe9\xefP}\xc3\xf2(\xb8\x93\xe3\xc9\xfe\x94C}4\xba\xd0\x8bA\xfa\xafn\xe2\x7f:\xc5\xfe^\xc5\xd3\xc7mD3&\xff1H\xf9\x1a_\x08Y\xe8?\x14y7\x88s\xbeE\\\x9d9\nF\xff\xc1d\xf6\x1d}\xf6\xb4\xed0\x86\xf2]\xd1WW\xe5[.\x9f\xf5\x97\xfa\xef8\x9f\xca+\xf9\xfe\x18\xf6\x1f\xf9"\x86\xfdGK\xfez[\xf6\x03*ox?#\xdc0\xae\xe6k\xd0\x7f1\xbc\xfe\x06\xed\xa7\xf1e\xbfTR\x9e\xa6e\xec\x19[\xe9\xbf\x9b\x96\xcf\x0f\x96\xf7\xebP\xe5{\xec\x8f\xf5\xd1\x90<\xb3=\x08|>P\x8c;\xe9\xcf\x9b\xc0\xfeye\xaf\xb2~\x18\xcf\xed)i\x9f\x8c-\xf6Q\xcb\xf3\x08c>\xcf\x18\xcb\xe7\x1b\xcd\x98\xceW\xd9\xfe\x1a\xa9\xafd\xafM\xb6\xa7\x9a\xf3\x1b\xb6\xf7\x06\xf5\xdd\xb1\xff\xe1\xa4\xfek\x92\x17\xef\xaf\xe4\xff\x18]\xfbKp\x1e\xb9Mc?\xbb\x9c\xce\x0f\xfd~7\xe7\xaf\xab\x9c\x8a\xb8~~\xfc\xf8\xf1x\xf8<\xf6\xfb\xf3\x87\xc4Y/\x95\xbcNV\xcf{d\x96z\x9es\x8d\x9e~\xf6fFO?\xfbiFO?\xfb\xe9\xcd\x12=m\x80\x85\x86\'\x90\x80\xde\xa8)\xf1\xcc\x8c\xf4\x02\xcd\x9c(\x8f\xa6[\x03\x0b\x08\x9ch\xd7m\xcb\xefD\x8e\xc4\xdf\xc8&k\xc1)W$\xf4\x84Y\xe6y\x8f\x87\t\xa9<\x17\x08\xb2\xc9\xf3\xf4\xc2\x1f\xe34nO\x0f\x19"1=\xe4\x8a\xf38\x16h\xe1N\x15bw\x00\xaaw\xb0\xf0T\x85.^\xaf\x81\xea\xdd^\xa5z\x89\xb9\x05\x92wT\xf0\xbb\x85\xdf\xf0\xdc\x0c\x85\xe9]\xa4o\x15\x90\xb5F\xb0\xb4j[~\x03\x01\xd7Y\xa4j\xb5 l\xb7H\xfaN\x08\xdb8\xd9\xdf\xcf\x0f\xc8\xdaJI"M;\x97\xf0\xee\x8a\x8a\x16j6]:0\x05\x9b\xb8o\xd3\x0c-P\xb3\xf0{\xec\xaf\xd0\xb1\x83\xd7Q\x92\x83\xee\xe3s\xa3\x93b\xad\xdb&Q\xb3!\xc4\xa6\xd6]? M;!ey$0\x83\xf5\x84\x8f\x9d\x0f\xf3\xbe\x0c\x7f>gA\xc06\x85\x93\x04n\xdcZ\xfc\xdd\x08\x8es]\x98\xd5\x19\x9b\xba\xa0\xcb\xc8\xa3>\xfb\x849\x8f\xfd\x12\x85\xfa\xd5\xdd"Q\xbaH\x8ej]4?tE\xf3\x89\x0f]\xd6\xfcO\xac]\xcd\xa6\x1ff\xfc\xa7\xdf\xaecs\xaeK/\x8f\xd3\xba\xbb\xc6v.\x96\xfb\'p\x9bm#O\xe1\x8c\xd0&\x04:\xb3\xb2\x0f\x1b\xd0\xa7\x8c>k\xc3\xd8O\xca+y&)\xf5m\xd5\xba\xca,bh\xf9T\xe4\xb8\xc5\x8aE(=\xfa\xaaG\xf6z\x02E7\xb0\x97\x11\xd0\x8a\xcb|#OU!p{\x96\xdb\xcb\xfd\xd3\x98\x9e\xa6a\x135\x01\xd3\x84\x04\xf2\xdd\x1d&TbzZ\xcb\xb0\xae\x7f\xdbT\xbe\xd8\xfd\xba\x9bYIN\xac\x0c\xe4\x7f\x8d\xf7\x97\xf5~7$\xe38)p\xdd.V\xcd\xb3I\xac\x12\xafY\xc3\x17\xff>\xb3\x86"\xc9\xe4\xa4\x97\x0b\xd6\x10\xde6\xb6w\xf8\xa2\xf4K\x97\xab\xebb\xcb\xbaa\xf1~\xf4\xc5ns\x1a>\xec&&\xac\x9a\x00Z\xaf\x17\xbb\xc7\xdd\x82\xe1\xb2\xe2e\xc5\x91\x88\xfb\n4P\xb1\x0b\xd9Z\xb9\xb3\xf4\xc5\xf0\xa0YZ{\xbc\x81\xbcz\xdfHfF\xff\x0b\x82\xa5\xcbB\x98\xb2\x19\x8a%Q\xc3\xe2\xd5_\x1c\xd5\xef\xa7\xdd\xd3\xd3\xef\xbb\xf7\xe3y\xfcmbI*\x01\xechQ\xdf\xef\x96\xae\xf6\xe0\n\xaf7`?Z\xb4%\xf3\xcb;\x97F4tC3\xb1\x07U?\xf7\x94\x94\x0c\xa3\n\xc3nb\x06\xaa\xb2e\xf3Ov(_\x99\xe1o\xbc\x96j\'\x9b|U\xff\xfc0K*\x1b;6I\xb7[\xa2I\xe7\x176p\xd4\x80\xd9\xda\xe3\x16]u\x90\xf7g\xd7E\xdb\xe07v\\\xd8\x931o\xbb\xd9\xfe\xf5\xfbp\xa0;\x17\x8e\xd3\n\xc8\x87Y>\xbf\x07\xba\x03\xe2\xf3{\xc0\xf3B\xa2a\x18+\xc4\xb4\xb3\xe5;\xa0\xba|\xc3;-\x9d\xff8\xee, \x1fk\xf8\xfc\x1c\x0c\xe7+._\x9d\xff\xa8\xfd|\x9ec\x1cJ\xff\x06\xcfoa\x82]\xd5\x1f[\x9e`\xf8<G;\xb7\xe6\xfb\x95\\\xbe\x99\xe0\xea>\x86\xdb\xe7\xf3\x1c\x8f\x97\xf9\xb9\xe0\xf8\xfc\x9c\xb1\xad\xc6\xe3x<\xad\x94\xb7\tE\xfe\xad<\x1f\x87|\xfe\xce\xb6\xb0.O\xf7K\xd9\xf2\xd0\xf9\xbd-\xf9\r\xca7[*]\xc9\xdb\xb3\xfc-\xe77\xd5z\t}ak\xea\xbb\xfa\xde0 \x87c\x99\xc3\x08\xc8!Z\xe6\x90\x03\xc5j1\xc7\x16\x14\xdf\xfb9^\x03%\xef\x01\x03\x85\xd55\xf5\x9a\xe4{\xd0`X\'\xf3\x1a\xd4\xf9v\x96\xaf\xab\xfeQ\xe7,s\xc6\xdc\x9e)\xfd\xd3\xfc\xac\xd4\x19\xcbg\xe6@\xf1c\x19S\xf0\x19sz\x019s\xcbg\xf0\x80\x9c\x84eN\xd7\xa3\xe1\xb6\xec\xafx\xe4\xcc,\xfb7\x9e\xeey3\x0e\\\x1e\xe7\xe3\xe9\xde\x96u\xce\xd3;\x9c\xdb\xc75\xb4\xccAx\xbaG\xb6\x05wr\xfcr=o;\x12\xe9\xc2\xfdm<=\xf7\x87\xcd\xcc\x9f\x98\xe4Un\xc5r\xdeu\x8fb\xa9\x1fv,\x96\xf2\xae\xf9\x17\xaf\xdf\xce\xfc\x0b\x91dr\xd2\xdf\x96\xfc\x0b\x11\xe1\x85\xb1\n\xe0\xe5s\x8a-e(\xb4k\xe9\xcc\xbd\x10\xaf5\x94\x14<g\xe3\xf9\x9b\xce\xd9c\xa9E\xe9j\xe2\xad\xbc\x1e/\xa7q\xac\xa6\xbf(d\xe1\xb7`\xe0\xd6[\xa8\xb4t\xf2\xae\xa3/\xcaT\xe5\xf9\x83\xa7:qlrl\xd6[\x0c\x8dI],\x9e\xc2\x17\xe2\xae\xd0\xd3\t\xe5\x00N\x87q,d\xbfp$\xa7\x90*\xf4\x86\x1c\x86Ty\x914\x88Bf\xc9KZ\x88\x99*\xbe\x92!\x8f\xa9\x9c\xc9A0\x1dp0\\KM\xfc)\x8e\x9f\x12\xb2\x98\xf8To`\rv\xfd\xea\xfb>z/\xb5s\x85\xf1S\xd3U*^\xd6<\\\x8a}\xadrV\xdf\x82\x0f6t3\xef\x0b\x03\xa5\xa2\xf7\x95N\xe9!)\xe1\xdan\x86\x89\'\x86qRo\'\xda\x85.\x19\x84H\xbd\xa5\xe1O\xbc\xb2\xd74p\x98\xb6\xae\n\x88\x03\xfa\x17b\x9fB\x89 \xa2sz#"\x8b\xec\x8d\x93{\x8a\x80\x9a\x0e\x1a};\x08~\x9aJT\x1c\xde\xaf\x068}\xc5`n\x9c\xe7\xaf\xbfO\xab\xe9\x81\x1e\x8b\xb6\xed\xc4g\\z\x97\x8b\xeb\x98\xb8\xba\xb6\x0b\xdb\x14\x8e\x14\x9a%\x07\xb2\x94h\xbd\xde|\xa3\x1by\xcb\x83T\x1e\x1d\x98\x1c\x80\xe0=\x07,\x046>t\xc1\xa3\xa5\xb1\xca\x17:\xdes\xc0\x03\xd5\xcf\x01\x0eT\xdfN\xda\xb7u\xc0\x82\xb7\x1c\xa0\x90\xf3]\x85\x1d\xe7\xbb\xaa}\x0eB\x92\xe3\xff\xa2\xad{\xde\xef\xc7\xcd\xf1\xb0\xc01\xcfrg\xf6n)w\xb2g\xc6"\xb8S\xa4re\xd7\xac\xcc\xe0|\x08\xd2\x10\xces\xaf\x9a\xc2\xe7sS\xf8|n\n\x9f/\x9aBs\x85f\xde\x16\r\xffK\x03\xa2a\xabM\xb4b\x0e\x85\x86\x98\xc0\xaes\xcb\x06\xb3"\xac\xa1\xc2\xb0\x9e\x9a\xdf\x9bDv4\xaa\xbf\x91(g\\\xf6\xb2\x9c\xa5e\x9d\xe7\x16\xdb\xdaw3Q\xcc\xa4\xc6\xdc\x9e\xd8#\xa4\xbc(\xfb\xaa\xe9\xbd\xa2F\xb7\x8do?e\xbf\xc9\x10\xdb\x12\x0cM\x01\xd07Lp\x10\xec\xb7\x11\x0c\xb94\xcd\xffH\xb4s+\xe9o\x0cdv\xc5\x06\x93=\x0eh\x83\xe1\xf7\xbaT\xbb\xc9\x9d\xb3\x85~\x95V\x0c\xe2\xda\xcdaF\xa1\xbf\xe2\xe5\xbcn\xa5\xe7+~\xcbN\xf7\x982\xa0m\xbe\x19\xe2\xbc\xd6M\nk\x1e\x92\xe6\x0e#\xf0\xe8\xc0\xa9\xaf\x81\x1eZ\x08qF\x9b\xcd\xe3Y\xe4\xd4\xc1t\xbf\x9a\x8dX\xd8\xee\xe7\xcb\xe2\xf8z\xe3\xdd\xcfHv\'\xcc\xf6(l\xe7u\x13\xfej\xfe\x02\n+N\xb9\x8f\xdff\xc7\xbfz\\_m\xcdW\xe5\xa5\x9e\xd3\xf3m{\xed\xd5\x9e\xda\xf7/\xbc\xbb\xb5\xadw\xae\x8d\xcf\xe8\x99\x0e\x89\xaaO\x93rCrP\xafP\xfa_\xaa\xf3O\xa0\xf7=\x85\xddd\xabKa^\xd9\xea*\xfe\xbc0c\x83X3\xee\xe4\xe7\x85\xb2\xbdL;(\xfa\x82\xd1s\x15\x07\x90Z\xc4\xcf\xc5\xf8\x0bC\xdf\xd0\x07\x87\x86a\'\xa1\xaa\xbe\x9c\x14-\xe7\xce\xb0BW\x9as\xf2\xebI<\xc1s\xdf\xae\xa3o\x10=\xc3 \xbeA\x8cu\xf1\x83E\xcb\xb0\x9aF\xe9(\x7f_\xd6\xd1\xd8\x1bnN\x8e=C\xc7P\x89O)\x1d\xde\\p\xf8\x86k\xab\xefI\x1d2\x0b9\xd8\x05\x83yB\x197\xf7\x9b\x87\x82\xde\x1c\xdf\xc48\x8cV\xc8\xf5\xf1\xda#\xb7\xee\xe9\xbb\xcd\xfc\xe5\x9b\x93\x9f\xba1\xf4\\\x97[\xbe\xed\x87\xbd9\xee\x8f\x1f\x8f\x87\xd5\xcb\xd3\xdd\xcc\x0b+y\xc9S>M\xfd0\xf6\x82\xbeD7,4SyZ\xd7\xf2\xaf\xf9Z\xdf\xcf\xbfA{\xf3\xfd\xcc\xd7z\xf3\xb7\xef\x97.\xf9\x87r^0jU.\x06\xa5\xe3e\x9dp*\xfc\x82\x07\x85\xb7\xfc\xc4/\xa0+4\n?\n\xae\x1d\xd8\x8f\x02\x8f\x08>\x98\xa2\xaf\xcf\xe47h\xdd(\xc2\x05\x86\x89\xd7\xf4\xa6\x07\xb9\xcc<\xa6k\xf2B\x9f)\xa5\x9eVP\x97L\xceR0\x80\x10\x03;FWDB\xdb\xf0\xb2H\xb6,\xd8f\x18\xf7#u{<\\u\x92\xd0\x0b\xc28\x01\xfc\x0c\xac\x07\x87\xcc\xf4\xe2+1r\x98\xf0\xf7\x00\x15\xc6\xe20\xd1=\xce\x8d0\x02\xf4|\xba\x95p\x83\xd0\x87\x1a\xaf\xbaD\x14;\xb0\x16\xfe\x0f\xfa<\xf8I\x18.\xab\xf6"\xa8\xc0\x15\x02\x83\xbe\x1ekK\x05\xa4:L7q\x8f\xa2t\xfa(\x9dk7B\xaf\xc6\xfb\xcb\xbe\xbf\xf4\x87\x15\xad\xf0\xc47zy:\xeeG\x16\xf1\xb8\xe0\x1a%\x01EG\x08\x9f\x9a\t\x8b\xb6\xdf\xb6\xc9<m\xfd<\xc4@u\xc9\xedi]r\x84zX\x8a\xa1M\x0cM\xf2\x87&\x9e\x10\xf5{\x1aw@B\x1d&^\xd0\xff<\x9e\xee\x1fypS/\xe8\xe5\xfe\x83\xd4\xc9\xd1\x1c\x97n\x99\x94\xf0\x1a\xd6\xe2\xf70\xf3r\x84+b\xd7\xe8\x8a\x8cWB\x04*\xafe\xe2\x16\xd1h\xe15\xf1\xfd\xc3a\xfa\x89\x17\xbfh\xcf6q\xe1Rx\x842j\xe9.\xeb\x1f\x1a\xb8\xf3_?\xf0v)\xaaa\xf6\n\x83~~\xf3+\x8c\x1e\xd3\x7f^\xfe~Qc\x1f\x0e\xa9\xb2J\x015\x11\x99\xe3L\x13\xb3\xe7\xa4\xd30\xc3\xd6\xc5\xe6:\x88\xd3\t\xdbt\xa2tC\xb7\x9e\xfbLa\x93b0\xc26\xc5Hu\xe09q\xe9\xa8\x94\x7f\xbd\xb7\x84\x970J\x10\xf6\xad\xb8\xf8O\xb8\xfa\x88\x81\xcbg\xc2_\xd4\xcf\x0e\x0b^\x1c)\x0e\\$^&\x07\x0e{_\x07f\x13O\x93?$\xf0\x81\x03\x8d\x83\xbcD\xc8\x81\xe9\xb2\xfd\xdb\x06\xfb\xe7\xfep\x89"\x9a\x19k\x91~5\xe6`^\xe6\xba\xd5\x9e\xf6\xc3\xd6z\x9a~\x95\x11\xf9\xb7\x99\x95\xfey\x1e\x8a\xf7\xf3O\xff}5\xf8\x80UU~\xceX\xa2\xd7\xc4\x17\xcd\x82\xda\xd8\xcch\xfe\x85o\xbd%\xfd1\xb1\xb9\xd3\xd9\xa1\x0cg\xa9\xa7\xa5T\xc1\xf8\xb7\xdf4~\xb4\xa2\xb3N\x8a\xed\xd43\xfa^|N\xdd\x0f\xc2,R\xfa\x92q\x94\xdc\x81\xfc\x9c\x1ac\xec\xdc\x92Ut\xe2\xb3\xe8\xb6\x18\xb6\xc6\x8b\x18;=1r\xd3i\xa0m\x9b\xa5.\xea\x92\x88yh\xe0\x1c?\x88\xcf\x99\xed\xfc4?(\xcbgz\xfc`9\xee3S\xd6}\xda\xc7\xfdr\xea\xe2x\xc4\t]Oij\xfe\x96Wl\xdcfb_\xa6\xcd\x9d\x17\xd7Y\x1c\xb4\xbf\xad\x13q\x86\xfe6}\xc3-\x7f:\x0e\xde\xda\x83\x1d}\x8aVK~j\xe7\x13\xdf\xe3\xfcv!\x8e\xa2\xb5\xc9\x04u\x10\x98\xe2\x83\x1b\xe2F\xfe\xcb\x9f\xff\x0f\x80\xf4\x08#'
    )

    #
    # CONSTRUCTOR
    #

    def __init__(
        self,
        background_color: typing.Optional[Color] = None,
        border_color: typing.Optional[Color] = None,
        border_dash_pattern: typing.List[int] = [],
        border_dash_phase: int = 0,
        border_width_bottom: int = 0,
        border_width_left: int = 0,
        border_width_right: int = 0,
        border_width_top: int = 0,
        dash_pattern: typing.List[int] = [],
        dash_phase: int = 0,
        fill_color: typing.Optional[Color] = None,
        horizontal_alignment: LayoutElement.HorizontalAlignment = LayoutElement.HorizontalAlignment.LEFT,
        line_width: int = 1,
        margin_bottom: int = 0,
        margin_left: int = 0,
        margin_right: int = 0,
        margin_top: int = 0,
        padding_bottom: int = 0,
        padding_left: int = 0,
        padding_right: int = 0,
        padding_top: int = 0,
        stroke_color: typing.Optional[Color] = X11Color.BLACK,
        vertical_alignment: LayoutElement.VerticalAlignment = LayoutElement.VerticalAlignment.TOP,
    ):
        """
        Initialize a map representation for a specified region.

        This constructor sets up the visual properties and layout options for the map, allowing
        customization of colors, borders, alignment, and other attributes.

        :param background_color: Optional background color of the map.
        :param border_color: Optional color for the border of the map.
        :param border_dash_pattern: Optional pattern for dashed borders.
        :param border_dash_phase: Optional phase for the dash pattern.
        :param border_width_bottom: Width of the bottom border.
        :param border_width_left: Width of the left border.
        :param border_width_right: Width of the right border.
        :param border_width_top: Width of the top border.
        :param dash_pattern: Optional pattern for dashed lines within the map.
        :param dash_phase: Optional phase for the dash pattern of lines.
        :param fill_color: Optional fill color for specific regions in the map.
        :param horizontal_alignment: Horizontal alignment of the map.
        :param line_width: Width of the lines drawn on the map.
        :param margin_bottom: Bottom margin around the map.
        :param margin_left: Left margin around the map.
        :param margin_right: Right margin around the map.
        :param margin_top: Top margin around the map.
        :param padding_bottom: Bottom padding inside the map.
        :param padding_left: Left padding inside the map.
        :param padding_right: Right padding inside the map.
        :param padding_top: Top padding inside the map.
        :param stroke_color: Color used for strokes on the map.
        :param vertical_alignment: Vertical alignment of the map.
        """
        import json
        import zlib

        super().__init__(
            geo_json=json.loads(
                zlib.decompress(MapOfOceania.__MAP_OF_OCEANIA_GEO_JSON_BYTES)
            ),
            background_color=background_color,
            border_color=border_color,
            border_dash_pattern=border_dash_pattern,
            border_dash_phase=border_dash_phase,
            border_width_bottom=border_width_bottom,
            border_width_left=border_width_left,
            border_width_right=border_width_right,
            border_width_top=border_width_top,
            dash_pattern=dash_pattern,
            dash_phase=dash_phase,
            fill_color=fill_color,
            horizontal_alignment=horizontal_alignment,
            line_width=line_width,
            margin_bottom=margin_bottom,
            margin_left=margin_left,
            margin_right=margin_right,
            margin_top=margin_top,
            padding_bottom=padding_bottom,
            padding_left=padding_left,
            padding_right=padding_right,
            padding_top=padding_top,
            stroke_color=stroke_color,
            vertical_alignment=vertical_alignment,
        )

    #
    # PRIVATE
    #

    #
    # PUBLIC
    #
