#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A specialized version of the Map class representing a map of South America.

This class inherits from the generic Map class and provides geographical data specific to South America.
It allows for rendering and customizing the appearance of South American countries, including setting
specific fill and stroke colors for individual countries.

Use this class to create and manipulate a map focused on South America within a PDF document.
"""
import typing

from borb.pdf.color.color import Color
from borb.pdf.color.x11_color import X11Color
from borb.pdf.layout_element.layout_element import LayoutElement
from borb.pdf.layout_element.shape.map import Map


class MapOfSouthAmerica(Map):
    """
    A specialized version of the Map class representing a map of South America.

    This class inherits from the generic Map class and provides geographical data specific to South America.
    It allows for rendering and customizing the appearance of South American countries, including setting
    specific fill and stroke colors for individual countries.

    Use this class to create and manipulate a map focused on South America within a PDF document.
    """

    __MAP_OF_SOUTH_AMERICA_GEO_JSON_BYTES: bytes = (
        b"x\xda\xad][\x8f$\xb5\x92\xfe+\xady\xd9]\tf\xd3\x97\xcc\xb4\xf7\x8dA\x03\x8b\xe0\x1c8\xa0a\x85\x0e\x08eeeA1Ew\x9f\xee*\xd0\x80\xf8\xef\x9bv\xd8\x8e\x8b\x9d5=G3\x0f5\x19\xb6\xd3\xf7tD|\x11\xe1\xfe\xf3\xd9\xf9\xcd\xfd\xf2\xec\x7fn\x9e}\xb2L\xe7\xcb\xc3\xf2\xf1\xdd\xe9\xb4\xcc\xe7\xe3\xdd\xed\xb3\x0fn\x9e\x1d \xf1q-\xf0\xcf?e\xd1P\xe0\xa7\xe5\xee\xd7\xe5\xfc\xf0fM\xc6\xfc\xbf]N\xe7\xe3Ww\xa77?A-\xf3\xdd\xdd\xc3\xfex;\x9d\xa1\xa2\xf5\xdf\x87\x83{>|p\xf3a\xaf\x9f\x0f?|p\x83\xb4}\xee\x81\x1e+\xba\x13\xb4\tt\xff\xdc\xe4\xf7\x15\xd0\xbc\xbeB\xfb\xe7\x1a\xe8>\xd2c\x07\xf5\xf5\xcf5\xd0*\xd3\nh\r\xf5\xdb\\\xded\xba\x03\xda>\x1f\xa1\xff.\xe7\xbb@\x9b\x9c\xaf\x9f[\xa0\xc7\\\xbf\x82\xf7Un\x7f\x80\xfc!\xd3\x06\xea\xf3\xb9\xbf\x89\xee\xd9x\xc2|\xfd\x10R\xfe\x19\xcb\x8445\xe2\x18C\x1b\xca\xe59\xf1qL\xca\xa7>\xaduX\xa0m\xa6C\x9fu\x87t\x98#\xadr\x9b#\xe4k\x9cs\x05\xf4H\xd7D\xe71\xa75\xd1\x96\xb7\xa7-\x8e!\xd2C\x9a\xf34\xa6\x95.\xf9\x06hO\xfb\xaf\xc7\x9c\xef\xe3\x9ck\xc7\xd7P\x97\xf1\xf8\xe7~\xa5M\x97\xc6\xbf\xe6\xf7\x81V)\x7f\xa5C\xff\x8dIk\xb0\x96w@\x1bF\xdb\xbc'\xba\xd8_\xd3\x0bz\xc8k\x0ck\xba\xd2l\x8d\xcd\x88k\x1a\xebs\x99V\xf0\xbeK\xe3[\xe90\x1e\xe3\xf3\x1eR\xb1\xff\xb6C:\xe4[\x8d{R\x01mXy\x93\xc7\xa7\xe2xW\xba\xbc\x1f\xda\xb7\x16\xf7\xb8\x01\xba\x94\xd7@\xf3\xf6z\x1c\xdf\x00\xf4\xc0\xda\xcb\xeb\x93\xf6\xb8\x1d\xf3\xf8\xe1\x9b\xb1.\xb7\xa7\xe1\xfd2^\x03\xe5}\xee\x7f\xfa\xa6\xf2\xfe[\xe9\x0ehV_E+\xec\xbf\x87o\xa2\xcb\xeb\xd7\x03\xad\xc47cD\xbe\xa1\xfb\xa5|ci\xbd\xca7\x98\xcf\x042\x9f\xe9\x9b\xf6t|+]\xce\x08y&p\xda\xe6\xf6\xd2\xf8\xfb<\x9e!\xd3\x1d\xd0]\xa6{6?}\x9e\xcf1\xd3\x9a\xcd\xbf\x85\xfdQ\xd6'\x95\x1fr\xfd6\xafo\xe9\x8f\x85\xf5W\xb9\xbfi?\xf4y|i\xffY\xb6~\x1a\xcb\x8fl\xffYX\x7f\x93\xd7\xdf\xc6\xf1X\xc5\xe7c\xdd\xef\x9e\xb6g\xc8~\xe8\xe1\xfb(\xf4\x00\xdf\x93f\xe5G\xfc\x1e\x06\xf8>{\xba\x1f\x8c\xc1\xf5\x8b\xdf\x9b\xc6\xfd2\xc2\xf9\xc0\xf3\xbb\\\x1f|?\xba\xec\xd7.\xd6\xa7\xc9\xf7\x1c\xcf\x9f>\xcf7\x9c\x07\x9a\x9c\xe1\n\xce\xcfr\xdeh8o\x1d-O\xcfg\x0ft9\x8f\xcay\xbe\xfe\xfbk\xe5\x9c\xf7\x0fw\xf7\xcb\xc3\xf9\x18\x19\xe7\x9f\xcfn\xa7_#\x83\xfd\xf8\xe7\xe3)\xb2\xdf\x90\xf0\xe3\xe9\xee\xf6'\x96z\xb8{\xf8u:\xfd\xb8\xdc\x86\xd4\xaf\x97\xfb\xcb\xeet\x9co\xee\x0e7\xb2\xc4\xe1a-q{9\x9drU\xf3q\xfa\xfdP\xb7\xf0x\xf7p\xaeS\xa7\xd3\x19_?\x1c\xef\x1f\x7fT],\xf5Y(r|\xbc\xfbq\xd2?.?\xc7\xa4/J\x92\xc9I\xff\xfb\x05\xd6\x14\xfa\xf1\xec\xfbK7\xe8)\xfc\x1a\x1b~m|\xb6\xe9\xb9\x14\xde\xdd\xa6\xc2\xde\x87\x02~w\x88\xbf\x1a\x9eK\xb1}c\xa6`FD\xdac\x9dv\x98r\x7f\xa0'\xf3\x8c=\x99g,\xf6\x90_=bu\xa7\xf4\xaa\x99\xc6\xf0\xbb\xf3\xf1w\x17S\xb0\x85\x9f\x97T\xac_\x86\xff\x08\xff\xed}\xfc\x9d\xe3\xaf\xc5r\xc7<X\x15\x07k\xe2`M\x1c\xac\xed\xb0\xd8\xa5\x1e\xc4q_\xf7\xee\x08\xcbH\x8b\xfd\x92\xc6j\xbaY\x85\xdf\x05g\xfa\xf5\x1dd\xcd\xcb\xd0}\x7f\xd9\xb9\xce\x95\xac\xdbS]\xf9\xfd\xa9\xee\xc4}c\xdf<\\\xd2\x98\xac\x0eSd\x8d\x8b\xbf;x\xc6M\xf7[\xfd\xea9o\x14\xd5/\xb4\xe9\xcb\xeb\xed*\xfb\x01\x8b\x95}\xe6\x86\xe6\x8a\xfev\xac\xdb\xfc\xe3gxi\x18\xc6u\tz\xad=\xc9:Wy\x7f\xad\xdf\xed\xd3E\xe5kRrfXJ\xa1@\xe5\x04m\x04\x1d\x05\xbe\x0e\x054\rt:p\x86x\xc0\xe4\xf3w\x00\xfe\x93\x8f\xa7\x95\xb4P\xba\xa7\xd9\xaa\xc3\xe2\x16ZK\xe2&\x1c\xf7\xaa\x88\xa8p<\xab\"\xd2k\xe8m\x11\x0fA|Q\x99=\x0cp\x1c\xab\xcc>\x07\x95\xf3\x13\r\xe2\x9b\xca\xec7\xd3Y|\x19@d^\xe9\x81\xd1Y\x84\xcf\xe5\t\x1d\xfb7\xa4\xd9\xe8]E[\xa0}\xa6\r\x1c\xc7)\x7f\x84\xfefv\xb0\xd2=\x1c\xdf\x9a\xe5gq{\xa5=\x15\xb7\x13\xbd\x8a\xdb\x1dm\x7f\xa55\xebO^\x9e>\x89\xf3\x1e\xc7\x9b\xc4y\x93\xe7\xcb\x01]\xe6\xdb@}=\xa3U\x1e?\xb0\xf3U|\xd7t}t\x11\x9f\x80]S\xda\x02]\xf6G\xcaWy7\xa5\xfa\xdd\x96z\xc0\xd5\x07\xa9^\xd4\xea\x07WO\xa4\xfaR\xab7R\xfd\xe9`\xfd\x8a\xba`a\xfdG\xaa\x02\xaa\x1e\xdfO\xfb\xa5|=\x1e\xf6S\xcfh\x83*\xa5\x87\xfd]\xc63\xb2\xfdN\xbe\xd6\xab\xdc\xfb\xc5\xdd\xe9\xf8\xdbq\xaa\xf87Ig\x1c\xfc\xab\xd3\xe5!\x9c\x0b\xabN>\x9dn\xbe9\xaf'D`\xe6u\xf1mv.\x9b\xcc\x0c]\xa6o\xb1\xf4\x17_T,\xfd\xc5\x97\x15K\x7f\xf1e\x93\xa5\xbbx\xd4:\xca\xcc\xe3\xaf\xc2g=6\xd8\xfb4S\xc6\x1eS\xf6\xe4\x19\xd8>\x94Y\x04\xdb\x87q-\xb7\x82\xf3\xcb\xe1\x02\xef\x97\xa9\xc8\xfd\xeb\xae'I\xc05%\x81\xd4jC\x16\xf0:J\x01\x07\x94\x05\x92\\\x10\xd3'HW\r\xe9`\xaf\xe2oO\x84\x03O\xd2\xfd\xa6\xb8\xa0\xe3\xc4\xd8\x1d\x8a\x0bIt\xe8\xf1YC\xca\"\xc4\x88u\x18kz\xb7\xec\xe9\xa4\x804!\xa7\xea\xd8\xdcE(R\x84.\x07\x91\">\x9b\xf0;\xe9J\xbc\xd8\xcd\x87\xb5\xd8.\xae\xf7n\t\xf3<\xf7\xd6\nQC\xb6q_R\x7f\xa7\xa9\xe7\xcd\x01\xa0\xd4\xa1T\x14\x0e\x16*u\xc4_\x8d\xcf\xf6 \xe4\x10\xd9\xfe\x19W\xfc\xcd\xd4\x90C\xeaF\xfa\x01\x1b\x81g\xd2\xc8\xe5\xe1\xa9{N|. \xb1\xc8\xdee\x99e\x8c\xbb-\xc8%\xeb\xf3rX\xab\xb3\x8b\x9fj\xf9\x85\x95\xdb\x87\xcfl-\xb7\xbcgY&\x82\x07\x88\xcd8NB.\xd3\xd42g_\x15\xbd\xa88\xa3^\x18\xf5\xe4\xac\x86\x81\xdc1\xa0V\xaa\"9P\xa5s@\x9d3\x90\x99\xa7'\x15uD\r>\xb0\x14\xc7\x01\x89\xcc\x80RC\x1e\xd5\xe3\x81\x93I\xd8R\x14mA\x1a\xc4\x11\x8f`\x03\x93\xed\x92\xf2\xa8\x14*\x83R\xf6\xe3\xdc\xa6\xe6FMne\x91\x1brnWsC\xc9-%7\x95\xdc\x96)\xafR\xb9M\xca\xf5Z\xdeQ\xe5z\xa5\x19\x98\xb0\xd6o)\x98Q\xfa3\x0e\xd0~\xd9\x03\x03\x94/`\x0cH\x1fEZ\x1cA\xfaPY\xba\x19\x01\x10V\x05<\x81\xf18$m\\x\x9fI\x17Ix\xd9\xc1\xe2\xe4\xb1;\xc5v\x98\xeb\xd2v\x1csn\x84E\x90\x8c(L\x9ag\x07Ri\xc6H\x1c\x08\xa1y\x14\xa9!\x8b5[\xf2\x198\x90\xf8\xcaG\x02KP\xe0\x99\x04i\xe7\r\xe4x\xae\x030%\x0f\x10\x84\xb1\x02O\x83&\xa0\x11y\x8a\x8b\x83d\xa8\xb9C\x1c\xc9FRgRERQ\x14)K\x9d\t\x84R\xf8\xcd\xc4m\x8b\x9fn\x94\x113\x99DT\xfc\xec\xa3D\x88x\xa5\xa5\x85\xbb\x94k\x18\xfa\x8b`L\x04w86St\x07<}\xae\x8af_-+\x9b\x90rYN\xdc\x84UD\x81m1\x8cU\x9fe0\x96\xb8%\x80}\xf5\xb2\x12\xc0H\x92)I_o\x0b`\x80\xac\x80\xd0\xe5Z\xe2V\x04\x18\xe61\nT]|VB\xacb]M\xc2)KzLI\x81\xfd\x1eZ\x12\xd5\xb84{\x01\xf2\xd3W\x91k\xfb\x87\xbbK\x0bN\xe9\xa2\x90\x14\xc4\x17\x13\x90\x8a,N\xcd\xfb\x16\xa8b\xe3\xaf\x03\xe1\xa9%\x1e\xc5\xc1\xda8X\x13\x07k\xb5\x10\x83\xd8\xc8@\xfc\xe1Ig:X\xdf\x92~\xa2\xdc\xb3\xec\xc2\xefa\xae\xe4\x9e\xbd\xf1.\xc0*\x8b\x84UX3\xf7\x8d\xa4z\xdf\x10\xe9\xe6\x10\xc5\x8c>\n\x18]\xfc5B\x96ao\x82 \xc3\x92\x88\x14s\xad2\x94Y`UA6\xa9\xd6\x16$\x14\xd6@\x11O\xfc~\x9d\x01?;\xd5\x10I o7\x1c\xdeQ\x0c\xf9\xf7\xac\x8f\x1a\x90|EU\xd7\x82\xf4\xaf\xaanO\xad\x81I\x15\xee\xf3\x99?$\xa4>[\x8ar\xf9B\xd7\xd6Kn\xddtm\x9aZ\xfbzx\xc7t\x087x\xb0n\x158A\x01\xda\xcdi\x83\xb4\xa1\xd6\xae\x04w\x14\xb4\xbcO\x0c\xc3\"\\\xa1\x01]/\xf0\x07\xb3~\xf5\x03\x1c\xb8\x99\x81\xaf\xb4\x03\xdaS8\xc4\x14x\x04\xd8\x95\xc9\xd6\x8bAe\xdaS\xb8b\xa5\x1d\x85\x8bL\x81\x03\x92u\xa9C\xf8B\x01=Rx\xc3\x16\x81\t,\xb2+\xad3\x9c1R\xeb\xda\x00l\xcc\x16\x81\x0b\xd6\xb4X7\x12\xbcU\xac\x1f\xb9>\x8d\xf5\xf7@\x0f\x14.\xb1\x1a\xe12\r\xd6\x10\x06\xa7Y\xb2\x87\x06jmK{\x06i\xb0\xa6\x16\xeb[\xda3v\xe0`\x9e\x1ds\xf9>\xd3e\x0fv`-R\x14\x0c,\xd6\xa3\xb4\xc7\xad\xe7pF\xdf\xa1@\xa8\xa8\xf5-\xc15}\x81{\x92\xc5]\xa3\x05~\xc3\xda\xa6\x9a\xd6\xba\xca\x9a'\xad}\xd2\x1aX[\x0b\xb95QZ\x1bkk$\xb7VJkfm\xed\x94\xd6Pi-\x95\xd6Tim\x95\xd6Xn\xad\xad\xad\xb9\xdc\xda\xdb\xb6\x06Sk1\xb7&\xd7\xd6fn\x8d\x96\xd6ji\xcd\x16\xd6ni\r\x97\xd6riM\x97\xd6\xf6\x865^Z\xeb\x995\xbf\xb2\xf6Ko\x80\xda[@z\x13po\x03\xe9\x8dP{+p\xb8R\xc2\x99\x12\xee\x94p\xa8\x84K%\x9c*\xe1\xd6$\xbe\x16\x9e\x92\xe6\xc3\xe2\xf9\xec\x80v\xf9\xfct`m,\xe7\xfd\x00\xf4\x90\xcb\x0f0_\x8a\xe5\x8fx\x1e\xf7t>Wf4\xb2|\x0b\xf5gEy\xa5\x07j\xdd\xecA\x90\xd7\xd9\xba\xda\x9b<\xffJ\xd0>\x97\x97\xedi\xa0}\xee\x8f\xa1\xd6U\xca\xcf\xae\x0b\xe5\x1f=\xfc\xb4\xdc\x9eW\x0e^I\xe6,\x87\x89\xe79g\xb9\xc9\x82\xfa\x93\x04\xf4\xba\xa9,\xa5\xd79[\xa2\xfaG_W\xa2:I2%\xe9\xd3\x96\xa8>\"\xec\xa3\r\x8aS\x1a \xa0\x01M\xa4\tC\x1d\x1a\x82|\xb4me\x11>\"\xa6~F\xd1~r$\xfd@\xf0T\xd7DR\xf3\xb8k0\xb5\x9e\x11\x10\xff\xebt\x02\xa8j\x1c\x93wd\xc4dd\t\xee\x1a\x84rPV\xb4\x05\xaf*\xa2\x15\x18\xd4\x13v\xfb\x98n\t\xe0\xba\xdf4\xc4\xee;\xa23\xe8\xf8\x1cS\xf6\x0e\xa1\xd6\x94\xd2\x02\\\xbb\x9e\xe8\x12qv\xd5\x8c:F\\\xcc\x9c~\x00\xebmLwM\x08\x16\x06\x0b \xe6\xad\x04a\xeb\t>nlQTE&\x8d\xaa\xc8.\x82\xb2\x07\x836\xdf\x83\xa9m\xbe}\xd8\x85;?\xae\xbf\xfb8+{}\x08V\xe0u\x7f\tu\xa5l\x91\xd8\xdf\x9d\xd0\\ \xf7\xcd\xad\x04i\xeb\xde\x12\x1d\xa6C\x85\xc3\x18TA\xcc>\xa6\x13\x9c\x16RL'4\x9c\xba\xf2s\xdaD\xbfL!\xbd\xa5\xea\xfc\xfbm^\xc4\xf7[\x7f\xb3\xa3\xc7\\=\x88\xfd\r\nR\xdd\xe5\xac%\xf9!\xc0\xf5\x833k%\xfdr\x18k]\xa9*\xf1\x1e\x81[8\xd7\xb3\x94\xd2G\xa9Y\xe3\x19?\x04\xaap\x9c\x8e\xe4\rPR\xe7\x92>P\x1d)\x999-p*\x85|A\x87\x92\x85\x0b\xae\\\xc3\xa0\x0e2\x10*\x9a\xbb-\xf2?\x1f\xa8\x81Q\xb4\x96\x1eK\xae=+zL\xecY\xf6\x89Z)\xc7\xf2:BE\x8e_J\xc6y\xb1\xc8\xfdl\xa04)i\x90\xb3\xea@\x95<\x13\xc67\x92Z\xf4\xdbp\xa9o\xa2\x05\xf0\xd7\xda\xe7\x87fl\xe2S\x8dB\xdb,\xb0j*s\xc0*c\x8b\x01\xfe\xfd\x9b\x8a\x01~S3\xc0o^\xb5\xb0\xaa\xf8\xe5%[\x87Q\x94\xd1\x11f\xd17\x98\xde\x0e\xd8\x9aB\xd6\xc7\xd9Z|\x96\xcc\xad\x1a\x12\xcc]\x9d\xfcH\x92[^BU\xaf\xe7\xf9J\xaf\x81\xa1\xd5\xad \xd6e\x08\xbeE\x11/\xca\xc3\xe6\xf8<\xb7\xd0/b4\\j\xee\x05\xbf\xfb\x06\x0f\x8b\xe7\x9b\xb7\x9ap2\xc6\xa5\xe2\xb3\xe4U\xd58\x80K\xd5\xc9\xed]\x84<*\x8c.\x1b\x0bg\x80\xce\xba\x9a/i\xe5\x8a\xb1\xb0\x9b$/\xaa\xaa\xbf?\xb5\x96\xee\xbe\xdd\x17\xe2\x99\xa4\x00\xec\"<\x81\x9d\xfd\xf1w\x16\\G\xb6r~h\xa5\x12_\xa5wk\xe4\"\xbe\x92'\xec4`-\xd583gqz^\xf7\xd8*\xa8\xacu\xf4\xa6op\x167tc6\x0c\xc6\x12\xef\x93\xb3\xf0s?\xe2G\x84\x0b\x18F\x8d\x81\x1a\t\xb2\x95\xddH\x00\xc7\"T\xac\xd3d\xcc\xa9c\xd4@\xb8N\xd4?\x89v\xd6\x11\xae\x13u\xddr~{8\xe9;\x92G\x9c\x90\x14r\x1d\xa8\xa5 8\xd1\xd6\xd3c\x0b#\xa1\xa2\xf6\x9e9\x0b\xb8>\r\x98\xa7\x035\x127\xa6\x01\xb5\xc6u^Fla\x08\x94e\x14\x1dC\xb6w\x813\x91C]\xb3\x0f%\r\xa1\x86\xfc\x9e\x83\xbe8\xe2\xe44 \xff\xa5\x9cSrU\xceq)7\x96\x9cz`<\x9dsx\xca\xfd\xcb\x0e\xb9\xca\x1d?\xbd\xbc\x99\x1a\xda!&3\xce\xf8\xf1\xdd\x87\xa1\x96\xe9|\xfc\r\xb5\xc3\xc0&\xab\xf2\xdbLR\xb4\x98Y\xa4H\xdeb\x90\x9f~W1H\x92\x94\x19\xe4\xa7\xaf\xbek1\xc8\x89z\xcd\xd0O\xbf\xe5 ;\x12\xe6\xc7\xfce\xaeh|b\x100i2\xf1\xb1\x91\x88\xfc0\xb6%<\x16\x9aJ\x9d\xac\x165\xba\x8a\x03\x02\xefK\\o\xbf\xed0\xa3\x89cL\xf7\x16\x9dM\x8d\xd4\r\xe6\x8a>&\xfa\t\x1cN&\xb6\xb6\x00\xd1\xc0\xa2\xd65Y\xd4\xc6\xe6\xda\xd3v\x9a\xbb\xb5\xb7\xf38\xda\xa2\x81\xd5\xfa\x96h\xe2>%\xfe2U:\xd6\xa7\x97\xe3\xd4V\xb0\x0c\xe10\x9e<\xb7\xd5)\xd1\xe2\xb9\xb5vD\x93\"\x95\xdb\xc3[4&\xd8*|\x934]\\Ds\x99\x85\xf5\xa3\x06o\x95\xf5\x8b\xf0\xdddj\x16VJ,\xa9\xc4\xfbda&\x1bQ\\\x06\xc5F\xa0u\xa65\x18e\xc6L;f\xa4\x01\xd0\xcd(\xa6Pa\x88\x02(J\x84\x1e3\xad*\x10\xad\t\xb2I\x10N\x82t\x12\xc4\x93 \x9f\x04\x01k\x90\xd02\x90\xd2f\xd0u\x10t\x19\xef\x08 \xa8#\n\x16\xc9\xef3\x88[\xe8\x9e\x86X\xa4\xf9)\xa0-\xa8k\x81\xe6\xa0g\x06mW\xda\x03m\x98\x0f\xaefL+\xf8\xa0\x8em\x9f\\\xe9\xb3[\xfb\xf4J\x9f\xdf\xa6O\xf0\x88\x8c\x90\xfb\x14\xd7>\xc7\xcc'Y\xfa,7|\x9aM\x8b&\x02\x89\xf4\x99\xe6>\xd5\xd2\xe7Z\xfadK\x9fm\xe9\xd3-}\xbe\xa5O\xb8\xf4\x19\xdf\xf0)\x17>\xe7\xc5%]8\xacK\x7fv\xe9\xef^\xfb\xc3s\x7fy\x7f\xdd\xe7\xaa\xe1\x93\xc5\\\xb6\xa4G\x97\xf4\xf8\x12\x0ea\xc2]\xacr&c\xaef\x95#\x1asS\xab\x9c\xd8\x84\x8b\x9bp\x80c\xeeq\xc2yN\xb8\xd6\t\xc7;\xee\x96'\x9c\xf6\xb2\x8f\x18\xba\x80\x01\xc9\xc2\x97:\x1e\x9d\xd9a\xf0\xe4Jvh\xec\xe9\x035\x12\xdf\xb4\x8e\x85\xea\x925\x8bj\x81bT1azF\xf5\x081\rI\x81\x18\x89\xb1G\xa1\xe7\xbaG\xb5\x00\x0c?\x1d3\xe5w,&B\xa1\th \xefY\xe8g\x9f)\x85\n\xcb\x10\xf9BQ.\x12e\xc97\xa2\xf1=\x1b\xc4_\xc5\xa8bbr(6C\xeb\x852 `\x97\xefP\x91\xf74\xbcG\xe3.,*\x17Q\x14\xef\x89\xca`Q\xb9\x18\x99\xc2\xc2\xd5\x17\xae\xda\x10\xb5\x87\xabD-u\xc90\x05I7U\xb0J=\xa3\xaa\x1bW\xeb\xa4\xcaG\xd5A\xae*r5\x92\xab\x98R\xfd\xa4\x90\xa4\x84+)\x94\xc9aN\x0e\x81rx\x94C\xa7\x16\xf2\x14\x91\x04\xca{\x06j1\x99\xb2\xa4\xa4\x86\xbe\x14j -h\x06-\xc6o\xdb\"e\x18\x15wH\x9a\x97\x8e\x8d\xbd\x83\xb1C\x9e\xf5\xf0ut9o\x0c\xdf\x83\xce\x94%\xbe\x7f\xd6\xa5O^S2{\\X\xb0`\xe6\xef\xda\x0e\xa904j!\x1a6\x1f&6I\x12\xb9\xe6$h\xe4>\x99\x94\x0b\xd3eS\xa4f\xae*\xbb\x02\x02i\xb2\xeb#\xbck\x92\xe3CZZ\x93|sRC\x06$\nB\x82S%\xcc\xaf\x81Nf]\xdcXv,\x1bp!\xc9Gz\xaa*\xc7\xe3\x18\x90\xd5\n71c\xf6\xe0\xd5\xb9\x9b\x96\xf2R\xe32\xefM]I\x1e\xb8\xf9\xab6\xc9\xa37\x1f\xec&y\xf0f\x87\x0f\xe3\xb3l\xe13=P\x8f\\\xe3y\xbc\x90\xedr\xfcM\x99E\x0f\x06p\x9f'\x99\xc9V\xc1\xf3\x85\x1a\xd0m\xf2\xe5,tZ\xb2\xec\xe0`\xfb\x1c\x1e\xeb\xf2\xfa\xf7 \xdb\xa9\xbc=\x06\xa0}\xde==\xc8\x92\x9e\xee\xa6b\xa0\xce\xf9#\xee\xb6\xe4@Ph\x0ft\xd9\xc7\x038\x1ch\xb2\x93\x89\xc3\x94\xca\xb2\xb7\xcb\xdf\x92\x01\xd9\\gZ\xca\xf2E\xd6\xbf\x1e\xdf\xf30\xfdq<\xd5\xe1=%\x99\xc1\x11\x9f,\xfb\x16\x18Q\x95\xbe\x12\xdc\xc3\xdb+\xb1=<y3\xb4\xa76W\xbf\xa8\xd1\xfa\x17_\x7f\xf46s5A\xbf\x93\x19L\xd3(\xde\xcd\x00\x1f0E'\xcc~Asu\x8e\xeb\x95\xa1=\x0f\xd3\xe3\xf1\xd4\x88\xed\xe1\xc3M\xa1=\xb1\xf0fd\x0f\xf4\x16\xfa\x99c|eL\xcf\x038\xa5\xd2jd\\O\x02\xe7\xe1w\xa8\xa2~\xdf\x16\xdd\x93 \xfa\x81\x86\x01oG\xf4\xec\x11\x9c\x07\xc8\x02\xcc\xccfF\xb8\xdeHW\xd607`L>\xd5!=|\x8a\x8egLl\x81\xf4\xa1\x9bf\x9d\x8f\x08\xd8;0*\xd7\x11=K\x17\xe0\xf9q\\\x7fg?K\x00\x03\x96\xaam.\x0eyoNuH\x0f\xef\xa5\x0c\xe6I\xe89\xfc6\x82\x8a\xdb!=\xd5NJA=\x0f\xcb\xda\xbd+Q=\xd7[{klO\xfd\x8dT{/E\xf5\xf0A\x17\xc8c\x1f\x02y\x9c\x1f\x0f\r\xb0\x03\xf3\xde'\xccQ\xfb\x9er\x18B\xc2\x14\x12\xc6\x900\x87\x84Aj\x98d\xf3\xe8\xa5\xef[\xf4e\xf2\xe0\xbb\xd6Q_'c\x11G\xd6@{\x82c3_\xd7T_\xbf\xe5\x1b+}g\xa5o-\xf7\xbd\x95\xbe\xb9\xcc\xd7\xe9\n\xebx\xf5p\xf9\xe92\xbd\xa9x\x07Ig\xcc\xe3\xcb\x87u\xfb\x9e\xa7\x13c\x1du\xe1m\xde!\x1b\xcc\xccC\xa6oq\x8fW5\x94\xfd\xaa\x86\xb2_}\xfd\xdd\x93\x9c\x9d\x98\xdd\xd7\x11\xb8\x9b\xba\n\xb5n\x82p\x9eF- \xf4\r\xcf\x14\xfa\x9eG\xc1O\xe4Hab\xab\xd4\xc7V*\xe1'\xa3\xec9\xc5\xc0\x93\xcb\xc7(xK\xd5\x08\xf2\x95\xda\xf4K\xa1\xf0\n\x1cO\xb0\xf8\xb8\xe9\xd6\xc4\x8c\xc1=\x01\xca{\xf2\x0b%}\xcb\xb9\xc9\x13\x93\xb0B\xe8<q\xa3\x1e\x1c\x9d\x04\xd7\x91\xc3\x03~S\xa56\xf7\x1b\xc1\xcd\x07\xe2\xb9\xd4!z>\xd9\xda6<\xec:\x08\xa8\x08Hz\x08*\rH\xba`>\xb2\xa5\xfb\x92\xfa\xfb\xf4\x8b\xe0:P\xf6\xd8\xb2\x0f\x1b\x1a\x12A|\x864\x05\xd6\x05\xc3\x91M\x9f\x9b\xbb\x80\xd8\x87\xdf\xad\x11\xe434b\x94r\x9bzO\xee\xb5\xe09\xb27\x99\xe9\xd8\xc5\xae\x95\x0ez\xf6\x80\xa87\x026\xba`L&%\xde#\x03\xaaC\xd0D\x80\x1a\x0b_\x13\xc1m\"\xf4M\x04\xc6\xb9\xabAu<\xe4N\x04\xe4\xd5\xe1z\"\x98O\x84\xfa\xd1@\xc0\x14q\x989\x9e\xcb*\xac\xa65k\x8c)\x14\xb9\xf0nG\xe3\x11\x15\xb6K\xf5l\x97UvC\x1a\xea\xb0\xcb\n\xd1\xae\xd1\x03*P&*\xe2\x006\xcf\xa2'%G@\x05\ne\xb1N\x98\xfe\x0e\xe38\r\xc9\x8bx}\x01LqU\xafr\xc5\x97\xf3e\xda\xdf=T\\\x91\xa4o\xfa>\xd5e\xb6\x99\xa1l'3C\x99\xbe\xc5\x0c_~\\1C\x92dJ\xd2\xab\xb71\xc3>>\x1b\xc9\xfa\xf4\x81~\xd6-6\x18\x0fe\xdfo\xb1\xbe\x98\x92\\\xa4\x04\x1b\x94c\x84\x99\xacR\x1f[\xa9\x92\rN\xfe\x89=?\xe4\xe1w\xb3\xff\xd7e\xfd\xde/\x0f-f\x08Lo\xaa\xd8 0\xc6\xe8\xdd\x1bO<3\xabm\xbf\xde\xb1bw\x862\xc6\x16\xd3\x8b\xb6`\xd5KF\x97,\xc5\x8a\\\xb0 ob\x923\x04\xac\xef\xe5k\x91\xda\xdc]\x84\xf5E\xfd*,a1\x19\xfb\r\x8dk\x8e\xc3\x9c\x0fc`\x80\xd1\xe39\xb8\xee\n\xd6'[\x02\xd6\xf7\xf2\xf5\xef<\x15z\xf5/^\x96\xb0>0\xe6N\x15'\xb2x\xe3\x81\x95\x16d\xd94\xb0\xbe\x97\xaf\x7fc\xa9D\xd1\xea\xdf\xa9\x11\xe9\x81\x0b\x0c\xb0\xde\x87\xce5\xf7!\xb0>\xd9\xc7\xa2o\x19\xb7\xb6:\xdap\x99E\x1f\xef\xe1\xeage\x1b\xbaWUn\xd4\xe6\xfd^\xa2\xc0\r\x17\xdc\xa8!\r\x1e\xdc\x18B\r%\xdc\x88\"\r,\xdc\xf8\xc2\r3\xc4h\xc3\r:\xc2\xda#lA\xb5\xa5H\xd8\x91\xc4\xd5\x104t\xbb\x0e\xec\x16a\xdf,(\\\x84\x8c\x8b\x80r\x11n.\x82\xd1E\xa8\xba\x08d\x17a\xee\"\x08\xbe\x92O(\x9b\xe3,\x90\xb3G\xce:%[\xa5,\x97\xb3cO!{\x10b\x14\x96\x1c\xd1\x980:0&\x0c\xa4\xce2{q\x97\x18\xec\x0b1\"A^\xb9\x1br\x04c\x02}\xaf\xc7:\r\xfaE\x8f\xc9cl$\xed\x95\x1b\x1c\xa3(6b\x9e\rT\xa9E\x07\xca\x93Z\x1c^\xee`\x035\xe6\x19\x8c\x9emE\xda[{\xed\xf1N\xcd1P\x96Q\x8e\x88\x89\xaa\xc3\xeb0\xc3\x84*\xbc\xc7\xc0pRG\xd2\x90hC\xa5\xd8\xd5\xa6J\xa1\x8d5\x14\xd6\x18*\xe8\"i\xc9\xa6\xa4\xa4\x8a$\xbd'D\x95\xc5\x8b({1\x98\xa7\x9a\x1531\xab\x0e\x0b\xfbHR\x13\xb3g\x01\x93\x1e{\xec\x02\xa5\xc8\xd5\x9e\x0e\x1b\xb1\xe8*\x98\xa9\x12\xa6\xd9\x93E\xd2\xb0d\x96tn\xc4\x00\xcc\x0e]\x13\xe1\xb3\x1d\xd1\xb2\xae\x88\xdb\xa2\x07\xe7GE\x02\xffH\x18\x9f!T\xfc \x06<\xf0\x0cy\x8fmB(I\x02\xae\x1d\xb1r&\x87CGj1,L\xd0`\x0b\x0e\xed\x93\xc3H?\x9d!\xb9&\x1aaC\xbe~S\xe7\xdd\xe9\xee\xd7]\xe3\xb2/\x9a\xb1}_g]\xe8\xca\x95\x9d\xb2\xa9rk\xa7\xcc\xd8\xbc\xb8\xf3\xcb\xfa\xe2\xce\xfa\x96\xaf\x8f\x9b\xb7|Qi5\xdd\xf2\x05\xcf\xbd\xbc\x82\xa2\xed\xa9\xd8\xe3]_\xd3\x82f\x81i~\xe2\x8d_\x9f\xdf\x9d.\xeb +\xb3@5\xf8t\xdf\xa7L&\xce\x8b\x9e\x8c\x81\xf4\xbe\xbaz)]\xff\t55\xc3\xd2\xa6\xea\xd6/x\x9e\x9ft\xf7\x17\x13Yg\xf2\xbc<\xe9\x06\xb0$\xb8\x92\x1b\xc0\xf4\x82\xa2,X\x13\xae\xde\x03\x96\xa6T\x1a\x0c>\x97Swl\xef2\xe2\xd8\x1fC\xcd\x96=\x86\x9dm\xde\x056\x1f\xfa`9p\xc3\xb5\xbb\xc0\xaa\xa6@\x90\xad\xfa{\x9f;\x16o\xe8\xb0,\x8fX\x10&ySW\xc2Z\xe6\xf8\xab\xaeX\x10\xaa\x8e\x80X\x9bf\xa8mAxBkW,\x08T\xa4}\xc2\x16M\xf7\x99\xcan\x16\xc9\xd6.\xeb\xdb\xf6\x10\xb0\xa0a\xb7\xdf\xbc\x1e\x0c\xca\xf5\x9d\xde\x91r\xefU\xb2\xddv\xbc\x1b\xdb\x8e{\xd2\xb1O:\xfeI\xc7\xc0\xdaqP:\x16J\xc7\xc3\xee\xaa\xe3b\xed\xd8(\x1d\x1f\xa5c\xa4t\x9c\xe4\xd1\xd72:[Fo\xcb\xe8\xee\x8d\xe8\xef\x1e\xe3\xdfx\xf4\xb8\x88.\x97\xd1\xe7Utz}Y\xa8\xbcL\x94_6*/#\x15\x97\x95\xca\xcbL\xab\xcbN\xc9\xfa_\xbf\xa1iz\x98\x9a&\x12\x9a\xb1}SS]\xe8\xcamM\xb2\xa9rc\x93\xcc\xd8\xbc\xb5\xe9\xa3\xfa\xd6\xa6\xda:\xf2U\xdb:\xe2\xaa\x80\xd2\xf1\x1d,\"\xe96\xa7=a\x9cKeg\x7f\xaa\x8d\xa4\x1ap\xba\xfb\xa9J~l&\xcb\xeb\x9f\xeaA=\xc1XR7&\xaf\x86R\x95-\xfe]\xec$\xec\xca\xa8w\xb1\x8d\xa4\xab\xa4\\e\x97\x7f\xbb\x85\xa4\x1aS\xbaf\xaaJno;b\x95W\xc4*\x7f\xddH\xb27h\x9b\xdf4\x92TM\xddcrm&I\xa5\x8f\xdb\xb7Qu\x95\xd9\xfc\xed\xd6\x92\xaa\x0f\xe7\xf6>\xa8n\xabz\xb7\xc6\xae\xddb\xf5d\xabI\xd5)n\xab\xbff6\xa9J\xbcO\xbcH:tRgO\xee\x08\xca\x9dD\xb9\x03)w.\xe5\x8e\xa7\xdc)U:\xacrgV\xee\xe8J\x9d`\xb9\x83,w\x9e\xe5\x8e\xb5\xd2\xe9\x96:\xe4rg]\xee\xc8\xcb\x9d|%\x8eF\x15;\xa9\xf4q\x85\x90+\x8b\\\x91\xe4J&W@\xb9r\xca\x15W\xae\xd4R\x85\x97+\xc3\\QfJ4W\xb0\x85\xf2\xcd\x15s\xa9\xb4s\x85\xbeR\xf6\x1d\xfb\x8bF\x04$\xe0\x00\x02\x05\x178\xf0\xc0A\t\x81XTx\x86@;\x18\x16\"\x90\x12\x95 \x19\xfa\xd7?\x14\xf9\xe30\x80\xd0\xd0\xbfe\x83\xb7\x81j\x00\x8d\xe8\x1f\xe2\xf1\x0c\x90!T\xc7J\xdaX\x8d\xe6]\xe8\xc8\x1f\xfcP\xe4\xef\x7f\xd0\xfe\x02\xccY\xdc?\x01\"U\x8a\xdd6^\xde\x85\x1b\xe90N\"\x93t\xcf`\xd4\xc4\xc0s\xa3,Y:9\xd84\xe5\xf4\xcb+3Q\xc8\x81|\xeb\x18Q\xa1\xd2\xbb\xf4\x8e5|7n\t\x8f\x07\xca@\xa8\x0e\xb6\x84%\x9e\xe5\x0e\x85C\x8d\x14\x0fT\xe5A\xacU\x80+\x0f~\xa5\x81\xb1<hV\x06\xd4\xd2`\xdbr(^\x154\xbf]n\x97?.\xcb\xa9\x86nX\x0e\x135\xe35\xcc\xd3\xc3q\xbae\x0e9\xad\x17@\xa8YK\xc1U\x99\xa1\xe8t\x83\xefO7\xfb\xe5\xa6\xeeB\x91M\xeb\xac,\x9c\x96\x9c\x0fn\xbe~\xf1V\t\xf5\xdb\xfa^\xd1o\xeb{E\xbf}\xf9\xf7\x16\xe4\xa30\\/\xf9z:\xf9w[\xda\x17\xbb\xef\xab\xeb\x88FrM\xd1\x86\xe12]\x04/\x81\x9fz&`-\x1a\xe9\x8f\xedt\x14OS\xff\xab\x11\xe9\xa19\"X\xc3FC\xf2\xdewr\x19Qz\x1e*\xc9tOP\xa2\xa6\x19\xb3\xc7`\xd6e\xa8$\xd3\xed?\x1c\x93eO\x0co\x85g\xe67\x1a\xe7\xbc#\x7f\\\xa6B\x83\xeaa\x82\xb0\xdaH?\xb7\xd3\x89\xb8\x1a\x05\xd59\x02B\xbb\tM\x9d\x8bo\\\x0e\x1f\xa4\xfb\x9d\xd26\xdf\xfd0\xc7\x1b\x99\x82\x00+\x84\xd6\xbaE\x90Z\xff\xafN\xdf\xe8!\x91[\xb5\xbc\x01(]\x82J\xafl\xd8\x13\x17Si\xf0\xac+?o\xec\x16\"\xbf>\xa1\xd1\x94\xbb\xdb\x0c\xa8\xa5Q\xd7\xb0\x93\x93D\xab\xab\\\xdbD\x8b\xea\x1e\x16a\xd6\x07\xe8\xa7W\xae\x0ff\xce~\x01\xb1\xb5!\xd2\xa6r\xc3\xc8\xca\xbdO\xc1\x16.c\xec\x8bX\x98\xfep\x9a\xa6`C\xcf/\x85\x88!0\x14\x1c\xa1\xf9\x1d\xad\xad\x07\xfbdO\"\x93<o-\xffa7C\xaf2\xed\xdfzQ\xc1'\xd3\xe9\xf5i\xba\xdd\xdf|\xf6\xf8\xbcb+$3\xfc\xf7x\xf3\xdf7\x7f\x9bN\xbf\xad\xe3~\xac\x83\x06D\xd9'\xe1\x19U\x03\xff\x19\x1e\x1eK+\xffU\xf1\x92V3\x84\x9d<\xe3\xef?\xe3\xac\xe5\x93\xcf+\xd6B\x922k\xf9\xe4\x8b\xcf[\xe0\x07\xd9\xb2F\xdd\x10n\xe3\x88\xb1\xc1VA\xe3\x87\x16\xb7\xd9I\x03C\x8d\x8a\xd0K\xf1\xa6\xd8\x9e\x9f\x06i\x85\x98;\x04V\x12\xbc\x02\xae\xa6\x0b\xbdiO\xb0\xa7<\x85\xc7\xdb\xc7\xe5$\x8d\x13\x9b\xf3\x0b\xac\xaa\x9e^\xe9nC\xa6\x89:>\xb0)\xa3\x9eE\xcc\xb6!\xa7\x0c\xbdq\x96\xe5\xb4\xc4\x86\xef.\xc7\xdb\xe5\xb1\xc5\xd8\xd2=z\x91\x81\x11\x98e\xe7obF\xe4o\xf3B\x1cwv\x04\x97I\xd7\xf2m{\xab\x82U#\xd6\x95\x80\x99\xc4\xfeFdv\xe9^\x07\xd3\x82dv\x84\xb5y4\x86\xa4\xbf\x9b\x16\x17\xbb\x03\x93\x08,vd\xf0\x1c\xab\xe9\x10\xdb\x81k\x91\x92\xf5\x04\xb8\xa3\x17\xdc1\xaf\xe4\x87\x8f\x7f\x1c\x7fZ\xce\xcbki4Y\xc5\xc1\xd3tY\x05\xc3\\R\xb0\xcb\xcf\x1e\xefNK\x9dIxf\x1f\xf9\xa4\x87\xcb\xc4\xd11\x08@\x9ft\x9b\xdf\xfa\xec\xa6\x83\x0b\x1e*\x87\xa1\x01\xfd,\xeb\x8b\xfb.\xdc.\xb1\x1b\x83\xb4\xb5\xb3\xfby\x9d\x80\xd9\x05a`G\x97\x04Xj\xee\xcfr\x8c\xbf\xb7\x82\xb3\xe6\xec7\x82\xb3~v\xfa\xb9\xb5q\x89\x0f\x91\xad\xfe\x04\xcbD\x9e)\x07\xb4\xc8\x01S\x19\x88\xb5\xe8o\x88'\x90\xc2;\xfa\x12\xf0\xb3P\xc8G\xf0\xe6\xdc\xed\xc7({\x0f\xd3Cu\xb7E\xf90?\xdaO\xa7)\x84\x00)\xe2.D<t\xff\xadq\xd8\x19K\xf6\xc3\xd3\xc7\xe1Z\xfeN\xd5\x11\x00\xc2j\xe3\x08\xa0\x1f?\x95\x0f\x9c\xbc\xab\xfd\x1f\x97\xef/j\x99\xc6\xdbP\x85\n1/+e\xee\xea\xdd\x89\xf7\xb8\xbbC`\xf8\xc1\xd8\xd8\xab\xe0\x836\x1e\xc2M'\xfd\xecw\xad\x9b\xdd\xb1\xb4\x1b\x82;Y.\xbd\xee\xd9\xbf\xfe\xfa\xe1\xaf\xff\x07\xd4l\x8e\""
    )

    #
    # CONSTRUCTOR
    #

    def __init__(
        self,
        background_color: typing.Optional[Color] = None,
        border_color: typing.Optional[Color] = None,
        border_dash_pattern: typing.List[int] = [],
        border_dash_phase: int = 0,
        border_width_bottom: int = 0,
        border_width_left: int = 0,
        border_width_right: int = 0,
        border_width_top: int = 0,
        dash_pattern: typing.List[int] = [],
        dash_phase: int = 0,
        fill_color: typing.Optional[Color] = None,
        horizontal_alignment: LayoutElement.HorizontalAlignment = LayoutElement.HorizontalAlignment.LEFT,
        line_width: int = 1,
        margin_bottom: int = 0,
        margin_left: int = 0,
        margin_right: int = 0,
        margin_top: int = 0,
        padding_bottom: int = 0,
        padding_left: int = 0,
        padding_right: int = 0,
        padding_top: int = 0,
        stroke_color: typing.Optional[Color] = X11Color.BLACK,
        vertical_alignment: LayoutElement.VerticalAlignment = LayoutElement.VerticalAlignment.TOP,
    ):
        """
        Initialize a map representation for a specified region.

        This constructor sets up the visual properties and layout options for the map, allowing
        customization of colors, borders, alignment, and other attributes.

        :param background_color: Optional background color of the map.
        :param border_color: Optional color for the border of the map.
        :param border_dash_pattern: Optional pattern for dashed borders.
        :param border_dash_phase: Optional phase for the dash pattern.
        :param border_width_bottom: Width of the bottom border.
        :param border_width_left: Width of the left border.
        :param border_width_right: Width of the right border.
        :param border_width_top: Width of the top border.
        :param dash_pattern: Optional pattern for dashed lines within the map.
        :param dash_phase: Optional phase for the dash pattern of lines.
        :param fill_color: Optional fill color for specific regions in the map.
        :param horizontal_alignment: Horizontal alignment of the map.
        :param line_width: Width of the lines drawn on the map.
        :param margin_bottom: Bottom margin around the map.
        :param margin_left: Left margin around the map.
        :param margin_right: Right margin around the map.
        :param margin_top: Top margin around the map.
        :param padding_bottom: Bottom padding inside the map.
        :param padding_left: Left padding inside the map.
        :param padding_right: Right padding inside the map.
        :param padding_top: Top padding inside the map.
        :param stroke_color: Color used for strokes on the map.
        :param vertical_alignment: Vertical alignment of the map.
        """
        import json
        import zlib

        super().__init__(
            geo_json=json.loads(
                zlib.decompress(MapOfSouthAmerica.__MAP_OF_SOUTH_AMERICA_GEO_JSON_BYTES)
            ),
            background_color=background_color,
            border_color=border_color,
            border_dash_pattern=border_dash_pattern,
            border_dash_phase=border_dash_phase,
            border_width_bottom=border_width_bottom,
            border_width_left=border_width_left,
            border_width_right=border_width_right,
            border_width_top=border_width_top,
            dash_pattern=dash_pattern,
            dash_phase=dash_phase,
            fill_color=fill_color,
            horizontal_alignment=horizontal_alignment,
            line_width=line_width,
            margin_bottom=margin_bottom,
            margin_left=margin_left,
            margin_right=margin_right,
            margin_top=margin_top,
            padding_bottom=padding_bottom,
            padding_left=padding_left,
            padding_right=padding_right,
            padding_top=padding_top,
            stroke_color=stroke_color,
            vertical_alignment=vertical_alignment,
        )

    #
    # PRIVATE
    #

    #
    # PUBLIC
    #
