#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
The 'Tz' operator: Set horizontal text scaling.

This operator adjusts the horizontal scaling of text by modifying the spacing between
glyphs. The scaling factor is specified as a percentage, where 100 means no scaling,
values greater than 100 expand the text horizontally, and values less than 100
compress it.
"""
import typing

from borb.pdf.page import Page
from borb.pdf.primitives import PDFType
from borb.pdf.toolkit.source.operator.operator import Operator
from borb.pdf.toolkit.source.operator.source import (
    Source,
)


class OperatorTz(Operator):
    """
    The 'Tz' operator: Set horizontal text scaling.

    This operator adjusts the horizontal scaling of text by modifying the spacing between
    glyphs. The scaling factor is specified as a percentage, where 100 means no scaling,
    values greater than 100 expand the text horizontally, and values less than 100
    compress it.
    """

    #
    # CONSTRUCTOR
    #

    #
    # PRIVATE
    #

    #
    # PUBLIC
    #

    def apply(
        self,
        operands: typing.List[PDFType],
        page: Page,
        source: Source,
    ) -> None:
        """
        Apply the operator's logic to the given `Page`.

        This method executes the operator using the provided operands, applying its
        effects to the specified `Page` via the `Source` processor. Subclasses should
        override this method to implement specific operator behavior.

        :param page: The `Page` object to which the operator is applied.
        :param source: The `Source` object managing the content stream.
        :param operands: A list of `PDFType` objects representing the operator's operands.
        """
        assert isinstance(operands[0], float) or isinstance(operands[0], int)
        source.horizontal_scaling = operands[0]
        pass

    def get_name(self) -> str:
        """
        Retrieve the name of the operator.

        The name is a string identifier that corresponds to the operator
        in a PDF content stream (e.g., "BT" for Begin Text or "q" for Save Graphics State).

        :return: The name of the operator as a string.
        """
        return "Tz"

    def get_number_of_operands(self) -> int:
        """
        Retrieve the expected number of operands for this operator.

        The number of operands varies depending on the operator's purpose. For example,
        some operators might require no operands, while others may require multiple.

        :return: The number of operands expected by this operator as an integer.
        """
        return 1
