#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Represents the Adobe Glyph List (AGL) for mapping glyph names to Unicode values.

The `AdobeGlyphList` class provides functionality for interpreting and managing glyph
names as specified by the Adobe Glyph List (AGL). The AGL is a widely used standard
for mapping glyph names to their corresponding Unicode values, facilitating text
encoding and rendering in PDF and other document formats.

This class serves as a utility for converting between glyph names and Unicode, allowing
developers to handle text rendering with proper character encoding. It includes methods
to retrieve Unicode values for given glyph names, ensuring that characters are displayed
accurately when rendering PDF content or processing textual data.

The Adobe Glyph List is particularly useful when working with non-standard fonts or
special characters that require explicit mapping to Unicode.
"""


class AdobeGlyphList:
    """
    Represents the Adobe Glyph List (AGL) for mapping glyph names to Unicode values.

    The `AdobeGlyphList` class provides functionality for interpreting and managing glyph
    names as specified by the Adobe Glyph List (AGL). The AGL is a widely used standard
    for mapping glyph names to their corresponding Unicode values, facilitating text
    encoding and rendering in PDF and other document formats.

    This class serves as a utility for converting between glyph names and Unicode, allowing
    developers to handle text rendering with proper character encoding. It includes methods
    to retrieve Unicode values for given glyph names, ensuring that characters are displayed
    accurately when rendering PDF content or processing textual data.

    The Adobe Glyph List is particularly useful when working with non-standard fonts or
    special characters that require explicit mapping to Unicode.
    """

    #
    # CONSTRUCTOR
    #

    #
    # PRIVATE
    #

    #
    # PUBLIC
    #

    ADOBE_CHARACTER_NAME_TO_CHARACTER = {
        "space": " ",
        "exclam": "!",
        "quotedbl": '"',
        "numbersign": "#",
        "dollar": "$",
        "percent": "%",
        "ampersand": "&",
        "quotesingle": "'",
        "parenleft": "(",
        "parenright": ")",
        "asterisk": "*",
        "plus": "+",
        "comma": ",",
        "hyphen": "-",
        "period": ".",
        "slash": "/",
        "zero": "0",
        "one": "1",
        "two": "2",
        "three": "3",
        "four": "4",
        "five": "5",
        "six": "6",
        "seven": "7",
        "eight": "8",
        "nine": "9",
        "colon": ":",
        "semicolon": ";",
        "less": "<",
        "equal": "=",
        "greater": ">",
        "question": "?",
        "at": "@",
        "A": "A",
        "B": "B",
        "C": "C",
        "D": "D",
        "E": "E",
        "F": "F",
        "G": "G",
        "H": "H",
        "I": "I",
        "J": "J",
        "K": "K",
        "L": "L",
        "M": "M",
        "N": "N",
        "O": "O",
        "P": "P",
        "Q": "Q",
        "R": "R",
        "S": "S",
        "T": "T",
        "U": "U",
        "V": "V",
        "W": "W",
        "X": "X",
        "Y": "Y",
        "Z": "Z",
        "bracketleft": "[",
        "backslash": "\\",
        "bracketright": "]",
        "asciicircum": "^",
        "underscore": "_",
        "grave": "`",
        "a": "a",
        "b": "b",
        "c": "c",
        "d": "d",
        "e": "e",
        "f": "f",
        "g": "g",
        "h": "h",
        "i": "i",
        "j": "j",
        "k": "k",
        "l": "l",
        "m": "m",
        "n": "n",
        "o": "o",
        "p": "p",
        "q": "q",
        "r": "r",
        "s": "s",
        "t": "t",
        "u": "u",
        "v": "v",
        "w": "w",
        "x": "x",
        "y": "y",
        "z": "z",
        "braceleft": "{",
        "bar": "|",
        "braceright": "}",
        "asciitilde": "~",
        "nbspace": "\xa0",
        "exclamdown": "¡",
        "cent": "¢",
        "sterling": "£",
        "currency": "¤",
        "yen": "¥",
        "brokenbar": "¦",
        "section": "§",
        "dieresis": "¨",
        "copyright": "©",
        "ordfeminine": "ª",
        "guillemetleft": "«",
        "logicalnot": "¬",
        "hyphensoft": "\xad",
        "registered": "®",
        "macron": "¯",
        "degree": "°",
        "plusminus": "±",
        "two.superior": "²",
        "three.superior": "³",
        "acute": "´",
        "mu.math": "µ",
        "paragraph": "¶",
        "periodcentered": "·",
        "cedilla": "¸",
        "one.superior": "¹",
        "ordmasculine": "º",
        "guillemetright": "»",
        "onequarter": "¼",
        "onehalf": "½",
        "threequarters": "¾",
        "questiondown": "¿",
        "Agrave": "À",
        "Aacute": "Á",
        "Acircumflex": "Â",
        "Atilde": "Ã",
        "Adieresis": "Ä",
        "Aring": "Å",
        "AE": "Æ",
        "Ccedilla": "Ç",
        "Egrave": "È",
        "Eacute": "É",
        "Ecircumflex": "Ê",
        "Edieresis": "Ë",
        "Igrave": "Ì",
        "Iacute": "Í",
        "Icircumflex": "Î",
        "Idieresis": "Ï",
        "Eth": "Ð",
        "Ntilde": "Ñ",
        "Ograve": "Ò",
        "Oacute": "Ó",
        "Ocircumflex": "Ô",
        "Otilde": "Õ",
        "Odieresis": "Ö",
        "multiply": "×",
        "Oslash": "Ø",
        "Ugrave": "Ù",
        "Uacute": "Ú",
        "Ucircumflex": "Û",
        "Udieresis": "Ü",
        "Yacute": "Ý",
        "Thorn": "Þ",
        "germandbls": "ß",
        "agrave": "à",
        "aacute": "á",
        "acircumflex": "â",
        "atilde": "ã",
        "adieresis": "ä",
        "aring": "å",
        "ae": "æ",
        "ccedilla": "ç",
        "egrave": "è",
        "eacute": "é",
        "ecircumflex": "ê",
        "edieresis": "ë",
        "igrave": "ì",
        "iacute": "í",
        "icircumflex": "î",
        "idieresis": "ï",
        "eth": "ð",
        "ntilde": "ñ",
        "ograve": "ò",
        "oacute": "ó",
        "ocircumflex": "ô",
        "otilde": "õ",
        "odieresis": "ö",
        "divide": "÷",
        "oslash": "ø",
        "ugrave": "ù",
        "uacute": "ú",
        "ucircumflex": "û",
        "udieresis": "ü",
        "yacute": "ý",
        "thorn": "þ",
        "ydieresis": "ÿ",
        "Amacron": "Ā",
        "amacron": "ā",
        "Abreve": "Ă",
        "abreve": "ă",
        "Aogonek": "Ą",
        "aogonek": "ą",
        "Cacute": "Ć",
        "cacute": "ć",
        "Ccircumflex": "Ĉ",
        "ccircumflex": "ĉ",
        "Cdotaccent": "Ċ",
        "cdotaccent": "ċ",
        "Ccaron": "Č",
        "ccaron": "č",
        "Dcaron": "Ď",
        "dcaron": "ď",
        "Dcroat": "Đ",
        "dcroat": "đ",
        "Emacron": "Ē",
        "emacron": "ē",
        "Ebreve": "Ĕ",
        "ebreve": "ĕ",
        "Edotaccent": "Ė",
        "edotaccent": "ė",
        "Eogonek": "Ę",
        "eogonek": "ę",
        "Ecaron": "Ě",
        "ecaron": "ě",
        "Gcircumflex": "Ĝ",
        "gcircumflex": "ĝ",
        "Gbreve": "Ğ",
        "gbreve": "ğ",
        "Gdotaccent": "Ġ",
        "gdotaccent": "ġ",
        "Gcommaaccent": "Ģ",
        "gcommaaccent": "ģ",
        "Hcircumflex": "Ĥ",
        "hcircumflex": "ĥ",
        "Hbar": "Ħ",
        "hbar": "ħ",
        "Itilde": "Ĩ",
        "itilde": "ĩ",
        "Imacron": "Ī",
        "imacron": "ī",
        "Ibreve": "Ĭ",
        "ibreve": "ĭ",
        "Iogonek": "Į",
        "iogonek": "į",
        "Idotaccent": "İ",
        "dotlessi": "ı",
        "IJ": "Ĳ",
        "ij": "ĳ",
        "Jcircumflex": "Ĵ",
        "jcircumflex": "ĵ",
        "Kcommaaccent": "Ķ",
        "kcommaaccent": "ķ",
        "kra": "ĸ",
        "Lacute": "Ĺ",
        "lacute": "ĺ",
        "Lcommaaccent": "Ļ",
        "lcommaaccent": "ļ",
        "Lcaron": "Ľ",
        "lcaron": "ľ",
        "Ldot": "Ŀ",
        "ldot": "ŀ",
        "Lslash": "Ł",
        "lslash": "ł",
        "Nacute": "Ń",
        "nacute": "ń",
        "Ncommaaccent": "Ņ",
        "ncommaaccent": "ņ",
        "Ncaron": "Ň",
        "ncaron": "ň",
        "napostrophe": "ŉ",
        "Eng": "Ŋ",
        "eng": "ŋ",
        "Omacron": "Ō",
        "omacron": "ō",
        "Obreve": "Ŏ",
        "obreve": "ŏ",
        "Ohungarumlaut": "Ő",
        "ohungarumlaut": "ő",
        "OE": "Œ",
        "oe": "œ",
        "Racute": "Ŕ",
        "racute": "ŕ",
        "Rcommaaccent": "Ŗ",
        "rcommaaccent": "ŗ",
        "Rcaron": "Ř",
        "rcaron": "ř",
        "Sacute": "Ś",
        "sacute": "ś",
        "Scircumflex": "Ŝ",
        "scircumflex": "ŝ",
        "Scedilla": "Ş",
        "scedilla": "ş",
        "Scaron": "Š",
        "scaron": "š",
        "Tcedilla": "Ţ",
        "tcedilla": "ţ",
        "Tcaron": "Ť",
        "tcaron": "ť",
        "Tbar": "Ŧ",
        "tbar": "ŧ",
        "Utilde": "Ũ",
        "utilde": "ũ",
        "Umacron": "Ū",
        "umacron": "ū",
        "Ubreve": "Ŭ",
        "ubreve": "ŭ",
        "Uring": "Ů",
        "uring": "ů",
        "Uhungarumlaut": "Ű",
        "uhungarumlaut": "ű",
        "Uogonek": "Ų",
        "uogonek": "ų",
        "Wcircumflex": "Ŵ",
        "wcircumflex": "ŵ",
        "Ycircumflex": "Ŷ",
        "ycircumflex": "ŷ",
        "Ydieresis": "Ÿ",
        "Zacute": "Ź",
        "zacute": "ź",
        "Zdotaccent": "Ż",
        "zdotaccent": "ż",
        "Zcaron": "Ž",
        "zcaron": "ž",
        "longs": "ſ",
        "bstroke": "ƀ",
        "Bhook": "Ɓ",
        "Btopbar": "Ƃ",
        "btopbar": "ƃ",
        "Tonesix": "Ƅ",
        "tonesix": "ƅ",
        "Oopen": "Ɔ",
        "Chook": "Ƈ",
        "chook": "ƈ",
        "Dafrican": "Ɖ",
        "Dhook": "Ɗ",
        "Dtopbar": "Ƌ",
        "dtopbar": "ƌ",
        "deltaturned": "ƍ",
        "Ereversed": "Ǝ",
        "Schwa": "Ə",
        "Eopen": "Ɛ",
        "Fhook": "Ƒ",
        "fhook": "ƒ",
        "Ghook": "Ɠ",
        "lt:Gamma": "Ɣ",
        "hv": "ƕ",
        "lt:Iota": "Ɩ",
        "Istroke": "Ɨ",
        "Khook": "Ƙ",
        "khook": "ƙ",
        "lbar": "ƚ",
        "lambdastroke": "ƛ",
        "Mturned": "Ɯ",
        "Nhookleft": "Ɲ",
        "nlongrightleg": "ƞ",
        "Obar": "Ɵ",
        "Ohorn": "Ơ",
        "ohorn": "ơ",
        "Oi": "Ƣ",
        "oi": "ƣ",
        "Phook": "Ƥ",
        "phook": "ƥ",
        "yr": "Ʀ",
        "Tonetwo": "Ƨ",
        "tonetwo": "ƨ",
        "Esh": "Ʃ",
        "eshreversedloop": "ƪ",
        "tpalatalhook": "ƫ",
        "Thook": "Ƭ",
        "thook": "ƭ",
        "Tretroflexhook": "Ʈ",
        "Uhorn": "Ư",
        "uhorn": "ư",
        "lt:Upsilon": "Ʊ",
        "Vhook": "Ʋ",
        "Yhook": "Ƴ",
        "yhook": "ƴ",
        "Zstroke": "Ƶ",
        "zstroke": "ƶ",
        "Ezh": "Ʒ",
        "Ezhreversed": "Ƹ",
        "ezhreversed": "ƹ",
        "ezhtail": "ƺ",
        "twostroke": "ƻ",
        "Tonefive": "Ƽ",
        "tonefive": "ƽ",
        "glottalinvertedstroke": "ƾ",
        "wynn": "ƿ",
        "clickdental": "ǀ",
        "clicklateral": "ǁ",
        "clickalveolar": "ǂ",
        "clickretroflex": "ǃ",
        "DZcaron": "Ǆ",
        "Dzcaron": "ǅ",
        "dzcaron": "ǆ",
        "LJ": "Ǉ",
        "Lj": "ǈ",
        "lj": "ǉ",
        "NJ": "Ǌ",
        "Nj": "ǋ",
        "nj": "ǌ",
        "Acaron": "Ǎ",
        "acaron": "ǎ",
        "Icaron": "Ǐ",
        "icaron": "ǐ",
        "Ocaron": "Ǒ",
        "ocaron": "ǒ",
        "Ucaron": "Ǔ",
        "ucaron": "ǔ",
        "Udieresismacron": "Ǖ",
        "udieresismacron": "ǖ",
        "Udieresisacute": "Ǘ",
        "udieresisacute": "ǘ",
        "Udieresiscaron": "Ǚ",
        "udieresiscaron": "ǚ",
        "Udieresisgrave": "Ǜ",
        "udieresisgrave": "ǜ",
        "eturned": "ǝ",
        "Adieresismacron": "Ǟ",
        "adieresismacron": "ǟ",
        "Adotmacron": "Ǡ",
        "adotmacron": "ǡ",
        "AEmacron": "Ǣ",
        "aemacron": "ǣ",
        "Gstroke": "Ǥ",
        "gstroke": "ǥ",
        "Gcaron": "Ǧ",
        "gcaron": "ǧ",
        "Kcaron": "Ǩ",
        "kcaron": "ǩ",
        "Oogonek": "Ǫ",
        "oogonek": "ǫ",
        "Oogonekmacron": "Ǭ",
        "oogonekmacron": "ǭ",
        "Ezhcaron": "Ǯ",
        "ezhcaron": "ǯ",
        "jcaron": "ǰ",
        "DZ": "Ǳ",
        "Dz": "ǲ",
        "dz": "ǳ",
        "Gacute": "Ǵ",
        "gacute": "ǵ",
        "Hwair": "Ƕ",
        "Wynn": "Ƿ",
        "Ngrave": "Ǹ",
        "ngrave": "ǹ",
        "Aringacute": "Ǻ",
        "aringacute": "ǻ",
        "AEacute": "Ǽ",
        "aeacute": "ǽ",
        "Oslashacute": "Ǿ",
        "oslashacute": "ǿ",
        "Agravedbl": "Ȁ",
        "agravedbl": "ȁ",
        "Ainvertedbreve": "Ȃ",
        "ainvertedbreve": "ȃ",
        "Egravedbl": "Ȅ",
        "egravedbl": "ȅ",
        "Einvertedbreve": "Ȇ",
        "einvertedbreve": "ȇ",
        "Igravedbl": "Ȉ",
        "igravedbl": "ȉ",
        "Iinvertedbreve": "Ȋ",
        "iinvertedbreve": "ȋ",
        "Ogravedbl": "Ȍ",
        "ogravedbl": "ȍ",
        "Oinvertedbreve": "Ȏ",
        "oinvertedbreve": "ȏ",
        "Rgravedbl": "Ȑ",
        "rgravedbl": "ȑ",
        "Rinvertedbreve": "Ȓ",
        "rinvertedbreve": "ȓ",
        "Ugravedbl": "Ȕ",
        "ugravedbl": "ȕ",
        "Uinvertedbreve": "Ȗ",
        "uinvertedbreve": "ȗ",
        "Scommaaccent": "Ș",
        "scommaaccent": "ș",
        "Tcommaaccent": "Ț",
        "tcommaaccent": "ț",
        "Yogh": "Ȝ",
        "yogh": "ȝ",
        "Hcaron": "Ȟ",
        "hcaron": "ȟ",
        "Nlongrightleg": "Ƞ",
        "dcurl": "ȡ",
        "Ou": "Ȣ",
        "ou": "ȣ",
        "Zhook": "Ȥ",
        "zhook": "ȥ",
        "Adot": "Ȧ",
        "adot": "ȧ",
        "Ecedilla": "Ȩ",
        "ecedilla": "ȩ",
        "Odieresismacron": "Ȫ",
        "odieresismacron": "ȫ",
        "Otildemacron": "Ȭ",
        "otildemacron": "ȭ",
        "Odot": "Ȯ",
        "odot": "ȯ",
        "Odotmacron": "Ȱ",
        "odotmacron": "ȱ",
        "Ymacron": "Ȳ",
        "ymacron": "ȳ",
        "lcurl": "ȴ",
        "ncurl": "ȵ",
        "tcurl": "ȶ",
        "dotlessj": "ȷ",
        "dbdigraph": "ȸ",
        "qpdigraph": "ȹ",
        "Astroke": "Ⱥ",
        "Cstroke": "Ȼ",
        "cstroke": "ȼ",
        "Lbar": "Ƚ",
        "Twithdiagonalstroke": "Ⱦ",
        "sswashtail": "ȿ",
        "zswashtail": "ɀ",
        "Glottalstop": "Ɂ",
        "glottalstop": "ɂ",
        "Bstroke": "Ƀ",
        "Ubar": "Ʉ",
        "Vturned": "Ʌ",
        "Estroke": "Ɇ",
        "estroke": "ɇ",
        "Jstroke": "Ɉ",
        "jstroke": "ɉ",
        "Qsmallhooktail": "Ɋ",
        "qhooktail": "ɋ",
        "Rstroke": "Ɍ",
        "rstroke": "ɍ",
        "Ystroke": "Ɏ",
        "ystroke": "ɏ",
        "aturned": "ɐ",
        "ipa:alpha": "ɑ",
        "alphaturned": "ɒ",
        "bhook": "ɓ",
        "oopen": "ɔ",
        "ccurl": "ɕ",
        "dtail": "ɖ",
        "dhook": "ɗ",
        "ipa:ereversed": "ɘ",
        "schwa": "ə",
        "schwahook": "ɚ",
        "eopen": "ɛ",
        "eopenreversed": "ɜ",
        "eopenreversedhook": "ɝ",
        "eopenreversedclosed": "ɞ",
        "dotlessjstroke": "ɟ",
        "ghook": "ɠ",
        "ipa:gscript": "ɡ",
        "Gsmall": "ɢ",
        "ipa:gamma": "ɣ",
        "ramshorn": "ɤ",
        "hturned": "ɥ",
        "hhook": "ɦ",
        "henghook": "ɧ",
        "istroke": "ɨ",
        "ipa:iota": "ɩ",
        "ipa:Ismall": "ɪ",
        "lmiddletilde": "ɫ",
        "lbelt": "ɬ",
        "lretroflex": "ɭ",
        "lezh": "ɮ",
        "mturned": "ɯ",
        "mlonglegturned": "ɰ",
        "mhook": "ɱ",
        "nhookleft": "ɲ",
        "nretroflex": "ɳ",
        "Nsmall": "ɴ",
        "obarred": "ɵ",
        "OEsmall": "ɶ",
        "omegaclosed": "ɷ",
        "ipa:phi": "ɸ",
        "rturned": "ɹ",
        "rlonglegturned": "ɺ",
        "rhookturned": "ɻ",
        "rlongleg": "ɼ",
        "ipa:rtail": "ɽ",
        "rfishhook": "ɾ",
        "rfishhookreversed": "ɿ",
        "Rsmall": "ʀ",
        "Rinvertedsmall": "ʁ",
        "shook": "ʂ",
        "esh": "ʃ",
        "dotlessjstrokehook": "ʄ",
        "eshsquatreversed": "ʅ",
        "eshcurl": "ʆ",
        "tturned": "ʇ",
        "tretroflex": "ʈ",
        "ubar": "ʉ",
        "ipa:upsilon": "ʊ",
        "vhook": "ʋ",
        "vturned": "ʌ",
        "wturned": "ʍ",
        "yturned": "ʎ",
        "Ysmall": "ʏ",
        "zretroflex": "ʐ",
        "zcurl": "ʑ",
        "ezh": "ʒ",
        "ezhcurl": "ʓ",
        "ipa:glottalstop": "ʔ",
        "pharyngealvoicedfricative": "ʕ",
        "glottalstopinverted": "ʖ",
        "Cstretched": "ʗ",
        "clickbilabial": "ʘ",
        "Bsmall": "ʙ",
        "eopenclosed": "ʚ",
        "Ghooksmall": "ʛ",
        "Hsmall": "ʜ",
        "jcrossedtail": "ʝ",
        "kturned": "ʞ",
        "Lsmall": "ʟ",
        "qhook": "ʠ",
        "glottalstopstroke": "ʡ",
        "glottalstopstrokereversed": "ʢ",
        "dzed": "ʣ",
        "dezh": "ʤ",
        "dzedcurl": "ʥ",
        "ts": "ʦ",
        "tesh": "ʧ",
        "tccurl": "ʨ",
        "feng": "ʩ",
        "ls": "ʪ",
        "lzed": "ʫ",
        "percussivebilabial": "ʬ",
        "percussivebidental": "ʭ",
        "hfishhookturned": "ʮ",
        "handtailfishhookturned": "ʯ",
        "hsupmod": "ʰ",
        "hhooksupmod": "ʱ",
        "jsupmod": "ʲ",
        "rsupmod": "ʳ",
        "rturnedsupmod": "ʴ",
        "rhookturnedsupmod": "ʵ",
        "Rsupinvertedmod": "ʶ",
        "wsupmod": "ʷ",
        "ysupmod": "ʸ",
        "primemod": "ʹ",
        "primedblmod": "ʺ",
        "commaturnedmod": "ʻ",
        "apostrophemod": "ʼ",
        "commareversedmod": "ʽ",
        "ringhalfrightmod": "ʾ",
        "ringhalfleftmod": "ʿ",
        "glottalstopmod": "ˀ",
        "glottalstopreversedmod": "ˁ",
        "arrowheadleftmod": "˂",
        "arrowheadrightmod": "˃",
        "arrowheadupmod": "˄",
        "arrowheaddownmod": "˅",
        "circumflex": "ˆ",
        "caron": "ˇ",
        "verticallinemod": "ˈ",
        "macronmod": "ˉ",
        "acutemod": "ˊ",
        "gravemod": "ˋ",
        "verticallinelowmod": "ˌ",
        "macronlowmod": "ˍ",
        "gravelowmod": "ˎ",
        "acutelowmod": "ˏ",
        "colontriangularmod": "ː",
        "colontriangularhalfmod": "ˑ",
        "ringhalfrightcentredmod": "˒",
        "ringhalfleftcentredmod": "˓",
        "tackupmod": "˔",
        "tackdownmod": "˕",
        "plussignmod": "˖",
        "minussignmod": "˗",
        "breve": "˘",
        "dotaccent": "˙",
        "ring": "˚",
        "ogonek": "˛",
        "tilde": "˜",
        "hungarumlaut": "˝",
        "rhotichookmod": "˞",
        "crossmod": "˟",
        "gammasupmod": "ˠ",
        "lsupmod": "ˡ",
        "ssupmod": "ˢ",
        "xsupmod": "ˣ",
        "glottalstopsupreversedmod": "ˤ",
        "tonebarextrahighmod": "˥",
        "tonebarhighmod": "˦",
        "tonebarmidmod": "˧",
        "tonebarlowmod": "˨",
        "tonebarextralowmod": "˩",
        "yintonemod": "˪",
        "yangtonemod": "˫",
        "voicingmod": "ˬ",
        "unaspiratedmod": "˭",
        "apostrophedblmod": "ˮ",
        "arrowheaddownlowmod": "˯",
        "arrowheaduplowmod": "˰",
        "arrowheadleftlowmod": "˱",
        "arrowheadrightlowmod": "˲",
        "ringlowmod": "˳",
        "gravemiddlemod": "˴",
        "gravedblmiddlemod": "˵",
        "acutedblmiddlemod": "˶",
        "tildelowmod": "˷",
        "colonraisedmod": "˸",
        "tonehighbeginmod": "˹",
        "tonehighendmod": "˺",
        "tonelowbeginmod": "˻",
        "tonelowendmod": "˼",
        "shelfmod": "˽",
        "shelfopenmod": "˾",
        "arrowleftlowmod": "˿",
        "gravecmb": "̀",
        "acutecmb": "́",
        "circumflexcmb": "̂",
        "tildecmb": "̃",
        "macroncmb": "̄",
        "overlinecmb": "̅",
        "brevecmb": "̆",
        "dotaccentcmb": "̇",
        "dieresiscmb": "̈",
        "hookabovecmb": "̉",
        "ringabovecmb": "̊",
        "hungarumlautcmb": "̋",
        "caroncmb": "̌",
        "verticallineabovecmb": "̍",
        "dblverticallineabovecmb": "̎",
        "gravedoublecmb": "̏",
        "candrabinducmb": "̐",
        "invertedbrevecmb": "̑",
        "commaturnedabovecmb": "̒",
        "turnedabovecmb": "̓",
        "reversedcommaabovecmb": "̔",
        "turnedcommaabovecmb": "̕",
        "gravebelowcmb": "̖",
        "acutebelowcmb": "̗",
        "lefttackbelowcmb": "̘",
        "righttackbelowcmb": "̙",
        "leftangleabovecmb": "̚",
        "horncmb": "̛",
        "halfleftringbelowcmb": "̜",
        "uptackbelowcmb": "̝",
        "downtackbelowcmb": "̞",
        "plusbelowcmb": "̟",
        "minusbelowcmb": "̠",
        "palatalizedhookbelowcmb": "̡",
        "retroflexhookbelowcmb": "̢",
        "dotbelowcmb": "̣",
        "dieresisbelowcmb": "̤",
        "ringbelowcmb": "̥",
        "commaaccentbelowcmb": "̦",
        "cedillacmb": "̧",
        "ogonekcmb": "̨",
        "verticallinebelowcmb": "̩",
        "bridgebelowcmb": "̪",
        "dblarchinvertedbelowcmb": "̫",
        "caronbelowcmb": "̬",
        "circumflexbelowcmb": "̭",
        "belowbrevecmb": "̮",
        "invertedbelowbrevecmb": "̯",
        "tildebelowcmb": "̰",
        "macronbelowcmb": "̱",
        "lowlinecmb": "̲",
        "lowlinedoublecmb": "̳",
        "tildeoverlaycmb": "̴",
        "overlaystrokeshortcmb": "̵",
        "overlaystrokelongcmb": "̶",
        "solidusshortoverlaycmb": "̷",
        "soliduslongoverlaycmb": "̸",
        "halfrightringbelowcmb": "̹",
        "invertedbridgebelowcmb": "̺",
        "squarebelowcmb": "̻",
        "seagullbelowcmb": "̼",
        "xabovecmb": "̽",
        "tildeverticalcmb": "̾",
        "dbloverlinecmb": "̿",
        "gravetonecmb": "̀",
        "acutetonecmb": "́",
        "perispomenicmb": "͂",
        "koroniscmb": "̓",
        "dialytikatonoscmb": "̈́",
        "iotasubcmb": "ͅ",
        "bridgeabovecmb": "͆",
        "equalbelowcmb": "͇",
        "dblverticallinebelowcmb": "͈",
        "leftanglebelowcmb": "͉",
        "nottildeabovecmb": "͊",
        "homotheticabovecmb": "͋",
        "almostequalabovecmb": "͌",
        "arrowleftrightbelowcmb": "͍",
        "arrowupbelowcmb": "͎",
        "graphemejoinercmb": "͏",
        "arrowheadrightabovecmb": "͐",
        "halfleftringabovecmb": "͑",
        "fermatacmb": "͒",
        "xbelowcmb": "͓",
        "arrowheadleftbelowcmb": "͔",
        "arrowheadrightbelowcmb": "͕",
        "arrowheadrightarrowheadupbelowcmb": "͖",
        "halfrightringabovecmb": "͗",
        "dotrightabovecmb": "͘",
        "asteriskbelowcmb": "͙",
        "doubleringbelowcmb": "͚",
        "zigzagabovecmb": "͛",
        "doublebelowbrevecmb": "͜",
        "doublebrevecmb": "͝",
        "macrondoublecmb": "͞",
        "macrondoublebelowcmb": "͟",
        "tildedoublecmb": "͠",
        "inverteddoublebrevecmb": "͡",
        "arrowrightdoublebelowcmb": "͢",
        "acmb": "ͣ",
        "ecmb": "ͤ",
        "icmb": "ͥ",
        "ocmb": "ͦ",
        "ucmb": "ͧ",
        "ccmb": "ͨ",
        "dcmb": "ͩ",
        "hcmb": "ͪ",
        "mcmb": "ͫ",
        "rcmb": "ͬ",
        "tcmb": "ͭ",
        "vcmb": "ͮ",
        "xcmb": "ͯ",
        "Heta": "Ͱ",
        "heta": "ͱ",
        "Sampiarchaic": "Ͳ",
        "sampiarchaic": "ͳ",
        "numeralsign": "ʹ",
        "lownumeralsign": "͵",
        "Digammapamphylian": "Ͷ",
        "digammapamphylian": "ͷ",
        "iotasub": "ͺ",
        "sigmalunatereversedsymbol": "ͻ",
        "sigmalunatedottedsymbol": "ͼ",
        "sigmalunatedottedreversedsymbol": "ͽ",
        "gr:question": ";",
        "Yot": "Ϳ",
        "tonos": "΄",
        "dieresistonos": "΅",
        "Alphatonos": "Ά",
        "anoteleia": "·",
        "Epsilontonos": "Έ",
        "Etatonos": "Ή",
        "Iotatonos": "Ί",
        "Omicrontonos": "Ό",
        "Upsilontonos": "Ύ",
        "Omegatonos": "Ώ",
        "iotadieresistonos": "ΐ",
        "Alpha": "Α",
        "Beta": "Β",
        "Gamma": "Γ",
        "Delta": "Δ",
        "Epsilon": "Ε",
        "Zeta": "Ζ",
        "Eta": "Η",
        "Theta": "Θ",
        "Iota": "Ι",
        "Kappa": "Κ",
        "Lambda": "Λ",
        "Mu": "Μ",
        "Nu": "Ν",
        "Xi": "Ξ",
        "Omicron": "Ο",
        "Pi": "Π",
        "Rho": "Ρ",
        "Sigma": "Σ",
        "Tau": "Τ",
        "Upsilon": "Υ",
        "Phi": "Φ",
        "Chi": "Χ",
        "Psi": "Ψ",
        "Omega": "Ω",
        "Iotadieresis": "Ϊ",
        "Upsilondieresis": "Ϋ",
        "alphatonos": "ά",
        "epsilontonos": "έ",
        "etatonos": "ή",
        "iotatonos": "ί",
        "upsilondieresistonos": "ΰ",
        "alpha": "α",
        "beta": "β",
        "gamma": "γ",
        "delta": "δ",
        "epsilon": "ε",
        "zeta": "ζ",
        "eta": "η",
        "theta": "θ",
        "iota": "ι",
        "kappa": "κ",
        "lambda": "λ",
        "mu": "μ",
        "nu": "ν",
        "xi": "ξ",
        "omicron": "ο",
        "pi": "π",
        "rho": "ρ",
        "finalsigma": "ς",
        "sigma": "σ",
        "tau": "τ",
        "upsilon": "υ",
        "phi": "φ",
        "chi": "χ",
        "psi": "ψ",
        "omega": "ω",
        "iotadieresis": "ϊ",
        "upsilondieresis": "ϋ",
        "omicrontonos": "ό",
        "upsilontonos": "ύ",
        "omegatonos": "ώ",
        "Kaisymbol": "Ϗ",
        "betasymbol": "ϐ",
        "theta.math": "ϑ",
        "Upsilonhooksymbol": "ϒ",
        "Upsilonacutehooksymbol": "ϓ",
        "Upsilonadieresishooksymbol": "ϔ",
        "phi.math": "ϕ",
        "pi.math": "ϖ",
        "kaisymbol": "ϗ",
        "Koppaarchaic": "Ϙ",
        "koppaarchaic": "ϙ",
        "Stigma": "Ϛ",
        "stigma": "ϛ",
        "Digamma": "Ϝ",
        "digamma": "ϝ",
        "Koppa": "Ϟ",
        "koppa": "ϟ",
        "Sampi": "Ϡ",
        "sampi": "ϡ",
        "Sheicoptic": "Ϣ",
        "sheicoptic": "ϣ",
        "Feicoptic": "Ϥ",
        "feicoptic": "ϥ",
        "Kheicoptic": "Ϧ",
        "kheicoptic": "ϧ",
        "Horicoptic": "Ϩ",
        "horicoptic": "ϩ",
        "Gangiacoptic": "Ϫ",
        "gangiacoptic": "ϫ",
        "Shimacoptic": "Ϭ",
        "shimacoptic": "ϭ",
        "Deicoptic": "Ϯ",
        "deicoptic": "ϯ",
        "kappa.math": "ϰ",
        "rhosymbol": "ϱ",
        "sigmalunatesymbol": "ϲ",
        "yot": "ϳ",
        "Thetasymbol": "ϴ",
        "epsilonlunatesymbol": "ϵ",
        "epsilonreversedlunatesymbol": "϶",
        "Sho": "Ϸ",
        "sho": "ϸ",
        "Sigmalunatesymbol": "Ϲ",
        "San": "Ϻ",
        "san": "ϻ",
        "rhostrokesymbol": "ϼ",
        "Sigmareversedlunatesymbol": "Ͻ",
        "Sigmalunatesymboldotted": "Ͼ",
        "Reverseddottedsigmalunatesymbol": "Ͽ",
        "Iegravecyr": "Ѐ",
        "Iocyr": "Ё",
        "Djecyr": "Ђ",
        "Gjecyr": "Ѓ",
        "Eukrcyr": "Є",
        "Dzecyr": "Ѕ",
        "Iukrcyr": "І",
        "Yukrcyr": "Ї",
        "Jecyr": "Ј",
        "Ljecyr": "Љ",
        "Njecyr": "Њ",
        "Tshecyr": "Ћ",
        "Kjecyr": "Ќ",
        "Igravecyr": "Ѝ",
        "Ushortcyr": "Ў",
        "Dzhecyr": "Џ",
        "Acyr": "А",
        "Becyr": "Б",
        "Vecyr": "В",
        "Gecyr": "Г",
        "Decyr": "Д",
        "Iecyr": "Е",
        "Zhecyr": "Ж",
        "Zecyr": "З",
        "Icyr": "И",
        "Ishortcyr": "Й",
        "Kacyr": "К",
        "Elcyr": "Л",
        "Emcyr": "М",
        "Encyr": "Н",
        "Ocyr": "О",
        "Pecyr": "П",
        "Ercyr": "Р",
        "Escyr": "С",
        "Tecyr": "Т",
        "Ucyr": "У",
        "Efcyr": "Ф",
        "Hacyr": "Х",
        "Tsecyr": "Ц",
        "Checyr": "Ч",
        "Shacyr": "Ш",
        "Shchacyr": "Щ",
        "Hardcyr": "Ъ",
        "Ylongcyr": "Ы",
        "Softcyr": "Ь",
        "Ereversedcyr": "Э",
        "Yucyr": "Ю",
        "Yacyr": "Я",
        "acyr": "а",
        "becyr": "б",
        "vecyr": "в",
        "gecyr": "г",
        "decyr": "д",
        "iecyr": "е",
        "zhecyr": "ж",
        "zecyr": "з",
        "icyr": "и",
        "ishortcyr": "й",
        "kacyr": "к",
        "elcyr": "л",
        "emcyr": "м",
        "encyr": "н",
        "ocyr": "о",
        "pecyr": "п",
        "ercyr": "р",
        "escyr": "с",
        "tecyr": "т",
        "ucyr": "у",
        "efcyr": "ф",
        "hacyr": "х",
        "tsecyr": "ц",
        "checyr": "ч",
        "shacyr": "ш",
        "shchacyr": "щ",
        "hardcyr": "ъ",
        "ylongcyr": "ы",
        "softcyr": "ь",
        "ereversedcyr": "э",
        "yucyr": "ю",
        "yacyr": "я",
        "iegravecyr": "ѐ",
        "iocyr": "ё",
        "djecyr": "ђ",
        "gjecyr": "ѓ",
        "eukrcyr": "є",
        "dzecyr": "ѕ",
        "iukrcyr": "і",
        "yukrcyr": "ї",
        "jecyr": "ј",
        "ljecyr": "љ",
        "njecyr": "њ",
        "tshecyr": "ћ",
        "kjecyr": "ќ",
        "igravecyr": "ѝ",
        "ushortcyr": "ў",
        "dzhecyr": "џ",
        "Omegacyr": "Ѡ",
        "omegacyr": "ѡ",
        "Yatcyr": "Ѣ",
        "yatcyr": "ѣ",
        "Eiotifiedcyr": "Ѥ",
        "eiotifiedcyr": "ѥ",
        "Yuslittlecyr": "Ѧ",
        "yuslittlecyr": "ѧ",
        "Yuslittleiotifiedcyr": "Ѩ",
        "yuslittleiotifiedcyr": "ѩ",
        "Yusbigcyr": "Ѫ",
        "yusbigcyr": "ѫ",
        "Yusbigiotifiedcyr": "Ѭ",
        "yusbigiotifiedcyr": "ѭ",
        "Ksicyr": "Ѯ",
        "ksicyr": "ѯ",
        "Psicyr": "Ѱ",
        "psicyr": "ѱ",
        "Fitacyr": "Ѳ",
        "fitacyr": "ѳ",
        "Izhitsacyr": "Ѵ",
        "izhitsacyr": "ѵ",
        "Izhitsagravedblcyr": "Ѷ",
        "izhitsagravedblcyr": "ѷ",
        "Ukcyr": "Ѹ",
        "ukcyr": "ѹ",
        "Omegaroundcyr": "Ѻ",
        "omegaroundcyr": "ѻ",
        "Omegatitlocyr": "Ѽ",
        "omegatitlocyr": "ѽ",
        "Otcyr": "Ѿ",
        "otcyr": "ѿ",
        "Koppacyr": "Ҁ",
        "koppacyr": "ҁ",
        "thousandscyr": "҂",
        "titlocmbcyr": "҃",
        "palatcmbcyr": "҄",
        "dasiacmbcyr": "҅",
        "psilicmbcyr": "҆",
        "pokrytiecmbcyr": "҇",
        "hundredthousandscmbcyr": "҈",
        "millionscmbcyr": "҉",
        "Ishortsharptailcyr": "Ҋ",
        "ishortsharptailcyr": "ҋ",
        "Semisoftcyr": "Ҍ",
        "semisoftcyr": "ҍ",
        "Ertickcyr": "Ҏ",
        "ertickcyr": "ҏ",
        "Geupcyr": "Ґ",
        "geupcyr": "ґ",
        "Gestrokecyr": "Ғ",
        "gestrokecyr": "ғ",
        "Gehookcyr": "Ҕ",
        "gehookcyr": "ҕ",
        "Zhetailcyr": "Җ",
        "zhetailcyr": "җ",
        "Zetailcyr": "Ҙ",
        "zetailcyr": "ҙ",
        "Katailcyr": "Қ",
        "katailcyr": "қ",
        "Kaverticalstrokecyr": "Ҝ",
        "kaverticalstrokecyr": "ҝ",
        "Kastrokecyr": "Ҟ",
        "kastrokecyr": "ҟ",
        "Kabashkcyr": "Ҡ",
        "kabashkcyr": "ҡ",
        "Entailcyr": "Ң",
        "entailcyr": "ң",
        "Engecyr": "Ҥ",
        "engecyr": "ҥ",
        "Pehookcyr": "Ҧ",
        "pehookcyr": "ҧ",
        "Haabkhcyr": "Ҩ",
        "haabkhcyr": "ҩ",
        "Estailcyr": "Ҫ",
        "estailcyr": "ҫ",
        "Tetailcyr": "Ҭ",
        "tetailcyr": "ҭ",
        "Ustraightcyr": "Ү",
        "ustraightcyr": "ү",
        "Ustraightstrokecyr": "Ұ",
        "ustraightstrokecyr": "ұ",
        "Xatailcyr": "Ҳ",
        "xatailcyr": "ҳ",
        "Tetsecyr": "Ҵ",
        "tetsecyr": "ҵ",
        "Chetailcyr": "Ҷ",
        "chetailcyr": "ҷ",
        "Chevertcyr": "Ҹ",
        "chevertcyr": "ҹ",
        "Shhacyr": "Һ",
        "shhacyr": "һ",
        "Cheabkhcyr": "Ҽ",
        "cheabkhcyr": "ҽ",
        "Cheabkhtailcyr": "Ҿ",
        "cheabkhtailcyr": "ҿ",
        "Palochkacyr": "Ӏ",
        "Zhebrevecyr": "Ӂ",
        "zhebrevecyr": "ӂ",
        "Kahookcyr": "Ӄ",
        "kahookcyr": "ӄ",
        "Elsharptailcyr": "Ӆ",
        "elsharptailcyr": "ӆ",
        "Enhookcyr": "Ӈ",
        "enhookcyr": "ӈ",
        "Ensharptailcyr": "Ӊ",
        "ensharptailcyr": "ӊ",
        "Chekhakascyr": "Ӌ",
        "chekhakascyr": "ӌ",
        "Emsharptailcyr": "Ӎ",
        "emsharptailcyr": "ӎ",
        "palochkacyr": "ӏ",
        "Abrevecyr": "Ӑ",
        "abrevecyr": "ӑ",
        "Adieresiscyr": "Ӓ",
        "adieresiscyr": "ӓ",
        "Aiecyr": "Ӕ",
        "aiecyr": "ӕ",
        "Iebrevecyr": "Ӗ",
        "iebrevecyr": "ӗ",
        "Schwacyr": "Ә",
        "schwacyr": "ә",
        "Schwadieresiscyr": "Ӛ",
        "schwadieresiscyr": "ӛ",
        "Zhedieresiscyr": "Ӝ",
        "zhedieresiscyr": "ӝ",
        "Zedieresiscyr": "Ӟ",
        "zedieresiscyr": "ӟ",
        "Dzeabkhcyr": "Ӡ",
        "dzeabkhcyr": "ӡ",
        "Imacroncyr": "Ӣ",
        "imacroncyr": "ӣ",
        "Idieresiscyr": "Ӥ",
        "idieresiscyr": "ӥ",
        "Odieresiscyr": "Ӧ",
        "odieresiscyr": "ӧ",
        "Obarcyr": "Ө",
        "obarcyr": "ө",
        "Obardieresiscyr": "Ӫ",
        "obardieresiscyr": "ӫ",
        "Ereverseddieresiscyr": "Ӭ",
        "ereverseddieresiscyr": "ӭ",
        "Umacroncyr": "Ӯ",
        "umacroncyr": "ӯ",
        "Udieresiscyr": "Ӱ",
        "udieresiscyr": "ӱ",
        "Uacutedblcyr": "Ӳ",
        "uacutedblcyr": "ӳ",
        "Chedieresiscyr": "Ӵ",
        "chedieresiscyr": "ӵ",
        "Getailcyr": "Ӷ",
        "getailcyr": "ӷ",
        "Ylongdieresiscyr": "Ӹ",
        "ylongdieresiscyr": "ӹ",
        "Gehookstrokecyr": "Ӻ",
        "gehookstrokecyr": "ӻ",
        "Hahookcyr": "Ӽ",
        "hahookcyr": "ӽ",
        "Hastrokecyr": "Ӿ",
        "hastrokecyr": "ӿ",
        "Dekomicyr": "Ԁ",
        "dekomicyr": "ԁ",
        "Djekomicyr": "Ԃ",
        "djekomicyr": "ԃ",
        "Zjekomicyr": "Ԅ",
        "zjekomicyr": "ԅ",
        "Dzjekomicyr": "Ԇ",
        "dzjekomicyr": "ԇ",
        "Ljekomicyr": "Ԉ",
        "ljekomicyr": "ԉ",
        "Njekomicyr": "Ԋ",
        "njekomicyr": "ԋ",
        "Sjekomicyr": "Ԍ",
        "sjekomicyr": "ԍ",
        "Tjekomicyr": "Ԏ",
        "tjekomicyr": "ԏ",
        "Reversedzecyr": "Ԑ",
        "reversedzecyr": "ԑ",
        "Elhookcyr": "Ԓ",
        "elhookcyr": "ԓ",
        "Lhacyr": "Ԕ",
        "lhacyr": "ԕ",
        "Rhacyr": "Ԗ",
        "rhacyr": "ԗ",
        "Yaecyr": "Ԙ",
        "yaecyr": "ԙ",
        "Qacyr": "Ԛ",
        "qacyr": "ԛ",
        "Wecyr": "Ԝ",
        "wecyr": "ԝ",
        "Kaaleutcyr": "Ԟ",
        "kaaleutcyr": "ԟ",
        "Elmiddlehookcyr": "Ԡ",
        "elmiddlehookcyr": "ԡ",
        "Enmiddlehookcyr": "Ԣ",
        "enmiddlehookcyr": "ԣ",
        "Petailcyr": "Ԥ",
        "petailcyr": "ԥ",
        "Shhatailcyr": "Ԧ",
        "shhatailcyr": "ԧ",
        "Enhookleftcyr": "Ԩ",
        "enhookleftcyr": "ԩ",
        "Dzzhecyr": "Ԫ",
        "dzzhecyr": "ԫ",
        "Dchecyr": "Ԭ",
        "dchecyr": "ԭ",
        "Eltailcyr": "Ԯ",
        "eltailcyr": "ԯ",
        "Aybarmn": "Ա",
        "Benarmn": "Բ",
        "Gimarmn": "Գ",
        "Daarmn": "Դ",
        "Echarmn": "Ե",
        "Zaarmn": "Զ",
        "Eharmn": "Է",
        "Etarmn": "Ը",
        "Toarmn": "Թ",
        "Zhearmn": "Ժ",
        "Iniarmn": "Ի",
        "Liwnarmn": "Լ",
        "Xeharmn": "Խ",
        "Caarmn": "Ծ",
        "Kenarmn": "Կ",
        "Hoarmn": "Հ",
        "Jaarmn": "Ձ",
        "Ghadarmn": "Ղ",
        "Cheharmn": "Ճ",
        "Menarmn": "Մ",
        "Yiarmn": "Յ",
        "Nowarmn": "Ն",
        "Shaarmn": "Շ",
        "Voarmn": "Ո",
        "Chaarmn": "Չ",
        "Peharmn": "Պ",
        "Jheharmn": "Ջ",
        "Raarmn": "Ռ",
        "Seharmn": "Ս",
        "Vewarmn": "Վ",
        "Tiwnarmn": "Տ",
        "Reharmn": "Ր",
        "Coarmn": "Ց",
        "Yiwnarmn": "Ւ",
        "Piwrarmn": "Փ",
        "Keharmn": "Ք",
        "Oharmn": "Օ",
        "Feharmn": "Ֆ",
        "ringhalfleftarmn": "ՙ",
        "apostrophearmn": "՚",
        "emphasismarkarmn": "՛",
        "exclamarmn": "՜",
        "commaarmn": "՝",
        "questionarmn": "՞",
        "abbreviationmarkarmn": "՟",
        "turnedaybarmn": "ՠ",
        "aybarmn": "ա",
        "benarmn": "բ",
        "gimarmn": "գ",
        "daarmn": "դ",
        "echarmn": "ե",
        "zaarmn": "զ",
        "eharmn": "է",
        "etarmn": "ը",
        "toarmn": "թ",
        "zhearmn": "ժ",
        "iniarmn": "ի",
        "liwnarmn": "լ",
        "xeharmn": "խ",
        "caarmn": "ծ",
        "kenarmn": "կ",
        "hoarmn": "հ",
        "jaarmn": "ձ",
        "ghadarmn": "ղ",
        "cheharmn": "ճ",
        "menarmn": "մ",
        "yiarmn": "յ",
        "nowarmn": "ն",
        "shaarmn": "շ",
        "voarmn": "ո",
        "chaarmn": "չ",
        "peharmn": "պ",
        "jheharmn": "ջ",
        "raarmn": "ռ",
        "seharmn": "ս",
        "vewarmn": "վ",
        "tiwnarmn": "տ",
        "reharmn": "ր",
        "coarmn": "ց",
        "yiwnarmn": "ւ",
        "piwrarmn": "փ",
        "keharmn": "ք",
        "oharmn": "օ",
        "feharmn": "ֆ",
        "ech_yiwnarmn": "և",
        "yiwithstrokearmn": "ֈ",
        "periodarmn": "։",
        "hyphenarmn": "֊",
        "rightfacingeternitysignarmn": "֍",
        "leftfacingeternitysignarmn": "֎",
        "dramsignarmn": "֏",
        "etnahta:hb": "֑",
        "segolta:hb": "֒",
        "shalshelet:hb": "֓",
        "zaqefQatan:hb": "֔",
        "zaqefGadol:hb": "֕",
        "tifcha:hb": "֖",
        "revia:hb": "֗",
        "zarqa:hb": "֘",
        "pashta:hb": "֙",
        "yetiv:hb": "֚",
        "tevir:hb": "֛",
        "azla:hb": "֜",
        "gereshMuqdam:hb": "֝",
        "SheneGerishin:hb": "֞",
        "qarneFarah:hb": "֟",
        "telishaGedolah:hb": "֠",
        "pazer:hb": "֡",
        "atnachHafukh:hb": "֢",
        "munach:hb": "֣",
        "mahpach:hb": "֤",
        "mercha:hb": "֥",
        "merchaKefulah:hb": "֦",
        "darga:hb": "֧",
        "qadma:hb": "֨",
        "telishaQetannah:hb": "֩",
        "yerachBenYomo:hb": "֪",
        "ole:hb": "֫",
        "iluy:hb": "֬",
        "dehi:hb": "֭",
        "tsinnorit:hb": "֮",
        "masoraCircle:hb": "֯",
        "sheva:hb": "ְ",
        "hatafSegol:hb": "ֱ",
        "hatafPatah:hb": "ֲ",
        "hatafQamats:hb": "ֳ",
        "hiriq:hb": "ִ",
        "tsere:hb": "ֵ",
        "segol:hb": "ֶ",
        "patah:hb": "ַ",
        "qamats:hb": "ָ",
        "holam:hb": "ֹ",
        "holamHaser:hb": "ֺ",
        "qubuts:hb": "ֻ",
        "dagesh:hb": "ּ",
        "meteg:hb": "ֽ",
        "maqaf:hb": "־",
        "rafe:hb": "ֿ",
        "paseq:hb": "׀",
        "shinDot:hb": "ׁ",
        "sinDot:hb": "ׂ",
        "sofPasuq:hb": "׃",
        "dotupper:hb": "ׄ",
        "dotlower:hb": "ׅ",
        "nunHafukha:hb": "׆",
        "qamatsQatan:hb": "ׇ",
        "alef:hb": "א",
        "bet:hb": "ב",
        "gimel:hb": "ג",
        "dalet:hb": "ד",
        "he:hb": "ה",
        "vav:hb": "ו",
        "zayin:hb": "ז",
        "het:hb": "ח",
        "tet:hb": "ט",
        "yod:hb": "י",
        "finalkaf:hb": "ך",
        "kaf:hb": "כ",
        "lamed:hb": "ל",
        "finalmem:hb": "ם",
        "mem:hb": "מ",
        "finalnun:hb": "ן",
        "nun:hb": "נ",
        "samekh:hb": "ס",
        "ayin:hb": "ע",
        "finalpe:hb": "ף",
        "pe:hb": "פ",
        "finaltsadi:hb": "ץ",
        "tsadi:hb": "צ",
        "qof:hb": "ק",
        "resh:hb": "ר",
        "shin:hb": "ש",
        "tav:hb": "ת",
        "yodtriangle:hb": "ׯ",
        "vav_vav:hb": "װ",
        "vav_yod:hb": "ױ",
        "yod_yod:hb": "ײ",
        "geresh:hb": "׳",
        "gershayim:hb": "״",
        "arnumbersign": "\u0600",
        "Sanah": "\u0601",
        "footnote": "\u0602",
        "Safha": "\u0603",
        "samvat": "\u0604",
        "numbermarkabove": "\u0605",
        "arcuberoot": "؆",
        "arfourthroot": "؇",
        "ray": "؈",
        "permille": "؉",
        "arperthousand": "؊",
        "afghani": "؋",
        "arcomma": "،",
        "dateseparator": "؍",
        "poeticverse": "؎",
        "misra": "؏",
        "HonSAW": "ؐ",
        "HonAA": "ؑ",
        "HonRA": "ؒ",
        "aHonRAA": "ؓ",
        "takhallus": "ؔ",
        "tahabove": "ؕ",
        "alefLamYehabove": "ؖ",
        "zainabove": "ؗ",
        "fathasmall": "ؘ",
        "dammasmall": "ؙ",
        "kasrasmall": "ؚ",
        "arsemicolon": "؛",
        "mark": "\u061c",
        "tripledot": "؞",
        "arquestion": "؟",
        "kashmiriyeh": "ؠ",
        "hamza": "ء",
        "alefmadda": "آ",
        "alefhamza": "أ",
        "wawhamza": "ؤ",
        "alefhamzabelow": "إ",
        "yehhamza": "ئ",
        "alef": "ا",
        "beh": "ب",
        "tehmarbuta": "ة",
        "teh": "ت",
        "theh": "ث",
        "jeem": "ج",
        "hah": "ح",
        "khah": "خ",
        "dal": "د",
        "thal": "ذ",
        "reh": "ر",
        "zain": "ز",
        "seen": "س",
        "sheen": "ش",
        "sad": "ص",
        "dad": "ض",
        "tah": "ط",
        "zah": "ظ",
        "arain": "ع",
        "ghain": "غ",
        "kehehtwodotsabove": "ػ",
        "kehehthreedotsbelow": "ؼ",
        "yehfarsiinvertedV": "ؽ",
        "yehfarsitwodotsabove": "ؾ",
        "yehfarsithreedotsabove": "ؿ",
        "kashida": "ـ",
        "feh": "ف",
        "qaf": "ق",
        "kaf": "ك",
        "lam": "ل",
        "meem": "م",
        "noon": "ن",
        "heh": "ه",
        "waw": "و",
        "alefmaksura": "ى",
        "yeh": "ي",
        "fathatan": "ً",
        "dammatan": "ٌ",
        "kasratan": "ٍ",
        "fatha": "َ",
        "damma": "ُ",
        "kasra": "ِ",
        "shadda": "ّ",
        "sukun": "ْ",
        "madda": "ٓ",
        "hamzaabove": "ٔ",
        "hamzabelow": "ٕ",
        "subscriptalef": "ٖ",
        "inverteddamma": "ٗ",
        "marknoonghunna": "٘",
        "zwarakay": "ٙ",
        "vowelVabove": "ٚ",
        "vowelinvertedVabove": "ٛ",
        "voweldotbelow": "ٜ",
        "dammareversed": "ٝ",
        "fathatwodotsdots": "ٞ",
        "wavyhamzabelow": "ٟ",
        "arzero": "۰",
        "arone": "۱",
        "artwo": "۲",
        "arthree": "۳",
        "arfour": "۴",
        "arfive": "۵",
        "arsix": "۶",
        "arseven": "۷",
        "areight": "۸",
        "arnine": "۹",
        "arpercent": "٪",
        "ardecimalseparator": "٫",
        "thousandsseparator": "٬",
        "fivepointedstar": "٭",
        "dotlessbeh": "ٮ",
        "dotlessqaf": "ٯ",
        "alefabove": "ٰ",
        "alefwasla": "ٱ",
        "alefwavyhamza": "ٲ",
        "alefwavyhamzabelow": "ٳ",
        "highhamza": "ٴ",
        "alefhighhamza": "ٵ",
        "wawhighhamza": "ٶ",
        "uhamza": "ٷ",
        "yehhighhamza": "ٸ",
        "tteh": "ٹ",
        "tteheh": "ٺ",
        "beeh": "ٻ",
        "tehring": "ټ",
        "tehdownthreedotsabove": "ٽ",
        "peh": "پ",
        "teheh": "ٿ",
        "beheh": "ڀ",
        "hahhamza": "ځ",
        "hahtwodotsvertical": "ڂ",
        "nyeh": "ڃ",
        "dyeh": "ڄ",
        "hahthreedotsabove": "څ",
        "tcheh": "چ",
        "tcheheh": "ڇ",
        "ddal": "ڈ",
        "dalring": "ډ",
        "daldotbelow": "ڊ",
        "daldotbelowtahsmall": "ڋ",
        "dahal": "ڌ",
        "ddahal": "ڍ",
        "dul": "ڎ",
        "daldownthreedotsabove": "ڏ",
        "dalfourdotsabove": "ڐ",
        "rreh": "ڑ",
        "rehVabove": "ڒ",
        "rehring": "ړ",
        "rehdotbelow": "ڔ",
        "rehVbelow": "ڕ",
        "rehdotbelowdotabove": "ږ",
        "rehtwodotsabove": "ڗ",
        "jeh": "ژ",
        "rehfourdotsabove": "ڙ",
        "seendotbelowdotabove": "ښ",
        "seenthreedotsbelow": "ڛ",
        "seenthreedotsbelowthreedotsabove": "ڜ",
        "sadtwodotsbelow": "ڝ",
        "sadthreedotsabove": "ڞ",
        "tahthreedotsabove": "ڟ",
        "ainthreedotsabove": "ڠ",
        "dotlessfeh": "ڡ",
        "fehdotbelowright": "ڢ",
        "fehdotbelow": "ڣ",
        "veh": "ڤ",
        "fehthreedotsbelow": "ڥ",
        "peheh": "ڦ",
        "qafdotabove": "ڧ",
        "qafthreedotsabove": "ڨ",
        "keheh": "ک",
        "kafswash": "ڪ",
        "kafring": "ګ",
        "kafdotabove": "ڬ",
        "ng": "ڭ",
        "kafthreedotsbelow": "ڮ",
        "gaf": "گ",
        "gafring": "ڰ",
        "ngoeh": "ڱ",
        "gaftwodotsbelow": "ڲ",
        "gueh": "ڳ",
        "gafthreedotsabove": "ڴ",
        "lamVabove": "ڵ",
        "lamdotabove": "ڶ",
        "lamthreedotsabove": "ڷ",
        "lamthreedotsbelow": "ڸ",
        "noondotbelow": "ڹ",
        "noonghunna": "ں",
        "rnoon": "ڻ",
        "noonring": "ڼ",
        "noonthreedotsabove": "ڽ",
        "hehdoachashmee": "ھ",
        "tchehdotabove": "ڿ",
        "hehyeh": "ۀ",
        "hehgoal": "ہ",
        "hehgoalhamza": "ۂ",
        "tehmarbutagoal": "ۃ",
        "wawring": "ۄ",
        "oekirghiz": "ۅ",
        "aroe": "ۆ",
        "aru": "ۇ",
        "aryu": "ۈ",
        "yukirghiz": "ۉ",
        "wawtwodotsabove": "ۊ",
        "ve": "ۋ",
        "yehfarsi": "ی",
        "yehtail": "ۍ",
        "yehVabove": "ێ",
        "wawdotabove": "ۏ",
        "are": "ې",
        "yehthreedotsbelow": "ۑ",
        "yehbarree": "ے",
        "yehbarreehamza": "ۓ",
        "periodurdu": "۔",
        "arae": "ە",
        "sad_lam_alefmaksuraabove": "ۖ",
        "qaf_lam_alefmaksuraabove": "ۗ",
        "meemabove.init": "ۘ",
        "lamalefabove": "ۙ",
        "jeemabove": "ۚ",
        "threedotsaboveabove": "ۛ",
        "seenabove": "ۜ",
        "Ayahend": "\u06dd",
        "RubElHizbstart": "۞",
        "roundedzeroabove": "۟",
        "zerosquareabove": "۠",
        "dotlesskhahabove": "ۡ",
        "meemabove": "ۢ",
        "seenlow": "ۣ",
        "maddaabove": "ۤ",
        "wawsmall": "ۥ",
        "yehsmall": "ۦ",
        "yehabove": "ۧ",
        "noonabove": "ۨ",
        "Sajdah": "۩",
        "stopbelow": "۪",
        "stopabove": "۫",
        "filledstopabove": "۬",
        "meembelow": "ۭ",
        "dalinvertedV": "ۮ",
        "rehinvertedV": "ۯ",
        "sheendotbelow": "ۺ",
        "daddotbelow": "ۻ",
        "ghaindotbelow": "ۼ",
        "ampersandSindhi": "۽",
        "menpostSindhi": "۾",
        "hehinvertedV": "ۿ",
        "behThreeDotsHorizontallyBelow": "ݐ",
        "behDotBelowThreeDotsAbove": "ݑ",
        "behThreeDotsUpBelow": "ݒ",
        "behThreeDotsUpBelowTwoDotsAbove": "ݓ",
        "behTwoDotsBelowDotAbove": "ݔ",
        "behInvertedSmallVBelow": "ݕ",
        "behSmallV": "ݖ",
        "hahTwoDotsAbove": "ݗ",
        "hahThreeDotsUpBelow": "ݘ",
        "dalTwoDotsVerticallyBelowSmallTah": "ݙ",
        "dalInvertedSmallVBelow": "ݚ",
        "rehStroke": "ݛ",
        "seenFourDotsAbove": "ݜ",
        "ainTwoDotsAbove": "ݝ",
        "ainThreeDotsDownAbove": "ݞ",
        "ainTwoDotsVerticallyAbove": "ݟ",
        "fehTwoDotsBelow": "ݠ",
        "fehThreeDotsUpBelow": "ݡ",
        "kehehDotAbove": "ݢ",
        "kehehThreeDotsAbove": "ݣ",
        "kehehThreeDotsUpBelow": "ݤ",
        "meemDotAbove": "ݥ",
        "meemDotBelow": "ݦ",
        "noonTwoDotsBelow": "ݧ",
        "noonSmallTah": "ݨ",
        "noonSmallV": "ݩ",
        "lamBar": "ݪ",
        "rehTwoDotsVerticallyAbove": "ݫ",
        "rehHamzaAbove": "ݬ",
        "seenTwoDotsVerticallyAbove": "ݭ",
        "hahSmallTahBelow": "ݮ",
        "hahSmallTahTwoDots": "ݯ",
        "seenSmallTahTwoDots": "ݰ",
        "rehSmallTahTwoDots": "ݱ",
        "hahSmallTahAbove": "ݲ",
        "alefDigitTwoAbove": "ݳ",
        "alefDigitThreeAbove": "ݴ",
        "farsiYehDigitTwoAbove": "ݵ",
        "farsiYehDigitThreeAbove": "ݶ",
        "farsiYehDigitFourBelow": "ݷ",
        "wawDigitTwoAbove": "ݸ",
        "wawDigitThreeAbove": "ݹ",
        "yehBarreeDigitTwoAbove": "ݺ",
        "yehBarreeDigitThreeAbove": "ݻ",
        "hahDigitFourBelow": "ݼ",
        "seenDigitFourAbove": "ݽ",
        "seenInvertedV": "ݾ",
        "kafTwoDotsAbove": "ݿ",
        "deva:candrabinduinverted": "ऀ",
        "deva:candrabindu": "ँ",
        "deva:anusvara": "ं",
        "deva:visarga": "ः",
        "deva:ashort": "ऄ",
        "deva:a": "अ",
        "deva:aa": "आ",
        "deva:i": "इ",
        "deva:ii": "ई",
        "deva:u": "उ",
        "deva:uu": "ऊ",
        "deva:vocalicr": "ऋ",
        "deva:vocalicl": "ऌ",
        "deva:ecandra": "ऍ",
        "deva:eshort": "ऎ",
        "deva:e": "ए",
        "deva:ai": "ऐ",
        "deva:ocandra": "ऑ",
        "deva:oshort": "ऒ",
        "deva:o": "ओ",
        "deva:au": "औ",
        "deva:ka": "क",
        "deva:kha": "ख",
        "deva:ga": "ग",
        "deva:gha": "घ",
        "deva:nga": "ङ",
        "deva:ca": "च",
        "deva:cha": "छ",
        "deva:ja": "ज",
        "deva:jha": "झ",
        "deva:nya": "ञ",
        "deva:tta": "ट",
        "deva:ttha": "ठ",
        "deva:dda": "ड",
        "deva:ddha": "ढ",
        "deva:nna": "ण",
        "deva:ta": "त",
        "deva:tha": "थ",
        "deva:da": "द",
        "deva:dha": "ध",
        "deva:na": "न",
        "deva:nnna": "ऩ",
        "deva:pa": "प",
        "deva:pha": "फ",
        "deva:ba": "ब",
        "deva:bha": "भ",
        "deva:ma": "म",
        "deva:ya": "य",
        "deva:ra": "र",
        "deva:rra": "ऱ",
        "deva:la": "ल",
        "deva:lla": "ळ",
        "deva:llla": "ऴ",
        "deva:va": "व",
        "deva:sha": "श",
        "deva:ssa": "ष",
        "deva:sa": "स",
        "deva:ha": "ह",
        "deva:oesign": "ऺ",
        "deva:ooesign": "ऻ",
        "deva:nukta": "़",
        "deva:avagraha": "ऽ",
        "deva:aasign": "ा",
        "deva:isign": "ि",
        "deva:iisign": "ी",
        "deva:usign": "ु",
        "deva:uusign": "ू",
        "deva:vocalicrsign": "ृ",
        "deva:vocalicrrsign": "ॄ",
        "deva:esigncandra": "ॅ",
        "deva:esignshort": "ॆ",
        "deva:esign": "े",
        "deva:aisign": "ै",
        "deva:osigncandra": "ॉ",
        "deva:osignshort": "ॊ",
        "deva:osign": "ो",
        "deva:ausign": "ौ",
        "deva:virama": "्",
        "deva:esignprishthamatra": "ॎ",
        "deva:awsign": "ॏ",
        "deva:om": "ॐ",
        "deva:udatta": "॑",
        "deva:anudatta": "॒",
        "deva:grave": "॓",
        "deva:acute": "॔",
        "deva:esigncandralong": "ॕ",
        "deva:uesign": "ॖ",
        "deva:uuesign": "ॗ",
        "deva:qa": "क़",
        "deva:khha": "ख़",
        "deva:ghha": "ग़",
        "deva:za": "ज़",
        "deva:dddha": "ड़",
        "deva:rha": "ढ़",
        "deva:fa": "फ़",
        "deva:yya": "य़",
        "deva:vocalicrr": "ॠ",
        "deva:vocalicll": "ॡ",
        "deva:vocaliclsign": "ॢ",
        "deva:vocalicllsign": "ॣ",
        "deva:danda": "।",
        "deva:dbldanda": "॥",
        "deva:zero": "०",
        "deva:one": "१",
        "deva:two": "२",
        "deva:three": "३",
        "deva:four": "४",
        "deva:five": "५",
        "deva:six": "६",
        "deva:seven": "७",
        "deva:eight": "८",
        "deva:nine": "९",
        "deva:abbreviation": "॰",
        "deva:dothigh": "ॱ",
        "deva:acandra": "ॲ",
        "deva:oe": "ॳ",
        "deva:ooe": "ॴ",
        "deva:aw": "ॵ",
        "deva:ue": "ॶ",
        "deva:uue": "ॷ",
        "deva:marwaridda": "ॸ",
        "deva:zha": "ॹ",
        "deva:yaheavy": "ॺ",
        "deva:gga": "ॻ",
        "deva:jja": "ॼ",
        "deva:glottalstop": "ॽ",
        "deva:ddda": "ॾ",
        "deva:bba": "ॿ",
        "beng:anji": "ঀ",
        "beng:candrabindu": "ঁ",
        "beng:anusvara": "ং",
        "beng:visarga": "ঃ",
        "beng:a": "অ",
        "beng:aa": "আ",
        "beng:i": "ই",
        "beng:ii": "ঈ",
        "beng:u": "উ",
        "beng:uu": "ঊ",
        "beng:vocalicr": "ঋ",
        "beng:vocalicl": "ঌ",
        "beng:e": "এ",
        "beng:ai": "ঐ",
        "beng:o": "ও",
        "beng:au": "ঔ",
        "beng:ka": "ক",
        "beng:kha": "খ",
        "beng:ga": "গ",
        "beng:gha": "ঘ",
        "beng:nga": "ঙ",
        "beng:ca": "চ",
        "beng:cha": "ছ",
        "beng:ja": "জ",
        "beng:jha": "ঝ",
        "beng:nya": "ঞ",
        "beng:tta": "ট",
        "beng:ttha": "ঠ",
        "beng:dda": "ড",
        "beng:ddha": "ঢ",
        "beng:nna": "ণ",
        "beng:ta": "ত",
        "beng:tha": "থ",
        "beng:da": "দ",
        "beng:dha": "ধ",
        "beng:na": "ন",
        "beng:pa": "প",
        "beng:pha": "ফ",
        "beng:ba": "ব",
        "beng:bha": "ভ",
        "beng:ma": "ম",
        "beng:ya": "য",
        "beng:ra": "র",
        "beng:la": "ল",
        "beng:sha": "শ",
        "beng:ssa": "ষ",
        "beng:sa": "স",
        "beng:ha": "হ",
        "beng:nukta": "়",
        "beng:avagraha": "ঽ",
        "beng:aasign": "া",
        "beng:isign": "ি",
        "beng:iisign": "ী",
        "beng:usign": "ু",
        "beng:uusign": "ূ",
        "beng:vocalicrsign": "ৃ",
        "beng:vocalicrrsign": "ৄ",
        "beng:esign": "ে",
        "beng:aisign": "ৈ",
        "beng:osign": "ো",
        "beng:ausign": "ৌ",
        "beng:virama": "্",
        "beng:khandata": "ৎ",
        "beng:aulengthmark": "ৗ",
        "beng:rra": "ড়",
        "beng:rha": "ঢ়",
        "beng:yya": "য়",
        "beng:vocalicrr": "ৠ",
        "beng:vocalicll": "ৡ",
        "beng:vocaliclsign": "ৢ",
        "beng:vocalicllsign": "ৣ",
        "beng:zero": "০",
        "beng:one": "১",
        "beng:two": "২",
        "beng:three": "৩",
        "beng:four": "৪",
        "beng:five": "৫",
        "beng:six": "৬",
        "beng:seven": "৭",
        "beng:eight": "৮",
        "beng:nine": "৯",
        "beng:ramiddiagonal": "ৰ",
        "beng:ralowdiagonal": "ৱ",
        "beng:rupeemark": "৲",
        "beng:rupee": "৳",
        "beng:onecurrencynumerator": "৴",
        "beng:twocurrencynumerator": "৵",
        "beng:threecurrencynumerator": "৶",
        "beng:fourcurrencynumerator": "৷",
        "beng:currencyoneless": "৸",
        "beng:sixteencurrencydenominator": "৹",
        "beng:isshar": "৺",
        "beng:gandamark": "৻",
        "beng:vedicanusvara": "ৼ",
        "beng:abbreviationsign": "৽",
        "beng:sandhimark": "৾",
        "guru:adakbindisign": "ਁ",
        "guru:bindisign": "ਂ",
        "guru:visarga": "ਃ",
        "guru:a": "ਅ",
        "guru:aa": "ਆ",
        "guru:i": "ਇ",
        "guru:ii": "ਈ",
        "guru:u": "ਉ",
        "guru:uu": "ਊ",
        "guru:ee": "ਏ",
        "guru:ai": "ਐ",
        "guru:oo": "ਓ",
        "guru:au": "ਔ",
        "guru:ka": "ਕ",
        "guru:kha": "ਖ",
        "guru:ga": "ਗ",
        "guru:gha": "ਘ",
        "guru:nga": "ਙ",
        "guru:ca": "ਚ",
        "guru:cha": "ਛ",
        "guru:ja": "ਜ",
        "guru:jha": "ਝ",
        "guru:nya": "ਞ",
        "guru:tta": "ਟ",
        "guru:ttha": "ਠ",
        "guru:dda": "ਡ",
        "guru:ddha": "ਢ",
        "guru:nna": "ਣ",
        "guru:ta": "ਤ",
        "guru:tha": "ਥ",
        "guru:da": "ਦ",
        "guru:dha": "ਧ",
        "guru:na": "ਨ",
        "guru:pa": "ਪ",
        "guru:pha": "ਫ",
        "guru:ba": "ਬ",
        "guru:bha": "ਭ",
        "guru:ma": "ਮ",
        "guru:ya": "ਯ",
        "guru:ra": "ਰ",
        "guru:la": "ਲ",
        "guru:lla": "ਲ਼",
        "guru:va": "ਵ",
        "guru:sha": "ਸ਼",
        "guru:sa": "ਸ",
        "guru:ha": "ਹ",
        "guru:nukta": "਼",
        "guru:aasign": "ਾ",
        "guru:isign": "ਿ",
        "guru:iisign": "ੀ",
        "guru:usign": "ੁ",
        "guru:uusign": "ੂ",
        "guru:eesign": "ੇ",
        "guru:aisign": "ੈ",
        "guru:oosign": "ੋ",
        "guru:ausign": "ੌ",
        "guru:virama": "੍",
        "guru:udaatsign": "ੑ",
        "guru:khha": "ਖ਼",
        "guru:ghha": "ਗ਼",
        "guru:za": "ਜ਼",
        "guru:rra": "ੜ",
        "guru:fa": "ਫ਼",
        "guru:zero": "੦",
        "guru:one": "੧",
        "guru:two": "੨",
        "guru:three": "੩",
        "guru:four": "੪",
        "guru:five": "੫",
        "guru:six": "੬",
        "guru:seven": "੭",
        "guru:eight": "੮",
        "guru:nine": "੯",
        "guru:tippi": "ੰ",
        "guru:addak": "ੱ",
        "guru:iri": "ੲ",
        "guru:ura": "ੳ",
        "guru:ekonkar": "ੴ",
        "guru:yakashsign": "ੵ",
        "guru:abbreviationsign": "੶",
        "gujr:candrabindu": "ઁ",
        "gujr:anusvara": "ં",
        "gujr:visarga": "ઃ",
        "gujr:a": "અ",
        "gujr:aa": "આ",
        "gujr:i": "ઇ",
        "gujr:ii": "ઈ",
        "gujr:u": "ઉ",
        "gujr:uu": "ઊ",
        "gujr:vocalicr": "ઋ",
        "gujr:vocalicl": "ઌ",
        "gujr:ecandra": "ઍ",
        "gujr:e": "એ",
        "gujr:ai": "ઐ",
        "gujr:ocandra": "ઑ",
        "gujr:o": "ઓ",
        "gujr:au": "ઔ",
        "gujr:ka": "ક",
        "gujr:kha": "ખ",
        "gujr:ga": "ગ",
        "gujr:gha": "ઘ",
        "gujr:nga": "ઙ",
        "gujr:ca": "ચ",
        "gujr:cha": "છ",
        "gujr:ja": "જ",
        "gujr:jha": "ઝ",
        "gujr:nya": "ઞ",
        "gujr:tta": "ટ",
        "gujr:ttha": "ઠ",
        "gujr:dda": "ડ",
        "gujr:ddha": "ઢ",
        "gujr:nna": "ણ",
        "gujr:ta": "ત",
        "gujr:tha": "થ",
        "gujr:da": "દ",
        "gujr:dha": "ધ",
        "gujr:na": "ન",
        "gujr:pa": "પ",
        "gujr:pha": "ફ",
        "gujr:ba": "બ",
        "gujr:bha": "ભ",
        "gujr:ma": "મ",
        "gujr:ya": "ય",
        "gujr:ra": "ર",
        "gujr:la": "લ",
        "gujr:lla": "ળ",
        "gujr:va": "વ",
        "gujr:sha": "શ",
        "gujr:ssa": "ષ",
        "gujr:sa": "સ",
        "gujr:ha": "હ",
        "gujr:nukta": "઼",
        "gujr:avagraha": "ઽ",
        "gujr:aasign": "ા",
        "gujr:isign": "િ",
        "gujr:iisign": "ી",
        "gujr:usign": "ુ",
        "gujr:uusign": "ૂ",
        "gujr:vocalicrsign": "ૃ",
        "gujr:vocalicrrsign": "ૄ",
        "gujr:esigncandra": "ૅ",
        "gujr:esign": "ે",
        "gujr:aisign": "ૈ",
        "gujr:osigncandra": "ૉ",
        "gujr:osign": "ો",
        "gujr:ausign": "ૌ",
        "gujr:virama": "્",
        "gujr:om": "ૐ",
        "gujr:vocalicrr": "ૠ",
        "gujr:vocalicll": "ૡ",
        "gujr:vocaliclsign": "ૢ",
        "gujr:vocalicllsign": "ૣ",
        "gujr:zero": "૦",
        "gujr:one": "૧",
        "gujr:two": "૨",
        "gujr:three": "૩",
        "gujr:four": "૪",
        "gujr:five": "૫",
        "gujr:six": "૬",
        "gujr:seven": "૭",
        "gujr:eight": "૮",
        "gujr:nine": "૯",
        "gujr:abbreviation": "૰",
        "gujr:rupee": "૱",
        "gujr:zha": "ૹ",
        "gujr:sukun": "ૺ",
        "gujr:shadda": "ૻ",
        "gujr:maddah": "ૼ",
        "gujr:threedotnuktaabove": "૽",
        "gujr:circlenuktaabove": "૾",
        "gujr:twocirclenuktaabove": "૿",
        "orya:candrabindu": "ଁ",
        "orya:anusvara": "ଂ",
        "orya:visarga": "ଃ",
        "orya:a": "ଅ",
        "orya:aa": "ଆ",
        "orya:i": "ଇ",
        "orya:ii": "ଈ",
        "orya:u": "ଉ",
        "orya:uu": "ଊ",
        "orya:vocalicr": "ଋ",
        "orya:vocalicl": "ଌ",
        "orya:e": "ଏ",
        "orya:ai": "ଐ",
        "orya:o": "ଓ",
        "orya:au": "ଔ",
        "orya:ka": "କ",
        "orya:kha": "ଖ",
        "orya:ga": "ଗ",
        "orya:gha": "ଘ",
        "orya:nga": "ଙ",
        "orya:ca": "ଚ",
        "orya:cha": "ଛ",
        "orya:ja": "ଜ",
        "orya:jha": "ଝ",
        "orya:nya": "ଞ",
        "orya:tta": "ଟ",
        "orya:ttha": "ଠ",
        "orya:dda": "ଡ",
        "orya:ddha": "ଢ",
        "orya:nna": "ଣ",
        "orya:ta": "ତ",
        "orya:tha": "ଥ",
        "orya:da": "ଦ",
        "orya:dha": "ଧ",
        "orya:na": "ନ",
        "orya:pa": "ପ",
        "orya:pha": "ଫ",
        "orya:ba": "ବ",
        "orya:bha": "ଭ",
        "orya:ma": "ମ",
        "orya:ya": "ଯ",
        "orya:ra": "ର",
        "orya:la": "ଲ",
        "orya:lla": "ଳ",
        "orya:va": "ଵ",
        "orya:sha": "ଶ",
        "orya:ssa": "ଷ",
        "orya:sa": "ସ",
        "orya:ha": "ହ",
        "orya:nukta": "଼",
        "orya:avagraha": "ଽ",
        "orya:aasign": "ା",
        "orya:isign": "ି",
        "orya:iisign": "ୀ",
        "orya:usign": "ୁ",
        "orya:uusign": "ୂ",
        "orya:vocalicrsign": "ୃ",
        "orya:vocalicrrsign": "ୄ",
        "orya:esign": "େ",
        "orya:aisign": "ୈ",
        "orya:osign": "ୋ",
        "orya:ausign": "ୌ",
        "orya:virama": "୍",
        "orya:ailengthmark": "ୖ",
        "orya:aulengthmark": "ୗ",
        "orya:rra": "ଡ଼",
        "orya:rha": "ଢ଼",
        "orya:yya": "ୟ",
        "orya:vocalicrr": "ୠ",
        "orya:vocalicll": "ୡ",
        "orya:vocaliclsign": "ୢ",
        "orya:vocalicllsign": "ୣ",
        "orya:zero": "୦",
        "orya:one": "୧",
        "orya:two": "୨",
        "orya:three": "୩",
        "orya:four": "୪",
        "orya:five": "୫",
        "orya:six": "୬",
        "orya:seven": "୭",
        "orya:eight": "୮",
        "orya:nine": "୯",
        "orya:isshar": "୰",
        "orya:wa": "ୱ",
        "orya:onequarter": "୲",
        "orya:onehalf": "୳",
        "orya:threequarters": "୴",
        "orya:onesixteenth": "୵",
        "orya:oneeighth": "୶",
        "orya:threesixteenths": "୷",
        "taml:anusvara": "ஂ",
        "taml:visarga": "ஃ",
        "taml:a": "அ",
        "taml:aa": "ஆ",
        "taml:i": "இ",
        "taml:ii": "ஈ",
        "taml:u": "உ",
        "taml:uu": "ஊ",
        "taml:e": "எ",
        "taml:ee": "ஏ",
        "taml:ai": "ஐ",
        "taml:o": "ஒ",
        "taml:oo": "ஓ",
        "taml:au": "ஔ",
        "taml:ka": "க",
        "taml:nga": "ங",
        "taml:ca": "ச",
        "taml:ja": "ஜ",
        "taml:nya": "ஞ",
        "taml:tta": "ட",
        "taml:nna": "ண",
        "taml:ta": "த",
        "taml:na": "ந",
        "taml:nnna": "ன",
        "taml:pa": "ப",
        "taml:ma": "ம",
        "taml:ya": "ய",
        "taml:ra": "ர",
        "taml:rra": "ற",
        "taml:la": "ல",
        "taml:lla": "ள",
        "taml:llla": "ழ",
        "taml:va": "வ",
        "taml:sha": "ஶ",
        "taml:ssa": "ஷ",
        "taml:sa": "ஸ",
        "taml:ha": "ஹ",
        "taml:aasign": "ா",
        "taml:isign": "ி",
        "taml:iisign": "ீ",
        "taml:usign": "ு",
        "taml:uusign": "ூ",
        "taml:esign": "ெ",
        "taml:eesign": "ே",
        "taml:aisign": "ை",
        "taml:osign": "ொ",
        "taml:oosign": "ோ",
        "taml:ausign": "ௌ",
        "taml:virama": "்",
        "taml:om": "ௐ",
        "taml:aulengthmark": "ௗ",
        "taml:zero": "௦",
        "taml:one": "௧",
        "taml:two": "௨",
        "taml:three": "௩",
        "taml:four": "௪",
        "taml:five": "௫",
        "taml:six": "௬",
        "taml:seven": "௭",
        "taml:eight": "௮",
        "taml:nine": "௯",
        "taml:ten": "௰",
        "taml:onehundred": "௱",
        "taml:onethousand": "௲",
        "taml:daysign": "௳",
        "taml:monthsign": "௴",
        "taml:yearsign": "௵",
        "taml:debitsign": "௶",
        "taml:creditsign": "௷",
        "taml:asabovesign": "௸",
        "taml:rupee": "௹",
        "taml:sign": "௺",
        "telu:combiningcandrabinduabovesign": "ఀ",
        "telu:candrabindusign": "ఁ",
        "telu:anusvara": "ం",
        "telu:visarga": "ః",
        "telu:combininganusvaraabovesign": "ఄ",
        "telu:a": "అ",
        "telu:aa": "ఆ",
        "telu:i": "ఇ",
        "telu:ii": "ఈ",
        "telu:u": "ఉ",
        "telu:uu": "ఊ",
        "telu:vocalicr": "ఋ",
        "telu:vocalicl": "ఌ",
        "telu:e": "ఎ",
        "telu:ee": "ఏ",
        "telu:ai": "ఐ",
        "telu:o": "ఒ",
        "telu:oo": "ఓ",
        "telu:au": "ఔ",
        "telu:ka": "క",
        "telu:kha": "ఖ",
        "telu:ga": "గ",
        "telu:gha": "ఘ",
        "telu:nga": "ఙ",
        "telu:ca": "చ",
        "telu:cha": "ఛ",
        "telu:ja": "జ",
        "telu:jha": "ఝ",
        "telu:nya": "ఞ",
        "telu:tta": "ట",
        "telu:ttha": "ఠ",
        "telu:dda": "డ",
        "telu:ddha": "ఢ",
        "telu:nna": "ణ",
        "telu:ta": "త",
        "telu:tha": "థ",
        "telu:da": "ద",
        "telu:dha": "ధ",
        "telu:na": "న",
        "telu:pa": "ప",
        "telu:pha": "ఫ",
        "telu:ba": "బ",
        "telu:bha": "భ",
        "telu:ma": "మ",
        "telu:ya": "య",
        "telu:ra": "ర",
        "telu:rra": "ఱ",
        "telu:la": "ల",
        "telu:lla": "ళ",
        "telu:llla": "ఴ",
        "telu:va": "వ",
        "telu:sha": "శ",
        "telu:ssa": "ష",
        "telu:sa": "స",
        "telu:ha": "హ",
        "telu:avagraha": "ఽ",
        "telu:aasign": "ా",
        "telu:isign": "ి",
        "telu:iisign": "ీ",
        "telu:usign": "ు",
        "telu:uusign": "ూ",
        "telu:vocalicrsign": "ృ",
        "telu:vocalicrrsign": "ౄ",
        "telu:esign": "ె",
        "telu:eesign": "ే",
        "telu:aisign": "ై",
        "telu:osign": "ొ",
        "telu:oosign": "ో",
        "telu:ausign": "ౌ",
        "telu:virama": "్",
        "telu:lengthmark": "ౕ",
        "telu:ailengthmark": "ౖ",
        "telu:tsa": "ౘ",
        "telu:dza": "ౙ",
        "telu:rrra": "ౚ",
        "telu:vocalicrr": "ౠ",
        "telu:vocalicll": "ౡ",
        "telu:vocaliclsign": "ౢ",
        "telu:vocalicllsign": "ౣ",
        "telu:zero": "౦",
        "telu:one": "౧",
        "telu:two": "౨",
        "telu:three": "౩",
        "telu:four": "౪",
        "telu:five": "౫",
        "telu:six": "౬",
        "telu:seven": "౭",
        "telu:eight": "౮",
        "telu:nine": "౯",
        "telu:siddhamsign": "౷",
        "telu:fractionzeroforoddpowersoffour": "౸",
        "telu:fractiononeforoddpowersoffour": "౹",
        "telu:fractiontwoforoddpowersoffour": "౺",
        "telu:fractionthreeforoddpowersoffour": "౻",
        "telu:fractiononeforevenpowersoffour": "౼",
        "telu:fractiontwoforevenpowersoffour": "౽",
        "telu:fractionthreeforevenpowersoffour": "౾",
        "telu:tuumusign": "౿",
        "knda:signspacingcandrabindu": "ಀ",
        "knda:signcandrabindu": "ಁ",
        "knda:anusvara": "ಂ",
        "knda:visarga": "ಃ",
        "knda:signsiddham": "಄",
        "knda:a": "ಅ",
        "knda:aa": "ಆ",
        "knda:i": "ಇ",
        "knda:ii": "ಈ",
        "knda:u": "ಉ",
        "knda:uu": "ಊ",
        "knda:rvocal": "ಋ",
        "knda:lvocal": "ಌ",
        "knda:e": "ಎ",
        "knda:ee": "ಏ",
        "knda:ai": "ಐ",
        "knda:o": "ಒ",
        "knda:oo": "ಓ",
        "knda:au": "ಔ",
        "knda:ka": "ಕ",
        "knda:kha": "ಖ",
        "knda:ga": "ಗ",
        "knda:gha": "ಘ",
        "knda:nga": "ಙ",
        "knda:ca": "ಚ",
        "knda:cha": "ಛ",
        "knda:ja": "ಜ",
        "knda:jha": "ಝ",
        "knda:nya": "ಞ",
        "knda:tta": "ಟ",
        "knda:ttha": "ಠ",
        "knda:dda": "ಡ",
        "knda:ddha": "ಢ",
        "knda:nna": "ಣ",
        "knda:ta": "ತ",
        "knda:tha": "ಥ",
        "knda:da": "ದ",
        "knda:dha": "ಧ",
        "knda:na": "ನ",
        "knda:pa": "ಪ",
        "knda:pha": "ಫ",
        "knda:ba": "ಬ",
        "knda:bha": "ಭ",
        "knda:ma": "ಮ",
        "knda:ya": "ಯ",
        "knda:ra": "ರ",
        "knda:rra": "ಱ",
        "knda:la": "ಲ",
        "knda:lla": "ಳ",
        "knda:va": "ವ",
        "knda:sha": "ಶ",
        "knda:ssa": "ಷ",
        "knda:sa": "ಸ",
        "knda:ha": "ಹ",
        "knda:nukta": "಼",
        "knda:avagraha": "ಽ",
        "knda:aasign": "ಾ",
        "knda:isign": "ಿ",
        "knda:iisign": "ೀ",
        "knda:usign": "ು",
        "knda:uusign": "ೂ",
        "knda:rvocalsign": "ೃ",
        "knda:rrvocalsign": "ೄ",
        "knda:esign": "ೆ",
        "knda:eesign": "ೇ",
        "knda:aisign": "ೈ",
        "knda:osign": "ೊ",
        "knda:oosign": "ೋ",
        "knda:ausign": "ೌ",
        "knda:virama": "್",
        "knda:length": "ೕ",
        "knda:ailength": "ೖ",
        "knda:fa": "ೞ",
        "knda:rrvocal": "ೠ",
        "knda:llvocal": "ೡ",
        "knda:lvocalsign": "ೢ",
        "knda:llvocalsign": "ೣ",
        "knda:zero": "೦",
        "knda:one": "೧",
        "knda:two": "೨",
        "knda:three": "೩",
        "knda:four": "೪",
        "knda:five": "೫",
        "knda:six": "೬",
        "knda:seven": "೭",
        "knda:eight": "೮",
        "knda:nine": "೯",
        "knda:jihvamuliya": "ೱ",
        "knda:upadhmaniya": "ೲ",
        "mlym:combininganusvaraabovesign": "ഀ",
        "mlym:candrabindusign": "ഁ",
        "mlym:anusvarasign": "ം",
        "mlym:visargasign": "ഃ",
        "mlym:a": "അ",
        "mlym:aa": "ആ",
        "mlym:i": "ഇ",
        "mlym:ii": "ഈ",
        "mlym:u": "ഉ",
        "mlym:uu": "ഊ",
        "mlym:rvocal": "ഋ",
        "mlym:lvocal": "ഌ",
        "mlym:e": "എ",
        "mlym:ee": "ഏ",
        "mlym:ai": "ഐ",
        "mlym:o": "ഒ",
        "mlym:oo": "ഓ",
        "mlym:au": "ഔ",
        "mlym:ka": "ക",
        "mlym:kha": "ഖ",
        "mlym:ga": "ഗ",
        "mlym:gha": "ഘ",
        "mlym:nga": "ങ",
        "mlym:ca": "ച",
        "mlym:cha": "ഛ",
        "mlym:ja": "ജ",
        "mlym:jha": "ഝ",
        "mlym:nya": "ഞ",
        "mlym:tta": "ട",
        "mlym:ttha": "ഠ",
        "mlym:dda": "ഡ",
        "mlym:ddha": "ഢ",
        "mlym:nna": "ണ",
        "mlym:ta": "ത",
        "mlym:tha": "ഥ",
        "mlym:da": "ദ",
        "mlym:dha": "ധ",
        "mlym:na": "ന",
        "mlym:nnna": "ഩ",
        "mlym:pa": "പ",
        "mlym:pha": "ഫ",
        "mlym:ba": "ബ",
        "mlym:bha": "ഭ",
        "mlym:ma": "മ",
        "mlym:ya": "യ",
        "mlym:ra": "ര",
        "mlym:rra": "റ",
        "mlym:la": "ല",
        "mlym:lla": "ള",
        "mlym:llla": "ഴ",
        "mlym:va": "വ",
        "mlym:sha": "ശ",
        "mlym:ssa": "ഷ",
        "mlym:sa": "സ",
        "mlym:ha": "ഹ",
        "mlym:ttta": "ഺ",
        "mlym:verticalbarviramasign": "഻",
        "mlym:circularviramasign": "഼",
        "mlym:avagrahasign": "ഽ",
        "mlym:aasign": "ാ",
        "mlym:isign": "ി",
        "mlym:iisign": "ീ",
        "mlym:usign": "ു",
        "mlym:uusign": "ൂ",
        "mlym:rvocalsign": "ൃ",
        "mlym:rrvocalsign": "ൄ",
        "mlym:esign": "െ",
        "mlym:eesign": "േ",
        "mlym:aisign": "ൈ",
        "mlym:osign": "ൊ",
        "mlym:oosign": "ോ",
        "mlym:ausign": "ൌ",
        "mlym:viramasign": "്",
        "mlym:dotreph": "ൎ",
        "mlym:parasign": "൏",
        "mlym:mchillu": "ൔ",
        "mlym:ychillu": "ൕ",
        "mlym:lllchillu": "ൖ",
        "mlym:aulength": "ൗ",
        "mlym:oneone-hundred-and-sixtieth": "൘",
        "mlym:onefortieth": "൙",
        "mlym:threeeightieths": "൚",
        "mlym:onetwentieth": "൛",
        "mlym:onetenth": "൜",
        "mlym:threetwentieths": "൝",
        "mlym:onefifth": "൞",
        "mlym:archaicii": "ൟ",
        "mlym:rrvocal": "ൠ",
        "mlym:llvocal": "ൡ",
        "mlym:lvocalsign": "ൢ",
        "mlym:llvocalsign": "ൣ",
        "mlym:zero": "൦",
        "mlym:one": "൧",
        "mlym:two": "൨",
        "mlym:three": "൩",
        "mlym:four": "൪",
        "mlym:five": "൫",
        "mlym:six": "൬",
        "mlym:seven": "൭",
        "mlym:eight": "൮",
        "mlym:nine": "൯",
        "mlym:ten": "൰",
        "mlym:onehundred": "൱",
        "mlym:onethousand": "൲",
        "mlym:onequarter": "൳",
        "mlym:onehalf": "൴",
        "mlym:threequarters": "൵",
        "mlym:onesixteenth": "൶",
        "mlym:oneeighth": "൷",
        "mlym:threesixteenths": "൸",
        "mlym:date": "൹",
        "mlym:nnchillu": "ൺ",
        "mlym:nchillu": "ൻ",
        "mlym:rrchillu": "ർ",
        "mlym:lchillu": "ൽ",
        "mlym:llchillu": "ൾ",
        "mlym:kchillu": "ൿ",
        "sinh:anusvara": "ං",
        "sinh:visarga": "ඃ",
        "sinh:a": "අ",
        "sinh:aa": "ආ",
        "sinh:ae": "ඇ",
        "sinh:aae": "ඈ",
        "sinh:i": "ඉ",
        "sinh:ii": "ඊ",
        "sinh:u": "උ",
        "sinh:uu": "ඌ",
        "sinh:vocalicr": "ඍ",
        "sinh:vocalicrr": "ඎ",
        "sinh:vocalicl": "ඏ",
        "sinh:vocalicll": "ඐ",
        "sinh:e": "එ",
        "sinh:ee": "ඒ",
        "sinh:ai": "ඓ",
        "sinh:o": "ඔ",
        "sinh:oo": "ඕ",
        "sinh:au": "ඖ",
        "sinh:ka": "ක",
        "sinh:kha": "ඛ",
        "sinh:ga": "ග",
        "sinh:gha": "ඝ",
        "sinh:nga": "ඞ",
        "sinh:nnga": "ඟ",
        "sinh:ca": "ච",
        "sinh:cha": "ඡ",
        "sinh:ja": "ජ",
        "sinh:jha": "ඣ",
        "sinh:nya": "ඤ",
        "sinh:jnya": "ඥ",
        "sinh:nyja": "ඦ",
        "sinh:tta": "ට",
        "sinh:ttha": "ඨ",
        "sinh:dda": "ඩ",
        "sinh:ddha": "ඪ",
        "sinh:nna": "ණ",
        "sinh:nndda": "ඬ",
        "sinh:ta": "ත",
        "sinh:tha": "ථ",
        "sinh:da": "ද",
        "sinh:dha": "ධ",
        "sinh:na": "න",
        "sinh:nda": "ඳ",
        "sinh:pa": "ප",
        "sinh:pha": "ඵ",
        "sinh:ba": "බ",
        "sinh:bha": "භ",
        "sinh:ma": "ම",
        "sinh:mba": "ඹ",
        "sinh:ya": "ය",
        "sinh:ra": "ර",
        "sinh:la": "ල",
        "sinh:va": "ව",
        "sinh:sha": "ශ",
        "sinh:ssa": "ෂ",
        "sinh:sa": "ස",
        "sinh:ha": "හ",
        "sinh:lla": "ළ",
        "sinh:fa": "ෆ",
        "sinh:virama": "්",
        "sinh:aasign": "ා",
        "sinh:aesign": "ැ",
        "sinh:aaesign": "ෑ",
        "sinh:isign": "ි",
        "sinh:iisign": "ී",
        "sinh:usign": "ු",
        "sinh:uusign": "ූ",
        "sinh:vocalicrsign": "ෘ",
        "sinh:esign": "ෙ",
        "sinh:eesign": "ේ",
        "sinh:aisign": "ෛ",
        "sinh:osign": "ො",
        "sinh:oosign": "ෝ",
        "sinh:ausign": "ෞ",
        "sinh:vocaliclsign": "ෟ",
        "sinh:zerolith": "෦",
        "sinh:onelith": "෧",
        "sinh:twolith": "෨",
        "sinh:threelith": "෩",
        "sinh:fourlith": "෪",
        "sinh:fivelith": "෫",
        "sinh:sixlith": "෬",
        "sinh:sevenlith": "෭",
        "sinh:eightlith": "෮",
        "sinh:ninelith": "෯",
        "sinh:vocalicrrsign": "ෲ",
        "sinh:vocalicllsign": "ෳ",
        "sinh:kunddaliya": "෴",
        "thai:kokai": "ก",
        "thai:khokhai": "ข",
        "thai:khokhuat": "ฃ",
        "thai:khokhwai": "ค",
        "thai:khokhon": "ฅ",
        "thai:khorakhang": "ฆ",
        "thai:ngongu": "ง",
        "thai:chochan": "จ",
        "thai:choching": "ฉ",
        "thai:chochang": "ช",
        "thai:soso": "ซ",
        "thai:chochoe": "ฌ",
        "thai:yoying": "ญ",
        "thai:dochada": "ฎ",
        "thai:topatak": "ฏ",
        "thai:thothan": "ฐ",
        "thai:thonangmontho": "ฑ",
        "thai:thophuthao": "ฒ",
        "thai:nonen": "ณ",
        "thai:dodek": "ด",
        "thai:totao": "ต",
        "thai:thothung": "ถ",
        "thai:thothahan": "ท",
        "thai:thothong": "ธ",
        "thai:nonu": "น",
        "thai:bobaimai": "บ",
        "thai:popla": "ป",
        "thai:phophung": "ผ",
        "thai:fofa": "ฝ",
        "thai:phophan": "พ",
        "thai:fofan": "ฟ",
        "thai:phosamphao": "ภ",
        "thai:moma": "ม",
        "thai:yoyak": "ย",
        "thai:rorua": "ร",
        "thai:ru": "ฤ",
        "thai:loling": "ล",
        "thai:lu": "ฦ",
        "thai:wowaen": "ว",
        "thai:sosala": "ศ",
        "thai:sorusi": "ษ",
        "thai:sosua": "ส",
        "thai:hohip": "ห",
        "thai:lochula": "ฬ",
        "thai:oang": "อ",
        "thai:honokhuk": "ฮ",
        "thai:paiyannoi": "ฯ",
        "thai:saraa": "ะ",
        "thai:maihan-akat": "ั",
        "thai:saraaa": "า",
        "thai:saraam": "ำ",
        "thai:sarai": "ิ",
        "thai:saraii": "ี",
        "thai:saraue": "ึ",
        "thai:sarauee": "ื",
        "thai:sarau": "ุ",
        "thai:sarauu": "ู",
        "thai:phinthu": "ฺ",
        "thai:baht": "฿",
        "thai:sarae": "เ",
        "thai:saraae": "แ",
        "thai:sarao": "โ",
        "thai:saraaimaimuan": "ใ",
        "thai:saraaimaimalai": "ไ",
        "thai:lakkhangyao": "ๅ",
        "thai:maiyamok": "ๆ",
        "thai:maitaikhu": "็",
        "thai:maiek": "่",
        "thai:maitho": "้",
        "thai:maitri": "๊",
        "thai:maichattawa": "๋",
        "thai:thanthakhat": "์",
        "thai:nikhahit": "ํ",
        "thai:yamakkan": "๎",
        "thai:fongman": "๏",
        "thai:zero": "๐",
        "thai:one": "๑",
        "thai:two": "๒",
        "thai:three": "๓",
        "thai:four": "๔",
        "thai:five": "๕",
        "thai:six": "๖",
        "thai:seven": "๗",
        "thai:eight": "๘",
        "thai:nine": "๙",
        "thai:angkhankhu": "๚",
        "thai:khomut": "๛",
        "tibt:omsyllable": "ༀ",
        "tibt:gteryigmgotruncatedamark": "༁",
        "tibt:gteryigmgoumrnambcadmamark": "༂",
        "tibt:gteryigmgoumgtertshegmamark": "༃",
        "tibt:yigmgomdunmainitialmark": "༄",
        "tibt:yigmgosgabmaclosingmark": "༅",
        "tibt:caretyigmgophurshadmamark": "༆",
        "tibt:yigmgotshegshadmamark": "༇",
        "tibt:sbrulshadmark": "༈",
        "tibt:bskuryigmgomark": "༉",
        "tibt:bkashogyigmgomark": "༊",
        "tibt:intersyllabictshegmark": "་",
        "tibt:delimitertshegbstarmark": "༌",
        "tibt:shadmark": "།",
        "tibt:nyisshadmark": "༎",
        "tibt:tshegshadmark": "༏",
        "tibt:nyistshegshadmark": "༐",
        "tibt:rinchenspungsshadmark": "༑",
        "tibt:rgyagramshadmark": "༒",
        "tibt:caretdzudrtagsmelongcanmark": "༓",
        "tibt:gtertshegmark": "༔",
        "tibt:chadrtagslogotypesign": "༕",
        "tibt:lhagrtagslogotypesign": "༖",
        "tibt:astrologicalsgragcancharrtagssign": "༗",
        "tibt:astrologicalkhyudpasign": "༘",
        "tibt:astrologicalsdongtshugssign": "༙",
        "tibt:rdeldkargcigsign": "༚",
        "tibt:rdeldkargnyissign": "༛",
        "tibt:rdeldkargsumsign": "༜",
        "tibt:rdelnaggcigsign": "༝",
        "tibt:rdelnaggnyissign": "༞",
        "tibt:rdeldkarrdelnagsign": "༟",
        "tibt:zero": "༠",
        "tibt:one": "༡",
        "tibt:two": "༢",
        "tibt:three": "༣",
        "tibt:four": "༤",
        "tibt:five": "༥",
        "tibt:six": "༦",
        "tibt:seven": "༧",
        "tibt:eight": "༨",
        "tibt:nine": "༩",
        "tibt:halfone": "༪",
        "tibt:halftwo": "༫",
        "tibt:halfthree": "༬",
        "tibt:halffour": "༭",
        "tibt:halffive": "༮",
        "tibt:halfsix": "༯",
        "tibt:halfseven": "༰",
        "tibt:halfeight": "༱",
        "tibt:halfnine": "༲",
        "tibt:halfzero": "༳",
        "tibt:bsdusrtagsmark": "༴",
        "tibt:ngasbzungnyizlamark": "༵",
        "tibt:caretdzudrtagsbzhimigcanmark": "༶",
        "tibt:ngasbzungsgorrtagsmark": "༷",
        "tibt:chemgomark": "༸",
        "tibt:tsaphrumark": "༹",
        "tibt:gugrtagsgyonmark": "༺",
        "tibt:gugrtagsgyasmark": "༻",
        "tibt:angkhanggyonmark": "༼",
        "tibt:angkhanggyasmark": "༽",
        "tibt:yartshessign": "༾",
        "tibt:martshessign": "༿",
        "tibt:ka": "ཀ",
        "tibt:kha": "ཁ",
        "tibt:ga": "ག",
        "tibt:gha": "གྷ",
        "tibt:nga": "ང",
        "tibt:ca": "ཅ",
        "tibt:cha": "ཆ",
        "tibt:ja": "ཇ",
        "tibt:nya": "ཉ",
        "tibt:tta": "ཊ",
        "tibt:ttha": "ཋ",
        "tibt:dda": "ཌ",
        "tibt:ddha": "ཌྷ",
        "tibt:nna": "ཎ",
        "tibt:ta": "ཏ",
        "tibt:tha": "ཐ",
        "tibt:da": "ད",
        "tibt:dha": "དྷ",
        "tibt:na": "ན",
        "tibt:pa": "པ",
        "tibt:pha": "ཕ",
        "tibt:ba": "བ",
        "tibt:bha": "བྷ",
        "tibt:ma": "མ",
        "tibt:tsa": "ཙ",
        "tibt:tsha": "ཚ",
        "tibt:dza": "ཛ",
        "tibt:dzha": "ཛྷ",
        "tibt:wa": "ཝ",
        "tibt:zha": "ཞ",
        "tibt:za": "ཟ",
        "tibt:AA": "འ",
        "tibt:ya": "ཡ",
        "tibt:ra": "ར",
        "tibt:la": "ལ",
        "tibt:sha": "ཤ",
        "tibt:ssa": "ཥ",
        "tibt:sa": "ས",
        "tibt:ha": "ཧ",
        "tibt:a": "ཨ",
        "tibt:kssa": "ཀྵ",
        "tibt:rafixed": "ཪ",
        "tibt:kka": "ཫ",
        "tibt:rra": "ཬ",
        "tibt:aavowelsign": "ཱ",
        "tibt:ivowelsign": "ི",
        "tibt:iivowelsign": "ཱི",
        "tibt:uvowelsign": "ུ",
        "tibt:uuvowelsign": "ཱུ",
        "tibt:rvocalicvowelsign": "ྲྀ",
        "tibt:rrvocalicvowelsign": "ཷ",
        "tibt:lvocalicvowelsign": "ླྀ",
        "tibt:llvocalicvowelsign": "ཹ",
        "tibt:evowelsign": "ེ",
        "tibt:eevowelsign": "ཻ",
        "tibt:ovowelsign": "ོ",
        "tibt:oovowelsign": "ཽ",
        "tibt:rjessungarosign": "ཾ",
        "tibt:rnambcadsign": "ཿ",
        "tibt:reversedivowelsign": "ྀ",
        "tibt:reversediivowelsign": "ཱྀ",
        "tibt:nyizlanaadasign": "ྂ",
        "tibt:snaldansign": "ྃ",
        "tibt:halantamark": "྄",
        "tibt:palutamark": "྅",
        "tibt:lcirtagssign": "྆",
        "tibt:yangrtagssign": "྇",
        "tibt:lcetsacansign": "ྈ",
        "tibt:mchucansign": "ྉ",
        "tibt:grucanrgyingssign": "ྊ",
        "tibt:grumedrgyingssign": "ྋ",
        "tibt:invertedmchucansign": "ྌ",
        "tibt:lcetsacansubjoinedsign": "ྍ",
        "tibt:mchucansubjoinedsign": "ྎ",
        "tibt:invertedmchucansubjoinedsign": "ྏ",
        "tibt:kasubjoined": "ྐ",
        "tibt:khasubjoined": "ྑ",
        "tibt:gasubjoined": "ྒ",
        "tibt:ghasubjoined": "ྒྷ",
        "tibt:ngasubjoined": "ྔ",
        "tibt:casubjoined": "ྕ",
        "tibt:chasubjoined": "ྖ",
        "tibt:jasubjoined": "ྗ",
        "tibt:nyasubjoined": "ྙ",
        "tibt:ttasubjoined": "ྚ",
        "tibt:tthasubjoined": "ྛ",
        "tibt:ddasubjoined": "ྜ",
        "tibt:ddhasubjoined": "ྜྷ",
        "tibt:nnasubjoined": "ྞ",
        "tibt:tasubjoined": "ྟ",
        "tibt:thasubjoined": "ྠ",
        "tibt:dasubjoined": "ྡ",
        "tibt:dhasubjoined": "ྡྷ",
        "tibt:nasubjoined": "ྣ",
        "tibt:pasubjoined": "ྤ",
        "tibt:phasubjoined": "ྥ",
        "tibt:basubjoined": "ྦ",
        "tibt:bhasubjoined": "ྦྷ",
        "tibt:masubjoined": "ྨ",
        "tibt:tsasubjoined": "ྩ",
        "tibt:tshasubjoined": "ྪ",
        "tibt:dzasubjoined": "ྫ",
        "tibt:dzhasubjoined": "ྫྷ",
        "tibt:wasubjoined": "ྭ",
        "tibt:zhasubjoined": "ྮ",
        "tibt:zasubjoined": "ྯ",
        "tibt:subjoinedAA": "ྰ",
        "tibt:yasubjoined": "ྱ",
        "tibt:rasubjoined": "ྲ",
        "tibt:lasubjoined": "ླ",
        "tibt:shasubjoined": "ྴ",
        "tibt:ssasubjoined": "ྵ",
        "tibt:sasubjoined": "ྶ",
        "tibt:hasubjoined": "ྷ",
        "tibt:asubjoined": "ྸ",
        "tibt:kssasubjoined": "ྐྵ",
        "tibt:wasubjoinedfixed": "ྺ",
        "tibt:yasubjoinedfixed": "ྻ",
        "tibt:rasubjoinedfixed": "ྼ",
        "tibt:kurukha": "྾",
        "tibt:kurukhabzhimigcan": "྿",
        "tibt:heavybeatcantillationsign": "࿀",
        "tibt:lightbeatcantillationsign": "࿁",
        "tibt:cangteucantillationsign": "࿂",
        "tibt:sbubchalcantillationsign": "࿃",
        "tibt:drilbusymbol": "࿄",
        "tibt:rdorjesymbol": "࿅",
        "tibt:padmagdansymbol": "࿆",
        "tibt:rdorjergyagramsymbol": "࿇",
        "tibt:phurpasymbol": "࿈",
        "tibt:norbusymbol": "࿉",
        "tibt:norbunyiskhyilsymbol": "࿊",
        "tibt:norbugsumkhyilsymbol": "࿋",
        "tibt:norbubzhikhyilsymbol": "࿌",
        "tibt:rdelnagrdeldkarsign": "࿎",
        "tibt:rdelnaggsumsign": "࿏",
        "tibt:bskashoggimgorgyanmark": "࿐",
        "tibt:mnyamyiggimgorgyanmark": "࿑",
        "tibt:nyistshegmark": "࿒",
        "tibt:brdarnyingyigmgomdunmainitialmark": "࿓",
        "tibt:brdarnyingyigmgosgabmaclosingmark": "࿔",
        "tibt:svastiright": "࿕",
        "tibt:svastileft": "࿖",
        "tibt:svastirightdot": "࿗",
        "tibt:svastileftdot": "࿘",
        "tibt:leadingmchanrtagsmark": "࿙",
        "tibt:trailingmchanrtagsmark": "࿚",
        "AnGeok": "Ⴀ",
        "BanGeok": "Ⴁ",
        "GanGeok": "Ⴂ",
        "DonGeok": "Ⴃ",
        "EnGeok": "Ⴄ",
        "VinGeok": "Ⴅ",
        "ZenGeok": "Ⴆ",
        "TanGeok": "Ⴇ",
        "InGeok": "Ⴈ",
        "KanGeok": "Ⴉ",
        "LasGeok": "Ⴊ",
        "ManGeok": "Ⴋ",
        "NarGeok": "Ⴌ",
        "OnGeok": "Ⴍ",
        "ParGeok": "Ⴎ",
        "ZharGeok": "Ⴏ",
        "RaeGeok": "Ⴐ",
        "SanGeok": "Ⴑ",
        "TarGeok": "Ⴒ",
        "UnGeok": "Ⴓ",
        "PharGeok": "Ⴔ",
        "KharGeok": "Ⴕ",
        "GhanGeok": "Ⴖ",
        "QarGeok": "Ⴗ",
        "ShinGeok": "Ⴘ",
        "ChinGeok": "Ⴙ",
        "CanGeok": "Ⴚ",
        "JilGeok": "Ⴛ",
        "CilGeok": "Ⴜ",
        "CharGeok": "Ⴝ",
        "XanGeok": "Ⴞ",
        "JhanGeok": "Ⴟ",
        "HaeGeok": "Ⴠ",
        "HeGeok": "Ⴡ",
        "HieGeok": "Ⴢ",
        "WeGeok": "Ⴣ",
        "HarGeok": "Ⴤ",
        "HoeGeok": "Ⴥ",
        "YnGeok": "Ⴧ",
        "AenGeok": "Ⴭ",
        "anGeor": "ა",
        "banGeor": "ბ",
        "ganGeor": "გ",
        "donGeor": "დ",
        "enGeor": "ე",
        "vinGeor": "ვ",
        "zenGeor": "ზ",
        "tanGeor": "თ",
        "inGeor": "ი",
        "kanGeor": "კ",
        "lasGeor": "ლ",
        "manGeor": "მ",
        "narGeor": "ნ",
        "onGeor": "ო",
        "parGeor": "პ",
        "zharGeor": "ჟ",
        "raeGeor": "რ",
        "sanGeor": "ს",
        "tarGeor": "ტ",
        "unGeor": "უ",
        "pharGeor": "ფ",
        "kharGeor": "ქ",
        "ghanGeor": "ღ",
        "qarGeor": "ყ",
        "shinGeor": "შ",
        "chinGeor": "ჩ",
        "canGeor": "ც",
        "jilGeor": "ძ",
        "cilGeor": "წ",
        "charGeor": "ჭ",
        "xanGeor": "ხ",
        "jhanGeor": "ჯ",
        "haeGeor": "ჰ",
        "heGeor": "ჱ",
        "hieGeor": "ჲ",
        "weGeor": "ჳ",
        "harGeor": "ჴ",
        "hoeGeor": "ჵ",
        "fiGeor": "ჶ",
        "ynGeor": "ჷ",
        "elifiGeor": "ჸ",
        "turnedganGeor": "ჹ",
        "ainGeor": "ჺ",
        "geor:paragraphseparator": "჻",
        "narmodGeor": "ჼ",
        "aenGeor": "ჽ",
        "hardsignGeor": "ჾ",
        "labialsignGeor": "ჿ",
        "ko:kiyeokchoseong": "ᄀ",
        "ko:ssangkiyeokchoseong": "ᄁ",
        "ko:nieunchoseong": "ᄂ",
        "ko:tikeutchoseong": "ᄃ",
        "ko:ssangtikeutchoseong": "ᄄ",
        "ko:rieulchoseong": "ᄅ",
        "ko:mieumchoseong": "ᄆ",
        "ko:pieupchoseong": "ᄇ",
        "ko:ssangpieupchoseong": "ᄈ",
        "ko:sioschoseong": "ᄉ",
        "ko:ssangsioschoseong": "ᄊ",
        "ko:ieungchoseong": "ᄋ",
        "ko:cieucchoseong": "ᄌ",
        "ko:ssangcieucchoseong": "ᄍ",
        "ko:chieuchchoseong": "ᄎ",
        "ko:khieukhchoseong": "ᄏ",
        "ko:thieuthchoseong": "ᄐ",
        "ko:phieuphchoseong": "ᄑ",
        "ko:hieuhchoseong": "ᄒ",
        "ko:nieunkiyeokchoseong": "ᄓ",
        "ko:ssangnieunchoseong": "ᄔ",
        "ko:nieuntikeutchoseong": "ᄕ",
        "ko:nieunpieupchoseong": "ᄖ",
        "ko:tikeutkiyeokchoseong": "ᄗ",
        "ko:rieulnieunchoseong": "ᄘ",
        "ko:ssangrieulchoseong": "ᄙ",
        "ko:rieulhieuhchoseong": "ᄚ",
        "ko:kapyeounrieulchoseong": "ᄛ",
        "ko:mieumpieupchoseong": "ᄜ",
        "ko:kapyeounmieumchoseong": "ᄝ",
        "ko:pieupkiyeokchoseong": "ᄞ",
        "ko:pieupnieunchoseong": "ᄟ",
        "ko:pieuptikeutchoseong": "ᄠ",
        "ko:pieupsioschoseong": "ᄡ",
        "ko:pieupsioskiyeokchoseong": "ᄢ",
        "ko:pieupsiostikeutchoseong": "ᄣ",
        "ko:pieupsiospieupchoseong": "ᄤ",
        "ko:pieupssangsioschoseong": "ᄥ",
        "ko:pieupsioscieucchoseong": "ᄦ",
        "ko:pieupcieucchoseong": "ᄧ",
        "ko:pieupchieuchchoseong": "ᄨ",
        "ko:pieupthieuthchoseong": "ᄩ",
        "ko:pieupphieuphchoseong": "ᄪ",
        "ko:kapyeounpieupchoseong": "ᄫ",
        "ko:kapyeounssangpieupchoseong": "ᄬ",
        "ko:sioskiyeokchoseong": "ᄭ",
        "ko:siosnieunchoseong": "ᄮ",
        "ko:siostikeutchoseong": "ᄯ",
        "ko:siosrieulchoseong": "ᄰ",
        "ko:siosmieumchoseong": "ᄱ",
        "ko:siospieupchoseong": "ᄲ",
        "ko:siospieupkiyeokchoseong": "ᄳ",
        "ko:siosssangsioschoseong": "ᄴ",
        "ko:siosieungchoseong": "ᄵ",
        "ko:sioscieucchoseong": "ᄶ",
        "ko:sioschieuchchoseong": "ᄷ",
        "ko:sioskhieukhchoseong": "ᄸ",
        "ko:siosthieuthchoseong": "ᄹ",
        "ko:siosphieuphchoseong": "ᄺ",
        "ko:sioshieuhchoseong": "ᄻ",
        "ko:chitueumsioschoseong": "ᄼ",
        "ko:chitueumssangsioschoseong": "ᄽ",
        "ko:ceongchieumsioschoseong": "ᄾ",
        "ko:ceongchieumssangsioschoseong": "ᄿ",
        "ko:pansioschoseong": "ᅀ",
        "ko:ieungkiyeokchoseong": "ᅁ",
        "ko:ieungtikeutchoseong": "ᅂ",
        "ko:ieungmieumchoseong": "ᅃ",
        "ko:ieungpieupchoseong": "ᅄ",
        "ko:ieungsioschoseong": "ᅅ",
        "ko:ieungpansioschoseong": "ᅆ",
        "ko:ssangieungchoseong": "ᅇ",
        "ko:ieungcieucchoseong": "ᅈ",
        "ko:ieungchieuchchoseong": "ᅉ",
        "ko:ieungthieuthchoseong": "ᅊ",
        "ko:ieungphieuphchoseong": "ᅋ",
        "ko:yesieungchoseong": "ᅌ",
        "ko:cieucieungchoseong": "ᅍ",
        "ko:chitueumcieucchoseong": "ᅎ",
        "ko:chitueumssangcieucchoseong": "ᅏ",
        "ko:ceongchieumcieucchoseong": "ᅐ",
        "ko:ceongchieumssangcieucchoseong": "ᅑ",
        "ko:chieuchkhieukhchoseong": "ᅒ",
        "ko:chieuchhieuhchoseong": "ᅓ",
        "ko:chitueumchieuchchoseong": "ᅔ",
        "ko:ceongchieumchieuchchoseong": "ᅕ",
        "ko:phieuphpieupchoseong": "ᅖ",
        "ko:kapyeounphieuphchoseong": "ᅗ",
        "ko:ssanghieuhchoseong": "ᅘ",
        "ko:yeorinhieuhchoseong": "ᅙ",
        "ko:kiyeoktikeutchoseong": "ᅚ",
        "ko:nieunsioschoseong": "ᅛ",
        "ko:nieuncieucchoseong": "ᅜ",
        "ko:nieunhieuhchoseong": "ᅝ",
        "ko:tikeutrieulchoseong": "ᅞ",
        "ko:fillerchoseong": "ᅟ",
        "ko:fillerjungseong": "ᅠ",
        "ko:ajungseong": "ᅡ",
        "ko:aejungseong": "ᅢ",
        "ko:yajungseong": "ᅣ",
        "ko:yaejungseong": "ᅤ",
        "ko:eojungseong": "ᅥ",
        "ko:ejungseong": "ᅦ",
        "ko:yeojungseong": "ᅧ",
        "ko:yejungseong": "ᅨ",
        "ko:ojungseong": "ᅩ",
        "ko:wajungseong": "ᅪ",
        "ko:waejungseong": "ᅫ",
        "ko:oejungseong": "ᅬ",
        "ko:yojungseong": "ᅭ",
        "ko:ujungseong": "ᅮ",
        "ko:weojungseong": "ᅯ",
        "ko:wejungseong": "ᅰ",
        "ko:wijungseong": "ᅱ",
        "ko:yujungseong": "ᅲ",
        "ko:eujungseong": "ᅳ",
        "ko:yijungseong": "ᅴ",
        "ko:ijungseong": "ᅵ",
        "ko:aojungseong": "ᅶ",
        "ko:aujungseong": "ᅷ",
        "ko:yaojungseong": "ᅸ",
        "ko:yayojungseong": "ᅹ",
        "ko:eoojungseong": "ᅺ",
        "ko:eoujungseong": "ᅻ",
        "ko:eo_eujungseong": "ᅼ",
        "ko:yeoojungseong": "ᅽ",
        "ko:yeoujungseong": "ᅾ",
        "ko:o_eojungseong": "ᅿ",
        "ko:o_ejungseong": "ᆀ",
        "ko:oyejungseong": "ᆁ",
        "ko:oojungseong": "ᆂ",
        "ko:oujungseong": "ᆃ",
        "ko:yoyajungseong": "ᆄ",
        "ko:yoyaejungseong": "ᆅ",
        "ko:yoyeojungseong": "ᆆ",
        "ko:yoojungseong": "ᆇ",
        "ko:yoijungseong": "ᆈ",
        "ko:uajungseong": "ᆉ",
        "ko:uaejungseong": "ᆊ",
        "ko:ueo_eujungseong": "ᆋ",
        "ko:uyejungseong": "ᆌ",
        "ko:uujungseong": "ᆍ",
        "ko:yuajungseong": "ᆎ",
        "ko:yueojungseong": "ᆏ",
        "ko:yuejungseong": "ᆐ",
        "ko:yuyeojungseong": "ᆑ",
        "ko:yuyejungseong": "ᆒ",
        "ko:yuujungseong": "ᆓ",
        "ko:yuijungseong": "ᆔ",
        "ko:euujungseong": "ᆕ",
        "ko:eueujungseong": "ᆖ",
        "ko:yiujungseong": "ᆗ",
        "ko:iajungseong": "ᆘ",
        "ko:iyajungseong": "ᆙ",
        "ko:iojungseong": "ᆚ",
        "ko:iujungseong": "ᆛ",
        "ko:ieujungseong": "ᆜ",
        "ko:iaraeajungseong": "ᆝ",
        "ko:araeajungseong": "ᆞ",
        "ko:araeaeojungseong": "ᆟ",
        "ko:araeaujungseong": "ᆠ",
        "ko:araeaijungseong": "ᆡ",
        "ko:ssangaraeajungseong": "ᆢ",
        "ko:aeujungseong": "ᆣ",
        "ko:yaujungseong": "ᆤ",
        "ko:yeoyajungseong": "ᆥ",
        "ko:oyajungseong": "ᆦ",
        "ko:oyaejungseong": "ᆧ",
        "ko:kiyeokjongseong": "ᆨ",
        "ko:ssangkiyeokjongseong": "ᆩ",
        "ko:kiyeoksiosjongseong": "ᆪ",
        "ko:nieunjongseong": "ᆫ",
        "ko:nieuncieucjongseong": "ᆬ",
        "ko:nieunhieuhjongseong": "ᆭ",
        "ko:tikeutjongseong": "ᆮ",
        "ko:rieuljongseong": "ᆯ",
        "ko:rieulkiyeokjongseong": "ᆰ",
        "ko:rieulmieumjongseong": "ᆱ",
        "ko:rieulpieupjongseong": "ᆲ",
        "ko:rieulsiosjongseong": "ᆳ",
        "ko:rieulthieuthjongseong": "ᆴ",
        "ko:rieulphieuphjongseong": "ᆵ",
        "ko:rieulhieuhjongseong": "ᆶ",
        "ko:mieumjongseong": "ᆷ",
        "ko:pieupjongseong": "ᆸ",
        "ko:pieupsiosjongseong": "ᆹ",
        "ko:siosjongseong": "ᆺ",
        "ko:ssangsiosjongseong": "ᆻ",
        "ko:ieungjongseong": "ᆼ",
        "ko:cieucjongseong": "ᆽ",
        "ko:chieuchjongseong": "ᆾ",
        "ko:khieukhjongseong": "ᆿ",
        "ko:thieuthjongseong": "ᇀ",
        "ko:phieuphjongseong": "ᇁ",
        "ko:hieuhjongseong": "ᇂ",
        "ko:kiyeokrieuljongseong": "ᇃ",
        "ko:kiyeoksioskiyeokjongseong": "ᇄ",
        "ko:nieunkiyeokjongseong": "ᇅ",
        "ko:nieuntikeutjongseong": "ᇆ",
        "ko:nieunsiosjongseong": "ᇇ",
        "ko:nieunpansiosjongseong": "ᇈ",
        "ko:nieunthieuthjongseong": "ᇉ",
        "ko:tikeutkiyeokjongseong": "ᇊ",
        "ko:tikeutrieuljongseong": "ᇋ",
        "ko:rieulkiyeoksiosjongseong": "ᇌ",
        "ko:rieulnieunjongseong": "ᇍ",
        "ko:rieultikeutjongseong": "ᇎ",
        "ko:rieultikeuthieuhjongseong": "ᇏ",
        "ko:ssangrieuljongseong": "ᇐ",
        "ko:rieulmieumkiyeokjongseong": "ᇑ",
        "ko:rieulmieumsiosjongseong": "ᇒ",
        "ko:rieulpieupsiosjongseong": "ᇓ",
        "ko:rieulpieuphieuhjongseong": "ᇔ",
        "ko:rieulkapyeounpieupjongseong": "ᇕ",
        "ko:rieulssangsiosjongseong": "ᇖ",
        "ko:rieulpansiosjongseong": "ᇗ",
        "ko:rieulkhieukhjongseong": "ᇘ",
        "ko:rieulyeorinhieuhjongseong": "ᇙ",
        "ko:mieumkiyeokjongseong": "ᇚ",
        "ko:mieumrieuljongseong": "ᇛ",
        "ko:mieumpieupjongseong": "ᇜ",
        "ko:mieumsiosjongseong": "ᇝ",
        "ko:mieumssangsiosjongseong": "ᇞ",
        "ko:mieumpansiosjongseong": "ᇟ",
        "ko:mieumchieuchjongseong": "ᇠ",
        "ko:mieumhieuhjongseong": "ᇡ",
        "ko:kapyeounmieumjongseong": "ᇢ",
        "ko:pieuprieuljongseong": "ᇣ",
        "ko:pieupphieuphjongseong": "ᇤ",
        "ko:pieuphieuhjongseong": "ᇥ",
        "ko:kapyeounpieupjongseong": "ᇦ",
        "ko:sioskiyeokjongseong": "ᇧ",
        "ko:siostikeutjongseong": "ᇨ",
        "ko:siosrieuljongseong": "ᇩ",
        "ko:siospieupjongseong": "ᇪ",
        "ko:pansiosjongseong": "ᇫ",
        "ko:ieungkiyeokjongseong": "ᇬ",
        "ko:ieungssangkiyeokjongseong": "ᇭ",
        "ko:ssangieungjongseong": "ᇮ",
        "ko:ieungkhieukhjongseong": "ᇯ",
        "ko:yesieungjongseong": "ᇰ",
        "ko:yesieungsiosjongseong": "ᇱ",
        "ko:yesieungpansiosjongseong": "ᇲ",
        "ko:phieuphpieupjongseong": "ᇳ",
        "ko:kapyeounphieuphjongseong": "ᇴ",
        "ko:hieuhnieunjongseong": "ᇵ",
        "ko:hieuhrieuljongseong": "ᇶ",
        "ko:hieuhmieumjongseong": "ᇷ",
        "ko:hieuhpieupjongseong": "ᇸ",
        "ko:yeorinhieuhjongseong": "ᇹ",
        "ko:kiyeoknieunjongseong": "ᇺ",
        "ko:kiyeokpieupjongseong": "ᇻ",
        "ko:kiyeokchieuchjongseong": "ᇼ",
        "ko:kiyeokkhieukhjongseong": "ᇽ",
        "ko:kiyeokhieuhjongseong": "ᇾ",
        "ko:ssangnieunjongseong": "ᇿ",
        "ethi:ha": "ሀ",
        "ethi:hu": "ሁ",
        "ethi:hi": "ሂ",
        "ethi:haa": "ሃ",
        "ethi:hee": "ሄ",
        "ethi:he": "ህ",
        "ethi:ho": "ሆ",
        "ethi:hoa": "ሇ",
        "ethi:la": "ለ",
        "ethi:lu": "ሉ",
        "ethi:li": "ሊ",
        "ethi:laa": "ላ",
        "ethi:lee": "ሌ",
        "ethi:le": "ል",
        "ethi:lo": "ሎ",
        "ethi:lwa": "ሏ",
        "ethi:hha": "ሐ",
        "ethi:hhu": "ሑ",
        "ethi:hhi": "ሒ",
        "ethi:hhaa": "ሓ",
        "ethi:hhee": "ሔ",
        "ethi:hhe": "ሕ",
        "ethi:hho": "ሖ",
        "ethi:hhwa": "ሗ",
        "ethi:ma": "መ",
        "ethi:mu": "ሙ",
        "ethi:mi": "ሚ",
        "ethi:maa": "ማ",
        "ethi:mee": "ሜ",
        "ethi:me": "ም",
        "ethi:mo": "ሞ",
        "ethi:mwa": "ሟ",
        "ethi:sza": "ሠ",
        "ethi:szu": "ሡ",
        "ethi:szi": "ሢ",
        "ethi:szaa": "ሣ",
        "ethi:szee": "ሤ",
        "ethi:sze": "ሥ",
        "ethi:szo": "ሦ",
        "ethi:szwa": "ሧ",
        "ethi:ra": "ረ",
        "ethi:ru": "ሩ",
        "ethi:ri": "ሪ",
        "ethi:raa": "ራ",
        "ethi:ree": "ሬ",
        "ethi:re": "ር",
        "ethi:ro": "ሮ",
        "ethi:rwa": "ሯ",
        "ethi:sa": "ሰ",
        "ethi:su": "ሱ",
        "ethi:si": "ሲ",
        "ethi:saa": "ሳ",
        "ethi:see": "ሴ",
        "ethi:se": "ስ",
        "ethi:so": "ሶ",
        "ethi:swa": "ሷ",
        "ethi:sha": "ሸ",
        "ethi:shu": "ሹ",
        "ethi:shi": "ሺ",
        "ethi:shaa": "ሻ",
        "ethi:shee": "ሼ",
        "ethi:she": "ሽ",
        "ethi:sho": "ሾ",
        "ethi:shwa": "ሿ",
        "ethi:qa": "ቀ",
        "ethi:qu": "ቁ",
        "ethi:qi": "ቂ",
        "ethi:qaa": "ቃ",
        "ethi:qee": "ቄ",
        "ethi:qe": "ቅ",
        "ethi:qo": "ቆ",
        "ethi:qoa": "ቇ",
        "ethi:qwa": "ቈ",
        "ethi:qwi": "ቊ",
        "ethi:qwaa": "ቋ",
        "ethi:qwee": "ቌ",
        "ethi:qwe": "ቍ",
        "ethi:qha": "ቐ",
        "ethi:qhu": "ቑ",
        "ethi:qhi": "ቒ",
        "ethi:qhaa": "ቓ",
        "ethi:qhee": "ቔ",
        "ethi:qhe": "ቕ",
        "ethi:qho": "ቖ",
        "ethi:qhwa": "ቘ",
        "ethi:qhwi": "ቚ",
        "ethi:qhwaa": "ቛ",
        "ethi:qhwee": "ቜ",
        "ethi:qhwe": "ቝ",
        "ethi:ba": "በ",
        "ethi:bu": "ቡ",
        "ethi:bi": "ቢ",
        "ethi:baa": "ባ",
        "ethi:bee": "ቤ",
        "ethi:be": "ብ",
        "ethi:bo": "ቦ",
        "ethi:bwa": "ቧ",
        "ethi:va": "ቨ",
        "ethi:vu": "ቩ",
        "ethi:vi": "ቪ",
        "ethi:vaa": "ቫ",
        "ethi:vee": "ቬ",
        "ethi:ve": "ቭ",
        "ethi:vo": "ቮ",
        "ethi:vwa": "ቯ",
        "ethi:ta": "ተ",
        "ethi:tu": "ቱ",
        "ethi:ti": "ቲ",
        "ethi:taa": "ታ",
        "ethi:tee": "ቴ",
        "ethi:te": "ት",
        "ethi:to": "ቶ",
        "ethi:twa": "ቷ",
        "ethi:ca": "ቸ",
        "ethi:cu": "ቹ",
        "ethi:ci": "ቺ",
        "ethi:caa": "ቻ",
        "ethi:cee": "ቼ",
        "ethi:ce": "ች",
        "ethi:co": "ቾ",
        "ethi:cwa": "ቿ",
        "ethi:xa": "ኀ",
        "ethi:xu": "ኁ",
        "ethi:xi": "ኂ",
        "ethi:xaa": "ኃ",
        "ethi:xee": "ኄ",
        "ethi:xe": "ኅ",
        "ethi:xo": "ኆ",
        "ethi:xoa": "ኇ",
        "ethi:xwa": "ኈ",
        "ethi:xwi": "ኊ",
        "ethi:xwaa": "ኋ",
        "ethi:xwee": "ኌ",
        "ethi:xwe": "ኍ",
        "ethi:na": "ነ",
        "ethi:nu": "ኑ",
        "ethi:ni": "ኒ",
        "ethi:naa": "ና",
        "ethi:nee": "ኔ",
        "ethi:ne": "ን",
        "ethi:no": "ኖ",
        "ethi:nwa": "ኗ",
        "ethi:nya": "ኘ",
        "ethi:nyu": "ኙ",
        "ethi:nyi": "ኚ",
        "ethi:nyaa": "ኛ",
        "ethi:nyee": "ኜ",
        "ethi:nye": "ኝ",
        "ethi:nyo": "ኞ",
        "ethi:nywa": "ኟ",
        "ethi:aglottal": "አ",
        "ethi:uglottal": "ኡ",
        "ethi:iglottal": "ኢ",
        "ethi:aaglottal": "ኣ",
        "ethi:eeglottal": "ኤ",
        "ethi:eglottal": "እ",
        "ethi:oglottal": "ኦ",
        "ethi:waglottal": "ኧ",
        "ethi:ka": "ከ",
        "ethi:ku": "ኩ",
        "ethi:ki": "ኪ",
        "ethi:kaa": "ካ",
        "ethi:kee": "ኬ",
        "ethi:ke": "ክ",
        "ethi:ko": "ኮ",
        "ethi:koa": "ኯ",
        "ethi:kwa": "ኰ",
        "ethi:kwi": "ኲ",
        "ethi:kwaa": "ኳ",
        "ethi:kwee": "ኴ",
        "ethi:kwe": "ኵ",
        "ethi:kxa": "ኸ",
        "ethi:kxu": "ኹ",
        "ethi:kxi": "ኺ",
        "ethi:kxaa": "ኻ",
        "ethi:kxee": "ኼ",
        "ethi:kxe": "ኽ",
        "ethi:kxo": "ኾ",
        "ethi:kxwa": "ዀ",
        "ethi:kxwi": "ዂ",
        "ethi:kxwaa": "ዃ",
        "ethi:kxwee": "ዄ",
        "ethi:kxwe": "ዅ",
        "ethi:wa": "ወ",
        "ethi:wu": "ዉ",
        "ethi:wi": "ዊ",
        "ethi:waa": "ዋ",
        "ethi:wee": "ዌ",
        "ethi:we": "ው",
        "ethi:wo": "ዎ",
        "ethi:woa": "ዏ",
        "ethi:pharyngeala": "ዐ",
        "ethi:pharyngealu": "ዑ",
        "ethi:pharyngeali": "ዒ",
        "ethi:pharyngealaa": "ዓ",
        "ethi:pharyngealee": "ዔ",
        "ethi:pharyngeale": "ዕ",
        "ethi:pharyngealo": "ዖ",
        "ethi:za": "ዘ",
        "ethi:zu": "ዙ",
        "ethi:zi": "ዚ",
        "ethi:zaa": "ዛ",
        "ethi:zee": "ዜ",
        "ethi:ze": "ዝ",
        "ethi:zo": "ዞ",
        "ethi:zwa": "ዟ",
        "ethi:zha": "ዠ",
        "ethi:zhu": "ዡ",
        "ethi:zhi": "ዢ",
        "ethi:zhaa": "ዣ",
        "ethi:zhee": "ዤ",
        "ethi:zhe": "ዥ",
        "ethi:zho": "ዦ",
        "ethi:zhwa": "ዧ",
        "ethi:ya": "የ",
        "ethi:yu": "ዩ",
        "ethi:yi": "ዪ",
        "ethi:yaa": "ያ",
        "ethi:yee": "ዬ",
        "ethi:ye": "ይ",
        "ethi:yo": "ዮ",
        "ethi:yoa": "ዯ",
        "ethi:da": "ደ",
        "ethi:du": "ዱ",
        "ethi:di": "ዲ",
        "ethi:daa": "ዳ",
        "ethi:dee": "ዴ",
        "ethi:de": "ድ",
        "ethi:do": "ዶ",
        "ethi:dwa": "ዷ",
        "ethi:dda": "ዸ",
        "ethi:ddu": "ዹ",
        "ethi:ddi": "ዺ",
        "ethi:ddaa": "ዻ",
        "ethi:ddee": "ዼ",
        "ethi:dde": "ዽ",
        "ethi:ddo": "ዾ",
        "ethi:ddwa": "ዿ",
        "ethi:ja": "ጀ",
        "ethi:ju": "ጁ",
        "ethi:ji": "ጂ",
        "ethi:jaa": "ጃ",
        "ethi:jee": "ጄ",
        "ethi:je": "ጅ",
        "ethi:jo": "ጆ",
        "ethi:jwa": "ጇ",
        "ethi:ga": "ገ",
        "ethi:gu": "ጉ",
        "ethi:gi": "ጊ",
        "ethi:gaa": "ጋ",
        "ethi:gee": "ጌ",
        "ethi:ge": "ግ",
        "ethi:go": "ጎ",
        "ethi:goa": "ጏ",
        "ethi:gwa": "ጐ",
        "ethi:gwi": "ጒ",
        "ethi:gwaa": "ጓ",
        "ethi:gwee": "ጔ",
        "ethi:gwe": "ጕ",
        "ethi:gga": "ጘ",
        "ethi:ggu": "ጙ",
        "ethi:ggi": "ጚ",
        "ethi:ggaa": "ጛ",
        "ethi:ggee": "ጜ",
        "ethi:gge": "ጝ",
        "ethi:ggo": "ጞ",
        "ethi:ggwaa": "ጟ",
        "ethi:tha": "ጠ",
        "ethi:thu": "ጡ",
        "ethi:thi": "ጢ",
        "ethi:thaa": "ጣ",
        "ethi:thee": "ጤ",
        "ethi:the": "ጥ",
        "ethi:tho": "ጦ",
        "ethi:thwa": "ጧ",
        "ethi:cha": "ጨ",
        "ethi:chu": "ጩ",
        "ethi:chi": "ጪ",
        "ethi:chaa": "ጫ",
        "ethi:chee": "ጬ",
        "ethi:che": "ጭ",
        "ethi:cho": "ጮ",
        "ethi:chwa": "ጯ",
        "ethi:pha": "ጰ",
        "ethi:phu": "ጱ",
        "ethi:phi": "ጲ",
        "ethi:phaa": "ጳ",
        "ethi:phee": "ጴ",
        "ethi:phe": "ጵ",
        "ethi:pho": "ጶ",
        "ethi:phwa": "ጷ",
        "ethi:tsa": "ጸ",
        "ethi:tsu": "ጹ",
        "ethi:tsi": "ጺ",
        "ethi:tsaa": "ጻ",
        "ethi:tsee": "ጼ",
        "ethi:tse": "ጽ",
        "ethi:tso": "ጾ",
        "ethi:tswa": "ጿ",
        "ethi:tza": "ፀ",
        "ethi:tzu": "ፁ",
        "ethi:tzi": "ፂ",
        "ethi:tzaa": "ፃ",
        "ethi:tzee": "ፄ",
        "ethi:tze": "ፅ",
        "ethi:tzo": "ፆ",
        "ethi:tzoa": "ፇ",
        "ethi:fa": "ፈ",
        "ethi:fu": "ፉ",
        "ethi:fi": "ፊ",
        "ethi:faa": "ፋ",
        "ethi:fee": "ፌ",
        "ethi:fe": "ፍ",
        "ethi:fo": "ፎ",
        "ethi:fwa": "ፏ",
        "ethi:pa": "ፐ",
        "ethi:pu": "ፑ",
        "ethi:pi": "ፒ",
        "ethi:paa": "ፓ",
        "ethi:pee": "ፔ",
        "ethi:pe": "ፕ",
        "ethi:po": "ፖ",
        "ethi:pwa": "ፗ",
        "ethi:rya": "ፘ",
        "ethi:mya": "ፙ",
        "ethi:fya": "ፚ",
        "ethi:geminationandvowellengthmarkcmb": "፝",
        "ethi:vowellengthmarkcmb": "፞",
        "ethi:geminationmarkcmb": "፟",
        "ethi:sectionmark": "፠",
        "ethi:wordspace": "፡",
        "ethi:fullstop": "።",
        "ethi:comma": "፣",
        "ethi:semicolon": "፤",
        "ethi:colon": "፥",
        "ethi:prefacecolon": "፦",
        "ethi:questionmark": "፧",
        "ethi:paragraphseparator": "፨",
        "ethi:one": "፩",
        "ethi:two": "፪",
        "ethi:three": "፫",
        "ethi:four": "፬",
        "ethi:five": "፭",
        "ethi:six": "፮",
        "ethi:seven": "፯",
        "ethi:eight": "፰",
        "ethi:nine": "፱",
        "ethi:ten": "፲",
        "ethi:twenty": "፳",
        "ethi:thirty": "፴",
        "ethi:forty": "፵",
        "ethi:fifty": "፶",
        "ethi:sixty": "፷",
        "ethi:seventy": "፸",
        "ethi:eighty": "፹",
        "ethi:ninety": "፺",
        "ethi:hundred": "፻",
        "ethi:tenthousand": "፼",
        "cher:a": "Ꭰ",
        "cher:e": "Ꭱ",
        "cher:i": "Ꭲ",
        "cher:o": "Ꭳ",
        "cher:u": "Ꭴ",
        "cher:v": "Ꭵ",
        "cher:ga": "Ꭶ",
        "cher:ka": "Ꭷ",
        "cher:ge": "Ꭸ",
        "cher:gi": "Ꭹ",
        "cher:go": "Ꭺ",
        "cher:gu": "Ꭻ",
        "cher:gv": "Ꭼ",
        "cher:ha": "Ꭽ",
        "cher:he": "Ꭾ",
        "cher:hi": "Ꭿ",
        "cher:ho": "Ꮀ",
        "cher:hu": "Ꮁ",
        "cher:hv": "Ꮂ",
        "cher:la": "Ꮃ",
        "cher:le": "Ꮄ",
        "cher:li": "Ꮅ",
        "cher:lo": "Ꮆ",
        "cher:lu": "Ꮇ",
        "cher:lv": "Ꮈ",
        "cher:ma": "Ꮉ",
        "cher:me": "Ꮊ",
        "cher:mi": "Ꮋ",
        "cher:mo": "Ꮌ",
        "cher:mu": "Ꮍ",
        "cher:na": "Ꮎ",
        "cher:hna": "Ꮏ",
        "cher:nah": "Ꮐ",
        "cher:ne": "Ꮑ",
        "cher:ni": "Ꮒ",
        "cher:no": "Ꮓ",
        "cher:nu": "Ꮔ",
        "cher:nv": "Ꮕ",
        "cher:qua": "Ꮖ",
        "cher:que": "Ꮗ",
        "cher:qui": "Ꮘ",
        "cher:quo": "Ꮙ",
        "cher:quu": "Ꮚ",
        "cher:quv": "Ꮛ",
        "cher:sa": "Ꮜ",
        "cher:s": "Ꮝ",
        "cher:se": "Ꮞ",
        "cher:si": "Ꮟ",
        "cher:so": "Ꮠ",
        "cher:su": "Ꮡ",
        "cher:sv": "Ꮢ",
        "cher:da": "Ꮣ",
        "cher:ta": "Ꮤ",
        "cher:de": "Ꮥ",
        "cher:te": "Ꮦ",
        "cher:di": "Ꮧ",
        "cher:ti": "Ꮨ",
        "cher:do": "Ꮩ",
        "cher:du": "Ꮪ",
        "cher:dv": "Ꮫ",
        "cher:dla": "Ꮬ",
        "cher:tla": "Ꮭ",
        "cher:tle": "Ꮮ",
        "cher:tli": "Ꮯ",
        "cher:tlo": "Ꮰ",
        "cher:tlu": "Ꮱ",
        "cher:tlv": "Ꮲ",
        "cher:tsa": "Ꮳ",
        "cher:tse": "Ꮴ",
        "cher:tsi": "Ꮵ",
        "cher:tso": "Ꮶ",
        "cher:tsu": "Ꮷ",
        "cher:tsv": "Ꮸ",
        "cher:wa": "Ꮹ",
        "cher:we": "Ꮺ",
        "cher:wi": "Ꮻ",
        "cher:wo": "Ꮼ",
        "cher:wu": "Ꮽ",
        "cher:wv": "Ꮾ",
        "cher:ya": "Ꮿ",
        "cher:ye": "Ᏸ",
        "cher:yi": "Ᏹ",
        "cher:yo": "Ᏺ",
        "cher:yu": "Ᏻ",
        "cher:yv": "Ᏼ",
        "cher:mv": "Ᏽ",
        "cher:yesmall": "ᏸ",
        "cher:yismall": "ᏹ",
        "cher:yosmall": "ᏺ",
        "cher:yusmall": "ᏻ",
        "cher:yvsmall": "ᏼ",
        "cher:mvsmall": "ᏽ",
        "runr:fehu_feoh_fe_f": "ᚠ",
        "runr:v": "ᚡ",
        "runr:uruz_ur_u": "ᚢ",
        "runr:yr": "ᚣ",
        "runr:y": "ᚤ",
        "runr:w": "ᚥ",
        "runr:thurisaz_thurs_thorn": "ᚦ",
        "runr:eth": "ᚧ",
        "runr:ansuz_a": "ᚨ",
        "runr:os_o": "ᚩ",
        "runr:ac_a": "ᚪ",
        "runr:aesc": "ᚫ",
        "runr:long_branch_oss_o": "ᚬ",
        "runr:short_twig_oss_o": "ᚭ",
        "runr:o": "ᚮ",
        "runr:oe": "ᚯ",
        "runr:on": "ᚰ",
        "runr:raido_rad_reid_r": "ᚱ",
        "runr:kauna": "ᚲ",
        "runr:cen": "ᚳ",
        "runr:kaun_k": "ᚴ",
        "runr:g": "ᚵ",
        "runr:eng": "ᚶ",
        "runr:gebo_gyfu_g": "ᚷ",
        "runr:gar": "ᚸ",
        "runr:wunjo_wynn_w": "ᚹ",
        "runr:haglaz_h": "ᚺ",
        "runr:haegl_h": "ᚻ",
        "runr:long_branch_hagall_h": "ᚼ",
        "runr:short_twig_hagall_h": "ᚽ",
        "runr:naudiz_nyd_naud_n": "ᚾ",
        "runr:short_twig_naud_n": "ᚿ",
        "runr:dotted_n": "ᛀ",
        "runr:isaz_is_iss_i": "ᛁ",
        "runr:e": "ᛂ",
        "runr:jeran_j": "ᛃ",
        "runr:ger": "ᛄ",
        "runr:long_branch_ar_ae": "ᛅ",
        "runr:short_twig_ar_a": "ᛆ",
        "runr:iwaz_eoh": "ᛇ",
        "runr:pertho_peorth_p": "ᛈ",
        "runr:algiz_eolhx": "ᛉ",
        "runr:sowilo_s": "ᛊ",
        "runr:sigel_long_branch_sol_s": "ᛋ",
        "runr:short_twig_sol_s": "ᛌ",
        "runr:c": "ᛍ",
        "runr:z": "ᛎ",
        "runr:tiwaz_tir_tyr_t": "ᛏ",
        "runr:short_twig_tyr_t": "ᛐ",
        "runr:d": "ᛑ",
        "runr:berkanan_beorc_bjarkan_b": "ᛒ",
        "runr:short_twig_bjarkan_b": "ᛓ",
        "runr:dotted_p": "ᛔ",
        "runr:open_p": "ᛕ",
        "runr:ehwaz_eh_e": "ᛖ",
        "runr:mannaz_man_m": "ᛗ",
        "runr:long_branch_madr_m": "ᛘ",
        "runr:short_twig_madr_m": "ᛙ",
        "runr:laukaz_lagu_logr_l": "ᛚ",
        "runr:dotted_l": "ᛛ",
        "runr:ingwaz": "ᛜ",
        "runr:ing": "ᛝ",
        "runr:dagaz_daeg_d": "ᛞ",
        "runr:othalan_ethel_o": "ᛟ",
        "runr:ear": "ᛠ",
        "runr:ior": "ᛡ",
        "runr:cweorth": "ᛢ",
        "runr:calc": "ᛣ",
        "runr:cealc": "ᛤ",
        "runr:stan": "ᛥ",
        "runr:long_branch_yr": "ᛦ",
        "runr:short_twig_yr": "ᛧ",
        "runr:icelandic_yr": "ᛨ",
        "runr:q": "ᛩ",
        "runr:x": "ᛪ",
        "runr:single_punctuation": "᛫",
        "runr:multiple_punctuation": "᛬",
        "runr:cross_punctuation": "᛭",
        "runr:arlaug_symbol": "ᛮ",
        "runr:tvimadur_symbol": "ᛯ",
        "runr:belgthor_symbol": "ᛰ",
        "runr:k": "ᛱ",
        "runr:sh": "ᛲ",
        "runr:oo": "ᛳ",
        "runr:franks_casket_os": "ᛴ",
        "runr:franks_casket_is": "ᛵ",
        "runr:franks_casket_eh": "ᛶ",
        "runr:franks_casket_ac": "ᛷ",
        "runr:franks_casket_aesc": "ᛸ",
        "mong:birga": "᠀",
        "mong:ellipsis": "᠁",
        "mong:comma": "᠂",
        "mong:period": "᠃",
        "mong:colon": "᠄",
        "mong:fourdots": "᠅",
        "mong:softhyphentodo": "᠆",
        "mong:syllableboundarymarkersibe": "᠇",
        "mong:commamanchu": "᠈",
        "mong:periodmanchu": "᠉",
        "mong:nirugu": "᠊",
        "mong:freevariationselectorone": "᠋",
        "mong:freevariationselectortwo": "᠌",
        "mong:freevariationselectorthree": "᠍",
        "mong:vowelseparator": "\u180e",
        "mong:zero": "᠐",
        "mong:one": "᠑",
        "mong:two": "᠒",
        "mong:three": "᠓",
        "mong:four": "᠔",
        "mong:five": "᠕",
        "mong:six": "᠖",
        "mong:seven": "᠗",
        "mong:eight": "᠘",
        "mong:nine": "᠙",
        "mong:a": "ᠠ",
        "mong:e": "ᠡ",
        "mong:i": "ᠢ",
        "mong:o": "ᠣ",
        "mong:u": "ᠤ",
        "mong:oe": "ᠥ",
        "mong:ue": "ᠦ",
        "mong:ee": "ᠧ",
        "mong:na": "ᠨ",
        "mong:ang": "ᠩ",
        "mong:ba": "ᠪ",
        "mong:pa": "ᠫ",
        "mong:qa": "ᠬ",
        "mong:ga": "ᠭ",
        "mong:ma": "ᠮ",
        "mong:la": "ᠯ",
        "mong:sa": "ᠰ",
        "mong:sha": "ᠱ",
        "mong:ta": "ᠲ",
        "mong:da": "ᠳ",
        "mong:cha": "ᠴ",
        "mong:ja": "ᠵ",
        "mong:ya": "ᠶ",
        "mong:ra": "ᠷ",
        "mong:wa": "ᠸ",
        "mong:fa": "ᠹ",
        "mong:ka": "ᠺ",
        "mong:kha": "ᠻ",
        "mong:tsa": "ᠼ",
        "mong:za": "ᠽ",
        "mong:haa": "ᠾ",
        "mong:zra": "ᠿ",
        "mong:lha": "ᡀ",
        "mong:zhi": "ᡁ",
        "mong:chi": "ᡂ",
        "mong:longvowelsigntodo": "ᡃ",
        "mong:etodo": "ᡄ",
        "mong:itodo": "ᡅ",
        "mong:otodo": "ᡆ",
        "mong:utodo": "ᡇ",
        "mong:oetodo": "ᡈ",
        "mong:uetodo": "ᡉ",
        "mong:angtodo": "ᡊ",
        "mong:batodo": "ᡋ",
        "mong:patodo": "ᡌ",
        "mong:qatodo": "ᡍ",
        "mong:gatodo": "ᡎ",
        "mong:matodo": "ᡏ",
        "mong:tatodo": "ᡐ",
        "mong:datodo": "ᡑ",
        "mong:chatodo": "ᡒ",
        "mong:jatodo": "ᡓ",
        "mong:tsatodo": "ᡔ",
        "mong:yatodo": "ᡕ",
        "mong:watodo": "ᡖ",
        "mong:katodo": "ᡗ",
        "mong:gaatodo": "ᡘ",
        "mong:haatodo": "ᡙ",
        "mong:jiatodo": "ᡚ",
        "mong:niatodo": "ᡛ",
        "mong:dzatodo": "ᡜ",
        "mong:esibe": "ᡝ",
        "mong:isibe": "ᡞ",
        "mong:iysibe": "ᡟ",
        "mong:uesibe": "ᡠ",
        "mong:usibe": "ᡡ",
        "mong:angsibe": "ᡢ",
        "mong:kasibe": "ᡣ",
        "mong:gasibe": "ᡤ",
        "mong:hasibe": "ᡥ",
        "mong:pasibe": "ᡦ",
        "mong:shasibe": "ᡧ",
        "mong:tasibe": "ᡨ",
        "mong:dasibe": "ᡩ",
        "mong:jasibe": "ᡪ",
        "mong:fasibe": "ᡫ",
        "mong:gaasibe": "ᡬ",
        "mong:haasibe": "ᡭ",
        "mong:tsasibe": "ᡮ",
        "mong:zasibe": "ᡯ",
        "mong:raasibe": "ᡰ",
        "mong:chasibe": "ᡱ",
        "mong:zhasibe": "ᡲ",
        "mong:imanchu": "ᡳ",
        "mong:kamanchu": "ᡴ",
        "mong:ramanchu": "ᡵ",
        "mong:famanchu": "ᡶ",
        "mong:zhamanchu": "ᡷ",
        "mong:chawithtwodots": "ᡸ",
        "mong:anusvaraonealigali": "ᢀ",
        "mong:visargaonealigali": "ᢁ",
        "mong:damarualigali": "ᢂ",
        "mong:ubadamaaligali": "ᢃ",
        "mong:ubadamaaligaliinverted": "ᢄ",
        "mong:baludaaligali": "ᢅ",
        "mong:baludaaligalithree": "ᢆ",
        "mong:aaligali": "ᢇ",
        "mong:ialigali": "ᢈ",
        "mong:kaaligali": "ᢉ",
        "mong:ngaaligali": "ᢊ",
        "mong:caaligali": "ᢋ",
        "mong:ttaaligali": "ᢌ",
        "mong:tthaaligali": "ᢍ",
        "mong:ddaaligali": "ᢎ",
        "mong:nnaaligali": "ᢏ",
        "mong:taaligali": "ᢐ",
        "mong:daaligali": "ᢑ",
        "mong:paaligali": "ᢒ",
        "mong:phaaligali": "ᢓ",
        "mong:ssaaligali": "ᢔ",
        "mong:zhaaligali": "ᢕ",
        "mong:zaaligali": "ᢖ",
        "mong:ahaligali": "ᢗ",
        "mong:tatodoaligali": "ᢘ",
        "mong:zhatodoaligali": "ᢙ",
        "mong:ghamanchualigali": "ᢚ",
        "mong:ngamanchualigali": "ᢛ",
        "mong:camanchualigali": "ᢜ",
        "mong:jhamanchualigali": "ᢝ",
        "mong:ttamanchualigali": "ᢞ",
        "mong:ddhamanchualigali": "ᢟ",
        "mong:tamanchualigali": "ᢠ",
        "mong:dhamanchualigali": "ᢡ",
        "mong:ssamanchualigali": "ᢢ",
        "mong:cyamanchualigali": "ᢣ",
        "mong:zhamanchualigali": "ᢤ",
        "mong:zamanchualigali": "ᢥ",
        "mong:ualigalihalf": "ᢦ",
        "mong:yaaligalihalf": "ᢧ",
        "mong:bhamanchualigali": "ᢨ",
        "mong:dagalgaaligali": "ᢩ",
        "mong:lhamanchualigali": "ᢪ",
        "AnGeor": "Ა",
        "BanGeor": "Ბ",
        "GanGeor": "Გ",
        "DonGeor": "Დ",
        "EnGeor": "Ე",
        "VinGeor": "Ვ",
        "ZenGeor": "Ზ",
        "TanGeor": "Თ",
        "InGeor": "Ი",
        "KanGeor": "Კ",
        "LasGeor": "Ლ",
        "ManGeor": "Მ",
        "NarGeor": "Ნ",
        "OnGeor": "Ო",
        "ParGeor": "Პ",
        "ZharGeor": "Ჟ",
        "RaeGeor": "Რ",
        "SanGeor": "Ს",
        "TarGeor": "Ტ",
        "UnGeor": "Უ",
        "PharGeor": "Ფ",
        "KharGeor": "Ქ",
        "GhanGeor": "Ღ",
        "QarGeor": "Ყ",
        "ShinGeor": "Შ",
        "ChinGeor": "Ჩ",
        "CanGeor": "Ც",
        "JilGeor": "Ძ",
        "CilGeor": "Წ",
        "CharGeor": "Ჭ",
        "XanGeor": "Ხ",
        "JhanGeor": "Ჯ",
        "HaeGeor": "Ჰ",
        "HeGeor": "Ჱ",
        "HieGeor": "Ჲ",
        "WeGeor": "Ჳ",
        "HarGeor": "Ჴ",
        "HoeGeor": "Ჵ",
        "FiGeor": "Ჶ",
        "YnGeor": "Ჷ",
        "ElifiGeor": "Ჸ",
        "TurnedganGeor": "Ჹ",
        "AinGeor": "Ჺ",
        "AenGeor": "Ჽ",
        "HardsignGeor": "Ჾ",
        "LabialsignGeor": "Ჿ",
        "ve:karshanatone": "᳐",
        "ve:sharatone": "᳑",
        "ve:prenkhatone": "᳒",
        "ve:nihshvasasign": "᳓",
        "ve:yajurmidlinesvaritasign": "᳔",
        "ve:yajuraggravatedindependentsvaritatone": "᳕",
        "ve:yajurindependentsvaritatone": "᳖",
        "ve:yajurkathakaindependentsvaritatone": "᳗",
        "ve:belowtonecandra": "᳘",
        "ve:yajurkathakaindependentsvaritaschroedertone": "᳙",
        "ve:svaritatonedbl": "᳚",
        "ve:svaritatonetpl": "᳛",
        "ve:kathakaanudattatone": "᳜",
        "ve:dotbelowtone": "᳝",
        "ve:twodotsbelowtone": "᳞",
        "ve:threedotsbelowtone": "᳟",
        "ve:rigkashmiriindependentsvaritatone": "᳠",
        "ve:atharvaindependentsvaritatone": "᳡",
        "ve:visargasvaritasign": "᳢",
        "ve:visargaudattasign": "᳣",
        "ve:visargaudattasignreversed": "᳤",
        "ve:visargaanudattasign": "᳥",
        "ve:visargaanudattasignreversed": "᳦",
        "ve:visargaudattawithtailsign": "᳧",
        "ve:visargaanudattawithtailsign": "᳨",
        "ve:anusvaraantargomukhasign": "ᳩ",
        "ve:anusvarabahirgomukhasign": "ᳪ",
        "ve:anusvaravamagomukhasign": "ᳫ",
        "ve:anusvaravamagomukhawithtailsign": "ᳬ",
        "ve:tiryaksign": "᳭",
        "ve:hexiformanusvarasignlong": "ᳮ",
        "ve:anusvarasignlong": "ᳯ",
        "ve:rthanganusvarasignlong": "ᳰ",
        "ve:anusvaraubhayatomukhasign": "ᳱ",
        "ve:ardhavisargasign": "ᳲ",
        "ve:rotatedardhavisargasign": "ᳳ",
        "ve:abovetonecandra": "᳴",
        "ve:jihvamuliyasign": "ᳵ",
        "ve:upadhmaniyasign": "ᳶ",
        "ve:atikramasign": "᳷",
        "ve:ringabovetone": "᳸",
        "ve:ringabovetonedbl": "᳹",
        "ve:anusvaraantargomukhasigndbl": "ᳺ",
        "Asmall": "ᴀ",
        "AEsmall": "ᴁ",
        "aeturned": "ᴂ",
        "Bbarsmall": "ᴃ",
        "Csmall": "ᴄ",
        "Dsmall": "ᴅ",
        "Ethsmall": "ᴆ",
        "Esmall": "ᴇ",
        "eopenturned": "ᴈ",
        "iturned": "ᴉ",
        "Jsmall": "ᴊ",
        "Ksmall": "ᴋ",
        "Lsmallstroke": "ᴌ",
        "Msmall": "ᴍ",
        "Nsmallreversed": "ᴎ",
        "Osmall": "ᴏ",
        "Oopensmall": "ᴐ",
        "osideways": "ᴑ",
        "oopensideways": "ᴒ",
        "ostrokesideways": "ᴓ",
        "oeturned": "ᴔ",
        "OUsmall": "ᴕ",
        "otophalf": "ᴖ",
        "obottomhalf": "ᴗ",
        "Psmall": "ᴘ",
        "Rsmallreversed": "ᴙ",
        "Rsmallturned": "ᴚ",
        "Tsmall": "ᴛ",
        "Usmall": "ᴜ",
        "usideways": "ᴝ",
        "usidewaysdieresised": "ᴞ",
        "mturnedsideways": "ᴟ",
        "Vsmall": "ᴠ",
        "Wsmall": "ᴡ",
        "Zsmall": "ᴢ",
        "Ezhsmall": "ᴣ",
        "spirantvoicedlaryngeal": "ᴤ",
        "phon:ain": "ᴥ",
        "gr:Gammasmall": "ᴦ",
        "gr:Lambdasmall": "ᴧ",
        "gr:Pismall": "ᴨ",
        "gr:RsmallHO": "ᴩ",
        "gr:Psismall": "ᴪ",
        "Elsmallcyr": "ᴫ",
        "Amod": "ᴬ",
        "Aemod": "ᴭ",
        "Bmod": "ᴮ",
        "Bbarmod": "ᴯ",
        "Dmod": "ᴰ",
        "Emod": "ᴱ",
        "Ereversedmod": "ᴲ",
        "Gmod": "ᴳ",
        "Hmod": "ᴴ",
        "Imod": "ᴵ",
        "Jmod": "ᴶ",
        "Kmod": "ᴷ",
        "Lmod": "ᴸ",
        "Mmod": "ᴹ",
        "Nmod": "ᴺ",
        "Nreversedmod": "ᴻ",
        "Omod": "ᴼ",
        "Oumod": "ᴽ",
        "Pmod": "ᴾ",
        "Rmod": "ᴿ",
        "Tmod": "ᵀ",
        "Umod": "ᵁ",
        "Wmod": "ᵂ",
        "amod": "ᵃ",
        "aturnedmod": "ᵄ",
        "alphamod": "ᵅ",
        "aeturnedmod": "ᵆ",
        "bmod": "ᵇ",
        "dmod": "ᵈ",
        "emod": "ᵉ",
        "schwamod": "ᵊ",
        "eopenmod": "ᵋ",
        "eopenturnedmod": "ᵌ",
        "gmod": "ᵍ",
        "iturnedmod": "ᵎ",
        "kmod": "ᵏ",
        "mmod": "ᵐ",
        "engmod": "ᵑ",
        "omod": "ᵒ",
        "oopenmod": "ᵓ",
        "otophalfmod": "ᵔ",
        "obottomhalfmod": "ᵕ",
        "pmod": "ᵖ",
        "tmod": "ᵗ",
        "umod": "ᵘ",
        "usidewaysmod": "ᵙ",
        "mturnedmod": "ᵚ",
        "vmod": "ᵛ",
        "ainmod": "ᵜ",
        "betamod": "ᵝ",
        "gr:gammamod": "ᵞ",
        "deltamod": "ᵟ",
        "gr:phimod": "ᵠ",
        "chimod": "ᵡ",
        "isubscript": "ᵢ",
        "rsubscript": "ᵣ",
        "usubscript": "ᵤ",
        "vsubscript": "ᵥ",
        "gr:betasubscript": "ᵦ",
        "gr:gammasubscript": "ᵧ",
        "gr:rhosubscript": "ᵨ",
        "gr:phisubscript": "ᵩ",
        "gr:chisubscript": "ᵪ",
        "ue": "ᵫ",
        "bmiddletilde": "ᵬ",
        "dmiddletilde": "ᵭ",
        "fmiddletilde": "ᵮ",
        "mmiddletilde": "ᵯ",
        "nmiddletilde": "ᵰ",
        "pmiddletilde": "ᵱ",
        "rmiddletilde": "ᵲ",
        "rfishmiddletilde": "ᵳ",
        "smiddletilde": "ᵴ",
        "tmiddletilde": "ᵵ",
        "zmiddletilde": "ᵶ",
        "gturned": "ᵷ",
        "ENcyrmod": "ᵸ",
        "ginsular": "ᵹ",
        "thstrike": "ᵺ",
        "Ismallstroke": "ᵻ",
        "iotastroke": "ᵼ",
        "pstroke": "ᵽ",
        "Usmallstroke": "ᵾ",
        "upsilonstroke": "ᵿ",
        "bpalatalhook": "ᶀ",
        "dpalatalhook": "ᶁ",
        "fpalatalhook": "ᶂ",
        "gpalatalhook": "ᶃ",
        "kpalatalhook": "ᶄ",
        "lpalatalhook": "ᶅ",
        "mpalatalhook": "ᶆ",
        "npalatalhook": "ᶇ",
        "ppalatalhook": "ᶈ",
        "rpalatalhook": "ᶉ",
        "spalatalhook": "ᶊ",
        "eshpalatalhook": "ᶋ",
        "vpalatalhook": "ᶌ",
        "xpalatalhook": "ᶍ",
        "zpalatalhook": "ᶎ",
        "aretroflexhook": "ᶏ",
        "alpharetroflexhook": "ᶐ",
        "dhooktail": "ᶑ",
        "eretroflexhook": "ᶒ",
        "eopenretroflexhook": "ᶓ",
        "eopenreversedretroflexhook": "ᶔ",
        "schwaretroflexhook": "ᶕ",
        "iretroflexhook": "ᶖ",
        "oopenretroflexhook": "ᶗ",
        "eshretroflexhook": "ᶘ",
        "uretroflexhook": "ᶙ",
        "ezhretroflexhook": "ᶚ",
        "alphaturnedmod": "ᶛ",
        "cmod": "ᶜ",
        "ccurlmod": "ᶝ",
        "ethmod": "ᶞ",
        "eopenreversedmod": "ᶟ",
        "fmod": "ᶠ",
        "dotlessjstrokemod": "ᶡ",
        "gscriptmod": "ᶢ",
        "hturnedmod": "ᶣ",
        "istrokemod": "ᶤ",
        "iotamod": "ᶥ",
        "Ismallmod": "ᶦ",
        "Istrokesmallmod": "ᶧ",
        "jcrossedtailmod": "ᶨ",
        "lretroflexhookmod": "ᶩ",
        "lpalatalhookmod": "ᶪ",
        "Lsmallmod": "ᶫ",
        "mhookmod": "ᶬ",
        "mlonglegturnedmod": "ᶭ",
        "nlefthookmod": "ᶮ",
        "nretroflexhookmod": "ᶯ",
        "Nsmallmod": "ᶰ",
        "obarmod": "ᶱ",
        "phimod": "ᶲ",
        "shookmod": "ᶳ",
        "eshmod": "ᶴ",
        "tpalatalhookmod": "ᶵ",
        "ubarmod": "ᶶ",
        "upsilonmod": "ᶷ",
        "Usmallmod": "ᶸ",
        "vhookmod": "ᶹ",
        "vturnedmod": "ᶺ",
        "zmod": "ᶻ",
        "zretroflexhookmod": "ᶼ",
        "zcurlmod": "ᶽ",
        "ezhmod": "ᶾ",
        "thetamod": "ᶿ",
        "dottedgravecmb": "᷀",
        "dottedacutecmb": "᷁",
        "snakebelowcmb": "᷂",
        "suspensionmarkcmb": "᷃",
        "macronacutecmb": "᷄",
        "gravemacroncmb": "᷅",
        "macrongravecmb": "᷆",
        "acutemacroncmb": "᷇",
        "graveacutegravecmb": "᷈",
        "acutegraveacutecmb": "᷉",
        "rbelowcmb": "᷊",
        "brevemacroncmb": "᷋",
        "macronbrevecmb": "᷌",
        "doubleabovecircumflexcmb": "᷍",
        "aboveogonekcmb": "᷎",
        "zigzagbelowcmb": "᷏",
        "isbelowcmb": "᷐",
        "urabovecmb": "᷑",
        "usabovecmb": "᷒",
        "aaboveflatcmb": "ᷓ",
        "aecmb": "ᷔ",
        "aocmb": "ᷕ",
        "avcmb": "ᷖ",
        "ccedillacmb": "ᷗ",
        "insulardcmb": "ᷘ",
        "ethcmb": "ᷙ",
        "gcmb": "ᷚ",
        "gsmallcmb": "ᷛ",
        "kcmb": "ᷜ",
        "lcmb": "ᷝ",
        "lsmallcmb": "ᷞ",
        "msmallcmb": "ᷟ",
        "ncmb": "ᷠ",
        "nsmallcmb": "ᷡ",
        "rsmallcmb": "ᷢ",
        "rrotundacmb": "ᷣ",
        "scmb": "ᷤ",
        "longscmb": "ᷥ",
        "zcmb": "ᷦ",
        "alphacmb": "ᷧ",
        "bcmb": "ᷨ",
        "betacmb": "ᷩ",
        "schwacmb": "ᷪ",
        "fcmb": "ᷫ",
        "lwithdoublemiddletildecmb": "ᷬ",
        "owithlightcentralizationstrokecmb": "ᷭ",
        "pcmb": "ᷮ",
        "eshcmb": "ᷯ",
        "uwithlightcentralizationstrokecmb": "ᷰ",
        "wcmb": "ᷱ",
        "adieresiscmb": "ᷲ",
        "odieresiscmb": "ᷳ",
        "udieresiscmb": "ᷴ",
        "uptackabovecmb": "᷵",
        "kavykaaboverightcmb": "᷶",
        "kavykaaboveleftcmb": "᷷",
        "dotaboveleftcmb": "᷸",
        "wideinvertedbridgebelowcmb": "᷹",
        "deletionmarkcmb": "᷻",
        "doubleinvertedbelowbrevecmb": "᷼",
        "almostequaltobelowcmb": "᷽",
        "leftarrowheadabovecmb": "᷾",
        "rightarrowheadanddownarrowheadbelowcmb": "᷿",
        "Aringbelow": "Ḁ",
        "aringbelow": "ḁ",
        "Bdot": "Ḃ",
        "bdot": "ḃ",
        "Bdotbelow": "Ḅ",
        "bdotbelow": "ḅ",
        "Blinebelow": "Ḇ",
        "blinebelow": "ḇ",
        "Ccedillaacute": "Ḉ",
        "ccedillaacute": "ḉ",
        "Ddot": "Ḋ",
        "ddot": "ḋ",
        "Ddotbelow": "Ḍ",
        "ddotbelow": "ḍ",
        "Dlinebelow": "Ḏ",
        "dlinebelow": "ḏ",
        "Dcommaaccent": "Ḑ",
        "dcommaaccent": "ḑ",
        "Dcircumflexbelow": "Ḓ",
        "dcircumflexbelow": "ḓ",
        "Emacrongrave": "Ḕ",
        "emacrongrave": "ḕ",
        "Emacronacute": "Ḗ",
        "emacronacute": "ḗ",
        "Ecircumflexbelow": "Ḙ",
        "ecircumflexbelow": "ḙ",
        "Etildebelow": "Ḛ",
        "etildebelow": "ḛ",
        "Ecedillabreve": "Ḝ",
        "ecedillabreve": "ḝ",
        "Fdot": "Ḟ",
        "fdot": "ḟ",
        "Gmacron": "Ḡ",
        "gmacron": "ḡ",
        "Hdot": "Ḣ",
        "hdot": "ḣ",
        "Hdotbelow": "Ḥ",
        "hdotbelow": "ḥ",
        "Hdieresis": "Ḧ",
        "hdieresis": "ḧ",
        "Hcedilla": "Ḩ",
        "hcedilla": "ḩ",
        "Hbrevebelow": "Ḫ",
        "hbrevebelow": "ḫ",
        "Itildebelow": "Ḭ",
        "itildebelow": "ḭ",
        "Idieresisacute": "Ḯ",
        "idieresisacute": "ḯ",
        "Kacute": "Ḱ",
        "kacute": "ḱ",
        "Kdotbelow": "Ḳ",
        "kdotbelow": "ḳ",
        "Klinebelow": "Ḵ",
        "klinebelow": "ḵ",
        "Ldotbelow": "Ḷ",
        "ldotbelow": "ḷ",
        "Lmacrondot": "Ḹ",
        "lmacrondot": "ḹ",
        "Llinebelow": "Ḻ",
        "llinebelow": "ḻ",
        "Lcircumflexbelow": "Ḽ",
        "lcircumflexbelow": "ḽ",
        "Macute": "Ḿ",
        "macute": "ḿ",
        "Mdot": "Ṁ",
        "mdot": "ṁ",
        "Mdotbelow": "Ṃ",
        "mdotbelow": "ṃ",
        "Ndot": "Ṅ",
        "ndot": "ṅ",
        "Ndotbelow": "Ṇ",
        "ndotbelow": "ṇ",
        "Nlinebelow": "Ṉ",
        "nlinebelow": "ṉ",
        "Ncircumflexbelow": "Ṋ",
        "ncircumflexbelow": "ṋ",
        "Otildeacute": "Ṍ",
        "otildeacute": "ṍ",
        "Otildedieresis": "Ṏ",
        "otildedieresis": "ṏ",
        "Omacrongrave": "Ṑ",
        "omacrongrave": "ṑ",
        "Omacronacute": "Ṓ",
        "omacronacute": "ṓ",
        "Pacute": "Ṕ",
        "pacute": "ṕ",
        "Pdot": "Ṗ",
        "pdot": "ṗ",
        "Rdot": "Ṙ",
        "rdot": "ṙ",
        "Rdotbelow": "Ṛ",
        "rdotbelow": "ṛ",
        "Rmacrondot": "Ṝ",
        "rmacrondot": "ṝ",
        "Rlinebelow": "Ṟ",
        "rlinebelow": "ṟ",
        "Sdot": "Ṡ",
        "sdot": "ṡ",
        "Sdotbelow": "Ṣ",
        "sdotbelow": "ṣ",
        "Sacutedotaccent": "Ṥ",
        "sacutedotaccent": "ṥ",
        "Scarondot": "Ṧ",
        "scarondot": "ṧ",
        "Sdotbelowdotabove": "Ṩ",
        "sdotbelowdotabove": "ṩ",
        "Tdot": "Ṫ",
        "tdot": "ṫ",
        "Tdotbelow": "Ṭ",
        "tdotbelow": "ṭ",
        "Tlinebelow": "Ṯ",
        "tlinebelow": "ṯ",
        "Tcircumflexbelow": "Ṱ",
        "tcircumflexbelow": "ṱ",
        "Udieresisbelow": "Ṳ",
        "udieresisbelow": "ṳ",
        "Utildebelow": "Ṵ",
        "utildebelow": "ṵ",
        "Ucircumflexbelow": "Ṷ",
        "ucircumflexbelow": "ṷ",
        "Utildeacute": "Ṹ",
        "utildeacute": "ṹ",
        "Umacrondieresis": "Ṻ",
        "umacrondieresis": "ṻ",
        "Vtilde": "Ṽ",
        "vtilde": "ṽ",
        "Vdotbelow": "Ṿ",
        "vdotbelow": "ṿ",
        "Wgrave": "Ẁ",
        "wgrave": "ẁ",
        "Wacute": "Ẃ",
        "wacute": "ẃ",
        "Wdieresis": "Ẅ",
        "wdieresis": "ẅ",
        "Wdot": "Ẇ",
        "wdot": "ẇ",
        "Wdotbelow": "Ẉ",
        "wdotbelow": "ẉ",
        "Xdot": "Ẋ",
        "xdot": "ẋ",
        "Xdieresis": "Ẍ",
        "xdieresis": "ẍ",
        "Ydot": "Ẏ",
        "ydot": "ẏ",
        "Zcircumflex": "Ẑ",
        "zcircumflex": "ẑ",
        "Zdotbelow": "Ẓ",
        "zdotbelow": "ẓ",
        "Zlinebelow": "Ẕ",
        "zlinebelow": "ẕ",
        "hlinebelow": "ẖ",
        "tdieresis": "ẗ",
        "wring": "ẘ",
        "yring": "ẙ",
        "arighthalfring": "ẚ",
        "longsdot": "ẛ",
        "longswithdiagonalstroke": "ẜ",
        "longswithhighstroke": "ẝ",
        "Germandbls": "ẞ",
        "lt:delta": "ẟ",
        "Adotbelow": "Ạ",
        "adotbelow": "ạ",
        "Ahoi": "Ả",
        "ahoi": "ả",
        "Acircumflexacute": "Ấ",
        "acircumflexacute": "ấ",
        "Acircumflexgrave": "Ầ",
        "acircumflexgrave": "ầ",
        "Acircumflexhoi": "Ẩ",
        "acircumflexhoi": "ẩ",
        "Acircumflextilde": "Ẫ",
        "acircumflextilde": "ẫ",
        "Acircumflexdotbelow": "Ậ",
        "acircumflexdotbelow": "ậ",
        "Abreveacute": "Ắ",
        "abreveacute": "ắ",
        "Abrevegrave": "Ằ",
        "abrevegrave": "ằ",
        "Abrevehoi": "Ẳ",
        "abrevehoi": "ẳ",
        "Abrevetilde": "Ẵ",
        "abrevetilde": "ẵ",
        "Abrevedotbelow": "Ặ",
        "abrevedotbelow": "ặ",
        "Edotbelow": "Ẹ",
        "edotbelow": "ẹ",
        "Ehoi": "Ẻ",
        "ehoi": "ẻ",
        "Etilde": "Ẽ",
        "etilde": "ẽ",
        "Ecircumflexacute": "Ế",
        "ecircumflexacute": "ế",
        "Ecircumflexgrave": "Ề",
        "ecircumflexgrave": "ề",
        "Ecircumflexhoi": "Ể",
        "ecircumflexhoi": "ể",
        "Ecircumflextilde": "Ễ",
        "ecircumflextilde": "ễ",
        "Ecircumflexdotbelow": "Ệ",
        "ecircumflexdotbelow": "ệ",
        "Ihoi": "Ỉ",
        "ihoi": "ỉ",
        "Idotbelow": "Ị",
        "idotbelow": "ị",
        "Odotbelow": "Ọ",
        "odotbelow": "ọ",
        "Ohoi": "Ỏ",
        "ohoi": "ỏ",
        "Ocircumflexacute": "Ố",
        "ocircumflexacute": "ố",
        "Ocircumflexgrave": "Ồ",
        "ocircumflexgrave": "ồ",
        "Ocircumflexhoi": "Ổ",
        "ocircumflexhoi": "ổ",
        "Ocircumflextilde": "Ỗ",
        "ocircumflextilde": "ỗ",
        "Ocircumflexdotbelow": "Ộ",
        "ocircumflexdotbelow": "ộ",
        "Ohornacute": "Ớ",
        "ohornacute": "ớ",
        "Ohorngrave": "Ờ",
        "ohorngrave": "ờ",
        "Ohornhoi": "Ở",
        "ohornhoi": "ở",
        "Ohorntilde": "Ỡ",
        "ohorntilde": "ỡ",
        "Ohorndotbelow": "Ợ",
        "ohorndotbelow": "ợ",
        "Udotbelow": "Ụ",
        "udotbelow": "ụ",
        "Uhoi": "Ủ",
        "uhoi": "ủ",
        "Uhornacute": "Ứ",
        "uhornacute": "ứ",
        "Uhorngrave": "Ừ",
        "uhorngrave": "ừ",
        "Uhornhoi": "Ử",
        "uhornhoi": "ử",
        "Uhorntilde": "Ữ",
        "uhorntilde": "ữ",
        "Uhorndotbelow": "Ự",
        "uhorndotbelow": "ự",
        "Ygrave": "Ỳ",
        "ygrave": "ỳ",
        "Ydotbelow": "Ỵ",
        "ydotbelow": "ỵ",
        "Yhoi": "Ỷ",
        "yhoi": "ỷ",
        "Ytilde": "Ỹ",
        "ytilde": "ỹ",
        "LLwelsh": "Ỻ",
        "llwelsh": "ỻ",
        "Vwelsh": "Ỽ",
        "vwelsh": "ỽ",
        "Yloop": "Ỿ",
        "yloop": "ỿ",
        "alphalenis": "ἀ",
        "alphaasper": "ἁ",
        "alphalenisgrave": "ἂ",
        "alphaaspergrave": "ἃ",
        "alphalenisacute": "ἄ",
        "alphaasperacute": "ἅ",
        "alphalenistilde": "ἆ",
        "alphaaspertilde": "ἇ",
        "Alphalenis": "Ἀ",
        "Alphaasper": "Ἁ",
        "Alphalenisgrave": "Ἂ",
        "Alphaaspergrave": "Ἃ",
        "Alphalenisacute": "Ἄ",
        "Alphaasperacute": "Ἅ",
        "Alphalenistilde": "Ἆ",
        "Alphaaspertilde": "Ἇ",
        "epsilonlenis": "ἐ",
        "epsilonasper": "ἑ",
        "epsilonlenisgrave": "ἒ",
        "epsilonaspergrave": "ἓ",
        "epsilonlenisacute": "ἔ",
        "epsilonasperacute": "ἕ",
        "Epsilonlenis": "Ἐ",
        "Epsilonasper": "Ἑ",
        "Epsilonlenisgrave": "Ἒ",
        "Epsilonaspergrave": "Ἓ",
        "Epsilonlenisacute": "Ἔ",
        "Epsilonasperacute": "Ἕ",
        "etalenis": "ἠ",
        "etaasper": "ἡ",
        "etalenisgrave": "ἢ",
        "etaaspergrave": "ἣ",
        "etalenisacute": "ἤ",
        "etaasperacute": "ἥ",
        "etalenistilde": "ἦ",
        "etaaspertilde": "ἧ",
        "Etalenis": "Ἠ",
        "Etaasper": "Ἡ",
        "Etalenisgrave": "Ἢ",
        "Etaaspergrave": "Ἣ",
        "Etalenisacute": "Ἤ",
        "Etaasperacute": "Ἥ",
        "Etalenistilde": "Ἦ",
        "Etaaspertilde": "Ἧ",
        "iotalenis": "ἰ",
        "iotaasper": "ἱ",
        "iotalenisgrave": "ἲ",
        "iotaaspergrave": "ἳ",
        "iotalenisacute": "ἴ",
        "iotaasperacute": "ἵ",
        "iotalenistilde": "ἶ",
        "iotaaspertilde": "ἷ",
        "Iotalenis": "Ἰ",
        "Iotaasper": "Ἱ",
        "Iotalenisgrave": "Ἲ",
        "Iotaaspergrave": "Ἳ",
        "Iotalenisacute": "Ἴ",
        "Iotaasperacute": "Ἵ",
        "Iotalenistilde": "Ἶ",
        "Iotaaspertilde": "Ἷ",
        "omicronlenis": "ὀ",
        "omicronasper": "ὁ",
        "omicronlenisgrave": "ὂ",
        "omicronaspergrave": "ὃ",
        "omicronlenisacute": "ὄ",
        "omicronasperacute": "ὅ",
        "Omicronlenis": "Ὀ",
        "Omicronasper": "Ὁ",
        "Omicronlenisgrave": "Ὂ",
        "Omicronaspergrave": "Ὃ",
        "Omicronlenisacute": "Ὄ",
        "Omicronasperacute": "Ὅ",
        "upsilonlenis": "ὐ",
        "upsilonasper": "ὑ",
        "upsilonlenisgrave": "ὒ",
        "upsilonaspergrave": "ὓ",
        "upsilonlenisacute": "ὔ",
        "upsilonasperacute": "ὕ",
        "upsilonlenistilde": "ὖ",
        "upsilonaspertilde": "ὗ",
        "Upsilonasper": "Ὑ",
        "Upsilonaspergrave": "Ὓ",
        "Upsilonasperacute": "Ὕ",
        "Upsilonaspertilde": "Ὗ",
        "omegalenis": "ὠ",
        "omegaasper": "ὡ",
        "omegalenisgrave": "ὢ",
        "omegaaspergrave": "ὣ",
        "omegalenisacute": "ὤ",
        "omegaasperacute": "ὥ",
        "omegalenistilde": "ὦ",
        "omegaaspertilde": "ὧ",
        "Omegalenis": "Ὠ",
        "Omegaasper": "Ὡ",
        "Omegalenisgrave": "Ὢ",
        "Omegaaspergrave": "Ὣ",
        "Omegalenisacute": "Ὤ",
        "Omegaasperacute": "Ὥ",
        "Omegalenistilde": "Ὦ",
        "Omegaaspertilde": "Ὧ",
        "alphagrave": "ὰ",
        "alphaacute": "ά",
        "epsilongrave": "ὲ",
        "epsilonacute": "έ",
        "etagrave": "ὴ",
        "etaacute": "ή",
        "iotagrave": "ὶ",
        "iotaacute": "ί",
        "omicrongrave": "ὸ",
        "omicronacute": "ό",
        "upsilongrave": "ὺ",
        "upsilonacute": "ύ",
        "omegagrave": "ὼ",
        "omegaacute": "ώ",
        "alphalenisiotasub": "ᾀ",
        "alphaasperiotasub": "ᾁ",
        "alphalenisgraveiotasub": "ᾂ",
        "alphaaspergraveiotasub": "ᾃ",
        "alphalenisacuteiotasub": "ᾄ",
        "alphaasperacuteiotasub": "ᾅ",
        "alphalenistildeiotasub": "ᾆ",
        "alphaaspertildeiotasub": "ᾇ",
        "Alphalenisiotasub": "ᾈ",
        "Alphaasperiotasub": "ᾉ",
        "Alphalenisgraveiotasub": "ᾊ",
        "Alphaaspergraveiotasub": "ᾋ",
        "Alphalenisacuteiotasub": "ᾌ",
        "Alphaasperacuteiotasub": "ᾍ",
        "Alphalenistildeiotasub": "ᾎ",
        "Alphaaspertildeiotasub": "ᾏ",
        "etalenisiotasub": "ᾐ",
        "etaasperiotasub": "ᾑ",
        "etalenisgraveiotasub": "ᾒ",
        "etaaspergraveiotasub": "ᾓ",
        "etalenisacuteiotasub": "ᾔ",
        "etaasperacuteiotasub": "ᾕ",
        "etalenistildeiotasub": "ᾖ",
        "etaaspertildeiotasub": "ᾗ",
        "Etalenisiotasub": "ᾘ",
        "Etaasperiotasub": "ᾙ",
        "Etalenisgraveiotasub": "ᾚ",
        "Etaaspergraveiotasub": "ᾛ",
        "Etalenisacuteiotasub": "ᾜ",
        "Etaasperacuteiotasub": "ᾝ",
        "Etalenistildeiotasub": "ᾞ",
        "Etaaspertildeiotasub": "ᾟ",
        "omegalenisiotasub": "ᾠ",
        "omegaasperiotasub": "ᾡ",
        "omegalenisgraveiotasub": "ᾢ",
        "omegaaspergraveiotasub": "ᾣ",
        "omegalenisacuteiotasub": "ᾤ",
        "omegaasperacuteiotasub": "ᾥ",
        "omegalenistildeiotasub": "ᾦ",
        "omegaaspertildeiotasub": "ᾧ",
        "Omegalenisiotasub": "ᾨ",
        "Omegaasperiotasub": "ᾩ",
        "Omegalenisgraveiotasub": "ᾪ",
        "Omegaaspergraveiotasub": "ᾫ",
        "Omegalenisacuteiotasub": "ᾬ",
        "Omegaasperacuteiotasub": "ᾭ",
        "Omegalenistildeiotasub": "ᾮ",
        "Omegaaspertildeiotasub": "ᾯ",
        "alphabreve": "ᾰ",
        "alphawithmacron": "ᾱ",
        "alphagraveiotasub": "ᾲ",
        "alphaiotasub": "ᾳ",
        "alphaacuteiotasub": "ᾴ",
        "alphatilde": "ᾶ",
        "alphatildeiotasub": "ᾷ",
        "Alphabreve": "Ᾰ",
        "Alphawithmacron": "Ᾱ",
        "Alphagrave": "Ὰ",
        "Alphaacute": "Ά",
        "Alphaiotasub": "ᾼ",
        "KORONIS": "᾽",
        "iotaadscript": "ι",
        "lenis": "᾿",
        "gr:tilde": "῀",
        "dieresistilde": "῁",
        "etagraveiotasub": "ῂ",
        "etaiotasub": "ῃ",
        "etaacuteiotasub": "ῄ",
        "etatilde": "ῆ",
        "etatildeiotasub": "ῇ",
        "Epsilongrave": "Ὲ",
        "Epsilonacute": "Έ",
        "Etagrave": "Ὴ",
        "Etaacute": "Ή",
        "Etaiotasub": "ῌ",
        "lenisgrave": "῍",
        "lenisacute": "῎",
        "lenistilde": "῏",
        "iotabreve": "ῐ",
        "iotawithmacron": "ῑ",
        "iotadieresisgrave": "ῒ",
        "iotadieresisacute": "ΐ",
        "iotatilde": "ῖ",
        "iotadieresistilde": "ῗ",
        "Iotabreve": "Ῐ",
        "Iotawithmacron": "Ῑ",
        "Iotagrave": "Ὶ",
        "Iotaacute": "Ί",
        "aspergrave": "῝",
        "asperacute": "῞",
        "aspertilde": "῟",
        "upsilonbreve": "ῠ",
        "upsilonwithmacron": "ῡ",
        "upsilondieresisgrave": "ῢ",
        "upsilondieresisacute": "ΰ",
        "rholenis": "ῤ",
        "rhoasper": "ῥ",
        "upsilontilde": "ῦ",
        "upsilondieresistilde": "ῧ",
        "Upsilonbreve": "Ῠ",
        "Upsilonwithmacron": "Ῡ",
        "Upsilongrave": "Ὺ",
        "Upsilonacute": "Ύ",
        "Rhoasper": "Ῥ",
        "dieresisgrave": "῭",
        "dieresisacute": "΅",
        "gr:grave": "`",
        "omegagraveiotasub": "ῲ",
        "omegaiotasub": "ῳ",
        "omegaacuteiotasub": "ῴ",
        "omegatilde": "ῶ",
        "omegatildeiotasub": "ῷ",
        "Omicrongrave": "Ὸ",
        "Omicronacute": "Ό",
        "Omegagrave": "Ὼ",
        "Omegaacute": "Ώ",
        "Omegaiotasub": "ῼ",
        "gr:acute": "´",
        "asper": "῾",
        "enquad": "\u2000",
        "emquad": "\u2001",
        "enspace": "\u2002",
        "emspace": "\u2003",
        "threeperemspace": "\u2004",
        "fourperemspace": "\u2005",
        "sixperemspace": "\u2006",
        "figurespace": "\u2007",
        "punctuationspace": "\u2008",
        "thinspace": "\u2009",
        "hairspace": "\u200a",
        "zerowidthspace": "\u200b",
        "zerowidthnonjoiner": "\u200c",
        "zerowidthjoiner": "\u200d",
        "lefttorightmark": "\u200e",
        "righttoleftmark": "\u200f",
        "gnrl:hyphen": "‐",
        "nonbreakinghyphen": "‑",
        "figuredash": "‒",
        "endash": "–",
        "emdash": "—",
        "horizontalbar": "―",
        "verticalbardbl": "‖",
        "underscoredbl": "‗",
        "quoteleft": "‘",
        "quoteright": "’",
        "quotesinglbase": "‚",
        "quotereversed": "‛",
        "quotedblleft": "“",
        "quotedblright": "”",
        "quotedblbase": "„",
        "quotedblreversed": "‟",
        "dagger": "†",
        "daggerdbl": "‡",
        "bullet": "•",
        "triangularbullet": "‣",
        "onedotenleader": "․",
        "twodotenleader": "‥",
        "ellipsis": "…",
        "hyphenationpoint": "‧",
        "lineseparator": "\u2028",
        "paragraphseparator": "\u2029",
        "lefttorightembed": "\u202a",
        "righttoleftembed": "\u202b",
        "popdirectionalformatting": "\u202c",
        "lefttorightoverride": "\u202d",
        "righttoleftoverride": "\u202e",
        "narrownobreakspace": "\u202f",
        "perthousand": "‰",
        "pertenthousandsign": "‱",
        "minute": "′",
        "second": "″",
        "millisecond": "‴",
        "minutereversed": "‵",
        "secondreversed": "‶",
        "millisecondreversed": "‷",
        "caret": "‸",
        "guilsinglleft": "‹",
        "guilsinglright": "›",
        "referencemark": "※",
        "exclamdbl": "‼",
        "interrobang": "‽",
        "overline": "‾",
        "undertie": "‿",
        "charactertie": "⁀",
        "caretinsertionpoint": "⁁",
        "asterism": "⁂",
        "hyphenbullet": "⁃",
        "fraction": "⁄",
        "bracketleftsquarequill": "⁅",
        "bracketrightsquarequill": "⁆",
        "questiondbl": "⁇",
        "questionexclamationmark": "⁈",
        "exclamationquestion": "⁉",
        "tironiansignet": "⁊",
        "pilcrowsignreversed": "⁋",
        "blackwardsbulletleft": "⁌",
        "blackwardsbulletright": "⁍",
        "lowasterisk": "⁎",
        "semicolonreversed": "⁏",
        "closeup": "⁐",
        "twoasterisksalignedvertically": "⁑",
        "commercialminussign": "⁒",
        "swungdash": "⁓",
        "invertedundertie": "⁔",
        "flowerpunctuationmark": "⁕",
        "threedotpunctuation": "⁖",
        "quadrupleminute": "⁗",
        "fourdotpunctuation": "⁘",
        "fivedotpunctuation": "⁙",
        "twodotpunctuation": "⁚",
        "fourdotmark": "⁛",
        "dottedcross": "⁜",
        "tricolon": "⁝",
        "verticalfourdots": "⁞",
        "mediummathematicalspace": "\u205f",
        "wordjoiner": "\u2060",
        "functionapplication": "\u2061",
        "invisibletimes": "\u2062",
        "invisibleseparator": "\u2063",
        "invisibleplus": "\u2064",
        "lefttorightisolate": "\u2066",
        "righttoleftisolate": "\u2067",
        "firststrongisolate": "\u2068",
        "popdirectionalisolate": "\u2069",
        "inhibitsymmetricswapping": "\u206a",
        "activatesymmetricswapping": "\u206b",
        "inhibitarabicformshaping": "\u206c",
        "activatearabicformshaping": "\u206d",
        "nationaldigitshapes": "\u206e",
        "nominaldigitshapes": "\u206f",
        "zero.superior": "⁰",
        "i.superior": "ⁱ",
        "four.superior": "⁴",
        "five.superior": "⁵",
        "six.superior": "⁶",
        "seven.superior": "⁷",
        "eight.superior": "⁸",
        "nine.superior": "⁹",
        "plus.superior": "⁺",
        "minus.superior": "⁻",
        "equal.superior": "⁼",
        "parenleft.superior": "⁽",
        "parenright.superior": "⁾",
        "n.superior": "ⁿ",
        "zero.inferior": "₀",
        "one.inferior": "₁",
        "two.inferior": "₂",
        "three.inferior": "₃",
        "four.inferior": "₄",
        "five.inferior": "₅",
        "six.inferior": "₆",
        "seven.inferior": "₇",
        "eight.inferior": "₈",
        "nine.inferior": "₉",
        "plus.inferior": "₊",
        "minus.inferior": "₋",
        "equal.inferior": "₌",
        "parenleft.inferior": "₍",
        "parenright.inferior": "₎",
        "a.inferior": "ₐ",
        "e.inferior": "ₑ",
        "o.inferior": "ₒ",
        "x.inferior": "ₓ",
        "schwa.inferior": "ₔ",
        "h.inferior": "ₕ",
        "k.inferior": "ₖ",
        "l.inferior": "ₗ",
        "m.inferior": "ₘ",
        "n.inferior": "ₙ",
        "p.inferior": "ₚ",
        "s.inferior": "ₛ",
        "t.inferior": "ₜ",
        "euroarchaic": "₠",
        "colonmonetary": "₡",
        "cruzeiro": "₢",
        "franc": "₣",
        "lira": "₤",
        "mill": "₥",
        "naira": "₦",
        "peseta": "₧",
        "rupee": "₨",
        "won": "₩",
        "newsheqel": "₪",
        "dong": "₫",
        "Euro": "€",
        "kip": "₭",
        "tugrik": "₮",
        "drachma": "₯",
        "germanpenny": "₰",
        "peso": "₱",
        "guarani": "₲",
        "austral": "₳",
        "hryvnia": "₴",
        "cedi": "₵",
        "livretournois": "₶",
        "spesmilo": "₷",
        "tenge": "₸",
        "indianrupee": "₹",
        "turkishlira": "₺",
        "nordicmark": "₻",
        "manat": "₼",
        "ruble": "₽",
        "lari": "₾",
        "bitcoin": "₿",
        "accountof": "℀",
        "addressedsubject": "℁",
        "Cdblstruck": "ℂ",
        "degreecelsius": "℃",
        "centreline": "℄",
        "careof": "℅",
        "cadauna": "℆",
        "euler": "ℇ",
        "scruple": "℈",
        "degreefahrenheit": "℉",
        "lttr:gscript": "ℊ",
        "Hscript": "ℋ",
        "Hfraktur": "ℌ",
        "Hdblstruck": "ℍ",
        "planck": "ℎ",
        "plancktwopi": "ℏ",
        "Iscript": "ℐ",
        "Ifraktur": "ℑ",
        "Lscript": "ℒ",
        "litre": "ℓ",
        "lbbar": "℔",
        "Ndblstruck": "ℕ",
        "numero": "№",
        "soundcopyright": "℗",
        "weierstrass": "℘",
        "Pdblstruck": "ℙ",
        "Qdblstruck": "ℚ",
        "Rscript": "ℛ",
        "Rfraktur": "ℜ",
        "Rdblstruck": "ℝ",
        "prescription": "℞",
        "response": "℟",
        "servicemark": "℠",
        "telephone": "℡",
        "trademark": "™",
        "versicle": "℣",
        "Zdblstruck": "ℤ",
        "ounce": "℥",
        "ohm": "Ω",
        "ohminverted": "℧",
        "Zfraktur": "ℨ",
        "iotaturned": "℩",
        "kelvin": "K",
        "angstrom": "Å",
        "Bscript": "ℬ",
        "Cfraktur": "ℭ",
        "estimated": "℮",
        "escript": "ℯ",
        "Escript": "ℰ",
        "Fscript": "ℱ",
        "Fturned": "Ⅎ",
        "Mscript": "ℳ",
        "oscript": "ℴ",
        "aleph": "ℵ",
        "bet": "ℶ",
        "gimel": "ℷ",
        "dalet": "ℸ",
        "information": "ℹ",
        "Qrotated": "℺",
        "facsimile": "℻",
        "pidblstruck": "ℼ",
        "gammadblstruck": "ℽ",
        "Gammadblstruck": "ℾ",
        "Pidblstruck": "ℿ",
        "summationdblstruck": "⅀",
        "Gturnedsans": "⅁",
        "Lturnedsans": "⅂",
        "Lreversedsans": "⅃",
        "Yturnedsans": "⅄",
        "Ddblstruckitalic": "ⅅ",
        "ddblstruckitalic": "ⅆ",
        "edblstruckitalic": "ⅇ",
        "idblstruckitalic": "ⅈ",
        "jdblstruckitalic": "ⅉ",
        "propertyline": "⅊",
        "ampersandturned": "⅋",
        "per": "⅌",
        "aktieselskab": "⅍",
        "fturned": "ⅎ",
        "forsamaritan": "⅏",
        "oneseventh": "⅐",
        "oneninth": "⅑",
        "onetenth": "⅒",
        "onethird": "⅓",
        "twothirds": "⅔",
        "onefifth": "⅕",
        "twofifths": "⅖",
        "threefifths": "⅗",
        "fourfifths": "⅘",
        "onesixth": "⅙",
        "fivesixths": "⅚",
        "oneeighth": "⅛",
        "threeeighths": "⅜",
        "fiveeighths": "⅝",
        "seveneighths": "⅞",
        "onefraction": "⅟",
        "one.roman": "Ⅰ",
        "two.roman": "Ⅱ",
        "three.roman": "Ⅲ",
        "four.roman": "Ⅳ",
        "five.roman": "Ⅴ",
        "six.roman": "Ⅵ",
        "seven.roman": "Ⅶ",
        "eight.roman": "Ⅷ",
        "nine.roman": "Ⅸ",
        "ten.roman": "Ⅹ",
        "eleven.roman": "Ⅺ",
        "twelve.roman": "Ⅻ",
        "fifty.roman": "Ⅼ",
        "onehundred.roman": "Ⅽ",
        "fivehundred.roman": "Ⅾ",
        "onethousand.roman": "Ⅿ",
        "one.romansmall": "ⅰ",
        "two.romansmall": "ⅱ",
        "three.romansmall": "ⅲ",
        "four.romansmall": "ⅳ",
        "five.romansmall": "ⅴ",
        "six.romansmall": "ⅵ",
        "seven.romansmall": "ⅶ",
        "eight.romansmall": "ⅷ",
        "nine.romansmall": "ⅸ",
        "ten.romansmall": "ⅹ",
        "eleven.romansmall": "ⅺ",
        "twelve.romansmall": "ⅻ",
        "fifty.romansmall": "ⅼ",
        "onehundred.romansmall": "ⅽ",
        "fivehundred.romansmall": "ⅾ",
        "onethousand.romansmall": "ⅿ",
        "onethousandcd.roman": "ↀ",
        "fivethousand.roman": "ↁ",
        "tenthousand.roman": "ↂ",
        "reversedonehundred.roman": "Ↄ",
        "creversed": "ↄ",
        "sixlateform.roman": "ↅ",
        "fiftyearlyform.roman": "ↆ",
        "fiftythousand.roman": "ↇ",
        "onehundredthousand.roman": "ↈ",
        "zerothirds": "↉",
        "turneddigittwo": "↊",
        "turneddigitthree": "↋",
        "arrowleft": "←",
        "arrowup": "↑",
        "arrowright": "→",
        "arrowdown": "↓",
        "arrowleftright": "↔",
        "arrowupdown": "↕",
        "arrowNW": "↖",
        "arrowNE": "↗",
        "arrowSE": "↘",
        "arrowSW": "↙",
        "arrowleftstroke": "↚",
        "arrowrightstroke": "↛",
        "arrowleftwave": "↜",
        "arrowrightwave": "↝",
        "arrowlefttwoheaded": "↞",
        "arrowuptwoheaded": "↟",
        "arrowrighttwoheaded": "↠",
        "arrowdowntwoheaded": "↡",
        "arrowlefttail": "↢",
        "arrowrighttail": "↣",
        "arrowleftfrombar": "↤",
        "arrowupfrombar": "↥",
        "arrowrightfrombar": "↦",
        "arrowdownfrombar": "↧",
        "arrowupdownwithbase": "↨",
        "arrowlefthook": "↩",
        "arrowrighthook": "↪",
        "arrowleftloop": "↫",
        "arrowrightloop": "↬",
        "arrowleftrightwave": "↭",
        "arrowleftrightstroke": "↮",
        "arrowdownzigzag": "↯",
        "arrowleftuptip": "↰",
        "arrowrightuptip": "↱",
        "arrowleftdowntip": "↲",
        "arrowrightdowntip": "↳",
        "arrowrightdowncorner": "↴",
        "arrowleftdowncorner": "↵",
        "arrowanticlockwisesemicircle": "↶",
        "arrowclockwisesemicircle": "↷",
        "arrowlongNWtobar": "↸",
        "arrowleftoverrighttobar": "↹",
        "arrowanticlockwiseopencircle": "↺",
        "arrowclockwiseopencircle": "↻",
        "harpoonleftbarbup": "↼",
        "harpoonleftbarbdown": "↽",
        "harpoonupbarbright": "↾",
        "harpoonupbarbleft": "↿",
        "harpoonrightbarbup": "⇀",
        "harpoonrightbarbdown": "⇁",
        "harpoondownbarbright": "⇂",
        "harpoondownbarbleft": "⇃",
        "rightarrowoverleftarrow": "⇄",
        "uparrowleftofdownarrow": "⇅",
        "leftarrowoverrightarrow": "⇆",
        "arrowspairedleft": "⇇",
        "arrowspairedup": "⇈",
        "arrowspairedright": "⇉",
        "arrowspaireddown": "⇊",
        "leftharpoonoverrightharpoon": "⇋",
        "rightharpoonoverleftharpoon": "⇌",
        "dblarrowleftstroke": "⇍",
        "dblarrowleftrightstroke": "⇎",
        "dblarrowrightstroke": "⇏",
        "dblarrowleft": "⇐",
        "dblarrowup": "⇑",
        "dblarrowright": "⇒",
        "dblarrowdown": "⇓",
        "dblarrowleftright": "⇔",
        "dblarrowupdown": "⇕",
        "dblarrowNW": "⇖",
        "dblarrowNE": "⇗",
        "dblarrowSE": "⇘",
        "dblarrowSW": "⇙",
        "triplearrowleft": "⇚",
        "triplearrowright": "⇛",
        "arrowleftsquiggle": "⇜",
        "arrowrightsquiggle": "⇝",
        "dblstrokearrowup": "⇞",
        "dblstrokearrowdown": "⇟",
        "arrowleftdashed": "⇠",
        "arrowupdashed": "⇡",
        "arrowrightdashed": "⇢",
        "arrowdowndashed": "⇣",
        "arrowlefttobar": "⇤",
        "arrowrighttobar": "⇥",
        "whitearrowleft": "⇦",
        "whitearrowup": "⇧",
        "whitearrowright": "⇨",
        "whitearrowdown": "⇩",
        "whitearrowupfrombar": "⇪",
        "whitearrowonpedestalup": "⇫",
        "horizontalbarwhitearrowonpedestalup": "⇬",
        "verticalbarwhitearrowonpedestalup": "⇭",
        "whitedblarrowup": "⇮",
        "whitedblarrowonpedestalup": "⇯",
        "whitearrowfromwallright": "⇰",
        "tocornerarrowNW": "⇱",
        "tocornerarrowSE": "⇲",
        "whitearrowupdown": "⇳",
        "arrowrightsmallcircle": "⇴",
        "downwarrowleftofuparrow": "⇵",
        "threerightarrows": "⇶",
        "verticalstrokearrowleft": "⇷",
        "verticalstrokearrowright": "⇸",
        "verticalstrokearrowleftright": "⇹",
        "verticalsdbltrokearrowleft": "⇺",
        "verticalsdbltrokearrowright": "⇻",
        "verticalsdbltrokearrowleftright": "⇼",
        "openheadarrowleft": "⇽",
        "openheadarrowright": "⇾",
        "openheadarrowleftright": "⇿",
        "universal": "∀",
        "complement": "∁",
        "partialdiff": "∂",
        "existential": "∃",
        "notexistential": "∄",
        "emptyset": "∅",
        "increment": "∆",
        "gradient": "∇",
        "element": "∈",
        "notelement": "∉",
        "elementsmall": "∊",
        "suchthat": "∋",
        "notcontains": "∌",
        "containsasmembersmall": "∍",
        "endpro": "∎",
        "product": "∏",
        "coproductarray": "∐",
        "summation": "∑",
        "minus": "−",
        "minusplus": "∓",
        "dotplus": "∔",
        "divisionslash": "∕",
        "setminus": "∖",
        "asteriskmath": "∗",
        "ringoperator": "∘",
        "bulletoperator": "∙",
        "radical": "√",
        "math:cuberoot": "∛",
        "math:fourthroot": "∜",
        "proportional": "∝",
        "infinity": "∞",
        "orthogonal": "∟",
        "angle": "∠",
        "measuredangle": "∡",
        "sphericalangle": "∢",
        "divides": "∣",
        "doesnotdivide": "∤",
        "parallel": "∥",
        "notparallel": "∦",
        "logicaland": "∧",
        "logicalor": "∨",
        "intersection": "∩",
        "union": "∪",
        "integral": "∫",
        "integraldbl": "∬",
        "integraltpl": "∭",
        "integralcontour": "∮",
        "integralsurface": "∯",
        "integralvolume": "∰",
        "integralclockwise": "∱",
        "integralcontourclockwise": "∲",
        "integralcontouranticlockwise": "∳",
        "therefore": "∴",
        "because": "∵",
        "ratio": "∶",
        "proportion": "∷",
        "dotminus": "∸",
        "excess": "∹",
        "geometricproportion": "∺",
        "homotic": "∻",
        "similar": "∼",
        "tildereversed": "∽",
        "lazysinverted": "∾",
        "sinewave": "∿",
        "wreathproduct": "≀",
        "nottilde": "≁",
        "minustilde": "≂",
        "asympticallyequal": "≃",
        "notasympticallyequal": "≄",
        "congruent": "≅",
        "approximatelybutnotactuallyequal": "≆",
        "neirapproximatelynoractuallyequal": "≇",
        "approxequal": "≈",
        "notalmostequal": "≉",
        "almostequalorequal": "≊",
        "tildetpl": "≋",
        "allequal": "≌",
        "equivalent": "≍",
        "geometricallyequivalent": "≎",
        "differencebetween": "≏",
        "approacheslimit": "≐",
        "geometricallyequal": "≑",
        "approximatelyequalorimage": "≒",
        "imageorapproximatelyequal": "≓",
        "colonequals": "≔",
        "equalscolon": "≕",
        "ringinequal": "≖",
        "ringequal": "≗",
        "corresponds": "≘",
        "estimates": "≙",
        "equiangular": "≚",
        "starequals": "≛",
        "deltaequal": "≜",
        "equalbydefinition": "≝",
        "measuredby": "≞",
        "questionedequal": "≟",
        "notequal": "≠",
        "equivalence": "≡",
        "notidentical": "≢",
        "strictlyequivalent": "≣",
        "lessequal": "≤",
        "greaterequal": "≥",
        "lessoverequal": "≦",
        "greateroverequal": "≧",
        "lessbutnotequal": "≨",
        "greaterbutnotequal": "≩",
        "muchless": "≪",
        "muchgreater": "≫",
        "between": "≬",
        "notequivalent": "≭",
        "notless": "≮",
        "notgreater": "≯",
        "neirlessnorequal": "≰",
        "neirgreaternorequal": "≱",
        "lessorequivalent": "≲",
        "greaterorequivalent": "≳",
        "neirlessnorequivalent": "≴",
        "neirgreaternorequivalent": "≵",
        "lessorgreater": "≶",
        "greaterorless": "≷",
        "neirlessnorgreater": "≸",
        "neirgreaternorless": "≹",
        "precedes": "≺",
        "succeeds": "≻",
        "precedesorequal": "≼",
        "succeedsorequal": "≽",
        "precedesorequivalent": "≾",
        "succeedsorequivalent": "≿",
        "doesnotprecede": "⊀",
        "doesnotsucceed": "⊁",
        "propersubset": "⊂",
        "propersuperset": "⊃",
        "notsubset": "⊄",
        "notasersetup": "⊅",
        "reflexsubset": "⊆",
        "reflexsuperset": "⊇",
        "neirasubsetnorequal": "⊈",
        "neirasersetnorequalup": "⊉",
        "subsetnotequal": "⊊",
        "sersetnotequalup": "⊋",
        "multiset": "⊌",
        "multisetmultiplication": "⊍",
        "multisetunion": "⊎",
        "squareimage": "⊏",
        "squareoriginal": "⊐",
        "squareimageorequal": "⊑",
        "squareoriginalorequal": "⊒",
        "squarecap": "⊓",
        "squarecup": "⊔",
        "circleplus": "⊕",
        "circledminus": "⊖",
        "circlemultiply": "⊗",
        "circleddivisionslash": "⊘",
        "circleddotoperator": "⊙",
        "circledringoperator": "⊚",
        "circledasteriskoperator": "⊛",
        "circledequals": "⊜",
        "circleddash": "⊝",
        "squaredplus": "⊞",
        "squaredminus": "⊟",
        "squaredtimes": "⊠",
        "squareddotoperator": "⊡",
        "tackright": "⊢",
        "tackleft": "⊣",
        "tackdown": "⊤",
        "tackup": "⊥",
        "assertion": "⊦",
        "models": "⊧",
        "true": "⊨",
        "forces": "⊩",
        "turnstiletplverticalbarright": "⊪",
        "turnstiledblverticalbarright": "⊫",
        "doesnotprove": "⊬",
        "nottrue": "⊭",
        "doesnotforce": "⊮",
        "negatedturnstiledblverticalbarright": "⊯",
        "precedesunderrelation": "⊰",
        "succeedsunderrelation": "⊱",
        "normalsubgroup": "⊲",
        "containsasnormalsubgroup": "⊳",
        "normalsubgroorequalup": "⊴",
        "containsasnormalsubgroorequalup": "⊵",
        "original": "⊶",
        "image": "⊷",
        "multimap": "⊸",
        "hermitianconjugatematrix": "⊹",
        "intercalate": "⊺",
        "xor": "⊻",
        "nand": "⊼",
        "nor": "⊽",
        "anglearcright": "⊾",
        "triangleright": "⊿",
        "logicalandarray": "⋀",
        "logicalorarray": "⋁",
        "intersectionarray": "⋂",
        "unionarray": "⋃",
        "diamondoperator": "⋄",
        "dotmath": "⋅",
        "staroperator": "⋆",
        "divisiontimes": "⋇",
        "math:bowtie": "⋈",
        "normalfacrsemidirectproductleft": "⋉",
        "normalfacrsemidirectproductright": "⋊",
        "semidirectproductleft": "⋋",
        "semidirectproductright": "⋌",
        "tildeequalsreversed": "⋍",
        "curlylogicalor": "⋎",
        "curlylogicaland": "⋏",
        "subsetdbl": "⋐",
        "sersetdblup": "⋑",
        "intersectiondbl": "⋒",
        "uniondbl": "⋓",
        "pitchfork": "⋔",
        "equalandparallel": "⋕",
        "lessdot": "⋖",
        "greaterdot": "⋗",
        "verymuchless": "⋘",
        "verymuchgreater": "⋙",
        "lessequalorgreater": "⋚",
        "greaterequalorless": "⋛",
        "equalorless": "⋜",
        "equalorgreater": "⋝",
        "equalorprecedes": "⋞",
        "equalorsucceeds": "⋟",
        "doesnotprecedeorequal": "⋠",
        "doesnotsucceedorequal": "⋡",
        "notsquareimageorequal": "⋢",
        "notsquareoriginalorequal": "⋣",
        "squareimageornotequal": "⋤",
        "squareoriginalornotequal": "⋥",
        "lessbutnotequivalent": "⋦",
        "greaterbutnotequivalent": "⋧",
        "precedesbutnotequivalent": "⋨",
        "succeedsbutnotequivalent": "⋩",
        "notnormalsubgroup": "⋪",
        "doesnotcontainasnormalsubgroup": "⋫",
        "notnormalsubgroorequalup": "⋬",
        "doesnotcontainasnormalsubgroorequalup": "⋭",
        "ellipsisvertical": "⋮",
        "ellipsismidhorizontal": "⋯",
        "ellipsisdiagonalupright": "⋰",
        "ellipsisdiagonaldownright": "⋱",
        "elementlonghorizontalstroke": "⋲",
        "elementverticalbarhorizontalstroke": "⋳",
        "elementsmallverticalbarhorizontalstroke": "⋴",
        "elementdotabove": "⋵",
        "elementoverbar": "⋶",
        "elementoverbarsmall": "⋷",
        "elementunderbar": "⋸",
        "elementtwoshorizontalstroke": "⋹",
        "containslonghorizontalstroke": "⋺",
        "containsverticalbarhorizontalstroke": "⋻",
        "containssmallverticalbarhorizontalstroke": "⋼",
        "containsoverbar": "⋽",
        "containsoverbarsmall": "⋾",
        "znotationbagmembership": "⋿",
        "diametersign": "⌀",
        "electricarrow": "⌁",
        "house": "⌂",
        "arrowheadup": "⌃",
        "arrowheaddown": "⌄",
        "projective": "⌅",
        "perspective": "⌆",
        "wavyline": "⌇",
        "ceilingleft": "⌈",
        "ceilingright": "⌉",
        "floorleft": "⌊",
        "floorright": "⌋",
        "cropbottomright": "⌌",
        "cropbottomleft": "⌍",
        "croptopright": "⌎",
        "croptopleft": "⌏",
        "revlogicalnot": "⌐",
        "lozengesquare": "⌑",
        "arc": "⌒",
        "segment": "⌓",
        "sector": "⌔",
        "telephonerecorder": "⌕",
        "positionindicator": "⌖",
        "viewdatasquare": "⌗",
        "placeofinterestsign": "⌘",
        "notsignturned": "⌙",
        "watch": "⌚",
        "hourglass": "⌛",
        "cornertopleft": "⌜",
        "cornertopright": "⌝",
        "cornerbottomleft": "⌞",
        "cornerbottomright": "⌟",
        "integraltp": "⌠",
        "integralbt": "⌡",
        "frown": "⌢",
        "smile": "⌣",
        "arrowheadtwobarsuphorizontal": "⌤",
        "option": "⌥",
        "eraseright": "⌦",
        "clear": "⌧",
        "board": "⌨",
        "angleleft": "〈",
        "angleright": "〉",
        "eraseleft": "⌫",
        "benzenering": "⌬",
        "cylindricity": "⌭",
        "allaroundprofile": "⌮",
        "symmetry": "⌯",
        "totalrunout": "⌰",
        "dimensionorigin": "⌱",
        "conicaltaper": "⌲",
        "slope": "⌳",
        "counterbore": "⌴",
        "countersink": "⌵",
        "beamfunc": "⌶",
        "squishquadfunc": "⌷",
        "quadequalfunc": "⌸",
        "quaddividefunc": "⌹",
        "quaddiamondfunc": "⌺",
        "quadjotfunc": "⌻",
        "quadcirclefunc": "⌼",
        "circlestilefunc": "⌽",
        "circlejotfunc": "⌾",
        "slashbarfunc": "⌿",
        "backslashbarfunc": "⍀",
        "quadslashfunc": "⍁",
        "quadbackslashfunc": "⍂",
        "quadlessfunc": "⍃",
        "quadgreaterfunc": "⍄",
        "vaneleftfunc": "⍅",
        "vanerightfunc": "⍆",
        "quadarrowleftfunc": "⍇",
        "quadarrowrightfunc": "⍈",
        "circlebackslashfunc": "⍉",
        "tackunderlinedownfunc": "⍊",
        "deltastilefunc": "⍋",
        "quadcaretdownfunc": "⍌",
        "quaddeltafunc": "⍍",
        "tackjotdownfunc": "⍎",
        "vaneupfunc": "⍏",
        "quadarrowupfunc": "⍐",
        "tackoverbarupfunc": "⍑",
        "delstilefunc": "⍒",
        "quadcaretupfunc": "⍓",
        "quaddelfunc": "⍔",
        "tackjotupfunc": "⍕",
        "vanedownfunc": "⍖",
        "quadarrowdownfunc": "⍗",
        "quoteunderlinefunc": "⍘",
        "deltaunderlinefunc": "⍙",
        "diamondunderlinefunc": "⍚",
        "jotunderlinefunc": "⍛",
        "circleunderlinefunc": "⍜",
        "shoejotupfunc": "⍝",
        "quotequadfunc": "⍞",
        "circlestarfunc": "⍟",
        "quadcolonfunc": "⍠",
        "tackdiaeresisupfunc": "⍡",
        "deldiaeresisfunc": "⍢",
        "stardiaeresisfunc": "⍣",
        "jotdiaeresisfunc": "⍤",
        "circlediaeresisfunc": "⍥",
        "shoestiledownfunc": "⍦",
        "shoestileleftfunc": "⍧",
        "tildediaeresisfunc": "⍨",
        "diaeresisgreaterfunc": "⍩",
        "commabarfunc": "⍪",
        "deltildefunc": "⍫",
        "zildefunc": "⍬",
        "stiletildefunc": "⍭",
        "semicolonunderlinefunc": "⍮",
        "quadnotequalfunc": "⍯",
        "quadquestionfunc": "⍰",
        "carettildedownfunc": "⍱",
        "carettildeupfunc": "⍲",
        "iotafunc": "⍳",
        "rhofunc": "⍴",
        "omegafunc": "⍵",
        "alphaunderlinefunc": "⍶",
        "epsilonunderlinefunc": "⍷",
        "iotaunderlinefunc": "⍸",
        "omegaunderlinefunc": "⍹",
        "alphafunc": "⍺",
        "notcheckmark": "⍻",
        "anglezigzagarrowdownright": "⍼",
        "shoulderedopenbox": "⍽",
        "misc:bell": "🔔",
        "linemiddledotvertical": "⍿",
        "insertion": "⎀",
        "continuousunderline": "⎁",
        "discontinuousunderline": "⎂",
        "emphasis": "⎃",
        "composition": "⎄",
        "centrelineverticalsquarewhite": "⎅",
        "enter": "⎆",
        "alternative": "⎇",
        "helm": "⎈",
        "circledbarnotchhorizontal": "⎉",
        "circledtriangledown": "⎊",
        "brokencirclenorthwestarrow": "⎋",
        "undo": "⎌",
        "monostable": "⎍",
        "hysteresis": "⎎",
        "htypeopencircuit": "⎏",
        "ltypeopencircuit": "⎐",
        "passivedown": "⎑",
        "outputpassiveup": "⎒",
        "directcurrentformtwo": "⎓",
        "softwarefunction": "⎔",
        "quadfunc": "⎕",
        "misc:decimalseparator": "⎖",
        "previouspage": "⎗",
        "nextpage": "⎘",
        "printscreen": "⎙",
        "clearscreen": "⎚",
        "parenhookupleft": "⎛",
        "parenextensionleft": "⎜",
        "parenlowerhookleft": "⎝",
        "parenhookupright": "⎞",
        "parenextensionright": "⎟",
        "parenlowerhookright": "⎠",
        "bracketcornerupleftsquare": "⎡",
        "bracketextensionleftsquare": "⎢",
        "bracketlowercornerleftsquare": "⎣",
        "bracketcorneruprightsquare": "⎤",
        "bracketextensionrightsquare": "⎥",
        "bracketlowercornerrightsquare": "⎦",
        "brackethookupleftcurly": "⎧",
        "bracketmiddlepieceleftcurly": "⎨",
        "bracketlowerhookleftcurly": "⎩",
        "bracketextensioncurly": "⎪",
        "brackethookuprightcurly": "⎫",
        "bracketmiddlepiecerightcurly": "⎬",
        "bracketlowerhookrightcurly": "⎭",
        "integralextension": "⎮",
        "lineextensionhorizontal": "⎯",
        "bracketsectionupleftlowerrightcurly": "⎰",
        "bracketsectionuprightlowerleftcurly": "⎱",
        "summationtop": "⎲",
        "summationbottom": "⎳",
        "brackettopsquare": "⎴",
        "bracketbottomsquare": "⎵",
        "bracketoverbrackettopbottomsquare": "⎶",
        "radicalbottom": "⎷",
        "boxlineverticalleft": "⎸",
        "boxlineverticalright": "⎹",
        "scanonehorizontal": "⎺",
        "scanthreehorizontal": "⎻",
        "scansevenhorizontal": "⎼",
        "scanninehorizontal": "⎽",
        "dentistrytopverticalright": "⎾",
        "dentistrybottomverticalright": "⎿",
        "dentistrycirclevertical": "⏀",
        "dentistrycircledownhorizontal": "⏁",
        "dentistrycircleuphorizontal": "⏂",
        "dentistrytrianglevertical": "⏃",
        "dentistrytriangledownhorizontal": "⏄",
        "dentistrytriangleuphorizontal": "⏅",
        "dentistrywavevertical": "⏆",
        "dentistrywavedownhorizontal": "⏇",
        "dentistrywaveuphorizontal": "⏈",
        "dentistrydownhorizontal": "⏉",
        "dentistryuphorizontal": "⏊",
        "dentistrytopverticalleft": "⏋",
        "dentistrybottomverticalleft": "⏌",
        "footsquare": "⏍",
        "return": "⏎",
        "eject": "⏏",
        "lineextensionvertical": "⏐",
        "brevemetrical": "⏑",
        "longovershortmetrical": "⏒",
        "shortoverlongmetrical": "⏓",
        "longovertwoshortsmetrical": "⏔",
        "twoshortsoverlongmetrical": "⏕",
        "twoshortsjoinedmetrical": "⏖",
        "trisememetrical": "⏗",
        "tetrasememetrical": "⏘",
        "pentasememetrical": "⏙",
        "earthground": "⏚",
        "fuse": "⏛",
        "parentop": "⏜",
        "parenbottom": "⏝",
        "brackettopcurly": "⏞",
        "bracketbottomcurly": "⏟",
        "bracketshelltop": "⏠",
        "bracketshellbottom": "⏡",
        "trapeziumwhite": "⏢",
        "benzeneringcircle": "⏣",
        "straightness": "⏤",
        "flatness": "⏥",
        "accurrent": "⏦",
        "electricalintersection": "⏧",
        "decimalexponent": "⏨",
        "blackpointingdoubletriangleright": "⏩",
        "blackpointingdoubletriangleleft": "⏪",
        "blackpointingdoubletriangleup": "⏫",
        "blackpointingdoubletriangledown": "⏬",
        "blackpointingdoubletrianglebarverticalright": "⏭",
        "blackpointingdoubletrianglebarverticalleft": "⏮",
        "blackpointingtriangledoublebarverticalright": "⏯",
        "alarmclock": "⏰",
        "swatchtop": "⏱",
        "timerclock": "⏲",
        "hourglassflowings": "⏳",
        "blackmediumpointingtriangleleft": "⏴",
        "blackmediumpointingtriangleright": "⏵",
        "blackmediumpointingtriangleup": "⏶",
        "blackmediumpointingtriangledown": "⏷",
        "doublebarvertical": "⏸",
        "blackforstopsquare": "⏹",
        "blackcircleforrecord": "⏺",
        "power": "⏻",
        "poweronoff": "⏼",
        "poweron": "⏽",
        "powersleep": "⏾",
        "observereye": "⏿",
        "cntr:null": "␀",
        "cntr:startofheading": "␁",
        "cntr:startoftext": "␂",
        "cntr:endoftext": "␃",
        "cntr:endoftransmission": "␄",
        "cntr:enquiry": "␅",
        "cntr:acknowledge": "␆",
        "cntr:bell": "␇",
        "cntr:backspace": "␈",
        "cntr:horizontaltab": "␉",
        "cntr:linefeed": "␊",
        "cntr:verticaltab": "␋",
        "cntr:formfeed": "␌",
        "cntr:carriagereturn": "␍",
        "cntr:shiftout": "␎",
        "cntr:shiftin": "␏",
        "cntr:datalinkescape": "␐",
        "cntr:devicecontrolone": "␑",
        "cntr:devicecontroltwo": "␒",
        "cntr:devicecontrolthree": "␓",
        "cntr:devicecontrolfour": "␔",
        "cntr:negativeacknowledge": "␕",
        "cntr:synchronousidle": "␖",
        "cntr:endoftransmissionblock": "␗",
        "cntr:cancel": "␘",
        "cntr:endofmedium": "␙",
        "cntr:substitute": "␚",
        "cntr:escape": "␛",
        "cntr:fileseparator": "␜",
        "cntr:groupseparator": "␝",
        "cntr:recordseparator": "␞",
        "cntr:unitseparator": "␟",
        "cntr:space": "␠",
        "cntr:delete": "␡",
        "cntr:blank": "␢",
        "cntr:openbox": "␣",
        "cntr:newline": "␤",
        "cntr:deleteformtwo": "␥",
        "cntr:substituteformtwo": "␦",
        "hook": "⑀",
        "ocr:chair": "⑁",
        "fork": "⑂",
        "invertedfork": "⑃",
        "beltbuckle": "⑄",
        "ocr:bowtie": "⑅",
        "branchbankidentification": "⑆",
        "amountofcheck": "⑇",
        "ocr:dash": "⑈",
        "customeraccountnumber": "⑉",
        "backslashdbl": "⑊",
        "onecircle": "①",
        "twocircle": "②",
        "threecircle": "③",
        "fourcircle": "④",
        "fivecircle": "⑤",
        "sixcircle": "⑥",
        "sevencircle": "⑦",
        "eightcircle": "⑧",
        "ninecircle": "⑨",
        "tencircle": "⑩",
        "elevencircle": "⑪",
        "twelvecircle": "⑫",
        "thirteencircle": "⑬",
        "fourteencircle": "⑭",
        "fifteencircle": "⑮",
        "sixteencircle": "⑯",
        "seventeencircle": "⑰",
        "eighteencircle": "⑱",
        "nineteencircle": "⑲",
        "twentycircle": "⑳",
        "oneparenthesized": "⑴",
        "twoparenthesized": "⑵",
        "threeparenthesized": "⑶",
        "fourparenthesized": "⑷",
        "fiveparenthesized": "⑸",
        "sixparenthesized": "⑹",
        "sevenparenthesized": "⑺",
        "eightparenthesized": "⑻",
        "nineparenthesized": "⑼",
        "tenparenthesized": "⑽",
        "elevenparenthesized": "⑾",
        "twelveparenthesized": "⑿",
        "thirteenparenthesized": "⒀",
        "fourteenparenthesized": "⒁",
        "fifteenparenthesized": "⒂",
        "sixteenparenthesized": "⒃",
        "seventeenparenthesized": "⒄",
        "eighteenparenthesized": "⒅",
        "nineteenparenthesized": "⒆",
        "twentyparenthesized": "⒇",
        "oneperiod": "⒈",
        "twoperiod": "⒉",
        "threeperiod": "⒊",
        "fourperiod": "⒋",
        "fiveperiod": "⒌",
        "sixperiod": "⒍",
        "sevenperiod": "⒎",
        "eightperiod": "⒏",
        "nineperiod": "⒐",
        "tenperiod": "⒑",
        "elevenperiod": "⒒",
        "twelveperiod": "⒓",
        "thirteenperiod": "⒔",
        "fourteenperiod": "⒕",
        "fifteenperiod": "⒖",
        "sixteenperiod": "⒗",
        "seventeenperiod": "⒘",
        "eighteenperiod": "⒙",
        "nineteenperiod": "⒚",
        "twentyperiod": "⒛",
        "aparenthesized": "⒜",
        "bparenthesized": "⒝",
        "cparenthesized": "⒞",
        "dparenthesized": "⒟",
        "eparenthesized": "⒠",
        "fparenthesized": "⒡",
        "gparenthesized": "⒢",
        "hparenthesized": "⒣",
        "iparenthesized": "⒤",
        "jparenthesized": "⒥",
        "kparenthesized": "⒦",
        "lparenthesized": "⒧",
        "mparenthesized": "⒨",
        "nparenthesized": "⒩",
        "oparenthesized": "⒪",
        "pparenthesized": "⒫",
        "qparenthesized": "⒬",
        "rparenthesized": "⒭",
        "sparenthesized": "⒮",
        "tparenthesized": "⒯",
        "uparenthesized": "⒰",
        "vparenthesized": "⒱",
        "wparenthesized": "⒲",
        "xparenthesized": "⒳",
        "yparenthesized": "⒴",
        "zparenthesized": "⒵",
        "Acircle": "Ⓐ",
        "Bcircle": "Ⓑ",
        "Ccircle": "Ⓒ",
        "Dcircle": "Ⓓ",
        "Ecircle": "Ⓔ",
        "Fcircle": "Ⓕ",
        "Gcircle": "Ⓖ",
        "Hcircle": "Ⓗ",
        "Icircle": "Ⓘ",
        "Jcircle": "Ⓙ",
        "Kcircle": "Ⓚ",
        "Lcircle": "Ⓛ",
        "Mcircle": "Ⓜ",
        "Ncircle": "Ⓝ",
        "Ocircle": "Ⓞ",
        "Pcircle": "Ⓟ",
        "Qcircle": "Ⓠ",
        "Rcircle": "Ⓡ",
        "Scircle": "Ⓢ",
        "Tcircle": "Ⓣ",
        "Ucircle": "Ⓤ",
        "Vcircle": "Ⓥ",
        "Wcircle": "Ⓦ",
        "Xcircle": "Ⓧ",
        "Ycircle": "Ⓨ",
        "Zcircle": "Ⓩ",
        "acircle": "ⓐ",
        "bcircle": "ⓑ",
        "ccircle": "ⓒ",
        "dcircle": "ⓓ",
        "ecircle": "ⓔ",
        "fcircle": "ⓕ",
        "gcircle": "ⓖ",
        "hcircle": "ⓗ",
        "icircle": "ⓘ",
        "jcircle": "ⓙ",
        "kcircle": "ⓚ",
        "lcircle": "ⓛ",
        "mcircle": "ⓜ",
        "ncircle": "ⓝ",
        "ocircle": "ⓞ",
        "pcircle": "ⓟ",
        "qcircle": "ⓠ",
        "rcircle": "ⓡ",
        "scircle": "ⓢ",
        "tcircle": "ⓣ",
        "ucircle": "ⓤ",
        "vcircle": "ⓥ",
        "wcircle": "ⓦ",
        "xcircle": "ⓧ",
        "ycircle": "ⓨ",
        "zcircle": "ⓩ",
        "zerocircle": "⓪",
        "elevencircleblack": "⓫",
        "twelvecircleblack": "⓬",
        "thirteencircleblack": "⓭",
        "fourteencircleblack": "⓮",
        "fifteencircleblack": "⓯",
        "sixteencircleblack": "⓰",
        "seventeencircleblack": "⓱",
        "eighteencircleblack": "⓲",
        "nineteencircleblack": "⓳",
        "twentycircleblack": "⓴",
        "onecircledbl": "⓵",
        "twocircledbl": "⓶",
        "threecircledbl": "⓷",
        "fourcircledbl": "⓸",
        "fivecircledbl": "⓹",
        "sixcircledbl": "⓺",
        "sevencircledbl": "⓻",
        "eightcircledbl": "⓼",
        "ninecircledbl": "⓽",
        "tencircledbl": "⓾",
        "zerocircleblack": "⓿",
        "lighthorz": "─",
        "heavyhorz": "━",
        "lightvert": "│",
        "heavyvert": "┃",
        "lighttrpldashhorz": "┄",
        "heavytrpldashhorz": "┅",
        "lighttrpldashvert": "┆",
        "heavytrpldashvert": "┇",
        "lightquaddashhorz": "┈",
        "heavyquaddashhorz": "┉",
        "lightquaddashvert": "┊",
        "heavyquaddashvert": "┋",
        "lightdnright": "┌",
        "dnlightrightheavy": "┍",
        "dnheavyrightlight": "┎",
        "heavydnright": "┏",
        "lightdnleft": "┐",
        "dnlightleftheavy": "┑",
        "dnheavyleftlight": "┒",
        "heavydnleft": "┓",
        "lightupright": "└",
        "uplightrightheavy": "┕",
        "upheavyrightlight": "┖",
        "heavyupright": "┗",
        "lightupleft": "┘",
        "uplightleftheavy": "┙",
        "upheavyleftlight": "┚",
        "heavyupleft": "┛",
        "lightvertright": "├",
        "vertlightrightheavy": "┝",
        "upheavyrightdnlight": "┞",
        "dnheavyrightuplight": "┟",
        "vertheavyrightlight": "┠",
        "dnlightrightupheavy": "┡",
        "uplightrightdnheavy": "┢",
        "heavyvertright": "┣",
        "lightvertleft": "┤",
        "vertlightleftheavy": "┥",
        "upheavyleftdnlight": "┦",
        "dnheavyleftuplight": "┧",
        "vertheavyleftlight": "┨",
        "dnlightleftupheavy": "┩",
        "uplightleftdnheavy": "┪",
        "heavyvertleft": "┫",
        "lightdnhorz": "┬",
        "leftheavyrightdnlight": "┭",
        "rightheavyleftdnlight": "┮",
        "dnlighthorzheavy": "┯",
        "dnheavyhorzlight": "┰",
        "rightlightleftdnheavy": "┱",
        "leftlightrightdnheavy": "┲",
        "heavydnhorz": "┳",
        "lightuphorz": "┴",
        "leftheavyrightuplight": "┵",
        "rightheavyleftuplight": "┶",
        "uplighthorzheavy": "┷",
        "upheavyhorzlight": "┸",
        "rightlightleftupheavy": "┹",
        "leftlightrightupheavy": "┺",
        "heavyuphorz": "┻",
        "lightverthorz": "┼",
        "leftheavyrightvertlight": "┽",
        "rightheavyleftvertlight": "┾",
        "vertlighthorzheavy": "┿",
        "upheavydnhorzlight": "╀",
        "dnheavyuphorzlight": "╁",
        "vertheavyhorzlight": "╂",
        "leftupheavyrightdnlight": "╃",
        "rightupheavyleftdnlight": "╄",
        "leftdnheavyrightuplight": "╅",
        "rightdnheavyleftuplight": "╆",
        "dnlightuphorzheavy": "╇",
        "uplightdnhorzheavy": "╈",
        "rightlightleftvertheavy": "╉",
        "leftlightrightvertheavy": "╊",
        "heavyverthorz": "╋",
        "lightdbldashhorz": "╌",
        "heavydbldashhorz": "╍",
        "lightdbldashvert": "╎",
        "heavydbldashvert": "╏",
        "dblhorz": "═",
        "dblvert": "║",
        "dnsngrightdbl": "╒",
        "dndblrightsng": "╓",
        "dbldnright": "╔",
        "dnsngleftdbl": "╕",
        "dndblleftsng": "╖",
        "dbldnleft": "╗",
        "upsngrightdbl": "╘",
        "updblrightsng": "╙",
        "dblupright": "╚",
        "upsngleftdbl": "╛",
        "updblleftsng": "╜",
        "dblupleft": "╝",
        "vertsngrightdbl": "╞",
        "vertdblrightsng": "╟",
        "dblvertright": "╠",
        "vertsngleftdbl": "╡",
        "vertdblleftsng": "╢",
        "dblvertleft": "╣",
        "dnsnghorzdbl": "╤",
        "dndblhorzsng": "╥",
        "dbldnhorz": "╦",
        "upsnghorzdbl": "╧",
        "updblhorzsng": "╨",
        "dbluphorz": "╩",
        "vertsnghorzdbl": "╪",
        "vertdblhorzsng": "╫",
        "dblverthorz": "╬",
        "lightarcdnright": "╭",
        "lightarcdnleft": "╮",
        "lightarcupleft": "╯",
        "lightarcupright": "╰",
        "lightdiaguprightdnleft": "╱",
        "lightdiagupleftdnright": "╲",
        "lightdiagcross": "╳",
        "lightleft": "╴",
        "lightup": "╵",
        "lightright": "╶",
        "lightdn": "╷",
        "heavyleft": "╸",
        "heavyup": "╹",
        "heavyright": "╺",
        "heavydn": "╻",
        "lightleftheavyright": "╼",
        "lightupheavydn": "╽",
        "heavyleftlightright": "╾",
        "heavyuplightdn": "╿",
        "upperHalfBlock": "▀",
        "lowerOneEighthBlock": "▁",
        "lowerOneQuarterBlock": "▂",
        "lowerThreeEighthsBlock": "▃",
        "lowerHalfBlock": "▄",
        "lowerFiveEighthsBlock": "▅",
        "lowerThreeQuartersBlock": "▆",
        "lowerSevenEighthsBlock": "▇",
        "fullBlock": "█",
        "leftSevenEighthsBlock": "▉",
        "leftThreeQuartersBlock": "▊",
        "leftFiveEighthsBlock": "▋",
        "leftHalfBlock": "▌",
        "leftThreeEighthsBlock": "▍",
        "leftOneQuarterBlock": "▎",
        "leftOneEighthBlock": "▏",
        "rightHalfBlock": "▐",
        "lightShade": "░",
        "mediumShade": "▒",
        "darkShade": "▓",
        "upperOneEighthBlock": "▔",
        "rightOneEighthBlock": "▕",
        "quadrantLowerLeft": "▖",
        "quadrantLowerRight": "▗",
        "quadrantUpperLeft": "▘",
        "quadrantUpperLeftAndLowerLeftAndLowerRight": "▙",
        "quadrantUpperLeftAndLowerRight": "▚",
        "quadrantUpperLeftAndUpperRightAndLowerLeft": "▛",
        "quadrantUpperLeftAndUpperRightAndLowerRight": "▜",
        "quadrantUpperRight": "▝",
        "quadrantUpperRightAndLowerLeft": "▞",
        "quadrantUpperRightAndLowerLeftAndLowerRight": "▟",
        "squareblack": "■",
        "squarewhite": "□",
        "squarewhiteround": "▢",
        "squarewhitewithsquaresmallblack": "▣",
        "squarehorizontalfill": "▤",
        "squareverticalfill": "▥",
        "squareorthogonalcrosshatchfill": "▦",
        "squareupperlefttolowerrightfill": "▧",
        "squareupperrighttolowerleftfill": "▨",
        "squarediagonalcrosshatchfill": "▩",
        "squaresmallblack": "▪",
        "squaresmallwhite": "▫",
        "rectangleblack": "▬",
        "rectanglewhite": "▭",
        "rectangleverticalblack": "▮",
        "rectangleverticalwhite": "▯",
        "parallelogramblack": "▰",
        "parallelogramwhite": "▱",
        "triangleupblack": "▲",
        "triangleupwhite": "△",
        "triangleupsmallblack": "▴",
        "triangleupsmallwhite": "▵",
        "trianglerightblack": "▶",
        "trianglerightwhite": "▷",
        "trianglerightsmallblack": "▸",
        "trianglerightsmallwhite": "▹",
        "pointerrightblack": "►",
        "pointerrightwhite": "▻",
        "triangledownblack": "▼",
        "triangledownwhite": "▽",
        "triangledownsmallblack": "▾",
        "triangledownsmallwhite": "▿",
        "triangleleftblack": "◀",
        "triangleleftwhite": "◁",
        "triangleleftsmallblack": "◂",
        "triangleleftsmallwhite": "◃",
        "pointerleftblack": "◄",
        "pointerleftwhite": "◅",
        "gmtr:diamondblack": "◆",
        "gmtr:diamondwhite": "◇",
        "diamondwhitewithdiamondsmallblack": "◈",
        "fisheye": "◉",
        "lozenge": "◊",
        "circlewhite": "○",
        "circledotted": "◌",
        "circleverticalfill": "◍",
        "bullseye": "◎",
        "circleblack": "●",
        "circlehalfleftblack": "◐",
        "circlehalfrightblack": "◑",
        "circlelowerhalfblack": "◒",
        "circleupperhalfblack": "◓",
        "circleupperquadrantrightblack": "◔",
        "circleallbutupperquadrantleftblack": "◕",
        "halfcircleleftblack": "◖",
        "halfcirclerightblack": "◗",
        "bulletinverse": "◘",
        "circleinversewhite": "◙",
        "upperhalfcircleinversewhite": "◚",
        "lowerhalfcircleinversewhite": "◛",
        "upperquadrantcirculararcleft": "◜",
        "upperquadrantcirculararcright": "◝",
        "lowerquadrantcirculararcright": "◞",
        "lowerquadrantcirculararcleft": "◟",
        "upperhalfcircle": "◠",
        "lowerhalfcircle": "◡",
        "lowertrianglerightblack": "◢",
        "lowertriangleleftblack": "◣",
        "uppertriangleleftblack": "◤",
        "uppertrianglerightblack": "◥",
        "openbullet": "◦",
        "squarehalfleftblack": "◧",
        "squarehalfrightblack": "◨",
        "squareupperdiagonalhalfleftblack": "◩",
        "squarelowerdiagonalhalfrightblack": "◪",
        "squarewhitebisectinglinevertical": "◫",
        "triangledotupwhite": "◬",
        "trianglehalfupleftblack": "◭",
        "trianglehalfuprightblack": "◮",
        "largecircle": "◯",
        "squarewhiteupperquadrantleft": "◰",
        "squarewhitelowerquadrantleft": "◱",
        "squarewhitelowerquadrantright": "◲",
        "squarewhiteupperquadrantright": "◳",
        "circleupperquadrantleftwhite": "◴",
        "circlelowerquadrantleftwhite": "◵",
        "circlelowerquadrantrightwhite": "◶",
        "circleupperquadrantrightwhite": "◷",
        "uppertriangleleft": "◸",
        "uppertriangleright": "◹",
        "lowertriangleleft": "◺",
        "squaremediumwhite": "◻",
        "squaremediumblack": "◼",
        "squaresmallmediumwhite": "◽",
        "squaresmallmediumblack": "◾",
        "lowertriangleright": "◿",
        "sunraysblack": "☀",
        "cloud": "☁",
        "umbrella": "☂",
        "snowman": "☃",
        "comet": "☄",
        "starblack": "★",
        "starwhite": "☆",
        "lightning": "☇",
        "thunderstorm": "☈",
        "sun": "☉",
        "nodeascending": "☊",
        "nodedescending": "☋",
        "conjunction": "☌",
        "opposition": "☍",
        "telephoneblack": "☎",
        "telephonewhite": "☏",
        "checkbox": "☐",
        "checkboxchecked": "☑",
        "checkboxx": "☒",
        "saltire": "☓",
        "umbrellaraindrops": "☔",
        "hotbeverage": "☕",
        "shogipiecewhite": "☖",
        "shogipieceblack": "☗",
        "shamrock": "☘",
        "floralheartbulletreversedrotated": "☙",
        "pointingindexleftblack": "☚",
        "pointingindexrightblack": "☛",
        "pointingindexleftwhite": "☜",
        "pointingindexupwhite": "☝",
        "pointingindexrightwhite": "☞",
        "pointingindexdownwhite": "☟",
        "skullcrossbones": "☠",
        "caution": "☡",
        "radioactive": "☢",
        "biohazard": "☣",
        "caduceus": "☤",
        "ankh": "☥",
        "orthodoxcross": "☦",
        "chirho": "☧",
        "crossoflorraine": "☨",
        "crossofjerusalem": "☩",
        "starcrescent": "☪",
        "farsi": "☫",
        "adishakti": "☬",
        "hammersickle": "☭",
        "peace": "☮",
        "yinyang": "☯",
        "trigramheaven": "☰",
        "trigramlake": "☱",
        "trigramfire": "☲",
        "trigramthunder": "☳",
        "trigramwind": "☴",
        "trigramwater": "☵",
        "trigrammountain": "☶",
        "trigramearth": "☷",
        "wheelofdharma": "☸",
        "frowningfacewhite": "☹",
        "smilingfacewhite": "☺",
        "smilingfaceblack": "☻",
        "sunrayswhite": "☼",
        "firstquartermoon": "☽",
        "lastquartermoon": "☾",
        "mercury": "☿",
        "female": "♀",
        "earth": "♁",
        "male": "♂",
        "jiterup": "♃",
        "saturn": "♄",
        "uranus": "♅",
        "neptune": "♆",
        "pluto": "♇",
        "aries": "♈",
        "taurus": "♉",
        "gemini": "♊",
        "cancer": "♋",
        "leo": "♌",
        "virgo": "♍",
        "libra": "♎",
        "scorpius": "♏",
        "sagittarius": "♐",
        "capricorn": "♑",
        "aquarius": "♒",
        "pisces": "♓",
        "kingwhite": "♔",
        "queenwhite": "♕",
        "rookwhite": "♖",
        "bishopwhite": "♗",
        "knightwhite": "♘",
        "pawnwhite": "♙",
        "kingblack": "♚",
        "queenblack": "♛",
        "rookblack": "♜",
        "bishopblack": "♝",
        "knightblack": "♞",
        "pawnblack": "♟",
        "spadeblack": "♠",
        "heartwhite": "♡",
        "misc:diamondwhite": "♢",
        "clubblack": "♣",
        "spadewhite": "♤",
        "heartblack": "♥",
        "misc:diamondblack": "♦",
        "clubwhite": "♧",
        "hotsprings": "♨",
        "quarternote": "♩",
        "eighthnote": "♪",
        "beamedeighthnotes": "♫",
        "beamedsixteenthnotes": "♬",
        "musicflat": "♭",
        "musicnatural": "♮",
        "musicsharp": "♯",
        "westsyriaccross": "♰",
        "eastsyriaccross": "♱",
        "recycleuniversal": "♲",
        "recycleoneplastics": "♳",
        "recycletwoplastics": "♴",
        "recyclethreeplastics": "♵",
        "recyclefourplastics": "♶",
        "recyclefiveplastics": "♷",
        "recyclesixplastics": "♸",
        "recyclesevenplastics": "♹",
        "recyclegeneric": "♺",
        "recycleuniversalblack": "♻",
        "recycledpaper": "♼",
        "recyclepartiallypaper": "♽",
        "permanentpaper": "♾",
        "wheelchair": "♿",
        "dieone": "⚀",
        "dietwo": "⚁",
        "diethree": "⚂",
        "diefour": "⚃",
        "diefive": "⚄",
        "diesix": "⚅",
        "circledotrightwhite": "⚆",
        "circletwodotswhite": "⚇",
        "circlewhitedotrightblack": "⚈",
        "circletwodotsblackwhite": "⚉",
        "monogramyang": "⚊",
        "monogramyin": "⚋",
        "digramgreateryang": "⚌",
        "digramlesseryin": "⚍",
        "digramlesseryang": "⚎",
        "digramgreateryin": "⚏",
        "flagwhite": "⚐",
        "flagblack": "⚑",
        "hammerpick": "⚒",
        "anchor": "⚓",
        "crossedswords": "⚔",
        "staffofaesculapius": "⚕",
        "scales": "⚖",
        "alembic": "⚗",
        "flower": "⚘",
        "gear": "⚙",
        "staffofhermes": "⚚",
        "atom": "⚛",
        "fleurdelis": "⚜",
        "staroutlinedwhite": "⚝",
        "threelinesconvergingright": "⚞",
        "threelinesconvergingleft": "⚟",
        "warning": "⚠",
        "highvoltage": "⚡",
        "dfemaledbl": "⚢",
        "dmaledbl": "⚣",
        "interlockedfemalemale": "⚤",
        "malefemale": "⚥",
        "malestroke": "⚦",
        "malestrokemalefemale": "⚧",
        "verticalmalestroke": "⚨",
        "horizontalmalestroke": "⚩",
        "mediumcirclewhite": "⚪",
        "mediumcircleblack": "⚫",
        "mediumsmallcirclewhite": "⚬",
        "marriage": "⚭",
        "divorce": "⚮",
        "unmarriedpartnership": "⚯",
        "coffin": "⚰",
        "funeralurn": "⚱",
        "neuter": "⚲",
        "ceres": "⚳",
        "pallas": "⚴",
        "juno": "⚵",
        "vesta": "⚶",
        "chiron": "⚷",
        "moonlilithblack": "⚸",
        "sextile": "⚹",
        "semisextile": "⚺",
        "quincunx": "⚻",
        "sesquiquadrate": "⚼",
        "soccerball": "⚽",
        "baseball": "⚾",
        "squaredkey": "⚿",
        "draughtsmanwhite": "⛀",
        "draughtskingwhite": "⛁",
        "draughtsmanblack": "⛂",
        "draughtskingblack": "⛃",
        "snowmanoutsnow": "⛄",
        "sunbehindcloud": "⛅",
        "rain": "⛆",
        "snowmanblack": "⛇",
        "thundercloudrain": "⛈",
        "turnedshogipiecewhite": "⛉",
        "turnedshogipieceblack": "⛊",
        "diamondinsquarewhite": "⛋",
        "crossinglanes": "⛌",
        "disabledcar": "⛍",
        "ophiuchus": "⛎",
        "pick": "⛏",
        "carsliding": "⛐",
        "helmetcrosswhite": "⛑",
        "circledcrossinglanes": "⛒",
        "chains": "⛓",
        "noentry": "⛔",
        "alternateonewayleftwaytraffic": "⛕",
        "twowayleftwaytrafficblack": "⛖",
        "twowayleftwaytrafficwhite": "⛗",
        "lanemergeleftblack": "⛘",
        "lanemergeleftwhite": "⛙",
        "driveslow": "⛚",
        "pointingtriangledownheavywhite": "⛛",
        "closedentryleft": "⛜",
        "squaredsaltire": "⛝",
        "fallingdiagonalincircleinsquareblackwhite": "⛞",
        "truckblack": "⛟",
        "restrictedentryoneleft": "⛠",
        "restrictedentrytwoleft": "⛡",
        "astronomicaluranus": "⛢",
        "circlestroketwodotsaboveheavy": "⛣",
        "pentagram": "⛤",
        "hedinterlacedpentagramright": "⛥",
        "hedinterlacedpentagramleft": "⛦",
        "invertedpentagram": "⛧",
        "crossonshieldblack": "⛨",
        "shintoshrine": "⛩",
        "church": "⛪",
        "castle": "⛫",
        "historicsite": "⛬",
        "gearouthub": "⛭",
        "gearhles": "⛮",
        "maplighthouse": "⛯",
        "mountain": "⛰",
        "umbrellaonground": "⛱",
        "fountain": "⛲",
        "flaginhole": "⛳",
        "ferry": "⛴",
        "sailboat": "⛵",
        "squarefourcorners": "⛶",
        "skier": "⛷",
        "iceskate": "⛸",
        "personball": "⛹",
        "tent": "⛺",
        "japanesebank": "⛻",
        "headstonegraveyard": "⛼",
        "fuelpump": "⛽",
        "consquareupblack": "⛾",
        "flaghorizontalmiddlestripeblackwhite": "⛿",
        "safetyscissorsblack": "✀",
        "scissorsupperblade": "✁",
        "scissorsblack": "✂",
        "scissorslowerblade": "✃",
        "scissorswhite": "✄",
        "checkwhiteheavy": "✅",
        "telephonelocationsign": "✆",
        "tapedrive": "✇",
        "airplane": "✈",
        "envelope": "✉",
        "raisedfist": "✊",
        "raisedh": "✋",
        "hvictory": "✌",
        "hwriting": "✍",
        "pencillowerright": "✎",
        "pencil": "✏",
        "pencilupperright": "✐",
        "nibwhite": "✑",
        "nibblack": "✒",
        "check": "✓",
        "checkheavy": "✔",
        "multiplicationx": "✕",
        "multiplicationxheavy": "✖",
        "ballotx": "✗",
        "ballotxheavy": "✘",
        "greekcrossoutlined": "✙",
        "greekcrossheavy": "✚",
        "crosscentreopen": "✛",
        "crosscentreopenheavy": "✜",
        "latincross": "✝",
        "latincrossshadowedwhite": "✞",
        "latincrossoutlined": "✟",
        "maltesecross": "✠",
        "starofdavid": "✡",
        "asteriskteardropfour": "✢",
        "asteriskballoonfour": "✣",
        "asteriskballoonheavyfour": "✤",
        "asteriskclubfour": "✥",
        "starpointedblackfour": "✦",
        "starpointedwhitefour": "✧",
        "sparkles": "✨",
        "staroutlinedstresswhite": "✩",
        "starcircledwhite": "✪",
        "starcentreopenblack": "✫",
        "starcentreblackwhite": "✬",
        "staroutlinedblack": "✭",
        "staroutlinedblackheavy": "✮",
        "starpinwheel": "✯",
        "starshadowedwhite": "✰",
        "asteriskheavy": "✱",
        "asteriskcentreopen": "✲",
        "spokedasteriskeight": "✳",
        "starpointedblackeight": "✴",
        "starpointedpinwheeleight": "✵",
        "starpointedblacksix": "✶",
        "compasstarpointedblackeight": "✷",
        "compasstarpointedblackheavyeight": "✸",
        "starpointedblacktwelve": "✹",
        "asteriskpointedsixteen": "✺",
        "asteriskteardrop": "✻",
        "asteriskteardropcentreopen": "✼",
        "asteriskteardropheavy": "✽",
        "florettepetalledblackwhitesix": "✾",
        "floretteblack": "✿",
        "florettewhite": "❀",
        "floretteoutlinedpetalledblackeight": "❁",
        "starcentreopenpointedcircledeight": "❂",
        "asteriskteardroppinwheelheavy": "❃",
        "snowflake": "❄",
        "snowflaketight": "❅",
        "chevronsnowflakeheavy": "❆",
        "sparkle": "❇",
        "sparkleheavy": "❈",
        "asteriskballoon": "❉",
        "asteriskteardroppropellereight": "❊",
        "asteriskteardroppropellerheavyeight": "❋",
        "cross": "❌",
        "circleshadowedwhite": "❍",
        "squaredcrossnegative": "❎",
        "squareshadowlowerrightwhite": "❏",
        "squareshadowupperrightwhite": "❐",
        "squarelowerrightshadowedwhite": "❑",
        "squareupperrightshadowedwhite": "❒",
        "questionblackornament": "❓",
        "questionwhiteornament": "❔",
        "exclamationwhiteornament": "❕",
        "diamondminusxblackwhite": "❖",
        "exclamationheavy": "❗",
        "verticalbarlight": "❘",
        "verticalbarmedium": "❙",
        "verticalbarheavy": "❚",
        "commaheavyturnedornament": "❛",
        "commaheavyornament": "❜",
        "commaheavydoubleturnedornament": "❝",
        "commaheavydoubleornament": "❞",
        "lowcommaheavyornament": "❟",
        "lowcommaheavydoubleornament": "❠",
        "curvedstemparagraphsignornament": "❡",
        "exclamationheavyornament": "❢",
        "heartexclamationheavyornament": "❣",
        "heartblackheavy": "❤",
        "heartbulletrotatedblackheavy": "❥",
        "floralheart": "❦",
        "floralheartbulletrotated": "❧",
        "parenthesisleftmediumornament": "❨",
        "parenthesisrightmediumornament": "❩",
        "parenthesisleftflattenedmediumornament": "❪",
        "parenthesisrightflattenedmediumornament": "❫",
        "bracketleftpointedanglemediumornament": "❬",
        "bracketrightpointedanglemediumornament": "❭",
        "quotationleftpointedangleheavyornament": "❮",
        "quotationrightpointedangleheavyornament": "❯",
        "bracketleftpointedangleheavyornament": "❰",
        "bracketrightpointedangleheavyornament": "❱",
        "bracketshellleftlightornament": "❲",
        "bracketshellrightlightornament": "❳",
        "curlybracketleftmediumornament": "❴",
        "curlybracketrightmediumornament": "❵",
        "onenegativecircled": "❶",
        "twonegativecircled": "❷",
        "threenegativecircled": "❸",
        "fournegativecircled": "❹",
        "fivenegativecircled": "❺",
        "sixnegativecircled": "❻",
        "sevennegativecircled": "❼",
        "eightnegativecircled": "❽",
        "ninenegativecircled": "❾",
        "tennegativecircled": "❿",
        "onesanscircled": "➀",
        "twosanscircled": "➁",
        "threesanscircled": "➂",
        "foursanscircled": "➃",
        "fivesanscircled": "➄",
        "sixsanscircled": "➅",
        "sevensanscircled": "➆",
        "eightsanscircled": "➇",
        "ninesanscircled": "➈",
        "tensanscircled": "➉",
        "onesansnegativecircled": "➊",
        "twosansnegativecircled": "➋",
        "threesansnegativecircled": "➌",
        "foursansnegativecircled": "➍",
        "fivesansnegativecircled": "➎",
        "sixsansnegativecircled": "➏",
        "sevensansnegativecircled": "➐",
        "eightsansnegativecircled": "➑",
        "ninesansnegativecircled": "➒",
        "tensansnegativecircled": "➓",
        "arrowrightwideheavy": "➔",
        "plussignheavy": "➕",
        "minussignheavy": "➖",
        "divisionsignheavy": "➗",
        "arrowheavySE": "➘",
        "arrowrightheavy": "➙",
        "arrowheavyNE": "➚",
        "arrowrightpointed": "➛",
        "arrowrightroundheavy": "➜",
        "arrowrighttriangle": "➝",
        "arrowrighttriangleheavy": "➞",
        "arrowrighttriangledashed": "➟",
        "arrowrighttriangledashedheavy": "➠",
        "arrowrightblack": "➡",
        "arrowheadrightthreeDtoplight": "➢",
        "arrowheadrightthreeDbottomlight": "➣",
        "arrowheadrightblack": "➤",
        "arrowrightcurvedownblackheavy": "➥",
        "arrowrightcurveupblackheavy": "➦",
        "arrowrightsquatblack": "➧",
        "arrowrightpointedblackheavy": "➨",
        "arrowrightrightshadedwhite": "➩",
        "arrowrightleftshadedwhite": "➪",
        "arrowrightbacktiltedshadowedwhite": "➫",
        "arrowrightfronttiltedshadowedwhite": "➬",
        "arrowshadowrightlowerwhiteheavy": "➭",
        "arrowshadowrightupperwhiteheavy": "➮",
        "arrowshadowrightnotchedlowerwhite": "➯",
        "curlyloop": "➰",
        "arrowshadowrightnotchedupperwhite": "➱",
        "arrowrightcircledwhiteheavy": "➲",
        "arrowrightfeatheredwhite": "➳",
        "arrowfeatheredblackSE": "➴",
        "arrowrightfeatheredblack": "➵",
        "arrowfeatheredblackNE": "➶",
        "arrowfeatheredblackheavySE": "➷",
        "arrowrightfeatheredblackheavy": "➸",
        "arrowfeatheredblackheavyNE": "➹",
        "arrowteardropright": "➺",
        "arrowteardroprightheavy": "➻",
        "arrowrightwedge": "➼",
        "arrowrightwedgeheavy": "➽",
        "arrowrightoutlinedopen": "➾",
        "curlyloopdouble": "➿",
        "threedimensionalangle": "⟀",
        "tricontainingtriwhiteanglesmall": "⟁",
        "perpendicular": "⟂",
        "opensubset": "⟃",
        "opensuperset": "⟄",
        "bagdelimitersshapeleft": "⟅",
        "bagdelimitersshaperight": "⟆",
        "ordotinside": "⟇",
        "solidussubsetreversepreceding": "⟈",
        "solidussupersetpreceding": "⟉",
        "verticalbarhorizontalstroke": "⟊",
        "risingdiagonal": "⟋",
        "longdivision": "⟌",
        "fallingdiagonal": "⟍",
        "dlogicalsquare": "⟎",
        "dlogicalorsquare": "⟏",
        "centreddotwhitediamond": "⟐",
        "andwithdot": "⟑",
        "elementopeningup": "⟒",
        "lowercornerdotright": "⟓",
        "cornerdotupleft": "⟔",
        "outerjoinleft": "⟕",
        "outerjoinright": "⟖",
        "outerjoinfull": "⟗",
        "largetackup": "⟘",
        "largetackdown": "⟙",
        "turnstileleftrightdbl": "⟚",
        "tackleftright": "⟛",
        "multimapleft": "⟜",
        "longtackright": "⟝",
        "longtackleft": "⟞",
        "tackcircleaboveup": "⟟",
        "lozengedividedbyrulehorizontal": "⟠",
        "convavediamondwhite": "⟡",
        "tickconvavediamondleftwhite": "⟢",
        "tickconvavediamondrightwhite": "⟣",
        "tickleftwhitesquare": "⟤",
        "tickrightwhitesquare": "⟥",
        "bracketwhitesquareleft": "⟦",
        "bracketwhitesquareright": "⟧",
        "bracketangleleft": "⟨",
        "bracketangleright": "⟩",
        "bracketangledblleft": "⟪",
        "bracketangledblright": "⟫",
        "bracketshellwhiteleft": "⟬",
        "bracketshellwhiteright": "⟭",
        "parenflatleft": "⟮",
        "parenflatright": "⟯",
        "brblank": "⠀",
        "dots1": "⠁",
        "dots2": "⠂",
        "dots12": "⠃",
        "dots3": "⠄",
        "dots13": "⠅",
        "dots23": "⠆",
        "dots123": "⠇",
        "dots4": "⠈",
        "dots14": "⠉",
        "dots24": "⠊",
        "dots124": "⠋",
        "dots34": "⠌",
        "dots134": "⠍",
        "dots234": "⠎",
        "dots1234": "⠏",
        "dots5": "⠐",
        "dots15": "⠑",
        "dots25": "⠒",
        "dots125": "⠓",
        "dots35": "⠔",
        "dots135": "⠕",
        "dots235": "⠖",
        "dots1235": "⠗",
        "dots45": "⠘",
        "dots145": "⠙",
        "dots245": "⠚",
        "dots1245": "⠛",
        "dots345": "⠜",
        "dots1345": "⠝",
        "dots2345": "⠞",
        "dots12345": "⠟",
        "dots6": "⠠",
        "dots16": "⠡",
        "dots26": "⠢",
        "dots126": "⠣",
        "dots36": "⠤",
        "dots136": "⠥",
        "dots236": "⠦",
        "dots1236": "⠧",
        "dots46": "⠨",
        "dots146": "⠩",
        "dots246": "⠪",
        "dots1246": "⠫",
        "dots346": "⠬",
        "dots1346": "⠭",
        "dots2346": "⠮",
        "dots12346": "⠯",
        "dots56": "⠰",
        "dots156": "⠱",
        "dots256": "⠲",
        "dots1256": "⠳",
        "dots356": "⠴",
        "dots1356": "⠵",
        "dots2356": "⠶",
        "dots12356": "⠷",
        "dots456": "⠸",
        "dots1456": "⠹",
        "dots2456": "⠺",
        "dots12456": "⠻",
        "dots3456": "⠼",
        "dots13456": "⠽",
        "dots23456": "⠾",
        "dots123456": "⠿",
        "dots7": "⡀",
        "dots17": "⡁",
        "dots27": "⡂",
        "dots127": "⡃",
        "dots37": "⡄",
        "dots137": "⡅",
        "dots237": "⡆",
        "dots1237": "⡇",
        "dots47": "⡈",
        "dots147": "⡉",
        "dots247": "⡊",
        "dots1247": "⡋",
        "dots347": "⡌",
        "dots1347": "⡍",
        "dots2347": "⡎",
        "dots12347": "⡏",
        "dots57": "⡐",
        "dots157": "⡑",
        "dots257": "⡒",
        "dots1257": "⡓",
        "dots357": "⡔",
        "dots1357": "⡕",
        "dots2357": "⡖",
        "dots12357": "⡗",
        "dots457": "⡘",
        "dots1457": "⡙",
        "dots2457": "⡚",
        "dots12457": "⡛",
        "dots3457": "⡜",
        "dots13457": "⡝",
        "dots23457": "⡞",
        "dots123457": "⡟",
        "dots67": "⡠",
        "dots167": "⡡",
        "dots267": "⡢",
        "dots1267": "⡣",
        "dots367": "⡤",
        "dots1367": "⡥",
        "dots2367": "⡦",
        "dots12367": "⡧",
        "dots467": "⡨",
        "dots1467": "⡩",
        "dots2467": "⡪",
        "dots12467": "⡫",
        "dots3467": "⡬",
        "dots13467": "⡭",
        "dots23467": "⡮",
        "dots123467": "⡯",
        "dots567": "⡰",
        "dots1567": "⡱",
        "dots2567": "⡲",
        "dots12567": "⡳",
        "dots3567": "⡴",
        "dots13567": "⡵",
        "dots23567": "⡶",
        "dots123567": "⡷",
        "dots4567": "⡸",
        "dots14567": "⡹",
        "dots24567": "⡺",
        "dots124567": "⡻",
        "dots34567": "⡼",
        "dots134567": "⡽",
        "dots234567": "⡾",
        "dots1234567": "⡿",
        "dots8": "⢀",
        "dots18": "⢁",
        "dots28": "⢂",
        "dots128": "⢃",
        "dots38": "⢄",
        "dots138": "⢅",
        "dots238": "⢆",
        "dots1238": "⢇",
        "dots48": "⢈",
        "dots148": "⢉",
        "dots248": "⢊",
        "dots1248": "⢋",
        "dots348": "⢌",
        "dots1348": "⢍",
        "dots2348": "⢎",
        "dots12348": "⢏",
        "dots58": "⢐",
        "dots158": "⢑",
        "dots258": "⢒",
        "dots1258": "⢓",
        "dots358": "⢔",
        "dots1358": "⢕",
        "dots2358": "⢖",
        "dots12358": "⢗",
        "dots458": "⢘",
        "dots1458": "⢙",
        "dots2458": "⢚",
        "dots12458": "⢛",
        "dots3458": "⢜",
        "dots13458": "⢝",
        "dots23458": "⢞",
        "dots123458": "⢟",
        "dots68": "⢠",
        "dots168": "⢡",
        "dots268": "⢢",
        "dots1268": "⢣",
        "dots368": "⢤",
        "dots1368": "⢥",
        "dots2368": "⢦",
        "dots12368": "⢧",
        "dots468": "⢨",
        "dots1468": "⢩",
        "dots2468": "⢪",
        "dots12468": "⢫",
        "dots3468": "⢬",
        "dots13468": "⢭",
        "dots23468": "⢮",
        "dots123468": "⢯",
        "dots568": "⢰",
        "dots1568": "⢱",
        "dots2568": "⢲",
        "dots12568": "⢳",
        "dots3568": "⢴",
        "dots13568": "⢵",
        "dots23568": "⢶",
        "dots123568": "⢷",
        "dots4568": "⢸",
        "dots14568": "⢹",
        "dots24568": "⢺",
        "dots124568": "⢻",
        "dots34568": "⢼",
        "dots134568": "⢽",
        "dots234568": "⢾",
        "dots1234568": "⢿",
        "dots78": "⣀",
        "dots178": "⣁",
        "dots278": "⣂",
        "dots1278": "⣃",
        "dots378": "⣄",
        "dots1378": "⣅",
        "dots2378": "⣆",
        "dots12378": "⣇",
        "dots478": "⣈",
        "dots1478": "⣉",
        "dots2478": "⣊",
        "dots12478": "⣋",
        "dots3478": "⣌",
        "dots13478": "⣍",
        "dots23478": "⣎",
        "dots123478": "⣏",
        "dots578": "⣐",
        "dots1578": "⣑",
        "dots2578": "⣒",
        "dots12578": "⣓",
        "dots3578": "⣔",
        "dots13578": "⣕",
        "dots23578": "⣖",
        "dots123578": "⣗",
        "dots4578": "⣘",
        "dots14578": "⣙",
        "dots24578": "⣚",
        "dots124578": "⣛",
        "dots34578": "⣜",
        "dots134578": "⣝",
        "dots234578": "⣞",
        "dots1234578": "⣟",
        "dots678": "⣠",
        "dots1678": "⣡",
        "dots2678": "⣢",
        "dots12678": "⣣",
        "dots3678": "⣤",
        "dots13678": "⣥",
        "dots23678": "⣦",
        "dots123678": "⣧",
        "dots4678": "⣨",
        "dots14678": "⣩",
        "dots24678": "⣪",
        "dots124678": "⣫",
        "dots34678": "⣬",
        "dots134678": "⣭",
        "dots234678": "⣮",
        "dots1234678": "⣯",
        "dots5678": "⣰",
        "dots15678": "⣱",
        "dots25678": "⣲",
        "dots125678": "⣳",
        "dots35678": "⣴",
        "dots135678": "⣵",
        "dots235678": "⣶",
        "dots1235678": "⣷",
        "dots45678": "⣸",
        "dots145678": "⣹",
        "dots245678": "⣺",
        "dots1245678": "⣻",
        "dots345678": "⣼",
        "dots1345678": "⣽",
        "dots2345678": "⣾",
        "dots12345678": "⣿",
        "Azu": "Ⰰ",
        "Buky": "Ⰱ",
        "Vede": "Ⰲ",
        "Glagoli": "Ⰳ",
        "Dobro": "Ⰴ",
        "Yestu": "Ⰵ",
        "Zhivete": "Ⰶ",
        "Dzelo": "Ⰷ",
        "Zemlja": "Ⰸ",
        "Izhe": "Ⰹ",
        "Initializhe": "Ⰺ",
        "glag:I": "Ⰻ",
        "Djervi": "Ⰼ",
        "Kako": "Ⰽ",
        "Ljudije": "Ⰾ",
        "Myslite": "Ⰿ",
        "Nashi": "Ⱀ",
        "Onu": "Ⱁ",
        "Pokoji": "Ⱂ",
        "Ritsi": "Ⱃ",
        "Slovo": "Ⱄ",
        "Tvrido": "Ⱅ",
        "Uku": "Ⱆ",
        "Fritu": "Ⱇ",
        "Heru": "Ⱈ",
        "Otu": "Ⱉ",
        "Pe": "Ⱊ",
        "Shta": "Ⱋ",
        "Tsi": "Ⱌ",
        "Chrivi": "Ⱍ",
        "Sha": "Ⱎ",
        "Yeru": "Ⱏ",
        "Yeri": "Ⱐ",
        "Yati": "Ⱑ",
        "Spideryha": "Ⱒ",
        "Yu": "Ⱓ",
        "Smallyus": "Ⱔ",
        "Smallyuswithtail": "Ⱕ",
        "Yo": "Ⱖ",
        "Iotatedsmallyus": "Ⱗ",
        "Bigyus": "Ⱘ",
        "Iotatedbigyus": "Ⱙ",
        "Fita": "Ⱚ",
        "Izhitsa": "Ⱛ",
        "Shtapic": "Ⱜ",
        "Trokutastia": "Ⱝ",
        "Latinatemyslite": "Ⱞ",
        "azu": "ⰰ",
        "buky": "ⰱ",
        "vede": "ⰲ",
        "glagoli": "ⰳ",
        "dobro": "ⰴ",
        "yestu": "ⰵ",
        "zhivete": "ⰶ",
        "dzelo": "ⰷ",
        "zemlja": "ⰸ",
        "izhe": "ⰹ",
        "initializhe": "ⰺ",
        "glag:i": "ⰻ",
        "djervi": "ⰼ",
        "kako": "ⰽ",
        "ljudije": "ⰾ",
        "myslite": "ⰿ",
        "nashi": "ⱀ",
        "onu": "ⱁ",
        "pokoji": "ⱂ",
        "ritsi": "ⱃ",
        "slovo": "ⱄ",
        "tvrido": "ⱅ",
        "uku": "ⱆ",
        "fritu": "ⱇ",
        "heru": "ⱈ",
        "otu": "ⱉ",
        "pe": "ⱊ",
        "shta": "ⱋ",
        "tsi": "ⱌ",
        "chrivi": "ⱍ",
        "sha": "ⱎ",
        "yeru": "ⱏ",
        "yeri": "ⱐ",
        "yati": "ⱑ",
        "spideryha": "ⱒ",
        "glag:yu": "ⱓ",
        "smallyus": "ⱔ",
        "smallyuswithtail": "ⱕ",
        "yo": "ⱖ",
        "iotatedsmallyus": "ⱗ",
        "bigyus": "ⱘ",
        "iotatedbigyus": "ⱙ",
        "fita": "ⱚ",
        "izhitsa": "ⱛ",
        "shtapic": "ⱜ",
        "trokutastia": "ⱝ",
        "latinatemyslite": "ⱞ",
        "Ldblbar": "Ⱡ",
        "ldblbar": "ⱡ",
        "Lmiddletilde": "Ɫ",
        "Pstroke": "Ᵽ",
        "Rtail": "Ɽ",
        "astroke": "ⱥ",
        "tstroke": "ⱦ",
        "Hdescender": "Ⱨ",
        "hdescender": "ⱨ",
        "Kdescender": "Ⱪ",
        "kdescender": "ⱪ",
        "Zdescender": "Ⱬ",
        "zdescender": "ⱬ",
        "lt:Alpha": "Ɑ",
        "Mhook": "Ɱ",
        "Aturned": "Ɐ",
        "lt:Alphaturned": "Ɒ",
        "vrighthook": "ⱱ",
        "Whook": "Ⱳ",
        "whook": "ⱳ",
        "vcurl": "ⱴ",
        "Hhalf": "Ⱶ",
        "hhalf": "ⱶ",
        "phitailless": "ⱷ",
        "enotch": "ⱸ",
        "rtailturned": "ⱹ",
        "olowringinside": "ⱺ",
        "Esmallturned": "ⱻ",
        "j.inferior": "ⱼ",
        "Vmod": "ⱽ",
        "Sswashtail": "Ȿ",
        "Zswashtail": "Ɀ",
        "anGeok": "ⴀ",
        "banGeok": "ⴁ",
        "ganGeok": "ⴂ",
        "donGeok": "ⴃ",
        "enGeok": "ⴄ",
        "vinGeok": "ⴅ",
        "zenGeok": "ⴆ",
        "tanGeok": "ⴇ",
        "inGeok": "ⴈ",
        "kanGeok": "ⴉ",
        "lasGeok": "ⴊ",
        "manGeok": "ⴋ",
        "narGeok": "ⴌ",
        "onGeok": "ⴍ",
        "parGeok": "ⴎ",
        "zharGeok": "ⴏ",
        "raeGeok": "ⴐ",
        "sanGeok": "ⴑ",
        "tarGeok": "ⴒ",
        "unGeok": "ⴓ",
        "pharGeok": "ⴔ",
        "kharGeok": "ⴕ",
        "ghanGeok": "ⴖ",
        "qarGeok": "ⴗ",
        "shinGeok": "ⴘ",
        "chinGeok": "ⴙ",
        "canGeok": "ⴚ",
        "jilGeok": "ⴛ",
        "cilGeok": "ⴜ",
        "charGeok": "ⴝ",
        "xanGeok": "ⴞ",
        "jhanGeok": "ⴟ",
        "haeGeok": "ⴠ",
        "heGeok": "ⴡ",
        "hieGeok": "ⴢ",
        "weGeok": "ⴣ",
        "harGeok": "ⴤ",
        "hoeGeok": "ⴥ",
        "ynGeok": "ⴧ",
        "aenGeok": "ⴭ",
        "anglemarkersubstitutionright": "⸀",
        "anglemarkerdottedsubstitutionright": "⸁",
        "bracketsubstitutionleft": "⸂",
        "bracketsubstitutionright": "⸃",
        "bracketdottedsubstitutionleft": "⸄",
        "bracketdottedsubstitutionright": "⸅",
        "markerraisedinterpolation": "⸆",
        "markerdottedraisedinterpolation": "⸇",
        "markerdottedtransposition": "⸈",
        "brackettranspositionleft": "⸉",
        "brackettranspositionright": "⸊",
        "squareraised": "⸋",
        "bracketraisedleft": "⸌",
        "bracketraisedright": "⸍",
        "coroniseditorial": "⸎",
        "paragraphos": "⸏",
        "paragraphosforked": "⸐",
        "paragraphosforkedreversed": "⸑",
        "hypodiastole": "⸒",
        "obelosdotted": "⸓",
        "ancoradown": "⸔",
        "ancoraup": "⸕",
        "angledottedright": "⸖",
        "hyphendbloblique": "⸗",
        "interrobanginverted": "⸘",
        "palmbranch": "⸙",
        "hyphendieresis": "⸚",
        "tildering": "⸛",
        "bracketparaphraselowleft": "⸜",
        "bracketparaphraselowright": "⸝",
        "tildedotaccent": "⸞",
        "tildedotbelow": "⸟",
        "barquillverticalleft": "⸠",
        "barquillverticalright": "⸡",
        "brackethalftopleft": "⸢",
        "brackethalftopright": "⸣",
        "brackethalfbottomleft": "⸤",
        "brackethalfbottomright": "⸥",
        "ubracketleft": "⸦",
        "ubracketright": "⸧",
        "parendblleft": "⸨",
        "parendblright": "⸩",
        "twodotsoveronedot": "⸪",
        "onedotovertwodots": "⸫",
        "dotsquarefour": "⸬",
        "fivedot": "⸭",
        "questionreversed": "⸮",
        "tildevertical": "ⸯ",
        "pointring": "⸰",
        "wordseparatormiddledot": "⸱",
        "turnedcomma": "⸲",
        "dotraised": "⸳",
        "commaraised": "⸴",
        "turnedsemicolon": "⸵",
        "daggerwithguardleft": "⸶",
        "daggerwithguardright": "⸷",
        "turneddagger": "⸸",
        "sectionsignhalftop": "⸹",
        "emdashdbl": "⸺",
        "emdashtpl": "⸻",
        "stenographicfullstop": "⸼",
        "sixdotsvertical": "⸽",
        "wigglylinevertical": "⸾",
        "capitulum": "⸿",
        "hyphendbl": "⹀",
        "commareversed": "⹁",
        "quotedbllowreversed": "⹂",
        "dashwithupturnleft": "⹃",
        "suspensiondbl": "⹄",
        "kavykainvertedlow": "⹅",
        "kavykawithkavykaaboveinvertedlow": "⹆",
        "kavykalow": "⹇",
        "kavykawithdotlow": "⹈",
        "stackedcommadbl": "⹉",
        "solidusdotted": "⹊",
        "tripledagger": "⹋",
        "medievalcomma": "⹌",
        "paragraphus": "⹍",
        "punctuselevatus": "⹎",
        "cornishversedivider": "⹏",
        "ideographicspace": "\u3000",
        "ideographiccomma": "、",
        "ideographicperiod": "。",
        "dittomark": "〃",
        "jis": "〄",
        "ideographiciterationmark": "々",
        "ideographicclose": "〆",
        "ideographiczero": "〇",
        "anglebracketleft": "〈",
        "anglebracketright": "〉",
        "dblanglebracketleft": "《",
        "dblanglebracketright": "》",
        "cornerbracketleft": "「",
        "cornerbracketright": "」",
        "whitecornerbracketleft": "『",
        "whitecornerbracketright": "』",
        "blacklenticularbracketleft": "【",
        "blacklenticularbracketright": "】",
        "postalmark": "〒",
        "getamark": "〓",
        "tortoiseshellbracketleft": "〔",
        "tortoiseshellbracketright": "〕",
        "whitelenticularbracketleft": "〖",
        "whitelenticularbracketright": "〗",
        "whitetortoiseshellbracketleft": "〘",
        "whitetortoiseshellbracketright": "〙",
        "whitesquarebracketleft": "〚",
        "whitesquarebracketright": "〛",
        "wavedash": "〜",
        "quotedblprimereversed": "〝",
        "quotedblprime": "〞",
        "lowquotedblprime": "〟",
        "postalmarkface": "〠",
        "onehangzhou": "〡",
        "twohangzhou": "〢",
        "threehangzhou": "〣",
        "fourhangzhou": "〤",
        "fivehangzhou": "〥",
        "sixhangzhou": "〦",
        "sevenhangzhou": "〧",
        "eighthangzhou": "〨",
        "ninehangzhou": "〩",
        "ideographicleveltonemark": "〪",
        "ideographicrisingtonemark": "〫",
        "ideographicdepartingtonemark": "〬",
        "ideographicenteringtonemark": "〭",
        "hangulsingledottonemark": "〮",
        "hanguldottonemarkdbl": "〯",
        "wavydash": "〰",
        "verticalkanarepeatmark": "〱",
        "verticalkanarepeatwithvoicedsoundmark": "〲",
        "verticalkanarepeatmarkupperhalf": "〳",
        "verticalkanarepeatwithvoicedsoundmarkupperhalf": "〴",
        "verticalkanarepeatmarklowerhalf": "〵",
        "circlepostalmark": "〶",
        "ideographictelegraphlinefeedseparatorsymbol": "〷",
        "tenhangzhou": "〸",
        "twentyhangzhou": "〹",
        "thirtyhangzhou": "〺",
        "verticalideographiciterationmark": "〻",
        "masumark": "〼",
        "partalternationmark": "〽",
        "ideographicvariationindicator": "〾",
        "ideographichalffillspace": "〿",
        "hira:asmall": "ぁ",
        "hira:a": "あ",
        "hira:ismall": "ぃ",
        "hira:i": "い",
        "hira:usmall": "ぅ",
        "hira:u": "う",
        "hira:esmall": "ぇ",
        "hira:e": "え",
        "hira:osmall": "ぉ",
        "hira:o": "お",
        "hira:ka": "か",
        "hira:ga": "が",
        "hira:ki": "き",
        "hira:gi": "ぎ",
        "hira:ku": "く",
        "hira:gu": "ぐ",
        "hira:ke": "け",
        "hira:ge": "げ",
        "hira:ko": "こ",
        "hira:go": "ご",
        "hira:sa": "さ",
        "hira:za": "ざ",
        "hira:si": "し",
        "hira:zi": "じ",
        "hira:su": "す",
        "hira:zu": "ず",
        "hira:se": "せ",
        "hira:ze": "ぜ",
        "hira:so": "そ",
        "hira:zo": "ぞ",
        "hira:ta": "た",
        "hira:da": "だ",
        "hira:ti": "ち",
        "hira:di": "ぢ",
        "hira:tusmall": "っ",
        "hira:tu": "つ",
        "hira:du": "づ",
        "hira:te": "て",
        "hira:de": "で",
        "hira:to": "と",
        "hira:do": "ど",
        "hira:na": "な",
        "hira:ni": "に",
        "hira:nu": "ぬ",
        "hira:ne": "ね",
        "hira:no": "の",
        "hira:ha": "は",
        "hira:ba": "ば",
        "hira:pa": "ぱ",
        "hira:hi": "ひ",
        "hira:bi": "び",
        "hira:pi": "ぴ",
        "hira:hu": "ふ",
        "hira:bu": "ぶ",
        "hira:pu": "ぷ",
        "hira:he": "へ",
        "hira:be": "べ",
        "hira:pe": "ぺ",
        "hira:ho": "ほ",
        "hira:bo": "ぼ",
        "hira:po": "ぽ",
        "hira:ma": "ま",
        "hira:mi": "み",
        "hira:mu": "む",
        "hira:me": "め",
        "hira:mo": "も",
        "hira:yasmall": "ゃ",
        "hira:ya": "や",
        "hira:yusmall": "ゅ",
        "hira:yu": "ゆ",
        "hira:yosmall": "ょ",
        "hira:yo": "よ",
        "hira:ra": "ら",
        "hira:ri": "り",
        "hira:ru": "る",
        "hira:re": "れ",
        "hira:ro": "ろ",
        "hira:wasmall": "ゎ",
        "hira:wa": "わ",
        "hira:wi": "ゐ",
        "hira:we": "ゑ",
        "hira:wo": "を",
        "hira:n": "ん",
        "hira:vu": "ゔ",
        "hira:kasmall": "ゕ",
        "hira:kesmall": "ゖ",
        "hira:voicedmarkkanacmb": "゙",
        "hira:semivoicedmarkkanacmb": "゚",
        "hira:voicedmarkkana": "゛",
        "hira:semivoicedmarkkana": "゜",
        "hira:iterationhiragana": "ゝ",
        "hira:voicediterationhiragana": "ゞ",
        "hira:digraphyori": "ゟ",
        "kata:doublehyphenkana": "゠",
        "kata:asmall": "ァ",
        "kata:a": "ア",
        "kata:ismall": "ィ",
        "kata:i": "イ",
        "kata:usmall": "ゥ",
        "kata:u": "ウ",
        "kata:esmall": "ェ",
        "kata:e": "エ",
        "kata:osmall": "ォ",
        "kata:o": "オ",
        "kata:ka": "カ",
        "kata:ga": "ガ",
        "kata:ki": "キ",
        "kata:gi": "ギ",
        "kata:ku": "ク",
        "kata:gu": "グ",
        "kata:ke": "ケ",
        "kata:ge": "ゲ",
        "kata:ko": "コ",
        "kata:go": "ゴ",
        "kata:sa": "サ",
        "kata:za": "ザ",
        "kata:si": "シ",
        "kata:zi": "ジ",
        "kata:su": "ス",
        "kata:zu": "ズ",
        "kata:se": "セ",
        "kata:ze": "ゼ",
        "kata:so": "ソ",
        "kata:zo": "ゾ",
        "kata:ta": "タ",
        "kata:da": "ダ",
        "kata:ti": "チ",
        "kata:di": "ヂ",
        "kata:tusmall": "ッ",
        "kata:tu": "ツ",
        "kata:du": "ヅ",
        "kata:te": "テ",
        "kata:de": "デ",
        "kata:to": "ト",
        "kata:do": "ド",
        "kata:na": "ナ",
        "kata:ni": "ニ",
        "kata:nu": "ヌ",
        "kata:ne": "ネ",
        "kata:no": "ノ",
        "kata:ha": "ハ",
        "kata:ba": "バ",
        "kata:pa": "パ",
        "kata:hi": "ヒ",
        "kata:bi": "ビ",
        "kata:pi": "ピ",
        "kata:hu": "フ",
        "kata:bu": "ブ",
        "kata:pu": "プ",
        "kata:he": "ヘ",
        "kata:be": "ベ",
        "kata:pe": "ペ",
        "kata:ho": "ホ",
        "kata:bo": "ボ",
        "kata:po": "ポ",
        "kata:ma": "マ",
        "kata:mi": "ミ",
        "kata:mu": "ム",
        "kata:me": "メ",
        "kata:mo": "モ",
        "kata:yasmall": "ャ",
        "kata:ya": "ヤ",
        "kata:yusmall": "ュ",
        "kata:yu": "ユ",
        "kata:yosmall": "ョ",
        "kata:yo": "ヨ",
        "kata:ra": "ラ",
        "kata:ri": "リ",
        "kata:ru": "ル",
        "kata:re": "レ",
        "kata:ro": "ロ",
        "kata:wasmall": "ヮ",
        "kata:wa": "ワ",
        "kata:wi": "ヰ",
        "kata:we": "ヱ",
        "kata:wo": "ヲ",
        "kata:n": "ン",
        "kata:vu": "ヴ",
        "kata:kasmall": "ヵ",
        "kata:kesmall": "ヶ",
        "kata:va": "ヷ",
        "kata:vi": "ヸ",
        "kata:ve": "ヹ",
        "kata:vo": "ヺ",
        "kata:middledot": "・",
        "kata:prolongedkana": "ー",
        "kata:iteration": "ヽ",
        "kata:voicediteration": "ヾ",
        "kata:digraphkoto": "ヿ",
        "bopo:b": "ㄅ",
        "bopo:p": "ㄆ",
        "bopo:m": "ㄇ",
        "bopo:f": "ㄈ",
        "bopo:d": "ㄉ",
        "bopo:t": "ㄊ",
        "bopo:n": "ㄋ",
        "bopo:l": "ㄌ",
        "bopo:g": "ㄍ",
        "bopo:k": "ㄎ",
        "bopo:h": "ㄏ",
        "bopo:j": "ㄐ",
        "bopo:q": "ㄑ",
        "bopo:x": "ㄒ",
        "bopo:zh": "ㄓ",
        "bopo:ch": "ㄔ",
        "bopo:sh": "ㄕ",
        "bopo:r": "ㄖ",
        "bopo:z": "ㄗ",
        "bopo:c": "ㄘ",
        "bopo:s": "ㄙ",
        "bopo:a": "ㄚ",
        "bopo:o": "ㄛ",
        "bopo:e": "ㄜ",
        "bopo:eh": "ㄝ",
        "bopo:ai": "ㄞ",
        "bopo:ei": "ㄟ",
        "bopo:au": "ㄠ",
        "bopo:ou": "ㄡ",
        "bopo:an": "ㄢ",
        "bopo:en": "ㄣ",
        "bopo:ang": "ㄤ",
        "bopo:eng": "ㄥ",
        "bopo:er": "ㄦ",
        "bopo:i": "ㄧ",
        "bopo:u": "ㄨ",
        "bopo:iu": "ㄩ",
        "bopo:v": "ㄪ",
        "bopo:ng": "ㄫ",
        "bopo:gn": "ㄬ",
        "bopo:ih": "ㄭ",
        "bopo:owithdotabove": "ㄮ",
        "bopo:nn": "ㄯ",
        "ko:kiyeok": "ㄱ",
        "ko:ssangkiyeok": "ㄲ",
        "ko:kiyeoksios": "ㄳ",
        "ko:nieun": "ㄴ",
        "ko:nieuncieuc": "ㄵ",
        "ko:nieunhieuh": "ㄶ",
        "ko:tikeut": "ㄷ",
        "ko:ssangtikeut": "ㄸ",
        "ko:rieul": "ㄹ",
        "ko:rieulkiyeok": "ㄺ",
        "ko:rieulmieum": "ㄻ",
        "ko:rieulpieup": "ㄼ",
        "ko:rieulsios": "ㄽ",
        "ko:rieulthieuth": "ㄾ",
        "ko:rieulphieuph": "ㄿ",
        "ko:rieulhieuh": "ㅀ",
        "ko:mieum": "ㅁ",
        "ko:pieup": "ㅂ",
        "ko:ssangpieup": "ㅃ",
        "ko:pieupsios": "ㅄ",
        "ko:sios": "ㅅ",
        "ko:ssangsios": "ㅆ",
        "ko:ieung": "ㅇ",
        "ko:cieuc": "ㅈ",
        "ko:ssangcieuc": "ㅉ",
        "ko:chieuch": "ㅊ",
        "ko:khieukh": "ㅋ",
        "ko:thieuth": "ㅌ",
        "ko:phieuph": "ㅍ",
        "ko:hieuh": "ㅎ",
        "ko:a": "ㅏ",
        "ko:ae": "ㅐ",
        "ko:ya": "ㅑ",
        "ko:yae": "ㅒ",
        "ko:eo": "ㅓ",
        "ko:e": "ㅔ",
        "ko:yeo": "ㅕ",
        "ko:ye": "ㅖ",
        "ko:o": "ㅗ",
        "ko:wa": "ㅘ",
        "ko:wae": "ㅙ",
        "ko:oe": "ㅚ",
        "ko:yo": "ㅛ",
        "ko:u": "ㅜ",
        "ko:weo": "ㅝ",
        "ko:we": "ㅞ",
        "ko:wi": "ㅟ",
        "ko:yu": "ㅠ",
        "ko:eu": "ㅡ",
        "ko:yi": "ㅢ",
        "ko:i": "ㅣ",
        "ko:filler": "ㅤ",
        "ko:ssangnieun": "ㅥ",
        "ko:nieuntikeut": "ㅦ",
        "ko:nieunsios": "ㅧ",
        "ko:nieunpansios": "ㅨ",
        "ko:rieulkiyeoksios": "ㅩ",
        "ko:rieultikeut": "ㅪ",
        "ko:rieulpieupsios": "ㅫ",
        "ko:rieulpansios": "ㅬ",
        "ko:rieulyeorinhieuh": "ㅭ",
        "ko:mieumpieup": "ㅮ",
        "ko:mieumsios": "ㅯ",
        "ko:mieumpansios": "ㅰ",
        "ko:kapyeounmieum": "ㅱ",
        "ko:pieupkiyeok": "ㅲ",
        "ko:pieuptikeut": "ㅳ",
        "ko:pieupsioskiyeok": "ㅴ",
        "ko:pieupsiostikeut": "ㅵ",
        "ko:pieupcieuc": "ㅶ",
        "ko:pieupthieuth": "ㅷ",
        "ko:kapyeounpieup": "ㅸ",
        "ko:kapyeounssangpieup": "ㅹ",
        "ko:sioskiyeok": "ㅺ",
        "ko:siosnieun": "ㅻ",
        "ko:siostikeut": "ㅼ",
        "ko:siospieup": "ㅽ",
        "ko:sioscieuc": "ㅾ",
        "ko:pansios": "ㅿ",
        "ko:ssangieung": "ㆀ",
        "ko:yesieung": "ㆁ",
        "ko:yesieungsios": "ㆂ",
        "ko:yesieungpansios": "ㆃ",
        "ko:kapyeounphieuph": "ㆄ",
        "ko:ssanghieuh": "ㆅ",
        "ko:yeorinhieuh": "ㆆ",
        "ko:yoya": "ㆇ",
        "ko:yoyae": "ㆈ",
        "ko:yoi": "ㆉ",
        "ko:yuyeo": "ㆊ",
        "ko:yuye": "ㆋ",
        "ko:yui": "ㆌ",
        "ko:araea": "ㆍ",
        "ko:araeae": "ㆎ",
        "hangulkiyeokparen": "㈀",
        "hangulnieunparen": "㈁",
        "hangultikeutparen": "㈂",
        "hangulrieulparen": "㈃",
        "hangulmieumparen": "㈄",
        "hangulpieupparen": "㈅",
        "hangulsiosparen": "㈆",
        "hangulieungparen": "㈇",
        "hangulcieucparen": "㈈",
        "hangulchieuchparen": "㈉",
        "hangulkhieukhparen": "㈊",
        "hangulthieuthparen": "㈋",
        "hangulphieuphparen": "㈌",
        "hangulhieuhparen": "㈍",
        "hangulkiyeokaparen": "㈎",
        "hangulnieunaparen": "㈏",
        "hangultikeutaparen": "㈐",
        "hangulrieulaparen": "㈑",
        "hangulmieumaparen": "㈒",
        "hangulpieupaparen": "㈓",
        "hangulsiosaparen": "㈔",
        "hangulieungaparen": "㈕",
        "hangulcieucaparen": "㈖",
        "hangulchieuchaparen": "㈗",
        "hangulkhieukhaparen": "㈘",
        "hangulthieuthaparen": "㈙",
        "hangulphieuphaparen": "㈚",
        "hangulhieuhaparen": "㈛",
        "hangulcieucuparen": "㈜",
        "ojeonparen": "㈝",
        "ohuparen": "㈞",
        "oneideographicparen": "㈠",
        "twoideographicparen": "㈡",
        "threeideographicparen": "㈢",
        "fourideographicparen": "㈣",
        "fiveideographicparen": "㈤",
        "sixideographicparen": "㈥",
        "sevenideographicparen": "㈦",
        "eightideographicparen": "㈧",
        "nineideographicparen": "㈨",
        "tenideographicparen": "㈩",
        "moonideographicparen": "㈪",
        "fireideographicparen": "㈫",
        "waterideographicparen": "㈬",
        "woodideographicparen": "㈭",
        "metalideographicparen": "㈮",
        "earthideographicparen": "㈯",
        "sunideographicparen": "㈰",
        "stockideographicparen": "㈱",
        "haveideographicparen": "㈲",
        "societyideographicparen": "㈳",
        "nameideographicparen": "㈴",
        "specialideographicparen": "㈵",
        "financialideographicparen": "㈶",
        "congratulationideographicparen": "㈷",
        "laborideographicparen": "㈸",
        "representideographicparen": "㈹",
        "callideographicparen": "㈺",
        "studyideographicparen": "㈻",
        "superviseideographicparen": "㈼",
        "enterpriseideographicparen": "㈽",
        "resourceideographicparen": "㈾",
        "allianceideographicparen": "㈿",
        "festivalideographicparen": "㉀",
        "restideographicparen": "㉁",
        "selfideographicparen": "㉂",
        "reachideographicparen": "㉃",
        "questionideographiccircled": "㉄",
        "kindergartenideographiccircled": "㉅",
        "schoolideographiccircled": "㉆",
        "kotoideographiccircled": "㉇",
        "tencirclesquare": "㉈",
        "twentycirclesquare": "㉉",
        "thirtycirclesquare": "㉊",
        "fortycirclesquare": "㉋",
        "fiftycirclesquare": "㉌",
        "sixtycirclesquare": "㉍",
        "seventycirclesquare": "㉎",
        "eightycirclesquare": "㉏",
        "partnership": "㉐",
        "twentyonecircle": "㉑",
        "twentytwocircle": "㉒",
        "twentythreecircle": "㉓",
        "twentyfourcircle": "㉔",
        "twentyfivecircle": "㉕",
        "twentysixcircle": "㉖",
        "twentysevencircle": "㉗",
        "twentyeightcircle": "㉘",
        "twentyninecircle": "㉙",
        "thirtycircle": "㉚",
        "thirtyonecircle": "㉛",
        "thirtytwocircle": "㉜",
        "thirtythreecircle": "㉝",
        "thirtyfourcircle": "㉞",
        "thirtyfivecircle": "㉟",
        "kiyeokcirclekorean": "㉠",
        "nieuncirclekorean": "㉡",
        "tikeutcirclekorean": "㉢",
        "rieulcirclekorean": "㉣",
        "mieumcirclekorean": "㉤",
        "pieupcirclekorean": "㉥",
        "sioscirclekorean": "㉦",
        "ieungcirclekorean": "㉧",
        "cieuccirclekorean": "㉨",
        "chieuchcirclekorean": "㉩",
        "khieukhcirclekorean": "㉪",
        "thieuthcirclekorean": "㉫",
        "phieuphcirclekorean": "㉬",
        "hieuhcirclekorean": "㉭",
        "kiyeokacirclekorean": "㉮",
        "nieunacirclekorean": "㉯",
        "tikeutacirclekorean": "㉰",
        "rieulacirclekorean": "㉱",
        "mieumacirclekorean": "㉲",
        "pieupacirclekorean": "㉳",
        "siosacirclekorean": "㉴",
        "ieungacirclekorean": "㉵",
        "cieucacirclekorean": "㉶",
        "chieuchacirclekorean": "㉷",
        "khieukhacirclekorean": "㉸",
        "thieuthacirclekorean": "㉹",
        "phieuphacirclekorean": "㉺",
        "hieuhacirclekorean": "㉻",
        "chamkocircle": "㉼",
        "jueuicircle": "㉽",
        "ieungucirclekorean": "㉾",
        "koreanstandardsymbol": "㉿",
        "oneideographiccircled": "㊀",
        "twoideographiccircled": "㊁",
        "threeideographiccircled": "㊂",
        "fourideographiccircled": "㊃",
        "fiveideographiccircled": "㊄",
        "sixideographiccircled": "㊅",
        "sevenideographiccircled": "㊆",
        "eightideographiccircled": "㊇",
        "nineideographiccircled": "㊈",
        "tenideographiccircled": "㊉",
        "moonideographiccircled": "㊊",
        "fireideographiccircled": "㊋",
        "waterideographiccircled": "㊌",
        "woodideographiccircled": "㊍",
        "metalideographiccircled": "㊎",
        "earthideographiccircled": "㊏",
        "sunideographiccircled": "㊐",
        "stockideographiccircled": "㊑",
        "haveideographiccircled": "㊒",
        "societyideographiccircled": "㊓",
        "nameideographiccircled": "㊔",
        "specialideographiccircled": "㊕",
        "financialideographiccircled": "㊖",
        "congratulationideographiccircled": "㊗",
        "laborideographiccircled": "㊘",
        "secretideographiccircled": "㊙",
        "maleideographiccircled": "㊚",
        "femaleideographiccircled": "㊛",
        "suitableideographiccircled": "㊜",
        "excellentideographiccircled": "㊝",
        "printideographiccircled": "㊞",
        "attentionideographiccircled": "㊟",
        "itemideographiccircled": "㊠",
        "restideographiccircled": "㊡",
        "copyideographiccircled": "㊢",
        "correctideographiccircled": "㊣",
        "highideographiccircled": "㊤",
        "centreideographiccircled": "㊥",
        "lowideographiccircled": "㊦",
        "leftideographiccircled": "㊧",
        "rightideographiccircled": "㊨",
        "medicineideographiccircled": "㊩",
        "religionideographiccircled": "㊪",
        "studyideographiccircled": "㊫",
        "superviseideographiccircled": "㊬",
        "enterpriseideographiccircled": "㊭",
        "resourceideographiccircled": "㊮",
        "allianceideographiccircled": "㊯",
        "nightideographiccircled": "㊰",
        "thirtysixcircle": "㊱",
        "thirtysevencircle": "㊲",
        "thirtyeightcircle": "㊳",
        "thirtyninecircle": "㊴",
        "fortycircle": "㊵",
        "fortyonecircle": "㊶",
        "fortytwocircle": "㊷",
        "fortythreecircle": "㊸",
        "fortyfourcircle": "㊹",
        "fortyfivecircle": "㊺",
        "fortysixcircle": "㊻",
        "fortysevencircle": "㊼",
        "fortyeightcircle": "㊽",
        "fortyninecircle": "㊾",
        "fiftycircle": "㊿",
        "januarytelegraph": "㋀",
        "februarytelegraph": "㋁",
        "marchtelegraph": "㋂",
        "apriltelegraph": "㋃",
        "maytelegraph": "㋄",
        "junetelegraph": "㋅",
        "julytelegraph": "㋆",
        "augusttelegraph": "㋇",
        "septembertelegraph": "㋈",
        "octobertelegraph": "㋉",
        "novembertelegraph": "㋊",
        "decembertelegraph": "㋋",
        "Hgfullwidth": "㋌",
        "ergfullwidth": "㋍",
        "eVfullwidth": "㋎",
        "LTDfullwidth": "㋏",
        "acirclekatakana": "㋐",
        "icirclekatakana": "㋑",
        "ucirclekatakana": "㋒",
        "ecirclekatakana": "㋓",
        "ocirclekatakana": "㋔",
        "kacirclekatakana": "㋕",
        "kicirclekatakana": "㋖",
        "kucirclekatakana": "㋗",
        "kecirclekatakana": "㋘",
        "kocirclekatakana": "㋙",
        "sacirclekatakana": "㋚",
        "sicirclekatakana": "㋛",
        "sucirclekatakana": "㋜",
        "secirclekatakana": "㋝",
        "socirclekatakana": "㋞",
        "tacirclekatakana": "㋟",
        "ticirclekatakana": "㋠",
        "tucirclekatakana": "㋡",
        "tecirclekatakana": "㋢",
        "tocirclekatakana": "㋣",
        "nacirclekatakana": "㋤",
        "nicirclekatakana": "㋥",
        "nucirclekatakana": "㋦",
        "necirclekatakana": "㋧",
        "nocirclekatakana": "㋨",
        "hacirclekatakana": "㋩",
        "hicirclekatakana": "㋪",
        "hucirclekatakana": "㋫",
        "hecirclekatakana": "㋬",
        "hocirclekatakana": "㋭",
        "macirclekatakana": "㋮",
        "micirclekatakana": "㋯",
        "mucirclekatakana": "㋰",
        "mecirclekatakana": "㋱",
        "mocirclekatakana": "㋲",
        "yacirclekatakana": "㋳",
        "yucirclekatakana": "㋴",
        "yocirclekatakana": "㋵",
        "racirclekatakana": "㋶",
        "ricirclekatakana": "㋷",
        "rucirclekatakana": "㋸",
        "recirclekatakana": "㋹",
        "rocirclekatakana": "㋺",
        "wacirclekatakana": "㋻",
        "wicirclekatakana": "㋼",
        "wecirclekatakana": "㋽",
        "wocirclekatakana": "㋾",
        "squareeranamereiwa": "㋿",
        "apaatosquare": "㌀",
        "aruhuasquare": "㌁",
        "anpeasquare": "㌂",
        "aarusquare": "㌃",
        "iningusquare": "㌄",
        "intisquare": "㌅",
        "uonsquare": "㌆",
        "esukuudosquare": "㌇",
        "eekaasquare": "㌈",
        "onsusquare": "㌉",
        "oomusquare": "㌊",
        "kairisquare": "㌋",
        "karattosquare": "㌌",
        "karoriisquare": "㌍",
        "garonsquare": "㌎",
        "ganmasquare": "㌏",
        "gigasquare": "㌐",
        "giniisquare": "㌑",
        "kyuriisquare": "㌒",
        "girudaasquare": "㌓",
        "kirosquare": "㌔",
        "kiroguramusquare": "㌕",
        "kiromeetorusquare": "㌖",
        "kirowattosquare": "㌗",
        "guramusquare": "㌘",
        "guramutonsquare": "㌙",
        "kuruzeirosquare": "㌚",
        "kuroonesquare": "㌛",
        "keesusquare": "㌜",
        "korunasquare": "㌝",
        "kooposquare": "㌞",
        "saikurusquare": "㌟",
        "santiimusquare": "㌠",
        "siringusquare": "㌡",
        "sentisquare": "㌢",
        "sentosquare": "㌣",
        "daasusquare": "㌤",
        "desisquare": "㌥",
        "dorusquare": "㌦",
        "tonsquare": "㌧",
        "nanosquare": "㌨",
        "nottosquare": "㌩",
        "haitusquare": "㌪",
        "paasentosquare": "㌫",
        "paatusquare": "㌬",
        "baarerusquare": "㌭",
        "piasutorusquare": "㌮",
        "pikurusquare": "㌯",
        "pikosquare": "㌰",
        "birusquare": "㌱",
        "huaraddosquare": "㌲",
        "huiitosquare": "㌳",
        "bussyerusquare": "㌴",
        "huransquare": "㌵",
        "hekutaarusquare": "㌶",
        "pesosquare": "㌷",
        "penihisquare": "㌸",
        "herutusquare": "㌹",
        "pensusquare": "㌺",
        "peezisquare": "㌻",
        "beetasquare": "㌼",
        "pointosquare": "㌽",
        "borutosquare": "㌾",
        "honsquare": "㌿",
        "pondosquare": "㍀",
        "hoorusquare": "㍁",
        "hoonsquare": "㍂",
        "maikurosquare": "㍃",
        "mairusquare": "㍄",
        "mahhasquare": "㍅",
        "marukusquare": "㍆",
        "mansyonsquare": "㍇",
        "mikuronsquare": "㍈",
        "mirisquare": "㍉",
        "miribaarusquare": "㍊",
        "megasquare": "㍋",
        "megatonsquare": "㍌",
        "meetorusquare": "㍍",
        "yaadosquare": "㍎",
        "yaarusquare": "㍏",
        "yuansquare": "㍐",
        "rittorusquare": "㍑",
        "rirasquare": "㍒",
        "rupiisquare": "㍓",
        "ruuburusquare": "㍔",
        "remusquare": "㍕",
        "rentogensquare": "㍖",
        "wattosquare": "㍗",
        "ideographictelegraphsymbolforhourzero": "㍘",
        "ideographictelegraphsymbolforhourone": "㍙",
        "ideographictelegraphsymbolforhourtwo": "㍚",
        "ideographictelegraphsymbolforhourthree": "㍛",
        "ideographictelegraphsymbolforhourfour": "㍜",
        "ideographictelegraphsymbolforhourfive": "㍝",
        "ideographictelegraphsymbolforhoursix": "㍞",
        "ideographictelegraphsymbolforhourseven": "㍟",
        "ideographictelegraphsymbolforhoureight": "㍠",
        "ideographictelegraphsymbolforhournine": "㍡",
        "ideographictelegraphsymbolforhourten": "㍢",
        "ideographictelegraphsymbolforhoureleven": "㍣",
        "ideographictelegraphsymbolforhourtwelve": "㍤",
        "ideographictelegraphsymbolforhourthirteen": "㍥",
        "ideographictelegraphsymbolforhourfourteen": "㍦",
        "ideographictelegraphsymbolforhourfifteen": "㍧",
        "ideographictelegraphsymbolforhoursixteen": "㍨",
        "ideographictelegraphsymbolforhourseventeen": "㍩",
        "ideographictelegraphsymbolforhoureighteen": "㍪",
        "ideographictelegraphsymbolforhournineteen": "㍫",
        "ideographictelegraphsymbolforhourtwenty": "㍬",
        "ideographictelegraphsymbolforhourtwentyone": "㍭",
        "ideographictelegraphsymbolforhourtwentytwo": "㍮",
        "ideographictelegraphsymbolforhourtwentythree": "㍯",
        "ideographictelegraphsymbolforhourtwentyfour": "㍰",
        "hpafullwidth": "㍱",
        "dafullwidth": "㍲",
        "aufullwidth": "㍳",
        "barfullwidth": "㍴",
        "ovfullwidth": "㍵",
        "pcfullwidth": "㍶",
        "dmfullwidth": "㍷",
        "dm2fullwidth": "㍸",
        "dm3fullwidth": "㍹",
        "iufullwidth": "㍺",
        "eranameheiseisquare": "㍻",
        "eranamesyouwasquare": "㍼",
        "eranametaisyousquare": "㍽",
        "eranamemeizisquare": "㍾",
        "corporationsquare": "㍿",
        "paampsfullwidth": "㎀",
        "nafullwidth": "㎁",
        "muafullwidth": "㎂",
        "mafullwidth": "㎃",
        "kafullwidth": "㎄",
        "kbfullwidth": "㎅",
        "mbfullwidth": "㎆",
        "gbfullwidth": "㎇",
        "calfullwidth": "㎈",
        "kcalfullwidth": "㎉",
        "pffullwidth": "㎊",
        "nffullwidth": "㎋",
        "muffullwidth": "㎌",
        "mugfullwidth": "㎍",
        "mgfullwidth": "㎎",
        "kgfullwidth": "㎏",
        "hzfullwidth": "㎐",
        "khzfullwidth": "㎑",
        "mhzfullwidth": "㎒",
        "ghzfullwidth": "㎓",
        "thzfullwidth": "㎔",
        "mulfullwidth": "㎕",
        "mlfullwidth": "㎖",
        "dlfullwidth": "㎗",
        "klfullwidth": "㎘",
        "fmfullwidth": "㎙",
        "nmfullwidth": "㎚",
        "mumfullwidth": "㎛",
        "mmfullwidth": "㎜",
        "cmfullwidth": "㎝",
        "kmfullwidth": "㎞",
        "mm2fullwidth": "㎟",
        "cm2fullwidth": "㎠",
        "m2fullwidth": "㎡",
        "km2fullwidth": "㎢",
        "mm3fullwidth": "㎣",
        "cm3fullwidth": "㎤",
        "m3fullwidth": "㎥",
        "km3fullwidth": "㎦",
        "moversfullwidth": "㎧",
        "movers2fullwidth": "㎨",
        "pafullwidth": "㎩",
        "kpafullwidth": "㎪",
        "mpafullwidth": "㎫",
        "gpafullwidth": "㎬",
        "radfullwidth": "㎭",
        "radoversfullwidth": "㎮",
        "radovers2fullwidth": "㎯",
        "psfullwidth": "㎰",
        "nsfullwidth": "㎱",
        "musfullwidth": "㎲",
        "msfullwidth": "㎳",
        "pvfullwidth": "㎴",
        "nvfullwidth": "㎵",
        "muvfullwidth": "㎶",
        "mvfullwidth": "㎷",
        "kvfullwidth": "㎸",
        "mvmegafullwidth": "㎹",
        "pwfullwidth": "㎺",
        "nwfullwidth": "㎻",
        "muwfullwidth": "㎼",
        "mwfullwidth": "㎽",
        "kwfullwidth": "㎾",
        "mwmegafullwidth": "㎿",
        "kohmfullwidth": "㏀",
        "mohmfullwidth": "㏁",
        "amfullwidth": "㏂",
        "bqfullwidth": "㏃",
        "ccfullwidth": "㏄",
        "cdfullwidth": "㏅",
        "coverkgfullwidth": "㏆",
        "cofullwidth": "㏇",
        "dbfullwidth": "㏈",
        "gyfullwidth": "㏉",
        "hafullwidth": "㏊",
        "hpfullwidth": "㏋",
        "infullwidth": "㏌",
        "kkfullwidth": "㏍",
        "kmcapitalfullwidth": "㏎",
        "ktfullwidth": "㏏",
        "lmfullwidth": "㏐",
        "lnfullwidth": "㏑",
        "logfullwidth": "㏒",
        "lxfullwidth": "㏓",
        "mbsmallfullwidth": "㏔",
        "milfullwidth": "㏕",
        "molfullwidth": "㏖",
        "phfullwidth": "㏗",
        "pmfullwidth": "㏘",
        "ppmfullwidth": "㏙",
        "prfullwidth": "㏚",
        "srfullwidth": "㏛",
        "svfullwidth": "㏜",
        "wbfullwidth": "㏝",
        "vovermfullwidth": "㏞",
        "aovermfullwidth": "㏟",
        "dayonetelegraph": "㏠",
        "daytwotelegraph": "㏡",
        "daythreetelegraph": "㏢",
        "dayfourtelegraph": "㏣",
        "dayfivetelegraph": "㏤",
        "daysixtelegraph": "㏥",
        "dayseventelegraph": "㏦",
        "dayeighttelegraph": "㏧",
        "dayninetelegraph": "㏨",
        "daytentelegraph": "㏩",
        "dayeleventelegraph": "㏪",
        "daytwelvetelegraph": "㏫",
        "daythirteentelegraph": "㏬",
        "dayfourteentelegraph": "㏭",
        "dayfifteentelegraph": "㏮",
        "daysixteentelegraph": "㏯",
        "dayseventeentelegraph": "㏰",
        "dayeighteentelegraph": "㏱",
        "daynineteentelegraph": "㏲",
        "daytwentytelegraph": "㏳",
        "daytwentyonetelegraph": "㏴",
        "daytwentytwotelegraph": "㏵",
        "daytwentythreetelegraph": "㏶",
        "daytwentyfourtelegraph": "㏷",
        "daytwentyfivetelegraph": "㏸",
        "daytwentysixtelegraph": "㏹",
        "daytwentyseventelegraph": "㏺",
        "daytwentyeighttelegraph": "㏻",
        "daytwentyninetelegraph": "㏼",
        "daythirtytelegraph": "㏽",
        "daythirtyonetelegraph": "㏾",
        "galsquare": "㏿",
        "stresstonemod": "꜠",
        "stresslowtonemod": "꜡",
        "Egyptalef": "Ꜣ",
        "egyptalef": "ꜣ",
        "Egyptain": "Ꜥ",
        "egyptain": "ꜥ",
        "Heng": "Ꜧ",
        "heng": "ꜧ",
        "Tz": "Ꜩ",
        "tz": "ꜩ",
        "Tresillo": "Ꜫ",
        "tresillo": "ꜫ",
        "Cuatrillo": "Ꜭ",
        "cuatrillo": "ꜭ",
        "Cuatrillocomma": "Ꜯ",
        "cuatrillocomma": "ꜯ",
        "Fsmall": "ꜰ",
        "Ssmall": "ꜱ",
        "AA": "Ꜳ",
        "aa": "ꜳ",
        "AO": "Ꜵ",
        "ao": "ꜵ",
        "AU": "Ꜷ",
        "au": "ꜷ",
        "AV": "Ꜹ",
        "av": "ꜹ",
        "AVhorizontalbar": "Ꜻ",
        "avhorizontalbar": "ꜻ",
        "AY": "Ꜽ",
        "ay": "ꜽ",
        "Cdotreversed": "Ꜿ",
        "cdotreversed": "ꜿ",
        "Kstroke": "Ꝁ",
        "kstroke": "ꝁ",
        "Kdiagonalstroke": "Ꝃ",
        "kdiagonalstroke": "ꝃ",
        "Kstrokediagonalstroke": "Ꝅ",
        "kstrokediagonalstroke": "ꝅ",
        "Lbroken": "Ꝇ",
        "lbroken": "ꝇ",
        "Lstroke": "Ꝉ",
        "lstroke": "ꝉ",
        "Ostroke": "Ꝋ",
        "ostroke": "ꝋ",
        "Oloop": "Ꝍ",
        "oloop": "ꝍ",
        "OO": "Ꝏ",
        "oo": "ꝏ",
        "Pstrokedescender": "Ꝑ",
        "pstrokedescender": "ꝑ",
        "Pflourish": "Ꝓ",
        "pflourish": "ꝓ",
        "Ptail": "Ꝕ",
        "ptail": "ꝕ",
        "Qstrokedescender": "Ꝗ",
        "qstrokedescender": "ꝗ",
        "Qdiagonalstroke": "Ꝙ",
        "qdiagonalstroke": "ꝙ",
        "Rrotunda": "Ꝛ",
        "rrotunda": "ꝛ",
        "Rumrotunda": "Ꝝ",
        "rumrotunda": "ꝝ",
        "Vdiagonalstroke": "Ꝟ",
        "vdiagonalstroke": "ꝟ",
        "Vy": "Ꝡ",
        "vy": "ꝡ",
        "Visigothicz": "Ꝣ",
        "visigothicz": "ꝣ",
        "Thornstroke": "Ꝥ",
        "thornstroke": "ꝥ",
        "Thornstrokedescender": "Ꝧ",
        "thornstrokedescender": "ꝧ",
        "Vend": "Ꝩ",
        "vend": "ꝩ",
        "Et": "Ꝫ",
        "et": "ꝫ",
        "Is": "Ꝭ",
        "is": "ꝭ",
        "Con": "Ꝯ",
        "con": "ꝯ",
        "usmod": "ꝰ",
        "dum": "ꝱ",
        "lum": "ꝲ",
        "mum": "ꝳ",
        "num": "ꝴ",
        "rum": "ꝵ",
        "Rumsmall": "ꝶ",
        "tum": "ꝷ",
        "um": "ꝸ",
        "Dinsular": "Ꝺ",
        "dinsular": "ꝺ",
        "Finsular": "Ꝼ",
        "finsular": "ꝼ",
        "Ginsular": "Ᵹ",
        "Ginsularturned": "Ꝿ",
        "ginsularturned": "ꝿ",
        "Lturned": "Ꞁ",
        "lturned": "ꞁ",
        "Rinsular": "Ꞃ",
        "rinsular": "ꞃ",
        "Sinsular": "Ꞅ",
        "sinsular": "ꞅ",
        "Tinsular": "Ꞇ",
        "tinsular": "ꞇ",
        "circumflexlow": "ꞈ",
        "colonmod": "꞉",
        "shortequalsmod": "꞊",
        "Saltillo": "Ꞌ",
        "saltillo": "ꞌ",
        "Hturned": "Ɥ",
        "lbeltretroflex": "ꞎ",
        "sinologicaldot": "ꞏ",
        "Ndescender": "Ꞑ",
        "ndescender": "ꞑ",
        "Cbar": "Ꞓ",
        "cbar": "ꞓ",
        "cpalatalhook": "ꞔ",
        "hpalatalhook": "ꞕ",
        "Bflourish": "Ꞗ",
        "bflourish": "ꞗ",
        "Fstroke": "Ꞙ",
        "fstroke": "ꞙ",
        "Volapukae": "Ꞛ",
        "volapukae": "ꞛ",
        "Volapukoe": "Ꞝ",
        "volapukoe": "ꞝ",
        "Volapukue": "Ꞟ",
        "volapukue": "ꞟ",
        "Gobliquestroke": "Ꞡ",
        "gobliquestroke": "ꞡ",
        "Kobliquestroke": "Ꞣ",
        "kobliquestroke": "ꞣ",
        "Nobliquestroke": "Ꞥ",
        "nobliquestroke": "ꞥ",
        "Robliquestroke": "Ꞧ",
        "robliquestroke": "ꞧ",
        "Sobliquestroke": "Ꞩ",
        "sobliquestroke": "ꞩ",
        "Hhook": "Ɦ",
        "Ereversedopen": "Ɜ",
        "Scriptg": "Ɡ",
        "Lbelt": "Ɬ",
        "Ismall": "Ɪ",
        "Qsmall": "ꞯ",
        "Kturned": "Ʞ",
        "Tturned": "Ʇ",
        "Jcrossed-tail": "Ʝ",
        "lt:Chi": "Ꭓ",
        "lt:Beta": "Ꞵ",
        "lt:beta": "ꞵ",
        "lt:Omega": "Ꞷ",
        "lt:omega": "ꞷ",
        "Ustroke": "Ꞹ",
        "ustroke": "ꞹ",
        "Glottala": "Ꞻ",
        "glottala": "ꞻ",
        "Glottali": "Ꞽ",
        "glottali": "ꞽ",
        "Glottalu": "Ꞿ",
        "glottalu": "ꞿ",
        "Anglicanaw": "Ꟃ",
        "anglicanaw": "ꟃ",
        "Cpalatalhook": "Ꞔ",
        "Shook": "Ʂ",
        "Zpalatalhook": "Ᶎ",
        "iepigraphicsideways": "ꟷ",
        "Hstrokemod": "ꟸ",
        "ligatureoemod": "ꟹ",
        "Mturnedsmall": "ꟺ",
        "freversedepigraphic": "ꟻ",
        "preversedepigraphic": "ꟼ",
        "mepigraphicinverted": "ꟽ",
        "iaepigraphic": "ꟾ",
        "archaicmepigraphic": "ꟿ",
        "panyangga": "ꦀ",
        "cecak": "ꦁ",
        "layar": "ꦂ",
        "wignyan": "ꦃ",
        "java:a": "ꦄ",
        "ikawi": "ꦅ",
        "java:i": "ꦆ",
        "ii": "ꦇ",
        "java:u": "ꦈ",
        "pacerek": "ꦉ",
        "ngalelet": "ꦊ",
        "ngaleletraswadi": "ꦋ",
        "java:e": "ꦌ",
        "ai": "ꦍ",
        "java:o": "ꦎ",
        "ka": "ꦏ",
        "kasasak": "ꦐ",
        "kamurda": "ꦑ",
        "ga": "ꦒ",
        "gamurda": "ꦓ",
        "nga": "ꦔ",
        "ca": "ꦕ",
        "camurda": "ꦖ",
        "ja": "ꦗ",
        "nyamurda": "ꦘ",
        "jamahaprana": "ꦙ",
        "nya": "ꦚ",
        "tta": "ꦛ",
        "ttamahaprana": "ꦜ",
        "dda": "ꦝ",
        "ddamahaprana": "ꦞ",
        "namurda": "ꦟ",
        "ta": "ꦠ",
        "tamurda": "ꦡ",
        "da": "ꦢ",
        "damahaprana": "ꦣ",
        "na": "ꦤ",
        "pa": "ꦥ",
        "pamurda": "ꦦ",
        "ba": "ꦧ",
        "bamurda": "ꦨ",
        "ma": "ꦩ",
        "ya": "ꦪ",
        "ra": "ꦫ",
        "raagung": "ꦬ",
        "la": "ꦭ",
        "wa": "ꦮ",
        "samurda": "ꦯ",
        "samahaprana": "ꦰ",
        "sa": "ꦱ",
        "ha": "ꦲ",
        "cecaktelu": "꦳",
        "tarungvowel": "ꦴ",
        "tolongvowel": "ꦵ",
        "wuluvowel": "ꦶ",
        "wulumelikvowel": "ꦷ",
        "sukuvowel": "ꦸ",
        "sukumendutvowel": "ꦹ",
        "talingvowel": "ꦺ",
        "dirgamurevowel": "ꦻ",
        "pepetvowel": "ꦼ",
        "keretconsonant": "ꦽ",
        "pengkalconsonant": "ꦾ",
        "cakraconsonant": "ꦿ",
        "pangkon": "꧀",
        "rerengganleft": "꧁",
        "rerengganright": "꧂",
        "andappada": "꧃",
        "madyapada": "꧄",
        "luhurpada": "꧅",
        "windupada": "꧆",
        "pangkatpada": "꧇",
        "lingsapada": "꧈",
        "lungsipada": "꧉",
        "adegpada": "꧊",
        "adegadegpada": "꧋",
        "piselehpada": "꧌",
        "turnedpiselehpada": "꧍",
        "pangrangkep": "ꧏ",
        "java:zero": "꧐",
        "java:one": "꧑",
        "java:two": "꧒",
        "java:three": "꧓",
        "java:four": "꧔",
        "java:five": "꧕",
        "java:six": "꧖",
        "java:seven": "꧗",
        "java:eight": "꧘",
        "java:nine": "꧙",
        "tirtatumetespada": "꧞",
        "isen-isenpada": "꧟",
        "redalphabar": "ꬰ",
        "areversedschwa": "ꬱ",
        "efractur": "ꬲ",
        "redebar": "ꬳ",
        "eflourish": "ꬴ",
        "flenis": "ꬵ",
        "gtailscript": "ꬶ",
        "llazyinverteds": "ꬷ",
        "lmiddledbltilde": "ꬸ",
        "lmiddlering": "ꬹ",
        "mtail": "ꬺ",
        "ntail": "ꬻ",
        "engtail": "ꬼ",
        "ofractur": "ꬽ",
        "ofracturstroke": "ꬾ",
        "ostrokeopen": "ꬿ",
        "oeinverted": "ꭀ",
        "oestroketurned": "ꭁ",
        "oehorizontalstroketurned": "ꭂ",
        "ooopenturned": "ꭃ",
        "ooopenstroketurned": "ꭄ",
        "stirrupr": "ꭅ",
        "Rrightlegsmall": "ꭆ",
        "routhandle": "ꭇ",
        "rdbl": "ꭈ",
        "rtail": "ꭉ",
        "rtaildbl": "ꭊ",
        "rscript": "ꭋ",
        "rringscript": "ꭌ",
        "baselineesh": "ꭍ",
        "urightlegshort": "ꭎ",
        "urightlegbarshort": "ꭏ",
        "ui": "ꭐ",
        "uiturned": "ꭑ",
        "ulefthook": "ꭒ",
        "lt:chi": "ꭓ",
        "lt:chilowrightring": "ꭔ",
        "lt:chilowleftserif": "ꭕ",
        "xlowrightring": "ꭖ",
        "xlongleftleg": "ꭗ",
        "xlongleftlegandlowrightring": "ꭘ",
        "xlongleftlegserif": "ꭙ",
        "yrightlegshort": "ꭚ",
        "breveinvertedmod": "꭛",
        "hengmod": "ꭜ",
        "llazyinvertedsmod": "ꭝ",
        "lmiddletildemod": "ꭞ",
        "ulefthookmod": "ꭟ",
        "sakhayat": "ꭠ",
        "iotifiede": "ꭡ",
        "oeopen": "ꭢ",
        "uo": "ꭣ",
        "alphainverted": "ꭤ",
        "lt:Omegasmall": "ꭥ",
        "dzdigraphretroflexhook": "ꭦ",
        "tsdigraphretroflexhook": "ꭧ",
        "chrk:asmall": "ꭰ",
        "chrk:esmall": "ꭱ",
        "chrk:ismall": "ꭲ",
        "chrk:osmall": "ꭳ",
        "chrk:usmall": "ꭴ",
        "chrk:vsmall": "ꭵ",
        "chrk:gasmall": "ꭶ",
        "chrk:kasmall": "ꭷ",
        "chrk:gesmall": "ꭸ",
        "chrk:gismall": "ꭹ",
        "chrk:gosmall": "ꭺ",
        "chrk:gusmall": "ꭻ",
        "chrk:gvsmall": "ꭼ",
        "chrk:hasmall": "ꭽ",
        "chrk:hesmall": "ꭾ",
        "chrk:hismall": "ꭿ",
        "chrk:hosmall": "ꮀ",
        "chrk:husmall": "ꮁ",
        "chrk:hvsmall": "ꮂ",
        "chrk:lasmall": "ꮃ",
        "chrk:lesmall": "ꮄ",
        "chrk:lismall": "ꮅ",
        "chrk:losmall": "ꮆ",
        "chrk:lusmall": "ꮇ",
        "chrk:lvsmall": "ꮈ",
        "chrk:masmall": "ꮉ",
        "chrk:mesmall": "ꮊ",
        "chrk:mismall": "ꮋ",
        "chrk:mosmall": "ꮌ",
        "chrk:musmall": "ꮍ",
        "chrk:nasmall": "ꮎ",
        "chrk:hnasmall": "ꮏ",
        "chrk:nahsmall": "ꮐ",
        "chrk:nesmall": "ꮑ",
        "chrk:nismall": "ꮒ",
        "chrk:nosmall": "ꮓ",
        "chrk:nusmall": "ꮔ",
        "chrk:nvsmall": "ꮕ",
        "chrk:quasmall": "ꮖ",
        "chrk:quesmall": "ꮗ",
        "chrk:quismall": "ꮘ",
        "chrk:quosmall": "ꮙ",
        "chrk:quusmall": "ꮚ",
        "chrk:quvsmall": "ꮛ",
        "chrk:sasmall": "ꮜ",
        "chrk:ssmall": "ꮝ",
        "chrk:sesmall": "ꮞ",
        "chrk:sismall": "ꮟ",
        "chrk:sosmall": "ꮠ",
        "chrk:susmall": "ꮡ",
        "chrk:svsmall": "ꮢ",
        "chrk:dasmall": "ꮣ",
        "chrk:tasmall": "ꮤ",
        "chrk:desmall": "ꮥ",
        "chrk:tesmall": "ꮦ",
        "chrk:dismall": "ꮧ",
        "chrk:tismall": "ꮨ",
        "chrk:dosmall": "ꮩ",
        "chrk:dusmall": "ꮪ",
        "chrk:dvsmall": "ꮫ",
        "chrk:dlasmall": "ꮬ",
        "chrk:tlasmall": "ꮭ",
        "chrk:tlesmall": "ꮮ",
        "chrk:tlismall": "ꮯ",
        "chrk:tlosmall": "ꮰ",
        "chrk:tlusmall": "ꮱ",
        "chrk:tlvsmall": "ꮲ",
        "chrk:tsasmall": "ꮳ",
        "chrk:tsesmall": "ꮴ",
        "chrk:tsismall": "ꮵ",
        "chrk:tsosmall": "ꮶ",
        "chrk:tsusmall": "ꮷ",
        "chrk:tsvsmall": "ꮸ",
        "chrk:wasmall": "ꮹ",
        "chrk:wesmall": "ꮺ",
        "chrk:wismall": "ꮻ",
        "chrk:wosmall": "ꮼ",
        "chrk:wusmall": "ꮽ",
        "chrk:wvsmall": "ꮾ",
        "chrk:yasmall": "ꮿ",
        "f_f": "ﬀ",
        "fi": "ﬁ",
        "fl": "ﬂ",
        "f_f_i": "ﬃ",
        "f_f_l": "ﬄ",
        "longs_t": "ﬅ",
        "s_t": "ﬆ",
        "men_nowarmn": "ﬓ",
        "men_echarmn": "ﬔ",
        "men_iniarmn": "ﬕ",
        "vew_nowarmn": "ﬖ",
        "men_xeharmn": "ﬗ",
        "yodwithhiriq:hb": "יִ",
        "varikajudeospanish:hb": "ﬞ",
        "yod_yod_patah:hb": "ײַ",
        "ayinalt:hb": "ﬠ",
        "alefwide:hb": "ﬡ",
        "daletwide:hb": "ﬢ",
        "hewide:hb": "ﬣ",
        "kafwide:hb": "ﬤ",
        "lamedwide:hb": "ﬥ",
        "finalmemwide:hb": "ﬦ",
        "reshwide:hb": "ﬧ",
        "tavwide:hb": "ﬨ",
        "plussignalt:hb": "﬩",
        "shinwithshinDot:hb": "שׁ",
        "shinwithsinDot:hb": "שׂ",
        "shinwithdageshandshinDot:hb": "שּׁ",
        "shinwithdageshandsinDot:hb": "שּׂ",
        "alefwithpatah:hb": "אַ",
        "alefwithqamats:hb": "אָ",
        "alefwithmapiq:hb": "אּ",
        "betwithdagesh:hb": "בּ",
        "gimelwithdagesh:hb": "גּ",
        "daletwithdagesh:hb": "דּ",
        "hewithmapiq:hb": "הּ",
        "vavwithdagesh:hb": "וּ",
        "zayinwithdagesh:hb": "זּ",
        "tetwithdagesh:hb": "טּ",
        "yodwithdagesh:hb": "יּ",
        "finalkafwithdagesh:hb": "ךּ",
        "kafwithdagesh:hb": "כּ",
        "lamedwithdagesh:hb": "לּ",
        "memwithdagesh:hb": "מּ",
        "nunwithdagesh:hb": "נּ",
        "samekhwithdagesh:hb": "סּ",
        "finalpewithdagesh:hb": "ףּ",
        "pewithdagesh:hb": "פּ",
        "tsadiwithdagesh:hb": "צּ",
        "qofwithdagesh:hb": "קּ",
        "reshwithdagesh:hb": "רּ",
        "shinwithdagesh:hb": "שּ",
        "tavwithdagesh:hb": "תּ",
        "vavwithholam:hb": "וֹ",
        "betwithrafe:hb": "בֿ",
        "kafwithrafe:hb": "כֿ",
        "pewithrafe:hb": "פֿ",
        "ligaturealeflamed:hb": "ﭏ",
        "alefwasla.isol": "ﭐ",
        "alefwasla.fina": "ﭑ",
        "beeh.isol": "ﭒ",
        "beeh.fina": "ﭓ",
        "beeh.init": "ﭔ",
        "beeh.medi": "ﭕ",
        "peh.isol": "ﭖ",
        "peh.fina": "ﭗ",
        "peh.init": "ﭘ",
        "peh.medi": "ﭙ",
        "beheh.isol": "ﭚ",
        "beheh.fina": "ﭛ",
        "beheh.init": "ﭜ",
        "beheh.medi": "ﭝ",
        "tteheh.isol": "ﭞ",
        "tteheh.fina": "ﭟ",
        "tteheh.init": "ﭠ",
        "tteheh.medi": "ﭡ",
        "teheh.isol": "ﭢ",
        "teheh.fina": "ﭣ",
        "teheh.init": "ﭤ",
        "teheh.medi": "ﭥ",
        "tteh.isol": "ﭦ",
        "tteh.fina": "ﭧ",
        "tteh.init": "ﭨ",
        "tteh.medi": "ﭩ",
        "veh.isol": "ﭪ",
        "veh.fina": "ﭫ",
        "veh.init": "ﭬ",
        "veh.medi": "ﭭ",
        "peheh.isol": "ﭮ",
        "peheh.fina": "ﭯ",
        "peheh.init": "ﭰ",
        "peheh.medi": "ﭱ",
        "dyeh.isol": "ﭲ",
        "dyeh.fina": "ﭳ",
        "dyeh.init": "ﭴ",
        "dyeh.medi": "ﭵ",
        "nyeh.isol": "ﭶ",
        "nyeh.fina": "ﭷ",
        "nyeh.init": "ﭸ",
        "nyeh.medi": "ﭹ",
        "tcheh.isol": "ﭺ",
        "tcheh.fina": "ﭻ",
        "tcheh.init": "ﭼ",
        "tcheh.medi": "ﭽ",
        "tcheheh.isol": "ﭾ",
        "tcheheh.fina": "ﭿ",
        "tcheheh.init": "ﮀ",
        "tcheheh.medi": "ﮁ",
        "ddahal.isol": "ﮂ",
        "ddahal.fina": "ﮃ",
        "dahal.isol": "ﮄ",
        "dahal.fina": "ﮅ",
        "dul.isol": "ﮆ",
        "dul.fina": "ﮇ",
        "ddal.isol": "ﮈ",
        "ddal.fina": "ﮉ",
        "jeh.isol": "ﮊ",
        "jeh.fina": "ﮋ",
        "rreh.isol": "ﮌ",
        "rreh.fina": "ﮍ",
        "keheh.isol": "ﮎ",
        "keheh.fina": "ﮏ",
        "keheh.init": "ﮐ",
        "keheh.medi": "ﮑ",
        "gaf.isol": "ﮒ",
        "gaf.fina": "ﮓ",
        "gaf.init": "ﮔ",
        "gaf.medi": "ﮕ",
        "gueh.isol": "ﮖ",
        "gueh.fina": "ﮗ",
        "gueh.init": "ﮘ",
        "gueh.medi": "ﮙ",
        "ngoeh.isol": "ﮚ",
        "ngoeh.fina": "ﮛ",
        "ngoeh.init": "ﮜ",
        "ngoeh.medi": "ﮝ",
        "noonghunna.isol": "ﮞ",
        "noonghunna.fina": "ﮟ",
        "rnoon.isol": "ﮠ",
        "rnoon.fina": "ﮡ",
        "rnoon.init": "ﮢ",
        "rnoon.medi": "ﮣ",
        "hehyeh.isol": "ﮤ",
        "hehyeh.fina": "ﮥ",
        "hehgoal.isol": "ﮦ",
        "hehgoal.fina": "ﮧ",
        "hehgoal.init": "ﮨ",
        "hehgoal.medi": "ﮩ",
        "hehdoachashmee.isol": "ﮪ",
        "hehdoachashmee.fina": "ﮫ",
        "hehdoachashmee.init": "ﮬ",
        "hehdoachashmee.medi": "ﮭ",
        "yehbarree.isol": "ﮮ",
        "yehbarree.fina": "ﮯ",
        "yehbarreehamza.isol": "ﮰ",
        "yehbarreehamza.fina": "ﮱ",
        "symboldotabove": "﮲",
        "symboldotbelow": "﮳",
        "symboltwodotsabove": "﮴",
        "symboltwodotsbelow": "﮵",
        "symbolabovethreedotsabove": "﮶",
        "symbolbelowthreedotsabove": "﮷",
        "symbolpointingabovedownthreedotsabove": "﮸",
        "symbolpointingbelowdownthreedotsabove": "﮹",
        "symbolfourdotsabove": "﮺",
        "symbolfourdotsbelow": "﮻",
        "symboldoubleverticalbarbelow": "﮼",
        "symboltwodotsverticallyabove": "﮽",
        "symboltwodotsverticallybelow": "﮾",
        "symbolring": "﮿",
        "symboltahabovesmall": "﯀",
        "symboltahbelowsmall": "﯁",
        "ng.isol": "ﯓ",
        "ng.fina": "ﯔ",
        "ng.init": "ﯕ",
        "ng.medi": "ﯖ",
        "u.isol": "ﯗ",
        "u.fina": "ﯘ",
        "oe.isol": "ﯙ",
        "oe.fina": "ﯚ",
        "yu.isol": "ﯛ",
        "yu.fina": "ﯜ",
        "uhamza.isol": "ﯝ",
        "ve.isol": "ﯞ",
        "ve.fina": "ﯟ",
        "oekirghiz.isol": "ﯠ",
        "oekirghiz.fina": "ﯡ",
        "yukirghiz.isol": "ﯢ",
        "yukirghiz.fina": "ﯣ",
        "e.isol": "ﯤ",
        "e.fina": "ﯥ",
        "e.init": "ﯦ",
        "e.medi": "ﯧ",
        "uighurkazakhkirghizalefmaksura.init": "ﯨ",
        "uighurkazakhkirghizalefmaksura.medi": "ﯩ",
        "yeh.init_hamzaabove.medi_alef.fina": "ﯪ",
        "yeh.medi_hamzaabove.medi_alef.fina": "ﯫ",
        "yeh.init_hamzaabove.medi_ae.fina": "ﯬ",
        "yeh.medi_hamzaabove.medi_ae.fina": "ﯭ",
        "yeh.init_hamzaabove.medi_waw.fina": "ﯮ",
        "yeh.medi_hamzaabove.medi_waw.fina": "ﯯ",
        "yeh.init_hamzaabove.medi_u.fina": "ﯰ",
        "yeh.medi_hamzaabove.medi_u.fina": "ﯱ",
        "yeh.init_hamzaabove.medi_oe.fina": "ﯲ",
        "yeh.medi_hamzaabove.medi_oe.fina": "ﯳ",
        "yeh.init_hamzaabove.medi_yu.fina": "ﯴ",
        "yeh.medi_hamzaabove.medi_yu.fina": "ﯵ",
        "yeh.init_hamzaabove.medi_e.fina": "ﯶ",
        "yeh.medi_hamzaabove.medi_e.fina": "ﯷ",
        "yeh.init_hamzaabove.medi_e.medi": "ﯸ",
        "uighurkirghizyeh.init_hamzaabove.medi_alefmaksura.fina": "ﯹ",
        "uighurkirghizyeh.medi_hamzaabove.medi_alefmaksura.fina": "ﯺ",
        "uighurkirghizyeh.init_hamzaabove.medi_alefmaksura.medi": "ﯻ",
        "yehfarsi.isol": "ﯼ",
        "yehfarsi.fina": "ﯽ",
        "yehfarsi.init": "ﯾ",
        "yehfarsi.medi": "ﯿ",
        "yeh.init_hamzaabove.medi_jeem.fina": "ﰀ",
        "yeh.init_hamzaabove.medi_hah.fina": "ﰁ",
        "yeh.init_hamzaabove.medi_meem.fina": "ﰂ",
        "yeh.init_hamzaabove.medi_alefmaksura.fina": "ﰃ",
        "yeh.init_hamzaabove.medi_yeh.fina": "ﰄ",
        "beh.init_jeem.fina": "ﰅ",
        "beh.init_hah.fina": "ﰆ",
        "beh.init_khah.fina": "ﰇ",
        "beh.init_meem.fina": "ﰈ",
        "beh.init_alefmaksura.fina": "ﰉ",
        "beh.init_yeh.fina": "ﰊ",
        "teh.init_jeem.fina": "ﰋ",
        "teh.init_hah.fina": "ﰌ",
        "teh.init_khah.fina": "ﰍ",
        "teh.init_meem.fina": "ﰎ",
        "teh.init_alefmaksura.fina": "ﰏ",
        "teh.init_yeh.fina": "ﰐ",
        "theh.init_jeem.fina": "ﰑ",
        "theh.init_meem.fina": "ﰒ",
        "theh.init_alefmaksura.fina": "ﰓ",
        "theh.init_yeh.fina": "ﰔ",
        "jeem.init_hah.fina": "ﰕ",
        "jeem.init_meem.fina": "ﰖ",
        "hah.init_jeem.fina": "ﰗ",
        "hah.init_meem.fina": "ﰘ",
        "khah.init_jeem.fina": "ﰙ",
        "khah.init_hah.fina": "ﰚ",
        "khah.init_meem.fina": "ﰛ",
        "seen.init_jeem.fina": "ﰜ",
        "seen.init_hah.fina": "ﰝ",
        "seen.init_khah.fina": "ﰞ",
        "seen.init_meem.fina": "ﰟ",
        "sad.init_hah.fina": "ﰠ",
        "sad.init_meem.fina": "ﰡ",
        "dad.init_jeem.fina": "ﰢ",
        "dad.init_hah.fina": "ﰣ",
        "dad.init_khah.fina": "ﰤ",
        "dad.init_meem.fina": "ﰥ",
        "tah.init_hah.fina": "ﰦ",
        "tah.init_meem.fina": "ﰧ",
        "zah.init_meem.fina": "ﰨ",
        "ain.init_jeem.fina": "ﰩ",
        "ain.init_meem.fina": "ﰪ",
        "ghain.init_jeem.fina": "ﰫ",
        "ghain.init_meem.fina": "ﰬ",
        "feh.init_jeem.fina": "ﰭ",
        "feh.init_hah.fina": "ﰮ",
        "feh.init_khah.fina": "ﰯ",
        "feh.init_meem.fina": "ﰰ",
        "feh.init_alefmaksura.fina": "ﰱ",
        "feh.init_yeh.fina": "ﰲ",
        "qaf.init_hah.fina": "ﰳ",
        "qaf.init_meem.fina": "ﰴ",
        "qaf.init_alefmaksura.fina": "ﰵ",
        "qaf.init_yeh.fina": "ﰶ",
        "kaf.init_alef.fina": "ﰷ",
        "kaf.init_jeem.fina": "ﰸ",
        "kaf.init_hah.fina": "ﰹ",
        "kaf.init_khah.fina": "ﰺ",
        "kaf.init_lam.fina": "ﰻ",
        "kaf.init_meem.fina": "ﰼ",
        "kaf.init_alefmaksura.fina": "ﰽ",
        "kaf.init_yeh.fina": "ﰾ",
        "lam.init_jeem.fina": "ﰿ",
        "lam.init_hah.fina": "ﱀ",
        "lam.init_khah.fina": "ﱁ",
        "lam.init_meem.fina": "ﱂ",
        "lam.init_alefmaksura.fina": "ﱃ",
        "lam.init_yeh.fina": "ﱄ",
        "meem.init_jeem.fina": "ﱅ",
        "meem.init_hah.fina": "ﱆ",
        "meem.init_khah.fina": "ﱇ",
        "meem.init_meem.fina": "ﱈ",
        "meem.init_alefmaksura.fina": "ﱉ",
        "meem.init_yeh.fina": "ﱊ",
        "noon.init_jeem.fina": "ﱋ",
        "noon.init_hah.fina": "ﱌ",
        "noon.init_khah.fina": "ﱍ",
        "noon.init_meem.fina": "ﱎ",
        "noon.init_alefmaksura.fina": "ﱏ",
        "noon.init_yeh.fina": "ﱐ",
        "heh.init_jeem.fina": "ﱑ",
        "heh.init_meem.fina": "ﱒ",
        "heh.init_alefmaksura.fina": "ﱓ",
        "heh.init_yeh.fina": "ﱔ",
        "yeh.init_jeem.fina": "ﱕ",
        "yeh.init_hah.fina": "ﱖ",
        "yeh.init_khah.fina": "ﱗ",
        "yeh.init_meem.fina": "ﱘ",
        "yeh.init_alefmaksura.fina": "ﱙ",
        "yeh.init_yeh.fina": "ﱚ",
        "thal.init_superscriptalef.fina": "ﱛ",
        "reh.init_superscriptalef.fina": "ﱜ",
        "alefmaksura.init_superscriptalef.fina": "ﱝ",
        "shaddaDammatanIsol": "ﱞ",
        "shaddaKasratanIsol": "ﱟ",
        "shaddaFathaIsol": "ﱠ",
        "shaddaDammaIsol": "ﱡ",
        "shaddaKasraIsol": "ﱢ",
        "shaddaAlefIsol": "ﱣ",
        "yeh.medi_hamzaabove.medi_reh.fina": "ﱤ",
        "yeh.medi_hamzaabove.medi_zain.fina": "ﱥ",
        "yeh.medi_hamzaabove.medi_meem.fina": "ﱦ",
        "yeh.medi_hamzaabove.medi_noon.fina": "ﱧ",
        "yeh.medi_hamzaabove.medi_alefmaksura.fina": "ﱨ",
        "yeh.medi_hamzaabove.medi_yeh.fina": "ﱩ",
        "beh.medi_reh.fina": "ﱪ",
        "beh.medi_zain.fina": "ﱫ",
        "beh.medi_meem.fina": "ﱬ",
        "beh.medi_noon.fina": "ﱭ",
        "beh.medi_alefmaksura.fina": "ﱮ",
        "beh.medi_yeh.fina": "ﱯ",
        "teh.medi_reh.fina": "ﱰ",
        "teh.medi_zain.fina": "ﱱ",
        "teh.medi_meem.fina": "ﱲ",
        "teh.medi_noon.fina": "ﱳ",
        "teh.medi_alefmaksura.fina": "ﱴ",
        "teh.medi_yeh.fina": "ﱵ",
        "theh.medi_reh.fina": "ﱶ",
        "theh.medi_zain.fina": "ﱷ",
        "theh.medi_meem.fina": "ﱸ",
        "theh.medi_noon.fina": "ﱹ",
        "theh.medi_alefmaksura.fina": "ﱺ",
        "theh.medi_yeh.fina": "ﱻ",
        "feh.medi_alefmaksura.fina": "ﱼ",
        "feh.medi_yeh.fina": "ﱽ",
        "qaf.medi_alefmaksura.fina": "ﱾ",
        "qaf.medi_yeh.fina": "ﱿ",
        "kaf.medi_alef.fina": "ﲀ",
        "kaf.medi_lam.fina": "ﲁ",
        "kaf.medi_meem.fina": "ﲂ",
        "kaf.medi_alefmaksura.fina": "ﲃ",
        "kaf.medi_yeh.fina": "ﲄ",
        "lam.medi_meem.fina": "ﲅ",
        "lam.medi_alefmaksura.fina": "ﲆ",
        "lam.medi_yeh.fina": "ﲇ",
        "meem.medi_alef.fina": "ﲈ",
        "meem.medi_meem.fina": "ﲉ",
        "noon.medi_reh.fina": "ﲊ",
        "noon.medi_zain.fina": "ﲋ",
        "noon.medi_meem.fina": "ﲌ",
        "noon.medi_noon.fina": "ﲍ",
        "noon.medi_alefmaksura.fina": "ﲎ",
        "noon.medi_yeh.fina": "ﲏ",
        "alefmaksura.medi_superscriptalef.fina": "ﲐ",
        "yeh.medi_reh.fina": "ﲑ",
        "yeh.medi_zain.fina": "ﲒ",
        "yeh.medi_meem.fina": "ﲓ",
        "yeh.medi_noon.fina": "ﲔ",
        "yeh.medi_alefmaksura.fina": "ﲕ",
        "yeh.medi_yeh.fina": "ﲖ",
        "yeh.init_hamzaabove.medi_jeem.medi": "ﲗ",
        "yeh.init_hamzaabove.medi_hah.medi": "ﲘ",
        "yeh.init_hamzaabove.medi_khah.medi": "ﲙ",
        "yeh.init_hamzaabove.medi_meem.medi": "ﲚ",
        "yeh.init_hamzaabove.medi_heh.medi": "ﲛ",
        "beh.init_jeem.medi": "ﲜ",
        "beh.init_hah.medi": "ﲝ",
        "beh.init_khah.medi": "ﲞ",
        "beh.init_meem.medi": "ﲟ",
        "beh.init_heh.medi": "ﲠ",
        "teh.init_jeem.medi": "ﲡ",
        "teh.init_hah.medi": "ﲢ",
        "teh.init_khah.medi": "ﲣ",
        "teh.init_meem.medi": "ﲤ",
        "teh.init_heh.medi": "ﲥ",
        "theh.init_meem.medi": "ﲦ",
        "jeem.init_hah.medi": "ﲧ",
        "jeem.init_meem.medi": "ﲨ",
        "hah.init_jeem.medi": "ﲩ",
        "hah.init_meem.medi": "ﲪ",
        "khah.init_jeem.medi": "ﲫ",
        "khah.init_meem.medi": "ﲬ",
        "seen.init_jeem.medi": "ﲭ",
        "seen.init_hah.medi": "ﲮ",
        "seen.init_khah.medi": "ﲯ",
        "seen.init_meem.medi": "ﲰ",
        "sad.init_hah.medi": "ﲱ",
        "sad.init_khah.medi": "ﲲ",
        "sad.init_meem.medi": "ﲳ",
        "dad.init_jeem.medi": "ﲴ",
        "dad.init_hah.medi": "ﲵ",
        "dad.init_khah.medi": "ﲶ",
        "dad.init_meem.medi": "ﲷ",
        "tah.init_hah.medi": "ﲸ",
        "zah.init_meem.medi": "ﲹ",
        "ain.init_jeem.medi": "ﲺ",
        "ain.init_meem.medi": "ﲻ",
        "ghain.init_jeem.medi": "ﲼ",
        "ghain.init_meem.medi": "ﲽ",
        "feh.init_jeem.medi": "ﲾ",
        "feh.init_hah.medi": "ﲿ",
        "feh.init_khah.medi": "ﳀ",
        "feh.init_meem.medi": "ﳁ",
        "qaf.init_hah.medi": "ﳂ",
        "qaf.init_meem.medi": "ﳃ",
        "kaf.init_jeem.medi": "ﳄ",
        "kaf.init_hah.medi": "ﳅ",
        "kaf.init_khah.medi": "ﳆ",
        "kaf.init_lam.medi": "ﳇ",
        "kaf.init_meem.medi": "ﳈ",
        "lam.init_jeem.medi": "ﳉ",
        "lam.init_hah.medi": "ﳊ",
        "lam.init_khah.medi": "ﳋ",
        "lam.init_meem.medi": "ﳌ",
        "lam.init_heh.medi": "ﳍ",
        "meem.init_jeem.medi": "ﳎ",
        "meem.init_hah.medi": "ﳏ",
        "meem.init_khah.medi": "ﳐ",
        "meem.init_meem.medi": "ﳑ",
        "noon.init_jeem.medi": "ﳒ",
        "noon.init_hah.medi": "ﳓ",
        "noon.init_khah.medi": "ﳔ",
        "noon.init_meem.medi": "ﳕ",
        "noon.init_heh.medi": "ﳖ",
        "heh.init_jeem.medi": "ﳗ",
        "heh.init_meem.medi": "ﳘ",
        "heh.init_superscriptalef.medi": "ﳙ",
        "yeh.init_jeem.medi": "ﳚ",
        "yeh.init_hah.medi": "ﳛ",
        "yeh.init_khah.medi": "ﳜ",
        "yeh.init_meem.medi": "ﳝ",
        "yeh.init_heh.medi": "ﳞ",
        "yeh.medi_hamzaabove.medi_meem.medi": "ﳟ",
        "yeh.medi_hamzaabove.medi_heh.medi": "ﳠ",
        "beh.medi_meem.medi": "ﳡ",
        "beh.medi_heh.medi": "ﳢ",
        "teh.medi_meem.medi": "ﳣ",
        "teh.medi_heh.medi": "ﳤ",
        "theh.medi_meem.medi": "ﳥ",
        "theh.medi_heh.medi": "ﳦ",
        "seen.medi_meem.medi": "ﳧ",
        "seen.medi_heh.medi": "ﳨ",
        "sheen.medi_meem.medi": "ﳩ",
        "sheen.medi_heh.medi": "ﳪ",
        "kaf.medi_lam.medi": "ﳫ",
        "kaf.medi_meem.medi": "ﳬ",
        "lam.medi_meem.medi": "ﳭ",
        "noon.medi_meem.medi": "ﳮ",
        "noon.medi_heh.medi": "ﳯ",
        "yeh.medi_meem.medi": "ﳰ",
        "yeh.medi_heh.medi": "ﳱ",
        "shaddaFathaMedi": "ﳲ",
        "shaddaDammaMedi": "ﳳ",
        "shaddaKasraMedi": "ﳴ",
        "tah.init_alefmaksura.fina": "ﳵ",
        "tah.init_yeh.fina": "ﳶ",
        "ain.init_alefmaksura.fina": "ﳷ",
        "ain.init_yeh.fina": "ﳸ",
        "ghain.init_alefmaksura.fina": "ﳹ",
        "ghain.init_yeh.fina": "ﳺ",
        "seen.init_alefmaksura.fina": "ﳻ",
        "seen.init_yeh.fina": "ﳼ",
        "sheen.init_alefmaksura.fina": "ﳽ",
        "sheen.init_yeh.fina": "ﳾ",
        "hah.init_alefmaksura.fina": "ﳿ",
        "hah.init_yeh.fina": "ﴀ",
        "jeem.init_alefmaksura.fina": "ﴁ",
        "jeem.init_yeh.fina": "ﴂ",
        "khah.init_alefmaksura.fina": "ﴃ",
        "khah.init_yeh.fina": "ﴄ",
        "sad.init_alefmaksura.fina": "ﴅ",
        "sad.init_yeh.fina": "ﴆ",
        "dad.init_alefmaksura.fina": "ﴇ",
        "dad.init_yeh.fina": "ﴈ",
        "sheen.init_jeem.fina": "ﴉ",
        "sheen.init_hah.fina": "ﴊ",
        "sheen.init_khah.fina": "ﴋ",
        "sheen.init_meem.fina": "ﴌ",
        "sheen.init_reh.fina": "ﴍ",
        "seen.init_reh.fina": "ﴎ",
        "sad.init_reh.fina": "ﴏ",
        "dad.init_reh.fina": "ﴐ",
        "tah.medi_alefmaksura.fina": "ﴑ",
        "tah.medi_yeh.fina": "ﴒ",
        "ain.medi_alefmaksura.fina": "ﴓ",
        "ain.medi_yeh.fina": "ﴔ",
        "ghain.medi_alefmaksura.fina": "ﴕ",
        "ghain.medi_yeh.fina": "ﴖ",
        "seen.medi_alefmaksura.fina": "ﴗ",
        "seen.medi_yeh.fina": "ﴘ",
        "sheen.medi_alefmaksura.fina": "ﴙ",
        "sheen.medi_yeh.fina": "ﴚ",
        "hah.medi_alefmaksura.fina": "ﴛ",
        "hah.medi_yeh.fina": "ﴜ",
        "jeem.medi_alefmaksura.fina": "ﴝ",
        "jeem.medi_yeh.fina": "ﴞ",
        "khah.medi_alefmaksura.fina": "ﴟ",
        "khah.medi_yeh.fina": "ﴠ",
        "sad.medi_alefmaksura.fina": "ﴡ",
        "sad.medi_yeh.fina": "ﴢ",
        "dad.medi_alefmaksura.fina": "ﴣ",
        "dad.medi_yeh.fina": "ﴤ",
        "sheen.medi_jeem.fina": "ﴥ",
        "sheen.medi_hah.fina": "ﴦ",
        "sheen.medi_khah.fina": "ﴧ",
        "sheen.medi_meem.fina": "ﴨ",
        "sheen.medi_reh.fina": "ﴩ",
        "seen.medi_reh.fina": "ﴪ",
        "sad.medi_reh.fina": "ﴫ",
        "dad.medi_reh.fina": "ﴬ",
        "sheen.init_jeem.medi": "ﴭ",
        "sheen.init_hah.medi": "ﴮ",
        "sheen.init_khah.medi": "ﴯ",
        "sheen.init_meem.medi": "ﴰ",
        "seen.init_heh.medi": "ﴱ",
        "sheen.init_heh.medi": "ﴲ",
        "tah.init_meem.medi": "ﴳ",
        "seen.medi_jeem.medi": "ﴴ",
        "seen.medi_hah.medi": "ﴵ",
        "seen.medi_khah.medi": "ﴶ",
        "sheen.medi_jeem.medi": "ﴷ",
        "sheen.medi_hah.medi": "ﴸ",
        "sheen.medi_khah.medi": "ﴹ",
        "tah.medi_meem.medi": "ﴺ",
        "zah.medi_meem.medi": "ﴻ",
        "alef.medi_fathatan.fina": "ﴼ",
        "alef.init_fathatan.fina": "ﴽ",
        "ornateleftparenthesis": "﴾",
        "ornaterightparenthesis": "﴿",
        "teh.init_jeem.medi_meem.medi": "ﵐ",
        "teh.medi_hah.medi_jeem.fina": "ﵑ",
        "teh.init_hah.medi_jeem.medi": "ﵒ",
        "teh.init_hah.medi_meem.medi": "ﵓ",
        "teh.init_khah.medi_meem.medi": "ﵔ",
        "teh.init_meem.medi_jeem.medi": "ﵕ",
        "teh.init_meem.medi_hah.medi": "ﵖ",
        "teh.init_meem.medi_khah.medi": "ﵗ",
        "jeem.medi_meem.medi_hah.fina": "ﵘ",
        "jeem.init_meem.medi_hah.medi": "ﵙ",
        "hah.medi_meem.medi_yeh.fina": "ﵚ",
        "hah.medi_meem.medi_alefmaksura.fina": "ﵛ",
        "seen.init_hah.medi_jeem.medi": "ﵜ",
        "seen.init_jeem.medi_hah.medi": "ﵝ",
        "seen.medi_jeem.medi_alefmaksura.fina": "ﵞ",
        "seen.medi_meem.medi_hah.fina": "ﵟ",
        "seen.init_meem.medi_hah.medi": "ﵠ",
        "seen.init_meem.medi_jeem.medi": "ﵡ",
        "seen.medi_meem.medi_meem.fina": "ﵢ",
        "seen.init_meem.medi_meem.medi": "ﵣ",
        "sad.medi_hah.medi_hah.fina": "ﵤ",
        "sad.init_hah.medi_hah.medi": "ﵥ",
        "sad.medi_meem.medi_meem.fina": "ﵦ",
        "sheen.medi_hah.medi_meem.fina": "ﵧ",
        "sheen.init_hah.medi_meem.medi": "ﵨ",
        "sheen.medi_jeem.medi_yeh.fina": "ﵩ",
        "sheen.medi_meem.medi_khah.fina": "ﵪ",
        "sheen.init_meem.medi_khah.medi": "ﵫ",
        "sheen.medi_meem.medi_meem.fina": "ﵬ",
        "sheen.init_meem.medi_meem.medi": "ﵭ",
        "dad.medi_hah.medi_alefmaksura.fina": "ﵮ",
        "dad.medi_khah.medi_meem.fina": "ﵯ",
        "dad.init_khah.medi_meem.medi": "ﵰ",
        "tah.medi_meem.medi_hah.fina": "ﵱ",
        "tah.init_meem.medi_hah.medi": "ﵲ",
        "tah.init_meem.medi_meem.medi": "ﵳ",
        "tah.medi_meem.medi_yeh.fina": "ﵴ",
        "ain.medi_jeem.medi_meem.fina": "ﵵ",
        "ain.medi_meem.medi_meem.fina": "ﵶ",
        "ain.init_meem.medi_meem.medi": "ﵷ",
        "ain.medi_meem.medi_alefmaksura.fina": "ﵸ",
        "ghain.medi_meem.medi_meem.fina": "ﵹ",
        "ghain.medi_meem.medi_yeh.fina": "ﵺ",
        "ghain.medi_meem.medi_alefmaksura.fina": "ﵻ",
        "feh.medi_khah.medi_meem.fina": "ﵼ",
        "feh.init_khah.medi_meem.medi": "ﵽ",
        "qaf.medi_meem.medi_hah.fina": "ﵾ",
        "qaf.medi_meem.medi_meem.fina": "ﵿ",
        "lam.medi_hah.medi_meem.fina": "ﶀ",
        "lam.medi_hah.medi_yeh.fina": "ﶁ",
        "lam.medi_hah.medi_alefmaksura.fina": "ﶂ",
        "lam.init_jeem.medi_jeem.medi": "ﶃ",
        "lam.medi_jeem.medi_jeem.fina": "ﶄ",
        "lam.medi_khah.medi_meem.fina": "ﶅ",
        "lam.init_khah.medi_meem.medi": "ﶆ",
        "lam.medi_meem.medi_hah.fina": "ﶇ",
        "lam.init_meem.medi_hah.medi": "ﶈ",
        "meem.init_hah.medi_jeem.medi": "ﶉ",
        "meem.init_hah.medi_meem.medi": "ﶊ",
        "meem.medi_hah.medi_yeh.fina": "ﶋ",
        "meem.init_jeem.medi_hah.medi": "ﶌ",
        "meem.init_jeem.medi_meem.medi": "ﶍ",
        "meem.init_khah.medi_jeem.medi": "ﶎ",
        "meem.init_khah.medi_meem.medi": "ﶏ",
        "meem.init_jeem.medi_khah.medi": "ﶒ",
        "heh.init_meem.medi_jeem.medi": "ﶓ",
        "heh.init_meem.medi_meem.medi": "ﶔ",
        "noon.init_hah.medi_meem.medi": "ﶕ",
        "noon.medi_hah.medi_alefmaksura.fina": "ﶖ",
        "noon.medi_jeem.medi_meem.fina": "ﶗ",
        "noon.init_jeem.medi_meem.medi": "ﶘ",
        "noon.medi_jeem.medi_alefmaksura.fina": "ﶙ",
        "noon.medi_meem.medi_yeh.fina": "ﶚ",
        "noon.medi_meem.medi_alefmaksura.fina": "ﶛ",
        "yeh.medi_meem.medi_meem.fina": "ﶜ",
        "yeh.init_meem.medi_meem.medi": "ﶝ",
        "beh.medi_khah.medi_yeh.fina": "ﶞ",
        "teh.medi_jeem.medi_yeh.fina": "ﶟ",
        "teh.medi_jeem.medi_alefmaksura.fina": "ﶠ",
        "teh.medi_khah.medi_yeh.fina": "ﶡ",
        "teh.medi_khah.medi_alefmaksura.fina": "ﶢ",
        "teh.medi_meem.medi_yeh.fina": "ﶣ",
        "teh.medi_meem.medi_alefmaksura.fina": "ﶤ",
        "jeem.medi_meem.medi_yeh.fina": "ﶥ",
        "jeem.medi_hah.medi_alefmaksura.fina": "ﶦ",
        "jeem.medi_meem.medi_alefmaksura.fina": "ﶧ",
        "seen.medi_khah.medi_alefmaksura.fina": "ﶨ",
        "sad.medi_hah.medi_yeh.fina": "ﶩ",
        "sheen.medi_hah.medi_yeh.fina": "ﶪ",
        "dad.medi_hah.medi_yeh.fina": "ﶫ",
        "lam.medi_jeem.medi_yeh.fina": "ﶬ",
        "lam.medi_meem.medi_yeh.fina": "ﶭ",
        "yeh.medi_hah.medi_yeh.fina": "ﶮ",
        "yeh.medi_jeem.medi_yeh.fina": "ﶯ",
        "yeh.medi_meem.medi_yeh.fina": "ﶰ",
        "meem.medi_meem.medi_yeh.fina": "ﶱ",
        "qaf.medi_meem.medi_yeh.fina": "ﶲ",
        "noon.medi_hah.medi_yeh.fina": "ﶳ",
        "qaf.init_meem.medi_hah.medi": "ﶴ",
        "lam.init_hah.medi_meem.medi": "ﶵ",
        "ain.medi_meem.medi_yeh.fina": "ﶶ",
        "kaf.medi_meem.medi_yeh.fina": "ﶷ",
        "noon.init_jeem.medi_hah.medi": "ﶸ",
        "meem.medi_khah.medi_yeh.fina": "ﶹ",
        "lam.init_jeem.medi_meem.medi": "ﶺ",
        "kaf.medi_meem.medi_meem.fina": "ﶻ",
        "lam.medi_jeem.medi_meem.fina": "ﶼ",
        "noon.medi_jeem.medi_hah.fina": "ﶽ",
        "jeem.medi_hah.medi_yeh.fina": "ﶾ",
        "hah.medi_jeem.medi_yeh.fina": "ﶿ",
        "meem.medi_jeem.medi_yeh.fina": "ﷀ",
        "feh.medi_meem.medi_yeh.fina": "ﷁ",
        "beh.medi_hah.medi_yeh.fina": "ﷂ",
        "kaf.init_meem.medi_meem.medi": "ﷃ",
        "ain.init_jeem.medi_meem.medi": "ﷄ",
        "sad.init_meem.medi_meem.medi": "ﷅ",
        "seen.medi_khah.medi_yeh.fina": "ﷆ",
        "noon.medi_jeem.medi_yeh.fina": "ﷇ",
        "SallaUsedAsKoranicStopSign": "ﷰ",
        "QalaUsedAsKoranicStopSign": "ﷱ",
        "Allah": "ﷲ",
        "Akbar": "ﷳ",
        "Mohammad": "ﷴ",
        "Salam": "ﷵ",
        "Rasoul": "ﷶ",
        "Alayhe": "ﷷ",
        "Wasallam": "ﷸ",
        "Salla": "ﷹ",
        "SallallahouAlayheWasallam": "ﷺ",
        "Jallajalalouhou": "ﷻ",
        "rial": "﷼",
        "BismillahArRahmanArRaheem": "﷽",
        "vert:comma": "︐",
        "vert:ideographiccomma": "︑",
        "vert:ideographicfullstop": "︒",
        "vert:colon": "︓",
        "vert:semicolon": "︔",
        "vert:exclam": "︕",
        "vert:question": "︖",
        "vert:bracketwhiteleft": "︗",
        "vert:brakcetwhiteright": "︘",
        "vert:ellipsishor": "︙",
        "twodotleadervertical": "︰",
        "emdashvertical": "︱",
        "endashvertical": "︲",
        "underscorevertical": "︳",
        "underscorewavyvertical": "︴",
        "parenleftvertical": "︵",
        "parenrightvertical": "︶",
        "braceleftvertical": "︷",
        "bracerightvertical": "︸",
        "tortoiseshellbracketleftvertical": "︹",
        "tortoiseshellbracketrightvertical": "︺",
        "blacklenticularbracketleftvertical": "︻",
        "blacklenticularbracketrightvertical": "︼",
        "dblanglebracketleftvertical": "︽",
        "dblanglebracketrightvertical": "︾",
        "anglebracketleftvertical": "︿",
        "anglebracketrightvertical": "﹀",
        "cornerbracketleftvertical": "﹁",
        "cornerbracketrightvertical": "﹂",
        "whitecornerbracketleftvertical": "﹃",
        "whitecornerbracketrightvertical": "﹄",
        "sesamedot": "﹅",
        "whitesesamedot": "﹆",
        "squarebracketleftvertical": "﹇",
        "squarebracketrightvertical": "﹈",
        "overlinedashed": "﹉",
        "overlinecenterline": "﹊",
        "overlinewavy": "﹋",
        "overlinedblwavy": "﹌",
        "underscoredashed": "﹍",
        "underscorecenterline": "﹎",
        "underscorewavy": "﹏",
        "commasmall": "﹐",
        "ideographiccommasmall": "﹑",
        "periodsmall": "﹒",
        "semicolonsmall": "﹔",
        "colonsmall": "﹕",
        "questionsmall": "﹖",
        "exclamsmall": "﹗",
        "emdashsmall": "﹘",
        "parenthesisleftsmall": "﹙",
        "parenthesisrightsmall": "﹚",
        "braceleftsmall": "﹛",
        "bracerightsmall": "﹜",
        "tortoiseshellbracketleftsmall": "﹝",
        "tortoiseshellbracketrightsmall": "﹞",
        "numbersignsmall": "﹟",
        "ampersandsmall": "﹠",
        "asterisksmall": "﹡",
        "plussmall": "﹢",
        "hyphensmall": "﹣",
        "lesssmall": "﹤",
        "greatersmall": "﹥",
        "equalsmall": "﹦",
        "backslashsmall": "﹨",
        "dollarsmall": "﹩",
        "percentsmall": "﹪",
        "commercialatsmall": "﹫",
        "fathatanIsol": "ﹰ",
        "tatweelFathatanAbove": "ﹱ",
        "dammatanIsol": "ﹲ",
        "kashidaFina": "ﹳ",
        "kasratanIsol": "ﹴ",
        "fathaIsol": "ﹶ",
        "fathaMedi": "ﹷ",
        "dammaIsol": "ﹸ",
        "dammaMedi": "ﹹ",
        "kasraIsol": "ﹺ",
        "kasraMedi": "ﹻ",
        "shaddaIsol": "ﹼ",
        "shaddaMedi": "ﹽ",
        "sukunIsol": "ﹾ",
        "sukunMedi": "ﹿ",
        "hamzaIsol": "ﺀ",
        "alefmadda.isol": "ﺁ",
        "alefmadda.fina": "ﺂ",
        "alefhamza.isol": "ﺃ",
        "alefhamza.fina": "ﺄ",
        "wawhamza.isol": "ﺅ",
        "wawhamza.fina": "ﺆ",
        "alefhamzabelow.isol": "ﺇ",
        "alefhamzabelow.fina": "ﺈ",
        "yehhamza.isol": "ﺉ",
        "yehhamza.fina": "ﺊ",
        "yehhamza.init": "ﺋ",
        "yehhamza.medi": "ﺌ",
        "alef.isol": "ﺍ",
        "alef.fina": "ﺎ",
        "beh.isol": "ﺏ",
        "beh.fina": "ﺐ",
        "beh.init": "ﺑ",
        "beh.medi": "ﺒ",
        "tehmarbuta.isol": "ﺓ",
        "tehmarbuta.fina": "ﺔ",
        "teh.isol": "ﺕ",
        "teh.fina": "ﺖ",
        "teh.init": "ﺗ",
        "teh.medi": "ﺘ",
        "theh.isol": "ﺙ",
        "theh.fina": "ﺚ",
        "theh.init": "ﺛ",
        "theh.medi": "ﺜ",
        "jeem.isol": "ﺝ",
        "jeem.fina": "ﺞ",
        "jeem.init": "ﺟ",
        "jeem.medi": "ﺠ",
        "hah.isol": "ﺡ",
        "hah.fina": "ﺢ",
        "hah.init": "ﺣ",
        "hah.medi": "ﺤ",
        "khah.isol": "ﺥ",
        "khah.fina": "ﺦ",
        "khah.init": "ﺧ",
        "khah.medi": "ﺨ",
        "dal.isol": "ﺩ",
        "dal.fina": "ﺪ",
        "thal.isol": "ﺫ",
        "thal.fina": "ﺬ",
        "reh.isol": "ﺭ",
        "reh.fina": "ﺮ",
        "zain.isol": "ﺯ",
        "zain.fina": "ﺰ",
        "seen.isol": "ﺱ",
        "seen.fina": "ﺲ",
        "seen.init": "ﺳ",
        "seen.medi": "ﺴ",
        "sheen.isol": "ﺵ",
        "sheen.fina": "ﺶ",
        "sheen.init": "ﺷ",
        "sheen.medi": "ﺸ",
        "sad.isol": "ﺹ",
        "sad.fina": "ﺺ",
        "sad.init": "ﺻ",
        "sad.medi": "ﺼ",
        "dad.isol": "ﺽ",
        "dad.fina": "ﺾ",
        "dad.init": "ﺿ",
        "dad.medi": "ﻀ",
        "tah.isol": "ﻁ",
        "tah.fina": "ﻂ",
        "tah.init": "ﻃ",
        "tah.medi": "ﻄ",
        "zah.isol": "ﻅ",
        "zah.fina": "ﻆ",
        "zah.init": "ﻇ",
        "zah.medi": "ﻈ",
        "ain.isol": "ﻉ",
        "ain.fina": "ﻊ",
        "ain.init": "ﻋ",
        "ain.medi": "ﻌ",
        "ghain.isol": "ﻍ",
        "ghain.fina": "ﻎ",
        "ghain.init": "ﻏ",
        "ghain.medi": "ﻐ",
        "feh.isol": "ﻑ",
        "feh.fina": "ﻒ",
        "feh.init": "ﻓ",
        "feh.medi": "ﻔ",
        "qaf.isol": "ﻕ",
        "qaf.fina": "ﻖ",
        "qaf.init": "ﻗ",
        "qaf.medi": "ﻘ",
        "kaf.isol": "ﻙ",
        "kaf.fina": "ﻚ",
        "kaf.init": "ﻛ",
        "kaf.medi": "ﻜ",
        "lam.isol": "ﻝ",
        "lam.fina": "ﻞ",
        "lam.init": "ﻟ",
        "lam.medi": "ﻠ",
        "meem.isol": "ﻡ",
        "meem.fina": "ﻢ",
        "meem.init": "ﻣ",
        "meem.medi": "ﻤ",
        "noon.isol": "ﻥ",
        "noon.fina": "ﻦ",
        "noon.init": "ﻧ",
        "noon.medi": "ﻨ",
        "heh.isol": "ﻩ",
        "heh.fina": "ﻪ",
        "heh.init": "ﻫ",
        "heh.medi": "ﻬ",
        "waw.isol": "ﻭ",
        "waw.fina": "ﻮ",
        "alefmaksura.isol": "ﻯ",
        "alefmaksura.fina": "ﻰ",
        "yeh.isol": "ﻱ",
        "yeh.fina": "ﻲ",
        "yeh.init": "ﻳ",
        "yeh.medi": "ﻴ",
        "lam.init_alef.medi_maddaabove.fina": "ﻵ",
        "lam.medi_alef.medi_maddaabove.fina": "ﻶ",
        "lam.init_alef.medi_hamzaabove.fina": "ﻷ",
        "lam.medi_alef.medi_hamzaabove.fina": "ﻸ",
        "lam.init_alef.medi_hamzabelow.fina": "ﻹ",
        "lam.medi_alef.medi_hamzabelow.fina": "ﻺ",
        "lam.init_alef.fina": "ﻻ",
        "lam.medi_alef.fina": "ﻼ",
        "zerowidthnobreakspace": "\ufeff",
        "fwd:exclam": "！",
        "fwd:quotedbl": "＂",
        "fwd:numbersign": "＃",
        "fwd:dollar": "＄",
        "fwd:percent": "％",
        "fwd:ampersand": "＆",
        "fwd:quotesingle": "＇",
        "fwd:parenthesisleft": "（",
        "fwd:parenthesisright": "）",
        "fwd:asterisk": "＊",
        "fwd:plus": "＋",
        "fwd:comma": "，",
        "fwd:hyphen": "－",
        "fwd:period": "．",
        "fwd:slash": "／",
        "fwd:zero": "０",
        "fwd:one": "１",
        "fwd:two": "２",
        "fwd:three": "３",
        "fwd:four": "４",
        "fwd:five": "５",
        "fwd:six": "６",
        "fwd:seven": "７",
        "fwd:eight": "８",
        "fwd:nine": "９",
        "fwd:colon": "：",
        "fwd:semicolon": "；",
        "fwd:less": "＜",
        "fwd:equal": "＝",
        "fwd:greater": "＞",
        "fwd:question": "？",
        "fwd:at": "＠",
        "fwd:A": "Ａ",
        "fwd:B": "Ｂ",
        "fwd:C": "Ｃ",
        "fwd:D": "Ｄ",
        "fwd:E": "Ｅ",
        "fwd:F": "Ｆ",
        "fwd:G": "Ｇ",
        "fwd:H": "Ｈ",
        "fwd:I": "Ｉ",
        "fwd:J": "Ｊ",
        "fwd:K": "Ｋ",
        "fwd:L": "Ｌ",
        "fwd:M": "Ｍ",
        "fwd:N": "Ｎ",
        "fwd:O": "Ｏ",
        "fwd:P": "Ｐ",
        "fwd:Q": "Ｑ",
        "fwd:R": "Ｒ",
        "fwd:S": "Ｓ",
        "fwd:T": "Ｔ",
        "fwd:U": "Ｕ",
        "fwd:V": "Ｖ",
        "fwd:W": "Ｗ",
        "fwd:X": "Ｘ",
        "fwd:Y": "Ｙ",
        "fwd:Z": "Ｚ",
        "fwd:bracketleft": "［",
        "fwd:backslash": "＼",
        "fwd:bracketright": "］",
        "fwd:asciicircum": "＾",
        "fwd:underscore": "＿",
        "fwd:grave": "｀",
        "fwd:a": "ａ",
        "fwd:b": "ｂ",
        "fwd:c": "ｃ",
        "fwd:d": "ｄ",
        "fwd:e": "ｅ",
        "fwd:f": "ｆ",
        "fwd:g": "ｇ",
        "fwd:h": "ｈ",
        "fwd:i": "ｉ",
        "fwd:j": "ｊ",
        "fwd:k": "ｋ",
        "fwd:l": "ｌ",
        "fwd:m": "ｍ",
        "fwd:n": "ｎ",
        "fwd:o": "ｏ",
        "fwd:p": "ｐ",
        "fwd:q": "ｑ",
        "fwd:r": "ｒ",
        "fwd:s": "ｓ",
        "fwd:t": "ｔ",
        "fwd:u": "ｕ",
        "fwd:v": "ｖ",
        "fwd:w": "ｗ",
        "fwd:x": "ｘ",
        "fwd:y": "ｙ",
        "fwd:z": "ｚ",
        "fwd:braceleft": "｛",
        "fwd:bar": "｜",
        "fwd:braceright": "｝",
        "fwd:asciitilde": "～",
        "fwd:leftwhiteparenthesis": "｟",
        "fwd:rightwhiteparenthesis": "｠",
        "hwd:ideographicfullstop": "｡",
        "hwd:leftcornerbracket": "｢",
        "hwd:rightcornerbracket": "｣",
        "hwd:ideographiccomma": "､",
        "hwd:kata:middledot": "･",
        "hwd:kata:wo": "ｦ",
        "hwd:kata:asmall": "ｧ",
        "hwd:kata:ismall": "ｨ",
        "hwd:kata:usmall": "ｩ",
        "hwd:kata:esmall": "ｪ",
        "hwd:kata:osmall": "ｫ",
        "hwd:kata:yasmall": "ｬ",
        "hwd:kata:yusmall": "ｭ",
        "hwd:kata:yosmall": "ｮ",
        "hwd:kata:tusmall": "ｯ",
        "hwd:kata:prolongedkana": "ｰ",
        "hwd:kata:a": "ｱ",
        "hwd:kata:i": "ｲ",
        "hwd:kata:u": "ｳ",
        "hwd:kata:e": "ｴ",
        "hwd:kata:o": "ｵ",
        "hwd:kata:ka": "ｶ",
        "hwd:kata:ki": "ｷ",
        "hwd:kata:ku": "ｸ",
        "hwd:kata:ke": "ｹ",
        "hwd:kata:ko": "ｺ",
        "hwd:kata:sa": "ｻ",
        "hwd:kata:si": "ｼ",
        "hwd:kata:su": "ｽ",
        "hwd:kata:se": "ｾ",
        "hwd:kata:so": "ｿ",
        "hwd:kata:ta": "ﾀ",
        "hwd:kata:ti": "ﾁ",
        "hwd:kata:tu": "ﾂ",
        "hwd:kata:te": "ﾃ",
        "hwd:kata:to": "ﾄ",
        "hwd:kata:na": "ﾅ",
        "hwd:kata:ni": "ﾆ",
        "hwd:kata:nu": "ﾇ",
        "hwd:kata:ne": "ﾈ",
        "hwd:kata:no": "ﾉ",
        "hwd:kata:ha": "ﾊ",
        "hwd:kata:hi": "ﾋ",
        "hwd:kata:hu": "ﾌ",
        "hwd:kata:he": "ﾍ",
        "hwd:kata:ho": "ﾎ",
        "hwd:kata:ma": "ﾏ",
        "hwd:kata:mi": "ﾐ",
        "hwd:kata:mu": "ﾑ",
        "hwd:kata:me": "ﾒ",
        "hwd:kata:mo": "ﾓ",
        "hwd:kata:ya": "ﾔ",
        "hwd:kata:yu": "ﾕ",
        "hwd:kata:yo": "ﾖ",
        "hwd:kata:ra": "ﾗ",
        "hwd:kata:ri": "ﾘ",
        "hwd:kata:ru": "ﾙ",
        "hwd:kata:re": "ﾚ",
        "hwd:kata:ro": "ﾛ",
        "hwd:kata:wa": "ﾜ",
        "hwd:kata:n": "ﾝ",
        "hwd:kata:voiced": "ﾞ",
        "hwd:kata:semi-voiced": "ﾟ",
        "hwd:hangulfiller": "ﾠ",
        "hwd:kiyeok": "ﾡ",
        "hwd:ssangkiyeok": "ﾢ",
        "hwd:kiyeoksios": "ﾣ",
        "hwd:nieun": "ﾤ",
        "hwd:nieuncieuc": "ﾥ",
        "hwd:nieunhieuh": "ﾦ",
        "hwd:tikeut": "ﾧ",
        "hwd:ssangtikeut": "ﾨ",
        "hwd:rieul": "ﾩ",
        "hwd:rieulkiyeok": "ﾪ",
        "hwd:rieulmieum": "ﾫ",
        "hwd:rieulpieup": "ﾬ",
        "hwd:rieulsios": "ﾭ",
        "hwd:rieulthieuth": "ﾮ",
        "hwd:rieulphieuph": "ﾯ",
        "hwd:rieulhieuh": "ﾰ",
        "hwd:mieum": "ﾱ",
        "hwd:pieup": "ﾲ",
        "hwd:ssangpieup": "ﾳ",
        "hwd:pieupsios": "ﾴ",
        "hwd:sios": "ﾵ",
        "hwd:ssangsios": "ﾶ",
        "hwd:ieung": "ﾷ",
        "hwd:cieuc": "ﾸ",
        "hwd:ssangcieuc": "ﾹ",
        "hwd:chieuch": "ﾺ",
        "hwd:khieukh": "ﾻ",
        "hwd:thieuth": "ﾼ",
        "hwd:phieuph": "ﾽ",
        "hwd:hieuh": "ﾾ",
        "hwd:a": "ￂ",
        "hwd:ae": "ￃ",
        "hwd:ya": "ￄ",
        "hwd:yae": "ￅ",
        "hwd:eo": "ￆ",
        "hwd:e": "ￇ",
        "hwd:yeo": "ￊ",
        "hwd:ye": "ￋ",
        "hwd:o": "ￌ",
        "hwd:wa": "ￍ",
        "hwd:wae": "ￎ",
        "hwd:oe": "ￏ",
        "hwd:yo": "ￒ",
        "hwd:u": "ￓ",
        "hwd:weo": "ￔ",
        "hwd:we": "ￕ",
        "hwd:wi": "ￖ",
        "hwd:yu": "ￗ",
        "hwd:eu": "ￚ",
        "hwd:yi": "ￛ",
        "hwd:i": "ￜ",
        "fwd:centsign": "￠",
        "fwd:poundsign": "￡",
        "fwd:notsign": "￢",
        "fwd:macron": "￣",
        "fwd:brokenbar": "￤",
        "fwd:yensign": "￥",
        "fwd:wonsign": "￦",
        "hwd:formslightvertical": "￨",
        "hwd:leftwardsarrow": "￩",
        "hwd:upwardsarrow": "￪",
        "hwd:rightwardsarrow": "￫",
        "hwd:downwardsarrow": "￬",
        "hwd:blacksquare": "￭",
        "hwd:whitecircle": "￮",
        "interlinearanchor": "\ufff9",
        "interlinearseparator": "\ufffa",
        "interlinearterminator": "\ufffb",
        "replacementcharobj": "￼",
        "replacementchar": "�",
        "zanb:a": "𑨀",
        "zanb:vowelsigni": "𑨁",
        "zanb:vowelsignue": "𑨂",
        "zanb:vowelsignu": "𑨃",
        "zanb:vowelsigne": "𑨄",
        "zanb:vowelsignoe": "𑨅",
        "zanb:vowelsigno": "𑨆",
        "zanb:vowelsignai": "𑨇",
        "zanb:vowelsignau": "𑨈",
        "zanb:vowelsignreversedi": "𑨉",
        "zanb:vowellengthmark": "𑨊",
        "zanb:ka": "𑨋",
        "zanb:kha": "𑨌",
        "zanb:ga": "𑨍",
        "zanb:gha": "𑨎",
        "zanb:nga": "𑨏",
        "zanb:ca": "𑨐",
        "zanb:cha": "𑨑",
        "zanb:ja": "𑨒",
        "zanb:nya": "𑨓",
        "zanb:tta": "𑨔",
        "zanb:ttha": "𑨕",
        "zanb:dda": "𑨖",
        "zanb:ddha": "𑨗",
        "zanb:nna": "𑨘",
        "zanb:ta": "𑨙",
        "zanb:tha": "𑨚",
        "zanb:da": "𑨛",
        "zanb:dha": "𑨜",
        "zanb:na": "𑨝",
        "zanb:pa": "𑨞",
        "zanb:pha": "𑨟",
        "zanb:ba": "𑨠",
        "zanb:bha": "𑨡",
        "zanb:ma": "𑨢",
        "zanb:tsa": "𑨣",
        "zanb:tsha": "𑨤",
        "zanb:dza": "𑨥",
        "zanb:dzha": "𑨦",
        "zanb:zha": "𑨧",
        "zanb:za": "𑨨",
        "zanb:dashA": "𑨩",
        "zanb:ya": "𑨪",
        "zanb:ra": "𑨫",
        "zanb:la": "𑨬",
        "zanb:va": "𑨭",
        "zanb:sha": "𑨮",
        "zanb:ssa": "𑨯",
        "zanb:sa": "𑨰",
        "zanb:ha": "𑨱",
        "zanb:kssa": "𑨲",
        "zanb:finalconsonantmark": "𑨳",
        "zanb:signvirama": "𑨴",
        "zanb:signcandrabindu": "𑨵",
        "zanb:signcandrabinduwithornament": "𑨶",
        "zanb:signcandrawithornament": "𑨷",
        "zanb:signanusvara": "𑨸",
        "zanb:signvisarga": "𑨹",
        "zanb:raclusterinit": "𑨺",
        "zanb:yaclusterfina": "𑨻",
        "zanb:raclusterfina": "𑨼",
        "zanb:laclusterfina": "𑨽",
        "zanb:vaclusterfina": "𑨾",
        "zanb:initialheadmark": "𑨿",
        "zanb:closingheadmark": "𑩀",
        "zanb:marktsheg": "𑩁",
        "zanb:markshad": "𑩂",
        "zanb:markdoubleshad": "𑩃",
        "zanb:marklongtsheg": "𑩄",
        "zanb:initialheadmarkdbllined": "𑩅",
        "zanb:closingheadmarkdbllined": "𑩆",
        "zanb:subjoiner": "𑩇",
        "Abold": "𝐀",
        "Bbold": "𝐁",
        "Cbold": "𝐂",
        "Dbold": "𝐃",
        "Ebold": "𝐄",
        "Fbold": "𝐅",
        "Gbold": "𝐆",
        "Hbold": "𝐇",
        "Ibold": "𝐈",
        "Jbold": "𝐉",
        "Kbold": "𝐊",
        "Lbold": "𝐋",
        "Mbold": "𝐌",
        "Nbold": "𝐍",
        "Obold": "𝐎",
        "Pbold": "𝐏",
        "Qbold": "𝐐",
        "Rbold": "𝐑",
        "Sbold": "𝐒",
        "Tbold": "𝐓",
        "Ubold": "𝐔",
        "Vbold": "𝐕",
        "Wbold": "𝐖",
        "Xbold": "𝐗",
        "Ybold": "𝐘",
        "Zbold": "𝐙",
        "abold": "𝐚",
        "bbold": "𝐛",
        "cbold": "𝐜",
        "dbold": "𝐝",
        "ebold": "𝐞",
        "fbold": "𝐟",
        "gbold": "𝐠",
        "hbold": "𝐡",
        "ibold": "𝐢",
        "jbold": "𝐣",
        "kbold": "𝐤",
        "lbold": "𝐥",
        "mbold": "𝐦",
        "nbold": "𝐧",
        "obold": "𝐨",
        "pbold": "𝐩",
        "qbold": "𝐪",
        "rbold": "𝐫",
        "sbold": "𝐬",
        "tbold": "𝐭",
        "ubold": "𝐮",
        "vbold": "𝐯",
        "wbold": "𝐰",
        "xbold": "𝐱",
        "ybold": "𝐲",
        "zbold": "𝐳",
        "Aitalic": "𝐴",
        "Bitalic": "𝐵",
        "Citalic": "𝐶",
        "Ditalic": "𝐷",
        "Eitalic": "𝐸",
        "Fitalic": "𝐹",
        "Gitalic": "𝐺",
        "Hitalic": "𝐻",
        "Iitalic": "𝐼",
        "Jitalic": "𝐽",
        "Kitalic": "𝐾",
        "Litalic": "𝐿",
        "Mitalic": "𝑀",
        "Nitalic": "𝑁",
        "Oitalic": "𝑂",
        "Pitalic": "𝑃",
        "Qitalic": "𝑄",
        "Ritalic": "𝑅",
        "Sitalic": "𝑆",
        "Titalic": "𝑇",
        "Uitalic": "𝑈",
        "Vitalic": "𝑉",
        "Witalic": "𝑊",
        "Xitalic": "𝑋",
        "Yitalic": "𝑌",
        "Zitalic": "𝑍",
        "aitalic": "𝑎",
        "bitalic": "𝑏",
        "citalic": "𝑐",
        "ditalic": "𝑑",
        "eitalic": "𝑒",
        "fitalic": "𝑓",
        "gitalic": "𝑔",
        "iitalic": "𝑖",
        "jitalic": "𝑗",
        "kitalic": "𝑘",
        "litalic": "𝑙",
        "mitalic": "𝑚",
        "nitalic": "𝑛",
        "oitalic": "𝑜",
        "pitalic": "𝑝",
        "qitalic": "𝑞",
        "ritalic": "𝑟",
        "sitalic": "𝑠",
        "titalic": "𝑡",
        "uitalic": "𝑢",
        "vitalic": "𝑣",
        "witalic": "𝑤",
        "xitalic": "𝑥",
        "yitalic": "𝑦",
        "zitalic": "𝑧",
        "Abolditalic": "𝑨",
        "Bbolditalic": "𝑩",
        "Cbolditalic": "𝑪",
        "Dbolditalic": "𝑫",
        "Ebolditalic": "𝑬",
        "Fbolditalic": "𝑭",
        "Gbolditalic": "𝑮",
        "Hbolditalic": "𝑯",
        "Ibolditalic": "𝑰",
        "Jbolditalic": "𝑱",
        "Kbolditalic": "𝑲",
        "Lbolditalic": "𝑳",
        "Mbolditalic": "𝑴",
        "Nbolditalic": "𝑵",
        "Obolditalic": "𝑶",
        "Pbolditalic": "𝑷",
        "Qbolditalic": "𝑸",
        "Rbolditalic": "𝑹",
        "Sbolditalic": "𝑺",
        "Tbolditalic": "𝑻",
        "Ubolditalic": "𝑼",
        "Vbolditalic": "𝑽",
        "Wbolditalic": "𝑾",
        "Xbolditalic": "𝑿",
        "Ybolditalic": "𝒀",
        "Zbolditalic": "𝒁",
        "abolditalic": "𝒂",
        "bbolditalic": "𝒃",
        "cbolditalic": "𝒄",
        "dbolditalic": "𝒅",
        "ebolditalic": "𝒆",
        "fbolditalic": "𝒇",
        "gbolditalic": "𝒈",
        "hbolditalic": "𝒉",
        "ibolditalic": "𝒊",
        "jbolditalic": "𝒋",
        "kbolditalic": "𝒌",
        "lbolditalic": "𝒍",
        "mbolditalic": "𝒎",
        "nbolditalic": "𝒏",
        "obolditalic": "𝒐",
        "pbolditalic": "𝒑",
        "qbolditalic": "𝒒",
        "rbolditalic": "𝒓",
        "sbolditalic": "𝒔",
        "tbolditalic": "𝒕",
        "ubolditalic": "𝒖",
        "vbolditalic": "𝒗",
        "wbolditalic": "𝒘",
        "xbolditalic": "𝒙",
        "ybolditalic": "𝒚",
        "zbolditalic": "𝒛",
        "Ascript": "𝒜",
        "Cscript": "𝒞",
        "Dscript": "𝒟",
        "Gscript": "𝒢",
        "Jscript": "𝒥",
        "Kscript": "𝒦",
        "Nscript": "𝒩",
        "Oscript": "𝒪",
        "Pscript": "𝒫",
        "Qscript": "𝒬",
        "Sscript": "𝒮",
        "Tscript": "𝒯",
        "Uscript": "𝒰",
        "Vscript": "𝒱",
        "Wscript": "𝒲",
        "Xscript": "𝒳",
        "Yscript": "𝒴",
        "Zscript": "𝒵",
        "ascript": "𝒶",
        "bscript": "𝒷",
        "cscript": "𝒸",
        "dscript": "𝒹",
        "fscript": "𝒻",
        "hscript": "𝒽",
        "iscript": "𝒾",
        "jscript": "𝒿",
        "kscript": "𝓀",
        "lscript": "𝓁",
        "mscript": "𝓂",
        "nscript": "𝓃",
        "pscript": "𝓅",
        "qscript": "𝓆",
        "math:rscript": "𝓇",
        "sscript": "𝓈",
        "tscript": "𝓉",
        "uscript": "𝓊",
        "vscript": "𝓋",
        "wscript": "𝓌",
        "xscript": "𝓍",
        "yscript": "𝓎",
        "zscript": "𝓏",
        "Aboldscript": "𝓐",
        "Bboldscript": "𝓑",
        "Cboldscript": "𝓒",
        "Dboldscript": "𝓓",
        "Eboldscript": "𝓔",
        "Fboldscript": "𝓕",
        "Gboldscript": "𝓖",
        "Hboldscript": "𝓗",
        "Iboldscript": "𝓘",
        "Jboldscript": "𝓙",
        "Kboldscript": "𝓚",
        "Lboldscript": "𝓛",
        "Mboldscript": "𝓜",
        "Nboldscript": "𝓝",
        "Oboldscript": "𝓞",
        "Pboldscript": "𝓟",
        "Qboldscript": "𝓠",
        "Rboldscript": "𝓡",
        "Sboldscript": "𝓢",
        "Tboldscript": "𝓣",
        "Uboldscript": "𝓤",
        "Vboldscript": "𝓥",
        "Wboldscript": "𝓦",
        "Xboldscript": "𝓧",
        "Yboldscript": "𝓨",
        "Zboldscript": "𝓩",
        "aboldscript": "𝓪",
        "bboldscript": "𝓫",
        "cboldscript": "𝓬",
        "dboldscript": "𝓭",
        "eboldscript": "𝓮",
        "fboldscript": "𝓯",
        "gboldscript": "𝓰",
        "hboldscript": "𝓱",
        "iboldscript": "𝓲",
        "jboldscript": "𝓳",
        "kboldscript": "𝓴",
        "lboldscript": "𝓵",
        "mboldscript": "𝓶",
        "nboldscript": "𝓷",
        "oboldscript": "𝓸",
        "pboldscript": "𝓹",
        "qboldscript": "𝓺",
        "rboldscript": "𝓻",
        "sboldscript": "𝓼",
        "tboldscript": "𝓽",
        "uboldscript": "𝓾",
        "vboldscript": "𝓿",
        "wboldscript": "𝔀",
        "xboldscript": "𝔁",
        "yboldscript": "𝔂",
        "zboldscript": "𝔃",
        "Afraktur": "𝔄",
        "Bfraktur": "𝔅",
        "Dfraktur": "𝔇",
        "Efraktur": "𝔈",
        "Ffraktur": "𝔉",
        "Gfraktur": "𝔊",
        "Jfraktur": "𝔍",
        "Kfraktur": "𝔎",
        "Lfraktur": "𝔏",
        "Mfraktur": "𝔐",
        "Nfraktur": "𝔑",
        "Ofraktur": "𝔒",
        "Pfraktur": "𝔓",
        "Qfraktur": "𝔔",
        "Sfraktur": "𝔖",
        "Tfraktur": "𝔗",
        "Ufraktur": "𝔘",
        "Vfraktur": "𝔙",
        "Wfraktur": "𝔚",
        "Xfraktur": "𝔛",
        "Yfraktur": "𝔜",
        "afraktur": "𝔞",
        "bfraktur": "𝔟",
        "cfraktur": "𝔠",
        "dfraktur": "𝔡",
        "efraktur": "𝔢",
        "ffraktur": "𝔣",
        "gfraktur": "𝔤",
        "hfraktur": "𝔥",
        "ifraktur": "𝔦",
        "jfraktur": "𝔧",
        "kfraktur": "𝔨",
        "lfraktur": "𝔩",
        "mfraktur": "𝔪",
        "nfraktur": "𝔫",
        "ofraktur": "𝔬",
        "pfraktur": "𝔭",
        "qfraktur": "𝔮",
        "rfraktur": "𝔯",
        "sfraktur": "𝔰",
        "tfraktur": "𝔱",
        "ufraktur": "𝔲",
        "vfraktur": "𝔳",
        "wfraktur": "𝔴",
        "xfraktur": "𝔵",
        "yfraktur": "𝔶",
        "zfraktur": "𝔷",
        "Adblstruck": "𝔸",
        "Bdblstruck": "𝔹",
        "Ddblstruck": "𝔻",
        "Edblstruck": "𝔼",
        "Fdblstruck": "𝔽",
        "Gdblstruck": "𝔾",
        "Idblstruck": "𝕀",
        "Jdblstruck": "𝕁",
        "Kdblstruck": "𝕂",
        "Ldblstruck": "𝕃",
        "Mdblstruck": "𝕄",
        "Odblstruck": "𝕆",
        "Sdblstruck": "𝕊",
        "Tdblstruck": "𝕋",
        "Udblstruck": "𝕌",
        "Vdblstruck": "𝕍",
        "Wdblstruck": "𝕎",
        "Xdblstruck": "𝕏",
        "Ydblstruck": "𝕐",
        "adblstruck": "𝕒",
        "bdblstruck": "𝕓",
        "cdblstruck": "𝕔",
        "ddblstruck": "𝕕",
        "edblstruck": "𝕖",
        "fdblstruck": "𝕗",
        "gdblstruck": "𝕘",
        "hdblstruck": "𝕙",
        "idblstruck": "𝕚",
        "jdblstruck": "𝕛",
        "kdblstruck": "𝕜",
        "ldblstruck": "𝕝",
        "mdblstruck": "𝕞",
        "ndblstruck": "𝕟",
        "odblstruck": "𝕠",
        "pdblstruck": "𝕡",
        "qdblstruck": "𝕢",
        "rdblstruck": "𝕣",
        "sdblstruck": "𝕤",
        "tdblstruck": "𝕥",
        "udblstruck": "𝕦",
        "vdblstruck": "𝕧",
        "wdblstruck": "𝕨",
        "xdblstruck": "𝕩",
        "ydblstruck": "𝕪",
        "zdblstruck": "𝕫",
        "Aboldfraktur": "𝕬",
        "Bboldfraktur": "𝕭",
        "Cboldfraktur": "𝕮",
        "Dboldfraktur": "𝕯",
        "Eboldfraktur": "𝕰",
        "Fboldfraktur": "𝕱",
        "Gboldfraktur": "𝕲",
        "Hboldfraktur": "𝕳",
        "Iboldfraktur": "𝕴",
        "Jboldfraktur": "𝕵",
        "Kboldfraktur": "𝕶",
        "Lboldfraktur": "𝕷",
        "Mboldfraktur": "𝕸",
        "Nboldfraktur": "𝕹",
        "Oboldfraktur": "𝕺",
        "Pboldfraktur": "𝕻",
        "Qboldfraktur": "𝕼",
        "Rboldfraktur": "𝕽",
        "Sboldfraktur": "𝕾",
        "Tboldfraktur": "𝕿",
        "Uboldfraktur": "𝖀",
        "Vboldfraktur": "𝖁",
        "Wboldfraktur": "𝖂",
        "Xboldfraktur": "𝖃",
        "Yboldfraktur": "𝖄",
        "Zboldfraktur": "𝖅",
        "aboldfraktur": "𝖆",
        "bboldfraktur": "𝖇",
        "cboldfraktur": "𝖈",
        "dboldfraktur": "𝖉",
        "eboldfraktur": "𝖊",
        "fboldfraktur": "𝖋",
        "gboldfraktur": "𝖌",
        "hboldfraktur": "𝖍",
        "iboldfraktur": "𝖎",
        "jboldfraktur": "𝖏",
        "kboldfraktur": "𝖐",
        "lboldfraktur": "𝖑",
        "mboldfraktur": "𝖒",
        "nboldfraktur": "𝖓",
        "oboldfraktur": "𝖔",
        "pboldfraktur": "𝖕",
        "qboldfraktur": "𝖖",
        "rboldfraktur": "𝖗",
        "sboldfraktur": "𝖘",
        "tboldfraktur": "𝖙",
        "uboldfraktur": "𝖚",
        "vboldfraktur": "𝖛",
        "wboldfraktur": "𝖜",
        "xboldfraktur": "𝖝",
        "yboldfraktur": "𝖞",
        "zboldfraktur": "𝖟",
        "Asans": "𝖠",
        "Bsans": "𝖡",
        "Csans": "𝖢",
        "Dsans": "𝖣",
        "Esans": "𝖤",
        "Fsans": "𝖥",
        "Gsans": "𝖦",
        "Hsans": "𝖧",
        "Isans": "𝖨",
        "Jsans": "𝖩",
        "Ksans": "𝖪",
        "Lsans": "𝖫",
        "Msans": "𝖬",
        "Nsans": "𝖭",
        "Osans": "𝖮",
        "Psans": "𝖯",
        "Qsans": "𝖰",
        "Rsans": "𝖱",
        "Ssans": "𝖲",
        "Tsans": "𝖳",
        "Usans": "𝖴",
        "Vsans": "𝖵",
        "Wsans": "𝖶",
        "Xsans": "𝖷",
        "Ysans": "𝖸",
        "Zsans": "𝖹",
        "asans": "𝖺",
        "bsans": "𝖻",
        "csans": "𝖼",
        "dsans": "𝖽",
        "esans": "𝖾",
        "fsans": "𝖿",
        "gsans": "𝗀",
        "hsans": "𝗁",
        "isans": "𝗂",
        "jsans": "𝗃",
        "ksans": "𝗄",
        "lsans": "𝗅",
        "msans": "𝗆",
        "nsans": "𝗇",
        "osans": "𝗈",
        "psans": "𝗉",
        "qsans": "𝗊",
        "rsans": "𝗋",
        "ssans": "𝗌",
        "tsans": "𝗍",
        "usans": "𝗎",
        "vsans": "𝗏",
        "wsans": "𝗐",
        "xsans": "𝗑",
        "ysans": "𝗒",
        "zsans": "𝗓",
        "Aboldsans": "𝗔",
        "Bboldsans": "𝗕",
        "Cboldsans": "𝗖",
        "Dboldsans": "𝗗",
        "Eboldsans": "𝗘",
        "Fboldsans": "𝗙",
        "Gboldsans": "𝗚",
        "Hboldsans": "𝗛",
        "Iboldsans": "𝗜",
        "Jboldsans": "𝗝",
        "Kboldsans": "𝗞",
        "Lboldsans": "𝗟",
        "Mboldsans": "𝗠",
        "Nboldsans": "𝗡",
        "Oboldsans": "𝗢",
        "Pboldsans": "𝗣",
        "Qboldsans": "𝗤",
        "Rboldsans": "𝗥",
        "Sboldsans": "𝗦",
        "Tboldsans": "𝗧",
        "Uboldsans": "𝗨",
        "Vboldsans": "𝗩",
        "Wboldsans": "𝗪",
        "Xboldsans": "𝗫",
        "Yboldsans": "𝗬",
        "Zboldsans": "𝗭",
        "aboldsans": "𝗮",
        "bboldsans": "𝗯",
        "cboldsans": "𝗰",
        "dboldsans": "𝗱",
        "eboldsans": "𝗲",
        "fboldsans": "𝗳",
        "gboldsans": "𝗴",
        "hboldsans": "𝗵",
        "iboldsans": "𝗶",
        "jboldsans": "𝗷",
        "kboldsans": "𝗸",
        "lboldsans": "𝗹",
        "mboldsans": "𝗺",
        "nboldsans": "𝗻",
        "oboldsans": "𝗼",
        "pboldsans": "𝗽",
        "qboldsans": "𝗾",
        "rboldsans": "𝗿",
        "sboldsans": "𝘀",
        "tboldsans": "𝘁",
        "uboldsans": "𝘂",
        "vboldsans": "𝘃",
        "wboldsans": "𝘄",
        "xboldsans": "𝘅",
        "yboldsans": "𝘆",
        "zboldsans": "𝘇",
        "Aitalicsans": "𝘈",
        "Bitalicsans": "𝘉",
        "Citalicsans": "𝘊",
        "Ditalicsans": "𝘋",
        "Eitalicsans": "𝘌",
        "Fitalicsans": "𝘍",
        "Gitalicsans": "𝘎",
        "Hitalicsans": "𝘏",
        "Iitalicsans": "𝘐",
        "Jitalicsans": "𝘑",
        "Kitalicsans": "𝘒",
        "Litalicsans": "𝘓",
        "Mitalicsans": "𝘔",
        "Nitalicsans": "𝘕",
        "Oitalicsans": "𝘖",
        "Pitalicsans": "𝘗",
        "Qitalicsans": "𝘘",
        "Ritalicsans": "𝘙",
        "Sitalicsans": "𝘚",
        "Titalicsans": "𝘛",
        "Uitalicsans": "𝘜",
        "Vitalicsans": "𝘝",
        "Witalicsans": "𝘞",
        "Xitalicsans": "𝘟",
        "Yitalicsans": "𝘠",
        "Zitalicsans": "𝘡",
        "aitalicsans": "𝘢",
        "bitalicsans": "𝘣",
        "citalicsans": "𝘤",
        "ditalicsans": "𝘥",
        "eitalicsans": "𝘦",
        "fitalicsans": "𝘧",
        "gitalicsans": "𝘨",
        "hitalicsans": "𝘩",
        "iitalicsans": "𝘪",
        "jitalicsans": "𝘫",
        "kitalicsans": "𝘬",
        "litalicsans": "𝘭",
        "mitalicsans": "𝘮",
        "nitalicsans": "𝘯",
        "oitalicsans": "𝘰",
        "pitalicsans": "𝘱",
        "qitalicsans": "𝘲",
        "ritalicsans": "𝘳",
        "sitalicsans": "𝘴",
        "titalicsans": "𝘵",
        "uitalicsans": "𝘶",
        "vitalicsans": "𝘷",
        "witalicsans": "𝘸",
        "xitalicsans": "𝘹",
        "yitalicsans": "𝘺",
        "zitalicsans": "𝘻",
        "Abolditalicsans": "𝘼",
        "Bbolditalicsans": "𝘽",
        "Cbolditalicsans": "𝘾",
        "Dbolditalicsans": "𝘿",
        "Ebolditalicsans": "𝙀",
        "Fbolditalicsans": "𝙁",
        "Gbolditalicsans": "𝙂",
        "Hbolditalicsans": "𝙃",
        "Ibolditalicsans": "𝙄",
        "Jbolditalicsans": "𝙅",
        "Kbolditalicsans": "𝙆",
        "Lbolditalicsans": "𝙇",
        "Mbolditalicsans": "𝙈",
        "Nbolditalicsans": "𝙉",
        "Obolditalicsans": "𝙊",
        "Pbolditalicsans": "𝙋",
        "Qbolditalicsans": "𝙌",
        "Rbolditalicsans": "𝙍",
        "Sbolditalicsans": "𝙎",
        "Tbolditalicsans": "𝙏",
        "Ubolditalicsans": "𝙐",
        "Vbolditalicsans": "𝙑",
        "Wbolditalicsans": "𝙒",
        "Xbolditalicsans": "𝙓",
        "Ybolditalicsans": "𝙔",
        "Zbolditalicsans": "𝙕",
        "abolditalicsans": "𝙖",
        "bbolditalicsans": "𝙗",
        "cbolditalicsans": "𝙘",
        "dbolditalicsans": "𝙙",
        "ebolditalicsans": "𝙚",
        "fbolditalicsans": "𝙛",
        "gbolditalicsans": "𝙜",
        "hbolditalicsans": "𝙝",
        "ibolditalicsans": "𝙞",
        "jbolditalicsans": "𝙟",
        "kbolditalicsans": "𝙠",
        "lbolditalicsans": "𝙡",
        "mbolditalicsans": "𝙢",
        "nbolditalicsans": "𝙣",
        "obolditalicsans": "𝙤",
        "pbolditalicsans": "𝙥",
        "qbolditalicsans": "𝙦",
        "rbolditalicsans": "𝙧",
        "sbolditalicsans": "𝙨",
        "tbolditalicsans": "𝙩",
        "ubolditalicsans": "𝙪",
        "vbolditalicsans": "𝙫",
        "wbolditalicsans": "𝙬",
        "xbolditalicsans": "𝙭",
        "ybolditalicsans": "𝙮",
        "zbolditalicsans": "𝙯",
        "Amono": "𝙰",
        "Bmono": "𝙱",
        "Cmono": "𝙲",
        "Dmono": "𝙳",
        "Emono": "𝙴",
        "Fmono": "𝙵",
        "Gmono": "𝙶",
        "Hmono": "𝙷",
        "Imono": "𝙸",
        "Jmono": "𝙹",
        "Kmono": "𝙺",
        "Lmono": "𝙻",
        "Mmono": "𝙼",
        "Nmono": "𝙽",
        "Omono": "𝙾",
        "Pmono": "𝙿",
        "Qmono": "𝚀",
        "Rmono": "𝚁",
        "Smono": "𝚂",
        "Tmono": "𝚃",
        "Umono": "𝚄",
        "Vmono": "𝚅",
        "Wmono": "𝚆",
        "Xmono": "𝚇",
        "Ymono": "𝚈",
        "Zmono": "𝚉",
        "amono": "𝚊",
        "bmono": "𝚋",
        "cmono": "𝚌",
        "dmono": "𝚍",
        "emono": "𝚎",
        "fmono": "𝚏",
        "gmono": "𝚐",
        "hmono": "𝚑",
        "imono": "𝚒",
        "jmono": "𝚓",
        "kmono": "𝚔",
        "lmono": "𝚕",
        "mmono": "𝚖",
        "nmono": "𝚗",
        "omono": "𝚘",
        "pmono": "𝚙",
        "qmono": "𝚚",
        "rmono": "𝚛",
        "smono": "𝚜",
        "tmono": "𝚝",
        "umono": "𝚞",
        "vmono": "𝚟",
        "wmono": "𝚠",
        "xmono": "𝚡",
        "ymono": "𝚢",
        "zmono": "𝚣",
        "dotlessiitalic": "𝚤",
        "dotlessjitalic": "𝚥",
        "Alphabold": "𝚨",
        "Betabold": "𝚩",
        "Gammabold": "𝚪",
        "Deltabold": "𝚫",
        "Epsilonbold": "𝚬",
        "Zetabold": "𝚭",
        "Etabold": "𝚮",
        "Thetabold": "𝚯",
        "Iotabold": "𝚰",
        "Kappabold": "𝚱",
        "Lamdabold": "𝚲",
        "Mubold": "𝚳",
        "Nubold": "𝚴",
        "Xibold": "𝚵",
        "Omicronbold": "𝚶",
        "Pibold": "𝚷",
        "Rhobold": "𝚸",
        "Thetasymbolbold": "𝚹",
        "Sigmabold": "𝚺",
        "Taubold": "𝚻",
        "Upsilonbold": "𝚼",
        "Phibold": "𝚽",
        "Chibold": "𝚾",
        "Psibold": "𝚿",
        "Omegabold": "𝛀",
        "boldnabla": "𝛁",
        "alphabold": "𝛂",
        "betabold": "𝛃",
        "gammabold": "𝛄",
        "deltabold": "𝛅",
        "epsilonbold": "𝛆",
        "zetabold": "𝛇",
        "etabold": "𝛈",
        "thetabold": "𝛉",
        "iotabold": "𝛊",
        "kappabold": "𝛋",
        "lamdabold": "𝛌",
        "mubold": "𝛍",
        "nubold": "𝛎",
        "xibold": "𝛏",
        "omicronbold": "𝛐",
        "pibold": "𝛑",
        "rhobold": "𝛒",
        "finalsigmabold": "𝛓",
        "sigmabold": "𝛔",
        "taubold": "𝛕",
        "upsilonbold": "𝛖",
        "phibold": "𝛗",
        "chibold": "𝛘",
        "psibold": "𝛙",
        "omegabold": "𝛚",
        "boldpartialdiff": "𝛛",
        "epsilonsymbolbold": "𝛜",
        "thetasymbolbold": "𝛝",
        "kappasymbolbold": "𝛞",
        "phisymbolbold": "𝛟",
        "rhosymbolbold": "𝛠",
        "pisymbolbold": "𝛡",
        "Alphaitalic": "𝛢",
        "Betaitalic": "𝛣",
        "Gammaitalic": "𝛤",
        "Deltaitalic": "𝛥",
        "Epsilonitalic": "𝛦",
        "Zetaitalic": "𝛧",
        "Etaitalic": "𝛨",
        "Thetaitalic": "𝛩",
        "Iotaitalic": "𝛪",
        "Kappaitalic": "𝛫",
        "Lamdaitalic": "𝛬",
        "Muitalic": "𝛭",
        "Nuitalic": "𝛮",
        "Xiitalic": "𝛯",
        "Omicronitalic": "𝛰",
        "Piitalic": "𝛱",
        "Rhoitalic": "𝛲",
        "Thetasymbolitalic": "𝛳",
        "Sigmaitalic": "𝛴",
        "Tauitalic": "𝛵",
        "Upsilonitalic": "𝛶",
        "Phiitalic": "𝛷",
        "Chiitalic": "𝛸",
        "Psiitalic": "𝛹",
        "Omegaitalic": "𝛺",
        "italicnabla": "𝛻",
        "alphaitalic": "𝛼",
        "betaitalic": "𝛽",
        "gammaitalic": "𝛾",
        "deltaitalic": "𝛿",
        "epsilonitalic": "𝜀",
        "zetaitalic": "𝜁",
        "etaitalic": "𝜂",
        "thetaitalic": "𝜃",
        "iotaitalic": "𝜄",
        "kappaitalic": "𝜅",
        "lamdaitalic": "𝜆",
        "muitalic": "𝜇",
        "nuitalic": "𝜈",
        "xiitalic": "𝜉",
        "omicronitalic": "𝜊",
        "piitalic": "𝜋",
        "rhoitalic": "𝜌",
        "finalsigmaitalic": "𝜍",
        "sigmaitalic": "𝜎",
        "tauitalic": "𝜏",
        "upsilonitalic": "𝜐",
        "phiitalic": "𝜑",
        "chiitalic": "𝜒",
        "psiitalic": "𝜓",
        "omegaitalic": "𝜔",
        "italicpartialdiff": "𝜕",
        "epsilonsymbolitalic": "𝜖",
        "thetasymbolitalic": "𝜗",
        "kappasymbolitalic": "𝜘",
        "phisymbolitalic": "𝜙",
        "rhosymbolitalic": "𝜚",
        "pisymbolitalic": "𝜛",
        "Alphabolditalic": "𝜜",
        "Betabolditalic": "𝜝",
        "Gammabolditalic": "𝜞",
        "Deltabolditalic": "𝜟",
        "Epsilonbolditalic": "𝜠",
        "Zetabolditalic": "𝜡",
        "Etabolditalic": "𝜢",
        "Thetabolditalic": "𝜣",
        "Iotabolditalic": "𝜤",
        "Kappabolditalic": "𝜥",
        "Lamdabolditalic": "𝜦",
        "Mubolditalic": "𝜧",
        "Nubolditalic": "𝜨",
        "Xibolditalic": "𝜩",
        "Omicronbolditalic": "𝜪",
        "Pibolditalic": "𝜫",
        "Rhobolditalic": "𝜬",
        "Thetasymbolbolditalic": "𝜭",
        "Sigmabolditalic": "𝜮",
        "Taubolditalic": "𝜯",
        "Upsilonbolditalic": "𝜰",
        "Phibolditalic": "𝜱",
        "Chibolditalic": "𝜲",
        "Psibolditalic": "𝜳",
        "Omegabolditalic": "𝜴",
        "bolditalicnabla": "𝜵",
        "alphabolditalic": "𝜶",
        "betabolditalic": "𝜷",
        "gammabolditalic": "𝜸",
        "deltabolditalic": "𝜹",
        "epsilonbolditalic": "𝜺",
        "zetabolditalic": "𝜻",
        "etabolditalic": "𝜼",
        "thetabolditalic": "𝜽",
        "iotabolditalic": "𝜾",
        "kappabolditalic": "𝜿",
        "lamdabolditalic": "𝝀",
        "mubolditalic": "𝝁",
        "nubolditalic": "𝝂",
        "xibolditalic": "𝝃",
        "omicronbolditalic": "𝝄",
        "pibolditalic": "𝝅",
        "rhobolditalic": "𝝆",
        "finalsigmabolditalic": "𝝇",
        "sigmabolditalic": "𝝈",
        "taubolditalic": "𝝉",
        "upsilonbolditalic": "𝝊",
        "phibolditalic": "𝝋",
        "chibolditalic": "𝝌",
        "psibolditalic": "𝝍",
        "omegabolditalic": "𝝎",
        "bolditalicpartialdiff": "𝝏",
        "epsilonsymbolbolditalic": "𝝐",
        "thetasymbolbolditalic": "𝝑",
        "kappasymbolbolditalic": "𝝒",
        "phisymbolbolditalic": "𝝓",
        "rhosymbolbolditalic": "𝝔",
        "pisymbolbolditalic": "𝝕",
        "Alphaboldsans": "𝝖",
        "Betaboldsans": "𝝗",
        "Gammaboldsans": "𝝘",
        "Deltaboldsans": "𝝙",
        "Epsilonboldsans": "𝝚",
        "Zetaboldsans": "𝝛",
        "Etaboldsans": "𝝜",
        "Thetaboldsans": "𝝝",
        "Iotaboldsans": "𝝞",
        "Kappaboldsans": "𝝟",
        "Lamdaboldsans": "𝝠",
        "Muboldsans": "𝝡",
        "Nuboldsans": "𝝢",
        "Xiboldsans": "𝝣",
        "Omicronboldsans": "𝝤",
        "Piboldsans": "𝝥",
        "Rhoboldsans": "𝝦",
        "Thetasymbolboldsans": "𝝧",
        "Sigmaboldsans": "𝝨",
        "Tauboldsans": "𝝩",
        "Upsilonboldsans": "𝝪",
        "Phiboldsans": "𝝫",
        "Chiboldsans": "𝝬",
        "Psiboldsans": "𝝭",
        "Omegaboldsans": "𝝮",
        "boldsansnabla": "𝝯",
        "alphaboldsans": "𝝰",
        "betaboldsans": "𝝱",
        "gammaboldsans": "𝝲",
        "deltaboldsans": "𝝳",
        "epsilonboldsans": "𝝴",
        "zetaboldsans": "𝝵",
        "etaboldsans": "𝝶",
        "thetaboldsans": "𝝷",
        "iotaboldsans": "𝝸",
        "kappaboldsans": "𝝹",
        "lamdaboldsans": "𝝺",
        "muboldsans": "𝝻",
        "nuboldsans": "𝝼",
        "xiboldsans": "𝝽",
        "omicronboldsans": "𝝾",
        "piboldsans": "𝝿",
        "rhoboldsans": "𝞀",
        "finalsigmaboldsans": "𝞁",
        "sigmaboldsans": "𝞂",
        "tauboldsans": "𝞃",
        "upsilonboldsans": "𝞄",
        "phiboldsans": "𝞅",
        "chiboldsans": "𝞆",
        "psiboldsans": "𝞇",
        "omegaboldsans": "𝞈",
        "boldsanspartialdiff": "𝞉",
        "epsilonsymbolboldsans": "𝞊",
        "thetasymbolboldsans": "𝞋",
        "kappasymbolboldsans": "𝞌",
        "phisymbolboldsans": "𝞍",
        "rhosymbolboldsans": "𝞎",
        "pisymbolboldsans": "𝞏",
        "Alphabolditalicsans": "𝞐",
        "Betabolditalicsans": "𝞑",
        "Gammabolditalicsans": "𝞒",
        "Deltabolditalicsans": "𝞓",
        "Epsilonbolditalicsans": "𝞔",
        "Zetabolditalicsans": "𝞕",
        "Etabolditalicsans": "𝞖",
        "Thetabolditalicsans": "𝞗",
        "Iotabolditalicsans": "𝞘",
        "Kappabolditalicsans": "𝞙",
        "Lamdabolditalicsans": "𝞚",
        "Mubolditalicsans": "𝞛",
        "Nubolditalicsans": "𝞜",
        "Xibolditalicsans": "𝞝",
        "Omicronbolditalicsans": "𝞞",
        "Pibolditalicsans": "𝞟",
        "Rhobolditalicsans": "𝞠",
        "Thetasymbolbolditalicsans": "𝞡",
        "Sigmabolditalicsans": "𝞢",
        "Taubolditalicsans": "𝞣",
        "Upsilonbolditalicsans": "𝞤",
        "Phibolditalicsans": "𝞥",
        "Chibolditalicsans": "𝞦",
        "Psibolditalicsans": "𝞧",
        "Omegabolditalicsans": "𝞨",
        "bolditalicsansnabla": "𝞩",
        "alphabolditalicsans": "𝞪",
        "betabolditalicsans": "𝞫",
        "gammabolditalicsans": "𝞬",
        "deltabolditalicsans": "𝞭",
        "epsilonbolditalicsans": "𝞮",
        "zetabolditalicsans": "𝞯",
        "etabolditalicsans": "𝞰",
        "thetabolditalicsans": "𝞱",
        "iotabolditalicsans": "𝞲",
        "kappabolditalicsans": "𝞳",
        "lamdabolditalicsans": "𝞴",
        "mubolditalicsans": "𝞵",
        "nubolditalicsans": "𝞶",
        "xibolditalicsans": "𝞷",
        "omicronbolditalicsans": "𝞸",
        "pibolditalicsans": "𝞹",
        "rhobolditalicsans": "𝞺",
        "finalsigmabolditalicsans": "𝞻",
        "sigmabolditalicsans": "𝞼",
        "taubolditalicsans": "𝞽",
        "upsilonbolditalicsans": "𝞾",
        "phibolditalicsans": "𝞿",
        "chibolditalicsans": "𝟀",
        "psibolditalicsans": "𝟁",
        "omegabolditalicsans": "𝟂",
        "bolditalicsanspartialdiff": "𝟃",
        "epsilonsymbolbolditalicsans": "𝟄",
        "thetasymbolbolditalicsans": "𝟅",
        "kappasymbolbolditalicsans": "𝟆",
        "phisymbolbolditalicsans": "𝟇",
        "rhosymbolbolditalicsans": "𝟈",
        "pisymbolbolditalicsans": "𝟉",
        "Digammabold": "𝟊",
        "digammabold": "𝟋",
        "zerobold": "𝟎",
        "onebold": "𝟏",
        "twobold": "𝟐",
        "threebold": "𝟑",
        "fourbold": "𝟒",
        "fivebold": "𝟓",
        "sixbold": "𝟔",
        "sevenbold": "𝟕",
        "eightbold": "𝟖",
        "ninebold": "𝟗",
        "zerodblstruck": "𝟘",
        "onedblstruck": "𝟙",
        "twodblstruck": "𝟚",
        "threedblstruck": "𝟛",
        "fourdblstruck": "𝟜",
        "fivedblstruck": "𝟝",
        "sixdblstruck": "𝟞",
        "sevendblstruck": "𝟟",
        "eightdblstruck": "𝟠",
        "ninedblstruck": "𝟡",
        "zerosans": "𝟢",
        "onesans": "𝟣",
        "twosans": "𝟤",
        "threesans": "𝟥",
        "foursans": "𝟦",
        "fivesans": "𝟧",
        "sixsans": "𝟨",
        "sevensans": "𝟩",
        "eightsans": "𝟪",
        "ninesans": "𝟫",
        "zeroboldsans": "𝟬",
        "oneboldsans": "𝟭",
        "twoboldsans": "𝟮",
        "threeboldsans": "𝟯",
        "fourboldsans": "𝟰",
        "fiveboldsans": "𝟱",
        "sixboldsans": "𝟲",
        "sevenboldsans": "𝟳",
        "eightboldsans": "𝟴",
        "nineboldsans": "𝟵",
        "zeromono": "𝟶",
        "onemono": "𝟷",
        "twomono": "𝟸",
        "threemono": "𝟹",
        "fourmono": "𝟺",
        "fivemono": "𝟻",
        "sixmono": "𝟼",
        "sevenmono": "𝟽",
        "eightmono": "𝟾",
        "ninemono": "𝟿",
        "dominohorizontalback": "🀰",
        "dominohorizontal_00_00": "🀱",
        "dominohorizontal_00_01": "🀲",
        "dominohorizontal_00_02": "🀳",
        "dominohorizontal_00_03": "🀴",
        "dominohorizontal_00_04": "🀵",
        "dominohorizontal_00_05": "🀶",
        "dominohorizontal_00_06": "🀷",
        "dominohorizontal_01_00": "🀸",
        "dominohorizontal_01_01": "🀹",
        "dominohorizontal_01_02": "🀺",
        "dominohorizontal_01_03": "🀻",
        "dominohorizontal_01_04": "🀼",
        "dominohorizontal_01_05": "🀽",
        "dominohorizontal_01_06": "🀾",
        "dominohorizontal_02_00": "🀿",
        "dominohorizontal_02_01": "🁀",
        "dominohorizontal_02_02": "🁁",
        "dominohorizontal_02_03": "🁂",
        "dominohorizontal_02_04": "🁃",
        "dominohorizontal_02_05": "🁄",
        "dominohorizontal_02_06": "🁅",
        "dominohorizontal_03_00": "🁆",
        "dominohorizontal_03_01": "🁇",
        "dominohorizontal_03_02": "🁈",
        "dominohorizontal_03_03": "🁉",
        "dominohorizontal_03_04": "🁊",
        "dominohorizontal_03_05": "🁋",
        "dominohorizontal_03_06": "🁌",
        "dominohorizontal_04_00": "🁍",
        "dominohorizontal_04_01": "🁎",
        "dominohorizontal_04_02": "🁏",
        "dominohorizontal_04_03": "🁐",
        "dominohorizontal_04_04": "🁑",
        "dominohorizontal_04_05": "🁒",
        "dominohorizontal_04_06": "🁓",
        "dominohorizontal_05_00": "🁔",
        "dominohorizontal_05_01": "🁕",
        "dominohorizontal_05_02": "🁖",
        "dominohorizontal_05_03": "🁗",
        "dominohorizontal_05_04": "🁘",
        "dominohorizontal_05_05": "🁙",
        "dominohorizontal_05_06": "🁚",
        "dominohorizontal_06_00": "🁛",
        "dominohorizontal_06_01": "🁜",
        "dominohorizontal_06_02": "🁝",
        "dominohorizontal_06_03": "🁞",
        "dominohorizontal_06_04": "🁟",
        "dominohorizontal_06_05": "🁠",
        "dominohorizontal_06_06": "🁡",
        "dominoverticalback": "🁢",
        "dominovertical_00_00": "🁣",
        "dominovertical_00_01": "🁤",
        "dominovertical_00_02": "🁥",
        "dominovertical_00_03": "🁦",
        "dominovertical_00_04": "🁧",
        "dominovertical_00_05": "🁨",
        "dominovertical_00_06": "🁩",
        "dominovertical_01_00": "🁪",
        "dominovertical_01_01": "🁫",
        "dominovertical_01_02": "🁬",
        "dominovertical_01_03": "🁭",
        "dominovertical_01_04": "🁮",
        "dominovertical_01_05": "🁯",
        "dominovertical_01_06": "🁰",
        "dominovertical_02_00": "🁱",
        "dominovertical_02_01": "🁲",
        "dominovertical_02_02": "🁳",
        "dominovertical_02_03": "🁴",
        "dominovertical_02_04": "🁵",
        "dominovertical_02_05": "🁶",
        "dominovertical_02_06": "🁷",
        "dominovertical_03_00": "🁸",
        "dominovertical_03_01": "🁹",
        "dominovertical_03_02": "🁺",
        "dominovertical_03_03": "🁻",
        "dominovertical_03_04": "🁼",
        "dominovertical_03_05": "🁽",
        "dominovertical_03_06": "🁾",
        "dominovertical_04_00": "🁿",
        "dominovertical_04_01": "🂀",
        "dominovertical_04_02": "🂁",
        "dominovertical_04_03": "🂂",
        "dominovertical_04_04": "🂃",
        "dominovertical_04_05": "🂄",
        "dominovertical_04_06": "🂅",
        "dominovertical_05_00": "🂆",
        "dominovertical_05_01": "🂇",
        "dominovertical_05_02": "🂈",
        "dominovertical_05_03": "🂉",
        "dominovertical_05_04": "🂊",
        "dominovertical_05_05": "🂋",
        "dominovertical_05_06": "🂌",
        "dominovertical_06_00": "🂍",
        "dominovertical_06_01": "🂎",
        "dominovertical_06_02": "🂏",
        "dominovertical_06_03": "🂐",
        "dominovertical_06_04": "🂑",
        "dominovertical_06_05": "🂒",
        "dominovertical_06_06": "🂓",
        "cards:back": "🂠",
        "cards:aceofspades": "🂡",
        "cards:twoofspades": "🂢",
        "cards:threeofspades": "🂣",
        "cards:fourofspades": "🂤",
        "cards:fiveofspades": "🂥",
        "cards:sixofspades": "🂦",
        "cards:sevenofspades": "🂧",
        "cards:eightofspades": "🂨",
        "cards:nineofspades": "🂩",
        "cards:tenofspades": "🂪",
        "cards:jackofspades": "🂫",
        "cards:knightofspades": "🂬",
        "cards:queenofspades": "🂭",
        "cards:kingofspades": "🂮",
        "cards:aceofhearts": "🂱",
        "cards:twoofhearts": "🂲",
        "cards:threeofhearts": "🂳",
        "cards:fourofhearts": "🂴",
        "cards:fiveofhearts": "🂵",
        "cards:sixofhearts": "🂶",
        "cards:sevenofhearts": "🂷",
        "cards:eightofhearts": "🂸",
        "cards:nineofhearts": "🂹",
        "cards:tenofhearts": "🂺",
        "cards:jackofhearts": "🂻",
        "cards:knightofhearts": "🂼",
        "cards:queenofhearts": "🂽",
        "cards:kingofhearts": "🂾",
        "cards:redjoker": "🂿",
        "cards:aceofdiamonds": "🃁",
        "cards:twoofdiamonds": "🃂",
        "cards:threeofdiamonds": "🃃",
        "cards:fourofdiamonds": "🃄",
        "cards:fiveofdiamonds": "🃅",
        "cards:sixofdiamonds": "🃆",
        "cards:sevenofdiamonds": "🃇",
        "cards:eightofdiamonds": "🃈",
        "cards:nineofdiamonds": "🃉",
        "cards:tenofdiamonds": "🃊",
        "cards:jackofdiamonds": "🃋",
        "cards:knightofdiamonds": "🃌",
        "cards:queenofdiamonds": "🃍",
        "cards:kingofdiamonds": "🃎",
        "cards:blackjoker": "🃏",
        "cards:aceofclubs": "🃑",
        "cards:twoofclubs": "🃒",
        "cards:threeofclubs": "🃓",
        "cards:fourofclubs": "🃔",
        "cards:fiveofclubs": "🃕",
        "cards:sixofclubs": "🃖",
        "cards:sevenofclubs": "🃗",
        "cards:eightofclubs": "🃘",
        "cards:nineofclubs": "🃙",
        "cards:tenofclubs": "🃚",
        "cards:jackofclubs": "🃛",
        "cards:knightofclubs": "🃜",
        "cards:queenofclubs": "🃝",
        "cards:kingofclubs": "🃞",
        "cards:whitejoker": "🃟",
        "cards:fool": "🃠",
        "cards:trump1": "🃡",
        "cards:trump2": "🃢",
        "cards:trump3": "🃣",
        "cards:trump4": "🃤",
        "cards:trump5": "🃥",
        "cards:trump6": "🃦",
        "cards:trump7": "🃧",
        "cards:trump8": "🃨",
        "cards:trump9": "🃩",
        "cards:trump10": "🃪",
        "cards:trump11": "🃫",
        "cards:trump12": "🃬",
        "cards:trump13": "🃭",
        "cards:trump14": "🃮",
        "cards:trump15": "🃯",
        "cards:trump16": "🃰",
        "cards:trump17": "🃱",
        "cards:trump18": "🃲",
        "cards:trump19": "🃳",
        "cards:trump20": "🃴",
        "cards:trump21": "🃵",
        "zerofullstop": "🄀",
        "zerocomma": "🄁",
        "onecomma": "🄂",
        "twocomma": "🄃",
        "threecomma": "🄄",
        "fourcomma": "🄅",
        "fivecomma": "🄆",
        "sixcomma": "🄇",
        "sevencomma": "🄈",
        "eightcomma": "🄉",
        "ninecomma": "🄊",
        "dingbatSAns-serifzerocircle": "🄋",
        "dingbatSAns-serifzerocircleblack": "🄌",
        "Aparens": "🄐",
        "Bparens": "🄑",
        "Cparens": "🄒",
        "Dparens": "🄓",
        "Eparens": "🄔",
        "Fparens": "🄕",
        "Gparens": "🄖",
        "Hparens": "🄗",
        "Iparens": "🄘",
        "Jparens": "🄙",
        "Kparens": "🄚",
        "Lparens": "🄛",
        "Mparens": "🄜",
        "Nparens": "🄝",
        "Oparens": "🄞",
        "Pparens": "🄟",
        "Qparens": "🄠",
        "Rparens": "🄡",
        "Sparens": "🄢",
        "Tparens": "🄣",
        "Uparens": "🄤",
        "Vparens": "🄥",
        "Wparens": "🄦",
        "Xparens": "🄧",
        "Yparens": "🄨",
        "Zparens": "🄩",
        "Sshell": "🄪",
        "Citaliccircle": "🄫",
        "Ritaliccircle": "🄬",
        "CDcircle": "🄭",
        "WZcircle": "🄮",
        "copyleftsymbol": "🄯",
        "Asquare": "🄰",
        "Bsquare": "🄱",
        "Csquare": "🄲",
        "Dsquare": "🄳",
        "Esquare": "🄴",
        "Fsquare": "🄵",
        "Gsquare": "🄶",
        "Hsquare": "🄷",
        "Isquare": "🄸",
        "Jsquare": "🄹",
        "Ksquare": "🄺",
        "Lsquare": "🄻",
        "Msquare": "🄼",
        "Nsquare": "🄽",
        "Osquare": "🄾",
        "Psquare": "🄿",
        "Qsquare": "🅀",
        "Rsquare": "🅁",
        "Ssquare": "🅂",
        "Tsquare": "🅃",
        "Usquare": "🅄",
        "Vsquare": "🅅",
        "Wsquare": "🅆",
        "Xsquare": "🅇",
        "Ysquare": "🅈",
        "Zsquare": "🅉",
        "HVsquare": "🅊",
        "MVsquare": "🅋",
        "SDsquare": "🅌",
        "SSsquare": "🅍",
        "PPVsquare": "🅎",
        "wcsquare": "🅏",
        "Acircleblack": "🅐",
        "Bcircleblack": "🅑",
        "Ccircleblack": "🅒",
        "Dcircleblack": "🅓",
        "Ecircleblack": "🅔",
        "Fcircleblack": "🅕",
        "Gcircleblack": "🅖",
        "Hcircleblack": "🅗",
        "Icircleblack": "🅘",
        "Jcircleblack": "🅙",
        "Kcircleblack": "🅚",
        "Lcircleblack": "🅛",
        "Mcircleblack": "🅜",
        "Ncircleblack": "🅝",
        "Ocircleblack": "🅞",
        "Pcircleblack": "🅟",
        "Qcircleblack": "🅠",
        "Rcircleblack": "🅡",
        "Scircleblack": "🅢",
        "Tcircleblack": "🅣",
        "Ucircleblack": "🅤",
        "Vcircleblack": "🅥",
        "Wcircleblack": "🅦",
        "Xcircleblack": "🅧",
        "Ycircleblack": "🅨",
        "Zcircleblack": "🅩",
        "raisedmcsign": "🅪",
        "raisedmdsign": "🅫",
        "raisedmrsign": "🅬",
        "Asquareblack": "🅰",
        "Bsquareblack": "🅱",
        "Csquareblack": "🅲",
        "Dsquareblack": "🅳",
        "Esquareblack": "🅴",
        "Fsquareblack": "🅵",
        "Gsquareblack": "🅶",
        "Hsquareblack": "🅷",
        "Isquareblack": "🅸",
        "Jsquareblack": "🅹",
        "Ksquareblack": "🅺",
        "Lsquareblack": "🅻",
        "Msquareblack": "🅼",
        "Nsquareblack": "🅽",
        "Osquareblack": "🅾",
        "Psquareblack": "🅿",
        "Qsquareblack": "🆀",
        "Rsquareblack": "🆁",
        "Ssquareblack": "🆂",
        "Tsquareblack": "🆃",
        "Usquareblack": "🆄",
        "Vsquareblack": "🆅",
        "Wsquareblack": "🆆",
        "Xsquareblack": "🆇",
        "Ysquareblack": "🆈",
        "Zsquareblack": "🆉",
        "Pcrosssquareblack": "🆊",
        "ICsquareblack": "🆋",
        "PAsquareblack": "🆌",
        "SAsquareblack": "🆍",
        "absquareblack": "🆎",
        "wcsquareblack": "🆏",
        "squaredj": "🆐",
        "clsquare": "🆑",
        "coolsquare": "🆒",
        "freesquare": "🆓",
        "idsquare": "🆔",
        "newsquare": "🆕",
        "ngsquare": "🆖",
        "oksquare": "🆗",
        "sossquare": "🆘",
        "upwithexclamationmarksquare": "🆙",
        "vssquare": "🆚",
        "threedsquare": "🆛",
        "secondscreensquare": "🆜",
        "twoksquare": "🆝",
        "fourksquare": "🆞",
        "eightksquare": "🆟",
        "fivepointonesquare": "🆠",
        "sevenpointonesquare": "🆡",
        "twenty-twopointtwosquare": "🆢",
        "sixtypsquare": "🆣",
        "onehundredtwentypsquare": "🆤",
        "dsquare": "🆥",
        "hcsquare": "🆦",
        "hdrsquare": "🆧",
        "hi-ressquare": "🆨",
        "losslesssquare": "🆩",
        "shvsquare": "🆪",
        "uhdsquare": "🆫",
        "vodsquare": "🆬",
        "regionalindicatorsymbollettera": "🇦",
        "regionalindicatorsymbolletterb": "🇧",
        "regionalindicatorsymbolletterc": "🇨",
        "regionalindicatorsymbolletterd": "🇩",
        "regionalindicatorsymbollettere": "🇪",
        "regionalindicatorsymbolletterf": "🇫",
        "regionalindicatorsymbolletterg": "🇬",
        "regionalindicatorsymbolletterh": "🇭",
        "regionalindicatorsymbolletteri": "🇮",
        "regionalindicatorsymbolletterj": "🇯",
        "regionalindicatorsymbolletterk": "🇰",
        "regionalindicatorsymbolletterl": "🇱",
        "regionalindicatorsymbolletterm": "🇲",
        "regionalindicatorsymbollettern": "🇳",
        "regionalindicatorsymbollettero": "🇴",
        "regionalindicatorsymbolletterp": "🇵",
        "regionalindicatorsymbolletterq": "🇶",
        "regionalindicatorsymbolletterr": "🇷",
        "regionalindicatorsymbolletters": "🇸",
        "regionalindicatorsymbollettert": "🇹",
        "regionalindicatorsymbolletteru": "🇺",
        "regionalindicatorsymbolletterv": "🇻",
        "regionalindicatorsymbolletterw": "🇼",
        "regionalindicatorsymbolletterx": "🇽",
        "regionalindicatorsymbollettery": "🇾",
        "regionalindicatorsymbolletterz": "🇿",
        "cyclone": "🌀",
        "foggy": "🌁",
        "closedUmbrella": "🌂",
        "nightStars": "🌃",
        "sunriseOverMountains": "🌄",
        "sunrise": "🌅",
        "cityscapeAtDusk": "🌆",
        "sunsetOverBuildings": "🌇",
        "rainbow": "🌈",
        "bridgeAtNight": "🌉",
        "waterWave": "🌊",
        "volcano": "🌋",
        "milkyWay": "🌌",
        "earthGlobeEuropeAfrica": "🌍",
        "earthGlobeAmericas": "🌎",
        "earthGlobeAsiaAustralia": "🌏",
        "globeMeridians": "🌐",
        "newMoon": "🌑",
        "waxingCrescentMoon": "🌒",
        "firstQuarterMoon": "🌓",
        "waxingGibbousMoon": "🌔",
        "fullMoon": "🌕",
        "waningGibbousMoon": "🌖",
        "lastQuarterMoon": "🌗",
        "waningCrescentMoon": "🌘",
        "crescentMoon": "🌙",
        "newMoonFace": "🌚",
        "firstQuarterMoonFace": "🌛",
        "lastQuarterMoonFace": "🌜",
        "fullMoonFace": "🌝",
        "sunFace": "🌞",
        "glowingStar": "🌟",
        "shootingStar": "🌠",
        "thermometer": "🌡",
        "blackDroplet": "🌢",
        "whiteSun": "🌣",
        "whiteSunSmallCloud": "🌤",
        "whiteSunBehindCloud": "🌥",
        "whiteSunBehindCloudRain": "🌦",
        "cloudRain": "🌧",
        "cloudSnow": "🌨",
        "cloudLightning": "🌩",
        "cloudTornado": "🌪",
        "fog": "🌫",
        "windBlowingFace": "🌬",
        "hotDog": "🌭",
        "taco": "🌮",
        "burrito": "🌯",
        "chestnut": "🌰",
        "seedling": "🌱",
        "evergreenTree": "🌲",
        "deciduousTree": "🌳",
        "palmTree": "🌴",
        "cactus": "🌵",
        "hotPepper": "🌶",
        "tulip": "🌷",
        "cherryBlossom": "🌸",
        "rose": "🌹",
        "hibiscus": "🌺",
        "sunflower": "🌻",
        "blossom": "🌼",
        "earOfMaize": "🌽",
        "earOfRice": "🌾",
        "herb": "🌿",
        "fourLeafClover": "🍀",
        "mapleLeaf": "🍁",
        "fallenLeaf": "🍂",
        "leafFlutteringInWind": "🍃",
        "mushroom": "🍄",
        "tomato": "🍅",
        "aubergine": "🍆",
        "grapes": "🍇",
        "melon": "🍈",
        "watermelon": "🍉",
        "tangerine": "🍊",
        "lemon": "🍋",
        "banana": "🍌",
        "pineapple": "🍍",
        "redApple": "🍎",
        "greenApple": "🍏",
        "pear": "🍐",
        "peach": "🍑",
        "cherries": "🍒",
        "strawberry": "🍓",
        "hamburger": "🍔",
        "sliceOfPizza": "🍕",
        "meatOnBone": "🍖",
        "poultryLeg": "🍗",
        "riceCracker": "🍘",
        "riceBall": "🍙",
        "cookedRice": "🍚",
        "curryAndRice": "🍛",
        "steamingBowl": "🍜",
        "spaghetti": "🍝",
        "bread": "🍞",
        "frenchFries": "🍟",
        "roastedSweetPotato": "🍠",
        "dango": "🍡",
        "oden": "🍢",
        "sushi": "🍣",
        "friedShrimp": "🍤",
        "fishCakeSwirlDesign": "🍥",
        "softIceCream": "🍦",
        "shavedIce": "🍧",
        "iceCream": "🍨",
        "doughnut": "🍩",
        "cookie": "🍪",
        "chocolateBar": "🍫",
        "candy": "🍬",
        "lollipop": "🍭",
        "custard": "🍮",
        "honeyPot": "🍯",
        "shortcake": "🍰",
        "bentoBox": "🍱",
        "potOfFood": "🍲",
        "cooking": "🍳",
        "forkKnife": "🍴",
        "teacupOutHandle": "🍵",
        "sakeBottleAndCup": "🍶",
        "wineGlass": "🍷",
        "cocktailGlass": "🍸",
        "tropicalDrink": "🍹",
        "beerMug": "🍺",
        "clinkingBeerMugs": "🍻",
        "babyBottle": "🍼",
        "forkKnifePlate": "🍽",
        "bottlePoppingCork": "🍾",
        "popcorn": "🍿",
        "ribbon": "🎀",
        "wrappedPresent": "🎁",
        "birthdayCake": "🎂",
        "jackOLantern": "🎃",
        "christmasTree": "🎄",
        "fatherChristmas": "🎅",
        "fireworks": "🎆",
        "fireworkSparkler": "🎇",
        "balloon": "🎈",
        "partyPopper": "🎉",
        "confettiBall": "🎊",
        "tanabataTree": "🎋",
        "crossedFlags": "🎌",
        "pineDecoration": "🎍",
        "japaneseDolls": "🎎",
        "carpStreamer": "🎏",
        "windChime": "🎐",
        "moonViewingCeremony": "🎑",
        "schoolSatchel": "🎒",
        "graduationCap": "🎓",
        "heartTipOnTheLeft": "🎔",
        "bouquetOfFlowers": "🎕",
        "militaryMedal": "🎖",
        "reminderRibbon": "🎗",
        "musicalKeyboardJacks": "🎘",
        "studioMicrophone": "🎙",
        "levelSlider": "🎚",
        "controlKnobs": "🎛",
        "beamedAscendingMusicalNotes": "🎜",
        "beamedDescendingMusicalNotes": "🎝",
        "filmFrames": "🎞",
        "admissionTickets": "🎟",
        "carouselHorse": "🎠",
        "ferrisWheel": "🎡",
        "rollerCoaster": "🎢",
        "fishingPoleAndFish": "🎣",
        "microphone": "🎤",
        "movieCamera": "🎥",
        "cinema": "🎦",
        "headphone": "🎧",
        "artistPalette": "🎨",
        "topHat": "🎩",
        "circusTent": "🎪",
        "ticket": "🎫",
        "clapperBoard": "🎬",
        "performingArts": "🎭",
        "videoGame": "🎮",
        "directHit": "🎯",
        "slotMachine": "🎰",
        "billiards": "🎱",
        "gameDie": "🎲",
        "bowling": "🎳",
        "flowerPlayingCards": "🎴",
        "musicalNote": "🎵",
        "multipleMusicalNotes": "🎶",
        "saxophone": "🎷",
        "guitar": "🎸",
        "musicalKeyboard": "🎹",
        "trumpet": "🎺",
        "violin": "🎻",
        "musicalScore": "🎼",
        "runningShirtSash": "🎽",
        "tennisRacquetAndBall": "🎾",
        "skiAndSkiBoot": "🎿",
        "basketballAndHoop": "🏀",
        "chequeredFlag": "🏁",
        "snowboarder": "🏂",
        "runner": "🏃",
        "surfer": "🏄",
        "sportsMedal": "🏅",
        "trophy": "🏆",
        "horseRacing": "🏇",
        "americanFootball": "🏈",
        "rugbyFootball": "🏉",
        "swimmer": "🏊",
        "weightLifter": "🏋",
        "golfer": "🏌",
        "racingMotorcycle": "🏍",
        "racingCar": "🏎",
        "cricketBatAndBall": "🏏",
        "volleyball": "🏐",
        "fieldHockeyStickAndBall": "🏑",
        "iceHockeyStickAndPuck": "🏒",
        "tableTennisPaddleAndBall": "🏓",
        "snowcappedMountain": "🏔",
        "camping": "🏕",
        "beachUmbrella": "🏖",
        "buildingConstruction": "🏗",
        "houseBuildings": "🏘",
        "cityscape": "🏙",
        "derelictHouseBuilding": "🏚",
        "classicalBuilding": "🏛",
        "desert": "🏜",
        "desertIsland": "🏝",
        "nationalPark": "🏞",
        "stadium": "🏟",
        "houseBuilding": "🏠",
        "houseGarden": "🏡",
        "officeBuilding": "🏢",
        "japanesePostOffice": "🏣",
        "europeanPostOffice": "🏤",
        "hospital": "🏥",
        "bank": "🏦",
        "automatedTellerMachine": "🏧",
        "hotel": "🏨",
        "loveHotel": "🏩",
        "convenienceStore": "🏪",
        "school": "🏫",
        "departmentStore": "🏬",
        "factory": "🏭",
        "izakayaLantern": "🏮",
        "japaneseCastle": "🏯",
        "europeanCastle": "🏰",
        "whitePennant": "🏱",
        "blackPennant": "🏲",
        "wavingWhiteFlag": "🏳",
        "wavingBlackFlag": "🏴",
        "rosette": "🏵",
        "blackRosette": "🏶",
        "label": "🏷",
        "badmintonRacquetAndShuttlecock": "🏸",
        "bowAndArrow": "🏹",
        "amphora": "🏺",
        "emojiModifierFitzpatrickType-1-2": "🏻",
        "emojiModifierFitzpatrickType-3": "🏼",
        "emojiModifierFitzpatrickType-4": "🏽",
        "emojiModifierFitzpatrickType-5": "🏾",
        "emojiModifierFitzpatrickType-6": "🏿",
        "rat": "🐀",
        "mouse": "🐁",
        "ox": "🐂",
        "waterBuffalo": "🐃",
        "cow": "🐄",
        "tiger": "🐅",
        "leopard": "🐆",
        "rabbit": "🐇",
        "cat": "🐈",
        "dragon": "🐉",
        "crocodile": "🐊",
        "whale": "🐋",
        "snail": "🐌",
        "snake": "🐍",
        "horse": "🐎",
        "ram": "🐏",
        "goat": "🐐",
        "sheep": "🐑",
        "monkey": "🐒",
        "rooster": "🐓",
        "chicken": "🐔",
        "dog": "🐕",
        "pig": "🐖",
        "boar": "🐗",
        "elephant": "🐘",
        "octopus": "🐙",
        "spiralShell": "🐚",
        "bug": "🐛",
        "ant": "🐜",
        "honeybee": "🐝",
        "ladyBeetle": "🐞",
        "fish": "🐟",
        "tropicalFish": "🐠",
        "blowfish": "🐡",
        "turtle": "🐢",
        "hatchingChick": "🐣",
        "babyChick": "🐤",
        "front-facingBabyChick": "🐥",
        "bird": "🐦",
        "penguin": "🐧",
        "koala": "🐨",
        "poodle": "🐩",
        "dromedaryCamel": "🐪",
        "bactrianCamel": "🐫",
        "dolphin": "🐬",
        "mouseFace": "🐭",
        "cowFace": "🐮",
        "tigerFace": "🐯",
        "rabbitFace": "🐰",
        "catFace": "🐱",
        "dragonFace": "🐲",
        "spoutingWhale": "🐳",
        "horseFace": "🐴",
        "monkeyFace": "🐵",
        "dogFace": "🐶",
        "pigFace": "🐷",
        "frogFace": "🐸",
        "hamsterFace": "🐹",
        "wolfFace": "🐺",
        "bearFace": "🐻",
        "pandaFace": "🐼",
        "pigNose": "🐽",
        "pawPrints": "🐾",
        "chipmunk": "🐿",
        "eyes": "👀",
        "eye": "👁",
        "ear": "👂",
        "nose": "👃",
        "mouth": "👄",
        "tongue": "👅",
        "whiteUpPointingBackhandIndex": "👆",
        "whiteDownPointingBackhandIndex": "👇",
        "whiteLeftPointingBackhandIndex": "👈",
        "whiteRightPointingBackhandIndex": "👉",
        "fistedHandSign": "👊",
        "wavingHandSign": "👋",
        "okHandSign": "👌",
        "thumbsUpSign": "👍",
        "thumbsDownSign": "👎",
        "clappingHandsSign": "👏",
        "openHandsSign": "👐",
        "crown": "👑",
        "womansHat": "👒",
        "eyeglasses": "👓",
        "necktie": "👔",
        "t-shirt": "👕",
        "jeans": "👖",
        "dress": "👗",
        "kimono": "👘",
        "bikini": "👙",
        "womansClothes": "👚",
        "purse": "👛",
        "handbag": "👜",
        "pouch": "👝",
        "mansShoe": "👞",
        "athleticShoe": "👟",
        "high-heeledShoe": "👠",
        "womansSandal": "👡",
        "womansBoots": "👢",
        "footprints": "👣",
        "bustInSilhouette": "👤",
        "bustsInSilhouette": "👥",
        "boy": "👦",
        "girl": "👧",
        "man": "👨",
        "woman": "👩",
        "family": "👪",
        "manAndWomanHoldingHands": "👫",
        "twoMenHoldingHands": "👬",
        "twoWomenHoldingHands": "👭",
        "policeOfficer": "👮",
        "womanBunnyEars": "👯",
        "brideVeil": "👰",
        "personBlondHair": "👱",
        "manGuaPiMao": "👲",
        "manTurban": "👳",
        "olderMan": "👴",
        "olderWoman": "👵",
        "misc:baby": "👶",
        "constructionWorker": "👷",
        "princess": "👸",
        "japaneseOgre": "👹",
        "japaneseGoblin": "👺",
        "ghost": "👻",
        "babyAngel": "👼",
        "extraterrestrialAlien": "👽",
        "alienMonster": "👾",
        "imp": "👿",
        "skull": "💀",
        "inmationDeskPerson": "💁",
        "guardsman": "💂",
        "dancer": "💃",
        "lipstick": "💄",
        "nailPolish": "💅",
        "faceMassage": "💆",
        "haircut": "💇",
        "barberPole": "💈",
        "syringe": "💉",
        "pill": "💊",
        "kissMark": "💋",
        "loveLetter": "💌",
        "misc:ring": "💍",
        "gemStone": "💎",
        "kiss": "💏",
        "bouquet": "💐",
        "coupleHeart": "💑",
        "wedding": "💒",
        "beatingHeart": "💓",
        "brokenHeart": "💔",
        "twoHearts": "💕",
        "sparklingHeart": "💖",
        "growingHeart": "💗",
        "heartArrow": "💘",
        "blueHeart": "💙",
        "greenHeart": "💚",
        "yellowHeart": "💛",
        "purpleHeart": "💜",
        "heartRibbon": "💝",
        "revolvingHearts": "💞",
        "heartDecoration": "💟",
        "diamondShapeADotInside": "💠",
        "electricLightBulb": "💡",
        "anger": "💢",
        "bomb": "💣",
        "sleeping": "💤",
        "collision": "💥",
        "splashingSweat": "💦",
        "droplet": "💧",
        "misc:dash": "💨",
        "pileOfPoo": "💩",
        "flexedBiceps": "💪",
        "dizzy": "💫",
        "speechBalloon": "💬",
        "thoughtBalloon": "💭",
        "whiteFlower": "💮",
        "hundredPoints": "💯",
        "moneyBag": "💰",
        "currencyExchange": "💱",
        "heavyDollarSign": "💲",
        "creditCard": "💳",
        "banknoteYenSign": "💴",
        "banknoteDollarSign": "💵",
        "banknoteEuroSign": "💶",
        "banknotePoundSign": "💷",
        "moneyWings": "💸",
        "chartUpwardsTrendAndYenSign": "💹",
        "seat": "💺",
        "personalComputer": "💻",
        "briefcase": "💼",
        "minidisc": "💽",
        "floppyDisk": "💾",
        "opticalDisc": "💿",
        "dvd": "📀",
        "fileFolder": "📁",
        "openFileFolder": "📂",
        "pageCurl": "📃",
        "pageFacingUp": "📄",
        "calendar": "📅",
        "tear-offCalendar": "📆",
        "cardIndex": "📇",
        "chartUpwardsTrend": "📈",
        "chartDownwardsTrend": "📉",
        "barChart": "📊",
        "clipboard": "📋",
        "pushpin": "📌",
        "roundPushpin": "📍",
        "paperclip": "📎",
        "straightRuler": "📏",
        "triangularRuler": "📐",
        "bookmarkTabs": "📑",
        "ledger": "📒",
        "notebook": "📓",
        "notebookDecorativeCover": "📔",
        "closedBook": "📕",
        "openBook": "📖",
        "greenBook": "📗",
        "blueBook": "📘",
        "orangeBook": "📙",
        "books": "📚",
        "nameBadge": "📛",
        "scroll": "📜",
        "memo": "📝",
        "telephoneReceiver": "📞",
        "pager": "📟",
        "faxMachine": "📠",
        "satelliteAntenna": "📡",
        "publicAddressLoudspeaker": "📢",
        "cheeringMegaphone": "📣",
        "outboxTray": "📤",
        "inboxTray": "📥",
        "package": "📦",
        "e-mail": "📧",
        "incomingEnvelope": "📨",
        "envelopeDownwardsArrowAbove": "📩",
        "closedMailboxLoweredFlag": "📪",
        "closedMailboxRaisedFlag": "📫",
        "openMailboxRaisedFlag": "📬",
        "openMailboxLoweredFlag": "📭",
        "postbox": "📮",
        "postalHorn": "📯",
        "newspaper": "📰",
        "mobilePhone": "📱",
        "mobilePhoneRightwardsArrowAtLeft": "📲",
        "vibrationMode": "📳",
        "mobilePhoneOff": "📴",
        "noMobilePhones": "📵",
        "antennaBars": "📶",
        "camera": "📷",
        "cameraFlash": "📸",
        "videoCamera": "📹",
        "television": "📺",
        "radio": "📻",
        "videocassette": "📼",
        "filmProjector": "📽",
        "portableStereo": "📾",
        "prayerBeads": "📿",
        "twistedRightwardsArrows": "🔀",
        "clockwiseRightwardsAndLeftwardsOpenCircleArrows": "🔁",
        "clockwiseRightwardsAndLeftwardsOpenCircleArrowsCircledOneOverlay": "🔂",
        "clockwiseDownwardsAndUpwardsOpenCircleArrows": "🔃",
        "anticlockwiseDownwardsAndUpwardsOpenCircleArrows": "🔄",
        "lowBrightness": "🔅",
        "highBrightness": "🔆",
        "speakerCancellationStroke": "🔇",
        "speaker": "🔈",
        "speakerOneSoundWave": "🔉",
        "speakerThreeSoundWaves": "🔊",
        "battery": "🔋",
        "electricPlug": "🔌",
        "left-pointingMagnifyingGlass": "🔍",
        "right-pointingMagnifyingGlass": "🔎",
        "lockInkPen": "🔏",
        "closedLockKey": "🔐",
        "key": "🔑",
        "lock": "🔒",
        "openLock": "🔓",
        "bellCancellationStroke": "🔕",
        "bookmark": "🔖",
        "link": "🔗",
        "radioButton": "🔘",
        "backLeftwardsArrowAbove": "🔙",
        "endLeftwardsArrowAbove": "🔚",
        "onExclamationMarkLeftRightArrowAbove": "🔛",
        "soonRightwardsArrowAbove": "🔜",
        "topUpwardsArrowAbove": "🔝",
        "noOneUnderEighteen": "🔞",
        "keycapTen": "🔟",
        "inputLatinCapitalLetters": "🔠",
        "inputLatinSmallLetters": "🔡",
        "inputNumbers": "🔢",
        "inputS": "🔣",
        "inputLatinLetters": "🔤",
        "fire": "🔥",
        "electricTorch": "🔦",
        "wrench": "🔧",
        "hammer": "🔨",
        "nutAndBolt": "🔩",
        "hocho": "🔪",
        "pistol": "🔫",
        "microscope": "🔬",
        "telescope": "🔭",
        "crystalBall": "🔮",
        "sixPointedStarMiddleDot": "🔯",
        "japaneseBeginner": "🔰",
        "tridentEmblem": "🔱",
        "blackSquareButton": "🔲",
        "whiteSquareButton": "🔳",
        "largeRedCircle": "🔴",
        "largeBlueCircle": "🔵",
        "largeOrangeDiamond": "🔶",
        "largeBlueDiamond": "🔷",
        "smallOrangeDiamond": "🔸",
        "smallBlueDiamond": "🔹",
        "redTriangleUp": "🔺",
        "redTriangleDOwn": "🔻",
        "smallRedTriangleUp": "🔼",
        "smallRedTriangleDOwn": "🔽",
        "lowerRightShadowedWhiteCircle": "🔾",
        "upperRightShadowedWhiteCircle": "🔿",
        "circledCrossPommee": "🕀",
        "crossPommeeHalf-circleBelow": "🕁",
        "crossPommee": "🕂",
        "notchedLeftSemicircleThreeDots": "🕃",
        "notchedRightSemicircleThreeDots": "🕄",
        "marksChapter": "🕅",
        "whiteLatinCross": "🕆",
        "heavyLatinCross": "🕇",
        "celticCross": "🕈",
        "om": "🕉",
        "doveOfPeace": "🕊",
        "kaaba": "🕋",
        "mosque": "🕌",
        "synagogue": "🕍",
        "menorahNineBranches": "🕎",
        "bowlOfHygieia": "🕏",
        "clockFaceOneOclock": "🕐",
        "clockFaceTwoOclock": "🕑",
        "clockFaceThreeOclock": "🕒",
        "clockFaceFourOclock": "🕓",
        "clockFaceFiveOclock": "🕔",
        "clockFaceSixOclock": "🕕",
        "clockFaceSevenOclock": "🕖",
        "clockFaceEightOclock": "🕗",
        "clockFaceNineOclock": "🕘",
        "clockFaceTenOclock": "🕙",
        "clockFaceElevenOclock": "🕚",
        "clockFaceTwelveOclock": "🕛",
        "clockFaceOne-thirty": "🕜",
        "clockFaceTwo-thirty": "🕝",
        "clockFaceThree-thirty": "🕞",
        "clockFaceFour-thirty": "🕟",
        "clockFaceFive-thirty": "🕠",
        "clockFaceSix-thirty": "🕡",
        "clockFaceSeven-thirty": "🕢",
        "clockFaceEight-thirty": "🕣",
        "clockFaceNine-thirty": "🕤",
        "clockFaceTen-thirty": "🕥",
        "clockFaceEleven-thirty": "🕦",
        "clockFaceTwelve-thirty": "🕧",
        "rightSpeaker": "🕨",
        "rightSpeakerOneSoundWave": "🕩",
        "rightSpeakerThreeSoundWaves": "🕪",
        "bullhorn": "🕫",
        "bullhornSoundWaves": "🕬",
        "ringingBell": "🕭",
        "book": "🕮",
        "candle": "🕯",
        "mantelpieceClock": "🕰",
        "blackSkullAndCrossbones": "🕱",
        "noPiracy": "🕲",
        "hole": "🕳",
        "manInBusinessSuitLevitating": "🕴",
        "sleuthOrSpy": "🕵",
        "darkSunglasses": "🕶",
        "spider": "🕷",
        "spiderWeb": "🕸",
        "joystick": "🕹",
        "manDancing": "🕺",
        "leftHandTelephoneReceiver": "🕻",
        "telephoneReceiverPage": "🕼",
        "rightHandTelephoneReceiver": "🕽",
        "whiteTouchtoneTelephone": "🕾",
        "blackTouchtoneTelephone": "🕿",
        "telephoneOnTopOfModem": "🖀",
        "clamshellMobilePhone": "🖁",
        "backOfEnvelope": "🖂",
        "stampedEnvelope": "🖃",
        "envelopeLightning": "🖄",
        "flyingEnvelope": "🖅",
        "penOverStampedEnvelope": "🖆",
        "linkedPaperclips": "🖇",
        "blackPushpin": "🖈",
        "lowerLeftPencil": "🖉",
        "lowerLeftBallpointPen": "🖊",
        "lowerLeftFountainPen": "🖋",
        "lowerLeftPaintbrush": "🖌",
        "lowerLeftCrayon": "🖍",
        "leftWritingHand": "🖎",
        "turnedOkHandSign": "🖏",
        "raisedHandFingersSplayed": "🖐",
        "reversedRaisedHandFingersSplayed": "🖑",
        "reversedThumbsUpSign": "🖒",
        "reversedThumbsDownSign": "🖓",
        "reversedVictoryHand": "🖔",
        "reversedHandMiddleFingerExtended": "🖕",
        "raisedHandPartBetweenMiddleAndRingFingers": "🖖",
        "whiteDownPointingLeftHandIndex": "🖗",
        "sidewaysWhiteLeftPointingIndex": "🖘",
        "sidewaysWhiteRightPointingIndex": "🖙",
        "sidewaysBlackLeftPointingIndex": "🖚",
        "sidewaysBlackRightPointingIndex": "🖛",
        "blackLeftPointingBackhandIndex": "🖜",
        "blackRightPointingBackhandIndex": "🖝",
        "sidewaysWhiteUpPointingIndex": "🖞",
        "sidewaysWhiteDownPointingIndex": "🖟",
        "sidewaysBlackUpPointingIndex": "🖠",
        "sidewaysBlackDownPointingIndex": "🖡",
        "blackUpPointingBackhandIndex": "🖢",
        "blackDownPointingBackhandIndex": "🖣",
        "blackHeart": "🖤",
        "desktopComputer": "🖥",
        "keyboardAndMouse": "🖦",
        "threeNetworkedComputers": "🖧",
        "printer": "🖨",
        "pocketCalculator": "🖩",
        "blackHardShellFloppyDisk": "🖪",
        "whiteHardShellFloppyDisk": "🖫",
        "softShellFloppyDisk": "🖬",
        "tapeCartridge": "🖭",
        "wiredKeyboard": "🖮",
        "oneButtonMouse": "🖯",
        "twoButtonMouse": "🖰",
        "threeButtonMouse": "🖱",
        "trackball": "🖲",
        "oldPersonalComputer": "🖳",
        "hardDisk": "🖴",
        "screen": "🖵",
        "printerIcon": "🖶",
        "faxIcon": "🖷",
        "opticalDiscIcon": "🖸",
        "documentText": "🖹",
        "documentTextAndPicture": "🖺",
        "documentPicture": "🖻",
        "framePicture": "🖼",
        "frameTiles": "🖽",
        "frameAnX": "🖾",
        "blackFolder": "🖿",
        "folder": "🗀",
        "openFolder": "🗁",
        "cardIndexDividers": "🗂",
        "cardFileBox": "🗃",
        "fileCabinet": "🗄",
        "emptyNote": "🗅",
        "emptyNotePage": "🗆",
        "emptyNotePad": "🗇",
        "note": "🗈",
        "notePage": "🗉",
        "notePad": "🗊",
        "emptyDocument": "🗋",
        "emptyPage": "🗌",
        "emptyPages": "🗍",
        "document": "🗎",
        "page": "🗏",
        "pages": "🗐",
        "wastebasket": "🗑",
        "spiralNotePad": "🗒",
        "spiralCalendarPad": "🗓",
        "desktopWindow": "🗔",
        "minimize": "🗕",
        "maximize": "🗖",
        "overlap": "🗗",
        "clockwiseRightAndLeftSemicircleArrows": "🗘",
        "cancellationX": "🗙",
        "increaseFontSize": "🗚",
        "decreaseFontSize": "🗛",
        "compression": "🗜",
        "oldKey": "🗝",
        "rolled-upNewspaper": "🗞",
        "pageCircledText": "🗟",
        "stockChart": "🗠",
        "daggerKnife": "🗡",
        "lips": "🗢",
        "speakingHeadInSilhouette": "🗣",
        "threeRaysAbove": "🗤",
        "threeRaysBelow": "🗥",
        "threeRaysLeft": "🗦",
        "threeRaysRight": "🗧",
        "leftSpeechBubble": "🗨",
        "rightSpeechBubble": "🗩",
        "twoSpeechBubbles": "🗪",
        "threeSpeechBubbles": "🗫",
        "leftThoughtBubble": "🗬",
        "rightThoughtBubble": "🗭",
        "leftAngerBubble": "🗮",
        "rightAngerBubble": "🗯",
        "moodBubble": "🗰",
        "lightningMoodBubble": "🗱",
        "lightningMood": "🗲",
        "ballotBoxBallot": "🗳",
        "ballotScriptX": "🗴",
        "ballotBoxScriptX": "🗵",
        "ballotBoldScriptX": "🗶",
        "ballotBoxBoldScriptX": "🗷",
        "lightCheckMark": "🗸",
        "ballotBoxBoldCheck": "🗹",
        "worldMap": "🗺",
        "mountFuji": "🗻",
        "tokyoTower": "🗼",
        "statueOfLiberty": "🗽",
        "silhouetteOfJapan": "🗾",
        "moyai": "🗿",
        "grinningFace": "😀",
        "grinningFaceWithSmilingEyes": "😁",
        "faceWithTearsOfJoy": "😂",
        "smilingFaceWithOpenMouth": "😃",
        "smilingFaceWithOpenMouthAndSmilingEyes": "😄",
        "smilingFaceWithOpenMouthAndColdSweat": "😅",
        "smilingFaceWithOpenMouthAndTightlyClosedEyes": "😆",
        "smilingFaceWithHalo": "😇",
        "smilingFaceWithHorns": "😈",
        "winkingFace": "😉",
        "smilingFaceWithSmilingEyes": "😊",
        "faceSavouringDeliciousFood": "😋",
        "relievedFace": "😌",
        "smilingFaceWithHeartShapedEyes": "😍",
        "smilingFaceWithSunglasses": "😎",
        "smirkingFace": "😏",
        "neutralFace": "😐",
        "expressionlessFace": "😑",
        "unamusedFace": "😒",
        "faceWithColdSweat": "😓",
        "pensiveFace": "😔",
        "confusedFace": "😕",
        "confoundedFace": "😖",
        "kissingFace": "😗",
        "faceThrowingAKiss": "😘",
        "kissingFaceWithSmilingEyes": "😙",
        "kissingFaceWithClosedEyes": "😚",
        "faceWithStuckOutTongue": "😛",
        "faceWithStuckOutTongueAndWinkingEye": "😜",
        "faceWithStuckOutTongueAndTightlyClosedEyes": "😝",
        "disappointedFace": "😞",
        "worriedFace": "😟",
        "angryFace": "😠",
        "poutingFace": "😡",
        "cryingFace": "😢",
        "perseveringFace": "😣",
        "faceWithLookOfTriumph": "😤",
        "disappointedButRelievedFace": "😥",
        "frowningFaceWithOpenMouth": "😦",
        "anguishedFace": "😧",
        "fearfulFace": "😨",
        "wearyFace": "😩",
        "sleepyFace": "😪",
        "tiredFace": "😫",
        "grimacingFace": "😬",
        "loudlyCryingFace": "😭",
        "faceWithOpenMouth": "😮",
        "hushedFace": "😯",
        "faceWithOpenMouthAndColdSweat": "😰",
        "faceScreamingInFear": "😱",
        "astonishedFace": "😲",
        "flushedFace": "😳",
        "sleepingFace": "😴",
        "dizzyFace": "😵",
        "faceWithoutMouth": "😶",
        "faceWithMedicalMask": "😷",
        "grinningCatFaceWithSmilingEyes": "😸",
        "catFaceWithTearsOfJoy": "😹",
        "smilingCatFaceWithOpenMouth": "😺",
        "smilingCatFaceWithHeartShapedEyes": "😻",
        "catFaceWithWrySmile": "😼",
        "kissingCatFaceWithClosedEyes": "😽",
        "poutingCatFace": "😾",
        "cryingCatFace": "😿",
        "wearyCatFace": "🙀",
        "slightlyFrowningFace": "🙁",
        "slightlySmilingFace": "🙂",
        "upsideDownFace": "🙃",
        "faceWithRollingEyes": "🙄",
        "faceWithNoGoodGesture": "🙅",
        "faceWithOkGesture": "🙆",
        "personBowingDeeply": "🙇",
        "seeNoEvilMonkey": "🙈",
        "hearNoEvilMonkey": "🙉",
        "speakNoEvilMonkey": "🙊",
        "happyPersonRaisingOneHand": "🙋",
        "personRaisingBothHandsInCelebration": "🙌",
        "personFrowning": "🙍",
        "personWithPoutingFace": "🙎",
        "personWithFoldedHands": "🙏",
        "rocket": "🚀",
        "helicopter": "🚁",
        "steamLocomotive": "🚂",
        "railwayCar": "🚃",
        "highSpeedTrain": "🚄",
        "highSpeedTrainWithBulletNose": "🚅",
        "train": "🚆",
        "metro": "🚇",
        "lightRail": "🚈",
        "station": "🚉",
        "tram": "🚊",
        "tramCar": "🚋",
        "bus": "🚌",
        "oncomingBus": "🚍",
        "trolleybus": "🚎",
        "busStop": "🚏",
        "minibus": "🚐",
        "ambulance": "🚑",
        "fireEngine": "🚒",
        "policeCar": "🚓",
        "oncomingPoliceCar": "🚔",
        "taxi": "🚕",
        "oncomingTaxi": "🚖",
        "automobile": "🚗",
        "oncomingAutomobile": "🚘",
        "recreationalVehicle": "🚙",
        "deliveryTruck": "🚚",
        "articulatedLorry": "🚛",
        "tractor": "🚜",
        "monorail": "🚝",
        "mountainRailway": "🚞",
        "suspensionRailway": "🚟",
        "mountainCableway": "🚠",
        "aerialTramway": "🚡",
        "ship": "🚢",
        "rowboat": "🚣",
        "speedboat": "🚤",
        "horizontalTrafficLight": "🚥",
        "verticalTrafficLight": "🚦",
        "constructionSign": "🚧",
        "policeCarsRevolvingLight": "🚨",
        "triangularFlagOnPost": "🚩",
        "door": "🚪",
        "noEntrySign": "🚫",
        "smoking": "🚬",
        "noSmoking": "🚭",
        "putLitterInItsPlace": "🚮",
        "doNotLitter": "🚯",
        "potableWater": "🚰",
        "nonPotableWater": "🚱",
        "bicycle": "🚲",
        "noBicycles": "🚳",
        "bicyclist": "🚴",
        "mountainBicyclist": "🚵",
        "pedestrian": "🚶",
        "noPedestrians": "🚷",
        "childrenCrossing": "🚸",
        "mens": "🚹",
        "womens": "🚺",
        "restroom": "🚻",
        "trns:baby": "🚼",
        "toilet": "🚽",
        "waterCloset": "🚾",
        "shower": "🚿",
        "bath": "🛀",
        "bathtub": "🛁",
        "passportControl": "🛂",
        "customs": "🛃",
        "baggageClaim": "🛄",
        "leftLuggage": "🛅",
        "triangleWithRoundedCorners": "🛆",
        "prohibitedSign": "🛇",
        "circledInformationSource": "🛈",
        "boys": "🛉",
        "girls": "🛊",
        "couchAndLamp": "🛋",
        "sleepingAccommodation": "🛌",
        "shoppingBags": "🛍",
        "bellhopBell": "🛎",
        "bed": "🛏",
        "placeOfWorship": "🛐",
        "octagonalSign": "🛑",
        "shoppingTrolley": "🛒",
        "stupa": "🛓",
        "pagoda": "🛔",
        "hinduTemple": "🛕",
        "hammerAndWrench": "🛠",
        "shield": "🛡",
        "oilDrum": "🛢",
        "motorway": "🛣",
        "railwayTrack": "🛤",
        "motorBoat": "🛥",
        "upPointingMilitaryAirplane": "🛦",
        "upPointingAirplane": "🛧",
        "upPointingSmallAirplane": "🛨",
        "smallAirplane": "🛩",
        "northeastPointingAirplane": "🛪",
        "airplaneDeparture": "🛫",
        "airplaneArriving": "🛬",
        "satellite": "🛰",
        "oncomingFireEngine": "🛱",
        "dieselLocomotive": "🛲",
        "passengerShip": "🛳",
        "scooter": "🛴",
        "motorScooter": "🛵",
        "canoe": "🛶",
        "sled": "🛷",
        "flyingSaucer": "🛸",
        "skateboard": "🛹",
        "autoRickshaw": "🛺",
    }
    ADOBE_CHARACTER_TO_CHARACTER_NAME = {
        v: k for k, v in ADOBE_CHARACTER_NAME_TO_CHARACTER.items()
    }
