#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A specialized version of the Map class representing a map of Africa.

This class inherits from the generic Map class and provides geographical data specific to Africa.
It allows for rendering and customizing the appearance of African countries, including setting
specific fill and stroke colors for individual countries.

Use this class to create and manipulate a map focused on Africa within a PDF document.
"""
import typing

from borb.pdf.color.color import Color
from borb.pdf.color.x11_color import X11Color
from borb.pdf.layout_element.layout_element import LayoutElement
from borb.pdf.layout_element.shape.map import Map


class MapOfAfrica(Map):
    """
    A specialized version of the Map class representing a map of Africa.

    This class inherits from the generic Map class and provides geographical data specific to Africa.
    It allows for rendering and customizing the appearance of African countries, including setting
    specific fill and stroke colors for individual countries.

    Use this class to create and manipulate a map focused on Africa within a PDF document.
    """

    __MAP_OF_AFRICA_GEO_JSON_BYTES: bytes = (
        b'x\xda\xd5\xbd[\x93\x1b7\x92?\xfaU\x18~\x99\x17\xd3Q\xb8\xd4\xed\xff&\xc9\xb2\xc6\xc7\x96=+i\xd6g<\xbb1Q$\x8b\x12\xadVwo7\xdb\x0eic\xbe\xfb!\x90\x99@"\x01\x14)\x9d\x9e\x89s^ZJ\x14\n@U\x81\xc8\xfb/\xff\xf7\xab\xe3\xc7\xdb\xf9\xab\xff\xb3\xfa\xea\xbby:>\xdc\xcd\xcfn\xae\xae\xe6\xed\xf1ps\xfd\xd5\xd7\xab\xaf\xf6\xd0x\x7f\xea\xf0\xf7\xff\x95]]\x87\xb7\xf3\xcd\x87\xf9x\xf7\xf1\xd4\x1c\xaf\xff\xe5\xe6\xea\xe3[\x18`{ss\xb7;\\OG\x18\xe3\xef\x7f\xb7\xfd7\xc3\xd7\xab\xe1\x9b\xe6\xbf\xbf^\xfd\xdd\xb6\xdf4_\xafZ$\xcc7}$\xb4\xebf\xbf1H(GhO\xa8o\xc6\xafW\xe6\x9b\x11\t\x1d\x89\x86\xddcF\xe86 \xd19\xc2zb\x80+-\x12\xbd#:$,\'T$:w\x8f\xc5\x01:7)\x11-L:D\xa2\xc5\x15\x9c\xfeu\x04\xccc\xdd<\x1d\x8ef\xdd\x95\x0e\xef\xb1n\x9e\x1e\x1e\xce\x18\xb7\xd0\xfe\x9b\x1e\x89\xc6\x11\x03\x12\xc6\xbd7\x8b\xc4\x10\t\xeb\xba\rx\x8f\'F>\x8fj\x02\xd5{jD\xea4\x84R|\xad\xea\xf4\x9e\x91\x1a=E\xcf\xee\xae\xc57qzG\x8a\x9e\xbew\xebU\xf8iN\x94\xbb\x0f?\xe1\xe9\x05\xb6\xfeZ\x8b\x9f@y\xaaG\xca\xc4k\xa7\x0f\xd7$\xd4\xe8)\x15\xbf\xb0\xc2\x0fy\xa2:O\xb5\xb8/\x1a\xbf\xce\x11)\xeb\xa9\xf4\x1a\x8d\xd2\xfb\xa7\xed\x12\xca"\x05oB\xe1}\xc6S\xb4\r\xbb\xe4\xda\x10\xdf ^\xc3\xb7{\xa2FO\xd1^6\xee3\xb4qc\x8f\xb8}\xfdN\xa2\xfd\x1f~\x0c\xff\xfd\xdf\xff<\xfdZn\xefnn\xe7\xbb\xe3\xc1\xffX\xfe\xf7\xab\xeb\xe9\x83\xff==?\xbe;\xdc\xdc\x1e&\xf7\x83rm\xff\xb8\xba\xb9~+/\xeco\xee>LW\xff\x98\xaf\xe1\x17\xba\x9b\xef\xa6\xab\xd5\xb7\xf3\x87\x9b\xed\xddt<lW\xaf\xe6\xdb\x87\xcd\xd5\xe9?7\xfbU\xe1\xbe\xfd\xdd\xe9\xbe\xeb\x87\xab+\x9ac{\x98\xfe\xd8\x17g\xbf\xbf\xb9;\x16/LW\xc78\xc8\xfep{\xff\x0f\xd5\xf8\x8eo\\\x97\xc3\xfd\xcd?&\xfd\x8f\xf9\x9dh2\xa1\xe9\xcfq$\xb7\x9a\xaf\xfe\xeb\xa1\xe9t\xeb\xffn\xdc_;\xf9\xbf\x83o\x19b\x8b\xee\xc3\x8d\x9bk\xbcq\x1cz\xf7wr\xb7\x8f\x9b\xbd\xfb;\x1a\xdf2\xc5\x96\xc9\xff\xddl\xfd\xdf9\x0c\xb2\x9bq\x90fk\xe1\x19\xe7\xebp\x11^p\xf6\xec\xfe\x93\x9d\x9aO\xad\xee\xc6y\x17/\xed\xa7\xf00\xbd\xff\xeb\x17\xbd\xdd\xc6\x87\xe9gh\x89\xb7\xdc\x85\x15\x8c\xb8\x828\xd3\x15^3\xa3r\x7f7\xa3\xff;\xb0\xff\xbb\xc72\xdb\xc6\xfd\xf5\x8fh6*\xdc\xfe\x8e\x1e\xae\xdd\xb9\x0e\xed<\xf9\xff\x8f\xfeo\xeb[,k\xb1\xf1\xc6\x03\xbd\xda\xc6\xbfZ\xffeF\xe3_\xa1\xf6\x7f\xed\xc6\xff\x7f\x92\xed&.\xfe\xdd\x03\xbe&\xf7l{\xc3\xdf\xe0aW|\xb1\x87cx\xb1\xac\xf57|\xa7\xa6\x99\x06\xf7w\xab\xfc\xff\'\xf7wg\xfd\xffu\xe8\xfc\xfe\x06:o\xfd#\xef\xf4\xfe\xf4w\xdb\xe9S\xb7]k\xb6\xae\xdd\xc6\xc7\xbc\xbeb\xcb\xf0\x9fr\x13\xae\xdd^\x95\xd6r{\xac>\xd3\xdd\x03\xbe3\xabw\xee\xafu\xaf\xd6\x9a\xc1\xff\x9d\xfd\xdf}l\xb1\xfb\xf8\x1b\xfb=\xce\xc46\xdf\xf1\x0e\x9b?\xde\xdc~\x8c\xd3<\xbc\xa7iT\x1b\xa7i;9\r\xb4\xb0i\x1e\xee\xd2\xad\t\x9b\x126\xe8f\xce7\xa8\xf8\xb5\xfd~(~\xb2O\xef`\xd4\xb6\xdf\x9e~r\xed\xd0\x9e\xee\xb6\xfb\xediQ\xdd\xc6}\x1c;\x8f\xbc\xf3\x11z\x0fCw\xea\xdd\xef\xb4f\xfd\xe6\xaf\xfey:\x18\x1fG\xf40^> \xc6\xefy\x0e2.=\xba\x13\xda\xc2I~""{\xd7\x03\\i\x91\xb0A\xc08\x11M\xec\xd6\xbb+-\x8e\xd6\xbb\x01Z\x1c\xads\\\xb0\x05\x96\xa1\xbd\xe4\xd0\xe1h^X\xe8\x81\x13\x9c\x08/\x05\xb4\x9c\xf0,O[\xc7f\x06\x1c\xda\x00\xf7\xe8\xf0\x8a\x0e\x8c\xffD\xb4\x8e\x18\x91\x18\x1c\xc7\x19\xe2h\'\xced\xe2\xac\'\xca\xe2\x82\x1aO\xa9\xb8\xd61,|\x08,\xec\xf4H*^\xe9\xe1J\x17_\xc3\x88\xc3\x8d\x05\x82^j\x1bW4"\xe7T\xf1K\xd0\xfa\xe0#\x8d(&(\xf7Z\xf1.\xa3\x90\xc3\xc2\xf7\x03n\x8f\x9c\xd9\x10\xd7\xee\x91R\x9e\xf77H\xf5\x05\x8a\xe4-\x9dP*\xca\x05x\xad\t\xc2X\xcfVi\xf0-6\x91\x1a\x83\xa4\xd7\x08b\x08C\x0c\\\x84K\xe4\xb9D\xd2c2`"\x1d\n\xb9\x91I\x94B\xd6\xec\xa2Hj\xd9>?\rm\xa3P\xacA\xf4e/\x97d<\x94\xaa\x87\xf8A\xdc\xaffQFy\xfd\xcd\xea\xf5\xc3n\xba\xced\x94\xd77\x0f\xc7w\xf1Z"\xa6p\x99\xa4\xdc\xaf.\x96\x88\xfe\x89dR\xbaV\x13N\xd6\xe3\x98I\'\xaf_g\xd2\xc9\xeb\xd7\xdf\x96\xa4\x13`\xe2\x1d\x97KV\xf18\xf5\xe7pg\x0c\xbb\xbcgW\xbb\x82\xd82u^T\xf1\xbcu\xbb\xf3\x82I\xcf\x04\x16\xb3\xf2\xc4\xe0/\xabx\x83?\xac\xc7i\x102\xcck\xcf\x92\xb6\xbb\xfb\xe4U\xc0\xcb/\xbd$\x90bNm\x9e\xf1\xa9\xeb\xd5n\xbe:u\xb9+\xc82\xf5\xa7Z\xd5\xdeLQ\xc89-\xe3\xb4\x82\xd5\xee\xc1-\xa5$\xe4\xec\xbc0\xb3\xf5\x7f-\x17rV^\xbe1L\xe2i\xfd\xff}\xa7\xc9\xdf\xb0\xd9\x95\xe4\x1e\xe3\xa5\x9c!J<\xbb\xdd\xca7)\xd6d\xa2\x88\xb4\xdb\x17\x84 \xed\xdf\xbb\xe7\xb7\xa3\xf5\x1f\xca\xf4L\xf0\x81\x0f\xe5\xf9\xfah\xb5oR \x0b\xf9\xff\x0fB"\xfa\xd6\xbf\xf0\xf1j\xfd\xfaSx\xf9B2\xf2_j\xf5z\xbe\x9a\x8e\xec\x8b\x81x\x04\xd7\xe0c\xed2!\xa95m\xefD"\xf7\xe2\xcci;x\x81\xa9\xf1\xff7\x99\x90\xb4\xf1\x12\xd5V\xab\xd3\xdf\x8d\x9e\x07!\x18\xfd\xfap\xd8\xad_\xdf\xcc|\xd7\x80X\x04k\xf8\xcb\xcdi\xf5\xca\xea\x87\xdd\xf5\xe1\xe6\x8f\x8fBH\xa2\x9denV\xbb\x9b\xd3Z\xafJ\x92\x12\x08, \xbc\x80\xd4\xb4\xf1\xff\x1fW\xfe\xb2\xf2M\xc67\x818\xd5@W!<\xbd\xfe(\xb6=\x08O/\xe0\'q=\x7f\x14{\x9f\tQLX2\x9aM\xd4\xc65\xe1\xdf\xe1\xf3V\xf6\xb0xr\xc0\xefc%\x7f]\xc3P<3@\xf8\xfai\xfa \x9e#H_\xfe\xab\x0fz\xbbw\x92\x94\x19s\x89\x0b{tN\x9e\xf7=\x1eO\xd6\x02E}\x8d\x0c\xf9D\x9d\x98\xd0\xba\tF\x9b\x13\xa5\x83F\xcf\xcc9\x89\xa1\'1\x01%\xc6\xa1\xc4l\x94\x18\x94\x06\x90\x82\x06$\xc6\xa8\x84\x0f\x89~\x0e\x94\x12\x94=K\x8d(g\x10\xc5\xac\nm\xc3e\x90\x13\xc5\xa4\x8eVq\x89\x04)\xea\xe9_\xc7\x89\xd2\x8cj\xc2\x98C\x94:NT\x17\x0c\t\'B\xb9GU\xb8\x926H\x06\xa7%w\xc1\nuz=}|\xa5\x9d\xbbB\xf6\x92\x16\x08x\x05^\xce ;\x87\x97\x83\x9a`|K\xbf\xdd\xe0\xbf\xab\x8d\xe6\x18\xff\x95\x97\x85\x83\x9b\x13//\xd8/X{\xd1|\x91\xca\x07\xb2\xef\x92l\x90\xce\x17\xe5\x82\xb4\xbd&\x13\xbc\xfe9\x17\t~\xceE\x82\x9f_\x96D\x02\xce\xf9\xdb\xf8+\xb6-\xbfZ\xe0\xfc\xc8\xd4\xbd\x0e=\xcd\x91\xa9o\xf4\x85\x06\x0b\xf9t\xc4\xe6E\xeb}\xa9\xb5\xcc\xd7\xd35W\xd8\xb7\x1b\xa8d\x9d\xe0\x9cy\x03\xdc\x18\xec\x15\x9b3v\x89\x84\r\xcf\xfe\xef\xf6\x8cE\x02\xf9,\xd8\x1f\xe6\xc8g\xfd\xe9\xbdl\x8bx\xfd\xe9\xe6\x030\xdc\xab\xcc\x18!\xdf\xd2\xa1\xb8\x8b\xa2-b\xb3\xf3\xf6\x87\xd9\xfd\x9d\xa7\x9a\x15B\r\xa7\xc7\xd9\x8cN\xb6\xd8\x8cS\xd9\xfe\x00s\x14\xcd\x0fr\xfa[ZT\xfe\x14\x8c\xa9*f\x0b\xd82\xde\x04\xacu\x90\xbcS\xccqd\xdf\xba\xc00\x97Go\xbb\x02\xff\xab\xff4\xa2\xa5a\xab\x04\xb3\x93\xcb"V\x07F\x83\x93(|\xbamTl\x8f\x06V\x87=\xfc\xebn\xb5\x1e\x1f\xdf\xb8\xe0\xad\x06k\xb2o{\xa6\xb46\x81\xea=\xa5\xa2J\xb7&\xcd\xd5k\xf4t\xbc\x02\xd5\x84\x8e\xda\x11\xcc\x81\x80L\xc2x\x06\xa8\x82%\xdf;7\xe8\x9e6\xeaq\x89\xee\x97h\x85\x89\xbe(\xbc\x16\xcc\x9f\x91x:\x12\x1fH\xe2\x1d\x11~\x93\xc4\xa3\xc2|-\x89\x17\x86\xfbg\x12\xcfM\xea\xd3\x11\xde\x9eDpH\xf9\x12\x977\xbc\x0baM\xc6\x7f\xcf6\xd7:\xb8\x17\x8c\xa7:\xa4\x94\xff4\xe44\xe2\x9f\xcd\xafw\x1d\x1dJ\xf4\x81\x17y\xdd\x0f\xf3\xf5\xc7\x9c\xd3\x85\xd6\xaa\xfe+{\xd4\xb9[:\x03\xf1\xb6\xb4\xb5\xc6\xd9~x\x9eq6\xd6dB\xd3O\x05\xcef\r3\xc2wg\xcc\xef\xa8\xc1\x82\x11~\xb8\x90\x83\x9d\x9e!\xe3_\xe9s\x01\xf7J\xfbE\xde5\x8dqm\x99\xb1\x12\xf8\x95\x18.\xaa\x99\x93\xe7J;\xd0\x1a\x996Y\xe4P\xbd\xff\xdb\x9c\xe1J\xa8\x1c6\xa0\xf1\x9d\xe5D\xe9\xd2\x80\x03\x89\xb6\xc2\x97f\xdc\xc7\xdb\xc0\x9d\x00Q\xe6;\xfb\xc6q\x9cf\xd7\x08^\x93\xbe\xcd\xdbR\x9b\x9f\xf7?\x1e@I\xbd.r\x185e*R\xd9\xa8\x9d.\xffX\xf8*L\x13\xcb\x06]0a\xc3\xd7\xdfn\x17\xf6\x00\xb0\x91t6b"C\xe3\xf6f\xbbu\x0c\xacb\xa1\xf6=\x1e\x9bqx5g=\x06\x0b]\xef)\xcb\x98\xc3\x18l\x81\xee\xe4:i\x03\xec@_\x9f\x14\x87\x96_\xd5x\\\xe1\xd5\xc8\nFN\xb6\xd0\x19\x0f\xd6\x13\xe9fU6\\\xf5B~\x1b\xae\xfa\xce]\xf0\x0e7@\x92\x81\xd1\x02\xa9\xf92\xa2-\xb2\x05\xb2\xe1\xa4\r\xabR@2\xb7\xf6:\xb8\x8c\xbd\x01\xd6\x91\xccr\xeb\xd6\xcc\xbc\xe3\xeey\x99\x7f\xdc\x916\x9ar\xd7\xa4\x9b\xd1\xd5&tn\x81\xe4\x13\x8d\xc1zL\x1fd\xf1\xa0\x7f9]M\x7f\x1c\xb2\x93>6W\x8f\xfa\xacK\xfd\xac\x17\x93\xd0a/\x9ak\xa7\xfd\xcb\xef\xb3\xd3\xfe\xe5/\xd9i\xff\xf2\x97\xefK\xa7}&\x94\xe1\xff\xd1\xf1Z\xb2]\xe6Z\x8b\xff\xff0\x82CV\x9c\xf6\xe2!\xe0=\xc9\xc6{j|8\x14\x0e\xfc\xe5Ef\xca\x8a\x1c<\x9e\xfe\xb9~\xb2\xcd\xec\x8ac\xc9\x96\xc8\xf5\x93\x86\xe9-\xf0w,\x19\x0e\xb9f\x02\xffG>!\xf8\x81X-0\x04\xd9X\xda\x0e\x91%\xa0*\xe2\xed~\xceP|\xfaks\x8b\x1f\xa8"}\xefT\x91\xbe\x91\xaa\x88\x18\xfc\xb6\xd8XZ\x06\xe3\r\xb9\xc6\xd10\xe3\x9a\xd4>\xc48\xc7\xf0\xed~/)\x1f\xcb\x83\x17\x95\x0f\xd86\x8b\x1b\x068\x85X\t\xb1\n\xd04:\xbd=\xbd\xd8~\xde\xdb\x9cU\x80\xa6\x01=\xda\xa9Q\x8f\xc90R\x8d!\xd5&RM\xa3\x17\xe2,\xd7P\xbc\x98\xbe\xa6#~\xf0=\xbb \xa8[OQ\xf0\x94;,{\x1c\x13F\xe9y\x1c\xcf\x1a\xadoxm\x08\xa3\xb4\xfe\x1cUQ\xde_\x93\xd3\x0f\xa5p\xb2l\t\x12o\r\xb1J\x03\xb0\x18\nV\x82\xc7bd\xcb\x8f\xfa\x8e\xae\x12\x89W{\xce\xcbT\x88\x9d*\xf1\xd0&0c\xce~S\xd6\x9c\xb2m\x8dO>\xa2\xff\xac\xf3\xd4\x10\xbdikt\xda\x1ao\x17<Q\xa6H\xe9\xf8\xa6\xc1KJ\xdfD\xc37!G2\xbc\xa06xZ\x8d\xff\xb2\xd4s\x88\x94\x01\x05\xc7\x06_k\xebwD\x17]{k\xd2\xc8\x1a\xdc;\r\xeb\xa9\x83\x13\x10([\xa4\x06O1\xff-)b\'\xca\xb2\xddI\xd7\x9a\xe8r\x0cT\xdc\xd5\x8b\\\xf7\xcdt\xfdi\xba.\xd8\x12\xf9\x85\x84\xf3\xfe\xf5\xfap\x9cw\x89-\xb1\xd0\xb7\xce\x82\xb3\x19\x89\tg\x17jl\xf8\xcd\xaf\x19\x1bfM&4=)\x99\x13\x99\xba\xe5O\xb4\xe0\x088\xa3\x80M\x96\xfb\x05\xbdJ\xb6\x95-\x17\xa8d\xa7\x87\xbc\x9fr\xad,{v`\xd4Y3\x8b}\x9a\xe4\xe2\xf3\xc7\xa9\xe8l8j\xd1\xc8h\x19\xe3\x06\xe5\xad\x93-K!P\x03S\xe4\xba\x8b\x94:\xed\xdfk\xe3\xd9\xb7g=\xa3\xd9r\xc7\xdeY5\xcf=\rz\xf82\x83c\xf6\xfe\x0e\xe5\x9d\xc6\x94\xbe=\xf8\xf1\xfc\xff\xbbE\x05p\xd78\xff\xe9\xb6\x1f\x1d\xb7\xd7\xdb\xa1hx\xcc\xa6\xba\xad4\x1f\x93g\xd1e\xa5Pk\xee\x06\xf3\x7f\xfb\xac\xa5\xac&f3\x1e\xd9^(*\x8b\x97L\xb6\x14\x015\x9d\xdf\x9a\x99E2[e\x8c}\xeaN\xf7uC\xab\x96uJ\xe8\xd7n\x9d\xa6\x1b\xfa=\xa6~\xf9\x19\xce\xad\x92S,q\x98E_\x1a\xf3\xb2%\xfe\xb7$\xba5\x89{M"b\xd3`Y\x19H\xcb\x83l)$\x87"i\xdb\x18\xb4l\r\x0fhF\x0f\x16\x066Y\x8bc\xda\xe8\xea\xa2P\x1e\xf0\x82\x91.h{\x1cS\xe1\xa31g\x9c\x1d0\xc8h\xf9\xdd]\xe0\xfb\xba\x9a\xaew\x15\xf7\x17]Z\x88\x8c)t\xab\xf2+\xde\x96\xfa\xbd\x925|N8\x0co3i\x9b\xe0V\xcbV}4\x1dZ\xf9\xdb\xd2\xfb/t\x87\xc1\xff!L\x06\xb9\x18\xe3n\xd8\xae\x8an\xb2\xe4epOYz\xe1>\xbd\xf0E\x1e3\xf6\x02\xe4\xe3r\x1fZ:\xf1\x85n4\x8c\x85\xe1*\xeb.\x06\xc1l\xfa\xcfw\xafm\x19\x174\x9f\xedjCc\xa7\x8f\x8d\xb1\x10\x0f\xd3-8\xdc\xf6>\x18\xa3\xbb\xda\x15\x1do\xc9\x1b9\xd4v\xf2\x92\xfb\r4_\xe0\x8e\xdb\xf1\x8c+\xce\xbd\xb3M?\xb8\xff\xdb\xdd\xb6\xe8\x90K&\xbe\xad^\xe0KE\xf6\xb8\xfb"\xb7\x1c\xfb?7\xac\xda\xa2\xbb.Y\xc3\xb1\xb6\xb3.\xf7\xda\x9d\x9d\xfcro\xde*\xfb=t\x10\xd9Rt\xf0%\xeb\xad\xf9\xf8N\xecR\xf5\xcd%\x9e\xbe\xa1\xd3\xbbGd\xa8k\xaf\xeb\x19\x0c\xf6]\xfb\x80\x08\x83\xda\xd6\xdag\x96\x184\xab\xae}\xf8\xaa\xd1\x81R\x9e\xea\x912\x9e\xf2\xacl\xedY\xa0A\x87\xd5\xda3=\x83:\xd4\xda\xc7\x83\x1a\xd4g\xd7\x9e\xe9\x19dzko\xd65h\xc7<\xe9\x83\xdaS\x9e\xad\x9e4\xc7\xd3(\x1a\xb5\xd7u\x8f\x14\x8c\xe2\xb5\x7f\x8d,\x1c)\xd4\xe9\xd7\xde\x16\x90Q\xb02\xaf\x852\xca\xdd\xd7\xd1\xf3y\xf5\x95\x93\x9d\xef\x0b\xabQ\xfee\xc4\xab\n\xef\x85\x91\x94\x8f2q\xe1\xc8D\x9e\x98\xb2F7\xa4\xb7S\x9fHC\xaf\xce;F5\xbd;\xe5y\xbdV4\xb2\x0f~\xd6\x8a\xbeH\xefGF\xae]%\xf1^\x08\x90\xd1\xf4m}\xea\x93\x0e\x9f\x0cHC\x8b\xf4\xc1\xd3qU\xad\x7f@K#\xfb\xb0k\x1d6\x06\xac\xaa\xa5\xe7%\xb2#\x12^\x8e\xa1\xe7\x1d<\xd9\x11\xa9\xfc\x9b\x14dO\xef\xaa\xf3\xdf\xb2\xe1/v\xa0\x89\xbc+4|i\xe5\x15r=\x86\x0f\xd8yjDj\xf0\x1bM\xc7Om0>h=\xe2ve\xdb\xc7\x99\xcdqk\x01\xd5\xe3\xb6\x1b\xfd\x0fB!\x05?\x16\x85\x1bt\xf4\xd4\xc0\xb6+Q\xe0\xc6F74nz4p\xe0\xcf\x83~r\xf1\x07\xb8l\xa0\xbf\xb9\xbb\xd9nor\x0b}lOd\xae\x1f\x0e\xd7ow7\x1f\xbc\x85>\xeb\xb2`\xa2\x17\xd3\x04\x1b\xbdh\xaf\x1a\xe9\xf3`\xa3\x97Or#\xfd\x93Wg\x82\x8d\xe0\xdc5^\xc62\n"\n\xebF\xfaF\x86\x19\xe3\xff7\x99\x91\xfe\xee\xe6\xfd\xfb\x1bi\xa5\x17\xcfFf\xfa\xbb\xbb\x87y{s_\xb5\xd4\xe3\xba\xfa\xe8\xaec\xec\x84l\xf4\xa7\xa1\xcf\x98\xe8\xb7*\xc6\x04o&\x90\x8f\xaa\xc6y\x1e\xe8\x0b\x8e\xdb][5\xcb\x83\\\xe3\xf9`\x12\xdb\x8b\xff\xdfd&z\xf7v \xef\'3\xd4\x9f.\xddd\x86\xfa\xf4\xbdE\xd9e\xd6\xfe\xaf\x97`\xb6\xa0\xcd\x17bs\'\'\xd9n\x86\xd6Y\xea\xf7mn\xa9O\xbf\xd5\xedUa!d\xaa\xbf;\xad\x84}\xa9\xb2\xb1\xde6LD\x98\xd8\xdf93\xd9\x9f\x86c\xb3\x80\xf8\xf1\xddt\x7f\xc6`\x7f\xc1\x04\xd2l\xbf\xb8\x87\xc8l\xcf\xf7\x10I\x10\'\xa1\xe3\xd4\xbd\xdb\xa9\xd37n\xed\xdc\xe6\xb2C\xd6\xe3\x11\xa5\x06\xc9c{v\xa2\x13e\x19\xd5\x12K\x02\xce\xd8F\xbe\t\xac\xd0\xd2U \rg\r\x9aN{\xb8\xaa\xe8j\x87\x9c\xd1\x10\xefS\x9el.a\x94\x82\xabf<7\xe1\xc8\x82_\x0bn.x\xbd\x90\x04\x84\x9c \xa4\x88T\xc6H\x05\x90T8I\x05\x17)\xd4\xd0\xc7X\xe4 \xbf|\xb3z=\xbd\x9b\xeerk\xf3/\xf3\xfdq\xbe\xbbf\x97\x13Vrj\xbe\x9b\xfe8\xac\x9e\xdcM\x9bR\x12\xeeE\x8c%\x9f#\xe1/\x95\xcb56\xf3K\xee\x0b~\xfe\xe7<\t\xf7\xf5\x9f/\x8ai\xf5\xe9\x8c\xc9OQ\xab<\xf1%\xe1E\xcc^=\x96\xf8\x12\xe4\xe5v\x8c#\xf1L\xdd\x99%\xbe\xa0!`d\xffo\x8a\xc6k\xf7\x8a\xee\xd3\xd7\x03\xdf\xa7\xf2\xeeP\xdd\xf7\x8d\xab\x9f\xb7\xdb\xc3n\xbe>NW%\x9d\xbf\xf2\x16(|?{\xf2<p\x16f\xb9\xc9ga<\xcfF\xf7s\x92\xfb\x02\xc1J3\xcf\x80\x01\x96\xd8GC\x000\xc9\xa5\x08[\xcb\xd8\xa3]\xc5&\xe4\x9b\x9a]V\xd1B0O%\xee\tI\xbf\x1d\xe3\x98<\r\x18xk\xc3rb\xcc\xc8L\x07M\xd1P\xfe\xd3\xc7\x87\xb7\xd3q\xfd\xfaS\xfa\x91\xd0B\x00\xaf\xef\xe9\xe9\xcfQ\xda\x08\xe4\xf7\x9b3~;\x8c\xbd\xb7\x9boZ\xcfo\xf7`+(\xd8\x07\x1c\xaf\xda6\xee\x85\xee\xdav\x00/\xb9\xe0\xba~/]\x1d~{?\xcb\xed\x84\x06\x02X\xce\xaf\xd3\xf6\xdd\xcd.7\xa1\xbf\x86\xb5\xcaM\xc0\xf817b\xef\xd9\xff-S\xca\x81\x99\xeeY\x1e\n6\xb5\x19\xafm\x04\xeb\xfeOo\xa1\xb0\xd9/\x05X\xf8\xd3\xe9\xe8\x12z\x8cZ\xf93\xad\xc4\xce{9\x1d\x9a\x0e\x92\xf5}\xde\xca2\x9e_\xf8=\xad\xe4\xef\x10~\x81\xf4k\x94\x86z\xb0\xc3|\x94\xdf(D\x7f\xf9\x1d\xd1\xd9Q;\xd6\xef\x9c#\xcey_\x88\x01+\xf7{<AA\r\x10\xc1\xeb\xf9\xac\xf2\x06h4 (\x1fs\x8b^W\x05X\x1b\xc0\xcf\x94\xb7\xc1\xa3M\x80\x08\x83D\xef\xdd\xaa\n\xa9\xceS4D\xeb)\x18\xa3Cw\xacF\xcaF7.h\xbak\x9dP\xb4\xc4\x16\x9d\xc8#R:\xc6\xcc\x82\x94\xb0\xc6\xe0/\x10\x12\xd6(\x06(\x8b\x8ei\x18\xc5`\xac-Q\xe0\xd0\x1e\x91j\xd8}\x1a{Z\xa4\xa0\xa7\x17\xa4@8\n\xf3)\x9c\xa1a\xd7Ppr\x1e\x85\xe8\xdeV\n]\xd8-\x8e\xc9\xdc\xdb8\x9f\nkQ\xec\x9a\xc5\xf7\xd2"\x05\x91\xc5\xf4\xb4\xf0\x06\r\xbb\x86>\x0e\xd0\xef\x9b\xd0\xd1\x84\xc8n\x18\x11\x85.x\r4\x82\x81+C\\\x86\xe6c\xa3\xcd\x02\xcd\x0e8t\xcb7\t\x98+\xf0\x9e\x0ev\x99\xc6\x8d\xe0\x83\xb9\xd9^\xc2\x08\x81\xb8\x19\x17\xe5\xa4g\'\xa9(\xd7\xb3\xb9\x0f\xe3\xf8n^\x85^UOG\xa9W]>\xf2=\xbf^\x89\x0129)v\xfb\xe6\xac\x90\xf4\xec\xbbLHz\xf6"\x13\x92\x9e\xfd\xfc\xa2\x9e\x0b\x0c\x96P\x16T\x04\x07\x12I>\x99\x90d\xb9\xf7\xa0\x8b\x87\x9d\x1d\xaaQ\xd5\xe3\xc8\x84\xa4\x1e\xd4\xf6U\x94\xa10m\xb8\xc9|\xfe6\xf3\x8dX\xde_\x08O\xf8^\xdf\xaf~H\xbe\xee\x99\xaf\xc6\xe4\xa8S\x1f\xaf\x19O\xae\xe3\xe4\xb3S\xd3n,8`\xf1\xdd\xe1Y\xcf\xa3\xbb\xbd\xb7\x87\xbf#\x10\xaa^A\x84\xb2_\xdb\xff<\xcc.\xabX\xacL\xc8V>\xb9\x9a\xfc*\xfb([\x91\xf0\x14%/\x8a"X1a\x0c\xa3\x03W1\x80\x1c\xc7\x00\x1f\x8ca\xe3mK\xf6\t&\x80!\xe8J\xcb\xa4-\xe6\x95A\xbb\x05\x08i3:jV\xdc\xa2\xc1\xbc6\xbaj\xe3\x00+\x06\x08]\x18\xc1\xd0\x83]c\x15\tm\xb8L\x16\xc3\x1c@\xacc\xce+\x8cZ?\xbd_0\x80\x1cV?\x80;\xe7\xd3\x11\xfc;w\xf7\xd3=\xfc\xef\xad\x90\xde*{\x0b\xe47\x7f\xb1\xb6g\x98\xb7\xc7\xb0\xc8\x07\xeb\xdd\x01\xce\xd7n\x9dK\xa7\xed\xf6\xbbB\x00\xbc\xd3\xee\xa7\xed\xec\x84\x91i\xbb?\xbd\x8c]\xd7\x9e\xee\x9cv\xddN\x08u~\xd2\xf5\xd3\xbb\xe9\xd3\xa7\xe9\xf7\xc3\x15\x13!1>>Y\xd4\xed\xb1\xb4\xdfo\xc4\xd2\x99\xdb\xa7\x89^\x15\xab\xa2`\x87\xb9\xc3*s\x01ML\x88\xc2\x98\xf8\x99IW\xa6h\x93\xf9\xa1\xfa\x0c\x18t\xef\xae\xaf\x9e=|x\xf7pw\xf88\x1f\x8b\x99]\x17\xaf\x14\x85\xbe\xcf\\\xe9C\xf1\xf8\xdc\xaa\xfc\x08\x80vv\x10p\x0frv\x1c\x80\xd0\xf7\xec\xbf\x1e\xd4\xbc\x1b\xaf\xdf\xae\xde\xf9/\xa3\'\xf1IB\xb4\x86vk\xed\xfaq\xbb\xb0\x8fb\xcc\x86n7\xc5\xde}\xb3yDI0\x0b\xebKC\xfe\xd2p@\x16*\x98\x86\x11\x9e\t8\x84\xf0M\x1d\x01[\xd6\x03G\xbe\xa1`N\x80\xbe\xa1pG\xbc\x16!`Z\x16\\\xafC\xb4&\x91\x1d\x04\xe67,V\x91\xe2\xf4q\xe5\x84\xc0\x02@<\xce~D\x18;\x98\xb5\xa0\xf9D(\x87\x9eH\xcd2\x1e\x00\x83gM\x90o\x80\xc2\x13\xe2@uG\x91\x9ec\x84\xc1\x89\x8bl\x93\x08S\xc0\xcfYS\x18\x88\xb6\x144jR\xb2\x89\x98<!XU\x9b$vUk\x1a\xaaA\xd2\xa6$.R1\x12\xfd1Za\xf8i\x8b\xd4\x10CL\xf1\x1a~G\xedMh\xa7/>&\x14\\k0|\x97(\xd8)#R*\xa1\x9a\xb8o\x14\xec0\x84\x9eQI\xd8\xaf\xa2\x9d\xd2D!q\x8d\xf05\xa0\xa5\x84k\xa0c\x0cA\xffP\xbeg\xcb4\x8e>h\x1c\xb0\xb2\x0e)\x13\xf7\xb72|\xb7\xa3>\x80\xf6C\x05i!\x912\x9eRHi\x7f\x1fi\x0e0J\xcf\xa4|\xf4)\xa1\x94O\x9aC\xaaqHm\x84k*\xa9\x16#5\x1c>f\xaa\x19\xa5ZS\xaaQIm+\xd5\xc4\xb8\x96\x96jpR\xbbK5\xbfD)L\xd4E\xa6#$*\xa6P>\x13\xb5\xb4\r\x89\xa5j\x80\xbcP\xda\x19m\x80f\xd0\xfex\xc1+\xba\x81nl\xc3\x12\xfa\x96\x06X-\xfaE0(.\x84\x80\xe8\xf1\x87e\xe2\x15\x0b\x80[*\xfe\\m\xf8e+N\x98\x88\xcb\xd5\x02\x86\x84\xc1\xf3 A\xecj\xe2h\t\x96W\x82\xf2%\xf0\xbf\x182X\x82\x19\xc6\xd1\xc4\x12\x9c1\x89@&\t\x8a\xa1\xf6\x11\xe3:\xdc3\x06\xbd\x0f"\xafU\x18:b6\xe8\x112\x87;\x16\x02\xde\x84C\xb5\x8b\xba\'\x9d\xb8\xf8\x14x\x1c\x87k\xa0\xc1\xd2}*\x06v#w\xd0\x01\xc2\xcb\xc4\xbcY\xc6\xa9\x16\xf5\xc4o\xe7\x0f\xdfxElU\xd6\x18+p\x95u\xe5\xf13n8\xabG\xd6\xc7\xaa\xa9\x94\xe1q\xce\xeb\x95/r\xbd\xf2\xdb\x82^\xf9\xed\xbfC\xaf\xcc\xba\x02|\x13F\xe7\xc1<Z\x1a\xb6M\x7f\xc6~\x0f\x8a\xe8d\xca\xb1\xe7%}\x93!sz\x15\xea\xdf\xaa\xdd\xba\xaf\xfd\xde}\xed\xfb\xedi\xb3,\xea\xba\x97l\xb2\xba\xda\x0bw\x83\x1at|\x14=\x18?\x17\x0b\xb9B7\xa8*c\x8cN\xe3\x17\xab\xcf@\xe3\xe3\x9fS\xa87Le\x9e27\xc5\xff\x7f\xf5mt\x8a\x18\x160\xc9\x82\t\xd0A2p\x07\xc9\xbfVM\xc7H\xcbM\xec\n9\t<\x03\x01\x95u\xcb\xd0\xc9\x9a\xe8\x8fQ\xed\xe3\xe8\xfb\xe1G\xf4\xfe\xe1\xfe\x0b\x94\xffx\xfb9;@\xfc\t~\x9eM\xe0\xa41*\x8fm\xb5\xf9\\\xfb\xc0f\xe3p\xd2\xb6\xc3~{\xa9\xb1\xe0\x87\xc3\xf5\xfd\xbb\xe9^\xba\x7f\xe8!?n?]O\xe1\xa0\x99\xfc\x13O\xcb\x16\x84\xfc\xe8\xa8\x9b\x14\x14G(\xdb\xca\xa0\nr\xb0\xf8\xffk\x96u\xd1g\xea\xfd\x94\xfb\x93\xfe\x8d\xd6\x8a\xec%\x82\xa9B\xee\x943V\x8b\xffW/#qX\xfd\x7f\xc6\xfc\xb1\xfa7\xdaH\xbe\xc5\x98\xe5\xd5\xf6\x9d\xbb8\xf7\x17XM.\xfa\xa5\x15-(\xd5;\x1f\xd7\x9a\xa2 \x01\x95\xa2P\x88D\xfb\x06j\xb4N\xc8e*m !\xe3T\x0fAe\xb2@\x0eHj -\xaah\xd8Y1MVc\xfe\xa7j\x01\x8c\x86\xb4e\xd0\n)LE\x01\xb6\x02\x05\xc3(\x0f\xfa\xb3\xd6mp7\xf9yIkD\x92\x14Et?\xa1\xfd\x03bp\xd6\x9a4;P\xa15i}h\x96\x18\x832\xdc\x01\xd90\x7f\x98\xa2G\x00\x83\x82B\x1b\x02j\xd5\xaa\x0fJ\xf6\x00\x00\x10\xa4\x1f\xb7p\xb5a\n\xb2\xea\xc3#hN\x0e\xa0~\xd0\xc8\xf0\x15\x14A\x1f+P}\xfb`Y\xc1{[\xd4\xfa\x1av\xaf\xb6\xb4H\x1b\xd5@Gv)\xc9\xd0\x92\xe3\xc8aU4Q\x07\x8fo\xd2y\xb9yE\x91\xd5\x0c\xb0\x86B\xe7\x86^{\xb2\xcd\x04\xe96\xe1\xa2\xa6\xf4\xd3\xf4\xe1\xb0)d\xb9\xb2\xf6\xaa\x17-\xefS\xd7}\xe4<\xa4\xe1\xc8\xf6jT\xd1\x93L\xb1\xf9)\x0f^\xfd\xe9I\t)\x0f\xb3az&\xcdN\x17A\xfbO,\n\x08\xc3Y!Hh{aR\xab|:x\x87Y\xeb}\xa9\x95\x85\xb4f\xeb\xc7\xfc\xd5a!\x97\x15\x86\x9b\xab@\xb7\x98\xdc\xb3e"\xf06\xa2\xf9c\xf8O1\xa6\xa7a\xf2\xe9(\x83v\xca\xa9\xac\x03\x13\x1bg\x96\xbe\xb3=\x9b\xc4zz\x0c(s\xb0\xc9rX\xe5\x0b;\x147T\x94\xd9\xb6\x00\xda\xefcpv\xa6\x92\xbb\xbai\xc6\xc1\x0be\xee\xef\xec\xd4\xd6B\xee*\xccQ\x04\xcd\x93\xd3\xdf\x1e\xabO\xc1d*\x16[\x83b\xc4\xc0\xf8~9oU\xcet\x8c\x9f\xbd\x0cq\x94M\x82\xa2\x84:\x93\xaf\x8a\x19\xe0\xe77^\x00\x8aM\xd6\x15\xf2jf\'4\xf7\xdb\xde,\xc3\xf3\xf7\xbbf\x97\xf5{\xccDU\xc9(\x05\x1b\x15L6c\xc1\t\x83\x16\xec[0\xf7\x8c\xf5s\xc1@\x83\xf7D\xdb`\x14\x1f\x80\x05\x13\xd9\x03\x83n\xf9i\xdf\x05\x86\xd5\x01\xd9\xa3\xf9R\x01i\x90\xc4\xab\r\xf3\x0chDJ\x02\xf3\xa6#\x89\xe9p\xd6\x8f\x1cJ\xa3\xf1Z\xa3\x9c\x10\xc9\x9e\x0f\xd5\xd2\xc8d\xf7\xc4\x07\xec#\xca\xbf#\x19\xce\xbf#uD\xfaw2\x06\x83\xf7_\xebh\xf6\xb3L\xc6@;#\x91\xe8N\xd2:\xf15Q6\r\xe2`\xe8h\xe4\xeca\xa2\x9e\xa1AP\xfa\x8cQ\xc9\xcb\x89$Aj\x18\xf6\xf8F%o\xc3\x84\x8f\xd2\xf0et\t\xcc\x05}#\x1a\xaa\x0f\xab\x1a\x81\xd4\xb1\x92@\xec\x8c\x9f,\x92(\xcbQI\x82\x16\xf6\x95a\x98 :\xc2\x80\xe0\x9e4|\xde1\xacj\x04rD\xd2\x8dl\x9ap\xd5\xa3\x14\x92\xad\x17\xdcbF\x07\x1b\xb5\x06r\xc0/\xe81>\xc8i\x06\xfe\x03c\x82 4\x02\xd9\xb3\xadB\x19k\x1a\xc4QcB\xed\t\x0f\tb\x83\xa7\xab\xe3W\xc1K\x12\xee\xd5t\x95\xc4\xa4\x16\xee5\xec\x87c\xc8F\x8eO\x14\xa5\xf1.%5\x90m\xf40\x84{\xc1/\x11V\x15I\x85\xa4f\xcb@\t\xd3\x98\xe0\xfa\xc0W\xd7\xf1\xce\xd1\xe3\xa1\xe1=s\x87\x8a\x89\xfe\x15\xc5\xbe\x91$\xd9\xf1\xe5\xe8`0\xc7S\x07\x9d\x97t\xbcip\x93\xd1\x89\x85\x1fM\x8f\xdc!\x11\xc6\xd6\x08\xeb\xd3$\xf7R\xda\x9f\xc6Gl\xc2v\x18\x80T\xdcn\x8f\x92>\x99\xf8\xc7\xd4\xaa?\x9e\xabA\x05\xd5\x02\x9e\xec\xef\x0e\xdb\xa9R\xe3!^<S\xe4!\xebx\xae\xca\x83\x986-\xf3 .V1\x9d\xf3\xd0\xae_sI\xf5\xd7\'\xdf}V\x99\x070\xb2+i\x8f\x07\xfb\xf9\x92\x08{I\x99\x87\xc1w\x9a65\xa3y\x19\x05\x1aK>L\xa7\xd7\xf2~*\xd6|\x10oL\x14}H/2\xdb\xb4\x96\xcf\x8a\xa9\xbc:\x0b\x7f\xbf\xb8\xf8\x83[\t\x9a\x95?\xb3\xfa\x03\xd8\x85\xa1\x02\xd6\xb6c&\xe4$@\xfes\xaa?`9,\x1e\x14\x9f\xd8\x82\xbf\xb4\x06D\x03&\xdaM\x84\x8a\xe1\x06Y\xdbp\x8bo\xad*\x04|\xcc\xcf\x08\xb0z\xe2oX\xa8\x16\x91~eY)b\xf2\xd9h\xbb\x96\x01Y_bA\xc5\xda\x11\xad\xf5\xa5\xb5\x1a\x1b\x00\xae\xe7\xde\x9e7\xa3\xfa\xd2\x12O\xd2m\x0b\xd2{ZTbr\x1b\xf8\xe3{)\xcc\xfbji\xb0{\xcf\x15\x97P\r\x18\xf7\xf2\x8a\x11\xe2W\x93\x95\x8c\xc0\xf7z\xc6\xfcxy\xfd\x88y\x1d\xd7\x03U\xbb\xd0:9Hc,\x96\xc2\x80\x1a\x13\xdb\xcc\xd8\xf8\xa5\x96\xca\xcf/M\x81\no\xf5\x1c8o_t5+\xfe\xf2\xeeP\xa9X1\xf6\xed\\\xabU\xe1\xaf=b\x84\x95KK\xd3Q\xa0\xd6\r\x0f\xe9\xd1\xf4\x7f\x87G\x83\xb1Nj\xe4ip\x10_M\xc9x\xca\'\xe3k\x1d\xacs\x90\x8a\xc7\xe2\xdb)u\x1c3\xe0(\x9a\xc2k\r\x9a\xe4\x17\x88\x04\xa0\x88v/R\x93T\xec5\x15\xd2L\x80h\xe9\x8a\x899t\x94\x15\xd7\x02\xd1\xc54\xb8\x91g\xc1\x8d<Q\x1cFCY\x05\xba\x8d4t\xebA\x05\x0c\xae\xa7\x89\x80\x03\xaaA\xa0\x82\x86]\xa3\xe0\xf5\x06a\x0b\x06\xa4\xc6\x08p\xa0\x08\x18\x81\xc5\xe2\x1b\x13\xcc\x89\xbd\xbf\xc6\x02\xdd\tB\x01l\x9a\x04\xa1\x00\xb6S\x82PP^\x07\n\xb3w8;\x85\xa8\xc0\xaaI\xbaT\xec\x89\x00\x95\xbc\t\n\x1d\x7f"\xca\x88\x1f\xd85\xcd\xed~\xb4\x16\x90v\x83\x08\xae\xf1Z\x17\xed\x87\xe1>\x83P\x0f\xac\xa0\x1be\xd9k\x80sP\\\x18\'\xe0\x07\x08r\t*@\x8bO\xd4G9\x9d\x80\x1f\xfc\xae&A\xb2\xf5{\xa4\x0f\xff?\x933\xffcb\xa5 \xf9.\xb4&\x82\x9dl\xadKq\xe9\xa8$\xbe\xa5\xad5\xb9\xed\xc7\xbfer\x1bk"\xb9\xed\xc7\xa7\x7f+Y\x18\xedgX\x15\x93:\x1b\x97Z\x12\xdd3d\xc5A\xd3\x07\x03\xf1\xea\xc7\x9a\x11\xd1\xe6\xf6\x1b!$\xf99\xea\x0ew\x10x\xb4\x17\x81vU\xd4\x1f\xf4n\xa3\xbf\xbb\x84\xeb\xa3?\xcb\x00\xf8c\xcd\xfc\x97>\xfc\xe1\x98?<Kz\x9f\x96\x8d~\xfe;T\xcd}?\xd6\x8c}?\x16L}?.\x1a\xfa\xb8\xf7R/\x18\xf7\xc4\xe7>\xde\xe5O\xcc\xc4\x80\x8d\x14\x03\x96\xacy\xf6B;^:[t\xc59\xa4\x9b%\xfb\x9d\xe8\xf1\x98l4\xe1\x10#\x1e\x92\xfe\x98\x1a\xf0\x9co9\xe1\x0f#\xafZ\x93V\xee\x95_R\xe0\xa1\xb2\x04\x1a\xab\xe0\x9e\x96\x06p\xe7!\xdaq\xe0JG\xbc\xd0\r\xd0\x07\x1e\xa5=\xa5\x19\x85&\x1b\xe5ME\xa6\x0f|H\xc5A\x90cu\t\xffj\xc35@.\xe9#\x8f\x0e\xd6\x8a\x06Wl"\xc7&\x1b\t\xdegB\x9e\x99\x8aO-\xf8^\xca\x13S~)yi\xcags\x1e\xdc\x10\xef\xa7\x0f\xb3\x8c\xa9\xfap}\xb8/A\xaa\xc6\xf6\xaaF\x9f\xf7Y@R\x15\xf3\x04 U\xd1^\xc5Q}\x9d\xe3\xa8\xfe\x94\xe3\xa8\xfe\xf5\xa7:\x8e*\x0f\x873u\xd4\xd4}\x84-\xe7H\xa9\x9b\x8bQS\x1f\xae\xe7\xfb\xbcj\xb4|P\x04M\x85\x98\x8f\xeb\xf9S\x156\xb5\xb2n\x84G\xf5\xa3\xd6\xd1Q\x11\xbe|\x17\xe3\xaf\xb6\xe6\\\t\xa6)\x0bW\x1aY\xe2\xf8B\xc9\x8b=\x0f\x1db\xc5\x0f\x19.*d\x89/c\xa4>\\\x83\xea\xfb)\x87H\x15\xef\xf0P\xdcB\xcc\xbd\xe4\xabb\xccm\x84F\xdd\x0c5\x80T=Dh\xd4q+\xebd\xc0G-r\x1cw\xe9\xd3o\x19:*>\xc5\xee~\x01\x1c\x15\x154\x0e\x85\xaa$\x14*|`\xc9wN\xcd\x0f\xf7U\x1cT>.\xf2\x1dUG=-\xef/L\x9d~\xf8\xb8\xbe>\xacw\xb9\xc2\xd6OC\x807\xed\xdai_\xf0\x1a\xb1\x1e\x90:\xfd\xf8E\x9dyBK\x8a\xb5\x9d\xe2p\xa7\x18\xdd\xb2\xec\x86N\x90\xbeEy\x08Q<\xa2TZB\xf3R\xbe\xb1,EV\xb4")i\xd1\x1090\xff\t\xd5\xce\xc0\x8c\x1f\xd5\x06\xab\xef\x08q\x16\r3\xf3\xaa.\xc4\xa6\xb7@R\xc1\xe6\x8e\x07G\xf4\x14+Au\xa81\x84\xa1a.\x82\x10v\xd1RPF_\n\x9d\x10\x81\x15"\xecB\x04e\x88\x90\r\x0c3\xe9\x92$\x16*,B\xa4\x0eA\x19\r\'\r]\xa5y\x15O\x1c\xc2\xab*(D\rO\xe0\x11):"\x81\'K\xefI\x92\x7fDj\x90H\x1c\x12iE"\xe9H\xa4$\x89\x84\xa5,\x9d)Iv\x12\xa9PY\xa2T\x92F%\x92\xac\xb2\x14,\x91\xa0\x95fo\xe5\x99]C\xd8\x81,#,\xcd\x16c?\xbcE\xe1\xe2\xd7\xe9C)\x90%6WE\x8b\xacK]\xb2\x10\x93\x90`!\x9akr\xc5\xafy\x10\xcb\xaf/s\xd7\xc0\xcb\xa7%\xc4[-\x83@\xce*\x9aI\xd4\xfa\xcc\xe2\xd5/U:_\xa7\x8f\x05oN<+\x88\x16\xa2\x91\xe1\xd7T\x96]\x89Z\xf1\xe3\x14u\xd0\x8e!\xcf^\x16\xa8\x028\xeb\x18\xa8rV\x9e\xc8\x11\xd6g&a\x9c\xd7S\xc5\x1b\x00\x19B6\x966\x0b\x0b*\xeebPqUW\xdd\xf6S\xb3\xa0\xab\x8a\xc1o\x8b\x8d\xb0\x0c\x08\xbe\xfcP\xd1T\xfb,$e)\x18E\xccp\x8c\x1f\xf3\xe39(\x17>\xfa\x82\xde\n\x1bi\xbb=\xbb\x91@\x96\x10\x0b\n\xe8+\xbbv>\xa3\xbez\x84\xf4\x7f\x89\xfa\xea\x8f\\\x87Z\x1e\x00\xd7<l\xb9a\xe8\x9a\x04\x9b\n)\x84c\x042\xd3\x8e\xe2\xc8l\x04j\xbe\x06]/\x92^e$0V\x85\x95|\rQ~\xc6\xd0\xd3\xa3\xab\xf7\te\x18\xfc\xd9@\x98l\xfe\xbe>\xce\xaf\x1ce\x89\xd2\x8e\xe2\xa0j\x1d\xad\xdb\x87^t\xf1y\xad\xebi\x18d\\\x1f\x81W}\x91aM\x14\xbe\xa7eW\xf1a\xbe\xbb\x9bV?\xce7\xd7s\xee*\x16\x17\xeb\xae\xe2r\xc7\x05Wqi\xda\xe0*.]\xac\xba\x8a\x7f\xcc\xcb\xff\xfe\x98\x97\xff\xfd\xf1y\x89\x1f\xb0"\x89<!\'A@\x1fD)o\x89z\x9e\xf3\x00(\xa4H0g+i\xaf\x1c!\xedj\x90\x8c\xa2\xf4\xd4\xe8\t.^\xba\x17\x97\xeap\xe7\xf0\xfb\xce\x9fPw\xc5\'D\xcc\xb3\xe2\xa4\x12\xea|\x8c\x95\x18)-h\xc5\xec\x9d-\xc3\xfe\x04\x9d\xb6\xadC\x9c\x8dLwM\x80\xce\xf2\x8a]\xfb*\xd890\x98f\xcf\xf3dV\xe5\xf2\xc2\x88\x8b>H\xcc\xf3\xd2\x83#\x90Y\xf1R}\xd32\xbe\x04\xae\xda!\x02\x9d\xcf[\xdf\xe2\xad\xaa\xdbB\x82\x8b\xde\xf9j\xc3\xb1\xdc\x17\xbag9R,\xc2\x9d\x97\xa6\xbe]\xb8\x04\x0b\xa6+u\xacst]\xb6\xd1\xd5I\xe9\x18kf\x89m\xb3\xcc\x89V\xfajK\xab8\xd6w\x99\x84=\x87u\xb4\xf6\xcb\xd7\xf1pw\xd9/\x02y\xa3\xfcE \xe0yi\xad\xc1\xa8;x\xae\x08E\xc4\xc0|\xdb5\xba\xa0d\x1b\xc7y\xdbM\xdb8>\xd9\xabG\xe6\x8dP\xc7\xbec, \x92>\xe4M\xe9\xc8<\x8c\'-\xe3\x8fJ\x13f\xa8\xc2\xa1\x02\x87jKd\xe0X&%\x1bOj\x86j\xcd\xc9\xce\xd7\xe2\xe0@\xe4\xa4\xfd\xacG\xec\xcb \xaf\xc3\x9d#\xceb\x90R\x8c\x1a\x92\xfb\xbc\xba\x1d\xc6L\n\x83\xac\x07\x9c^1\xaaIz\x12^\xbb\x0f\xd5b\xd7\x80j\x91j<\xa5\x91\xd2\x9eR\xb1\xe7\x18nS\x8e\xb0\x08\xbe\xed\xcb\xaft\xf1\xa6!\x8cg\xa2\xe4\x80W\xd8h\x11\x0b\xd5r\xa2\xe7\xc4\x18\x05\x83\x11\xe4\t\x13\x91VQ\xa9\x87w\xd8s\x0cqZ\x01X\xa3\x99\xb8\x92\n/\xa9`\x93\n=\xa9@\x94\nK\\\x90\x12R\x96\x90\xc1R\x01M\no\xa9`\xc7\x85\xbeT \xc4:6\x01\xf9V\xb11-~\xc2\x86\xc1\xe2\x86o\x08\xf9;Tw\x061t\xc3^\x00\xa86\xa5\xe2\xbd},u\x83\x90\xf0\x9c\x84\xce=C\xd8\xe5d\xcfv\'\xe0\xcb\xc5\x1f\x0f\xfd\x86uJv\xcbr\xdc\x8b\x87\xc3\xf5\x9ck\xee\xb1\xb9*\xbbe]\xeaR\x9b\x98\x84\xe45\xd1\\\x93\xd4^\xfcg&\xa9\xbd\xc8=\x02/\xbe/y\x040\x1d\xfe\xc2r\xd6<:\x0fe4!l\x89%\xc3[\x91\x8d\xf7\x85FV\xbfz/+dX\xa9\x89\xbb{\xc14^T\xc7M^\xb5\x94\x15B\xdb\xd5\x95r\xcd\x04\xa5\x86\xc9FE\xa5\x9c\x07\xb9\r\xc5\xda\xa5\xe2\x11A\xd4\x91\x8d\xa5/\x1d\xc5\x9bi^\xacf6\xcd\x06\x8d\xf5Bt\xf1\x03J\xa1\xe5\xc5\x1f\xc9,\xb7\xc7\xf4]\x964m#\xbd\xc1$\x01\x145m\xf1\x18\x188v(J |ht\x0f\x97\x87\xfe2t\xac=\xafZ&\xe4\x8d\xd2\xf6\t\xe2\x86r_\xbeUC[U\xc2\xe7~v=\xba\xfe\xf1\x95p\xc1\x86\xda\xa8\xca\x0e\xa0\xbb\x8e\x91\xb9v\x9c\xc3u\x81a6\x8eP\xc8\x15;\x87\xe1B\xa5O\xda\x80\xfbBWX\xe5\n\x1b\xe6\x89P/\xeb\x11\x08&A\x84Z\x1d\x9e\x01u5\xcd:\xd5\xba\xb9F\x9ej\xeb\x92\x192F\x99\xf0P\xc1]\x13\xbe\xcb8r\xc2\xab\x05\x17\xc7\x17{.\xf0g\xbe;\x14C\x7f\xa8\xbdz\xdc\xe7}\x16C\x81\x92yX0P\xd2^\r\x07\xcaa\xcc\x7f|U\x08\x07z\xf5\x19\xe1@\t\x92\xcaBh\x10\x14\xb3\xee\xa3\xad6\xd1\xca/\n\x16J\x9e1D\x0b\xa5\xad\xf7\xa5\xd6\xa5\x88!~\x08d\xa6\xdbl\xf8\x85\xf8!\xd4\xad\xd5\xb9\xaa\x99[\x99d\xc8\xe3\xa5\x8b,\x83\x97\xbbn\xfah\xbb\xc5\x1aa\xcd\xf9x\xa3\xc3\x06\xce\xad\xbbR\xc4Q\xf2\x84\x87\xe2\x86\xca\xa2\x8eF\x16\xdc\\\x88=\xc2r\xd8\xbd\x0bb\xdez\xdb.\xe8\xca\xc58\xa4d\xa6\xdbr\xeb\xb1\xfa\x18\x95@$%5\xe5zPR2\xd31~\xf7\xf3qI\xd9$gc\x94\x10`\xa6\x93\xb6\xde\xca.\x0cQK\xc9\x1aKqKP\xd5\xab\xccy\x86\xed\xdeR\xbf\xde:\xb5\xf8\x91\xf9O\x8a\xfa\x95\xe0\x81%Ha\t\x86\x98@\x17c\xb8c\t"Y\x82U\x96\xa0\x98%\xf8f\x02\xf9\x8ca\xa2%hi\t\x8e\x1aGXK\xb0\xd7RT6\x81\xd7\xc6\x90\xdc\x12\xb4\xe7\x04\x07:A\x88N\xb0\xa3!\xea9\xe0\x19\xb8@\xa5\x80q\xe7b\x98B7\x17\x8a\x14p\xc1#\x0c\x9a\xb2\xb0\x9c>\xa2\xe4E\x88p\xff\x08m$\xbap\x8f\xe7\xba\x16G3\x81\xb1\xaa\x0e\xb8l\x1b\xb3\xb0\x08\\\xa1\x031\x82\x9e\xa7\x8f\xdd<\xaf\xefC\x8a\x98\r\x9a(\xbc\xb7!\xa4\x87\rA\x0f\x85\xb0a\xac\x9c\x05\xb1\xc0c\xf8>MP\x10\xe1cQ5S@tdp\x8fc4\x14@\xfe\x1ci\x87\x10\x17L\n?Pc\xb8d\x83r\n\xf8\x92\xe37,8}\x08N\xef.\x18\xeba;F\x8f~|E\xe0\xc1\xee\x83\xeb_\x07y\n2C[\xee\xcan\xb9#\xfbl\xc5\xadg\xf3\xf5\xf1n\xba\xc2\x04\xa3\xeb\x14\x1b\x8d$\x8aB\'Y\x13\x05\x18\xe3\x05\x1d\x17@\xdd\x16nN\xb1\xdc\xceu\xacb\xba\xbd\xc91\xdd\xbe\xcb1\xdd\x96\x12\xca.\xc2t\xbb(\xc5,\x87\x7f\x1b\xa2e\x12\xb1\xdb\xc6j5\xb0\xa9\xe7uT?#\xf7\x0c\xd4\xe1\xc7Gg\xfb\x15>\nd\xe3\\\xa7\x18mB|:\xfb\xfd\x8a\xa8l\xfe\xae\x1b\xc8\x82*z9.Cb\xbb<%\x8e\xd7\xaeB\xb86]\x8c\xf6\x96\x18l[\xff|~\xa1\x87\xa2\xd7\x04\x81\xd0X\x90\x1f$\xca\xa1\xefd\xcf\x80\xd5*yrI\xf9\xf4\xf1\xd1\x90\xdb\x1e\x1d\x81\x8d\x97s\xdb@(\xc1\xfa\x82\xcc\xbd\xe6\\\xed\x1a\x88+\xe8y\xe4\xe2gf\xeeai\x9b/\x03T\xc3\x9c>\xfc\xee\x9f\x9f\xeb\x17\xf0\xd409\xed\xcd|\xfdvzW\x07S{\x16w\xd4u.\x1f\xdb\xd9USi\xfd[\xf9\xe2D\xc0\xed\xe80\x9d\xb6m;~y: ,s\xba\xc2\x94\xc0\xe9\xfa>\x9c\x01\xf3{!lGX5\x97(\xd8Nw7\xbb\xf77\x7f\xb8\xdf\xf7\xc7\xf7\x93\xcf\x1d\xb4\xf7\xef\xcf\xa0\xad\xc1\x91\xb0~"_\x0c\xf3`u2\x8b\x0f\xe0\xc38\xa0\x18\xe4\xd8\xa1\x97k\xc7\xdcG\x9f\x99\xe2\xf7\x888lB]x\x96\x9c\xad\xa7\xd7\xb2\xba\xc3\xb7\x97E\x98\xfe|w\x9c.Ky\xfcw\xbe\x9a\x7fm\xf6c\xce8/\xca{\xfc,\x84\xb6\xf3)\x92o\xee\x1eNd)I\x12~\x9f.\x11\xf2\x12\xa0\xb5j\xefG\x86\xaaOD\xcfD(\x15\xe2j"\xc8&"n*\xfdr\xc98\x95\x9a\x89\xb2H\xb5\xd1\x0f\xa2\x13w\xa4&od\x17\xa3K\x03\xe530\x95\tr\xb9\xf1\x94e2\xbb\t\xb9|nL\x1bf7\x9eb\x88.\xca\x86\x08U7f\x1b\x9eh\xf4T\x97g\x91&\xf9\xa5i\xe6i\xccH\xf5\x16HM0$c\xf8\x7f\xc7\xfe\xefet\xaa\xe0hz\x1ci@J\xf9\x19\x89rOA\x81\xd1\x80\xf9Mq\x9d\xe0\xa2\x1c\xc2(\xee\x1a*W0\x03\x85\xf3\xc2\xdc\x04\xedf<r\x8c\xea\xc25x3\x03R\xd6S\x96]C5\xd3\xb4\xfcK\x98\x96;z\x8dE7\xabA\xaa\x8f\x0e3C\xbe\xb6\x0e\xa9&\xf8\xef\x80\xc0-GW(x{\x10\x04y\xeb\x8c\xc1\xd1M\x0c\xcd\xa6=\x07A\xde\xb4\xcb\x0cy\xbdu\x8c\xe9\xce\xa8&\x02\xc60\xca\xc4\xddi4z\xf4TD\x9a\t1\xe6\x8a\xf9&\rwq\x02\x808\xad\x12\xc2\x80U\xc4\xf6h\xc3]\xb0a\xc6\x80\xe4\xdd\x04\x1f1\x80\xf8\x10\xd1\xc3\xe0]D\xfb\x199\x12\xd0\xc8u\xc1x\xa5a\xb3\xb6\xf8\x16\x19\xd0^X\x9f\x851\x86\x88\xb27p\x8c\xbd\x81\xff8\x86\x10\xca\x8d\x87\xc7rX\xd9\xc3\x8e!\x19\x84x2j],*%{-D\x91%\xb3\x84\xf0\xb1\xa4\xb5\x1a7\xf6\xd7<n,G\xf9~\xfd\xedO\xe7Jl2\xe8n\x00{&\x1c\xcfj\xc4\x98\xe75\x88*Bj\x99\x0c\x06K\x9e\x01\xa3\xc0\xd2\xb6\x14\x08\xe4\xba\x1a\xfc\xb5\xb84\x0c\xf5\xbaI\x87\x96A^\xdcEiY\x0c\xf1\xae\x1e\xcb\xd52\x18\xe6\xe6L\xc8\x16f\x1d)\x8e\x8b,c\xb2>\xe5\x0f\x8a!Y\xc9\xd2\x0f\x85\x1d\xc0\x82\xb0\xbc\x19y\xef\x03\xaf\\\xe8\xb3\xe1\x95\xa0c\xe0\x95r\xb8uz\xce\xc2\xacnf>,\x06X%S\xdd\x1e\xf9G17\xf5\xc8*\xc3\x90&Ph\x92AS\xc9\xc8\x18-\x95\xb4eaRK\x83\xca\x08(\xd8\x16\xc3P\xdc\x16\x18\xef\x94\xcc\x16\xc2\x7f\xf5\xd6g\x06\x99R\xc1\xc5\xae\xe9\xf1\xda\xe3\xc9+\x16\x8eb\x8cL\xb2\x1a\xc3%\x802x\x84\xf7H\xb1\x90\xa6@5x_\x1f\x8fwK,\x83\x8f\x89\xac\xcbj\x1e!d\x89-\xc0(\nY\x9eB\xaa\x8f\x8c\'P\x1d\xde\xd7DV\xc9\x9ea\xb9\xe8\xc2o\x87\xcd\xcd\x03\x93\xdaC\xb5\x05v\xa1zz\x16:\xd5\x0f\xcfl*:?\xb3\x0b\xb5#\xf4\xdb\xff+;BY\x93\tM\xdf\xd7\x8dj\xdc\xbd7\xb0\x14\xb7\xa9\x9e\x8a\xc1r\xfe\xf1 \xb5\xc5\x10\x8fo\xef\xb7\xef\x0e\x1b\xfe\x1cX!@>\x1e\x9c\xa4\x7fK\xfbJ\xe3\x12w\xe1\xf1\x95f&\xa1|x\x91\xe1\xe9\xab&\x93\xe9\x86ga\xece\xfd`\x96\xa3\xc7\x83?\xfe\x94\xa1\x86\xb6\x1c\xd5\xbe\x9e\x98\xb1g\xae<\xcd \xe7e<\xc8\xb7\x9f\xee\xd3\x97\x01\'m\xf6d\x18\x13\x92\xf6e\xa7\xad\x0fv\xddu\x90\xd8Y0;\xb8\xe7\xde\xec\\\x1a\xe7N\xefe\x02g6\x1b"\xb4;\x03A\xbfM\xe7\xbc-\xef\xdb\x0cp\xbd\x93\x0e;81\tW\\\x1c\xc0\xd9xp\x06?K\xe7\xce\x80\xcc\x17\'i\xbbj\xbc\xc8\xe2\xde\x82\x039[Q\xf0\xcc\xd9\xc6\x85\x9d\xce\x8d\xc3\xfb4\xbb\xa6\x10\r\xc2z\xb4\xfdI\x99~\xc4\xac\xce\x92\xfa\x10U\x8bT\xed\xe0*I\xaa\xae\xa4\xaaL\xaa\xe6x\x01Ya\xe0\x86\x818Pt}\x18\x1f\x80\xa1\xdap\x06k?{\xcbNd\x13\xceuPl\x0c;\xe5Q\xa9\x14|$\xe51\x92\xff\xc0\xb9>\xe2\x0c0J\xcbfGe\xd4B\xdc+*\xa3\x16T\x03\\\x19>C\xa4\x94\xa7z|\xf6\x96]\xebQ5\xa5w\x06J\xac\x16\x8a\xdb"Wy~w8\xde\x15\xa2\x03Y{\xc2S^\x1fO_\xd71\x94\xbcC\x9d\x9f\xc8I\x88\x9d\xc8\xf6\x1a7y\xfe*\xafy\x9f\x07\x8b<\x7fU\xe4&m\xe6\x90\xb9,Xd\xc8\x83B*eo\x16\x83E\xe43\xc2k\xccZ\xefK\xad\x8c\xd3\xf4\x99k\x9e?\x85\x0c\x16\xf1\x18j\xe3\xdd\xc7\xe3\xbb\xbb\x85 C\x16%\x82\x12\xfc =\x01K>\x80&\xb3\xd3OY\x0cI=\xf8\xb0\xc9#F*UN*\x91$\xf2e\x01\'\xcaZ\x8b\x1b\x8dE\'\x0e\xd1$\xbe\x1d\x16CI0\xe1\xc2\x0b\x18;\xed\xde\xd5fh\x86b(\x89\x9c\xef\x16[?\xa6\xadlmed\x81]\x164\xc2\xad\xb1\xe5\x08F97\xb0&h-\xb1\xa6\xf6\xb3\xe6x\xb8\xbblKV\xe2G\xe4\xe2"d\xdc\xe0"B&\x17&\xda\xeb~\\\x8e"\x81\xde\x10s\xd2\xf6\x83\xfeWD1\xf2\x18\xfe\x1eM<,\xf20D\xf4wx\x8d\xa8>\x9aT\xd6\xde\xf6\xc2(\xcd\xb2\x04:4\xb7\xc0}d\x8a\xd1HYv_\x9b\xac\x85\x1bf\xd6\x16\xd2\t:$\x9a\x18\xd0\x8e\xe6Z 43\x15\xad53\xf1\xae\r\xc4OX$|\x0c\x81\x89\xf7 \xeb\\k\x08\x9cl\xb0\x9b\x89\x84\x85\x08\x1bZ\x1b\x8b\xc9la\x80&\xbe\t\xcb\xe38mx\x9b,tS\x04u\xe6\xe1\x9e}\x1e"*\x82G\x93\xb0\xd2$\xe0T\x86\xa2\xc6\xc0\xca$\x8bB\xe6W\x88\xcc\x8b\xb6\x94\xad\xc1\xf28D\x86G\x9a\xfcA\x1fr9\xfa\xc2;\x0f\xec\x89\xcd\xee\xfe\xf4\xfd\xef7\x87\xbb<\xfb\xb2\xdc\xa5\xaa\xfc\x9d\xba\xdc}\\=\xbb\x99\xee\x8f\x92_\'\xfd\x9e\xdd\x9cF\xfcVN\xca\n\xec\xf9\xcb\x87\xf4r\x08\xc0\xa8/\xbb\xc6\xd7\xbf\xcf\xc3\xfe\x9f}\x9f\x87^@\xb7b\x82&\x06\xb1\xed\xc0\xe8\x96EO\x18V=Go\xab\x85\xd8\xb7\x1b`\xef+f|\x03\xce\xbec\x97\x13._\x0cwx~\xb5\x9f\xaf7\xf3\xe1\xfa= \x8d\xde\x1fe\xba\xa6\xf8\x12\x8c\xf7?\xbb\xb9?\xba\xa2X\xab\x97\xd3\xdd\xfepU\xb5\xd9\x15\x9e\xb8\xf2\x94\xf0}\x17>\x0b\x13\x05\x14\x0b\x04\xb0<f\x00\xe4\x02\x9e\xb1\xd9\xc9P\x83\xa4\xa8\x9b\xc9ZJ\x96\xc0\x1d/\xa3fXl\xc0\xcc3\x17l\x15\xe4\x0e\xbd\xf5\x90\xa4\t\x9fM34"\xc3\x91|Y\x116#\xd5\xd9\xe7W\xf3\x1em\x87\xc7\xed\xfd\xcd\xf5\xf1v\xba;\n\x89\xe2/\xd3\xf5q:\xac^L\'\x0e\xf1V\xc8\x15\xf8\xd1\xfe\xf4\xe4\xf4Y\x0f\x0b\x85\xcc\xb6Q\xbc@\xd5\xd7\xb7\xcc\xfbx\x95a\x11\xc5Bf\xce\xa5\x8e\xa2F\xebtHP\x89\xc1\x13\xbf\x19{)v\x9c\xbe\xef\xcd\xdd\xfb\x07\xb6\xb7@\xf0\xf8\xe5\xe3\xe6\xee\xd3\x0c\xea\xf1\xbc\xfa\xc1\xa3\xf0\xb6\x9b\xeda\xf5\x1a\xf0xo\x1c\x1c\xef\xfc\x9b\x10L\xf0\xe1n`G~(\xe9\xce\xac2\x96\xd5k21\xeaF\x8d\xfe2\x03\xe2C\x03$fhJ\x91\x05~7\xd7\xf7n\xed\x99\xf7\xfa\xbb\xc3\xd5\xee\xe0\x97\xbc?\xadxzw\xb8*\xaa\xd8\xa5\xb58{\x88m\xba\xb3\xcb\x88R\rVWDU{\x95\xd9\xc7\xb9&\x0e\x02\xd0$D\x9c\xa7\xde\x0b\xbc[==\xb8\xffl\xcd\xf5\xea\'_honr\xdc\xa5\xd1Yi@\xde\x19\xf6N\xf0\xec\xed\xdc\x95\xa2f;\xdf\xcf\x85at\xbb\xbew\xe9\xa1\x9b\xe1Q\xd3C+I\x9c!\x1d\x02TY\xc5\xb3\xf3LL\x0f\x05\x15\x95\'\xeb)Ki\x16`\x16\xb5\tf\x01\xe9\xe4\x94\x1e\xda&\xb9\x15\xaa\xa5\x89\xa02n\x1b\xb2\x15\xfb\x84\xea<\xd5\xa2\xb4\xd1&\x94\xf1F\x01\xcd\xae\x91`\xd3\xa1w\xb5\x8b\x82\x89\x8e\xd2\xcc\xc8(\xc0\xf3%\x04[\x8f\xa5K\xe1\xa0\x1e\x13\x98\xfch~YT\xf1\x01\xbd\xb5\xa0\xae\xfb7\x81\xa1\xa6\xe0\xf5D|\x1e\x8b\x0b\xc4{\xfa\xb8Z\x83\x8f\xd5\xc5\xa1\xa9l\x98\x97\n\xf1\xf1\x15\xfa\xa8\xfdJ\x1b\xd4\xf5A\xd2i\xd0~@T\xcb\xdf(\xbe\xd0\x06\xc5\xba\x86})\xfanL2${\xc5\x1a\x1e\x0bm\x19 \\\x92\xb5\x04D\xc0p\xcd\xa2%\x85$Bf\x1fY\xb7\x18D\xdb\'\x94e\x1f\x89\x92(\xa5\x04\xcc\xa5\xe3Tr\x96Ru*q\xa7\xd28\x97\xd4S)>\x95\xf0e\x06o\x9a\xddk\x16\xb2\x82\xd3\x8c\xe14\x9b8\xcd4N\xb3\x90\xd3\xccf\x95\xe42\xa7\x99\xcdi\xd63\xbfO\xa4K\xe7\xc9\xd4I\xaa\xb5H\xc4\xce\xd2\xb4\xdb\x12y\xc6\x98\xf4r\xba\xca\xdd\x13\xd4X\x95NE\x87\xba\x19)\x19\x9e\xc4\xce\xa4\xb1&h\xbe\xcc\x91@^\xe6H /\x7f,\x19\x90\x10\r\'A\xff\xa8F\xe0\xa2`\xa8\x8bn\x87d\xa9\xf0\x1e\xd2\xa6{l\x02\xb4\xc1R\xceP\x9b\xa3\x12\x08y/\x1d1Jx[\xe6\x9c\xc5\x84\xa1\x92`63\xb3\xcd\xb6\xea5\x80PK\x14\xaat\xd1S\x90\xac\x03$\xa9\xb4)\xffzQl\xda\xcdd\x899_\xab`$\xab\x8c\x10\x86\x92\xa1o\x0bM\xf9\x02\x98d\xb3\x95\xf1n\x99\x0f \xb9\xf3\x98\xbf{&\x9ad\x83\x15m\xfd\x8b\x1f\x17d\x8bd\x02\x92"N\x1a\xcb\x16\xec%\xb9\xe40zk\x15ZJ\xfeuqck\xc8\xca\xa0\xc8\xa05d\xa0\x90\xdd\x7f\xad\xc84\xdd\x13\xa9\x19\x87\xc2{C^f\x87\xecZ\x11\xa9<\xd9R\xb2}\xe7I\xcbs\xef;\xba\xd7\xe2\xd5\x8e\'\xdbw\x11\x0e\xc0z2\x08*\x03\x93\'\xa4\xdc"\xa4\x1a!\xf3d\x12\x91\x90\x97\x844\xd5/\xc0id`\x1b\t\x14\x87\x00\xea\x100\x1e-v\x0e\xe4\xc0G\xee\x90\x93\xb4\xfc=\x87\x89\xc0\xffB\x9c\x1a\x12\x80\xe2\x13\xb5\xf8A\r\x9f\x88\x91\\\x1e@\xc8\x83p\xd5ro\n\xa1\x18\xb0\xce\\\xb2@|\x04\x13?Y\xb2\x91h\xa2\x00\xb5`S\xb2\xe3\x9d\xe3&\\\x8e@\x9a\xaf\xe7\xb7\xd3U\x1e\x83\x14\xdb\xebpVY\x9f\x85\x18$1O\x88B\x12\xed\xd58\xa4\x17y\x1cR\x8e\x8a\xf0\xfa\xf9\x85qH]DK\xa0\xab\xf58\xa4\x9e\xe5y\xf4\x0c9\x01\xf9\x9b\x8cI\x12O\x84\xbe"\xd9z_j\xcd\x82\x92X\x95\xe3l\x9d\x18\x94\x04N\r\xcc\x80O\xa6\x90\xf1I\x1c3\xb9\x8d\xd5\xe5\x13F\xd8Wc\x95\x92\x12\xef\xdbz\x94R\xcf \x14z\x06\xad\xa0\xb3X%\xf7\xf0`q\xb8\x92\xd1J\xe2\xbd\x1c\x8a{\x85\x99\x166\x00\x0f\xe5\xbd\x15U\x13\x82r+\xdc(\xe7[\x99\xb6M\x16\xb7$F\xbf-\xb7\x16W"\xe3\x96\x92\xaa8-+\x93\x8d|O\xc60\x89\xe10\x8aI\xb4\xca8\xa6\x8b\')\xa3:\x11\x9a\xc2\xc2\x0e\xc3\xf8\xa6\xea\x0eKA\x9d\x00e\xa1\xd5\x93\x87lR\xb6\xe0\xab\xc0~\x0ek\x01\xfa\xf5\xda<j\xc6+\x80\x00\x9a\xa0/\x128\xb0\xfb?\x01\xc0\xba\xb3\x1d\xf5P\x03X:6j\x9e\r]\xe8X\x18-\xaa\\}\xbcB\xc1\xbcH\xf4q\x00\xe4?\x96\xc7b[\xce!\xe8\x8c\xf7\xf7$gx\x87D\x0b\xc4\x10YP\x8fL\x04\x89!r\x94\x11U\x97!`\xe3\x93\xcf\x1d\r\x06&@\xf8\xf7\x91\xc3\xa1\xc6cx\xb1\x9a\xd0\x93\xfc\xffM,VClE\x19\x1e\xf4\x8c\xdcK\x87\xd4V\x9dP]\xd4\x9a \xed\x95\xf4\xaf\x00\x03\xd4\xc52B\xa4)\xe2\x0c\xa8o*\xfc@\xd41\x86\xfc\x82\xc02\x84R\xdf:f\xbbj\x96%\n\x16\x97><g\x1f\xb2D\x01\xc8\xa2\x0b%\x89\xda\x80q\x01/\xae\xa7\xb2Amp\xa2\x8c\xe0\x13\xc2R\rCH\xea\xc5L\xe4!\x94}@Y\xaf\x87\xeci\xfc\x9c1\xdd\xb9e\xd9\xce-\xe4D\x8f\xf0\xff&d\xa7Z\xf0\xe4\xc4]\xc6wuw\x0e\xa0\xf8\xa7\xc3\xdb"\xf2\x05kO\xb8\xfaw\xf3nv\t\x87I\xc9\xed\xac\xefB\xc9m1_(\xb9-\xdak\xdc\xfd\xa7\x1c\x01\xe3\xa7\x17y\xc9\xed\x17O\xaa%\xb71xa\xfb\x19\x08\x18\xbc\xe46\x048\x8c_\x82\x80!\x9f\x11\x0bo\xcb\xd6\xfbR\xab,\xbc\x8d\x9e\xe2\x0c{ \x0bj\xc8\x86\x97%\xb7\xc7\xc8\xdf\xb1>\xc2\xb9\x18\x06^\x13\x01\xb8\xfcY\x04\x0c^l\xbb\x19XV`sa\xdc\xc2\xe91j\x08\x18\xf2\t\x0f\xc5\r%Kn\xbb\xb0\x94P\r\xa1_F\xc3\xf0\xe5\xb7\x01\r\x03\x83\xecjh\x18r\xd6\xdbr\xeb\xb1\xfaHY\xfd\xed\x81\xb1\xeb\xf3h\x18r&\xac\xbf}\xf8\xad\x86\x86\xc1*$\xe4\x93\x9c\xad\xbf\x9d\xa0al/D\xc3\x90k\x0c\xb2\x01VR\x98[\xd2\xbd+q\x0c\xa3\xf3$\xb4\xc6}\xc7\xd0\xef_%\x1bx\x05\x16\xb5j\x05\xa7+\x1a\xb6\xfb \x1b(\x88\x1a0\xc1\xdc<R\x1f\x15\x19T\x83\xa2A\x0b\xc4\x18\xcd\xb6\n\r\x90*\x9a\xab\xa9\xc2\x1evC\xab\xb6\x8e\x8c\x91\xd4]H\x96B\xfdU\x17\x04\x8dD\x04\xc9\x84\x93 \xb6$\x02M\x94t\xb8\x04\x94JFBb:\xcbg\x9e\xce\xd7\x87<\x7f%\xb4V5G\xd9\xa3\xceY\xd2\x19\x88\xaf\xa4\xad5\xae\xf2\xf4\xa7\x8c\xab<\xcd\x03\xaf\x9f\x96u\xc6A\xf2\x96b\xa6\n\xc7L\xe2Uu\xb2L\x95t\xc5\xf0FD\xdb=\xb6\x81\xc5\xb3\x94\xa9\xc2\x17EB\xbc`\x0cOQX?\\W\r\x9f\x18G\xcd\xb4A\xe4\nE\xef\xb4\xcaxC)C\x85#\x1f\xf1\xba8Y\x86J\xfa\xccp\xd4\x8b\xb6\xc2\x17f\xd6\xd01\x1e\xf6\x85\xac\x14\x845j\xb6\xa38\xc2\xd3\xf1nKma\xde\xa2{7W\xb9\x86b>J:(\x9c\xd4i\x1b;\xa7\xb3A\t:\xaf\x14c6d@\x8a]\xeen\x95\x1f?\xa2\xd1;\xa6\xd0n\x06Ur\xa9:\xa1\xc1U#\xfd\xcc\xe3\xf6\xe5\xc3\xd5\xf1\xb0t\xe6\xa2\xcc\xbe\xa6\xaabP+\xa5\r\x15C[Oi\xa4\xac\xa7\x98D\xef\x9c](\xab\x8f\xc9}\xc6\x8f\xd9\xb11\xa9\x8ez\x9c\x0f*\x96S\xef\x8eW\xe2\\wA\xb9i\x12\nV\xc0k\x9fG\xaa\xf3=;\xa4\xdcz\xa8\x1a\x1b\x14M\xef\x03\xfaP\xebk\x88P\xe5v\xe5\xa9&\xc2\x14\xad\xfbP\xb3\xb3a\xd7\xa0>|\x1fp\x8fl\x9cA\xfb\xa0\xe8u\xcfqz\xd6Tp\xa6\xc1\x95\x11\x053\x98\x88\xdb\x93Q\xbcP\xcd\x10\xae\r\xb1\x90\x10^\x1bC\x85\x1b($42\x8a\xa5.[^wFS)\x99&fJ\xc7"5Y\xcd\x9aRE\x9bX\xef\xa6T\r\'\xd6\xcaI*\xe9\x88:;\xa2\n\x8f\xa8\xd1#*\xf8(*\x154\xb2\xcfB\x95\x83\xd4@\x95\x83L\xd4rCm e\xa9r\x10m\xbf\x16\xc8&*\xa9a^\xdc\xd4*V\xea\xeb\xf9\xc8\x8a\x16\xd9G\xcd\xd5\x9b\xb1\xa3V\xbb&G\x01\re\x836\xdcBa\x9d\x96\xedr2\x18\xa8P\xce\x89\xd5\xed\r\x85u\xc0\x8c\x1e\x8a\x01\x11\x89\x12\x05\xfe(\xc8q\x0c\xe6\x80\xf5\x18\x165\xc6:;\x81j\xa2\xeaNe\xab\xf0Z\x1f\xa8>\xeeY|\x96\x8e\x9b\x1f\xfc\xefuY\xf6xr\xfd\xf6\xe6*Wqcs"}\xfce\xbe\xb9\xbd\x9a\xfft\x9f\xa8\xb8Y\xdf\xba\x1c"f#AD4\xd7$\x91\'?g\x92\x08k"I\xe4\xc9\x8b\x9fK\x92Hf\xb1\xc6\xc0\x1b[\x0f\xd7o9\xf6Q\xd4r\xc71\xb6\x83\r\x1b\xc2\xf9\x08\x0bR\xc8,\xe2\xe1\xe0E\xca\xc6\xfbB#\x93X\xb44cW\x16\x0f\xd2\x8b\x1c\\\x06\xe5\xed\x98V;\xb1\x9a\t\x9bs\x91\xf9\xdcv\xdd2\x87n1\x1a\xdf\xbf\xbdFG\xdb5\x04\xd7\x11\xde\xa3\x90h\xc4\x9a\xb1V\xbdh,m\x16\x16x\xafc5\x9e\x8d\x85\xda\x07\x85\x90{\x17\xf24m\x9dV\xee\x90\x1c\x85\x80#\x06\xbf-6\x96\x96\xc1d\x1c\x06K\x82j\xe3\xcc\x9c\xbe2RM\x8cs,}A&\xeb\\:\xb8\x8c\xaa\xbf`\xf3\x80\xf4#\xa6\x0e\n\xe8\xc6\x15\xbdl\xadS@]\xdd\x81\x82\xea){<\xa2\xd2\t|\\\xc7ji\r\x90C\x84*Xk\xc3\xc1\n\xd6\x9aP\x11\xbd\xf3wMe\xd3\xb57\x1b:\xb2gE\xdbt\x02\xe2\xe8\xc86"p\xacC\x99l\xe0Z:\xe2x`g\x1b\xf1\x1b\x1di\x18\x0f\x0fC\x01\xf3\xd4]\xe0\xf0\nH\x12\x1c\xf0j\x1f\x91\x1c\x1dI\xe5\xc5{ \xa9\x129\x8eL\xc2\x8a\x86\'"\xc6\x8b\x0f8p2V*/\x90jHd\x1b\x15\xe1Z4\xaf\r\x88\x1co\x08\x80\x0e\xc9UKL{,\xd7\x06\x14\x95\x03q\xa8\x96}\x14E\x18\x10\x1d\x8c<\x86\x12\x85\xfe\xc56\x01\xa6\xa2\x07\xb2M\xc9\x91\xef\r\xb4\x12`-D\xad\x02\xf8E\xd8H\xcb*\xf9\xcd\xf1\xfe\x0f\x8e\xf5\x14\xb4rv\xa1\xae\x98\xe7\x9d\x16ts9UP\xcf\xe5\x85\xaa\x86\xfe,\xd7\xd0\x7f\xc95\xf4_\x9e\xd45t\x96\x08\x9a\x009\xf4\xdc\x9eU\xd5\xdc1)zX\nm\';\xb1\xd4\xe8\xe5C\xa2R\x9f5\xdfS\xf3C\x05\x96\xefK\x1e\x04\xb5\xfdl\xb2\xb2\xae\xbfg\x10z\x19V\xc5\xc4\xeb\x0e\xa9\xaa\r\x80\xa7O\'\xe8\x15\x9c\xc3\xda\xbam`\x93\xe5\xb5\r2&\x9d\xec\xc9\xd2f \x9f\x12\xcd\x06Ysy\xf31\xe3\x01\xc4\x9e\xdb\x18o\xbe\x9d\n&\x04\x07\xcb\xbb\x9d\x1d\xfe\xd6\xb6\xb3\rX\x8a\xa59ANr[i>\x16?~fW\x98Y\xb2\x99b \xfd\x9ciJ\xfe\x9bMv\x0c{\xe2w\xde\x9c\xd9\x1b>o\xb2\x87\xe2/\xae\x1f/\xdc\xa8h\x99\x90k\x8d\xc9n\xedi\xcb\x0f\xc6\x8d\xed\x02\xbeO\x7f\xe7\x8d\xc9ytw\xfa \xa7\xbf\xbd\x87\xfej&\xf3\x98\x89\xd9P`Wc\x98\x11\x16\x01\xd5:\xd4\x045\xec\xaa\x1e\xe1X\xd6\x01\xadH\xf0\xf7\xe4\xd0\x16G\xba8\xf03v\x900\x0b\xc1J\x04\xa3\xc9\xd8P\xc2\xa4D\xf1[Q\x1aW\x14\xce\x15euE\xd1\xdd\xac$oR\xb07-\xe7+\x8a\xfdFrd\x85\x8c\x19\xd91\x1d\xdd\x04\x95\xddD\x98\xa90\xaf\t\x9at\xcf\xc9!`R\xf5\xec\xe5\x18T\xf7\xc7\xf4\xde1\\\xc5\xd7N\x13\xb5\xec\xb5cEe\x1d\x11\xad\xc2\xdeX.\t{\xf8\xb0\x996\x7f\xe4\xe9i\xfcB\xbd,l\xdei\xa10\xac\x9c*\x94\x86\x95\x17\xaa\xc5asw\xeb\xaf9\xdb\xfd\xf5\x97\xe7\xd5\xe2\xb0h\x12g\xc51\xd1i\xc4\x8e\x88\xf3\xe8$y\xa1\xd89C-I\n\x06f\xb5\x00\xc5\x03c\xd9\xd8\xac\xf9>6?\xcc\xd5\xd2\xb1y\xc5\xcf\xc2Ceed\xb3\xc9D!\xd9\x1c\xc2\xa4PZ\x96CG\xb5\xf5\x02\xb3cVf6c\xc4\xe7K\xce\xee+%gg\xc6\xacY\xba\x18{b,?+\x9f\x18\x0b\xd0f\xcd\xe5M)\x91O\xb0\x08m\x13QP\x98\xf3$\xa2\xa0\xf8\xc8\xe6\xadO\xfc\x9a\x1b_\xe0/C\x9e\xca\xa6\xba\xad4\x87\x85A\xf4\x19\xdf\x13YY\xda!+K\xdb\xb0\xff\xebb!\xbfl\xc6#\xdb*\xbf\xcf\xd5*\xb5h\xfd\xbft2Y\xb1vy\xe7\xee\xb4\xac[+WI<\xf9\xd4\xd59kwN\x0e\x02(\x94\xb1/\xa6b\xed]\xb9\r\xec75\x8fY\xa3\xaf\x84-\xc9p\'9&e\x8aW)\xb1,S\x9c\xcb\x14\x033\xc5\xc7\xe4\xd8\x99\x12W\xd3$T\x8a\xc7\x99bu\xa68\x9e)\xc6\'G\xcdO\x11\xf5\x13\xb0}\x0e\xc3\x9f\x00\xf4\xa7\xd0\xfd\x02\xd4?\x81\xfbg\x85\x00\x92\x12\x01\xa2x@RV\x80\x15\x1cHJ\x11\xb4,?\x1cB\x90\x87p\xc5\x03T\x0e1\xc2k\x0c%\x0f\xe2\xf3`\\X\x13.u!\x19_Y^\xda\x0e\xe3\xaac\xc71\xa6\x13\xc1\\!d\xcc\xf2\xfc\x1d\x0c.\xd3a\xcc\x8e\x05\xb3\xd9$\xd0\xcdb\x92W\x13\x8d\xf5\xc1\x84n\xf9\xaeR\x90\x90\xd5\x85\x82\x10\xc6\x83\xee\xd0Z\xfa\x88n\x8a\x95#\x9a\xf0NZO\xf5\x8cR\xa1\xa7\xf2\x94\x89\xcf\xa0u\x18\xd3Q\xb4N/d\x06\xb11\xfe\x16\x96s\xe5\xdfM\xbb<;\x1e\x1b\xab\xa2\x87\xe8\xb0P}\x80\x0f\x1f\x12\xddyc\xb5\xaa\xc0\xb7\x99\xb8\xf1&\xc7\x90|\x03\xdd\xa4\x96\x0f\n1\xaf\xd2\xb2/\x89\x15\x13\x13\x1f:!&\xbc\xb9\xdf\xf2e"\xc6~\xd2t\x9f5E\xe1`\xe8\x8a\x93\x03\xfb\x7f\x93\x0e-@\xccP\xd9N\x12\xc4K\xa1W\xdd\x9f\x18\x037%\x06>1\x16\xad\x04C~\x86 \xea;\xc1\x91\x93\xe7\xc1$\xed\x03kb\xc1S\xca\xab\xc4\x9e\x13o\x0b\xa6\xe7\xadc$\x1b\xbb\x93F\xe77\xf7\xbfMl@\xe0\xb7\xcf>\xf1\xa6\xb0MJ<V\xf3\xe2\xecVp\xd1\xf4\xd5\x1e#\x98O\xcf\x9aYPty\xac\xc8$\xf9w\x1c\x06\xc1\x0c\xd3\xc9"0\xb5\x93\x84\xda\xfd\xd8\x174\xd2\xde\x874\xed7\xfd\xe3\x82\xad\xb8\xb3%bs\x00\xd5%\x14+k\xaa\xf1\xf8]{K\xb2\x1eC&/P,\xb9\xd44<\xaf\xd74!}\xb5\xf7\x14!\xa5\x9c\xceN\x83\xa7,$\xba\x1aJQ\xf5|\xd6\x844Hw"\x9a\x90\x99\xe3{j\xcai\xf2\x15s4%\xa6\xf8\x19B\x9a\x8a\xaf\xcd\x83A\xb1\x90fk\x08\xe4D!\xd5c\xb2\xae\xf2\xd4\x08\xa1U\xaepO\x88\xde\x02\xa2\x0b\x81M\x06\xed\xcd\x1e\x10\xc6t\x14Nk<\xd1C\xfc\xad#z\x16N\x1d\x89>\xde\xe3\xf9h$`h\x1b\xaf\x84\x88_\xe5\x9f\xa2\x0f!\xbf\xc6\xd2h\x9d\x7fZ\x13\xef\xc1\xd7B\x04\x86\x12\xab\xf82\xbd\x12m\x1a\n\xe8\x1e\xfc\xa7\xb3@\x8c\x9e\xc0Pd\xf8\xe0*v\xc3\xbd\xe03auOW\\\xb7\x10\xb2l<\xa1\xe2h-\x1f\xda\x86Xq\xe3)\x8b\xd4\xe0\xa9.\xfa\xa5\xb5\r\x01\x14\x8dgT\xf8\x1e:\xcf\xa7:x\xdf}\x14\xd9\x92\x0c\xef$\xf7;\xc9\nO\xf2\xc5\x93L\xf2$\xc7<M?\x17\xa9\xe9\xf1\'\xb3\xec/\xbe*\xc7D\xb3\xf6\xb2\xc7\xf8\xdb\xf9\xc3\xcd\xf6n:\x9e\xf8e\xe2<\xbe\xba<>Z\xce\x1d\xdc\xc7W\x97\xc5G?\xc9\xb3\x9f\xbe\xfd5\x87\x10\xfd\xf5\xc9\x99\xec\'\x8c,\xd62\xca\xd3\xa8\x92\x17\xb9\x8b\xbe\xe1/\x8a\x8c\x86\xa7\x9be\xd8\x9b|ht!\xdf\xbd\x9d\xaf\x0eu\xbc\xb7\xda\x03Ph\xaa\xf4$_\x850\xdc\x05\x90\x97\r\xf3\'_X\'\x90\xa7\x03s\x8c\xd1\xb3\xc1\xd2\xe0V\xc6\x0c\xe1\xdd\x17\x04K\xb3G\xca\xdc\xcdW\xbfM\x9f\xa6\xc3\x9dt8_\xd5\xa2\xa5\xc1\xe3<o.\x8e\x96\xde\xb6\xaeD\xc5vpk.$\x18\xc3L\xbfI\xd5\xfb\xd4|(\xc4H\x9f\xbe5<\xc9\xd5a\xc1\x0f\xbd\x918\xa1\x19V\nn\xb0\xa3\x84\x1e\x9d?M\x1f\xd9\xcb\xc8\x9c\xd0K#?|\xe9\x0f\x07\x9d\xd0\xa5}\x17\xb2\x93;\xf7\x81[o\xc5\x10\x91\xce\x85\x88h\xe8\xed2\xa4\xfe\xa5q\xd1\xc6\x82\xb5\x14\xb5\r\x83\xa6d\x15pJ[\x16\x94\x05@\xa5kR\xc4\xa0\x04\x03\'\x07\x16J\x04\xe8\xa4\x91\x1c\x93`0\x8b\xd6a\xe4{gI\xbc\x970Cqd";\x16\x1bF$\xfa\xd1-x\x9d)\xe5\x9a\x86"(T\x88\xa6\x0bf\xe8\x91\xcc\xdf\x0cd4\x84l\x99\x9e\x0c\xeb]\x04q\x8dv\xe7`\xa3\x1f\x90D\x0b~\x17\xcb@\xacC\xf5\n\x9bX\xff\xb13i\x87\xa6\x05\xbf\x82\nCq7\x83\x01\xeb?#[\x08#P\xbc\xb3\t#w@\xf6\xbc\xb3\r\x9d\x1b [\\\x95f>xc(\x8c\xc02\xc3:\x85\x11\x90%\x9d^\x9d&\xdf\xbfffw\x1dM\xf6*%\x1b\x16F`\x14E\x06\x0c)\xd92\xdf\x00\xc9\'\x06\x9c\xfd\xe1\x89\x84\xffFX\xf03\xfb\xbe\xb0\xfe\x0b\xdf\x80\xf0\x1c\x08\xbf\x82\xf0:\x08\x9fD\xe2\xb1\x10\xfe\x8c\xcc\xdb\x91\xf8B\x84\xa7$\xf3\xa3`\xb0\xdf\x10+w\xac\xc9\x8c\x01\x05@\xd6*~A\xc5"\x03\r\x067\xd8\xb0\tq\xf37\xb1\xfcH\x9c\xc8\xc2\xf6\xa6/\x08{#\x00\n\xb7\xf4\x80\xb4s\x06\xbef\x08@\t\x13ag\x13\xea\x9c\x8c@\xd22\xba\x94\xc4\x00E\xc3\xaf\xaas0\xf0/o>M\x1f6\xae\xa4\\\x8e\xb4\x92\\\xaa\xe3\xad\x94\xba-\xa0\xae\xe4\x13\x06\xec\x95\xfcR\x15\x81\xe5\xd7\x1c\x81%\x97\xe6^\xfe\xfck\x15\x81\x05\x0b0eV_*\xa3X\xc5d\x81\x18\x87\xa4\x8aa\xee\x84\xc1r\x88\x12\xb1\xe5\xe6\xde=\xa0\xacUXxn\xc4n\xc9/H\xf0\x16\xfe\x18\xc8m[\x19\xea>\x8d\x12\xd4\xa50\xa1\x88z\xc0Z~\x1c\x9do\x1b+BO\xac\xd2_1\xdf\x1d\x1d,-s\xbcp\x97\x0b\xd6\xe1\xabB\xc1@\xa4C\xe2r\x99\xcb.\x17*\xb7\'\xe1b\xe0\t\xdfK\xc8\x98\xac\x99o\xbcm\tro\xf6b\xde\xa6c~\x16\x06\xc2\xc7\x80\xf3C\xf0\xc34\xf9\xa48\x9fE1\xfb"w\xa7\x0f!\xd1d\xf2/p{U\\!\xe2\xca\xdcx\x91\xa8\x977e\x103\x10\x10\xc8\xed8\xdc\x15\x82\x05\xd4$\xf4Le\xf0\xe3]qE\x19\x12\xcd\xe2\x9cTY\xad\x8aPS\xfb!R\x195\x89Y\x93\xbf\xb9\x90\xa0a\\\x15\xc2nh\x15\xd5\x84n\x95\xdd\x14\x9c.\xbe\x9f\x9d\x9b1\xe9\xf7\x88\xf2\xa0`\xd4\xc8{1\x15\xc1(\x88R\xe8\xd3`\x89.\x14vB\xd2"9\xb2\xd0A\xa3\x92\xa0D\x1a*\x92\x89\x08 \x04\x04\xb6\xaaE\x9e0\xbf\xfe\xe3\xa4\xaa_\xe7\xd8[?\x1c\xae\xdf\xeen>\xb8S\x9f\xf7IX\xc3r\x9f:_\xc8&%\xae\x90]\x00\x9e\xf0\xd5\xa9\xf9\xd3\xe1j\xba\x06\xd3{d\r\xbf\xe4\xac\xe1u\xce\x1a^\xff\xf2k\x1d\xdd=\x0b\xc9\xa1Z!<\xa1\xad\x8e\xf1\xbe\x1c\ngy\x8a[\x11\xd8\xeb\xf9\xbdxb\x04y\xcf\x9a\xb10\x13\xbe\x87\x05\xcd\xbf\xf6D)\x8c\x85\xe0\x12\xf9|\x12\xeb=\xaf\xd9\x94\xd7{\xdd\xd5\xc1\xc1\x9a,\x0e\x8e;\xe19\x12|S\x85\x0ek\xf6\x0c7%Ch\xa5\xb2#\x0cO\xa5\xc90T~\x07\xd3\xff\xa7\xc3\x1eJ\xab^I\'@\xf6"\x0e\xe5\x8d)a\xe0\xa1\x08\x14F\xc9\x81[\x1e\xdc\x03\x9b\n\x18\xfcV\xbb\xdd\x01\x11s\xaeP\x89+\x0e\xb5\xcd@V\xf8\xbe\xe7p\xf0r\x95\x88\x07\x7f\x7f\xff0\x1d1!\xf2PG\x85\xcf\xa2\xd8\xa8V\tO\xd7\x93\xa0+\xd9R\x10\x1d\xfe\xfe\xf7t)\x19>|e6\xca\xdd\xab\x170\xf9\x92\xed\x8cX\xf1\xf2\xfd\x04_}\xeb\xb6\xaa\xf3\xbe;x\x95\x19\n\xd9\x96\n\x9b\x98\xbd\xa6\x08\xbb\xd3\xb6\xddAb\xf6c\xb2\rP\xabu8\xf9\x81\x1a\x18[@\xef:\x94\x02\\\x9b\xa4\xa7\t\x11d\xca\xe7\xef)\x16l\x87\x9a\x8d\x1e1\xef\x8fS&\x84\xe55q>\xa8(\xb8\xa6d0\x0c$\x0f\x1c\x8b\xd6\xb9\x1cG\xfdp\xf7p:\x96\xf20\xea\xd8^\x8f\xa2\xce\xfa,\x04Q\x8byB\x0c\xb5h\xaf\x86P\xff-\x0f\xa1\xceq\xc3\x9f~\xfb\xfd\x99\x10j\xc3*\xbbb\x10\xe7\xbe\x0e\xf0\xc8"\xb4\xc0\x1c\x8c\xd1Z\xbc\x18\xba*\xf2\x07\xf9\\\x185-[\xefK\xad\xe5\x98\xe9\xda\xca3\x9e\x90M\xb2\x14,\xdd\xcaz\xe3\xdb]\x06\xf3=\x9e\t\x96F\x9bp\xcb\xd8\x80\xa9\xe3H\x82J\xa0X5n\x15Ko\xa3\xda\xa0\x8al@>\x18\xc6G\xcb\xd6\xe2\xbeb\xd1\xd1]\xb4\tc.R\x1e\x88\x05\xa8\xdb\x9ba\xde\x02\x0e\xb7\x8c\x8b\x16\xa3\xdf\x96[\x8b+\x91A\xd1X\x10\xaaaX\xd5;V\xb9O\x9e\xe7r\xb8c\xf1\x9b\xcb`\xe8\xe5I\xca\xc7xe\xd7\rC\xf1\xe8\x96+\x08\x91\xcf\x10Y\xe5\xdd\xd2\x0e\xf0\xba \xea[\xa7\xb0B\x8f\xc7.F\x85\x06T4\xe5\xa0\x854\x08\xf1C\xf1\xbc\xcc\xa8\xf4d\xcdN\xdd\xf4DVIO/\xf9\x7f\xa3\x19\x15\x13]\xbc\xf1\'\t\xb7VavZ\xf5\xe2\xd9\xfd\xea\x8f\x13[\xcf=}\xb1\xb9zrg]\xea\x07\xb7\x98\x84\xcem\xd1\\;\xb6_\xfd\x92\x1d\xdb\xaf\xf2\x10\xdcW\xc5\xcc\x17\xbe\xf5\x92\xa8\xfb}5\xdb%9\xa4+\x19.\xc9\xb1-}y\xaf\x1e\x92\xc7\xc27\x97>+\x1c\xda\xa2\'\x0b\xb2\xbdt\xd9X\x8cC\x0c\x1eCk\xd4R\x06\x0b\x1e\xcdE\x84\xa3\xa1\x92\xb5b\xaaN;t\xcbq\xa9<;\x8e3G\x9dx\x03p\x1a\x8b\xc7\x81\xc3X\xf4d\x86\x9b\r\xab\x89\x00rwS\x80\xfb\xedA\xe2\xb6\xbe\x10\xab<\x8b\xc5\x8c\xb7\xc5\xc6\xd22\x98x\xdd\xc8B\x01\xc9\x11,\xfdqbp8\x81\xc5\xe0,`\xe7\xd2\xc1\x1f\xce\xee|\xaa\xcf*\x8e^\xb1\x9e\x98s\xd2\xb9\\\x92v\xef2O\xf6\x9b9?y\xfbn\xee\xa9\xc7\xd8\xb4\xf61O^\xc01hB\x12\x84\xb7\x8b\'\x92q\xa4\xd8\x19\x9d\x9e\x84\xd9)\xd9\xf91;\xd6\xb3\t\xa7\xebi\x86\x86\x8b\xd7\xf1\xd4w\xf1\x8f\xe1\x8a\x0b\xa9\x0c\xee\x17W\x83=$\xc0\xb8H\x8c\xc0\r\\\x84M\xf0K0\x02\xa2w\xc2\x00.\xc8\x08\x87\xf6^\x14\x13\x9c\x176\x12>\xe6\x92\xaa\xf4y\xd7D\xe2 0\xdcG\xa5\x82\'\xc1\x85\x8c\x84{\xb4{8\xaa\x84\xee\x9fTE\xc2\xbf\xe6EV\xf1\xd7\xb7EV\x11\x9b\xab\xac"\xebRg\x15b\x12b\x15\xa2\xb9\xc6*\xfe\x9a\x07\x7f\xfc5\x07\xc7\xfbk\x11\x1cO3e\xd3L\x17\xb1\x8aa\x94\x10\xb7\x17\xb0\x07\xf1(\xf0\xb6d\xe3}\xa11\x0b\xf3\x182<\xd2%&\xf1\xf3\xc3\xdb\n\x97\x18\xb9e\x87!\x06\\\xc8+\xd0\x8e\xd324\xbcs\xbc\x02\xc26@hG\xcc\xdbK9\x86x)\xc01dci\xcf0\xabM\x17\xc1o\xab\x1cc\xdb9A`\xda6e\x8e\xf1\xf3\xfc\xb6\xc02\xc4\x94\xb7\xa5u0\x96a2$\xda%\x96!\xc6\x01\x96!\x1a\x19\xcb\xb8t\xf0\xc82\xd0\xac\xc2\xb6\xd5"\xcb\x10S\xb3@P\x87\x14?\xf7\xba\xca2\x1aW\xef\x1bz<2\xcb Qz\x0cIw\x06\xc8\xc4\xf4A\x11u\x90\xb0g\x9a\x90\x91h\x80\xd4H* yF\xa2!\xce\xd1\xd3P\x94\x19\xd8\x02\xc9\xef\xd5C\xc8Wo\x81\xec\xd2e4\xcb\'\xee\x8f\xf3\xfd\xcd\xf1\xddMv\xe4\xb2\xf6\x9aM>\xefR?s\xe54t\xe8\xca\xf6\xda\xa9\xfb\xe3\x9b\xec\xd4\xfd\xf1uv\xea\xfe\xf8\xba\x04\xd9B\xa512s\x1f\xc2\xb7\x94\x04t\xcd\x02\xec\xb8\r\x1e\x8b\xb2\x89\xf3V>\x05\xbc\xaa\xac\xf5>\xb4\xde\x94\x9c\xb0\xf6\xec\x02\xe1\x94\xcd\xc6\x8d\xa7,\xaf\x86f""\x0bU=+\x9d\xac\xdb\xccr\x8eryI\xfe\xd6\x11W\x0e\xed\xe4\x90G\x8eE\xcf\xc49*\x17\n\x07i\xd6Z\xdc\x08L\xf8\xf6\x87\xe8f\x07\x85\xcar\xb1\x1b*\x9d*W;b\xa7&Y\x91[\x8e{[n=\x16>\r\xb3\x80l\x18jg\x96\xb5\xcd\xf8\x07\x9c\xa2r\xf4c\xf1\xb31\xdb\xc7\xa5\xc3?\x88=\x8d\xd1\x96\xd9\x96\xf1\x82\x02\xdf8p\x9a\xca\x15Dg\xa7s\xb6\xf4;\x07\x0f\xd4\xe9v(a\xd2\xb9\xac2\xd6\xe3\xf1\x8e\xd3,_\x88\xe7\x12\x95\xf2\x8cb\x0eR)?\xa9\x98\xd6\x94f<%\xc9P,M*I\xa0\x12\xa9UI\xd2\x15K\xc7\xf2Qd] \xdaP\xef\x13\x88\x96\xc3o\xb7a\x05\xb1\xe2(Lj\xf9\xe3\x9a\xf0|^471!\xc9$9N\x01\xfc\xae\t\xfa\x00V\xca\xe0\xd9a:Lj\x82\x0e\x01\xb0a: \x8c\xb1\x01<~\x9c\x0e\xb0\xe4Q\xd5\x802L\x9a\xa2\xe2;\xf6\xff!\xa8\x06#,\x19\x03\xec\xc7\xf0,\x03\xab\xa4\x8a\xf0\xdc\x19T7\x87\xf0\xe6\xd0\xde\x1c\xf2;\xc1\x02OQ\xc2\x13\xfcp\x81,\xce0\xc7\x134r\x81S\xce\x10\xcc\x13ls\x81z\x9e\x02\xa2s\xb0\xf4\x14H]\x82\xac\xa7\x00\xec\xba\x00\xd5>\x88\xfc\xb9\xe5\xfc\xb2\xd3\xbfw779\xde+\xbfP\xcf3\xcb;-\xe4\x9a\xc9\xa9B\xbe\x99\xbcP\xcd9{\x99\x97S}\x99\x97S}\xf9\xeaL\xc4\xbc528)A\x17\x1f\xaa\xc8\xb0c\x05\x83\r\xe3\xad\xfa\xdcu"X\xfc\x0f\xeeQ\x1fd\xf0|\xf6\x020u\xcd5{\x0f\xf0T\x02\x8e\x9dF\xf9\x14\x95\xf5c\x9dT?\xc9C\x11Bv\xca\x82\xaa\xaa\x9e\x92\x92\t\xae\x8f\xe1T\x899n_\xafl:\xb0*[\xdc/"\xa1e\xe5\x0b\x03\xe6\x9f\xb5\xc6}\xf4P\x02\x98\x9d\xc0\xf2\xa6\x98/d[\x01\x9b\xdd\xce\xce\xfe\xb6\x99\\eR\xe7\x11\x11\x82\x80\x9f\xf9f\x96\xa8\xb3rA\xb7\xb4\xa0\t>`{\xfa\xa2\xd5\xf2\xa2<B\xa9\xcd=\x17B,\x90S\x81X [\xb3\xba\xa1\x97N"+\x85r\xfc\x84\xca\xfe\x02\xb1 \xdb\xc4\xc12\xd7:\xef\xd28;e\xd6\xf9=\n~l\xea\xd1b\x8fG\x94\x0b8\xc3IYQ\xc2\xa4\x12\xf6\x9536\x95\x14\xb2H\xd2\x90\x03\xd76\xc1\x82\x05\x05\xa1\x1a\xce3\xc9\x8e\xa70\x1a7\x0c\x01>\x13&T\xac#\xa2\xa7\x8a^\x19D\xc4U\x01]\x93yl\x10U\x94\xeeS\xdcK\x8f(\xbb\xb1(\x08x\xe2\x9b\xc8\x07\xc9\x83\x8f\xa07\xc0Z\x10 \x86\x13\r\x95#i<\x113\xd1B\x8e\x9b7<6\x81\x9b\xab\xc0S\x8d \xce\x02\xaf\xbc\x986\x05\x96\x14Z\x13~\xe4[\xe7\xfb9\xe4q]\xc4\x8c\xd2\x19\x88\x13\xa5\xad56\xf4\xe2i\xc6\x86\xc0N\x97\xb0\xa1\x17O\x9e\x9e+[5U\xe0H\x8a\xac\xa7\xaf\xb0\x9em\x91\xd1\x9c\x9e$c3\xe9\xd3\x01\x8fy1y\xbc\x8c\xbd)r\x98}\xb6@\xc9U\xc4\x98\x91\xa50\xeb\x1cg,\xe0\x8eG\x84\xcf"6\xb9\xce P\x8al\xa4g\xee\xf5\xb1\xc8:\xd2\xa5\x01\xe3\x10m\x85\x8f\xce\x98\xc66\xc2\x8b\x15\x18\xc5\xb4m\x9c\xd3|\xd7HT\xf2t\xbc\xdbR\xdb\x91\xbd\xfa\xd9\x14uE\x93!\x85\xccEv\x90\x8e|,|\x14\xc6\n\xce\x0f\xca\x8e\xff=+\x0c1,\x1c\xfc\xe9l\xe1\xd4\xf7\xa5\xa0\x06;\x17\x8a6b9\xa9}\xb7{\\\r0\xc5^\xe0\xb8\x0c\x12\xb3!\xc5sH\xb1\x1eR\x1c\x88\x14#\x82\xe3G\xa4\xd8\x12%\xdc\t\xcb\xcfu\x86W\x91bY\xa4zk*\xc3\xa7\xf2=/\xcc\x94\x16mJ\x0b:\xa5\xc5\x9e\xd2BPi\x91\xa8\xb4\x80TR\\*\xa9;\x95T\xa4JjU%U\xac\x92\xfaVI\xe5\xab\xa4&VR-K\xd4\xd1b\x88-I\xed\xad\xa4XFRFC\x14\xd8HJo\xb0\xd2\x92T\x0fx\x8c\x04>\x02U\x97\xc6R\xd3&~\xc7\xa4\xeetR\x91:\xa9U\x9dT\xb1N\xea[\'\x95\xaf\x93\x9a\xd8I.u\x92e\x9d\xe4_\xa7\xa9\xd9\xa0)j\x13\xf6\x95\xf2;\xbe\x15\xd8#\xe7\xcbH\x95\x8bH-\xeaz\xb2\xc7\x99\xc2Q\xe5\xb2Q\xe7\x8bF\xe5~\xb1\x9f\x9e\xe7E\xa3\x9e\x9fU\xf1\xb2\x02R\xc5t\xe8Z\xb9(,\x03\xd9\x94\xcaB\x95\x8aB\xc9\x92P\x10\xdd\xca/,W\x85*U\x82\xaa\xd6\x81\xc28\xe6\xac\x0e\x14\xc6:\xebsu\xa0\x00\x98\x84!mU\x8a?\xf5,\xe5\xa5\xc9\x8b<\xb1\x15\xb2\xeaN\xa5\xdaN%p\x92,Ky_+\xf1\xe8"\x8ec\x96r_\xac\xe3T\xaa\xe2$k8\xe5\xdf\xe4\x92\nN\xa5\xaaM\xa5\x9aM\x9fU\xb1\xe93\xaa4e\xc9\xc8\xe9\n\x8a\x15\x99\xcau\x1aC\x8f\xc7\x846\x91u\xec\xd3\x1a\xf7\x9aU\x84\'\xaaG\xb4\x07\xcb*\xf7R=]\x8dT\xc3\xca\xe9\x9a\xa40\xafI\xea\xf2\x9a\xa4\xd2.\xd57\xd6H5\xbc\xda\xb1\xbf\x8f\xd0% H\xa2E\xd8\x13:\xf2\xc7\xec\xc8/2\x83&c \tkI\x98N\xc2\x8e\x92\x1aQ\xa2z\x14\xab+\x95T\x9cj\xd0tM\xeb\xb6\xb1\xdf\x9a\xeaT\x8d\x11h%\xf4\xd4|\x10|O\xf4\x954 \x8eu\xf1\xf5\xa2\x8d{\x8dV\xed1~\x14\xeaf\xe1\x9e\x01?e\x13\xad\x97l\x0b\x9c\x8b\xbd~\x7f\xda:\xab\xef\xa6\xfb\x9bR\x00vr1\xe1B\x95\x8b\x8b\xe1\xd7\xf9T,\x06;\xbfX\r\xd3\xf8\xcf<\x10\xfb\xbb<\x10\xfb\xbb\'\x97\x05b\x1b\x99\xb4\xa3\xfb\x95\'T\x9eJpY\x84\xf6.\x9a%\x93\x0c\x9ey\xe5\x89\rcfC\xd1\xe9X|\x1d\xe5\xf7\x9e\x06q\xe7\x97\x96"\xb9\x13c\x12;\xea\x16\x9f<Dx\x17\x96qi\x98\xf7\x94e\xfe\xa8\x95o\xeab\xe0\x08\xa5\r]\x14\xf9\x8d\xd6N\x9e\x06dW\x9e\xb0y\x0e\xd1e!\xe1\xbbh\x18\xa5\n[\xc0\x86\xfdG\xd4\x9b\xc8\x93\xc9]\x9a\xc7\x8a\xe7\xaf(\x04\x8c\x17.\xd5\x7f\t\xe5\xd0\xf1i\x17+t\xb9\x97qjQ\xe0O\xad\x05\x93\xfb0\xc6\x9d\xaf\xdd\x08E\x19w\xc6#|\xaaa\x9b\x87\x97\xe7\xcb\xb8]\xb8T_\xfcR\xb4\xb9\x99\xb2\xec\xa1f\xed\xe3O,K\xfaQEWlq\xb2c}w.\x05\xa4\xc3:x^\xd1%\xeb(G\xaa\x97~U\xe7\x0f\x94\x10\xc2\x9e/<\x8dco{\x17+\xe9\xb0\xbb\x1dV\xb7\xfb\xee\xcew[\x10-x\xef]\xb3Kz?\x9e\xb0!\x18#+\xd2\x98\x94oL\xeb:\xf2z\x8f\xbc\x0e$\xaf\x0f\xc9\xebF*\xe7>\x1c\xc3\x0c\x1d\xa9\x88\xbe\x7f\x84-\xebI&\xe8\x83\xef\xcf\xb7\xa3\xeb\xaf\x815\xb4Qn n\xd9 \xef\xec3\xee\x1e\x1fo\x91\x8f\xbe\xb9y\x9b\xf3OjL\xf8\xa6k\xbcZ\xb2\x8c~\xf5\n`m\xfc\xd5\xffy\x98W\xfe\x8e\xe9p\x1f\xf7]\xe0\xa9\xc9\xb4\xc4K\x93\xc6\x1a\x0f}\x93\xd7Iz\x93\x87:\xbe)\xd7I\x9a\xb2P\xc7\x12c\xf4q\x81\x84\x83\xd0\x17\xf9\\\xb2\xd4\xf8~\x04_K\x9a\x18?\x9b\xb2x3\xc9\xa5\xd2\xc1$d\xe4VF,\x96\x99\xcd \xe3\x13\x8b\x1cD\xed\x19"A_d\x08\xc9j\x80\x11\xa4M\xf9\xe7c\xba\xd9\x10\xf5\xb1\x8d\xcd\x0ex\x17\x1e\xe3k\x16\xc9 \x99d\xb8\xdbBS>)\x0b1\xd4\xcc\x1ei\x8a\xa7pr\xe71\x7f\xeb,\xa4pi\xb0x\x94B\xbd\x83\xcagE\x08I>A8\x1d=t\xa7\xab-T*y\x1b\xae=\xe6\xc9\x97j\x01\xc9\x11by\xe5Z~\xec\xf0\xe3H\x1cS\xe1\xf8\xe2\xc7\xda\xc0J\xe9\xc6c\x10\x94\xa5\x96\x83B\xb6A\xc9hB\x08\nh\x1c\xf1\x8a\x0f\xc3\xb0\xa8b\xe802\xa8"}\xd0D:\xb7.]RKR\x8d\x85k3\xa9\xa6\x93jA\x89\x86\x14_\xdb\xb2\xb7\xe9]\xa9\xb4Nh\xadZ\xc4d\x8f\x05o\xd3\xbbRE\x9d\xb4\xb5\xeam\xfas\xeem\xfas\xeem\xfas1\xa9h\xba\xa8l\xce8\x9c-\x8c\x93\xae\x15\xfdI\xef\xf2\x928i\x1b\xcb\x13\x9a\x16*\xde\x88\x81\x8aN\xa4\xf3\x05m\xf4E\x85k\xd4p\xb64\xcd\x8bw\x90\x7f\x9f\x15\xa7I\xd7y(|D\xe9=\x82C\xb4P\x90\x06\xbdGy\x11\x9at\xbc\xdbR\x1bz\x8f\xca\xa5g\xcc\xd9\x123\xe9h\xe41*\xd7\x961\x17\xd5\x90\x81\xb3\x13\xc2\x05\x17+\xc6\xa4S\xa7\xae\xa2r\x89\x98a\xef\xcb\xc7\x9c\x84\xca\xc7\xb4Z\x01\x80\xbaB\xff\xf9\x1a\x80\xd2\x835J\xb5\xe8^\x08d\x1b-:k\x95\xb8%"\xa9\x89\x1cb\x84\x16]\xc5\xf0\x01\x8f\x9c\xc5\x8c``\xc0\xe7$\xb3\tyl;N\xb6\xd8\xb9!2\xb9\xda\xe1\xd5@\x1a>o\x87h\xf7\r\x91\x1d_s|\x1b\xcb\xe7\xe4\xc3\xe1z\x9e\xd6O\x0f\xf7\xf7\xd3C~^\xca\xab\xf5s\xb3\xd2s\xe1\xfc,\xce\x1c\xce\xd1\xe2\xd5\xday\xfa\x97\xbf\xe6\xe7i\x9e\xa4\xf9\xe2\xa7\xa7\xd5\xf34E]aZ\x17\xc7\xeb2<\xc6l\xa8:\xf4S\xd8\x95\xb5\x04\xec\xdap\xff\xc4(\xcf\xe4\xe2s\xe3\xd9\\\xbev\x9f^\x83s\xee\xa1\xea\xfaO\nS\xf3g\xe4!\xc3\xdcf\x9d\x89\xc5n.P2\xe6l1\xf2\x9c\xe7\xc9=#+d\x8eg\xfe:\xb3\xf6\x8c,B\xdd0&\x81\x03\xd5\x99\x047\xdep\x13N\x02\x06\xa6x\xcaP5\x04\x81\x1bl\xacW\xe4\xa9\xf4\n\xc3\x81A\x87\xca$\xcd6\xfe\x85\xac\xe1{H^S\xfc\x80\x87\x85\r\xcfx\xcf\x1c],\x00\n\x0b\x08a\x1b\xb0\xdct97\x9aMC\x8e\x16@\x08\xdbz\x06\xe6R\x8b$\x7frsg\xdf\x12\xf9\xd4\x1fn]+y\xed\x98n\x04yo\xc6\xbcr\x98\x17p\x9b\xf8\x9f\x07\xd5;\x1b\x98\x99\x04#\xd9$\x9f+\xbe%\xe4w\xa7K\xf2J\xc6\xf62\x00\x98\xc2:\xe0re\x1d\x92C\x92\xe7\x8f\xfd\x90\x92\x9f\x90\x95\xec\xb2\xfa\xce\x02\xebTn\x17\xb7\xca\x15\x07v\x80\xb3\x04\x1f\xd6\xcf\xdb]\xc9N\xd3;\xd8Y\xd5\xf5\x008\x1bz?\xae\x9d\xc6t\xe0\x89\x8e\xf8\xa1Z\xc7l"\xfc\x7f\x9b\xfc\x8f\xea\xbe\xb4\x1e\x19]c\x05\xcf>"\xdcC\xedS\xd3\x84\xd2\xa7\x83G\xb1WxMG\x84{\xa8\xb0\x1a\xa8\x1e)V\x07\xcdP\xbd\x96\x01Q\xf3\t\xdb\xa0\xf7\xd7tL\xaa5*\xe4\xc16qL\x80*5*\xe0\x896q\x14\x00*\xa5\xf9\x00j\x95\xe6\x03P\xcf@y\xff\x8a\x89I\xaf)\x05\x80\xf1C\xc4\xd9\xd4c\xc0\xf3\xec<\xa5"\xba\xa7\x1e\x02\xc8\xa6\xf6\xb0\xe9CL\x97\xa5J\x03\x06b\x07\x86\xb0\x16\xe5)z\x06\xeb\xc7\x1c#\xe8i\x98]#D}\x8f\xf7\x99\x88Q\x0f\xb8\xa4\x01+\xd6\xc7\xb2\xe8\x96C\xe3\x12\xa0; \x89j\x13pE[O\r\t\xa5\x10\x7f\xb7\xf7q\x0c\x1a)\xd8?\x8b\x92\xc9\xf3\xb7\x1fo\x8f\x99D\x12Z\x13I\xe4\xc9\xdd\xb4I0\xdfe\xb7\xba\x18\x92NC\xe2\x87o\xfdzE\xe3~sV\x02y\x9e\xc78<\xcf\rb\xcf_\xfc\xad\n\x15j\xdazD\xc3\xcc\xa4\x87\xae\x18\xc5\xe0\xebmX\xb7\xea\x0c\xc6=}B\x10\x17\x9e\xbf=\xdc\x96S\xcd\xcak\xd9\xc7\xa2\x1e\xa3\x9f\xa4\n\xd5\xce\x03\x19\xd0i\xd3\xf0D\xb3j8\x03\xc6\x9aw\x19F\xfb\xae\n\xe0irX6\x19\xd3pzx\xf7\xa4\x1f$D\xe7|\x9f\x81\xb0\x9f\xde\xc9\xf1xS\xc7Y\xf3\x7fw}%\xd9l\xdb{\x8c\x87\xd1\x89\x1b;\xbd\x91*\xe0\xf3\xf4\xad!\x98\x9a\xfb\n\x12I\xed\xb4\x8a\xa2\xf3\xa3e\x86/\xe0\xa3{\xc6\xbf\xb4`\x94\xcf\xc5V@\x94\xcdS_e\xd4=\xfcS\x8aoh\xec\x85\xd3<\x9c\xdb\xbe\x08\xb3~X=\xfb\xaf\x075O\xbb\xdb\x9c\xd1\xf5N\xc2s\x88\xe9\x05\xb6\x16\xaf=\xa6\x8e\xd8c\x04a\xd0\x99\x86\x18P\xe8`\x8dGN\x1a\x0c\xc5\x1a\xf8U\nZ\x80 l*\x04\xb5\xc6\x98.\xb2\xafA\xa9\x8b\x94"\x03YZ\x14\xa3\xc3\xf3\x95\xa8\x86\x15\xd3\x00%\x95\xacl-\x86\x9aiv\r\xb3\xa2\xd6#\x06\xa8\x11\xd5\xc7@\xb65\x86\x002\xb2g\x81\x12\x18\xcaMv=H\x90\xe2\xe4\xe0\x03$\xc2;\xd0<\x00\x03b\xf6\xda\xf8\xc2\xac_a\xc35aZ>\xc6Hvto\x8bW\x03\tW\x13\xed\xbd\x8b\xaa/\xbc\t\x95\x92=\xd7\x84\xfb\xa8\xfa\xba{\x87\xd8\xd9\xc4\xa2&\x1e\xf6;\xc6\xe5\xe1U\xaae\x82#\xc7j&q\xbf,\xe3[O\x0fw\x87\xe3t]\xa8Z\x92^J\xb8\xd6\xf7\xf7W\xd3\x07Q\xac\xa4\xd8}\x01\xe7:\x9f8\xe0\\\xe7\x97\xaa8\xd7\xafr\x9c\xebW9\xce\xf5\xab7\xe7p\xae\x15S\x9f33%\xa9\xd5u\xcc\xebmV\xb5\xc4fF\xcc\xf3\xd5L\xdc\x83\xcf\xee\xc13NXx%\x08\x81\x9d_\xa8@`+\x16\xe7\x97=a\x06d\x17\xc6]\x80\xc0VYx\xc3\xc8!/\x98\xce\xbcP\xecd\x96P\x18\t\x07\x1d\xb2pA[\x07\xc5\x1eY\x10C\x13\xe1M\x13\xeb\xeaR\xf5\x13xf\xb4\xb6f\x05P\no\xfaP\xdb\xae\x12\x1e\x1b\xa3\t}\xc0\xc2f\xcf\xd5\xe1\n<\xf6\xc6\xdb]v\xcd\xb6\x8cq\x1a\xa6\xf4\x8b\xddH\x8cl\xdaF\xd2J\xcb\x1eP_\x97\xeb\xa2l\x99\x89U\xb30\x81\x81\xe5ns\x03,\xb4\xef%bva#\x13Z\xb6_\xf8\xc7\xa9\x0e\x97}\xf1\xe4\xa8\xed\xee\xcfAg\x9f\xd9\xf9\xfe\xaf\xe4\xfe\x85\xfd\x1f\xf0P7\xbb\xd3\xc7\x1b\x95\xbb\xb3\x1d\\f\x1e\x043\x96K\xac\x0c\xc6U\xe5\x81\xb2*\xa2\xf7c\x96X\xe1\xd9\xcb2\xcb,&<\xf1L\xa84C*\xa6\x8b\x85\x91\x96u\x9c\xff\xf9f%,EA\xd1\xf9\x9f\x87\xe9x\xfa\xcc\xd3\x15\xebQ5\xbf.\xf5^\xd0}Jw\xa5zP\xb5GU\x11\xfa!7\xc5\xfeG\xc1\x14\xfb\x1f_`\x8a\xe5u~X\x18\x0c\xf7\xe4\'\x86K\xb8y\xacF\xdd\xa1F\xd5\xb3\xe4^\x88\xc6k8\x93Y\xd5,\xbbE\x15,\xbc\xb0\xb7\xe9\xfbB]\xac\xfa>\xb9\x19w\xf5|K\xdd>\xcb\x8c\xbb\xf8N\xf0\'\xba\xad\x1bsW\xf0oXb\x91i1\xfb,\xc4\xa9\x17\x92\x8b\xb9\xa1\xd7\x8fz\xa15\xf8\x0b\x8c\xbb\x96\xa9\x8e\xa3\xc4\xf8\xe6,\x8f\xbdj\xc6\xf2\xbc\xd2\xe8\xcfDd\x7f\xbag\x18P{\xc6\n!p\xbeclq\xbf\xaaY\x8cs\x1dt\xbe\xbe\x82Pu\xc77T\xab\x0eK\x06\xe2\xd5\x0f\xefNG\xd8\xd5\xe1\xfex\xf8c*\x9a\x89\xd9\xef=\xaf\xd7=\xec:\xeb\xa1\x8dLn7\xce\xd5\xd6\xc1\x01RoN\xe2\xc6\x8a\x9b\x8c\xa5\xf6J\xd3MWr\xe1\x89\x95\xf8\x15\xe4\x1d\xfeq}x\x7f\xf3\xc7T\xb5\x16\xb3\xe5W\xa1\xc21\xb4/\x87\n\x9f%K3\xacV\x98\xddr\xb7&0\xb6\xd5%fh\xa9>\xbf\xff\xbd\xfc3F\xe8q\xb8\xbcr&\xe7\xfb:\x00\xf9%\xcf\x80\xb6\xe8\xea3\xac.1^\x9f\xc10\xaf\x1d\x90h\xb5N\r\xd9\x05{\xf5\xe9\xcb\xfe\xdf\xb0}\xb7\xef\xdc\x1d\xca\x95g;\xe9\xf4\xaa\x00\x18\xc36\x9f4d\x17\xd8:\xef-\x0c\xd9\xff\x02\xdf\xb0I\x95MF\xb2D6\xd2L\x03i\xd3{)\xd1-\x90<]\x01]\xc5&\xba{\rKt\x08C\x99t\xa2\xc4\'\xcd\xae\x8a\xcec\xcc\xc3C\xd3EH\xaf`\x0fx&\t\xfb\xc3\xa6\xa0\xa4\xbey7\xaf\xe2\xa5\xaa\x94q,u[\xca\xc7v]\xbf^\x9dF/de\xe7\xd7\xaa\xd12Or\x91"\x87\x08y\xf1\xf2\xe9\xb9h\x99\xac\x00S9r\xa6?_tiQ\x01\x15/\x19\x03\x13\x93w\x97dk\'\x8d\xe5L\xed\xbcdM\x1ep\xe3\xc6\x99/\x8b\xb8\xc9\x1c\xaeK\xbc7+\xa9T\x8e\xc1\xe9\x97\n(-j\x8c\xe2\rPVw\xdax,4f\x919\xacrR\x81\xd7M[\x15\xaa%m[k\xb3\x1c\xefd\xf0\xdbb#,\x03\x94\xbf\xe4J%\\\'\xab\x8b\x94\xb1\x1a1\xc31~\xcc\x8fg\xc3w\xb2\nHEV\x90\x08\x8cK\x1b\x89r\xbf\x93\x05E\xb7\xa4Cb\x04\xe7b\xf94\xef[\xc5{<\xe6\tn\xb1\xda#\xc6\xefX\xc4\xd8\xc5\xf4\xac\x16KC\xe2\xa1\xdc\xa6e\xf8\xdaPGR#i\x81\xecyg\xb4ZZ@J\x0fE\'G(\x96\x87\x9d\xe9j\x97.\x83\xd2\x7f\xc7\xa4\x06\xa5\x05\xf4\r\xaaBh\x01e\x83\x0c\x91v\xa0r\x870T\x0f\x08\xed\x18\xbed{\xaa\x149 \xa9\xa0\xae"L\xd4QA&\xc8A\xc6:\x92\xe8\xa4\xb3\x96\xea3\x19$\xb1\x98\x13,\x03\xaa\x10R\x1dIk\xa8Dc\x87\xa4\x81e\x0c)\xa9\x90\xb4\xac\xf6\xa5\x05\xe8^\x8d*/\x91T\xfa\x13PQ\xc8\xb4j\xb1\xa0!BgYK57\x1b\xb6H5$kV}\xe8l\xf8{N\xab\x1fZK\x1f\x85\xdeF\xcb?hK\x1ft`\xaf\x8e\x91#\xdb\x1b\xb6Oj#\x9e\xbeQ\x03\xa4b\xdf($\xb9\x0fT9q@r\x00\x92\xbe\xef\x98\xeeX\rd\xc3\xb7\x8a>[9q\xdaMo\xa7\xfb\xedtW\xb0,\xf3K\xf5\xca\x89\xa5nK\x16\xe5l\xc2hQ\xce.U-\xca9\xcb~\x99\xbbC_~\xfb\xa2jQ\x06\x1ca`\xdf\x06\xd2\xbf\xf59\x07)\xb0\xec.\xfe\x9f\xb3\xf2\xcd\x90%\xd7\x95\x13\xc2\xf11\xdfOw\x99\xf98{~2\x1fg\x17\xa4\xf9\x18\xc5n\xc6\xd29{\x071\x9d\x83\x7fe\xbe\xd7\xd2\xe4ES\xf2\x86\x1b\x8e\r31gb\x00j\xeb\x0b\x95\x15\rCUV1Q\xae\x9cf>G=\x99\xd0\x93\xa5`\xc0]\xb5\x98\x1b\xd7d\xe6c\xff\x9c\x9f\xde\x83\x05\xf9.\xb3\x1fg\x1f\xe7P\xdb\x9c,\xed\r\x94\xe0&\x8a\nP5\x0b1\xc4\n\xf9\xe9\xa3\x0f\x9b\xd2.\xff\x0f\xc2z\xb1j\x16 \x89\xe5\x19\xeb\x85u\xddV/\xc4\x05\xc3C&\xab\xae\x98\x91Y"\x19\x87U\xc6(%\x86\x07\x96\xa5\xb7\x17Vp\xbc\xab\\\xa8\x98\x91\xbfdriF\x1e\x06)\x8f\xf3]or\xd3q\xf6=Cu\xee\xa9\xdb\x02\xba3\xc5\x19cq-=\x15\nj\x8d^pwH\xcf\x85\xde\x9f\'\xa1\xbc|\xb8:\x1e\x96\xc4\x94\xbf\xaf;\x8f\xf4\xb1n)\x94\xf8D\xbbs\xbf%\xbd\xad\x83\xca\xdb-i\x97\x1d\x94\xedn\x91\x1d\xaf;(\xe5\xdbR\x06F\x07\xdc\xacE\x06\x15\xfa\x07\xba\xa7\xfe#\xd1]B\x0feZ{M\xf1k\xc8\xf7\x87{LH\xfe\x06>gB\xce?bQkA\x9bH\x1b\xa0\r\xd1\x16\xe8\x91h_9\xcc\x12j@\xef\xdf\x89!\xbfq\x0b\xc5\xba\r9\x8e\xdb\x0e\xc0\xae\xc9\x19\xdcB-s\xd3\xd1x\x08\x86MI&-\xf0V3\xd0;SD\xe33C\xc1g\x83\x00\xae\x8e\xf6\xebGQ\x84\xae[z~\xbcn\x1b\x1a\x0f\xaaK[z\x1f\x1d\x88PVQ\x7f\xa8\x10n\x1b\x1a\xbf\x85\xfb)\xa7\x1f\xbf\xa9\xa5\x00\xf2\x0e\xc4\x19K\xa1\xea4\x9e\x8e\xe3\xb7@w\xd4\xdf\x02=\xd2x\xee\xf9\xac\x89{\xc6\x8f\xc7\xf6\x90\xfb\x9e6\x04\x15\xc0\x9e\x89t\x0f\xfd[\x1a\x1f\xbe\xbf%O}\xd7\x01\xddS\xff\x96\xe8\xb0\x07\xfd\xf3\x0cqOk\xa0{\xbe\xc7\xed\x18\xf7\x9c{?mC\xe3C\x8d\xa16\xbc_\x90\x9cZ\x8al\xef`\x7f\xb5\x9a\xfa\x87=\x1b\xeeo\x81\x86\xf9{\xc0\xc3k\xe9\xfd\xf5\xf0=[B\xa9 \x9a\xe6\xeb\xa1X:\xa7\xf1\xba%\xda\xc2\xfa\r\xdd\xdf\xc1\xf3\x8d|<K\xfb\xaf\xd7\xd0\xbf\xa7\xf1`=\x96\xf6k\xaf\xe8{tD\xf7\xfc{\xf4\xb0_-EY\xf4\x8a\xbe\xa7%z\x00Z\x13\xdd\xc2\xf7\x1f\x92\xf9L|^\x05\xfb\xc5$\xf3\x89\xf7\x15\xf6+^7c\xa4-\xff\xfd\xf4X\x07p\x88\xeb\xf7\xbf\x9f^\xd0\x14\x8f\xd17\xe9\xef\x99\xe8pf\x81\xdad\xac\xa0\xe9\xfc\xe8\x1b:_\x14\xd1-\x9cG\x96\xfa\x8fp^\x85\xfe\x88h\x1f\xae\xf7\x00x\xdf\x12\xdd@)\xdc\x96\xf6\x93\x81\x12\xb5\xc9\x99\xa8\xbbx\xdd\x02\xad\x13:\xfc\xbe\xe0\xf7\xa3m\xfc=5\xa0C\x05Z\x81\xd6\xd4\xd3\xef\xc5\x80\x9e4\xf0\xf3@\xb3\xf3\xc02\xa5\xcb\xd1x\xbd\xa1\xf3g\x10t\x0f4\xae\x0f\xeb\x80\x05\x9e\x82\xef\xc3\xc6\xf3\x19U\xbe\x81\xce\xcf\x81\xa9\x8ft\xfe\x93\xfe\xe8\xfaw\xf0\xbeTr\xbd\x8f\xe7q\xcb\xdfg\x0b\x15\xe5\xe3uT@)\xb6\xa7\xb54~O\xb4\x02\x1a\xef7\xf4\xfe\x95\xa0G\xea/\xe7\xd3@\x8f\xb4\x1e\xc3\xea\x1d$\xfclY\xbbzr\xf7v\xbe>\x1e\n\x99\x82\xc9\x15\x11k\nW>\x13\x9c2\x9f\x8a\xd4\xaa\xfcJM\xabz\x92\xc7\xe9<\xc9\xe3t\x9e\xbczq\x06H\x0b\x8a\xcd\x80G_s\xa0\x81\xc4\xffZ*9\xd3e0%\xdb\x88\x92\xcc\xcb\xcf@\xf6v\n_"\xf4,z\xee<N\'\x7f#\xa0g\xe5\xed,\x9d[\xc7g\x1a\xf3\xd2eI\xe5V\xa1Z\x85/Z\rj\xddrm\xaa\xcd\xcbKr\x7fg\xbd\x06\xf1\x90\xd5\xa8\xc9\xab\x0f\x97L\xabM\x9b\xe1\x8al\x99\xae\xc5\xaa\xd6`\x16y\xb3\x90\x08\t\x0f\x8b\x95z\xa5\xd15\x7f\xc1\x87\xca\x16e\xa6W\x1d!E6\xbc\xaa\x8d\xaa\xa11\xb7\xd6\x060\x11\xffV\xb0\nq\x9e8\x19\xb6H1B\x07\xae~\xcc\xd2(\xf3\xd52\x9d\xaaa\xeeC\x86\xdf\x05\xfe\xb4B5b\xa9M\xe5\x83\x1fq\x13\xfd6\xb9\xf6\x92*\xf5\xe5sf\xfe\xbc\xec7\xdb\x8fY\xd8\x83\x04\xf3\xcc\x97\x1c4\xa9\xce\x19\xe6\xbb\xc1\xb8\xb0\x9ay\xdf\x17\xb4\'\xd9\xe3\x9f\xff\xfc\xef\x7f\xfe?\x14{\xbf\xe0'
    )

    #
    # CONSTRUCTOR
    #

    def __init__(
        self,
        background_color: typing.Optional[Color] = None,
        border_color: typing.Optional[Color] = None,
        border_dash_pattern: typing.List[int] = [],
        border_dash_phase: int = 0,
        border_width_bottom: int = 0,
        border_width_left: int = 0,
        border_width_right: int = 0,
        border_width_top: int = 0,
        dash_pattern: typing.List[int] = [],
        dash_phase: int = 0,
        fill_color: typing.Optional[Color] = None,
        horizontal_alignment: LayoutElement.HorizontalAlignment = LayoutElement.HorizontalAlignment.LEFT,
        line_width: int = 1,
        margin_bottom: int = 0,
        margin_left: int = 0,
        margin_right: int = 0,
        margin_top: int = 0,
        padding_bottom: int = 0,
        padding_left: int = 0,
        padding_right: int = 0,
        padding_top: int = 0,
        stroke_color: typing.Optional[Color] = X11Color.BLACK,
        vertical_alignment: LayoutElement.VerticalAlignment = LayoutElement.VerticalAlignment.TOP,
    ):
        """
        Initialize a map representation for a specified region.

        This constructor sets up the visual properties and layout options for the map, allowing
        customization of colors, borders, alignment, and other attributes.

        :param background_color: Optional background color of the map.
        :param border_color: Optional color for the border of the map.
        :param border_dash_pattern: Optional pattern for dashed borders.
        :param border_dash_phase: Optional phase for the dash pattern.
        :param border_width_bottom: Width of the bottom border.
        :param border_width_left: Width of the left border.
        :param border_width_right: Width of the right border.
        :param border_width_top: Width of the top border.
        :param dash_pattern: Optional pattern for dashed lines within the map.
        :param dash_phase: Optional phase for the dash pattern of lines.
        :param fill_color: Optional fill color for specific regions in the map.
        :param horizontal_alignment: Horizontal alignment of the map.
        :param line_width: Width of the lines drawn on the map.
        :param margin_bottom: Bottom margin around the map.
        :param margin_left: Left margin around the map.
        :param margin_right: Right margin around the map.
        :param margin_top: Top margin around the map.
        :param padding_bottom: Bottom padding inside the map.
        :param padding_left: Left padding inside the map.
        :param padding_right: Right padding inside the map.
        :param padding_top: Top padding inside the map.
        :param stroke_color: Color used for strokes on the map.
        :param vertical_alignment: Vertical alignment of the map.
        """
        import json
        import zlib

        super().__init__(
            geo_json=json.loads(
                zlib.decompress(MapOfAfrica.__MAP_OF_AFRICA_GEO_JSON_BYTES)
            ),
            background_color=background_color,
            border_color=border_color,
            border_dash_pattern=border_dash_pattern,
            border_dash_phase=border_dash_phase,
            border_width_bottom=border_width_bottom,
            border_width_left=border_width_left,
            border_width_right=border_width_right,
            border_width_top=border_width_top,
            dash_pattern=dash_pattern,
            dash_phase=dash_phase,
            fill_color=fill_color,
            horizontal_alignment=horizontal_alignment,
            line_width=line_width,
            margin_bottom=margin_bottom,
            margin_left=margin_left,
            margin_right=margin_right,
            margin_top=margin_top,
            padding_bottom=padding_bottom,
            padding_left=padding_left,
            padding_right=padding_right,
            padding_top=padding_top,
            stroke_color=stroke_color,
            vertical_alignment=vertical_alignment,
        )

    #
    # PRIVATE
    #

    #
    # PUBLIC
    #
