#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A specialized version of the Map class representing a map of Europe.

This class inherits from the generic Map class and provides geographical data specific to Europe.
It allows for rendering and customizing the appearance of European countries, including setting
specific fill and stroke colors for individual countries.

Use this class to create and manipulate a map focused on Europe within a PDF document.
"""
import typing

from borb.pdf.color.color import Color
from borb.pdf.color.x11_color import X11Color
from borb.pdf.layout_element.layout_element import LayoutElement
from borb.pdf.layout_element.shape.map import Map


class MapOfEurope(Map):
    """
    A specialized version of the Map class representing a map of Europe.

    This class inherits from the generic Map class and provides geographical data specific to Europe.
    It allows for rendering and customizing the appearance of European countries, including setting
    specific fill and stroke colors for individual countries.

    Use this class to create and manipulate a map focused on Europe within a PDF document.
    """

    __MAP_OF_EUROPE_GEO_JSON_BYTES: bytes = (
        b"x\xda\xb5}[\x97\xdb\xb6\x92\xee_\xd1\xca\xcby\xd9\x9dE\x10\x00\x01\x9e7_rbO\x12'\xabm\xef=\xb3gf\xedEQ\x94\xad\xb8-\xf5Hjg\xecY\xf3\xdf\x0f\x89\xaa\x02\n\x17R\xeaLO\x1e:.\x02\xc2\x8d \xbeB]\xff\xeb\xbb\xf3\xd7\xfb\xe1\xbb\xff\xbb\xfa\xee\xff\r\xdd\xf9\xe18\xbc8\xdc\xdd\r\xfdyw\xd8\x7f\xf7\x97\xd5w[xx\x1a+\xfc\xeb\x7f\xa5U\xa7\n\x1f\x86\xc3\xe7\xe1|\xfc:>\x0e\xe5\xbf<\xdc\x9dw\xbf\x1d\xee\xbe~\x80V\xfa\xc3\xe1\xb8\xd9\xed\xbb344\xfew\xa3\xc5\xf7\xe6/+\xf5}\xfd\xef\x7fY\x8dT\xfd}\xfd\x97\x95\x0cT\xf3\x97U\xfd\xbd&\xaa\x9d(\x01\x94\xfc^E\x94\xab)\x89\xb2\x13\xa5\x80R\xdf\x0bVS}\xafYM\xf5\xbd\x9c(C\x14\xef]}_MTC\x94b\xe3t\xad\xa8\xef[VS\x7fo\xd9X4\xfd\xce\x8dZ\xd3X\xc442Ee~\xee\xff>\xd1\xff\xfa\xaf\xcd\xd4\xbfja\xc2\x8d+l\xa1G;\xcda$\xaa\x890S\x07\xca\xc2\x1c\x8c\x1b\x89\x81&\x8dk\xc0@o\xd0\x80\xa1\xd6\xac#$\x10\xe3xU\x03\xd3&\x02K\xb4#T\xf8M\x03\x9d\x02\xa1\xe17\xc6\rG\xd3o\x8c#plSk\x8a\x8fM\xc1\xd2\x1b\xb7\x80\x92:\xd5\x8ep%\xca\xcdGB\xa7\xd25\x8d%\xd2\xb5\x86;\x00\xd6\x0e_]\xe5:\xada\xcd\xab\xe9-\x8e\x04,\xab\xc0\xb6+\xa4\xda\xd0\xf8H)7\"*\xab\xc3\x04oj\\<\xd8(\xd05\xae\xeb\r\xbco\x8b5\x15\xbe\x00\x835\xa5\xa3Zl\x13\xca\x1a\xd6{\x8b[C\xb86[\x98\x81\x98~\xa6+\xe8\xce\xfdj$\\\x1b\xf5\xd4\xd9\xb89\x04\x10\xc6\x95X\xbf8\x81\x80\xdf(XC\xe9\x9an\x81\xb0\xae\xc4\x8dV\xe3>\xd2@\xc0x\xf0\xfd\xd2v\xc3\xfdgqQ\xdd\xd8[|_\x15\x10\x8d+\xa9\x81\x98~'\xa0\x11\xeb\xde\x8a\x80\xdfX\xb7L\xf8\x8aBk\xe3\x7f\xff=~\xfe\xf7\xc7\xc3\xfdp<\xef\xdc\xd7\xff_\xdf\xed\xbb\xcfp\x8a\x1c\xbb}\xef\x0e\x91\xe9\xc9?\xee\x0e\xfb\x0f\xf1\xe3\xed\xe1\xf8\xb9\xbb\xfb\xc7\xb0\x87\xc7\xc3\xbe\xff\xb8\xba\x1d\xee\x1f\xd6w\xbb\x9e\x95o\x8fc\xf9\xfe\xe1\xee\x8eZ\xeaw\xdd\x1f\xdbB\x0f\xa7\xc3\xf1\\x\xdc\xdd\x9dC\x03\xdb\xdd\xfd\xe9\x1f\xa2r\xd5n\xa7*\xbb\xd3\xe1\x1f]\xfd\x8f\xe1c\xf2H\xfaG\xcfBK\xd3H\xbe\xfb\xb7\x87\xaaQb\xfa+\xdd_\xd5\xb8\x7f\xcb\xe9om|\xe5\xf5\x1e+\xb7\xddz\xfa\xdbo\xa6\xbf\xeb\xca\xfd\x1d\xdcs\xcb\x9e[\xff\xc3\x8d_\xbdO\xc7a\xd7\x7f\xf4\x05\xb4P\xd1\xec\xdc\xa2\xc3\xc3]\xe7\x9fn\xbb\xd2Hk\x93\x8eW\x85\xf1\xbae\xceZ\xbf\xc3vd;\xd5\x97k\xe1\xfe\xae\xc3\xdfn\x0b\xcf\xfdO>\x0e\xf8\x13=4\xee\xafu\x7f\x95\xfb\x1bF\xf8qG\xcbS\xbb\xe5\x91\xfd\xf4W\xb9\xc5\x90n\x91\xa4[\xa4\xda\xb2\xe7a\x91>>\xb0i\x1f\x8e\xa7ocy5\x88\x0f\xbe\xc2n3U\xf8\xcdU8\x85\xa7\xe7\xd2j\xfd\x8e\xab%\xab\x8d\x9e\xfe\x0e\xed\xf4w+\xa7\xbf\xeb\xd6W\xfbt\x80j\x1b]\x8d\xb3Y\x9bv\\\x8c\xbe\xee\x94\xaf\xb0\xbf\x0bon\xf7\xfb'\xff\xfc\xde?\xef\x7f\x0f\xbd\xde\xfb\xb1\xb8\xb1\x9bPr|\xc0\xb5Q\xf5\xb4lJM\xeb\xa1$\xfc\xdd\xb8'\x8d\xfb\xb7u\xff\xde\x86/\xe0\x0b\x1b\xc0\xa7\xf0\x1a\xcf\xfe\xdd\x9eB/\x0f\x9f\xae\xebE7I/\x0f\xc7\xcb\x9b\xcbW\xfe\xe2\xde\xf3o\x1f\xe1\xfd\xdc\xfb\xe7\xdf>B#M?\xad\xb9n\xb6\x1bVt\x8e\xcaL\xb5\xfe\xee\xbf\xc7\x83\xe6z\x06e\x897\x91\x0eo42\x0e\xd2\xa1\x03\xa3\x94\xa3$R\xc6Q\xeet\x94\x12\x7f\x07e\x8eo\x98p\x1f)\xe1\xa8\x06\xa9\xdaQPS;.B`+\xda\xfd\x8e\x8ez\xa2\xe0w\r\x9e\xfcP\xd3`\x19\xc0\xa7E\x8ci\x91j\x18,T\xc8E4\x11\x05\xbd\xb7\xee\xb4\xb6\xd8_\xebp\xc2b\x0f-r\x12\xd4&\xb0\x12\x16)\x19\xb8\x8c\x91b\x18\n#\x1b\xa9\nGm\x03\xd71\xce\x08\xa8&\xcc\x9d\x98\x10\xa4\x90\xd7\x18W\xa9u\x14\xad\xa0\t\x98\r\xab\xeb)\x89e\xf4\x1e\xe22\xe9(\x18\x99\xc0\x9a2\xa2\xa0\xbf\n\xa9\x06)\xd6J\r8\x88\x8c\xcfH\x01\x1b\xe4\xe6^[\xe4\x83$RuT\xa6\x1d\xd5D5[\xa4\xda0\x16O)\xd6C\x83\xad\xb4\xc8\xafP\x99es\xa8p\x05\xe32\xfa]\x1b\xe6\x87sh\xb0\xf7\x16\xdf\x11\x9f\x91\xf1\xbf\x13\xe1M\xe3\xa8-\xb2%\x16\xb9\x1e\xb7\xd6\xb5A\xfe\x08~\xd7\xe0\xeeQH5\xac&0\x1dDi\xacY!U\x87}V\xc3{78jx\xd3T\xd3}c\x8c\x12ld5\xdf\xadu\x1d\xf5^cM\x85\x94\x0c{\x1ek\xb6\xbe\x15\x1b8\xee\xda\xb1d\xc4D\xd5\x12\xbf8\x89T\x1b\xbe\xbf\x1a\xee\x02\x15\x8eZ\"+\xd7\xb0\xb2@5\x8ejq\xee\x92Q\rR6\xac\xae\xff\x9d\xc5S\x83(h\x85\xde;\xef\xa1\xc5V\x14\xee\x90:\xa2\x1av\xf6\x10e\x91j\xdd\tV\xe1\xd7A\xe7\xe0\"\x1b\xf7\xfe\xd3\xb1\xdb\xeds>\x8e=\x8f\x18\xb9\xfc\xf9<\x03\x97\xb6M\x1c\\\xfa|\x8e\x85{\xff[\xc6\xc2\xbd\x7f\x96\xb1p\xef\x7f\xba-\xb0p50@\xd6\xfd\x95e\x1cS\xdd,Sg\x8d\xfb\xdbN\x7f[\x9d2x\xbd\x01\x06/a\xea\xd2ye+\xc6X\xba\xf7\xfd\x08\xd7E\x96\x0eG\xe8F\xde\xb5\xe9\xc8\xfb\x1e\xc6\x9f0vY'\x81\xb3s\x1c\\\xaf\x1d\x1f\xd7\xb9\x7f\x0b\xc6\xebm.py\x9b\xca\xfd\xd5\x8e\xbf3\x81\xe3\xc3\xe7\xad{\x02\xffV%\xeeo\xeb\xf8\xbb\xa9\xafV\xe8\x94\x07T\x06x\xc0\x84\xef\x9b&\xf3\xfb\xbeK\x98=\x98b\x970{\xd3:\xf2\xa7\x81\xd9\xeb\x1a\xf7w\x1bX\xbeNM\x7f\xfb.c\xf9\xfafz\xaf\x9bqu'\xc6\xcf\x8c\x7f{c&&p\xdc\x02\t\xfb\xf7\xeb0\x0e\xc3q;[\xb6\xdc\xc0\x02\xa6#\xbc\xa7\x11\xba\xfa5\x7f\xdd\x8c\x07\x94\x8e#\xeb2\x1e\xcd\x06NMV\t\x0f\x98\xf6t\xf6\x9b\xe0\xeb\xbe\xc8\x02\xcet\xa2M\xb1\x93\xc0\x02\xe2v\xcb6c\xb6\r\x81\x11L\xc7El\xa0\x1a\xd4\xf8\x03-\xd4z\xfak\xaa\x9c\x194U\xbf\xa5\x1a\xb6\xa97O\xc8\x12\xe2\xd1\x8b\x0c\x06\xc0\xb1\xd6\x1cH\xb5\xf6 \xdb:\x8a\x1f\xa8\x08\xff\xd2\x89\x01FJ`\xd9t\xbc*\xac\xe9\x04\x14\x1a\xe5Lx\xf4*\x7f\x10Ol\xa6\xc4\x9a\xc0\x82\xa2|\nYP\x92\xa18`\xd3\xd23;\xdaQ\xd4f^\x86\x12\xb0\x84\xe5\x8d\xa9\x18\x14R\xc0\x88\xc1\xa4\x8e I\xce\x02T\x0c^1\xb0\xc5\xa0\x17\x03b\x0c\x961\x90\xc6 \x1b\x03\xb0\xc4\xd9\x12\xdb\x00,\xbd\x0e,\x05\xad\x04R\xb8J\xf8;\xe9k\x02E\xfdqJ;\n\xe5_\xb5\xe6\xef\x16\x18\xa1\xf1\xbd\x13#\xa4\xc3\x9e\xa8\x9dD\x8d\xf6\x0b\xdbg\x8b\x90\xfb|\xb8\xeb\x8e\x0f\xa7\x0cr\xd9\xf3\x08rIh\xb2:lWy\x9dy\xf8M\xfb!\xf8M\x9f\xcf\xc1\xef\xf3_3\xf8}\xfe/\x19\xfc>\xff\xb9\x08\xbf6\x00\xacR\x01\xbeP\xa6b\x99\x8cb\x1e\x84\xbb>\x80\xed\xba\x0e\x92\x15\x04a\x07*\xeb&\x01\xe1tv\xb0\x86\xd9S\x07\xc2\xcfw\xc3\xdd\xe18>/\x03\xb1]\x1e}\x02\xc1\xcfw\xee\x98o\xc7\x16\x1fN\xa7]\x11\x8bA\xb2\xa2SD^o\x03.\xf7\xad\xfb+/\xe1\xb2p\x7f\xfb\x80\xc5\x88\xcb\x80\xd4\xa2\x84\xc5}\xc0\\Y\x07\x19\x0cbq]\x96\xc1\x0cp\xb7o\x8f\x87\xe3\xe1\xf4m^\x16\x93\xae0\xc03\xac\xf0\xb4\x1eEyL\x1b\xc0yX;\xa9L\xef\xa42&\x83\xe8u?\xc9\xd3\x00\x9c\xd7v\x9akA6\xf3\xb7\xdd\xf9\xe6\xf6\xe1t\xd7\xed7\t4?\xdfM\x0b-T=\x0ef\xfa\x87^'(\x8d[a\x9a\xdb\xb6\x8b\x86\x1b\xa0Z\x08\x87\x93\xda\xfd]\xbb\xbfC@T\xc7e*\xc7\xea\xe0\xdf\x19\x01N\xbaR\x00\xde\xcf\x87\xaf\xdd\xb7\xd58\xfa\xaf%\x00\x87\xaeAVs\xa1\xeb\xbe\x00\xe3\xb0\x95\x11\xb4/oh\x00\xf3t\x9c\x04\xe6\xa61#\xc3\xa0\xb6\xfd\xd8\x90\xd9N,D\xa3\xbbm\x01\xd2\xa3zVc\xbd'\x04\xf6\xf80\x8e\x0f\xea\xf8\x10\x8f\x0f\xf8\xf8\xf0\x97\tU\x07(\x87[\xa9\xff\x1dQM\xb8kj\xe5o\xac\xd2\xf5\x0e %\x90\xaa\x91\x12\x0e\x18\xe8V\n0\xc1\xef\xa1\x8d\x1f\x19P\nG]1\xc6\x05\xe6\x80\xb2\x106\xf7E\xb0\xf9yw\xfe\xf8\x10\xdd4\x08n\xa2\x92Y\xc0)\xd5\x9a\x87\x9c\xbc7\x02\x9d\xbcd\x0ev~~\x95\xc1\xce\xcf\xef2\xd8\xf9\xf9\xdd\xfb\x92\xe0^1H\xe9\xc2\xde\xbe\xf2\xd6\x87 \xe3\xee,\x9d\x0e \xe3\xce\xe1v\xdd\xa7\xc2}\xac\x19\x95&p4\xce\xbb{\x18\xf6\t\x1c\xe5\xab\x01\x804>\x7f\x98\xb9\x16\xc2\xd4\xe0\x13\x9e\x9bZ\xdf'\x98\x84\xed-`\x91C\x9b\xb5e(\xa4\x1fs;\xec\xc3-p\x83\xf7\xc2\x02\xf2 \xda\xb8u\xaau\xb8\x11VM\xd0\x01`\xe9\x16p)A\xa1q\x1e_\x00v\xf8\xe2\x00\xf0dk\xb6;\x17\x1f\x07\xd8\x19:w\x0f\xb4\xeeNX\xbb\x7f\xaf\xe1\xdf9\xec8&dSO\xd8\xd9\xeb\xe9\x1d\xac\xeb\xe9\x87\xd3\xbf\x13\xf0\x19;<<\xfc\xc1^\xf5==\xfe#\xbd\x12Nc[\xba\x13\x8a5\x83\x8f\xda\xfd\xbb.\xde\x03\xd3\xddu\xa6\xb7\xfe\xa5\xdb\x97q\xe4r\xcb\x0f\xc9\xd7\x04\x9b\xcdm{\x92\xa7p4i\xd8\xb64\t\x82\xb8\x81\xe4\xf8\xd1M\xcb\xdd6f\xfc\xa9^\xb7\xeb\x02r\xa45\x1e\x87\x19\x97\r\x18\x84qbQ\x83\xdaZa+G\xe8@\xa0\x08\x7f\xaa\xd8\xc6$\xfd\x0e\xb5\xaf\x8a\x84\xbd\xa0\xd0\xb5(jv\xd5'1h\x10 +P\xb9#|(\x83\xfas\x83\x14S\x7fc+xo\xca\xa8*\xe8q'1\xaf\xa3j\xa4\x14\xa3\xa0?\xfa]\x83e\xa0T\xd0\xa8\x96\x17H\xe9\xa0\xb1W\n)\x18\x99\x03(R/+\x89\xbf\x83\xb2\x1a\xb5\xcd5\xaa&\x98\xd2\x1e\x15\x15\xd2\xab-*f-`q%$*\x1c\xc0\xe0\xc0\xa0\xc2\x01\xd6\x8c\xa9I\x94\xf6\xbf\xab\x99\xaa\xc0D\"\x7f\x12{W\x11%\x0b\n\x8eT\xf9\xc1\x15#\xb1\xd2\xa4\xa4PA\xc1s\xac\x88\x89\x954\xb1\x02'Q\xee\xc4\x8a\x9fX)\x14+\x8cRe\x12W4\xc5J\xa8TA\xc5\x95W\xb1b+Vz\xa5\n\xb1zV\x8e\x90R\xb1\xfca^n\x11\xcb4Ry\x07\x97\x85\xc4r\x92T\x86\xc2\xe5+\xa9\xec%\x96\xcb0\x99M,\xcf\xc9d=\\\x0e\x14\xcb\x88\x0c\xb2a$\xc8\x9f\xfa3\xbeL:J#e\x1cE5\xa7\x1e\xd0\xc4\xa4v\xe6?\xba\xf5*\xa2\xcaQ\xba@\xb9\xdd\xda\xa0\xe9\x07\xfcn\xa4tN\x81\x94\xa6\x11~\xeeS\x19\xdaL\xc1\xca\x8fT\x1bTK\x8d\xe4\n\xb1Fy\x19\x98vT\xcb\xda\x0c\xab4\xb5\xd9D\xad\x18?\xa3\xa9\x15\xeb%?\xe3\xdemH\xf5\xe2V\xb7i\xbd\xbcJ8\xaa)P5Rm\xd8\x9f\r)4\x1d[Km*g\"\xd4\xe0\xd7\xa8\xa0\x15\\y\xa4\x1a\xff5V\x8e\xa2\xef]9\x8a\x94\x88m\xa8)\x1d\x83\xdf4\xfe[\xb1n\xeemPZ6t\xda\x00\xa5\xfc\x97Z;\x8a\x94\xa4\xb0\xba\x16)\xe1\xca\xf8\x1c\x94/\x83\xd5\x15x\x864\xaeL\xe3\xa8\x15\xab\xd9\xe2X4\x9e\xad0?\x8d\xe7.\xccO\x853\xb9\xc1\xab\x00\x9c\xd7~%\x9c:\xb5Ak2\xe5\xce\x89\x06Un\xca]u\x1a\xeb\xf1H:\x8a\xd0\xc2\x867=a\x96\xa3\x9a\x88\xaa\xd8\xef\x1a\x8f\\m\xd8/\xca\xf2w\xa4+\xde\xbb\x86\x91\xa1\xc9\x15\x18\xe35x\x9aj\x1aY\x8deP\xd3\xcdOk\xbe\xeb\xb4\xc1QC\x0f\xee\xf4\xf6m\xba\xef\xbdA\x1b\xbb\x06v\x08\x965\xb0C\xc8L\xaf\xc2}\xe6zo$RP\x06\xef\x1dwrcqdn,M\x8b3j\xb0\xac\x0e\xbb\xb5\xb1\xb8\xaf\xc1L\xab\xc1V\xa0M\x185\xee\xf9\xc6a\x9cAu#R\x02-\xf6\\\x7f\x06q\x1a\xfa3h1\xd7\xb8\xf9\x19\xe4\x12\x8cS}R\x99q\xd7Q\x83'\xbbq\xe7\xa7A.\xc1\xd4\xd8\xa6`5+_\x06_1\xb4I\xefH!U\x87=a\x04\xbew\x89\xbf\x83o\xac\x0em\xd2n5\xd1\x17g\x14\xb6\x02\xbf\xd3\xb8C\xa8L\x877fh\xe5i,6\xac\xae\x918\xce\x06k*7\x87\x06\xcb\x04\x9b\xadk\xc5 V\x19\xb7\x97\xfc*i\\\xcf\x16)(\xa3\x91I\xd7\nP\x8e_2\x88\xb7\xc6\xa9\xbc\xe9\xad\x18\x83+\x0f5[\xde\x8a\x15\xb8\xd6\xae?[a\xcd\x06)\xed\xde\x1fP\x0eo\r\n\x1e,\x8c\x13O\x14\xeb\xbeF_\xe6\xac=\r\x9e\x1a\xd6\x9d(\x06O\x14\xeb\xb8\x19\xa2\xda\n)0\x06tf\xac\x06O\xf6\xd6\xbdM\x83\xa7b\x0b3\xc23\xa4\x85=\x88\xe7`k\xb1\xcc\xad\xa7\xa8\x1c\xe0\x06\xd2!\xae\xc1/^T\xb5#\xf1\xf5\x8a\n\xde\x0c\xee\x19Q5\xee\xd5\x18\xfa-\xcc\x1291Q\x19\xfcmE$\x8c\x10\x18\xf1\xca\"\tM\t\xf7=\x07\xd2\xd9\x86\x18\xdc\x80B8\x1e\x88\xe6:\x95\xc2\xd4aT\x02\xf6<\x8ag\x84p\x07\x13-\xa8\xa8\x1cv\x19EM\xc1KST\xd9\x9d\xc4\xf4f\xa6\x96+^*\xf1\x9dJ\x1aF\x15^\xb1\x10\xee\xe4\x0c\xbf\xb5\xf8Z\xb1\xb4\xc5\xca\xb0\x1a {\xa2\xcf{\"\xa5#a\xbe\xb5;\x06\xfdok\x13u\x04`\x1c~\xebP\xdc g \xc0\x92\xc5\xd4\xa1\xd4\x84\xbd-\x80\x81\xf3w\x1c\xe0\xd9h\x07\x0b\x80O\xfa\xb8\x84\x84\x19\t\x1c\x150\xef\xf4\xd1\x08\xe0\xb4\rr\xa2\x02\x18h\x7f\xb3\x92\xf1\xa8T\x85\xa3\x82\x8eT\x8b$\x8cJW\xf8\x01BS\x1a\x96\x9d\x06\xa9a\xfaxh\n\xddbiK$\x1cq\x9aH\xe3\xce\x0e\x18sS\xe11\x03\xc3hj<\x9b\xa1\xa3F\xe11\x8e\x955?\xc7Ec\xf0P\xc2\xca\x80\xe7\xc8\xf1\tSaiEd\xc5\x9a20_2\x1e\xf6'\x1a^-5\x0e\xb2\xa5\x9bf\xc3\x06i+\xd6\xae#\xb4'\xbe\xe7$\xdcP\x89\xc3\x10\xc6 \x8b\xd1P\xa9\x0c\\\x0c\xden\x1b\xda\xbc\x06x8\xdaD\xa6E\x86\xb2!\xb2v\xa4\xe4-\x93\x99\xb7q\xfa<\xe2L\x05\x02\tm\x13S#\xdf\xea\x17\xc68\xa6V\x12)\x1d7\x0cSo`T\x15\xadq\x83\xa5\x96^\x88u\xa55\xbd\xae\xd6\x95\xe2\xeb\x02i\xaeo\n\xc4\xb9-Uvg\x95\xb6\x81\x14\x81{\xa7\xca\x86\xfa\x95(\xb6\x8d*\xe3\x81)\x1a\xb8\xaf\xd0\x01\x83\xa5\x8a\xfa\xad\xf0\xc6\"\x89\xac\xc2\xa5Hh\x8bw\xa4\x8a\xc8:\xa8B\x85vg\xbf\xf6[\xbb\xc1\xeb\x9c!2jJ\xe3\xa5F\x11\xc9n.\xd4\x14\x9d\xbe\x9e\xb4\xd4/\\W\x04\r\xb2\x0e\xf7\x95i\x82-{G\r\xbdPA\xcb\xce\xf7F\xe3\xce*\xbf\x19\x9a\x1a+7\xd42\xbb\xc3L\x9f\xa3d[E7X\x19\x07\xa9p\x18\x96f\x047%\xe8W\x83\xd4\xdd\xd2Z\xc1\xdd\xd3Re\x81[E\xd3\x89b\xd9\xdeP\x16\xf7\x06,\x1dH=\xfc|U\x8d\xfdVtVUn\xad\x04\x1d{\xf0~\xf1\xd8k\xb8*@\x00\x93O\xd7U<\x05\xe9.+@\xc4\xe07\x03\x1c\x8at\x9b\x15*\xba>O\xa4\n7\xf4\xe9\x8cl\xc2\xb5\x1f\x8fLr\x12\x10(v\xb0t6\x83\xdc\xc1D\xc3 [\xc3i\xcc`\x92\xe9g\xa4\x83W\x85@\x83MIMI\x14\x02yLhcR\xba\xca\xd8r\xc5eD\x02.\xe8\xe4w\xf0h\x12|\x16\xd4\"\xd9\x12p\x89\x98\x94\xc1sE\x80d\x80\\\\&\xb2\r\x96\x9e\xd3\x04\x05_\r\x89\xb6\x97\x82V\x03\xe4G5\x91\xcc_G\xa0\xa9\xab%X\x93h\xe5X\xd3\xe2\xe8 Q\xa2ax\xb2\xe2n(\x02-9\xe9\x98\x07y\x02\xb9\x9e\x08\x146TT\xd9\xbdA\xb2\xbc\x10\xa8\xf1\xa2\xef\x08\xcc2u\x1dH\xc1\x0e\x8a\x1a\x1cO\xd0\x1aB\xa0\x16\x8a^w-\xf0T\x81\xc5\xa9+\x14\x05Y\"M\xb0\xb9\x08d\xc5+\x8b\xc0\xd4\xc8 \xc4\n$,\xac0\xe8\xcb\x02M\t\xf2\x97j\x89]\xd2l\xfaB\xa3Y\xa6 \xdeJ\x05a\xd9\xc4\x97EM\t\x14\xba)b\xe2L0\x0cE\x16\xcf{\xf1T\x16;\x92\xc4x2\x8f\x9f\x89im\x83\xa9\xa8\xa84\xb7\x15\x9dx\xd8\x86\x97J\xf4\xf5\x11\xc4\x0e\xc3|5\x91\"\x08\xea&V\x1a\xde\xa0&\x92\x99\xbd\x00\xdbM\xeb\xda\x1a\x0e\x03-\x1c)8\x86\xd6\xa0\xe3\x90AV\x9ey\x0e\xb5*\xa1D\x18N+#\xaa\xc66-^\x1dL\x98\x96\xb5hIKW\x0e\x1b\x96\xcb\x1a\\\xcb:\\Uh\xed,\xbdA\x83\x94\x08kc\x15\xb7\xc0\xb5d\x81\xdb\"\xa5\x82\xfb\x94\x15X\xc6.Q\xb4\xed\x81\xdf\xa1\xdf\x19\x83&?\n/m:\x1c\xad\xa6\xc1\x83V\x87+#}\x00F&\x94\x0e\xe770+\x1a\x05?F \x05\xd7I\x120\xd6x!\x17\x01}A\x1c@:\xe0Fs\x88h4o\xb3!\x81f\x85\x14||\x06)\x13\x98\x83\x86\xbe5\x83\x14\xec\x10\x15D\x1a\xb4_\x1a\xc1\xcd\xb9\x1arT\xb3A0\xc2\xa8&\xec\x02\xe4\x02p\xe5c\xe6Ck\xdc\x13M\x10\xd9\xf8\xb2\x1a\xcd\xb9\x0c\x8azl\xa0\x94\xe5\xbfS\xb1?\x83A\x08\xd3A\xecD_\xad2(*'\x91\x94\x0e\xe8\xa6\x0c\x8a\xedM\x1041\xca\x04\x01?\xf6\x87\">\xa6\xa0A\x95M\xeb\xcez+\xf8E\xd8\xe2\xba\xb5\xee$\xb0t\xe2VpQ\xc4\xcd\xdd\xc2u\x06\x99\x0f\xf8@\r\xf2\x1e\xad;\xab\x0c\xb2\x0f-\xdc\xddpZ\xadC\x04\x8b\x9fA\xeb6\x94\xc5\xcf\x80\r\x06\x877\x9d\x19\xd6\xfd\xd8\x1f?\xca\xf5c\x88\x14\x8e\x948$\xb8n\xb7t7\x87~kv\xfa\x84\x96B\xc3\xd4\x17\xf0&$\x06\x98\xb8\x0f\xcdI\x8d\x97j8\xcb\x95\xf3W4\x8a.ltO\xb6\x8cc2\x8c'\xaa\xd8\r\x1c/\x8a\x9a\xb0:\xf4KCQ\x16\xe5\x1a\x92\xee\x7f&\x086&^\x0e:\xf3\xbc\\\xc5\x87\xd2\xe0@k\"\x95#5U\xa6\x96\xfd\xbcI\x94\xe6\xaf\xa2\x96\xdd\xd3\x15]\xf9a\xb0\x8a\xee\xf8\x9aH\xb8k\xd7qe\xcf\xa2UL \xc0:\"\x8d\xa1;\xa0m\xe5\xa5\xb0@\x91vOF\x14\xec\x1b\x1d62\x95i\xd7\x8d\xc5cP\x0b\xaci\x90\x12\xd1\xef\xa0\x07\xd2P\xb6a\xf7)\x8beu\xd0I\xda\xca\xcbyuT&\xd8\xa8\xc3\x1cpV\x89\xa9L\x85G/,B\x8be\n\xa9\x96\x19\xce\xc4\xa62\xb1\x19Mjb\x13\x9b\xdfP\x7f8\x02Mo\xc9\xe0\xd1\xd5\x86\xdd\xa8Aj\x81\x9bI\x1b\x14\x0e5xT\x82\x98I\xa2\xf4W\x07aW\xd3`\x99\r\x07<I\xbe\x1a\x92\x92\xe9\xa8\xac\xc6V\xa0?\x13\x8ef\xda\xd9p\xfb3\xb86\xda\xf0\xed\x02\xd79\x92\x90h\x92\xb5h\xac\xa9\x83\xd4X7\xa8\x98n\x82l\x9d\xa4#\xe0fO\xf2\x1d-\xa2V\x04\x17\x05\xe9\x9a\xcb\\t\x8d\xbd[<\xee\x99\xc0\x89\xad\xae\xff`\xc1K\x9e\x04T\xca\x9dy\x04U\x13Y3\xeeT\x91\xe3\xb3\xa1\xa3\xc4\x04O\x1d\xac\xac\xd8]\xac\tjX\xfc\xfeT\x13\x9a\xd2\x8cWW5w\x04\xc3\xcf\x91|\xbf\xa6\x13\xad\xe5\xbf\xad#n\x1cK\xe9:\x89\xbf%\xe9\x0e^\tEh\x8a\x99,O\xa4a<3\x8cY\xb3#\xa4\xe6\xb7\xbc:\xfa\x04\xd8\xd2\xe1j\xde\x08\x94~\xe3\x9d\xe3f\x12\x08UAS\xe5\xca\x99\xfc\xfd\x86$.\xe4\xfa.\x8c\xe0\x9a\x98\x1b\x01\xda\x02\xd2\x82\xdd\x08`]H\xbft#@+@\xda.G3\xdd\xd4\xcd$\xc2\xa9\x82V\xcb\xd1mD+\x944\xd1\xf8\x1a\xac\xdfz\xba\x8e\xda'M\x18\xb5\x0fZ\xbaPnq|\xc6\xd3&h\xb9n&\xf9S\x1btu7$\xad\xf2\xbf\xe7R\xb1\x9bH`\xe6\xfan\x82~)Z\xeb\xb0\xfe\x16\xbf\xa16\xb4@_\xd4Md\"Bc!\xe1\xa4\xa3\xab\x886\xf8\xcdIO3{\x93\xa8/\xec?s|\xe4N\x91\xb1\xc3d\xeaL\xc9\x1d-c'\xcc\x94\xd2\xcc\x96\xa2AJ#%\xc3mX\x82K\x9f\xf2\x1a\xda\xd6Q\x8a\xb9d\xe2[G1\x01\xae:\xde|\xb5\xb7'hX\x0f\x12\xa96q\xf3\\\xb6_\xbc\x8dm\x87\xc9x\x11\x1e\xefW\xffo\xd8\x0c\xc7\xce\xc7;\x89\xac\x18\x97\xaa\xcc\x9b0&\x1d\x92\xfdb\xb9\xb5EC\xc6\xdb\xb7\x99!\xe3\xed\xfb\xcc\x90\xf1\xf6\xfd\xdb\x82!\xe3\xa3\xec\xe4y\xd4\x81us\xa5a\xe24\xa1\xc8P\x9a\xafZb\x96x;c!\x9f\x8f\x12\x8d\xc2\xd2\xb8\x03\xb7s\x16\xf1\xddcl\xdec\x0bwfyX\xf2<[\xb4j\xffu\xd9\x96=\x9e\xef\xee\\X\x18fM\xb8\x01\x93\xf59\x0bB3L\xb6\x83\xf5\xa6/Z\r\x96\xcd\xd5o\x0f\xa7,\x94\xc0\xed\xa2mz]1\xa3\xf0\xcb6\xe8\xb7_\x93\xb7\x7f\xa6\xf7T4\x1b\xcc[_\x08\x1b\xb0hT\xfe\xe6C\xc1;\xec\xa2)\xf9\xff\xa2\x11\xb9 \xbb*\x94/\xe3\xe5\x145\x02( 1\x11\x85j#\xbc\xe2\xa2v\x00/\xd8\xa8\x0c\xe1A^<\x85\xfa\x1a\x14\x82\x10\xc5eXtQ\xd6\xa1\xa6\xe7\x89l\xc4!E\xa1\x85\xbcP\x8dS\xc4\xd3\x90\x937\xeaeP`LT\x15\xc2\xde\x88&\xaa\xd9`\x19\xaa\x1dP\xe0\x84\x9cP\xd4\x9fB\x87l\xbc\xf7\xa1|\x13\xefyQ\xf0\x9c\xc8![\xd4\xd1J\x900J\x86\xdf\xf9\xf5\x94\x91\x18\x94DZm\xe8\xdd\xbf?\xf2*C\xdd\x89\x7f\xb7\x8b \xf3\xe2\xdb\xd0\x7f,\xa0\x8c{^\nZ\x03g\xe5l1\x1czw\xdd\xea\x16|g\\\x85\xffx\x18V\xe7\x1e\xbci\xecH|\x97AO:\x0c\xc2\x9e\xf49\x00\xce\xf4\xad\x89\xaa\x1fN\xe3I\x13\x03\xcf\x0f\x7f\xcf\x80\xe7\xc5\xdf3\xe0y\xf1\xf7\x1fJ\x8e[&\xb5\xfc\x95\xcc\xa6^\xc9\x02\xfc\xb4]p\xd3j\xf5\xca!O\x97\xbaL\xb7\xdcx^\xa5\xf1q\xe0\t\xd5O\x90\xea\xdd\xa9\xff8-@fE\x9f\xaf?\xc7\xac\xe1\x1e\x0e\xcc\xa9\xac[\xbd\x18\x9b(:y\x01`i\x80-vp\xa1\x9b\xeb4\x9d\xc66\xe0\xff\x9a\xa0\xda;|\x9b\xed\x7f<\x94\xc1M\x05XC\xa7/s1\xa8\xce\xffq\x88\xb6\xbe\x80n\xa2\x0b\xbe\\\x02\xd6\\\x803\xb5LC\xec\x08\x16z\xa7\xde&@\xf8\xe24|\\\xc0AX\xdaO\xab\x17\x03l2\x8e\x87S\x19\xae-_\xda\x00\x8c\x93\xef\xc4\x08\x89\x0e\x18\xd72w\xba\xee\xd7\xe3\xfa\xf4[\x9dB\xe2\xbb\xd3\xef\xd3\xebvCZ'\xc0\xe8^\xf9\xd7\x04\x19_\x84\xd7P\x84F\x13\xdc\xb6\x94^\x00\xc5wc\xbf\x9fv\x999\xfd\xd4xo\x86q\x15\x1e>\x7f|8\xee\xbe\x0e\xe7]\t#\xb3n\x160\xd22\xb3\xf9\xae]\xa5;\xb1\x17\xf9N\x84\xe7\t\x9c\xbe\xf8\xb7\x071l\xda\xfd\x87\x95[\x83m\xdd\xad\xde\xc2b\xf4y,\x1ei\x0c\xb8X\x17b\xf1\xf82\xe7\xa0=v\xa9\x95\xed\x9f<:\x0f\x1e\xdd\xfe\x8e\x8c2\xf8\x9a\x1d\xeb\xd2\x83\n\xa8A\xf1&\x8eR\xf0\xf8\x907\xc9!\x9f\x03@\n\x0e\x1c8bP\x89\x01'\x06\xa3\x14\xa88\x88\xc5\x00\x97\x82_\xcd\xb4\xb35\xc22\x038\xafQ\xac1j\n\xf5\xde\x04\xd96\x98\x1fyJ`\xe4\x17T Ez\xc9\x8aG\x85i\xb1\xc9&D\x82\xc3_Y\xae\x9f$c\xf8\xa6\x10\x96\x90\x05,\x8c\xf8\x8d(\xc8a\x12\xfe\xb0\x0e\x92\xfe\x86k\xef\x1a\xaekj\xf0\x95a\xb5\x86\x136h\xbf\r\xee\x08l\xa0e\n\x1b,\xc1\xb8\x86L\x0ce\xb1D\xfb0w\xa4f\x01\xb7uE\x01\xf0t\x903\x82\x8a\x12e\xc9d3^\x05\x82D\xda\xa0\xa7!\x19\x0e\x898iWTL\xdcS\xa3>I\x87]A\x1a$\xf6\r,2%?\x0e#/\xb1\xff\x9a1%\xecy\x1cb\xcf\xddL\xefV\xdc{/\xaf;\x7f\xf1M\xfb#\xee#}>w\xdd\xfd\xf1\x97\x8c\xebx\xf9C\xc6u\xbc\xfc\xe1\xfd|\xb4\x16\x15N\xc0+=\xf68[\x81n\xe1\xc0P\x0c\xb9\xaf^\xc2V\xbc\x1c\x1e\xce#gQ\xb8\x03\xa73\x06\x86\xe2\xd9\xdd\xf0y.dK=7\xfc\x84cxv75\xf2a\xbf\x10\x82O\xb3\x10-\xfdc\\\xf1jvE\x1eXX\x96\x05&\xa2g\x17e\xe0\x0fxP\xbe*\xe1\x15\xde\x00\xb0\x8c'\xfe<\xc7\xf0On\xe9\x12V\x01\xd6\xb3xy\xeey`\x16\x95_\x9e\x9d;boL\x9f\xf0\x08/\x1fv\xe7\xd2\xc5\xf9\xcdn\xf8\xdc\xa7\xfc\x01\xbc\xb9\x8fEO;\xc9@\x1b.\xb9}\x1a'o\x8e[\xf8z\xfaT\xb8B?\xbb\xfb<\xe7{\xb7a>\xdc=\xeb\xd7\x84\xbf3Q_\x98'w\xcf\xae\xd8\xba\xe8\xfa\xf9\x85^\xb0\x10\xd5\xc4\x1e\x94x\x01\xbd\x9d\x04\x16\xe5\xb8|X\xf6\xb4\xc8\xbf\xe8\xfe\x12;\xca\xc4N4\xa9\x83\rw\xbe\x01m?QNY\xe2k\x82\xbe\x9db\x8d)\xfc\x9d\rQ\xc9\xb4\xf5\xb1\xbf\x84\xa3$\x8b\x12f}\xe8\x11\x19,\xc60dI\xeb\xfb\x03\xc3\xc3&DO\xd3\xadw\x8d)\xcfv\xf1\xc4\xff\xe1t>\x94<\xb5\xd9\xf3Y?\xed\xbc\xce\xfcI\x9f\xf6C'}\xfa|\xee\xa4\xff\xe1Mv\xd2\xff\x90\x9f\xf4?\xbc}W:\xe95\x0b\xa7\xca\xdc\x98/G\xe4\x02af~i\\?\xca\x1f{\x9cc\xe1\xc4Og\x0e'~\xfa4\x8d\xd157\x8b\xcc\r\x1b\x1a*\x1e\xfa\x9a]\x11\x99\x17\xf6\x95q\xb9P\n\xca\xe5\xa2\x97\x0e\xfd\n<\xacY\xdcU\x011\xba\xd6W\xfab\xbb\xebP{\xfa\xb6\x80\x02\xe9\xc2\xed\x8a\xdb\x8b\x85\xea\xb2\x10\x8b5\xb8e\xcf:d\xf7n\xe6S\xec\x8f\x7f{\xd8\x88\xaeZp\xc8N_\xf5\xfd]i\x14\xf746w\x7f\x923A\xba6L\xaeZ\x07Y\xe8\"@\xa4\xfd\x9f\xc3^(\xe3\x83^\xead\xe12yi7\x16}\xb3\xd3u\xf0\xde\xd9\xf5\x04.M\xdf\x8eoC\xf7\x13N\xa9\xa1\xed\n7F1\xf9\x92'\xf5\x86\xa7D\x8c\xd4\xd12v\xc2\xe4\x0e\x9a\xa9\xf3&\x0f\xca\x14\xc7\x08\x89ch\xc4\xf15\xe2\xd8\x1bq\\\x8e8fG\x1c\xcf\x83(\x8bT\x13,\xae!\x02\xa5\xc7\x1d\x893\xaa\x18\xee\x98\"B\xc5\xe8\x15#[\x8cza\x95\x96\xe3\x80t\xe7/\xa5  \xfe\xf1|\x04\x90\xb4\xcaB\xf8\x8f\xb8\x13\x1f\xfb#~<\x1b\xf8\xe3\xc7<\xf0\xc7_\xf3\xc0\x1f\x7f}6\x1b\xf8\x03>\x04\xfc\x04\xc4u\xc1>\xb8DrsmP\x8f\xe1\\B\x91d\x9e\x18\xd2c\x98\x03\x11.d]\x84\x8f\xa9\xbb\xc3\xferD)\xf5\x18\xfc\xe8\x19r\\\xd4\xa5\xb1hQ.\x9at\xeb8\xd7\x0b1;\xa6Q\xcfBD\xb2V\x18\xb3\x03&Z\xd4\xb21p\xd8\xc8Y]\x9bq\xd1:&$]\x0f\xd3\xfa\x96\xe2t\x0c%Xp\xd1\xa1\xc4\xa1\x10\xabcXD\x06\x0c\xa8Qe\x015\xca\x98\x90\xed\x9c\xf3\xd1o\x92\xa5\x80\x1dY\xfbKa\xbb\xdd\xbe2m\x1a\xaa#\x0f\xcf\x11\xbf\x03/'\xac\xa7[\x99\x15\x13\x0e\x98a\xabf\x11\x00\xebM\x11\xde\xcdf\xfax\xfe\x04\x02\\\x11\xabC\xa3\r\xa3\t\n:2Q\x03\x05\x16\x19V\n\x8b\xc6\x8f\x06Ob\x1d\x0c2\xd1\xd5\x8e\x94T`\x82e\xbc\x1a\xacqT\x13Td\xc6\xbb\xd0\xa0\x91\x17\xda8:3A\xf2\xa7!gF\x13\xa4}\xc6\x92_ Rm\x90\xcc\xf99H4\x00E\xc7\x04\xd7J\x98\x1f\xcd\x96\x0cC\"\xe7\xf9\xd8\xcd>s\xc1\xaf\x82C\x19\xdc\x9cL\xc5#8\xfb2\xf25\x13\x88=&85\xd7\x91\x9bv]\xa3\xf35\xa1[\x1d\x9c\xd2\xea*\xea\x9d\x9c\xddD0\xf2#Gb\xe1n@\xd4\x830\xd8C\x85\x8aF\x1bQ\"8\x0b\xc3j\x90\x931\xc8\xd1\x1aoo\xca\xdd\xf3}Y\x15$\xa8\xde\xcdM\xa0\x97\x9b\x08\x92:\xef}T\xa3\xdb\x1aIz\xa5sT\xa2\xdf\xe9p\xa7C\xe9_\xf4F\xe9\x86G\x8eT\xd2'S!\xaf*0\xa0\xc6\x89k\xbcM\x82\x01\x9f\x1b\x06\xd9\xe1\xb9\x11\x92\x9b\xb0\x1b\x92\xe4\"^\xef\xcfW\xa3\xcd\x13\xe9>m\xf0\xa5\x16\r\xbe\xaa\x06_\x00\x7f\xe1\x02\xfd\xa5\xe9\x92[\xb1\xad\xa1\xd0\x85\xb3\xc2\x8d\xa2\x18E\xfe\x9c,\n\xa9\t\xb1(X+l\x9b\x92E\xa8\xbb\xc8\xd3\xc7\t\xd7e\x13\x82qW\xc1>\xba\xaep\xd37a\xeb\x90\xcd5~\xaa\xde-\x12\xdb$+S\x1dlRk\x81\x96\xad\xc4z1Cnp\x9e\xf5ead4V\xfa\x94\xdb\xc0\xb6\xd1g\x0e\xee.\xf4\x99\xe3JR\xcd\nm\xc0e\xe8\xd3X?G\xe9(\xf6\x91\xb9\xdf-sjo\x0e\xc7?\xba\\\xec\x1b\x1eG\x9c\xdaO\xbb\xfd\x87\xcd\xe1\xf3\xc4\xa8e5\xe6\x19\xb5\xa4\x0fb\xd4\x92\xc7s\x8c\xdaM\xdbf\x9c\xda\x9b_3N\xed\xcd\xaf\xb7\x17\x14\xcc\x98N\x82)\xbb\x90_\xebK\xd1@-\xd3&\xcb\x94G\xebM\xc2\xa3Ms\x19>d:\xe3d\x8a\xc0\xa3\x8d\x0f\x1f\x86\x0fE\x1e-\x1b!\x8b\x08\r\xdc\xd9\xf8c\x88Mf?\x149\xb4M\x1a\xe1\xd31\x9er\xedT\xc1\xeb\x8b6O<\xfe6\xbb\xe7\xe3\xdf\xfa\x02\xe7\x86\xa2\xdc6\x15\xf4\x02\xff\xc6\xec\xb4P\xc4Ksi?dq\xd6`A3\x9em\xfaI\xf4\x98\tw\x87\x10\xd8\x13\xa3q\x1b\x08\xf2\x99sn\xc2L<[;E\xdd\xee\x1b\x17\xde\xd3\x98\x94s{sH\xdf)\n}\xd3\x91\xdd\x9fKo\x951m\x9b4\x82&2m\xa0\x1d\x96\x0b\x0c\xdc\xd8\xe8\x874\xf3\x8a[\x02\xc8\xf22+\xf6]\xeel\x89\x9bk\xca\xe1:7ujY\xd5\xad\xde\x7f-(}\xa7\x0b\x86\xee*QR\xfa\xfa\xb2'T\xf1\xc60\x19Ch\x0c\xaf)\xf4rXN!\x9b\xc3y\x0c\xf5e6\xc0\x16\xd8\x87\x98\xb5\x88\xd9\x8e\x98%\x89\xd9\x95\x88\x95\x89\xd9\x9c\x84\x05\xaa\xf9\xef \xb0(E/\x02\xff\xcb&\x12/4QHP\xb2|F&+\x84\x12Ul~\x10#HzF\xca:\xb6\xc1\x04\xc6\xd5\xaf\xa7A7ybx-[y\x83\xc2j\xc6\x80\x91\xc0\x1d\xb8\x08\x12K\x08\xf2'\x17L\r\xdfD\n{\xcd\xd5\xdb\x8cj\x82\xc0D\x88\xc8\r=\xec\x90E |\xfb\xc7\xb0a\xdf:\x01ax<\x07\x84Y\x8dy L\xfa  L\x1e\xcf\x01\xe1\xdb\xbfe8\xf86\x17\x84\xbf\xfd\xdb%C+)3\x04\xdc\x96$\x166\x84$\x85\x04\x15\xce|v.\x1d\xc5\xdb\xfec<\rX\xa8dn\x80\x80o\x1f\x86r\x821>\xb4\xbaI\x86\x06\x00\xf8\x16bY\xdaM\xd1\x06Jf\xa1E\x01\xf8\xd4%\xe0k3\xb0\x93\x97\xc4\x146\x038\x07y\xb5(\xa6\x99x\x8bP\xb7\x99\x97RLk\x95!\xde\xdb/\xc3\xb7\"\xde\x81\x00;F:\x87\x83\xf0\xef\x82\t\x94\x13c\x03\xde\xade\x86w\x7f\x8f\xdf\x14\x80\xdd\xdbo\x7f\x0cy\x9e1|\x07m_\x16_\xdb\x14v\x163\x8c\x8d\x13<\xee2\x94\x9b$$\xb2:- \xdd\\7K\xb6\xc22\x95RX[\xcc,\xf1\xee\xa3Se\xea\xaf+Pl\xee&\xb2\x97\xfb\\~\xad\xf4\xe0l\x98l!\xaeh({R\xcd&\x87\x80(D^\x12>/\r\xad\x17\x87\xdd\x8bC\xf2\xb1p}I(\xbf8\xcc_\x1c\x020\n\x0f\xb8\x14H0\xa1\x1a\x04f\x15.\x84\x04\x16p\xa9\xa1x\x1cu\x1d\xd5\x14H1\x89\x0b\xc5|\xa9id\r+\x93\xde\xcdO\x85\xf0y59\x12\xb5\xe1\xaaF\x01\xf3 i\x049\r%\xb0\x99B*\x87\xdb\x18\x8ac\x98\x8e\xa5\x18\xb1\x84#\x96~\xc4\x92\x91Xj\x12KTbiK,\x89\t;d9\xa5\xe6n\x1f\xc9\x06}N\xcd\xf0|VJ\x9f\xd7YH\xac\x99\xf4\xe33k&\xcfgSk\xbe\xceSk\xbe\xceSk\xbe~3\x9bZ\x13\xb5\xbd\xec\x12\xe8\x84\xc8K\t6Q,o\x83\xe8\x1e\xb5\xc0\xdbr\xca\xcdN\xa4)7w\xfb}Ah\x9fN\x1a3n\xc2\xd3\x85\x9c\x9b|\n\xdcX)\x83Gl\xaa\x08\x8e\r\x8b\xc1\xbda\xb97\x99\xc9\xcfe\xa0TY&\xa6\x9e\xfd[^\xc8\xc9\x892|\x1b\xe4\xfc\xee4FCb4\x0b\x12iV\x88q-\xe7Q3_\xbc\xdd\xb9\xfc<M\xcd\xd9\xc9\x90\x9a\x93\xa7\xe9\xecKi:'p_\x1b;\xfdU\x9b\xd4@(}\xad\xf7w\xe5\x11\xdc\xfb\x91a@\xee\xcdn>Sgd\x9d\xb3\x06PcO\xd4R\xd6\xced<g\xbe7v_\xe7\xf3v\xa2\xbd\xd0b\x9f\x17sx\xaaf\x95\xc6\x90\x87\xddjm!\x95\xa7\x18:\xb3_\xfd\xdc\xe5\xe0j\xeb\xae\x9f\xcb\xe0\x04eO\x9c\xbb)7!\r\xc6\xa5\xf5L\x82\xe4$u\xb2\xe5\rPk\xcb\xba\xd2\x87\xff\x1c>\xaf\x0f\x0f\xc7\x0f\xb9\xbe4*\x8aN\xe3\x1f\x8f\xe3\x9b\\\xbd|\xe8?~uj\xd3R\xcd\x05\xd5i\xde\xa7W\x9f\xe6E\xb3*\xd4\xf7\xb9\n5w9\xfc\xf9\xfd?\xcf\xe7N\xe0\x19\xf3$3h\xb4\xa9E\xba\xecf\x95\xaapE\x89\xf2\xe6YvQ\x19Xr\x1f\x91Zn\xb6\xa9 \x0f\xa6\xcfg\x8f\xda\xd6|YP\xe3J?8\xcc*]\xa3\x84f5;\xc2g\xe6\xda\xa5\x87z\xa9\xf3T)\xcb\xef<CP\xd3\xa2Ug\xcd\xeal\x98p\x10\xefK\xf3*[\x9e\x8aO\x04kOT\xe2rAa=\xab\xca\xe5\xa9\xf8\xd0\n\xb4g\xf6\x9f\x99]\xa8H\x05\x85\xf9KA\xb5\xee\xc3\xa7SZ\x80\x9b\xf8tJ\xdf\n\xd3\xee\xaeC\xae\xbe\xf5:\x9c\xfb\x9b\x86\x97\xe6RC;\x81D/\xa6W\xb5\xde8$p\x12\xc4)\x93_\xaa\xf5\xcd\x06\x8c\xe9\x19\xf2\x01\xdf\x9fgvQ\xaa\xf5U\x03\xcb\xaa\xc7s\x03\x81q\xa7`\xa9y@\xee'Smp6(T\x07;\x8ds\x9f\x8d,U\x0b\xff\x0f\x06\x90\xaa\x8b\xf1[\xc8\xbe\xf8l\xff\xa3\xea8\xdf\xff\xde\xb4T6\xe3\x1ek\xec0]\xb5l-J\x89\x81\xa6\xed\xcbj<%^d^\x06\x15)\xe2,'\x18DD\xf1\xb2\x14\xfa9\xc2\xe5\x8a\xbbqR\xe8,`\xe9\xd1\xf9\x13\t\x1d\x9cL$z\x9c\xc8e\x82\xf2\xddI\xaf\n\xa4\xa0`\x9a;\xb0\x84\xf9\\J)\xf7a\xf7\xf0\xb9\x94R\x8e\x9e\xcf\t\xcb\xf2*\x8b\x19\xe5\xa2nXF\xb9\xe8\xf9lF\xb9\x1f\xf2\x8cr\xb9\xbc\xec\xf9\x0f?\xcfg\x94S\xccU\xaa\x0bPue\xfe8t\x17\xd8\xce$\xf5\x19\xf2\\r\x1fr\xb7\xc3t\xb6\x98K\x0e\xb3\xbf}\xd8\xf5\x97\x92\xc9\xb5\x96;\x7f\xa5i\xe4\xa6\xd6\xb9{(\xc3\x15\x87\x19\x1d\xbf(Hv\x81\xd8^J\x13wQY\x94%\x88\x8b\x10\x82\x19\xfb\xd4\xa9\xb1O\xba&>'\\\xae0r\x0f\x0f\xf3\xd9\xe0\xe0\xb0\x1f\xe6\x14E.\x0f\\7L\xe8\xd4\xf3lCp\xcc\xbb\xc6\x8b\x9e\x83\xc9X0\xeb[\xe9\xa5-%|S\xfdE\rQ\xbai(\xbd\x1b\\0\xcc\xee\xd3|~\xb7\xb9\x9e\x96l?\xb3\xcf\xa2fV\x9fJ\xa79\xdd\x9c\xd8\xac\xcd\xd5D\xeb\xcd\xd8\x84\xae\xebvr\xb0\xdf6\x05e\x11\xaf\xd1\xa8\xfa)%h\x90F\xa6\xe6Jt\x15\xd2}B\xba\x1b\xc9\xf2{\x07\n\x12\xe3\x08\x96%\\x\x19\xcft\xc4W>\xc8T\xe5(\x1b\xa4A\xbe&\xf8\xd0\xe1\xe9\xeb\xcb\xda\"\xc5\x12\xea\xd4\x15O\xa1\x03j\x1a\nc\nV\xa0\x8cj\xa3\xf9\xd1l/\xd9\x01\x9c?\xae~\xe9\xfaaS\xf4\n(\x94\xcfJ\x87\xe6\xeb.\xda\x08\x14\xfbg\xc6\x02\xc5\xf2\xb9\xc3\xff\x97\x9f\xb2\xc3\x9f=\x92\xfe\xd1\xcb\xd2\xdd\x04<]\xea )*y\r\xac2\xbdJ\xee\x91\xc5\x13\xc4\xb5%7\x836\xf5G\x87\x7f\xaf\xabU\xb8\xbb \xa4p\x9f\x03\xf1'|\x11\xc6\x15\xdc|\xee\xbe\xb9\xf5+\x19*\x14\x97\x17\xd0\xc6?_m\x86;\xf7z\x87\xd2egq\xd9(V\xcbji\xa92c\xd4_\x1c\xaa:\xb5\xc9n?\xac6\x0fS\xef\x9bY\xb4r\x9d\xd0\xddF3\xb4\x12+W\x89\xbb\xb8u\xac\xd2\xa3lY\x87p%B%Q\xc1\x17b\x05\xff\x03/{&8\xcb\xaa\x0e]\xc9m\xa2\r\xc6\xaf(\x14\x03CX\xd8\x17pq\x02\xc9\x99\xe4>\x14\xe2O\xf9Vt\x9fn\xdc\x1b\xce-O\x13\xc7\xfc_\xbaO\xb8\x0f\xde\x9f\xbbc\n\xb5\xd9.\xd9d\xb0\xab\xa5p\xda\xa9\x8d\x03\xdc\xb5\x08Ny\xf39\xf16VL7-\xe7v\xb1\xad*\xe7\xa6\xb7\x94\x19o\xb2\xdb\xd8\xdc\xd0`\x8a\xe8\x1cF\xfa\x1b\xcc\x19\xb2\xb5\xee\x0f}\x96O=^\x98\xd5\xe6\x90l\x7f&\xb4\xe3\xa0Zg\x8e~\x1bfY\xbb]9\x14\xe6\xbe\x7f\x1d\xfb\xb1\xba\xd2\xe5\x03\xbe\xe8O\xe9\x17\r,\xc0O\x0f\xdf\x86\xaf\xf4\xc6\xca\xc6\xbe[&\xf1\xab3\xdf\x8f(Y\xfb#\x07\xbc\xa4\x83\x9b\xfd\xf4W\xd7\x1e\"Y\\\x02\xc7gt\xdb\xbep\x80\x85{\xe2\xb4\xf3\xda\xae\x01\x89\xe2\xd8ZkM!\xf6\x0f\xd6k;VO\xc8\xa7d?b\xe6\x80\xa8\x86A~\xc5\r^U\xe5\x19\x87\xf1r&[_\xb3r\x94\x89\xa8\x16\xad6\x94\xfb\x1dE\xc3\x94\x8e2\xacLx\xfb\x0e`F\xe2\xb2\xb6HI\x167\x1dr\xd3\xd5\xfew6\xe4\xf3\x03\xebJ\x8a\xce^W\xf8;fA\xe9\x19\x95e\xe6\xa7\xc4&\x89\x84\xbdZdi\x9e\xdd\xad\x8b\xa9h\xd9\xf3Y\x16&\xaf3\xcf\xba\xa4\xfd\x10\xcb\x92>\x9fcU\x9e\xfd\x9c\xb1*\xec\x91\xf4\x8f\x9e_pe\xaf\x1f\x93|\xd66\xe1\x9e\xcao\xaeW\xb3\x130\xbb\x8c\x91H'M\xde\xec\xeb+\x9c\xd9\xeb\xcb)f\xb1\xd7\x12\x03 \xd8E\xb5~\x8c+;\xd3b\xc1\x05\xf6\xb2W\xa3N/\xad(\xc4\x1c\xaeD\xdeq\x1as\tf\xd3\xa5\xda\x15\xf7\x12\xf3f\xac\xc3=v\x9a\xca\xa27\xe3t\xcd[\xf7b\t>\xa1\xa7\"l\xa6\x83 \x7f\xf7\xf5bv\xd9\x8ae\x11\x17Wz\xbbg\x9b\x0b\xbd\xdd\x8f\xfb\xee\xcb\xc3\xf9n\xc4\xb0\x02\x9c-vt\xd1\xa1q~\x0b\x16\xbdY\xd2\xb5 \x9ci\x9b\xe9\xc5k\xb7\xc6z3\xb9\xb3,\xb95BmSO\x1b'\xa9\xfd\xa4\xce\x8d\xa5\xb35\x9c\xc2\xf1\t]:\xbdm\xf4;~^K\xdf\x8a\r):\xf0\x84\x96\xfe\xbc\x16\x11%Y+t\x91m\xa3\x8b,\x99t\xd8\x882!7\x07\x18x\xf8Q\xa7\x17`\x15\xcd\x8f_\x9c\xd3K5\xad\xcb\"\x92\xfct8\x1d\xbe\x1c2 \t\x8fgq$\xab2\x0f#I'\x84\"\xc9\xe39\x10\xf9\xe9\xaf\x19\x88\xfcs~\xdfE[\xfa\xf4\xbe+\xb3\xf0\x9f\x96ism)\x12\x8afz\xb6upg\xec\xd7\tL$\xc3G\x81p\xf2\xf0Tx\x18 \xa2c\xbee\xa0>\xe3\x89\xa1Uj!\x9f6\x1e\x10\xa2cJ0\xc9\xfe\x82?c\xc3J+&\xfaT\x17\x84\x9ex\x13\xe4qB5W\x91\x95B\xa1\xe8pOC3\xc253#\\\xff\xf9\x08k?M\x91F\xbf\xc0}%\x01\x96d]v\xa5\xdd\xc5\xac\x0b\x9dFl\xed\xa2\x8dn\xfa$L\x17\x0f\xe4\xe1\xc1e\xab'\xebBa\x1d\xc4lSXI:\xba\xf7\x0f\xff8$x\x92\xd4\xcc\x82\x8e2\xbfu\x87\xb1\xa8s\xc2#\x9fe\x1c\xc7+\n\xde]\xba4\xa6(\xdeW\xd0\xf5=\x01\xa0d\x10\xe7\xb0\xb3\x8a7\xa9k\x1b\x0f\xa0\x13mj\x99e;Oo8\xc9x\xbc\xf9a;\xed2\xb3\x99\xb8\xa8\xa6\xb7\xb2\xa0\xfdJk<\x1d\xaa\xdc\xb8\xf8]\x92\"\x82\x13\xa5\x90\xaa\x1de\x91\x9a\xee'\xd6\xd7T\x8e\xa2\x9a\xc2\xdd]*\xd6\n\xder|\x99\t\xad(4z\xbb\x81\xfcRx\xcbA\x8a\"^7\xfc\x96sC\x99\xc9[lEF\x94\nxpC\xd9\xce\x89b\xc8xc\x11\x7f\xa0\x87\x16k\xb6\x8cB\xd4\xbci1Oy\xc5\xda\xc4xa7\r\xe2$\xd4\xd4X\x13(\x85i\xae`\xd4\x12s\xb4c\x0cp\x9e/\xebF\xf0\xfc\xed\x04\xd2\r\x10\x86e\xceB\xe8C\x85`\x15ni\x92\x0f\x9fD\xd954\x002\xef*\xb4\x86wG`\r\xf0:z\xe3z\x95\x18E\xc7\x15I\xb4\xe9\xbfq|\x82D/\xb5\x1b\xd7\x864\x14\xab\xdc\xbd{\xda%\xaeg\x89\xb1\xf4o\\0\x1cO\xa9\x88rJKI\xd1\xe8\xdd\xbaI\x8a\xcd\xee\xecq$E\x9ewJL\xd9\xf8w\xaf\x1d\xd5F{\xf4\x82I\xce\xdb\xfbnWp\x05\xa0\xa7\xb3\x9e\x00I\x85\x05G\x80\xa8\x03\xef\x07\x10=\x9du\x03\xf8-\x8f\x87\xf3\xb6\x10\x0f\xe7\xb7K\xf1p\x1es]\x8c\"\xe1tKn\x00\xf7\xa5[a<3\x8aws\xdf9\xa0\x12\x97b\xde\x98\xe1\"SN\x91o\xee\xe7\xc2\x9d\xb1\x98\xdf\x80\xeeW\xde\x0e\x01\xd1Ax\x8b\x86.\xf2\x823@\xcd\xa2\xa5\xe6n\x00\xe3\xfa|=\xdc-8\x01@\x85\xd4\x0b`\x9a\xd8\xbc\x17\xc0\xa6\x0b\x81\xcdf-\xff7\xb2\x05O7\x9b`\xf3\xd4\xe5\xefC\x82\xcd\xafv\xa7o\xf7])v\xcd\xfd\\t3\x9b\"\xf3\x157\xbdt\xbfD\xd6\xff\xf73\xb1\xcd\xaaf\xa9\xa7\x85\xab\x1eD:]\xdeU\x19\xf6\xbe\x83\xeb\xed\xd7\xd5\xf3n\xbfz\xf31\xbf\xf3\xd9\xd6L\xb789\t\x8aM\xad\xdb\x82\xb5bZ\xe3\xa9\xc3\x16$a$Y\x80\xc9(\xf4\xa4E\x1f*\x1d\x88\x86\x08\x19\xc2\xcaX\x8c\xf9\x02\t\xa4\xd0C\xab\x06\x82\xbbVU\x18)\xc6\x04\xd7q\xaa(*l\x90\xfb\xaf7>rh\x1bz\x86\xd4y\xde\xe9\x8bj\xd6>\x90(\x8d7\xcc\x91R\xc2\xd4X\x97\xdc\xf6L\x88\xbd#(C\xaaeN`:\x8a\xa5\xa9\xbdO>\xb5\xb2\x8c\n/\x87\xfd\xe7\xee\xf8)\xc3\x05\xf6|\x0e\x19\xf2*\xf3\xd8\x90vC\xe8\x90>\x9f\xc3\x87\x97\xcf\xf2\xc8\x98\xf9M\xf0\xe5\x9b\x9f.\xc5\xe3\xde2d\xd0\x99[h)6w\xec \x96F\xc6\x8c\xec.u\x1a\x1f\xd3}dj?Ds\x84\x85Lg\x0e\xf8\xf1r\xfc\x00\xc6\xc7e\xfb\x97m\x86m\xd9\x0c2[\x98\x97]\xda{\x80\x0f\xf5\x18\x81\xa2dB\xc4!\x00\x07\xf3bbV0\x82\x83EP\xee\xc1]/\xb2\x8a\xd4\t\x94\xbc\x9c\x13!\xa6\xeb\x05\x102No\x17\xaf\x17\xf3\x9d6\xcc\nr\x08\xbed%+H9\xd9<\x82>\xae`\xf98\xf6\xed\x161\xf3${Y\xc0\x92\xfc\x1524QW\"\xc8\xd8p4\xdb\xf3\x91\xcd\xb6l\xfe\xa2\x1e\x01\x19\xd6^\xb9\x89x \xcc\x11(~q:)\xd1\x7f\xcc\x13P\x0cr\x04\xedv\xe8\nv/\xbeL?\xad\xcfX\x1dR\xe3\xd4p\x91\xc1\xb49\xe0%\xe6\xcbZL\x9b##\xaab5\x95\xf72\xb2!i\x0f\xf8\x1c\xd1%\xa7\xa6+\x0f\xf9*Az\x1f\x8a\x0c\x03\xc2\xc06\xc4\x1a\xf3\xbfSXfB@\x0b_\xb3\xc6\x0b\x90\r\xe9\xd8|\x9b5\xe6\x16VQY\x13\x95\x19f\xb5\x83\xe1MPl\xa8\xbc*\x0e\xd2\t\xd5!\xa5\x1c\xa5/B!\xa2\xf6\xea\xb6:d\x04Cag\x13\x89\"\x8dw\xb5f\xd1\xb4qd\xc1E\x1b\x92c\x08\xa4\xac\xa3\xaa\x103\x94Q-k\x05\xd2%\x19\xbf\x82-\xab\xd9`\x0cpZ\xeb&\xa2Z\xd6\x9f\xc1V\xe8\x1d\xc1\xa8U\xf0\xd6SQ\xbc;\xd5De\xda\xbfw\xdaY\xcby\x95\x0e\x9f\x8b\xaa8\xf6<\x16\xa1f\xcf\x17R(%m\xfb\x1cJ\xc9\xf3\xd9\xc4I\xbf\xe6\x89\x93\xf2\xf0\"\xb7\xbf\xbe\xbf\x908\xe9Q\x91\xa4\x11\x0b\xd7s\xd1\xa3\xafH\xa8\xf4\x19\x113\xbfm\xa53\xa7\xb4Js\x01\xa5\x97g\x91\xe9\xe0n\x0f\xae\xa5\xf9\x04K\x91/\xc2c\xa2JG!G\xae\x8b*\x8d\x08\xb9N\x91\xf3\n%\xdc\xb8Ls\x08\x9a\xae\xd5\xae\xb8\xa3R\x8f\x02\xc0L\xc0\xcfy{\x16\xeb\xfc\xaf\x1d~\xce*\xe1n\x0f\xc3\xe7aF\x0b7\x8e\xed!\x87R\x9aL\xbb_H\xd8\x84\x92Rg\xc6\xa1\xd6\x17\x91\xb5\xbc\xc70w\xd3a.\xf0t\xa1+y\x9de\xc8\x85mX\xbc\x9c\xa5\xef\xc4KF]~'\xb4\xf7X\xd0\xc0A\x96'\xb4\xf7\xf8_\xd1\xbd\xd5x\xcc+fDj=pq@\x88\xc1\"\x06\x92\x18d\"\x00\x12-& \xaeC\xec\r:\xa03J\x87\xec}\x10\x88\xce\xa7\xael0s%\xe5\x9b\x822\x1d\xa2t\xf8\xac\x96\r\xf6\xde\xb2<N\xc6\xd7di\x93\xb1M\xe3#\x86T\x01\x80\x04e\x7fhY\x16)\xe3\xa3p\xb1u\xc19X?\xdb:\xa2\x0cKs\xd1b\xcd\x9a\xad\x92\x8dlO\x82\x8b\xb6e\x94\xe0\xc92\xd8\x1b[\x04\xb5W\x0f\xfb\x0f\xdd1\x8f\x9d\xc5\x9e\xcf\xea\x05\xf3:\xf3\x00\x97\xf6C\x00\x97>\x9f\x03\xb8W\xb9\x9b\xde\xab\xdcM\xef\xd5\xfb7\x97\xe2gi\x1e\x12\xbe\x04jm\x002'p\xf3Nv\xcc\x8c\xb5*\xa6Gx?M%\xc5\xb1t\x82\x80c\xd3\xd3\xa3;\x976\xdd\xac\x15*\x1a\xa73N\x9d\x07,\x9e\xc9\x97\xf0j|\x81\xc7\xb2q\tG4\xc9\xfe\x8a`o\xbat#\xccm?y\xfe\x84\x05ts\xabX\xd5A\x1f\x88x\x97fK\xf8\xa5\xfb\xf0\xb5;\xceK\x15\xa7\xa9u\xc7\x0c\xd1\xc6e\xff8\x1c\xcb\xe1\xb4\xb6\xe1J\xd89`\x1b\xba\x19/\x89\x8d\xde\x8c\xef\xb9\xeb\x9dq\xe6x\x92&`\x86}g\x12\xc6\xbf\xb9\xbbR\xfb\xe1\x98fPpo\xb8lL\xc2cYm\x82\xcb\x02\xdaX\x96al\x9c\xe4pL\xf1\xeb\x97\xae\x1f\xc7t:3\xb7c\x06a\x92\xf9]\xb0PZ.\xb2\xd9\xe5\xac\t=s\xd9w\xbb\xc2gUJ\xc0+\xdd\xe0\xc1X\xb1\xb2 (\x04\x1f\x88\x92\x99bR\xe3I\x01\x0b\xd2\x02\x8a\x10\x81\x83\xd2\xf4%`\x16\x1f\x9b\xf1\x91\x9a\x1e\xb7\xf5\xdc1\x9d\x1d\xe1\xfcx\x8f\x8f\xfe\x18\x16b\xc8H\xe1\x84C\r\xdd|t!\ra\x9a\xa20N_\x18\xa76\xe4i\x08\xe3\x94\x88i\xba\xc48\x95b\x9cf\xb1-P\x8a\x19Y\x86\xc0\x946\xe4=\xaa+lS\xb1\xdbi\x1b\xdd9)\xd8ex\x7f\xcbJ\xae\xbb\xc3\x97\xeeS\xe1R\xc6\x0b\"\x00\x83\x82\xd9\x14\x84eMW\xda\x8bWv\xe5\x8d-;\x9b\xe7\xd7\xb4\xb7\xb9X\xf3\xed_KbM)3\x97s\xc1p@>B\xf9\x85.\xe7\xcc\xe4\x05\xf1N_yq\x1b\xe7\xfdG\xf7i\xd8\xa5\x1a\xb2t\x9dHI6=\x8fR\xcd\x95ud|r\xf0\x1c,\x0c\xb2\xdb\xdb[lp!|\x16s(\x8f,)\xbb\xabte\xdcq\x1c\xed)\xcdcBi\xd5\xa9=\x0c\x9a\xbd\\\x8e\x01\xfe\xf6\xdb\x1d\xd8\xbe\x0c\xe2Sv\xa5\x83u\xcf\xc3kM\xcb\xd1G\t/S\xe5\x1a\xa4\xdc\x05\xdb\xcan3g[\xe9\xe2Q\xac\xad\xf6\x16\x96\x1b\xa7\x8f+\\\xeep$\x19\x1e\xbe\x05/\x84\xb10\x8f\xb8\xe5\xf6\x01Lm.\xef \xb7\x7f\xe1\xd6'\xdc\\\x7fF\x07\x07{/\xbb\xe3\xc1\xf3\xf2%o\xb1\xb7G\xe5\x18*|\x9c\x85]\\\xbc\xfce\x1f\x8dw0\xd4\xd36m6\x93\xe3\xba\xdaN\x96\xa13\x89\x08\xcb\xf5\x9e\x10L\x9d\x15\x87\x96^\xe4\xa7C\xba8\x10\xf9Q\xc29\xa4j/\xfc\xac\x1d\xa5\xd9\xef\x82\x93\"x\x807(\x1c\xacBbAL$Q\xf9 U*$\x16\xac\xa3T\x82\xe8\xdc\xd8zA%\x07\xfd\x18Pb\xb0\x89\x80(\x06\xa9\x04\xc0bpK\x81\x8f'9\x8cs\x16\xc7\xf9\x8c\xe3\\\xc7i\x1e\xe48G2\xcf\x9f\x1c\xe7V.\xa7m4\xc5t\x8fq*\xc8<Md\x9aB\xd2\x86\xe8\xd8>\xed\x1e\xc4\xb5TQ\x86f\xe5\xef\xb3@\x99\xc0\xd4\xe8\x10q\x93Q5%\xfd\x93AZ\xe0)\xd8!$\x02\x0f\xfbl\x11\xf4\xc7\xbdZ\x8a\xf9\x15\x1e\xcf\xdeX\xb3*\xf3\x98\x9ftB\x88\x9f<\x9e\x03\xfa\xdfrw\x88\xdfrw\x88\xdf~]p\xdb\xb7\x8f\x89\xf7\xd5\x05@\xffS\x91\xbe\xc6ie\x12\xd8d\xaa\x80\xe5\xe3\xc3\xc5\xdc\x1c\xca\\yO\x9d\x1a*\x9b\xb9t\x15\x83\xeeuHv\x7fY\xf2\xcan\xa8\x08\xe0\x00\xd7\xdb\x92\xce\xb2c\x10\xcd\x82x\xc1=5\x0b\xdf\xf5\xf3\xb0\xff\xf0uX0y\x81\xd5\xca\xa09]/\xe6\xbb\xbf\t\xe2\xd6K\x81\xbb\xe4dU?\x1b\xb8+~y\xf7\xf4\xec\xf4)\x05\xe0\xf1\xe1bb\x8e-CB\xe6Q\xafl\x82{\x80\xb8q\xb7g\xffV\x97\x1c\xf5\n\x8d\xb7\xb3\xa0\n\xc6-\xfcCX\x8c\xc3\xf5\xbc+\x06\xe0j\xfa\xa1\x9e\x0b\xc0\x05eO\x1c\x80\x0b\x8c\xf7\x08.o \xa4\x8a\xf4\x06\x9f\x00\x90d\x8e\xc9\x8e\xf0\x1bQ\xe1\xf9n\xd1\x02\x13\xd0\x93\xac3\rk\xd4\xa2u\x0c\xd9\x94\x82\x85\t\xb3L\xa5\x04\xac7\x86'R\x05\xebSO\x85\x81.\x9e\xb7\xaf\x8fC\xf1\xc0e\xcf\xa3\x137\x7f>\x7f\xcc\xa6m\xd39\x9b>\x9fM\xac\x97\x07V|\x9d\xc7Gy}[<h\x17\x13\x99\x93\xc7\xfb\x8a\xb9\xa8u\xac\xf8Q'\xf3\x9fL\xeb\xee\xfe\r\xee\xd2\xe0\xeb\x96G`AY\xe4\x9f9\xf4_\x1f\x0b\xc1\x1d\xd3\x85\x87c\x1f\xaa\xce\xdf\xdf\xfa~ne\x92c\x1f\x1a\x1a\xe6\xcdS\x0cS\xb8m\xc3\x8d\r\xdd\xe1E\xc8\xdaD@\xb0b\x89\x9c\xdc\x8f\xfbz\xc5\xac%\xc551\"\xe1g\xf3~u-\x13t^\x8a\x14\xe9\xec\xfa\xe8vW=*F\xa4\xf3e\xdf\x1c\xaf\xc8.\x0f\xcb\x98\xa1M\xfa\x9aR\x0f;0\xae\x04E\xdf\x05\xcc\x99\xee|\x90GvB\x9ei\xd3\x16\x90\xe7\xf5p,\x84\x8c\xcc\x06\x87\xda\xbd\xe1\xde\xa1O7qc\xddj\xd3\xad\xd2\xe1ff\x98\xdc\x1b\xae\xba2ld\xb2\xab\xb9-f\xd2]f\x8b\xb9\xd8\xdd\x92\xf3\xddUy\xe8W\xc1U\xb4n\xd2\xcfe\x11\xd9\n)\xeb\xd3\xcf4\xceE\x88\xae{e\xccsY\x08\xc1]\xefO \xdfe3\xce\x04\xffb\xc8I\xe1(\x87*\xed\xc1\t,!k4\x8f7\xac&\x07.gEy#\xf1\x1aCn\x07\x9c\xaay\xd6\xf1\x1b\xca,\xde\x14(\xba\xfeX\xecS\xb0\xf1(\x1c\x8f\x0e\xa6\xfcd}y\xa3q\xe4\x92Q\x8dw\xea`\xb9\x0eo4\xda\x9eZ\xd6\x8a\xf5\xbd\xd7\x8c\x92I\x19d>l\xd8\xfcP%y\x03y\xd4\x03U\x07\x9bU\xacIn\x055\xce\xa8e\xbf\xa39\x08\xb45m\xd0\xe1\x81]\xfd*\\nt\x93\xd0\x81-\x11x\xeb\x84[\xa0\x9b7]A]s\x94\x9b\xc2\xb5&\xa8\x81&\xdcio*\x0c%gqH:\xdc\x93q\t*\xefY\xd1\x84[\xf3\x8d\xc2\x9a\xb4\xe4u\xb8{\xdfP\xe0\xba:x\xa4h\xf4\x0c\x01\xff\x0c-\xfcF\xe1\x14\xdd\xaf\xc9\xafE\x06y\x05\xbe\x1cb\xdc\x14\xb2q\x16\xa9&HD\xd8\xe6[f\xab\xde\xefw\xe7a\xb3B\x0b\xdb\x8c\xbb\xca\x8b#&+.v\x99\xeb\x8f\xe3\xa7\xbbz~\xdc\x9d\xbb\xdd~5EVu\x01q\x86\xe3~\xf5\x18\x8elfX\xc4\x98\xcd\x14\xcf\xf1g\xef\xf3\x10F?>\xcf\xf8\xb3\x1f\x9f\xdf^\xa5\xb7\xc5\xbf\x8aE\xb1kW3*^`\xdf6\x8c?+\x05/\x02f)\x8f\xbeJ!\x8c\x98p\xbc\xe7,V\xc2J\xfdu8\x0e\xbb\xfd\xee\xc3x\n\xae~rgt3R\xe33f:Yzo\xa9y\xd3\xd8\xc8a5V\xd9\x1cf\x03\xe2qP\x89o\xd8\x0bN%\xb7\x87\xaf\xdd\xc3\xe7\xe1fl\xba\xc4z\x99\xc0\x18\xe1]\xbbg\xce\xa3\x98\x8ei\x15B\x14!\xc7\xc431\xad\xb3pE\xdby%\xf2\xc0\xfe\xc2E}\x9dE\x1c\xc2b\x96\xcci\xc3\xc2\x15\x15#\x0e\x01\xd3\xa5jf;\xe5^]eX\xd2f\xb0R\x16\xabT\x1e\xcf\x15\xd4\x98\xfe\"\x95\xcd\xff\xf0\xe1\xebp\x82@\xa3w\xe7\xf1\x15\x82\x06_\xdc}=\x95\xd95\xf7-N\x11wn\xbb\xaf\x99\x1d\xd6\xf0\x01^\xf5\xb9\xe4\xb9\n\x8c\x1a\x84\xf4\x03\x95\xf5\xba\xc0\xa25\xd58\xcen\xd3l\x12\xb6l\xda\x8d\xe3\xf6\x1bw\xd9a\xbf\xdb\x7f:\xee~\xff\x94\xaa\xadw\xc3\xdd\xa7n<,\xbe\x9e\x0b&\xcd\xa5m8\xa3\xbf\xce]U\x07\x16H\x95k\xb4\xebk\x9di\xce\x87\xe3\xda-\\\xc1b\xeb\xf9\xeex7u \xf4v\xe4\x82\x7f:vww\x8e\xb5\x13\xc5`\n\xcb\x03E\x9f\xda+F\xba\x14,<;\x97@\xd8?\xb2hKgQ\xa6\x05\xf8\xeb4\rw\xdc\x88N\x8c\x8c\xde\x7f<8\x9eO\xf4\xab\x9f\xc1\x82\xae\xdb\xaf>B\x06\x0fs\xbfz\xb6\xff\xb8\x02MJ\xb5\xf2q\x83\xe6XB[oE\xe2\xef\xcc\"\x90\xbb2S\xad\x9f\x9c\r\x84\x04\x19\x92\xcc\xb0\xc1\xa3Q{c\xe0\xc9\x9fR\xf1\x94\x11R{\x1d\x82f\xbf\x93XF\xc6\xd5\xd0\x8a\n\xc9\xa8\x19e#\n~W\x87d\x1d\xaeM\xca^\x18\x07I\x94\xe8]+C2\x0f%|\n\x8b:x\x96\xa2\xe9\xb7\xf0\xb3\x12,\x12C\x83\x91\x18\x9a\xd0\xa7\x0f\xa7\xd8\xa0\xe3\xa9\r\t;|\x99B/\xe0\x96\x8d\xa5\xf23V,~\x92\xc4\x9a\x153.\xaf\xbc\xf6\xc3:\x8a+\xd0+\x1e\x04\xd2GZ\x02\x1f\xd5\xd6K\xdd\xab\xe0\xbb\x8c\xabO\xf9\x18\x15\xfa<\xd7\x8c2\xbe\x15\xe1\xa8\x96\xb5\x19\x92x[G1)?y\xb3\x82\x06\xc7S\x8e\x83\x94!\x15\xb8\x0c~\xb6\x10sC\x06\xd3\x0c\xe3(\x1bE\x92\x92\xc5\xb8Rq\xcc)\x1e\x8f*\x8eU\x95\xc6\xb1\xe2\x010\xe3\xe0\x98i\xe0L\x1b\xed\x1e\xdaK\xcb\x1c\xdf\xc8\xa1\r\xfd\x90qz\xe1q\xc4\xe1\xbd\x1a\xee\xee\xc6c\xbc\x7f\x9c\xadB\xd2\x07\xb1m\xc9\xe39v\xed\xc7\xdb\x9c]\xbb\xcd\xd9\xb5\xdb\x17\x97\xd8\xb5,Ew\xa6\x06`q8L*\xf3\x02\x13\x84u\xea\x97\xfb\xe3q7\xf4\x1f\x87RN\x92d~\xc0N\x8d\x0f\xcb)\xba\xf0\xb6\xbd4<\xe0\x9b~\x04\x88\xb7}Qb\xa5\x19\xe7\xe3\xfev=\x8b\xb6Q4\xa2\xcbStm\x1f\xc1\xc8d\x9e\xb8?\x02\xafy\x84\xff}\x98\x97\x1c\xfd\xcb\xc3\xbec\xbc\xdf\xee\\X\x1f\xc6~p\xc6\xc3\xb9U\r\xb9Cn\xb7\xd9Z\xb0\x95\x03\xe7\xdc\x84\t\x99^\xd6\xa7\xa1\x94Rd\xea63\x0e\xc0\x95\x9eI\xc7%\xb8\x85\xdc\xe5t\\c\x0f\x9f\nR X\x84\x19{\xb9\xb9.\x96<q/o#\xb4\x91\xfb\xba\xfa\x19\\\xa9\xees;\xb9\xa1\x9a\x92\x80\x08U0\xec\xa6\xb2\xcdSf\xdf*\xda\x95\x05c\xe7\xd8\xf496\x8b\x8eM\xa6\xc1\x98\x9b\xe5\xe0\xf6\x94B\xc3k\x152\x80{3\xec:2\xc3\xae\xd1\x83G\x84\x0c\xe0\xbe\x072\x17\xb7\xc1\xa7\x96\xcaZ\xb4\xf3FB\x876Z\xb48\x97\x9c\x08\xae\xb7\xcax\xf7]\x15\xea\x81\xdb\xaf/\x13X\xa6\x83\x13\xae_\x92\x1a\x9b4!\x83\xa3\xaf\t8`\xbc\xf3.3,\x84\x04\xe6dt\x0897\xbdQ\x9eB\xf3D\xd2\xd6s\x13=\x8d\xd6\x08\x15[t\x1b\xbd\x9e`\xe8\xc7M\x02\xcb\xe6\x82\x17\xcc\xe9\x9e=\x9c\xce\xc7R\xb4\xc1\xf0|>\xda`Vg!\xda`\xd2\x8f\x8f6\x98<\x9f\x8d6\x98[\x83?{\x97G\x1b|\xff\xee\xaal\xca\x8a\x07\x94(\xc6\x19\xd4\xa9\x05]\xbb-C\xd6\xa2\xd5\xdct\xc0m\x9a\xd3y8\x96\x04\x03\xe9\xdc1\xe8`\xf24U\xbc\xd4]*\x13\x90*\r7\xf80\xb60\xc2\xe6l\xbcA\x8c\x1d%\x83\"\xe5\xb21x\x95\xc5\x8b\xb2W\x19\x83\xbb\xebVd+\x87\xd7\xf2\r\xd3\x97,F\x1d|8}\x8bV\x04#\x0e&\xeb\xb4+\xee'\x86p]\xd0\xbe\x83\x85\xdc\xe4\xc5D\x98W\xb2\x8dk\xa6\xe8\xc5\x18\x84\xa2^\x07\xe4\xcbm\xe3~=\x8c\xefx_\xb8u\xa7\xc3\xb9?{\xad\x8fH\x8a\xb2\xd8\x83u\x88\x16\x01\x17\xd4Esq\xb6\xd5v\x9f\xd2\xb4\x94\xcf\xbe\x8c\x9d=\x1c\xcbj\xfa\xc5\xde\x96t 5\x8bJ\xd1f\x82*\x93\xbb\x18\x8f\x93\xce\x83A\xe9\xb6\xd1\xd3\x85t\xda\t3V\xe3mc\xa2\x1aO}q\x8d\x91&A\xa1\x18\xa1b\xf4\x8a\x91-F=\xb8\xd45<\x01\xb3\xa2(\x0f\x12\xbdQMH\xf8\xacBR`\x15<\x94\xb1Ly\x80\x01\xcf_J\xf1\xacC\x18&\xa10P\x12K\xeaL\xf9\x01 A\xb1\xa7\x1a\xbc\xde\x1aV&<\xe2\xdbp\xf5E\xbb\xf2\xca\x1b\x80\xa9p\xadB\xa7\xac\xca\xbboA\\%\x1b\x12E\xfbP\xbe\x14\xf2J\x05\x133V\x13\xaeq*X\xb5K\xeb\xf9\x8d&\\\xff 1\xb5\xb4~F&\\E\x91\"\xcb|\x08\xac\x14\xff\xae\x8d~G\xe6|t)\xaf\x82\xb1\x9d\x9f\x91\xc290\x83:\xba6\"\x98\xd3*\xd5\xb8\x82\x921\x0f\xb4\xba\xb0{j\x1f\xc1\x83\x85\xa2\x14\xe0] \x89\xa5\x01\xf7o\x0cx\xd2\x06\xcfp\x8b\x1b\xc2uf\xb8?\xb9\xc18\\\x06\x08\x1d\xf6\x06\xa0?\xda\xdc5\xe89_A\x89\x08\xae\xf2\r\xf7\x0fop\xfbb5\x19\\\xee\x0cV\xb3>\xe4\n\xb9\xed\xc1&h(\x80K\xc5\t\xe6~M\xcc\x9b\x8c\xf8:\x9dpy\x14\x1f\xc5\xa9Z(\xec\x1a\x18D\xb2w[\x07\xa1\x03\xb0J\x14\xca\r\xf6\xbcl\xfc\x1b\x83\xb2*|\x9b2\xfan\xa5\xe1_\x95\xefO\xe2^\xaa\x82y\xa4\x046\xca\x8d\xcer\x89A\xcb\x05E-\x97\xc6\xb4\xb8\xcfj_\x8d\x08\x8b-\xb6\xfe\xdd\x86\x12\xf6}Yl\x1a\xf7\x03uz\xc1n\xe7\xdc\xdd\xe5\x9e}\xfeil\xb33>\xddu\xfb\xc7\xc9\x1a\xe2\x0e\xbc\xe9N\xf4t\x8e\x85{\xfd.7\xdc\xc9Y\xb8\xd7\xef\x9e\xcdF\x00Ck\x1c3\x93\xaa\xa2\xc8\xc8\x19\xa6\xdb\x19\x82\xe1L\xe6\xca\xe7\x96#\xb3\x88\x8cg\x86\x961n\xe1.\x18\xc6Dz\x1a\xb5\xa0\xa7\x81~\xe7c\x7f\xf1\x10.\xeb\xebx\xb46pg\x91Qd1&4S\x94\xa0\xd5J\xea\xa4\xf7\xeb]w\xfa6/gH\x96cw.<L\xd5\x1c\xeb\xed2\xe7e A]\xbf\xc4s\xb9.\x8a\xae\xe6\x7fC\x8f\x84\xfec\xea\xa2\xf7\xfa\x8c\xfa\x9b\xdd\x82\t\nW\x06\xac\x97\xccN\x92\x1d\xc3\xedN\xa6m\xb3`v\x92uq1\xces\xbe\xe9g\x9c\x0b\x1c\x1f\x98\xab\x04\x1aQM\x01\x9e\xdb\xa9\x852\x97e\xb6\x93\x1e\x83\xd5x:\xf9Cr)\x8f\xae\xeb\xd1E\x9e]\xf1\x13\x9e,\xc2\x8c\x08N\x12\xa0a\x10\x14\x81S\x04[\x11\xa0EP\x17\x81`\x83\r(\x9f(\x8f:\x8dD!\xf0\x1b\xc3\xc1\x16\xe7J\xd1JT\x00h\x04\x1d\x8bR\x97&\x0c\x14MD\xc2b-\xfb\xc1\xfd\xb1;\x7fK\x8c\xb1\xbc+\\\\\x16{\xc3\xfd\xb1;\x9dV/\x0e\xfb\xed\xb0\x19\x8e\xddyw\xb82\xf6c\xa1?\xef\x14W(\x9b\x8d\x03\xf9\xf7\x0c\x05^\xbc\xcaP\xe0\xc5\xab\x1f\xe6\x1d\xe2,\xc3\x02\x19\xee\x1dKNp\"`A\xfb4\xa6\x94o\xfb\x8f\x7f\x0c\xbbo\xa9?\\a%\x005\xde>\xec\xbe\x15\xa3\x98\xb09\xf1\xacd,\r%z\xc0=\x8c/\xee\xa2P\xba\x0eJ\xf9\xfebR\xec\\4\xdd2\xe3G}\xc9\xe5\x8d\xa9\xd2\xf9\x95\x9e\xe7J}\xa49\xe4[t\x16\xfb\xbdO\xdd\xe0\xa6]\x9b\xfa\xc0}\xd9}\x1b\xd7z\xde\x03\x0e\xb1\xa6\x9d\t,\xd9\x9bJ\x15\xe5\xd8\x7f\x1f_\xe2\xa9h\xe7\xf8\xf6\xdb\x1f\xdd\xef}\x97\xdf\xe8\xdf>\x80k?\xa4\xaf+:\xba\xd94\xc3\x8e\xf3\x92'\xa9vu\xf1\x82\x9fn\xb8(\xe4\xe4\x17@Bs\x1cJ^oWw\xbd\x14\xe6\x84\xedS\xd8\xa1\x86e\x06\xbe\xda\xb0\xf1\x9dK\x9d=\xe8\xaf+\xe77X\xe7Y\xf6\x8c\xd2.\xe5\xe9Pp|ceO\x99\x0e\xb5\r\xb6\\\x06M\xb9\x84?\xd85F\xe4o\xd0r\xbf\rYS\x05\xcb\xb2M\x06mQf\xd2(gi\x94\xcdt)\x03\xaa\r&i\n}\x00p8\"\x0c4\x8cz9A\xde0\xd9\x83M[\xf9\x94'\xc7\x8b\xcb\xe6\xa2@\x8e\x95V\xe5\xaa\x0by\xf1\n\xdd\xfa\x9cx\x85\xb29\xb4x\x93{U\xbd\xc9\xbd\xaa\xde\xfc\\\xcc\x87g\x1e\xefU\xc53p7\x7f\x0e'\x8az\xcc7\xbb\tm\x13C\xf8\xfd\xdcj\xa0\xe3U\x07g\xcai8\xad\x9ew\xbf\x1fN%\x17,s\xd1\xfe\xfe\xb7\xee\xeb\xe9\xe6yw\x9a\x8d\x0f\xc2\xd5\x99\xf3\x06\xf3\x17\xe2\x83\\4\x92\x07\xa5\xa6;fZ\x87s\x8f\xc5\x86W\x87\xbb\xd4\x02\x9e\xf2\xa7F6\xde\xe8\x85\xd5\r\xa7\xdd\xb8l\xa7\xd3nV\x16\x1c\xd9\xc3Wy\xc8+1\xc9\x7f\xd7\xd2\xf9F\xcfyb\xbd\xa1\xf7\x9a\xc6!>\x14-\xe2\xe3w\xba\xfbO\xf6R\xd9=\x84\x19K\xa1\xe1\xbb\xbel\xad\xce\x1c\x851\xcf\x9a\x1b\x1a\x86\xc2\x1a\xff\xc9\x021\x9f\x8flE\x8bBaf\x05u\xfd\x10\xa4-\xd9Kq+\xc5\xed2X\x00\xa0\xa4\xaaT4{*\xb9zYi\xcc\x9c\xd9;\x94=\xb1\xab\xd7b\x9c\xaa(\xa2U\x12n1\r\xc5\xc8\xc34\xc6!\x1c\xe3\xf0\x8eq\xe8\xc74,d)d\xa4*\x86\x9aTA\xba\x885Qp\x8c\xb6,\xd2\xd7lXf\x1bJ\xc3\xca{\xa8\xfdXT\x94\xcb\xa6]\xc8l\x13g\xbd\x893\xe2\xc4\xd9r\xe2L:q\x96\x9d8\x03O\x9c\x9d\x87g\xee\x89\xb3\xfa\xa4\x19\x7f\xe2l@q\xa6 \x19\x95\xb16\xf1\xddJ\xef1\xaeC^\x03\x0c\xfe\x82\xc0\xec\xcb\x1a\xf6;\\yA\xc1H\x15\xcbU\xa7\xfc\xee\xa9\x16(\x15v\x88\x88\x82\x9f\xb2=\xb8|a\x1c\x8e\xebR\xd8\x14\xffxV\xcf\x9bUY\xb8%\xc6\x9d\xf8\x0bb\xfcx6\xa6e\xee\xdaw\x9b\xe7\x08x{[J)'5\xbb\t\xda\xabB\xa3\xe4\xf1\x9d\xbb\xfe\xdap(\xd3\x84\xf2|\x01\xf14\xf1\xe2\x17?d7\xbfl\xc4W8QC\xc7\xf3AP4s\x81\xab\xafK\x12`Y\xda\x9fK\xc1Nxd\xe7\xba\xbf\"\xb4I<y\xbc\xcc%\x0fK[\x84\xdd\xe5\xd6,S\x9c\x9c\x93\x18\n\xe9d\x85.\xc8\xf3\xa0f\"\x98\x0c\xc7/e\x89a\xd29\xde\xe9\xc0<i\xfcMw!o*\xa2\xa2X\xba\xbd%\x1b\x06C\x95\x80)\xf3X4\x17\xd2k\xa6\x97\x8b\x175\xbe\xadf\x14\xb2\xc9\xa4\xbd>\xd6\xba;\x96\xf3'3\xc3\xe4\x86^\x0eA\t\xf5\xc0\xab\xccl&\xc7\x81'\x0eA\x99\x05s\xe4\x81\x1e\xe3 \x90q\x80\xc8\x14\x94\xf9a\x99\x1e\xa4\xfc\x90\xb5\x187Z\xb0\xb8Y\x81\xaa\xd8\xef\x1a\x0e\xd8hA\xa5#\x83\x1d\xed\xf5U6\x809j^\x95\xd7\x84j\x06\r\rj\xf6(\xe8$K\xaa\x83\xfaT\xe9g\xdb\x04P\xc6\xb9\xd7\x1e\x0c8e\x90\xb2Q\x0f5\x1b\xa7\xf4\x9aP\x16\xed\x1a\xf5\xa2\xa8>D\xf3$\xe5-\xbd\xf8\xba(\x9c{\xcdt\xd0\xcakI\r\xabI\x94.P\n\xdf\x8af\xb6d:2\x8e\xd2~\x9c\xb2@\x99\xa0\xdb\xf5\xbb@\xa3$\xb7N\x02\x83.\x82\xe4\x8b\xe3\xa1;\x17P\x92=\x9f\x85\xc9\xbc\xce<N\xa6\xfd\x10P\xa6\xcfg\x83c\xe6V\xbb\xafr\xab\xddW\xb7\x7f\x9d\xcd\x9b\xc7\xc3\xd6r\xc3\xa2\x05\xd4\xe4\xdeRQ$h\x86\x94\x18.\xb3\xb9\x12G\x7fr\xf3\xcd\x804]\x06@\xd2\xe9i\xd9\xba\xb7k\xcbS\x82\xb30\x0b&\x06\xcd\x0f\xb3\xc9\xf6z\x16\x0f\xba\xe4\x80>\x9bN/\x8a\x04]15\xdc\x82y\x94N\x8d\xa1 BI\x05\x88\xda\\\xc4\xd8W\x87#JM\xcf\xf3J\xbai\x9dO\x19\xe6N\xeb\xf0\xad\xac\xa6\xdb\x86\x10b\xe0H\xde\x8b\x19\xe8\x9d\xb2)P\x081p!\xdf\xd4\xdb\xaa\x08\xc0\xf0\xb6\x8b\x08\xfc\xe2\xe3\xe1X\x8a 6\x0e\x11fT\xb6\x12\xaeu\x1a\xec2\n\xe8U/ r\xb6\xf5\xf0R\x8c\x90\xfce,\x9c\x03\xe5+:]\x00\xe8|\xb3J\xb5\x00\xd3\xe9\xb7\xe0qZL\x1b\x05\x02F\x83m\xd4\x0cNc=\xabY\xbd'\xc5\xe9\xd8\xee)\xb6\x04\x8e\xad\x84c\x0b\xe2\xd4\xba8\x0e\xd6\xcc\xb1?>\xcb\xd3s>\xc6\x00\x19!I\xbb\x80+j\x16\x8fRk\xad\xd8\x92\xcb^\x83$S\x90\xb7a?\x13\xa7r\xb8\x94\xc8\xbbPi9X\xe5\xb0\x9f\tV9\\\x93L\xe0m~\xf1b\x8fB\x94\xca7\x8f\x88R\x89\x0e!\xcd\xff(J\xe5#\xb3{O1\n\x87R\"\xb7t!X\x98\xcaa\xff\xa8(\x95\xdc\xf6\xbf\x18\xa5\x92\xe2\xd9?*R\xa5~DV\xb7B\xa4\xca\xea\x7f\x14\xa9\x92\xa7\xf1\xb92Re\xbb/E\xaa\x8c\xd7rW\xde\x833q*\xdb\xe5\x1c\xe0Q\x9c\xca)^\xd8l\x12\x02\xec\xb0|\xcf\xa3X\x95C)C\\qrW\x84\xaa\xd4\x97=\\aL\xc5P\x95\xc3\xfeq\xa1*\xf5uY\tf6.\x97\xb7\xce\xdd\x0b\xd37V\nO\xd9\xe8I\x9b\xbe\x94\xa2\x80\xd7\x86\xfb\xe1\xffV\xa2\x82X\xd0\xc9\xf3\xed\xc4\xb9x\xe2<=q\x0e\x9f8\xbfO\x9c\xfb'\xce\x0bT\xce\x19$\xf1w\x86\x89K-Z\xe4VX&XB\xef\xd4\x8d\x94\xbb\x98\xc6\xee\xa7\xb1kj\xec\xb6\x1a\xbb\xb4\xd6\xdc\xca7\x11\xcf\x96E\xb7MA\xe4\x9b\x8a\x83\xb9\xa88\x15#\xd3\xca/\"\xe1\xf3\x87\xbb8\x88;!!/\x98E\xc2B\xa5y$\xcc\xba\"$\xcc\n\xe6\x90\xf0y\xeeg\xf2\xfc\xc7\x0c\t\x9f\xffx;\x1f\xc6\xd1}s\xb2\xcbR\xcf\xcdc \x88\x1d\xc1H\x050\xb0\x94{\xe0\"\xfa\xe1$3\xf4\xcb&\x0f\xe8\x97=N\xe3N\xcc\xcd\xe4\n1%\r\xa5t\xc5\xaa\xb3d;\xeb\xc7\xa7&\x10Y\xe0\xc7\xebR\x13\x80\xd0R\t\x86\x84\x1b\x96\xa6`\xb8\xd2'e\x9a\x1f\xdcKr\xc7\x94l]w\xe5-\xc8\x02C6A\xc4\xc9\x93\x16\x94\xf2\xf0l&-YHZP\x84@\xec*\x0b\xd5\xfc\xfc\x01\xf0\xefC\xc1\x82\xa5<#\xa6\x94\x14i4GL4p\xd9r%\xdf\x97g\xbeI\xe6\xbc3\xaf\xe8p\xc9lrq\x0b\xcf\x18Of\xef\xc8'\xbf\xdbN\xc1;u\xdd\xa1\x0b\xca\x1c\xe8\x15\xeb=)\xdcU\x08#<\xd8~\xed\x85\x9b&h\xcf0\xaaq\xedE\x9d*\xa4\xceF\x8dU\xed\x05\x8aLw\x97\t\x14ca\xa3\x89\xb4`\xd5E-\x98\xca\xb5g\xb1f-\xd3\xba\xc5\x1a9\x9a\xed\"\xc4\xfcr\xd8\x9f\x87\xfd\xf0\xe1x\xc8@&.\x8a`\xa6X4\x0f.\x85N\x08^\nEs\x00\xf3\xcb?e\x00\xf3K\x1e\xbe\xf2\x977?\\\xcas\xda\xf3]\x9e\x85D\x82\xf0\x95\x91q\xe1\xb6\x84<\x99\x01\x0b\xf89\"\xda\xb0{W\xee\xb2\xdf\xaf\x13\xfc)\xacBi\xa1\x19\x04\x15\n\xd2\x948\x91\xafu\xc7l^\xb2<%\xca& 4\xb5\x8e\x0c=\xfc/\x1a@\x00$\x96\xeb\xad\xd7\x99\xfc\x8f\xc3\x15\x8f\x8e\xb4\x10\xf2h\xc8\xf2\xe6\xd8\x99\x1c:%\x1bK\xc8\x07\xe7\xa2\xe4\x92\xd5\xcc&\x8bf\xc4\xaci\x00\xbab\tb\x9ag\x87\xd6\x19\x82\x01'\x80Ux\x0b\xbb\xb9m\xcd\x92\xc7\xd5\xccz\xc6\x01X\xefnr\xeb\nnu9tu\x13\\m\xc4\x14\x05\x18lk0\x9e\xc0x\xabK\x00\xac\xd01\x8a\t\xbfu\xc7\xfd\xe1\x03L\xe3\x98bX\xe1g\xec\xfaf2U\xda\x86!L\x965g\x06\xcc\n}\x00\x9a\xfd\xd4\x1d\xbb)\xa1y%\xc4\xb6t\x9f3Y\x07\x8b\xdd_\x8e@\xc4>\x0c\xd3\x96\xefv3\x9f\x07\x80]a&\x04w\xed\xb0q1%\x8d(\x98\xd5\xa8\xc93\xc5\xd4S\x9fZ8\xcfL5h\x900>\xad\xa1\r\x07\x9eX\x03\x16k\xc7b\xcdY\xaaU\xe3\x1a\xb7T\x1bW-h\xf1\xb8\x86/\xd6\xfe\xc5\x9a\xc1Xkx\xd9\x88CEzI\x15\x01d0\x13\x89MHb`\xe5\xa0\x1b\x03rX\xb3\xe5\x1b\xd9\xe14\xc5(\x9b\x02\x06\xbe\x1a\x8e\xdf\xbe\xcfofq\x85\xe1\xc3\xe1\xcbn\x9f\xdf\xd3.V[\xb8\xa9\xcd\xfe4\xbe\xb7-W\x9b\xbd\xc5\xe51\x08\x9f?\xcboq\xaf_]\x02Y&9A,mX\x0cB\xaej\x8a\x82\xe1\xb3kR9\xe3x\x1fD\x9f\xf3\xc2M\x17\xb0\xae\x95\xab4I]\x94\x91\x9ci\xdd\x10\xa2y\x92\x1f\x0e\xe3\xd9uqZ\xdba\xbfz\x08\x8b\xfb\x07_\xdcK\xef\x98_%\xa1\xd2\xd7b\x95\xecZ\x99-g\xdf\xe3r\xae\xb25\xac\xc31\x16\x0b\xb0\xd2\x0b\xa7\x9b\xcc\xcd\xd4?b\xfd4\x88\x8b\xf7O\x99\xcb]W!\x045Z\xc3\xb6\xab\xe0f\x81\xacA\xc3\xa4\xb62e\x16\xb0%`\x13\xcc\x85\xdb\xebP\x90\xe0\xae\xd8\xe5V\xb1L\xecM\xe0\x1f\xb8\xf8\x17\xfd5\xaaK\x97\xde6\x15\x05\x97D\xbe\x10%Q\x013\xb1\n\xd9\xfc\xa2\x9c\xee\xcc\xc3\x03\xf8\x11\xe4A\xd6\xa9_\x08\x852J\xaf\xcf\x87\xd3\xb7q\xc3L\xaf\xabO\xb7\x0b^\xa2aGm\xba}qO\xed\xf8\xe90lV?\xe4U\xd8\xf5\x9a\xc7|X\x07\x95\xe6\x16l\x8al\xb8|\xf7\x10\x84\xd1\xc5?Z\xab`q\xc4R\xbd{n\xa6\xdf\xfa\xe8\x10A\x16\xbd\x9a\xd2\n\x0e\x8aL\x91\xfa~j\xa5\xeb\xa7w\x03fI\xeb\xaa\xb5\xe9m}\x9a\x05\x88\xabWCy\xbax}?\xb88\x88\xebi\xce\xbb\x15\xae]\xf4\xc9\xe2U\x1e\xd8#X\x9bb{\xe5k=D\x7f\x88\x05\xd9+ V!^\x12\xe7\xa0\xd0\x83D\xa7,\x8c\xac\x17r(\xa1P\x00O\x9fC\xffqU\xda\x06g\xffY\xbbmr\x1a>]\x90\x11\xf0\xd1\x13\xf3\xb4\x02\xe2)G\xffp\xbc|\x9a\x01*\xf03\rb\x84\xf33\x8d3hs\x02\xfaL4\xef\x96\xc3\xc7\x81,-[\x9c\x03\x03d\xf0AJ?\xf1j\xb6\xf7\xfc\x1dXz\x01\xf7\xe6,\xbd\xda\xaa\x93s93&\xaf\x1b.\xc3\x9fBIN\xde\xc3v\xd3\xac\x93\xb6\x9c\xb4\xdfl\xaa'\xcd\xb1\xd1\xa2\xac\x9b\xd2a\x98 M\x87\xe4\x18*\xe4\xd8\xa8\x82l\x1d\"\x8e3J\xb2V\x1a\x94\xa6\x13\x055\x15Rm\x08\xffwC1,d\x88pN\x91\"n\x0cF\x98\xa0\xd4\x1c:\x04&\xf4eU\x18\x8b\xc4\x90P\xd0\n\xc5\x1d\xb81\x18\xd9\xc0\xb2V\x8c\xaf\xa9C\xf4\x82\x1b\xc7\x7fR\x00\xc5\x1b\x8b\xb1\x0c*\xa4\xa0\x8c\xad\x12\x85o\xbc\xa1\xc8\x03\x12\xd3\x8bH6\x96\x16\xa3,\x18\xa4T\x88\xca\x01+O\x11;\xf0=\xd0\xdc-\x06=\xb0\x8c\xa2\xc0\xdb\xf4\xc6tD]L\xf5t<?|\xe8\xee\n\xc9\x9eBA\xc4\x85B\xc1\xc3p\x1a\x1e\x17\xcb \xeb)d|J\nfs>\xe5\xc9\x1d\x7f\xcb#\x1a\xfcv\xfb\xee*6\x13M6\xbbTb\xc9\xc4\xbbi\x96\x91\xc8vY\xb1\xb8\xd6\x86\xc7;\xc82?%\xd3\xe3\xcb\xc8\x1f\x9f\x8a\x8f\x03;\x07\xd9z\xae\x186\xe5\x7fJ;(&\x80\xea\x99\xc1\x15rU\xf2\x111\x10xJ(\x14\xb2p\xa3,`\x9e\xfa\xf9\xf4P\"3sV\x99~\xa0N\xd8\x19\x98Y\x1eZ\x80\xf2D%\xf3\xde\xf9]v\x18\x1f\xdf\x1dfSE!Sb\x99^`]H\x155\xf4\xc4ol\xea\x89\xd5\xeb\xfa\xcaf\t\xa3\x921\xdcG\x8fs?\xa5\xb4~9g\x94b\x16OJ\xa62\xfa\xc5\xe8\tY\x0fg\xbfI\x86O\xcc\xcd\xb5\x9cO\xea\x9a\x8e\x17$(\xf9\xce\xed\xb6\xc5\x9d\x8b\xb0\xeb\"3\xbb\x8b\x98p\xe1\xb2F\xf4=\xac\xde|\xcca\xd7\xaaFL\x7f'\xcf\xe2)\xcdqI|\x92\xd4xB\xdd@\x831\x06)nq\xcb\x92\x1eS\x8a`\x8d\xaai\xc9jZLe\x0c\xf2\xf8\x16\xa3\x1f\xda\x88\xd2H\xb1\x18\x8a5\xf9;\xb5H\xb5\xc1\xb8\xabn\xb9Q\x98\xac\xd0\xf0KEe\xd4J\x1d\x95\x89\xf0\xbb\xdar\xa3oOI6j4\xf3\xae\xc9\x04\x9cT\xefu0\xe7\xaa-\x9a\x80\xb7\xac,\xb4)\x82\xd9Ym0\xf8\x82bf\x00\xc6+\xe9[\xb6Ja\xad/h&\xee6\x87/]A-\xe1\x9f\xcf\xaa\xbe\xf3:K\xca\x89\xb8\x9f\xa0\x99\x88\x9f\xcf\xaa%^\xe6j\x89\x97\xb9Z\xe2e)8O$\x8fT\xc1\xcf/\xceY<\xab}\x00\x8d7\xb8\xdb\xc6\x19\x8a\x13\xe0\xf2\xb9\x90\xa6Iu\x0f\x99\x92!\x9e)i\x18\xc6\xaa_\xca\xe9\x0bg\x86]\x1b\xce\x8dg:\x05\xd7\xde0\xafG\xe0>\xb5<x\xcf%o\x1c\xae5@\x95\xb6\xccn\xfa\xea\x82\xbe\x80\xab\xb4!\xa5A\x9c\x928\xd3\x0b\xc4kF\xfa\x80\xe4\xe9\xb9\xb8\x92\xa9*\x00\xd1\xaa\x85\xb4\xc3\x05\xf1\xff\xc4\xad\xac\x1d\xb2O\xa9\x863\x91\xbfk\xbeh\xb6\xf5\xcb\x01\xf4\xd6\x9b\xee\x8f].\xf3\xbf\x83\xa8\x0b\xa2\xec\xa7\xe3d\x15\xb1\x1aY1\xa4\xa8\x17E\xfd\xf0\xaeS\xbdu\xba>\x0c\xa2\xe6:\x8br\r\xcf\x8a\xf4#\x9ep\x9bg\x16\xce\x04\xf8\xf18\xfc\xddO5\xad\xcf\x08\xd5\x8an\x8a\x9b04\x85{\x9d\xab\x07\x1e<\xac\xde\x13\xde\xd9\x843^j\xf04\xbf\x81\x98z\x8d\xcf?\xe4\xc2\xe85\x94n\t\xacu\x1b\x94\x88\xdf\x08\x17\xa4\xa7\x91\x94*\xd1\x1d\xf7\r\xa5\xdc\x01\x1dr\x83\"\xf2\x1b\x08\xcf\xdeP\xc6'\xf0*\xf5MA\x18\xfbFQ\x16$\x17\xc4\xd1\xf7\xebI_Y:\x12[v\x16W\r\xe5v\x02\x7f\xdc\xa6\xa1\xca\x15N\x01\x07\xd9b\xa9\xe4S\xa0\\L\xa0\x8c\xe0\xabA\x8b\xb3\x1c\xd4\xad\x9fI\xc6\xd8\x0f\x17\xd3\xdf\xe6u\x16b\xbb\xf53\x89\x19\xfb\xeb\x123\xbe~\x91\xc7w\xcb\xbd7_\xbf-&f\xac\xd3\x98>WFl\x80$\x89\x10\xd9\x07\xbd:\xffTb\xc4S)1b_L\x8cxJ<\xfbg2#\xca\xcb\x99\x11O\xb3\x99\x11[&\x9d\xfe\x13\x81\x19x\x0eC\xf1\x88\x1c\x86\x10\r\x0e]=\xaf\r\xcc\xf0\xfa[\xb4J\x18\x15\xeeTNWx\xba&]!\x1a\x06_\x99\xaePy\xf3\xe0\xb9 \r\xaf\xff\xe9TJZx*\x86h\x18\x1f;<\xa9\xa3\xb2,J\x9c\xf83\x89\nO\xb3\x89\n\xbf]HT(\xfel\xa2B\x9e}P\xae\xae\x8c\xd2\x93\xee\xfc\xe0s2a\xb8\xee\xdbu\xc9\xcf\x04\xca\xb6\x13\x82\xfc\xfb\x7f\xff\x7fK\x0e\xc70"
    )

    #
    # CONSTRUCTOR
    #

    def __init__(
        self,
        background_color: typing.Optional[Color] = None,
        border_color: typing.Optional[Color] = None,
        border_dash_pattern: typing.List[int] = [],
        border_dash_phase: int = 0,
        border_width_bottom: int = 0,
        border_width_left: int = 0,
        border_width_right: int = 0,
        border_width_top: int = 0,
        dash_pattern: typing.List[int] = [],
        dash_phase: int = 0,
        fill_color: typing.Optional[Color] = None,
        horizontal_alignment: LayoutElement.HorizontalAlignment = LayoutElement.HorizontalAlignment.LEFT,
        line_width: int = 1,
        margin_bottom: int = 0,
        margin_left: int = 0,
        margin_right: int = 0,
        margin_top: int = 0,
        padding_bottom: int = 0,
        padding_left: int = 0,
        padding_right: int = 0,
        padding_top: int = 0,
        stroke_color: typing.Optional[Color] = X11Color.BLACK,
        vertical_alignment: LayoutElement.VerticalAlignment = LayoutElement.VerticalAlignment.TOP,
    ):
        """
        Initialize a map representation for a specified region.

        This constructor sets up the visual properties and layout options for the map, allowing
        customization of colors, borders, alignment, and other attributes.

        :param background_color: Optional background color of the map.
        :param border_color: Optional color for the border of the map.
        :param border_dash_pattern: Optional pattern for dashed borders.
        :param border_dash_phase: Optional phase for the dash pattern.
        :param border_width_bottom: Width of the bottom border.
        :param border_width_left: Width of the left border.
        :param border_width_right: Width of the right border.
        :param border_width_top: Width of the top border.
        :param dash_pattern: Optional pattern for dashed lines within the map.
        :param dash_phase: Optional phase for the dash pattern of lines.
        :param fill_color: Optional fill color for specific regions in the map.
        :param horizontal_alignment: Horizontal alignment of the map.
        :param line_width: Width of the lines drawn on the map.
        :param margin_bottom: Bottom margin around the map.
        :param margin_left: Left margin around the map.
        :param margin_right: Right margin around the map.
        :param margin_top: Top margin around the map.
        :param padding_bottom: Bottom padding inside the map.
        :param padding_left: Left padding inside the map.
        :param padding_right: Right padding inside the map.
        :param padding_top: Top padding inside the map.
        :param stroke_color: Color used for strokes on the map.
        :param vertical_alignment: Vertical alignment of the map.
        """
        import json
        import zlib

        super().__init__(
            geo_json=json.loads(
                zlib.decompress(MapOfEurope.__MAP_OF_EUROPE_GEO_JSON_BYTES)
            ),
            background_color=background_color,
            border_color=border_color,
            border_dash_pattern=border_dash_pattern,
            border_dash_phase=border_dash_phase,
            border_width_bottom=border_width_bottom,
            border_width_left=border_width_left,
            border_width_right=border_width_right,
            border_width_top=border_width_top,
            dash_pattern=dash_pattern,
            dash_phase=dash_phase,
            fill_color=fill_color,
            horizontal_alignment=horizontal_alignment,
            line_width=line_width,
            margin_bottom=margin_bottom,
            margin_left=margin_left,
            margin_right=margin_right,
            margin_top=margin_top,
            padding_bottom=padding_bottom,
            padding_left=padding_left,
            padding_right=padding_right,
            padding_top=padding_top,
            stroke_color=stroke_color,
            vertical_alignment=vertical_alignment,
        )

    #
    # PRIVATE
    #

    #
    # PUBLIC
    #
