#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
A specialized version of the Map class representing a map of North America.

This class inherits from the generic Map class and provides geographical data specific to North America.
It allows for rendering and customizing the appearance of North American countries, including setting
specific fill and stroke colors for individual countries.

Use this class to create and manipulate a map focused on North America within a PDF document.
"""
import typing

from borb.pdf.color.color import Color
from borb.pdf.color.x11_color import X11Color
from borb.pdf.layout_element.layout_element import LayoutElement
from borb.pdf.layout_element.shape.map import Map


class MapOfNorthAmerica(Map):
    """
    A specialized version of the Map class representing a map of North America.

    This class inherits from the generic Map class and provides geographical data specific to North America.
    It allows for rendering and customizing the appearance of North American countries, including setting
    specific fill and stroke colors for individual countries.

    Use this class to create and manipulate a map focused on North America within a PDF document.
    """

    __MAP_OF_NORTH_AMERICA_GEO_JSON_BYTES: bytes = (
        b"x\xda\xbd}[\x93%7\xb1\xee_Y\xe1\x17^\xb0\xa3t\x97\xce\x9b\rl\x9b\x8b\r\xc7\x1e\x88\x1d\xb0\tb]\xedf\xda\xdd\xb3\xbb{\xcc6\x04\xff\xfd\xac\xcaL\xa5>\xa9\xaa\xd6\x0c\xec\x89\xf3\xb2\xba\xb3\xa4\xd2\xbd\x94\xb7O\xa9\x7f|\xf4\xf2\xe3\x9b\xf3G\xffg\xf7\xd1\x7f\x9c\xf7/o\x9f\xce?{\xbc\xbf?\x1f_\xee\x1e\x1f>\xfa\xe9\xee\xa3\x0b?|\xbef\xf8\xd3?\xc6\xacs\x86o\xcf\x8f\xdf\x9f_\x9e~\xbc>n\xe9\xbf{\xbc\xff\xf1[.\xe0\xf8\xf8\xf8t\xba{\xd8\xbfp\x19\x7f\xfa\xd3\xc7\xd9~\x12~\xba+\x9f\xc4?\xfftGT\x99\xa9\xd0Q\xa6R\xe9\xa7\xbb\xfcI\x81\xb4\xfcI\xaeT\x9e\xa9\xd8\xa5y\xa6\xdc'\xd3L\xd9J\x85.\x8d\xcaL\x95\x8aP&QZ\xbb\xe3\xb68\xa6\xfc'\x0e\xda\xe99g\xacT\x9a)ig\x98k7S-\xc6s1\xb9&\x1ax1p\xa1\x9a\xd6\x17\x93\xb1\x98Jz\xc8{%\xf5\xd5Bd\xc1T\xd3\xde\x8dDZh\xd1\x95\x9c\xa0\xf5-\xb3\xff\xc4\xf7\xa9\x16+rRQ\x82\xd1l\xf5:zw\xea\xe6\xe1JNu\x92d\xe2\xff\xfc\xe7\x7f^\x97\xc6\x9b\xa7\xc77\xe7\xa7\x97;Z\x19\xff\xf8\xe8a\xff=-\x9e\x9f=>\xbf\xecw_\xdf\x1d\xf7\xf3\xfa\x99\x9f\xfe\xe5\xfe\xf1\xe1\xdbe\xd2\xe5\xf1\xe9\xfb\xfd\xfd_\xce\x0fs\xd2\xd7\xe77o\x0f\xf7w\xc7\xdd\xe3e\xb7\x9a\xed\xf2t\xcd\xf6\xf0\xf6\xfe\xbe\x16z\xbc\xdb\xff\xed\xb2Q\xe1\xf3\xe3\xd3\xcbF\xd2\xfe\xfe\xa5\x15t\xb9{\xf3\xfc\x173Q\xd6o\xe6,w\xcf\x8f\x7f\xd9\xdb\xbf\x9c\xbf\xa3G_\xeb#\xa7\x8f~\xd9J\x9a[\xf4\xd1\x7f\xbd\x9d\xa2w\xf4\x9b\xe7_G\xff\xdb=\xfd&zb(u\xdfr\xda\xa4\x85\x1c\x1e\xa4\x90R\xc2\xfc{<\xcc\xbf\x87L\xff\x9f\xe8\xf9\x85\x9e\x9cw\xf4g\xa2\xdfK{\xe1p\xd6\xb2N[\xe3\xcfC\xbc\x96\xf0\xbc\x91p\xd9\xd7\xbe\xed\x0b\xf4d\xa3o\xc7#\xe6l\x85<mU{/\xa5\xbb2\x97\xe5\xe8}wt\xf4\xeb\xe7\xdf\x83\x99\xbb\xeb\xf6\x86~/\xf4h\xcf\tZ\xcawg)%\x9c\xe7\xb6\x84S\xa0\xff\r\xfd\x9f\xe9\xd7\xef\xe8\x11\x13\x05\xb2\xfaV\xca]\x9d\x00C\xe3\xe9i\x02\x1cM\x80\xa7\t04\xda\x8e'\xc0\xd1\x04\xf8\xa9\xbd\xe0\xda\x04|\xf7v\xa3\xc3w\xa79\xe1\xd7\x92\xf0\x1a\x12\xb6\x96\xe9_e\x02\xdctp\xf4[\xe8\xf72\xff\x9e\xf7\xf3\xef\xfe\xa0\x99_?r\xe6\xe3%\\G\xefh\xf7\xd7\xdf\xd3t\xbc6\xf2P\xf6\xd7\xb1=\x9eS\xeb\xf0\xc3\xfdF\x95o\xee\xb5\x91O?B\x1b\xdfl\xb5\xf1\xe9\xad\x0c\x9d7\xf3\xdcxw\x9e\x7f\xbd\xa1_KO\xa6\x8f\xe7?v\"\"\xd3/g\x9d\xda\xb7\xfa\xc3F\xf1/O\x1b\x83\xf6\xf6\xf5\x87\xa8\xf7\xedS\xbf\xc4\xf1\xf3M\xe5=\x97\xf8\x0fw\x1b\x8d\xff\xfbw\\z\xf0\xe7\xeb*\x89a^\xc3\xf9r\xb8\xb6\xb4\x9c\x8f\xd7\xdf`\xf7\x13d~Y\xe6\x0ee\xae\x1fr\xff\xf3\xba\xe9~(\x1e\xbe\xd8\xf7;\xae0\xf0\x8c\x05G\x19\xf8M\xc7\x8d\x06^5p\xb28\xb3\xcf+\xe9+\x19\x88\xcc\x95\x9c3\xdb\x9a9Q3l\xe5\xd9IRK%\xe7\x8a\\\xcb\xec\xfbTG\xa9S%'$#\xb5\xd9U\xd9\x80\xeb\x052\x13\x99\xb1\x91@\xba\x9e\x9c{\xe4[\xf7\xbbw\xb9\xfb\x90\x1a\x88T\xben\x89\xf4 [\\\xc9\xb8J\xf28\xfb6)\xb9'#\x91\t\xa7\x0c\xde\xb5}\xea\x84\xa4\x93w\x0b\xc8[&\xd46;j\x06\x90\xb6\xcf\xec\xfa\xa28\xd5\xa1<\xe1\xdb\xbb<\x92\x11\xebuM^\x0b8\x83\x95\xf4\x98\xd9\xb6\x92y]\x95\x9e\x8c\xb8\x9eM{7\xe3\"l\x8b\xff\xa6 \xf3\xd5\xf5{~\xda\x7f\xfbv)\xc7t)\x9bb\xccZ\xaem)fY[\x15b\x96)[2\xccW\xbf_\xc80_\xfdr!\xc3|\xf5\xcb\x9f\xad\xc90q)\xa5\xb4\xedO\xfe\xdf\xb7\x8drU\x86\xd9\xe7\xa5x\x02b\x0b\xfd_\x12\xc96\x86\xf2\xb3ls\\\x15d\x96\xfd\xe61^y\xfe\xbc\xfe\xbcI1\xdc\xbb~\x13\x1f{G\x8d\xa9\xbd\xe3\xd1\x18e\x99\x95\xaa\x9b(s\"\xf9\xa4\xa0\xacB\x02\r\xcb0$\xe2\x1cH\xc4\xa1\xf1q\xc7\xb0-\xcfL\xa3\xc0\"\"\x8cmr\xcei\xda\x14d,\xcd\x82\xbb\xa0\x8c\xd2d\x17\xfe\xdf\xd0,\x10\xcf,S\x1c$\x98e7Y\x80\xf9\xea\xca\x85\x87\xe7\x1b+\xb4\x89/\xb3@;\x0b+$\xb8\xb0\x103\xd1\x13\xbb\x10_\x0e\xf6\x98YX\xb9\xfe\x9f\xd2u\xc4\xf6\xc7\xcbq\x10\\\x96u\xbd\xb9_o\xdb\x9b\xd6\xb6k/\xa7\xb3\xc1D\x90]NK!\x81$\x89\xa9\xfdOR\x81\xa7\xcfb)\xba,[\xf4\xf2\xb4\xde\"\x10\\\xa8\xd2\x10\xff\x8dJ\xdf\x0e\x9f\xec{-\xe7QbY\xb6Y\x05\x96\xa3;\xb2\xc0q\x15D\xec\xf1Zx\xf2\xa7\xe3\x8a\xa8\xb2\x9e\xef\x03\n)\xc9\xd0>]\x84\xb7\\\xc9H\xa4\xad\xe4\x9cZu\xff+Y\x88\x8c}\xaa\xebIfD\xc9\x12_\xaa\x16\x86D\xb6\x88+\xc9\xfc!1\xe7\xd1T'%\xcb\xbb^R]%\xb9\xa8T33\x19j\xc9\\\xaf\xaf\xa4\xc3\xcc\\o5X(\x19\xb0\xa8jkH\xccZ\xaby\xe1\x9a9\"\xd9\xc6\xea&O\xfbb\x7f\xf7r\xb7\xe0g\xfat\x93\x97\x8d9\xb6\xf9X_C\xe5a\xfd\xd3-\xfe\xf5\xc5\xa7\x0b\xfe\xf5\xc5\xab\x05\xff\xfa\xe2\xd5\xaa\x0e\x9e\xda\xd2g.\xc6z\xaa\xdf\xaf\xf0)\xda\xa5\x85\x1fe\xda\tg\x95i\xe6\\\x03\x0f\xea\xdb\xcd\xe32<{\xae\xcfh\x939\xad1\x1fh\x9a\x05F\xc4\r<\x1e\x07&\xf3\xc5\x9eJ\xba`%\x8d\xcb0O\xd97Uy\x7f^\xe1 '\x0f<\xa2\x80F\\V\xf8\x85\xa3\xd1\xa0\x9d\xa2X\xcf\xfa\xed\xc0\x11\xfa>37\x18\x9e\xad\xcc3p\x01R\\\xe7A\xbe\xfeo\x96\x8ak\xf0\xb3\xe2\x9a\xe6\x9d\xffd/\xd3\xb0\xe7\xaf\x8c\xc8\x9b\xfbeuoV\x9a\xd0\xf6yQ\x06C\xd3\x15\x89C\xd6\r\x96\x9f\x1c\x96\xbc`G\xfb\xb5\x83}9\x83\xa2\x99\x07f\xd0\xd7\xfe\xf2\xb4|\x06L\x00\n\r\xa9\x15\x1a\xe2\xcaf\x7f\x045\x94\xd5SX8\xbc\xa9\xf7\xf5\xd4\r=\x9e\xd2\xb5\xec\x90\xdc\x8a\xbe\ti\x1f~\xe3\xd6\x1ds\xb1\x17\xaf\xee\xd4\xf9\xc6.\xbf\xe0\x01\xb2\xebM\xb2c*iq\xc7\x9cH\xcd\xa9d,\x92\xd9U\xd2\xf6\xa4#\x92\xdb\x1c\xb3dV\xd2A\x9b\xafd\x02\x0e!E\xd5]\xfeJF\xdc\xf4\xb9\x19@\x1a\xe4.S\xcfl\xa6\x9ea\x18yW\x87nf\t\xa9\x1f\xba\xd4\x86\xae\x0e\xfbM\x1e\xf0\xf3\xc7\xef\xef\x1e\xae\xfc\xffaw\xdd\xe1?Y0\x83.\x99\x18\xc0\x823\xdc\xcc\xb2\xcd\x1a\xd6_\xeb\xf8\xc4\x8d,[L\xe3\xe7_/\x98\xc6\xcf\x7f\xbb`\x1a?\xff\xed\x97+L\xc3\xd2\xe7\xe4C\xdb\xa5\xc5\n\x04&[[v\xc0Y<\xfd\x0f\xa2\x95\xbc\xbc\xbf\xa5C\xf9\xb8\xa6+\x99f\xef\xdd\x9f\x9b\xde\xb4\xa5C\xed3\x19 \xf7\xfbf\x1af\xb5\xaa\x1c!\x93oEp\x1e~R\xf3\x0f\xec\x8d\x87\xfb\xf5\xfe\xe1\xee\xf9\xf8\xdd\xb9\x8e\xf9\xeb\x81\xe1\xdd\x98\x14\xe6~\xd7\xc7\xf3\x06}\xd9\xcfi\xfb\x9d\xe6_\xd3\xc4n\x0f\xf9\xf1\xb8[\x1f`\xde\xf8\x96j\x1c\x0c-s\xcf\xaf\x89U\x14j\xe7\x7f\xbf=\xefN\xd2\x98\xbb\x87\xf3\x1a;\xf5M!;\x1cA\x81\xdbP\xe6\xf8\xf9\x9el\xc1\xf5\xe5\x04/\x83\xa5\x9a\xf5>\xd1\x01=\xd8\xb1\xcd&\xbff-\xef\xecA\xcbck\xf6qi\xc0\xde\x01\x9bw\xf0\xc2\x19\xb2.\xd5H|\xb2_S\x1dM\xb3\x81\xdbsS#\x97*\xa5\xe5\xe5\xc8z\xa4u\x0b\xfd\xf2\xd8\x0c\xe8\xf62H\x13u\xd9\xdd\xed~M\xeb&\xfe\x9d\x85'\xf3\xf4\xbc\x7f\x16]m\x106\xea\xd2\xdc\xd5w\x07\xc1\x83\xd27\x97\x1f\x08\"\xa4\x82\x9e.\xbdj\x1aL2\xb3\xf55\xcf*M\xbc\x9c\x96\xaa)M\xe1\xe1p\xc9\xa8\xa6\xeeHC\xbd\x0e\xc7)\xce\x86\xf7\xfd)\x9e\x06\xc9\xa5\xb6e\xff\xf0\xac\x9f\xd7\xf9\xf5 \xc5\xe8W8\xea\xae\xef\xfe\xae@\x8d\xf5\xcd\x14>kf*\xc6l\xa8\xb7\xfc\\\x04!\xd4@/\xbb\xff\x8d\xb44\x88C\xda\xb5\xe7\xd7\xfb\xdd\x93\xcc\xe1\xf9a\x10\x90$\xd7\xeego\xbf\xff\xee\xed\xd3\xdd\x8f\xe7uqi\xd1C\xd1\x9f7ti\xec\xa1?.D\xba\xf7\xefa_\xf4\x8al\x86{\x1a\xcbi\xfd\x9e\xc6\xcfig\xcb\xf9\xdf\xd8\xd9\xc4\xcb\xf0_o\xcd\xf9T\x1e\xbe\xdd}G\xab\xc2\xb6\x15\xb1\xd4\xe2\xcb\xec\x1aI\xc7\xe4PS_\xd1\xe2)_\x8c\xd3\xb9\xcb\xf7!]\rEl\xc0b\x8b-b\xd5\x16;n\x163\xb6\xa9d@\x83q\xeeM\xef\xb9\xb7\xb5'\xb1\xe2\x17\xf4\x0fh\xe6$\x96x\xdfg6}\xc9\x16\x9b\xa1\x1e\x80\"6\xfeP\xc9\x0cda\t\xce\x89L\xaad\xa9\x99\x13\xf6\xa8H\x8fl%c?\x1aupn\x8al\xbf\xb8\xdf}\xb3\xbf\xffa\x7fz|Z\xc8kC\xda\xa6\n\xbf\x9eo[Z[\xab\xb3\x8aiki[\xf2\xd9/\x96\x8e\xf5o\xfe\xb0\x90\xcf\xbe\xf9\xcd\x1f\xd6\xe43\x10\xba\xd83\xc7\xff{\xb0s\xa1\xac\xe0\xcc\x8a\xa0\x95\xd9\xc8lw\xcd\xb3.ViK\x02\xd2\t\xc4\xa7\x08>\xf8QXZ\xeb4\x0f\xf3j\xca\xf3V\n\xc8B\x8b\xee\xe1\x93j\xba\xbb\xd1I\x16y\x96u7\xf9\x86\x8d\xcc\x07\xf6\xa6;\x10d\x0e\xf4k\xe1\x89\x07W\xfc\xaa\x8c\xc2\xc2\xc3q\x07n\xf6#H\x1ea\x94E\xceyE\xc2\x98\xd8 \xcd\xb3\xc1nv\xb1N\xdb&18\xb0Z\xa3H\xe2F\xbb\xc4\xa2\xeb,-\xac\r\xfb\xdd\xe6\xcam\xd2\xc1,8\xbb\xe9Lr\xc1!\xb4\xffOS\x93\x1d\xce+~w\xb2\xae\x1c\xa7y\x08\x0f\xc7\x99O\x89\xbcP\x16~\xf7\xb5\xfaY\n\xb8>\xfe[\xffx\xb3\xc1`\xd40\xc0\xef\x0e\xc0\xe9,<\x07\xbe\xe9G\x06\xbdV<3\xe5\xb5\x94\xc6\x8c\xff\x17\x15\xbf\x1d>oa\x80$\xd1\xcagP\x1f\x8d\x9f\x01rO\xf8\x0c\x98?\xae5\xb8\xf2\xc3\xecg\t6\xd0\xdc$\x7f\x8e\xcc\xf9\x96\xfc0\x87\x99\xf9&;7\x1a\xf2}@~X\xac\xb8H\x85\x99T2W\x92\xbd\xad\x06S\x83\xf0\xa5\xc2\x1a~\xac\xa9l;\x00\xd2\x13\xe9+\x19\x89T\xae\xc5\xef\xa6Z\x94!\xb2T2\x90)A+\x9a\xd6\xc8\x8c\x1c\xaf\x92\xc2\xd2\x81\xb4DNH\x86\xca\x1e3\xf1a #\x91\t\xd92\xa4\xda>5\xc1hh\xc9\x06I\xdf\xf8\xb0A\xcf\xf4B\x0c\x19\xf8\xf0\xc0\xa5\x91\x87\x0f\x1c~\xc9\xff#\xa6\x1aj\x06\x90]Qm\xf6o2\xfc\xcf\xdf^\x97\xcc\x95?/}\xcf]\xca&\xb3_\xcb\xb5\xcd\xea\x97\xb5UF\xbfL\xd9b\xf3\x9f\xbfZ\xb0\xf9\xcf\x97\xb6\xfb\xcf_\xad\x99az\xbf2\xda\xee\x9b\x98\xdc\xd9_V\xf1s\x9b~e0\x83$\xb0\xb6\x80\x0c\xb0\xf0=/\xfb\xcdc\xbc\xf2\xfcy\xfd9 \xe8.\x1b\xbd\xbb\xd5/\xe6\xec+\xd55\xd6\xbe\xe2K\x06\x00]hV\t\xf1=\x1f\xb6\xad\x0fK\xbfr\x06c\xc2q\x1b2\x97\xd6y6C\xe6|\x023\x02\xf0\xf8\x05bn\xd9Mf\xe3+\xcf7V%\xc0\xe5\xc4\xbbL\x0c;\x12\xf3>\xb3\xefy\xc1\xb6g\xef\xf2Uw73\xe7:\x94\xb9\xe3\xb3\xd7y`\xd8\xcb\xba\x98]\x7f\xfe\xb7\xc5\xf3\x8d\xb6\x81\x8e\xeeF\x16)^\x84\x00Z;\xb0\xd4\x85:\xbd,\xfcec\x9d,\xbc\x0c\xffN\xa5\x00\x91\xdbX\xc3\xc7\xe3;W23\xe7e\x0bUUud{Is\x1b\xae\x02\xf8\x91\xdd\xc9K\xd6<\xbb\x975\x1f\xc1\x1a)\xdf\x87TU\xc9=kU)$k\xbbU\x95\x91\xf6y%\x13\xf1\x08k\xab\xf1\x9dTF[\xe1J)\x0b\x19*9\x11)v\xfc4s\x10k\xaaW\x83\x10f\xb6b\xe6\xae\xa4E\x92\x10u\xb6\x02\xec\x12A\xcal\xc5\xea%\x82\x85!i\x89T\xef\xb4#\xd2\xd4w'\xf4\x97\x84\xde\x9b\x12%\xf3T\x1b\xd99W\xb8\xcd\x15\x1b~%\x03\xd6\x9b%U\xc9\x80m&\xc6m+<K\xc6J\xd1Z\x93\xf47W2\xd0XMu\x162\x91\xb6\x82\xd0-\x8c\xb3\xa6F8Up%\x13`\xc8\xda\xbb\xe4\xc9o\xa4\x97Y(\x95\x0c\xd8\x0c\x1a+L\xf5\xfd\xbb\x16\xebuRo\x06T\x9cu\x1dp\xbe-\xa4\xb6\xccn\x03\xe9\xdf\x1eV \xf4\xf2p\x1b<\xdfg\xb8\x01\x9b\xc7\xe2\x150\x8f\x0f7\xa1\xf2K\x98\x19<R\xa8\xfc\xef?{\x07T\xde\xe6w\x80\xe1\xd9\t\x92\xc9=\xbd_\x07\x89\xfd\x1a\x1b,8\xf7\xee\xd1\xf3\xe2\xd1\x88m\xdfh\x8c\xa0\xd7\xbb\xc2\x06\xdc\xba0\xdf\x13\xaa\xd2\xef\x80\xa7\x8bA\xff\x06\xfe\\L\xe6\xf4?\xb1K\xb2..\x99f\xd7s\x01\x98w\x8f\x96\x13\n\x8a\xee\x89x\"\xa9\xb83\x13\x9cU\xdc\x158\xf9\xec/\x98\x15\xda\x11<\x8e\x85\nl\xbc{\xb4\xacz\xc4\x8awV\xd6\x91\xcdu\x85\t\x0c\x9cL\x9e\xc7\xc36\n|\xa3\xc4\x11\xe6-\x1ck}\xce\xc5\xd0\x8a\xd5765\xfbe\xc2\xe9\xe2W\xac\xa8-\xed\xc3\x82\xb4\x07\xe4\xed\x80\xcb\xedP\xbb\x03\xa6w\x81\xf8\x1d\xf0\xc0\x03Zx\xc0\x12wH\xe3\xdb\xb0\xe4\x05\x86y@8\xa7\x1b\xe8\xe8\x01;\xbd@V\x0f\xb8\xeb\xe1\xdd\x01\xb3\xdd!\xba\x07\xbc\xf7\x02\r\x1e\xfa\xcc\x83\xa5x\xb0#wV\xe6\xc1\x06\xbd\xb0P\x0f\xf6\xeb\xdc\xdb\x82;\xdb\xf7\x86e<m\xa8\xb3\x9d\xb2;\xa8\xc2\x0bE\xd9 \x99z\x9d;\x89\xce]p4B\xeb\xafG2\xca\xbbJ\x0e\xa9\x06\x8b\x8a4\xec\xb1Mw\xeaI\x8f\x99Co\x18\x08\xfd\xbb\xbe\xb7\x04\xf8\xbe^/\xf5\"\xfb\xbd\x92\x1ea\xe7\xa1a\xc5\xebwt\x1b\x1b\xf7\xf8pz\xfb\xb4\x7f^\xc2\xe3 a\x1b!\xb7\xcct\x03$7V\xa58\xb91a\x13*\xf7\xdb%T\xee\xab%T\xee\xab\x9foB\xe5\xd8\xe3\xd4\x1b\x96\xf1\xa8\xd7&l\xaeC\x17\x98\xa6t#\xc8\xfb\x90G \xdd\xd81\xc1\xd2-\x1e?\xaf>\x1e\xc1t\xef\xddx\x81\xd4-\xaai\x9c\xfc\x02\xfe~?r\xf5\xea\xc4\xdf\x86\xd8\x05\xf0\xabw\xb6o\xb6\x92o\xc3\xed\x8e\xcdq.<\xdf4H\xb6\xb8\xd2\xf3\x08\xc3\x1b\xfb!H\xbc\xc5\xe3\xf5\xc5\xd4\xf8\xff\x89\xcc\xda\x17>Z\x96\x9b, \x08\xed\xa5\xae|$fy\xf0a\xaa\xd8\xec\xf9\x98\xd9\x88\xd3\x1b+|\xb3\xf1x\xbdy\x0bE\xf9\x0c\x0el\xdfx<\x82\xa4\xbd\x19\x01xc\xa9/\xeb+`\xa1 \xffk\x95\x8d\xc0<\xf1\xe8\xc6\xa5\x95zuY\n`olT\xc3\xec\xcd\xeau\x99.\xa7\n\xae\x9e\xcf\x84\xad\x08\x1e\x87\xd9\xb9\xb5\xc8\xf7\xaf\t!_\xbe\xbd\x7f\xb9\xbb%\x89\xfc\xe9ccI\xb1\xf2\x15\x0fg\xec\x84\x94!v\xeb\x8bP\xb1\xa3\x1c\xfc?a\xca\xd4\xbd5\xf9O\xba2'\xb2W+]\xe80X#\x03\x93\x1dQ\r\xdf^\xda\xaa\xe4\x95\xdd\xf9\n0,\xa4\x12\xfb\x8a\xc1.NRc\xb5\xcf2\x19\xaa\xf5\x96I5\x04g\"\x1d\x18\x91}n\x96n\x87d\x9e\xb9P\xcbL|\xd5\xa7\xc6*\x03\x91\x11X\xe5\x95\xc4#\xde>6\xee\x97\x89\x8c\xc0\x1b}\xec\x0e\xb3\xfbj\xf7\xe7\x93\xeeH\x9a>3\x93\x0eTb\x1f\xdb\xd1\xa8\xd8\x93\\\x11\x8a\x9c>\xb4#\xfcL:Pk\xbd\x8a3\xe4\xc9\xf0\xaei\xe6\x9e\xc8\t\x0e\xfc{8\xdde\x884=9\xc19toZ\xbd\x89\xc8\x84%+i\xa8\x19\xaa\xa8\x93q\xc1\x83]\x86\xeb-H\xd6V\xa5B\x15U\xc10\x91\xcc\xe5]3\xe2$\xe8QJ=I\xf2)\x92\x81Fc\xaa\xa4'R\r1<V\x19\x8c8\xbe\x8a\xf6\x89\xcc\x07\x8d$\xdf\x0c\x92\x1e\x86=\x91#\xa7\x91\xfc\xd9\x84\xda\xe6I\x8a*\x80E\xf5\xd5\t\xc4\xc0\xd5\xeb\xaa\xf3\x15\xd6Z`\x11\xc6\xdc\xa7\x92\x98|%\r\x92U\xc6\x8cI\x9a\xa1\xa9\x13t0f!\x15lkpp&Y*\xa9\x92\x91H\x03 \xdf6e\x95t\x80\x9f\xd5\xb5!E\x01ia]]\xeb-\x94\xda\x8d\x86ix\xe1Hd\xc0\x81\x05\x92\xdfup\x94\x03IN\xd53#\xf3,L\xed\xb0\x86\xa5T\x83Ei*O\xf7T\x9b\xc1\x1b\xe1\x82\xcc`\xd1\xbb\x92\xa1O\xf5`\xfes\xa53\x0e\xba\xdc\xec}\x13\xa5\xaaa\xd1\xf6d 2\xc0\x12u\xa5\x99\x1d\r\x15\x95{\xd2W\xd2\x13\xa9\x99\xe7zS3hND:\xb0`\xba\xd4\xcc\x8e\x96\xdeu`\x85tz(\x86\xbe#\x97\x9b\xa9\xd4\x11i\xea78\x11i15u\x06M\x97\xda78g\x8em4\n\x91\x11[\x15\xda\xf7;\xf7H\xbf\xd0$d\x00s\xa7sm\x1b\x89H\x16I\r\xb0\x8d8\xd7v\xa4\xb9Uz\xacy\xa2f\x80\xb9sN5\xcd\xbe9w\x7f\xea677\xb5\xcc\xb3\x8d\xb2\xb4\x92g;cn%39Ur6w*O\xa9\xa4E24#\xac'RS\x13\x91\xa6\xd6k1\x95ZeU\x1b3\x92\xb9\xa0\xc56\xb6\xedzNM]\xc4\x18\x9bZ\xe6Dm\xc6\xb80\xb6t'wmil\xd1\xd0h\x18\xb0L\xd8>8\x8b\xad\xdf\x02\xdb\x1e\xae\x99-(\xb2njJ?\x8f\xb3\x03\xde\xddRy\x06\xa7\x8e\xef\xb7\xa2H\x8fo\xad\xaad\x80wm\xe9\x94\xfe6e\x04-\xd3\xd42If\x957\xb0\xfb,\x8dh\x8f\n\x9b\x95k\x7f\x8b\x93ws%3f\xf62t\xa1\x8aL\x91\xc6YR\xa3\x90\xae\n[\x86&%W\xd2\x13\xe9\x91T$\x01\x19Wt!\xc9\xbbu\xf6\x0b\xfb\t\xea\xba*Y\x16\x83\xf4\x88\x18\xaeU\xc8B\x91\xa2r%\x03,\x95Y$4\xd4J\xa3t\x80N\x98\xa9~\x0e^\xe9\x04#b\xa6:`\x95v\xb2`&\xa5\x0bL\xc6,\x92\xe2\x00\xcft\x82\xf5f&\xdeO'M\xa7AuF\xd3\xc9:\xa2\xdf\xf2L\x07\xa2k\xfd4\x18+t\xad\x9fF\xa7\xd1\xc6\x0c\xb4\x93\x9d\xa4\x8a\xe1$&\xea\xce2\xd3\x89\xe8\x84\x82\xb9\xabN\x8bYl7]~\x9aI\xe7@\xac/D\xd7\xfc\xfcU8-\x8f\xac]\xcek~\xde\xf7\xaa\x903\xd3\xbci\xba\xaa4\x08\xed\x95\x8eDG\xa5y\x0b\xb6B\xd3\xfc\xb9\xa84\xcd\x9fK\x9a\xbf\xd22~\xd6\t70J'`a3]\x88\xae\xf9\xc9\xa8\xe4'm\x9f\xef\xd8t\xa5u|-\xcb\xdd\x16\xdeG\xb9e\xa6Y\xca\x89X\x9f\xcab5\x7f\xdd\x81k\xfd\t\xdeO\xa41X\xa5Y\xbf\xf0Z\x9e\x01\x05c\xee\x7f\x04\x89l\xa6\x1d\x08l2>P^S\xe2\xfe<?\x99\x15\xbb@\x9c[=\x843m\xc1\t8\xd3\x06\xbc\x80&\xf8\xee\x90\xea\x92\x0er\xac\xca*\x1d\xe0\x8c\xebL\xf3!\xa5\xa2t\x19\xd2\x0b\x1c\xcb2!J\xfdIivaf\xcc\xdf\xb5\x97i;\xd0N\xe9<\xa4\xd7\xfe\xb71\xa1:\xec\xa4uDqnF\xa5\xfd@\xc7!?\xbbh\xcb&\x8d\x1e\xdd\x99vCz\xad\x1f\xdb\x94\xc1)\xdc\xe8:nI|\xaa#=\xe6\xb7Pf\x16\x17g\xd2w\xca@\xb3\xdb\xd6m\xd2\xb9\xa3\xb38T\x1b\xcdm\x08\x03\x1d\x95\xae\xf5\xb76\x95\xce\xcf:\xd3\x0e\xbc\xc13\x1d\xc0-;\xd3\xecy5J\xa7\x81\x8eCy\xb5|\xad3\xd2\xbe\x1c+\xb77\x91\xd6h\xd45\x13i_\x0f\xfa\rD\x92X\x83\xae\xc1Hc\x0b\xe9I\xca\xb3\x9a^\xcbo\xfd$n\x1d\xf4[\x0e\xf4-\x87\xa4\xfd\"\xf59\xe8^\x17ho\x0b\xfa\xad\x07\xdakB\xd4\xb9\xa2\xbd)$-\x8f\xf6\x92\xa0\xbc3\x908\x10t\xaf\x84\xfa\xb5M\x9e\xf8K\xd4om\x83\xae\xe6\x80J\xeb8\xf9\xa9\xa7]aZ(\x9a\xfd\xa0\xdc\xd4\xd1(\x05\xdd\t\x1c\x8dR\xd0\x9d\xc4\x05\xa1\xa5\xc5\x8e\x94\x99\xa0;\x83#M\x08\xde'\xab\x7f\xd0\xdd\xd2Y\x19\x01i\xbd#n\x12\xa2\xd6O\xad\rA\xdf\xaf\xb4Sz\xae\xdfk\xfd\xef\xa2I_\x0c\xca\xbdf\xbeLtP\xdaQ\xfd^\xdb\xcb3f\xb5?\x86\xdao\xb4\xff\x13\xd1V\xc7'v4}}!k\xf9E\xc6'\xe8l\xf0\xaa\xac\xb3G+,\xeaN\xef\xbd\xccV\x9dM\xda)\xe3\xa4\xef\x93t\x10uW\xf2$\x9d\xc4I\xcb\xcb\xc3\xfb\xb9\xffJ<}\x89\xad\xfe0I\xba\xf4?\x18I\xb7J\x97.?\xd99Z}\x81\xf4\xa6\xd8vM*/\xb6\xdd\xc6H\xfb\x13~Q\xad\xbf\xf2\xc5\xb4\xfa]\xbf~\x82\x97/\xa2q\"\x87\xeb\x87w\xe9\xa0\xdc\x95wq\xf8\xe2h\xf7\x0b\x11v;O\xf4\x84;Q\xa8Z\x92\x89\xd4\x1f\xa0I\xa9\nAw!+\xb4\xbc\x1f\x89\xfb\x87\xea\x1f4\x918oPi*\xf2\xf7\xa1\xd2G\xe4/\xbe\xbdO\xbaT[\xdf\xd1Hz\xd1\xf6Dlo\x9c\xa4?^\xfb\x93\x86\xfev\xeb\x97\xb9\x03\xecXI\xd6k\xe3\x16S7\xdeY\xc6;v\xe5e\xdd\xa1\x8a|\x0f\x1ew\xf6\xf6~\x9c\xa4\xbc6\x9e\x9e\xe6\xd7h\xffx?H\xda\xffi\xa0\r\xee'\xb1\xae\xbf\xa8\xe9\x01\xd7\xb7\x8cg\xdb\xf5\xfd\x82K8\xfc~\"I\x16Q9r$)\"\x1aH\x9f\xdfWN\xc5\xf3\x17U:\x8f$\xfdE\xa7\xe9T\x7ft\xcaUh=@:\xed/\xd1i\xf9d:\x00z\x92\xf7\xb3\xa6\xcf\xdf\xaf\xd7\xf28\x7f[O\x9c\xdfC~\xdf\xd1V\xf2\xb7\xf1\xe2\xfcN\xdb\xd7\xa7\x07\xa9\xcf\xebxpy\t\xb9h\x0c\xda\x1f\xfa\x9eb\x80\xf4D\xdc\xa7\x8dW \xba\xad\xf74\xd0\xb9\xcbo\x86\xf7\xad\xbc\x9f\xba\xf7\x13\x94\xef\x89\x8e\xdd|*\xc7\x96\xf6f\xe8\x8f%\xbat\xef7\x9a8n,\xf0}\x17\xa2\x0b\xae\xbf\xd4\xd6\xd3$t[\xff\x13\xd1\x05\xf7\x9b\x84R\xe7u\xbd$\xd56YrO\x06\xf6?G\xf9\x93\xee\x7f\xa5+\x8f\xf6\x1b(\xcfJ\xfe\xa8\xfbm\xc2\xf6\xf0\xfe\x9e\x90_D\xa2\xad\xf2\x13No\xfc\x86\xeb\xab\xfc\xc2\r\xf9\xe9{l\xe3qC\x1aQ\x89\x85C\x1e\xb45\xcd6\xdb\xa8\x1a+[\x87\xa3jh\xb1H~\xafk\x8ci\xa7t\xff\x8d\x15y\x7f\xd2\xf2\x1c\xa6\xb3\xb5\xbd\x95\xc7\xf1)Zy\xc9Hy-\xbd\xb6\xf76\x16\xe1\xf7\x0fw/\xe7\xd3\xee\x9b\x97\xd9\x0b7\xc3\x0b>\xfd\xfe\xfc\xb4\x16R\xb7\xcb\xb8\x00(\xdc*\xe6\x9d \x85E\xd1\x1dR\xe1\x9d-\xdcB.\xfc~y\x1e\x10\x1e9}\xf4\xe9;\xce\x03\x8a\xc7\xd5/\xc2\xfe\x08<y\x11\xb5AP\xca\x9c\xe9\xd4\xf0\x03\xb6\xacEg8\x8f\xd1\xeb$\x1ao\xc0H\r\xbbv\xd8\x801\x10\x12\xc4\x17\"/t\xa8\x884\x86\xf5\xdd\x88\xcb\xf0\x87\xf3\xd3\xf9\xee\xe1\xee\xdb\x97\xf3<\xc0\xd7\x11~\x18p\x13\xef\x1c}9~\xf8\xfc\xb2?=>\xef\xae\xd9\xaf\x7f6O J8\xa24\x0e\xa8\x0c\xe2\xd6\xc0\xf9\xc4cl[\xa6\xf7\x88\x07<\x03\xf9\x8e\xe5\xda\xf0\xe7\x8f\xaf\xcdZ\x85d\xa4\x06\xc98\x168\xc7p\x02\xa8\x06\x9do8Z>\xd88A\xf0\x85C\x8b\xdc\x80G!$\xf8\x02\xbe\xc6\xb1\x94$rC\x00\xb8\x07\x84}\xa0up}a\xf3<\xa4\x00=\"@?\xf6\x18\x9b\xc1,\"\x10\xaf\xc5]\xe0\x83\x90\x93\x05\xec\xa7Y`B\xfd\xeev\x94\x85];_\xc9\x87.\xf8\x00\x86\xbcp\xb9\x11\xb2\x98V\xcf\x1c\x8f\xe1\x17\xdf\xfex~f\xa8\xe5\xfd\xcb\x8e\xa6\xca\xdc\xdf\xef\xbf\x7f|=@N\xe4\x8d\xdd7\xf4\xe7e@\x9e\xcc\xeb\xf2n^u\xd7\xdf\xd3O\xc6\xb5\t0T:\xa61\x9f%mQ\x8d\xe78\x0c\xd3u8r\x9e\xd1\x14\xeb\xd1\x18(\x0e\xc3J\xb8\x85\xf9\xbb\xb9~6\xa73\x7f\x19\x0f\xbb\x1f\xf6\x0f;i\xeax\xdc\xf2e\xff\xf0\xe3\xee\x8f\x7f=\x9f\x1e\x1e\x8f\x7f\x7f\x84\xe8 r\xeer\xfd\xd3\x19\x8f^\x12\x84\xd4\x9b\x11\xc1\x8a\xdb\x17\x03P\xea\x80}v\xf7t?O\x8e\t\x979\x9a\xc5\xf9\x87\xfb\xf3\xcb\xfd5i\xfb\x8c\xe5\x05\xe0)\x07\x00\xa6$8\xc5!!\x10v\xadI~y\xe0#s(\xab\t\xce}\x04\x80\xb8`\x1c\x87\xbc\x02t\xc1o{\x19\x0f\xfch\xc6 g'{s\xf7\x90\x80\x08\xb2\xeb\x84\xf7\x0c\xb5\xfc\xc5\xe3~\x0e\x1ab\xce\x17\xb7@\xcd\xa4K>\x0f+\xa6\x9d\x1e\xe1\xb44\x1d\xfe\x7f\xa0c\\\x8fe\xb1\xac\x9a\xa9\xe8c\xd9\xa0\xa4\xa2\x94%\x8fJP\xd1\xcb\x92j\x1d\xd4\x10g\xc90\x17\xac\x1a\xaaY5j\x86h2\x92\x05\x07\xa6\r6M\xbc\xbf)c4}\xf4\xa6\x91\xd1t2\x9aVF\xd3\xcbh\x9a\x19M7\xa3ig4\xfd\x8c\xa6\xa1\xd1t\xd4\x19\x96F\xb3\xd3\xc2,\xb5n\xb6\xda4s\xb9\"\x82h\xd7\x9a\xa8\xb3\xc9\xadm\x82\xbd\xa3\xd9\x8ch\x98\xf2D\xb7\xd1*]yF\xd2\x8b\xce^FA\xd8\xd6\xfa\xdbj\xf0(8\xdb*\xf8\x07\\=M0\xb7d\xe2n\xf5\xb1\x9b\x01\xca\xa7\xd9i\x8a\x08\xbb\x15Z{-\xcd.\xb4\xcfH\xff\xb3\xbau\n\xbeo\x92\xf4\x7fR7\x13+B5\x7f\xe8\x15#C\x8aGS\x9c\x0c+\xce\t\xd2\x03\xd1IQc\x99\xe8\xacn\xb1\x82\x8a\xd8D\xb0\x95\xa8\x86\x99)I\xfe\xa2\xe9\xac\xa89t\xb3\xc5\xe66$\xc3\x04\xd0C\xfb'n_\x067\xa0\xeb\xeaw\x92\xdf\xa8\xdb1\x80\xe2X\x8a4O=\xa0\xbe'#\x0cF!#\x8a\xd6U\xd8fPm~B\xea\xb1\xec \x0b\xd1\x80\x8b\x17I\xd67M\xcd\xec`\x95\x17Z\xc4\xaa\x9d\x15R\xc6Su\x170\xa2.U+{aU\xd1\xb4C\xeaL:8;\xae\xeb\xabXY\xde\t\x0e\xa9\xebj\xe4\xd3\xd21w\xdeq\xfdvr\x96\xa9\x88\x95\xc4\x95\xc1\x10\xba\x98\x9a\x0b\xbfK\rRr\x86\x93\x08\xb1\xc7\n\xe8\"f\xec\x9a6\x92\x11\x0c\xb14@\x03\xae\x90l\xa5U\x063\xa7\x06w\xf0`7\xe0\xb3_\x11\xe0wlup\x80BP#\x05\xe3\xe7\xd5&\xc1\xc7\x16\xa2\xe2*\xc8\x87\xa3\x16\x17>\\\x10\xe14A!r\x82\xf3\xfd\xaa\xfa\x96j\xed\x99\xea\x1c\xb1q)\x03V V\xaf\x8a,\x95j\n`\xc4\xa4ZNy]\xe9n\\\xaa\xe7@\x17\x03:\x16\x8a\x15+)\"\x18Bj\xe8\x07\xf6Z\x14\x98n5\x99r\x7f\xd5\x02\xc9\x83\x13B\x03p \x93b\x90\xbf\x9a\xdb\x19}\x18B;c\xe7\x81\x032\x181\xe8\x89\x0f6\rV\xa7Pb\xcb\xb9i\xa4!2\x00\xb8'\xd8vx\x91\xcd\xb0z\x0c4\x83U6e1\xda\x1a84\x19*\xfa\x81\xc1L\xda\xdfT-\xc8\xb6fv`\x80N\xbd=TR\xe18f\xa69\xca\x00H\x8ap\xc0\xd4\x83-1US\xa3\x9e\neR\xc1j\x99H\x0fP6\xb5K\x8a\xc9\x02`ph\xd5\xe4X\x97#9\xb5\xc0\x98\xdcf\x8d\xa2\xe9\xa1G1u\x0e\r\xf1\xaaij\x10'\x1b\xf77\xd65)\x08\xc2j\x8c\x15l\xa2\x95\xb1\x92\xa2x~\xab\xa9Z,\xdd\xf5\x1b\x8c,\xfd\xd4IQ\xbb<\xbf\xcbN\xc9P\xcf\xdd\xb0\x1fU\xad\xea\x81\x8e\xc3\x84jp\n\xdc\xe6\xfa\xd1\xb1sY\x97\x19\xfb\xbaC\x1dIv\x1b\x87\xba\x8b\x86$\xcb\xcc\xd7z3\x91\xa66r\"\x91P{\x94\x884\xb5\xfb,P\xba:V\x1e\xc9(\xa4\xad\xe3\xcc\xf0\xe8Pg!\x10i\x10\x04\x99Z\x84U\xc6(d\xc4\x8f\xa66\x83\t0\xcd\x91\x16\xb0\x16\x15\xc9\xe2\xe8\xeb.\x1a\x19>QwQI\xad\xcb\x9b\xdd\x17\xad\xe4\x8ai\xb6\xb5\x83\x16\xd1\xa5\x15\x02[`\x06\xdb\xbbd\xf9WL3\xbb\x84[f\x12\xef\x14 \x1b\x9d@o\x136\xd2w#\xa9\x90_\xb6\xd9+\xa8\x97\x97\xa8\xaf(@\xb6\xf0\xb6\x92c_\xd1\x02\x11;\xe0e;4m\xbe\x89\xc4\xedq\xba\x03\x8aw\x89\xf1\xed\x10\xc0\x03>x\x81\x1e\x0e7\x90\xc7\x03.y@-\x0f\x98\xe6\x05\xe29\xdf@K\x0fX\xea\x01i=\xe0\xb0\x07\x94v\x8f\xe1\x1e\x10\xde#\xfe{@\x87\x0f\xd8\xf1\x01Y\xbe\xc0\x9dw\xa8\xf4\x01\xb3\xbe@\xb4\x0fx\xf7\x01\r\xdfa\xe5\x07$\xfd*\xce>v\x17\xd95r@\xf0/\xf0\xfd\x03\xfa\x7f8\x1b0\x9c\x1c\x18\xce\x15t\xa7\x0e\x863\t\xc3\x89\x85\xe1<\xc3p\xdaa8\x0b\xd1\x9f\x94\x18\xceQ\x8c\xa7,\x863\x18\xc3\t\x8d\xe1\xfc\xc6\xe2tGw\xf6c8\x19\xd2\x9d\x1a\xe9O\x94\x8c\x07N\x96\x07R\xfaC.\xd3\xe6!\x97\xfe\xf8K\xf7V\x7flf\x05\x8f\xc5\xc7x\xa3\x82\x83Y\x98\xd4\xa5^\xa5\xb8\x81L\xb0\\#D;\xc8H\xb6\x92ke,\xbb$\xdb0\xbf\xb3X\xafB\x12MR\xaa\xbe\x13F\x04'\xd7\t\x1c\x9a\xcaP\xeb\xe4\xda^\xe0\xa0d\x96]\x12\x9c\xba\xf0H\x92\x93%\xd5n@\xabtT&\x19\x06\x03\xf2Z\x13X\xd8I\x03\xa4C\xe9\xa6\x88\xfc\x12\x01\xe7\x1cM\xd7+\x15\x84\xa1\xa2Z7/\xbf\x14:Q9Uy\x80q\xb4\xc97\xad*\x13Y*\xe9\xe8]=\x89T\x88\xd4\x95<\x11\xe9`\x9d\xa7\x00\xab\x811\xbc\xa9\x82xx\xe97\x92\xb8QJMW\xf4D\xea\x97\xd0\x91\xe4\xddl\x99[\xc9Z\x19\xb7]\xbf$\xeeYn\x9f\x0e\xa7\x1a\x00\xfd\xa6\xfa\xf5\x17^\r\x9a\x9a\xe5\xdd\xd0\x93\x05\xdf\xd5\x8aZ\xbd:\xdfY\x06\xd9\x03H:A\xc84O$* m\x90\xad\x0cr\xa7\xba\xd6\xdd\xb2\xd0\xc7\x99bS\xb7\r\x91\xda\x14\x1e$\x0b\x8aOR]\x86'H3\x9b\x9ed\xf7\xa8\x96l\xa4\"\x83]\x08]\x8c4]\t|\x02\xbd\x91<\x93\xa1\xdb\xa5Sh,\xac+\x8a\xf8{K\x9d\x84t\xf81\xf9\xc6+\x03,W\xde@\xda8\xbbnm\xb3\xfe\xd5R\xdb\xa4\xa8\xff\xd5\x18\x99J\x8b\xa6\x99\xa4\xa6\x113\xc9\x12\xad\xa6\x1cb\x92IM5l\xea\xe9\xd2\x13\xac#,\x1f\xeb\x9c`\xf9\xccu\xf2;Q\xeb,\xb0p\xe72\xc3*\x1d\xb5\x8e\x80\xe5A\xf9\xb5\xce\xc0\xf6\xdaj\xf0`<h\xa8\x9et\x86V\xfa\xd2t\x02\x03\xd27\x03`}\xd5cXE\xf0\xd5\xd8\x16\x98\xe5V\xfb\x07\xa3\xf8Z\xaa\x13\xd2V2\x03c\x0c\x15\x0e\x9c*9\x01\xda7Tpp\x979\xb6\xcc\x018?c\x9fT\xeag\xa8\x94\xb2zI\xad[\tCfUL`\xb4j#C\x07Bf\xd4T+*\xf5d\x19I\x94)X%R1!\x14\xd1'BM\xf5\xa0|\x84jC\xd7II8eAT\xfc\x08]@\xb2\xce\xafnAl\xa4\x04{\x02\x82a2\xfb\xf9\xeb\x8e\xa4\xe4\x04\x9c\xb3\x19j&\x01%$8\xb4\xa3\x98\x00\x0e\x0e\xd4R\x99\x89\xfbN\xd8S\xab\x8e\x18\xb5\\;\xf1R0\x95$\x1aE\xfc\xe4\n\xb0Qs\x8a\xc5.\x04\xe9`\x82h\x0c\xb17\xb64K\x14cm \xc0B\x1d\x1c\x95%\xb2pm\x0f\xe6\x89\xc6\xd3\x99\x99(O\xf6b^\xd43m\x06\xec\x89|Z.\x99v\xcb\x85\x05\x9c\nC4\xd4\xfa\x1d\x93\xa0J\xf0\xdc\xa1\x1a\xf2b5\xfd&P\x8c#\xa8\xa0h\x99\x8ddQ\x8a\xb1\xe9\x8d\x05IjU\x04\x05\x95\x87`\x02\xe5V\x07H\xb1J\x1e\x8e?\xaa\x11p@6\xb1\xa2\xa8KMpau\x17\x16XY\xdd\x85\x05uVW\x8f\x80\xaal\xeb \xca\x8b\ng\xd1\x8a<\xa6\xd6\xcc\x0e\x94\xdbh\xda\xbb\x05mFF*\xc2\x13\x8d\xda\x0c>\xd1\xa8\x15\xf1\xd5U\n-K^\xbe\x87\xd4\x93\x1e4\xc3\x969\xcb\xb7\xa4z#\x8f\xb3\x833|\x8d\xf4B\x068\x1c\x19\xd5\xdc\xe6d\x06\x1d\\\xc6\xa5\xd6s\xb1s\xa5\xd6f\x07kC\x1a\x99\xdbq\xc0\x02>\x0c>\xb3\xa8\xb6u\x96o\xd5\xb8\xccF\xc0FV\xe0\x96Eq\xb54!\x1e\xed\xf2\xb9GI\xe5$\xbe\x1e\x07a_\xd2\xd4\x85-U\xc8\x16\x1b\x88\xd5\x88\xcf\x06\xe2f\xd3g^l\xbbSk\t\x02\xa23\x19\xc0N\x9d \xdaN$\xd2b\xaam\x02J\xa78X\xd1\x052\xc4\xcd\xd3\xad\x80\x8f\x12&\xdb\xac\x9cu\xdfPA\xd4\x8b\x90bzq\x0f\x83\xde&\x8d\xafJ\xb6\xf9\xa4vn'\x9d\xc4\xc3\xfe\xc9\xb6\xa8\x01\x1e\xea\x16a\xd06i\x9d\x8b\xf2\xe0]i\x15\xb5V\xa9(b\xad\x08\x97\xed\x00NO\xf3\xd6\xa5\x07~\xf8\x80\x94\n\xf0\xe29\x03\xba\x97\xe8g\xdau\xefWq5\xe8\x01\xac\xa9\xa3\xcb\xa2\xfe\x00\xb20\xb6O\xfb \xbe\\=\xc4%\xbe^=\xc4%0v\x07\xe9\x16l\xe73\x1d;\xdaJz\xf3~\x06z\xdf\xa8\xaf\x98M\xf1\r\x06\xcf&\xd4\xd2\xd1\x1e\xca\xab\xed\xd36O<5\xed\xa0\x9e\x13\xda\xaa*\x9f;\x11q\x12\xf5\xc4U\x8f\\\x02\xe5ev\xd89\x94Z'+\x12\xa4S\x07\xa0E\tr\xf2B'M\xf7\xa0\xdf`\xf3\xdaRqr\x16+tg\xc528m\x13\xd81\x8c\x15+\x82:m\xa3\x08\x8a\xcd\xe9;\x81\xd0)N\xe1\x80Nb\x16\x87\xc2@g}\xdf\x11\x1d\xb5\xfc\xd4A\x12\xd8\x94\xdd\xce\xc6U\x13s\x07i\x80\xf68\xa1\xdb\xd9\xbb\xda\xdf6\x06F>\xa1v\xfe\x8f\x15\x0c\xdb}\x0e\x8d\xae\xbbNQGq\x80=jN\xcf\xb0+\xcd\xcb\xdf\xc2\xb64\x97\x1f@\xccht\xe7\xf8n\x08X\xeb\x04A\xdb\xfa\\\xc0\x0b:\xd3l\xa8h\xe7\x053\xec=2\x06\t\xcf\x03\xb2z\xe5\xba\xcf\xb1\xd3\xa0\xa6\xd4is\xb3\xf3\x99?\xd1I\xd7\xd2H'\xd0\r\xe7\xfc\x0e\xcc\x13U\x1bRh\x87\xb1\x9d*=k\\i\xa0\x0bh\x84\xb3\xf6\xc3\xed\xb1\xdd\x16\x14\xe0LhZI\xd73\x9a\x86\xbf\x85\xb6%Y\xd1\x92\r:\xf3\xa1\xfdE\xf4\xd5\xa0\xe9\x01\xd4\xe89\x1d\xb5\xec\xd9y\xcf\xe9\t\x9d\xf9\xad~\x18\xcf6\xc6Q\xd6\x8d\xe9\xbeW=\x17;\xf1<\x1a\xa0y\x9d\xb4\xfd\x00EN9g\xdc\x00\x19\x93\xe9\x01\x14S\x05P\x04\xa5\rH\x0b3\xed;\x80\x81\xef\x01\x12S\xec\x84\xd8\xb9O\xfd\xfb\xa5\x83W\x18A\x01D<\x15\xac\x1e\xf6\n\xb7h\xf0\x8c\xd0\xb9\xdc\xe7\x194X\x1b\x9f\x02n\xbd3\xb1\xc7q\x9b _\x8d\xedVT\xa3m\x0fG1\xbe\xc7\x91\x9b\x8a<\x08x\xca\x18\xe8\nw)\xd8\xbe\xf6U\xca\nlt\x11\xe8B;\xa5\x1c\xf1+\x95\xf2\xdb)h\xde\xcd\x15\xa3m\xbc\xd0\x06OMC~+\x92\xd1\xa4\xe3mp\x97\x11\x9b\x87\x83/\x90\xa5\x8a\xd8\xadP\x03p\x13\x14\x80\xe6t\xdf\xad\xce$\xab\xd5\xe2\xa9qJo+z\x1aV\xb0\x91<^\x19\x1e\x1akK\x91.F8\xc0\xdfR\x93\x98r\x03\xda\xf2T\xa2\x8a\xb2\xe9E4\x91\xd5\xde0\x9a\xa5AAH\x9fo;\xecDVr\xe8\xbb\x95\xc2\xc2@\xe7\xae\xdd\x161\xfeS\xe5\x02\x11\xe18@\xfb\xa1\xef\xf5\xeb\xeewT\x07;f-O%\xce~Sa\x10N\xb3\x19\xd6-/A\x08\x83\x14j\x87\xd1\xbe6w\x80\xad\x8ff\xa0c\xd7a\xa4#TU'\xb2\xd1\x93Hx\x11-\xa9@\xd6v\x83\x15\xf7\xfaB\x9e\x9a}\xda\x11Y@\x1e\xce\xa6\x19\xb3\x13\x91\x88\xdc\xc9\x80\xeb1\xf4n\x02e#O\r\xac\x91\x89\xc4\x88\x9b\xa9\x8f\xb6\xa1\x12\x1a[&Ri\xee\xa0\t\xe5\xb1*\xaeY\xb4\xe0j\xaa\xef\xec}b\xd0-\xcd(k\x88T\xa1}B2Jj\x82\xc5\x9b\xa7\x06\xa7\xaac\xa5\xc3G\xd2l6\xddU\x0f\xd96%\xa9 I}\xce\xb6a{\x02\x91\t<q\x19BD[\xcc\xccc\r\xae\x9c\xb9\"\x08D\xe5\x884\xa0nf\xd7\"\xf5$L%\xf6\xd4R'I\xb5\xe0\xee\xd7\xcc1H\xe6\xa9\x9a1\x124\x83m\x1e\xdaf%\xd5\xf6\xe0`p\xf8\x00T6\xcd\xba\xc0\x8b)\xd4\x8a\x02\x92u\xb9\x94\xea\xd1\x0e\xb0L\xc5\xc6S\x9a\xe2\xced\x04=^'\x95uq][)t\xc2?\x8fU\x82\xb8E\x1e\\\x11\xe2\x98\xca\rrS@\x13\x10\xc7Tjn\xe8\x0c\xee\x80TD[\x9b\xfaw;2\xf6Zol\xce8\x0bb\x91\x98\xd6c\xfb<\"\x08=\xa2\xd6\xc7\x86\x91\xc3z\xc5Y\x90\xda\xd5e\xe8\x89\xe1\xe8\xb5\xaa\xcb\xb0\x1dO{\x94\xfb\xee3\xfeJ\x15\x1b\xf9J\x01A\xe7\xf1\xa3\r\xf2-9p\x07\xb7\x0f\x9e&\xa5m\x07\xc4T\xf5Kc\xb0V\x9e\x1a\x82.\xe3V\xe2e\xa3\x890\x1a-\xb5\xc8\xbb\x05\xcc\x18\xb8I\xf1\x9e\x15\xc1)CKT-\x93,\xef\xa4\x16\xaa\xaa\x00\\\x8f#H!i\x01\xdb\x97\xea\x19?\x83\xef\xe6\xb6\xf2\x0c\x00A\x85L\rtU\xeb\x85\xcd\xd9!\x16\xb2\x02\x87\rx\x0bU\xb0+\xa9O\xcd\x02\xab\x9c\x80\xf1(\x08\xb7d\x11\xf2:\x1e55o\x9d\x05\x93\x13G\xfbA(hmUmh\xf4\x9d\xd3\x82A2\xaa\xfc\xf1\xc9E_\x1a8'\x83\x0fCIS3;$\x9d@\x86<\xc2~\n\xccW\xac\xce\x07\xdc\xa1\xd4S!u\xc7\x06\xfc\x9az\x12\x81\x14\x9aY1F\x063{\xf1c(\x12\xa8\xd6\xfb\x8e\xb8\xfc\xfb\x87\xfdi%2\xbf>\xee\x8e\xe0-\x1e\xdf\x88\xc8\xdf\x17\xac1\xf9\xfb\xc7\x9bQ\xf9?]F\xe5\xfft\x19\x95\xff\xd3\xaf\xb6\xa3\xf2CP\xdd[Q\xf9\xcfpM\xf1\xb9E\x01^\xc6\xe6\xef\x1b>\x8e\x07\xc6\xe7\x9f\x1f\xf2\xc5\xaf\x9ba\xfa-\x06\xfeM\xab-\x95\x90\xfdC\rc\xd0~\xb8\xc8\x17o\xbe\xe3[rVOzu\x97\xe7\xbaw\x04\xf0\xb7y\xbc\xbbn\x19\xba\xbfo\xa1\x04\xef\x1f\x1e\xae\xcd=\x9c\x9c\xa2\x00\xbeG:3u\\\t\xdd\x7f.s\xa0\xde\xa9\xd0}\xb5\xe7E\x00\xff\xbeX\t\xe1?<|\xd9\x9a\x991\x96?^\xb7\xda\xdd\xfc\xb6\x88\xeb\xdf\xd7 \x91\xfd\xfb\x87cX\xff\xf7(|#\xc4?\xdc\xa8\x9a7\x02\xfd\xf7Uk\xa8\x7f;\x1f(\xbc.\xaf\xcb|\xfd\x1b\xbc\xd5\xe2\xee\x8e9>`\xd8\xffeX\xae1l\xd7\x18\xd6k\x0c\xfb5\x86\x05\x1b\xc2\x86\x8da\xc5\x16a\xc7\xc6\xb0dc\xd8\xb21\xac\xd9\x18\xf6l\x0c\x8b6\x86M\x1b\xc3\xaa\x8da\xd7\xc6\xb0lc\xd8\xb61\xac\xdb\x18\xf6m\x0c\x0b\xd7\x87\x8d\x1b\xa2\xca-c\xceu\x11\xe9\x86xuC4\xbb\x8dXw\x13\xe8\x8d\xd675\x92#\x15i\xe6\x02\x17\xd9\x94\x1a[\xa9`\x8c=\xd3R-D\x9cb\x9d\xdc\x94\x0ef\xa41\xb3\n\x87\xd0\xea!q\x06\xf0s\x9eH\xd5\x9f\x02\x91\x1e\xc04&7\xa8\xcdDd\x02aK\x83w\xb1\xdef xa\xd7\xaa\xa9\xbb!\xa8L\x12\xfc\t\x11\x81\x1a+*\xd7\x0b\x83\x02H\xb1H\x1a$c\x17wJ\xc8\xa9\xbb\xc0\xc0N\x9dlmJ\x07[\xd4xe\x9a\x8a\xb2\xb5\xc9\xadU\x06F\x83Em$COf\xba\xf7\xb0\x80\x8ekr\x7f\r\"\x90f\xfbF\xc5\xf1\xbe\xc5\xc5m\x8c\xc3]\x8d\xddM\x8e\xc3=\x8f\x8b[ \x87;\"\xbb\x1b$\x87\xfb%\x87\xdb'W\xef\xa6\x04\x18Vw\xafe\xbd\x83!\x82\xd6\xae\xd79\xb0\xe1\xc1\xc4v\xdc\xc8a\xbd\xf5V\x89\x0c\xe2\xaa\xdef\xc1\xf0\xb1VT\x96{#\x0c\x08\xc2&\xb6\xc3W8\x1a\xe2\x192\x1a\xb4\x86\xf7\x12\xa3\x1e\xb7\x894^\xd3\\CV\xe6xBO\x93\xc9\xb0W\x15\xf8@d\xefk\xe1\xeb\xa6\xd0}\x13b\xbe\xd7[\xb4\xc4Xe\x9bi9\xc8G\xd3\x9d=\xb3\xb8\xb7:\xd8\x17d\xefm\xe1\xd0\xa4|5~\xb2)\xd9\x0e\xc60\xab\x013&\xde\x85\xbc\xd6\x9f\xbb\xb8\xaf3\xaf\xc0\xd0\xae3\xeda\x03\xac\xe9h\xae\xcf\x10\x1bt6\xe7s\xe8P\xabt\x84p\xaf3o\xb2\x03\x9d!\x84\xad\xb8/\x1a/0V\x82\xba:\xa53\x04\xc4\x9dy\x1fGj\xcdJ3\xaf\xb2h\x0cw\x06x)\xf3\xba\xd0\xf1R\xe5u\xc6K~\xdf\xa57\xe3t\xe5m\x16\x8d\xdd\x8d7\x19'![\x8d\xd2\x9e\xfa\x97\x91Ww4\xf3\xaa\xe6\x9e\x99:\x9a/8K\xe0\xde\xe9B\xa4\xcax\xb5\xf1\xe6\x90\xb1\x11\xdc;<\x9f\te\x03\xab'\x7f\xcd$<+\x0f\xb4C@_\xa3e\xbe\x1d\xac\x07\xbe3\xcf\x0ft\x07\x10l\xeb\x91a\xd4\x1d\xed\xf0}\x06\xf3Y\x0f\xb2L\xc2\xf5*\xfd\xf5=\xc0\xd0\x06H\xe7\xf59u\xe3\x17!\x84j\x00\xee_\xe7+\x823%C\xd0\xe4y~y\xbc;g\x88M\xe0\\\x98\xba\xf9\xf1\xb2\xde\x13\xae\xb7\x16\xc2V\xe9\x88\xeb\r\xd6w\xe8e%\xc3A\xab\xa7\xde\xfd\x06\xeb\x9d,\x9c\xb0\xbe\x9b,yS\xa7\xfd\xf2\xfc?w\xc7\xc7\x85N\xdb\x1e\xaf\x85\x95\xa1\xd4\xfd\xc32\xf8\xcc\xb6\x8a;\xd4SU\xdc\xe1\xf1\x96\x8a\xfb\xe5\x7f.T\\xTU\xdc/\x7f\xf1\x9f\xef\n\x1d\x13\x9a\xd2+\xd7\xca\xf3\x8d\xacn3\x12\x0c\xdf\x1c\x8b\xe1]\xe4&\xf9\x0b<?\x0c\n\xf0\xdc\xad\xd7\x8f\x83\x02<\xf4\x95\x15\xe0/I\xc5*]\n\xdcy\x13\x9arC\x17yj\x8c\x84A\xf5\x9d\xcb\xfe\xef\xb7\xe75\xdd\x17\xa2\x9c\x90\xd2\xdeE99\x1e\xd7\xe2\x9b\x9cA\xf75\x10\xc0D\xae\xb3[\xd1\x801\xec\x08\x86.\x91{\xde!\x04\x89?\x0c:1\x8d\x15E\x1eq\x83b\xfc\xe5\xf9\xf53\x0e\xe3\x9d,\x99\xe7g\x1c\xad\xa6\x1as8\x11\xbe\xe1n\x8e\xb8s\xfdu\xcb\x10\"\xfb\x10\xe6;l\xe6;E\x8f\x970*\xc8\xc3$\xbd\xb9\xaf\r\xf9\xf1\xf5\xa0 \xaf\xcd\x1c(\xc8\x10f\x83Ckx\xb3\x08\xb61\xaa\xc9\xc3\xb2yyj\xa3\xb0\xaa'\xffku4m\x99\x97\x15\x86\xdb\xc0\xdb\xd7\xeb\xe5\x88\x83\xce\xbc\xd6_U\x9c\xf3l\x15\xca%\xcdj\xb1?\x87\x15\xc5y\xcc\xf1!\xef\xcb[\xbd\x87<w>'\x93\xfb\xab\xd9\xe0\xf8R\xb8)\xe9;P\x92T-\x18\xb4\x04\x83\x99S;\xc3\xce\xd7\xa1\x87>\xd5\xf4\xa9xg\x8cJ\xebr;z\xec\xd0\x7f*5\x8fW\xa9\x0f\x17\xad/\xaea\xaf\x83s\x93\x11|v\xbe\xbf\xfb\xfby\xc1\x08\xda\xe3\x8e\x11,\x1eo\xef\xfcC\xc1u\xe7\x1f\x1eo\xed\xfc\x9f}\xb1\xd8\xf9?\xfb\xe3b\xe7\xff\xec7\x7f\\\xdb\xf9!P\x18\xef\xf3\xce\xae\xed\xf3\xc7\xb6\xcf\xcb\xad\xe0\xbc\xb7\x1f\x87]}h\xf28\x12\xb0\xab\xcf\x0f\x8f\xe7\xb5\xb0[\xd0$\xfe\xda\xa0I\xbc\x95\x8f\x05\x0e;\xf9q\x82\xfd\xfc\x00\x01\xae\xe2\xdaNn\xe0\x1eo\xda\xc3Oqm\xf7>B\xd0(\x1a\x01\x8a\x0c$Q\xa6\xdcq\xd8\xb1\x87\x16\xf2f=>\\\x9bd\xb8\x86\xac\xb4\xc8O|\r)\xdc\xbc\xa9[\xf5q\xde3\x0e4A\xc7\x92\xf3\xb0U\x0f\x85\xbfY}\xb8\xd6\x0c\xd8\xa5\r\xec\xa0\x07\xd8;\xd3\xb03\x0f%\xbc\xac\xcd\x14l\xcc\x8bbC\x1c\x8a};\xacR1]\xae/\x0c\xde\x80\x87\xea\xea\xde\xeb/\xb39<X{\x1d\xd2`\xd2\xca\x05\xda\xf9X\xae<\xf1:\x9b\xc7\x7f\xeb\x9a\xb0\x9b{/\x07\x1c\xd0[Z\xc8\x10\xa6>Wv+\xb6\xc0\x05\xb3W\xb1\xc1\xbbggf{o\xf6\xa16$8\xbd\x17\xc07\x0b\xc1\x11f/`; Li\x1e\x10\xe4\xb9\x1d'\xbcn|\xa59\xd6)\xad\x80\x837w\xf7P\xe7v\xfc\xd1wTh\xb5gFe\xb4{Vf\xa7je\x0e\x0cai\xefA\x8f\xe4\xacV;Db\xdbH\xf0M/\xa9\x1d\x12IP\x1fi5\xb9;\xd2\x9c\x1b\x90 wT\x81\xf7H{\x84\xf7J\x1b\xa5\xfa^\xec\xd2\xf0\x86\xad\xdc\xeed)\xf3\x08\x9a\x8e\npz\xa5\xb4R\xecL\xd9\x8e\xc2{\xbc\x81J\x1d\xe5\xa1\xf6\x89\xdb\x82G\xd6Jw\xf4\xb3\xb4\xd3m\xb1\xd5\xce8\xfb\xd2\xc5\xe9(m\xf5P)\x1e\x0e\xaf\x96\x06\x02\x80\xde\xb6\x95|\x93O\xfen\x7f\xfdo\xe9\x04l\x8f7/\n]d\xd9\xe6\x99C%\x95g\x0e\x8f\xb7x\xe6\xef\xbe\\\xf0\xcc\xdf-\x1d\x82\xbf[u\x08Z\xb8\xceP\x02\xb8\xad9\x04\xf7\xfb\r\x87\xe0y\xd5!84\x9c\xc7f|\xf8\xac\x0f7\x1d\x82\xe9\xbc\xee\x10\\\xb4\x94Y\xe9X\x83\xb2R\x89\x1b\xb9t\x08\x1e\xdf\xe5\x10\xf4\xe0\x10<o:\x04)\xe8]\xef\x10<\xaf:\x04\x87\x162+\x1d\x1f\xae\xcd=\xb0R\xd3\x1c\x82\xa7\xf3\x82\x89\x9e\xaei\xea\x10,\xc7\x91\x89\x0e\xc5\xbeY}\xf8\xb253\xc0I/\x1b>\xbb\xe3\xaa\xa63\xd4\xf0\xb26]\xc0O\xdf\xb7\xf0\xc6U\xdf{\xa50o\x1d\xaaV\xbd\xe6t\xf1\xd5\xddW\xf6\xf1\xb8\xa2\xd7`\x8e\xab\xa4\xf2!y\xabg\x9cU\xf5\xffy\xc6\xd2T\x93\x1a\x87\xfdo$3\xbf\x8avtApe\xfc\xaeMBr\xaa\x9d\xa4df:\x1c\xf8M!j6\n\x9e\x8dIW\xc1`\xb6\x92\x0c`\x9bj\xc9\x08\x8dch|\xaevT>x\xd2R\x99\x9fU\xe4\x98u\x02\xe9\xb1\xb5U\x11\x90c&HQ\x15\x85\xdf\xa1\xce\xd8\x00\xa8p \xb6\x87)\x94\xc8\xf4\xa8$;\t\xe0\xaf\x86o\x168\xa0\x18\xe3r\x0733\xf5\\qFR=\x05YN.OP\xb2\x9e\xd4\xe1\x9b\x8b\x14\xa7i\x18H\xa6 \xf8\"'\xfd\xb5\xbf\x1d\xa4\xbe\xc8y'W\x8b\xc2C\x11\xb6G\xf6\xda\x1a\xa4!\xd4\x81El\xb3\x1chP\xd2\xcai\xa8\xd8\x93\xb6NY\x97\xea\xe5`\x96\x16\x85G\xa7x\x06S\x9d#k\x04!\xaf\x13\x8a\x10o[\x11\xe028\xa1;\x86aC\x17w\xdc\xc6.Z\xa1\xed\xd1\xe5\xb6\x06%7\xf5\xdd\t\xb0\xec\xb6F\xcf\x93\xc5?u\x01\xf1\x9c\xe9\xc0\\\xceJ\x98@n\xb3\xf3r\xbeS\xc8(\x87Ay~]\x8d\xccV\xeaW\xe6\xe1\x04\xa7+\x12\xba\x9e\xeb\xf5\x93\x9ch\xcc=\xc9\xf5z\xd3\x9d\xd2\xf5V\xc2t\xa4Jz8\x85\xe9\xf9\x90b\xf5\xff\xf0\xb7\xaf\x91\xd9\xbc\x97 `\xdcH\x1f\xe5\x06\x02y7\xf7d\x91\xf3\x9d\xbe\x92\x18D#\xd4\xc8\xe5\xb1\x9e\x1e7\x10\xcc\x9fc\xc1\xebq\xe0\xe0\xbaP\xfa\xa1\x86\xbc\xc3\xd3\xe3\x1a\xa6\x8f\xcf\xa5k\x84\xc3P\x8f+\xa4J\xe2\xd9\x860uA\xf1\x83\x95\xf9\x8dp\xa6]Ao|\x88=\xc2\xe1\xf9\x84\x99\xbd\xacIM\xf5\xb0\xcc\x82\xebI#d\x84\xd3\xf2zt\x92{\xa4\xab=\xe0\xf2\xe5\x03\xfd\nq\x0f\xf5\xf4@\x84\xd3\xf0\n\xa6\xe7\xd8\x00\x8a]\xe7\xd3\xf0z\xf8&d\xd9\x17LO\x86\x8a\xabs\xb0\x89\xf0ie\x05\x98\xc6\x8a(5\x80\xfamd\xe9\x82Sp\x10\xa8\x04Q\xbd\x18Q\xea\x00\xd7\xabA\x1a\xf8\xe4\xafn||NV\x01\xa6\x02\xd5\xad\xb3\x1bKw\x14\x8e\xcf2\xeb\x96\xcag\x99\x15\xe6\xcb\xc7\xa23\x9c\xdd\xb6\x80\xfb\x14hr\x9d\x83\xd8o\xf4\xd1Jj\xae!\xc0\x1cp\xa3\x90\x04l\xad\xf3g\x90d\xb5\xa8~ca\x12N\xe6\xebg3\x01\xaf\xf2\xe4}\xd6\x92\xbd\x17Xs\xaa\xa9\xc8\x14\x81U\xdf\xd4\x1e>\x7f:\x9f\x1f\xee\xf7\x0f\xa7\x85\x02\xd1\xa5t:\xc4Z\xca\xb6\xea\xb0\xac\xa1j\x0f\xcb\x94-\x05\xe2\xf3\xdf,\x14\x08xT\x15\x88\xcf\xbf\xfe\xcd\x9a\x02ql\x01\x9d\xc5\xb9\x12\x17\xf1\xfa\x05o\xb8\x864L\x18\x1d\x1fC\xee7\xf3\x9cD\xd9\xbf\x8c*\x88<7\x83\xf2\xf1\xf9\x13y\x17b\xdf\xf7\x95\xb1\x055\xe4\xf3\xa7G~~\xb7_\xc3%^\xc6\xa0\xd5\xd8\xc7E\xefX\x19\xd1\"\xd7L{\x0e\x82\xd0_\xc0IsX\xe8)\x1eL~f\xcd\xe4g\x17q\xe7\xa7f\x04\xacP\xc65\x10c\x02\xb7\xcd\xd4\x8c\x80\xa2\xc5\xb0Y\x90\xfe\xb7\x0e\x82\xcf\x9bA\xa3Y\x1fmVl\x96\xa3}\xf7\xb25\xdaM\xc1\x99\x97Ao+\xbc8zR\xda\xff\xc7\xb2P|\xf6\xa7\x0b\xa9<\xf3\xeb\x87\x94\xe7_\x7f:\x0e\xea\xcfrR\xde\xc8\xf3E\x83\xde@SI\r\xb2]2\xa8B\x0eb\xabC\x94v10\xa2\x16\xe3\x9b\xc9\xd1_\x06\x15i} _\x9e\xb6\xd3@a\xfa\x17\x9b\xc0\xe6Ih\x02 +W\xd6\xfan\xb4U\xf2\x92\xcf\xa3\xcbh9\xdfU\xb1\x8ay\xd6\xddJL\x81\x8c\x96\xd3R\xb1\xc2\x1c9\xda\xd3\x87\x8f\xde\x9e*\xe2/\x83\xa5H=\xfel\xa0l\xa9\x95\x8c\x10\x8aP\xe1/P\x94\x9e>H\x02\xd8\xa8\xa7D\xf0\xed$\xd7(\xa3\xe5I\xb1\n\xa9\xde\xab\x1b\xfa\x9a\xbbv%8\x16\xcf6Q\xc5\xee\xb01\x15\xc9\x00\xc8\r!]\xdf/\xd7\xc2\xd8N\x98\x99QA\xbe\xf5\xbaV\xf4\x0eg\xd2\xfe\xbb\xab:\xfd\xbc\xf4&\xb5\xe7=V\xfe\xf1\xfb\xef\x1f\x1f\xfev\xbe2\xa4\xeffS\xd9\xcbw\xe7\xdd2\xef\r\x1f\x13\xe7\xfd\xe9\xee\xd5w+\x9e\xa6\xb5\xc4M\x7f\xd3\x7f,\xfdM\xcbKj>\xfb\xe2\x9bM\xdb\x99\xb0\xb8\xb4\xb49\xb0\xa3u\xd3\x03%W\xc5\x94\xa55\x8do\x9b\x89\xc0\xe3\xd8e55S\x1c\xff\xca]4g\xb8mf\xe1\xc4\x1a\xa6\x86'\xe0U?\xe0\xe8\xca\x1a\x9e\x8e\xbe\xac\xed\xee\x8e~\xad\xb1\xf8u\xc7\x16\xf1:r\xc6\x89\x1d\xae\xbb\xa6e\xfb:\x16qo\x95\x1d\xf8\xbd<$\xdf0\xd6\xb1\x97\xab,\x8du\x8c`\x18\xbd_\xd4\x8f\x8f\x9f\xff~\xf7\xed\xf9\xe5<^\x8f\xc2\xa9\xa3\x1bl\xe8:\x18\xef\x88\xb7\x1d/\x1b\xc6\xbb\xc3\xd1\\\xfbp\na\xddx\xc7\xe5\xfe\xe4y\xf4\x81\xcd\x8f\x7f\x1c}`C\x1b\x16N0f\nn\xb4\xae\t\xa4`\x0f\xce\xb1\xc0\xd7\x89\x9c[2\xdf3\"\xfc\x86/,\xb1\xabv\xbf\xb1\x15/\xb04\xee\xaf_\xd2\xa6/\xedf\xeb\xfc\xb1\xb5Q\xeeBy\xaf\xd6\xe5mw\x9cy\xe7\xe7+N\xb9\xa1C\xbd\xe50\xf8c\xbem9\x94\x1c\x1f\xd8r\xc8Q\xa3MUr##\x85\xa7\x16E\xb9\xac\x91\x06\x8eq\x99\t\x8eYa\x1a\x97\xeb{\xf2\x1d\xd8\x82WOw\x0fw\xa7\xfdiw\x15\x05v\xaf\x1e\x0f\xfbo\x97\x88\xb3\x8d<\x9b\xde\x94\xdb\xf9\xb7Y\xc5\xad\xb6T\x8eq+\xcf\x16\xe3x\xf5\xf3\x05\xe3x\xf5j\xc18^\xbd\xfa\xed\x1a\xe3\xd8o\xe8L{<\xed\xc4\xff\xef\xda\x15h\xfc\x9a\xfc\x0f\x9b\xb1\xdb\x0f\x00\x1e\xe04\xfe\xa6zui\x8cF8\x10}M\xa5\xb8\x1d\\]v\x18y\x96(m#\nN\x87\xf1\xedr\x18\x85\xed\xdc\x18hf?\x9a\xe3\xc71\x1d\x18\xd1~]\x1d\x93\x1b\x8b\xd6\x86owk\xfcX\xe4\x85\xf1c\x06FMya\x14\xd4\xc7\xe7\x97\x8f\xc7\xf66\xef\x92_\\)v\x1a\xff?\xc2\xd1\xb3\xfad\xd7py\xa2\xf2\x95]+O\xf4B\xe0\x95\xf2\xb6\x83\xd7.k\xfc1oh\x84\x05\x8e\xb5\xb9]\xbb\xc7\x8c_\x90\xffMS*\xd7\x91\x7f~T\x1b\x19\xedg\x01\xf9G\xd7BU\xde\xca+\xcax~\x81\t\xbe\xf8\xec\x00X\x94\x0c*\xe9\x88\x18\xd45\xc1s\xf1<\xae\x0c\xe6\xc4\x9a\xeb\xb4\x7fX\xe4\xe8?\xf3\xf3\x98\xde\x18\xf4\xbc;W\xe5\xf3\xc8G\xef\xa6\xa6\x88\x1eY\x05=\xb4\x9c\xcc\xd0\x0f~\xe9\x87\xb3\x07\x05\xb3\x1c\xecQ\x8e\xe7\xb1Rz\xed\xc8\xc9\xcc>I\xe6\xf7\xfb\xe3y\x1a8}k\xea\xa2/o$\xc3\x8f\x9c\xe1n\x91\xfe\x8e\xbe\xc2\xc5fvq)\xd8\t\xfe\xf7x\x12o\x07\x17\x8b\xf1{\xc2Y\x97\xec\xfa<\xc8\x00\xda\x94\xc7\xe3wcc^\x9e\xba\x1c?,Z\x0b\x97\xa4m\xb4V\xa06\xcb\xd6\xd6[\xd2\xde\xb7\xb5\xe0L,\xb76\x98\x0c[G\xce\xb8\xc1\xa4\xf2\x9e\x1b\x0c\xcb\x12\xad\xdb\xb4\xb0\xa7\xb1\xefz\xdd\x99\x9d\x0bN\xfby=\x86\xe3,\x08\xe5\xcb\xbc\x11\x07\x9f\x8f\xf3\xb1\xc4\xf9\xfcd\x95.V\xb1\x98nF@3\x1e\xa8La\xce\xe7f\xffq\xb4\xb3\x9c\to~H\x89$v\xc0J\x8eF\xa1\xc0\xcaH\xa7Y\x8c^\xe3QS'\xb8m\xa2\x91I\x00\x8e\x05I-*uG\xb7\xa0\xde\xdb\xb0\x8e\xb7\xd7\x87\x8f\xbb\xaf\xd7\xc0\xf0C\xdaM\xcdu=\xef\r\xa4\xc7J\xbd\n\xf7XI\xdb\x12?\xbe\xfe\xbfK\xcc\xc7\xd7K\xcc\xc7\xd7\xbf\xdc\xc6|\xe4\xb6\xc8\x913vb\x89\xdb\x96,@\x01\x15\xd3,\xe2*\xa7\xf1z\xd4\xe3a7\n!\x1b`\xfa\xb5a\x10\x04\xc9Z\xca\xf3V\xca\x08#\xb9\xdd\xe1\r,\xb4\x00K\x1e\x97\xf5\x8e\xe0\x12b\xd7\xc7\xd0\xee/e\x81@\xae&\xbd0g7`\xcd\xdd\xe6\xe0\x8c:\x11\x86\x0c\x97\x8fVF\xbd[0\xf8\x1bH\xfd}\xe3\xb5b\xe6=!\x0f\xde\x8d\xf6\xdf\r\xd8\xfe\xda\x08\x0b|\xa5\xa6,\xa0\xfb+\xc3\x06\xaa0!\xf6\xf7\xcco\x0f#\xef]A\xf2\x9f|\xbaf8\xd2\x88\x1cJ\xf2\xcaD\x190\xba\xc4\xf6\xaf5Y\x00/s\xc3\x9e~|=r\xcd\x95\x06\x8f\x90\x17\xb9\x93\xf3\xd4\x98\x91\\\xbay\xfe\x98\xb8\xcd\xf2\x8a\xcd\x91\x1f\xae5\xeb\xa5[f\xaf\x1f7A1\\=^\xe7\xf9\xfe\xd5\x8fh\x19\\\xfe\x95u\xed\xde\xf9E\x08\x80f\xa5\x13j\xec=\x9em\xe5K\xe5|$N5\xf9\x15\x93\xefz\xbe\x0f\x8bV\x8d\xc0\x1c8\xa0\x90\xb2\x0e\xbe\xd7F\xb9\x10\x07\x14R>#\x99\x01\xce\xca\xa9\t\xec\xa8\x06\x02\n9,*\x0b\x8fB\x93.\x92\xf1}x\xd4\xaf\xf6\xdf\xef\xd7\xae\x02\x87\xe7\x1doZ>\xdf\xe6Cc\xd9\x95\x07\x8d\xcf\xb7\xf8\xcf\xaf\x96\x98CxT\xf9\xcf\xaf>\xfdr\xd3e\xb84\xb6 \xcfY\x0fKr\xdcp\x07\xc2\x1d\xde9a\x00\x93\x81\xb5P\xef^\x8f\xc0\xc4\xb1\xcf\xccR\xc6\xa7\xa0\x7fn\xb4\xdf\xc6w\xde\x89=\x17J\xc2\xdee\xfd\xf4\x96\xa8~q\x1d\xa2\x88'\xb96\x9c\x81?\x01\xe3gY0\x88\xd5`&G8\xd0\x95o\xdc\x1d=\x0e\x08\xef\xfe\xe3\x90\x8a\x87\xefn\xc8\x0c\xee=\xde\xe5\x1d\x9b@\x89\x07x\x0et\xb2\x0cn\x92(\xb8\xc9~\x16Y\x8fi\xde\xf1\x8f\xe74\xee\xf2c\xb3\xde\xe8\xd3\xbf\xbe\x1e\xbdyc^\xd0\x85.\x080\xa4\xdfr\xe3\xe4\xd6X\xd0\xcb\xd3\xdaX\x80\xf2\xf2\xbe\xc5\xbf\x1d\xbe\x92\xe5\xd1\xad\x05\x96qlKS\x1b\xc2\xf5\x05\x7fN\x13\xc71Y\xb9+\x99r\xe4\xe3|x\x8er\xfc\xf3\x9f\x7f\xfe\xe7\xff\x03,\xe6\x99\xde"
    )

    #
    # CONSTRUCTOR
    #

    def __init__(
        self,
        background_color: typing.Optional[Color] = None,
        border_color: typing.Optional[Color] = None,
        border_dash_pattern: typing.List[int] = [],
        border_dash_phase: int = 0,
        border_width_bottom: int = 0,
        border_width_left: int = 0,
        border_width_right: int = 0,
        border_width_top: int = 0,
        dash_pattern: typing.List[int] = [],
        dash_phase: int = 0,
        fill_color: typing.Optional[Color] = None,
        horizontal_alignment: LayoutElement.HorizontalAlignment = LayoutElement.HorizontalAlignment.LEFT,
        line_width: int = 1,
        margin_bottom: int = 0,
        margin_left: int = 0,
        margin_right: int = 0,
        margin_top: int = 0,
        padding_bottom: int = 0,
        padding_left: int = 0,
        padding_right: int = 0,
        padding_top: int = 0,
        stroke_color: typing.Optional[Color] = X11Color.BLACK,
        vertical_alignment: LayoutElement.VerticalAlignment = LayoutElement.VerticalAlignment.TOP,
    ):
        """
        Initialize a map representation for a specified region.

        This constructor sets up the visual properties and layout options for the map, allowing
        customization of colors, borders, alignment, and other attributes.

        :param background_color: Optional background color of the map.
        :param border_color: Optional color for the border of the map.
        :param border_dash_pattern: Optional pattern for dashed borders.
        :param border_dash_phase: Optional phase for the dash pattern.
        :param border_width_bottom: Width of the bottom border.
        :param border_width_left: Width of the left border.
        :param border_width_right: Width of the right border.
        :param border_width_top: Width of the top border.
        :param dash_pattern: Optional pattern for dashed lines within the map.
        :param dash_phase: Optional phase for the dash pattern of lines.
        :param fill_color: Optional fill color for specific regions in the map.
        :param horizontal_alignment: Horizontal alignment of the map.
        :param line_width: Width of the lines drawn on the map.
        :param margin_bottom: Bottom margin around the map.
        :param margin_left: Left margin around the map.
        :param margin_right: Right margin around the map.
        :param margin_top: Top margin around the map.
        :param padding_bottom: Bottom padding inside the map.
        :param padding_left: Left padding inside the map.
        :param padding_right: Right padding inside the map.
        :param padding_top: Top padding inside the map.
        :param stroke_color: Color used for strokes on the map.
        :param vertical_alignment: Vertical alignment of the map.
        """
        import json
        import zlib

        super().__init__(
            geo_json=json.loads(
                zlib.decompress(MapOfNorthAmerica.__MAP_OF_NORTH_AMERICA_GEO_JSON_BYTES)
            ),
            background_color=background_color,
            border_color=border_color,
            border_dash_pattern=border_dash_pattern,
            border_dash_phase=border_dash_phase,
            border_width_bottom=border_width_bottom,
            border_width_left=border_width_left,
            border_width_right=border_width_right,
            border_width_top=border_width_top,
            dash_pattern=dash_pattern,
            dash_phase=dash_phase,
            fill_color=fill_color,
            horizontal_alignment=horizontal_alignment,
            line_width=line_width,
            margin_bottom=margin_bottom,
            margin_left=margin_left,
            margin_right=margin_right,
            margin_top=margin_top,
            padding_bottom=padding_bottom,
            padding_left=padding_left,
            padding_right=padding_right,
            padding_top=padding_top,
            stroke_color=stroke_color,
            vertical_alignment=vertical_alignment,
        )

    #
    # PRIVATE
    #

    #
    # PUBLIC
    #
